/*
 * (C) 1999-2003 Lars Knoll (knoll@kde.org)
 * Copyright (C) 2004, 2005, 2006, 2007, 2008 Apple Inc. All rights reserved.
 * Copyright (C) 2013 Intel Corporation. All rights reserved.
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public License
 * along with this library; see the file COPYING.LIB.  If not, write to
 * the Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1301, USA.
 */

// Generated from template:
//   templates/StylePropertyShorthand.cpp.tmpl
// and input files:
//   ../../../../src/3rdparty/chromium/third_party/WebKit/Source/core/css/CSSProperties.json5


#include "StylePropertyShorthand.h"

#include "platform/RuntimeEnabledFeatures.h"
#include "platform/wtf/HashMap.h"
#include "platform/wtf/StdLibExtras.h"

namespace blink {

const StylePropertyShorthand& backgroundRepeatShorthand() {
  static constexpr CSSPropertyID backgroundRepeatProperties[] = {
    CSSPropertyBackgroundRepeatX,
    CSSPropertyBackgroundRepeatY,
  };

  static constexpr StylePropertyShorthand backgroundRepeatLonghands (CSSPropertyBackgroundRepeat, backgroundRepeatProperties, WTF_ARRAY_LENGTH(backgroundRepeatProperties));
  return backgroundRepeatLonghands;
}

const StylePropertyShorthand& scrollPaddingBlockShorthand() {
  static constexpr CSSPropertyID scrollPaddingBlockProperties[] = {
    CSSPropertyScrollPaddingBlockStart,
    CSSPropertyScrollPaddingBlockEnd,
  };

  static constexpr StylePropertyShorthand scrollPaddingBlockLonghands (CSSPropertyScrollPaddingBlock, scrollPaddingBlockProperties, WTF_ARRAY_LENGTH(scrollPaddingBlockProperties));
  return scrollPaddingBlockLonghands;
}

const StylePropertyShorthand& flexShorthand() {
  static constexpr CSSPropertyID flexProperties[] = {
    CSSPropertyFlexGrow,
    CSSPropertyFlexShrink,
    CSSPropertyFlexBasis,
  };

  static constexpr StylePropertyShorthand flexLonghands (CSSPropertyFlex, flexProperties, WTF_ARRAY_LENGTH(flexProperties));
  return flexLonghands;
}

const StylePropertyShorthand& markerShorthand() {
  static constexpr CSSPropertyID markerProperties[] = {
    CSSPropertyMarkerStart,
    CSSPropertyMarkerMid,
    CSSPropertyMarkerEnd,
  };

  static constexpr StylePropertyShorthand markerLonghands (CSSPropertyMarker, markerProperties, WTF_ARRAY_LENGTH(markerProperties));
  return markerLonghands;
}

const StylePropertyShorthand& flexFlowShorthand() {
  static constexpr CSSPropertyID flexFlowProperties[] = {
    CSSPropertyFlexDirection,
    CSSPropertyFlexWrap,
  };

  static constexpr StylePropertyShorthand flexFlowLonghands (CSSPropertyFlexFlow, flexFlowProperties, WTF_ARRAY_LENGTH(flexFlowProperties));
  return flexFlowLonghands;
}

const StylePropertyShorthand& borderRightShorthand() {
  static constexpr CSSPropertyID borderRightProperties[] = {
    CSSPropertyBorderRightWidth,
    CSSPropertyBorderRightStyle,
    CSSPropertyBorderRightColor,
  };

  static constexpr StylePropertyShorthand borderRightLonghands (CSSPropertyBorderRight, borderRightProperties, WTF_ARRAY_LENGTH(borderRightProperties));
  return borderRightLonghands;
}

const StylePropertyShorthand& borderStyleShorthand() {
  static constexpr CSSPropertyID borderStyleProperties[] = {
    CSSPropertyBorderTopStyle,
    CSSPropertyBorderRightStyle,
    CSSPropertyBorderBottomStyle,
    CSSPropertyBorderLeftStyle,
  };

  static constexpr StylePropertyShorthand borderStyleLonghands (CSSPropertyBorderStyle, borderStyleProperties, WTF_ARRAY_LENGTH(borderStyleProperties));
  return borderStyleLonghands;
}

const StylePropertyShorthand& overflowShorthand() {
  static constexpr CSSPropertyID overflowProperties[] = {
    CSSPropertyOverflowX,
    CSSPropertyOverflowY,
  };

  static constexpr StylePropertyShorthand overflowLonghands (CSSPropertyOverflow, overflowProperties, WTF_ARRAY_LENGTH(overflowProperties));
  return overflowLonghands;
}

const StylePropertyShorthand& scrollPaddingInlineShorthand() {
  static constexpr CSSPropertyID scrollPaddingInlineProperties[] = {
    CSSPropertyScrollPaddingInlineStart,
    CSSPropertyScrollPaddingInlineEnd,
  };

  static constexpr StylePropertyShorthand scrollPaddingInlineLonghands (CSSPropertyScrollPaddingInline, scrollPaddingInlineProperties, WTF_ARRAY_LENGTH(scrollPaddingInlineProperties));
  return scrollPaddingInlineLonghands;
}

const StylePropertyShorthand& webkitTextStrokeShorthand() {
  static constexpr CSSPropertyID webkitTextStrokeProperties[] = {
    CSSPropertyWebkitTextStrokeWidth,
    CSSPropertyWebkitTextStrokeColor,
  };

  static constexpr StylePropertyShorthand webkitTextStrokeLonghands (CSSPropertyWebkitTextStroke, webkitTextStrokeProperties, WTF_ARRAY_LENGTH(webkitTextStrokeProperties));
  return webkitTextStrokeLonghands;
}

const StylePropertyShorthand& scrollPaddingShorthand() {
  static constexpr CSSPropertyID scrollPaddingProperties[] = {
    CSSPropertyScrollPaddingTop,
    CSSPropertyScrollPaddingRight,
    CSSPropertyScrollPaddingBottom,
    CSSPropertyScrollPaddingLeft,
  };

  static constexpr StylePropertyShorthand scrollPaddingLonghands (CSSPropertyScrollPadding, scrollPaddingProperties, WTF_ARRAY_LENGTH(scrollPaddingProperties));
  return scrollPaddingLonghands;
}

const StylePropertyShorthand& pageBreakInsideShorthand() {
  static constexpr CSSPropertyID pageBreakInsideProperties[] = {
    CSSPropertyBreakInside,
  };

  static constexpr StylePropertyShorthand pageBreakInsideLonghands (CSSPropertyPageBreakInside, pageBreakInsideProperties, WTF_ARRAY_LENGTH(pageBreakInsideProperties));
  return pageBreakInsideLonghands;
}

const StylePropertyShorthand& listStyleShorthand() {
  static constexpr CSSPropertyID listStyleProperties[] = {
    CSSPropertyListStyleType,
    CSSPropertyListStylePosition,
    CSSPropertyListStyleImage,
  };

  static constexpr StylePropertyShorthand listStyleLonghands (CSSPropertyListStyle, listStyleProperties, WTF_ARRAY_LENGTH(listStyleProperties));
  return listStyleLonghands;
}

const StylePropertyShorthand& scrollSnapMarginInlineShorthand() {
  static constexpr CSSPropertyID scrollSnapMarginInlineProperties[] = {
    CSSPropertyScrollSnapMarginInlineStart,
    CSSPropertyScrollSnapMarginInlineEnd,
  };

  static constexpr StylePropertyShorthand scrollSnapMarginInlineLonghands (CSSPropertyScrollSnapMarginInline, scrollSnapMarginInlineProperties, WTF_ARRAY_LENGTH(scrollSnapMarginInlineProperties));
  return scrollSnapMarginInlineLonghands;
}

const StylePropertyShorthand& scrollSnapMarginBlockShorthand() {
  static constexpr CSSPropertyID scrollSnapMarginBlockProperties[] = {
    CSSPropertyScrollSnapMarginBlockStart,
    CSSPropertyScrollSnapMarginBlockEnd,
  };

  static constexpr StylePropertyShorthand scrollSnapMarginBlockLonghands (CSSPropertyScrollSnapMarginBlock, scrollSnapMarginBlockProperties, WTF_ARRAY_LENGTH(scrollSnapMarginBlockProperties));
  return scrollSnapMarginBlockLonghands;
}

const StylePropertyShorthand& placeSelfShorthand() {
  static constexpr CSSPropertyID placeSelfProperties[] = {
    CSSPropertyAlignSelf,
    CSSPropertyJustifySelf,
  };

  static constexpr StylePropertyShorthand placeSelfLonghands (CSSPropertyPlaceSelf, placeSelfProperties, WTF_ARRAY_LENGTH(placeSelfProperties));
  return placeSelfLonghands;
}

const StylePropertyShorthand& webkitMarginCollapseShorthand() {
  static constexpr CSSPropertyID webkitMarginCollapseProperties[] = {
    CSSPropertyWebkitMarginBeforeCollapse,
    CSSPropertyWebkitMarginAfterCollapse,
  };

  static constexpr StylePropertyShorthand webkitMarginCollapseLonghands (CSSPropertyWebkitMarginCollapse, webkitMarginCollapseProperties, WTF_ARRAY_LENGTH(webkitMarginCollapseProperties));
  return webkitMarginCollapseLonghands;
}

const StylePropertyShorthand& marginShorthand() {
  static constexpr CSSPropertyID marginProperties[] = {
    CSSPropertyMarginTop,
    CSSPropertyMarginRight,
    CSSPropertyMarginBottom,
    CSSPropertyMarginLeft,
  };

  static constexpr StylePropertyShorthand marginLonghands (CSSPropertyMargin, marginProperties, WTF_ARRAY_LENGTH(marginProperties));
  return marginLonghands;
}

const StylePropertyShorthand& scrollBoundaryBehaviorShorthand() {
  static constexpr CSSPropertyID scrollBoundaryBehaviorProperties[] = {
    CSSPropertyScrollBoundaryBehaviorX,
    CSSPropertyScrollBoundaryBehaviorY,
  };

  static constexpr StylePropertyShorthand scrollBoundaryBehaviorLonghands (CSSPropertyScrollBoundaryBehavior, scrollBoundaryBehaviorProperties, WTF_ARRAY_LENGTH(scrollBoundaryBehaviorProperties));
  return scrollBoundaryBehaviorLonghands;
}

const StylePropertyShorthand& pageBreakBeforeShorthand() {
  static constexpr CSSPropertyID pageBreakBeforeProperties[] = {
    CSSPropertyBreakBefore,
  };

  static constexpr StylePropertyShorthand pageBreakBeforeLonghands (CSSPropertyPageBreakBefore, pageBreakBeforeProperties, WTF_ARRAY_LENGTH(pageBreakBeforeProperties));
  return pageBreakBeforeLonghands;
}

const StylePropertyShorthand& borderSpacingShorthand() {
  static constexpr CSSPropertyID borderSpacingProperties[] = {
    CSSPropertyWebkitBorderHorizontalSpacing,
    CSSPropertyWebkitBorderVerticalSpacing,
  };

  static constexpr StylePropertyShorthand borderSpacingLonghands (CSSPropertyBorderSpacing, borderSpacingProperties, WTF_ARRAY_LENGTH(borderSpacingProperties));
  return borderSpacingLonghands;
}

const StylePropertyShorthand& textDecorationShorthand() {
  static constexpr CSSPropertyID textDecorationProperties[] = {
    CSSPropertyTextDecorationLine,
    CSSPropertyTextDecorationStyle,
    CSSPropertyTextDecorationColor,
  };

  static constexpr StylePropertyShorthand textDecorationLonghands (CSSPropertyTextDecoration, textDecorationProperties, WTF_ARRAY_LENGTH(textDecorationProperties));
  return textDecorationLonghands;
}

const StylePropertyShorthand& gridGapShorthand() {
  static constexpr CSSPropertyID gridGapProperties[] = {
    CSSPropertyGridRowGap,
    CSSPropertyGridColumnGap,
  };

  static constexpr StylePropertyShorthand gridGapLonghands (CSSPropertyGridGap, gridGapProperties, WTF_ARRAY_LENGTH(gridGapProperties));
  return gridGapLonghands;
}

const StylePropertyShorthand& backgroundShorthand() {
  static constexpr CSSPropertyID backgroundProperties[] = {
    CSSPropertyBackgroundImage,
    CSSPropertyBackgroundPositionX,
    CSSPropertyBackgroundPositionY,
    CSSPropertyBackgroundSize,
    CSSPropertyBackgroundRepeatX,
    CSSPropertyBackgroundRepeatY,
    CSSPropertyBackgroundAttachment,
    CSSPropertyBackgroundOrigin,
    CSSPropertyBackgroundClip,
    CSSPropertyBackgroundColor,
  };

  static constexpr StylePropertyShorthand backgroundLonghands (CSSPropertyBackground, backgroundProperties, WTF_ARRAY_LENGTH(backgroundProperties));
  return backgroundLonghands;
}

const StylePropertyShorthand& webkitMaskRepeatShorthand() {
  static constexpr CSSPropertyID webkitMaskRepeatProperties[] = {
    CSSPropertyWebkitMaskRepeatX,
    CSSPropertyWebkitMaskRepeatY,
  };

  static constexpr StylePropertyShorthand webkitMaskRepeatLonghands (CSSPropertyWebkitMaskRepeat, webkitMaskRepeatProperties, WTF_ARRAY_LENGTH(webkitMaskRepeatProperties));
  return webkitMaskRepeatLonghands;
}

const StylePropertyShorthand& columnRuleShorthand() {
  static constexpr CSSPropertyID columnRuleProperties[] = {
    CSSPropertyColumnRuleWidth,
    CSSPropertyColumnRuleStyle,
    CSSPropertyColumnRuleColor,
  };

  static constexpr StylePropertyShorthand columnRuleLonghands (CSSPropertyColumnRule, columnRuleProperties, WTF_ARRAY_LENGTH(columnRuleProperties));
  return columnRuleLonghands;
}

const StylePropertyShorthand& gridShorthand() {
  static constexpr CSSPropertyID gridProperties[] = {
    CSSPropertyGridTemplateRows,
    CSSPropertyGridTemplateColumns,
    CSSPropertyGridTemplateAreas,
    CSSPropertyGridAutoFlow,
    CSSPropertyGridAutoRows,
    CSSPropertyGridAutoColumns,
    CSSPropertyGridColumnGap,
    CSSPropertyGridRowGap,
  };

  static constexpr StylePropertyShorthand gridLonghands (CSSPropertyGrid, gridProperties, WTF_ARRAY_LENGTH(gridProperties));
  return gridLonghands;
}

const StylePropertyShorthand& webkitBorderEndShorthand() {
  static constexpr CSSPropertyID webkitBorderEndProperties[] = {
    CSSPropertyWebkitBorderEndWidth,
    CSSPropertyWebkitBorderEndStyle,
    CSSPropertyWebkitBorderEndColor,
  };

  static constexpr StylePropertyShorthand webkitBorderEndLonghands (CSSPropertyWebkitBorderEnd, webkitBorderEndProperties, WTF_ARRAY_LENGTH(webkitBorderEndProperties));
  return webkitBorderEndLonghands;
}

const StylePropertyShorthand& scrollSnapMarginShorthand() {
  static constexpr CSSPropertyID scrollSnapMarginProperties[] = {
    CSSPropertyScrollSnapMarginTop,
    CSSPropertyScrollSnapMarginRight,
    CSSPropertyScrollSnapMarginBottom,
    CSSPropertyScrollSnapMarginLeft,
  };

  static constexpr StylePropertyShorthand scrollSnapMarginLonghands (CSSPropertyScrollSnapMargin, scrollSnapMarginProperties, WTF_ARRAY_LENGTH(scrollSnapMarginProperties));
  return scrollSnapMarginLonghands;
}

const StylePropertyShorthand& borderTopShorthand() {
  static constexpr CSSPropertyID borderTopProperties[] = {
    CSSPropertyBorderTopWidth,
    CSSPropertyBorderTopStyle,
    CSSPropertyBorderTopColor,
  };

  static constexpr StylePropertyShorthand borderTopLonghands (CSSPropertyBorderTop, borderTopProperties, WTF_ARRAY_LENGTH(borderTopProperties));
  return borderTopLonghands;
}

const StylePropertyShorthand& webkitColumnBreakAfterShorthand() {
  static constexpr CSSPropertyID webkitColumnBreakAfterProperties[] = {
    CSSPropertyBreakAfter,
  };

  static constexpr StylePropertyShorthand webkitColumnBreakAfterLonghands (CSSPropertyWebkitColumnBreakAfter, webkitColumnBreakAfterProperties, WTF_ARRAY_LENGTH(webkitColumnBreakAfterProperties));
  return webkitColumnBreakAfterLonghands;
}

const StylePropertyShorthand& webkitBorderAfterShorthand() {
  static constexpr CSSPropertyID webkitBorderAfterProperties[] = {
    CSSPropertyWebkitBorderAfterWidth,
    CSSPropertyWebkitBorderAfterStyle,
    CSSPropertyWebkitBorderAfterColor,
  };

  static constexpr StylePropertyShorthand webkitBorderAfterLonghands (CSSPropertyWebkitBorderAfter, webkitBorderAfterProperties, WTF_ARRAY_LENGTH(webkitBorderAfterProperties));
  return webkitBorderAfterLonghands;
}

const StylePropertyShorthand& outlineShorthand() {
  static constexpr CSSPropertyID outlineProperties[] = {
    CSSPropertyOutlineColor,
    CSSPropertyOutlineStyle,
    CSSPropertyOutlineWidth,
  };

  static constexpr StylePropertyShorthand outlineLonghands (CSSPropertyOutline, outlineProperties, WTF_ARRAY_LENGTH(outlineProperties));
  return outlineLonghands;
}

const StylePropertyShorthand& gridAreaShorthand() {
  static constexpr CSSPropertyID gridAreaProperties[] = {
    CSSPropertyGridRowStart,
    CSSPropertyGridColumnStart,
    CSSPropertyGridRowEnd,
    CSSPropertyGridColumnEnd,
  };

  static constexpr StylePropertyShorthand gridAreaLonghands (CSSPropertyGridArea, gridAreaProperties, WTF_ARRAY_LENGTH(gridAreaProperties));
  return gridAreaLonghands;
}

const StylePropertyShorthand& fontShorthand() {
  static constexpr CSSPropertyID fontProperties[] = {
    CSSPropertyFontStyle,
    CSSPropertyFontVariantLigatures,
    CSSPropertyFontVariantCaps,
    CSSPropertyFontVariantNumeric,
    CSSPropertyFontWeight,
    CSSPropertyFontStretch,
    CSSPropertyFontSize,
    CSSPropertyLineHeight,
    CSSPropertyFontFamily,
  };

  static constexpr StylePropertyShorthand fontLonghands (CSSPropertyFont, fontProperties, WTF_ARRAY_LENGTH(fontProperties));
  return fontLonghands;
}

const StylePropertyShorthand& placeItemsShorthand() {
  static constexpr CSSPropertyID placeItemsProperties[] = {
    CSSPropertyAlignItems,
    CSSPropertyJustifyItems,
  };

  static constexpr StylePropertyShorthand placeItemsLonghands (CSSPropertyPlaceItems, placeItemsProperties, WTF_ARRAY_LENGTH(placeItemsProperties));
  return placeItemsLonghands;
}

const StylePropertyShorthand& transitionShorthand() {
  static constexpr CSSPropertyID transitionProperties[] = {
    CSSPropertyTransitionProperty,
    CSSPropertyTransitionDuration,
    CSSPropertyTransitionTimingFunction,
    CSSPropertyTransitionDelay,
  };

  static constexpr StylePropertyShorthand transitionLonghands (CSSPropertyTransition, transitionProperties, WTF_ARRAY_LENGTH(transitionProperties));
  return transitionLonghands;
}

const StylePropertyShorthand& animationShorthand() {
  static constexpr CSSPropertyID animationProperties[] = {
    CSSPropertyAnimationName,
    CSSPropertyAnimationDuration,
    CSSPropertyAnimationTimingFunction,
    CSSPropertyAnimationDelay,
    CSSPropertyAnimationIterationCount,
    CSSPropertyAnimationDirection,
    CSSPropertyAnimationFillMode,
    CSSPropertyAnimationPlayState,
  };

  static constexpr StylePropertyShorthand animationLonghands (CSSPropertyAnimation, animationProperties, WTF_ARRAY_LENGTH(animationProperties));
  return animationLonghands;
}

const StylePropertyShorthand& webkitBorderBeforeShorthand() {
  static constexpr CSSPropertyID webkitBorderBeforeProperties[] = {
    CSSPropertyWebkitBorderBeforeWidth,
    CSSPropertyWebkitBorderBeforeStyle,
    CSSPropertyWebkitBorderBeforeColor,
  };

  static constexpr StylePropertyShorthand webkitBorderBeforeLonghands (CSSPropertyWebkitBorderBefore, webkitBorderBeforeProperties, WTF_ARRAY_LENGTH(webkitBorderBeforeProperties));
  return webkitBorderBeforeLonghands;
}

const StylePropertyShorthand& columnsShorthand() {
  static constexpr CSSPropertyID columnsProperties[] = {
    CSSPropertyColumnWidth,
    CSSPropertyColumnCount,
  };

  static constexpr StylePropertyShorthand columnsLonghands (CSSPropertyColumns, columnsProperties, WTF_ARRAY_LENGTH(columnsProperties));
  return columnsLonghands;
}

const StylePropertyShorthand& fontVariantShorthand() {
  static constexpr CSSPropertyID fontVariantProperties[] = {
    CSSPropertyFontVariantLigatures,
    CSSPropertyFontVariantCaps,
    CSSPropertyFontVariantNumeric,
  };

  static constexpr StylePropertyShorthand fontVariantLonghands (CSSPropertyFontVariant, fontVariantProperties, WTF_ARRAY_LENGTH(fontVariantProperties));
  return fontVariantLonghands;
}

const StylePropertyShorthand& paddingShorthand() {
  static constexpr CSSPropertyID paddingProperties[] = {
    CSSPropertyPaddingTop,
    CSSPropertyPaddingRight,
    CSSPropertyPaddingBottom,
    CSSPropertyPaddingLeft,
  };

  static constexpr StylePropertyShorthand paddingLonghands (CSSPropertyPadding, paddingProperties, WTF_ARRAY_LENGTH(paddingProperties));
  return paddingLonghands;
}

const StylePropertyShorthand& webkitMaskPositionShorthand() {
  static constexpr CSSPropertyID webkitMaskPositionProperties[] = {
    CSSPropertyWebkitMaskPositionX,
    CSSPropertyWebkitMaskPositionY,
  };

  static constexpr StylePropertyShorthand webkitMaskPositionLonghands (CSSPropertyWebkitMaskPosition, webkitMaskPositionProperties, WTF_ARRAY_LENGTH(webkitMaskPositionProperties));
  return webkitMaskPositionLonghands;
}

const StylePropertyShorthand& borderLeftShorthand() {
  static constexpr CSSPropertyID borderLeftProperties[] = {
    CSSPropertyBorderLeftWidth,
    CSSPropertyBorderLeftStyle,
    CSSPropertyBorderLeftColor,
  };

  static constexpr StylePropertyShorthand borderLeftLonghands (CSSPropertyBorderLeft, borderLeftProperties, WTF_ARRAY_LENGTH(borderLeftProperties));
  return borderLeftLonghands;
}

const StylePropertyShorthand& webkitMaskShorthand() {
  static constexpr CSSPropertyID webkitMaskProperties[] = {
    CSSPropertyWebkitMaskImage,
    CSSPropertyWebkitMaskPositionX,
    CSSPropertyWebkitMaskPositionY,
    CSSPropertyWebkitMaskSize,
    CSSPropertyWebkitMaskRepeatX,
    CSSPropertyWebkitMaskRepeatY,
    CSSPropertyWebkitMaskOrigin,
    CSSPropertyWebkitMaskClip,
  };

  static constexpr StylePropertyShorthand webkitMaskLonghands (CSSPropertyWebkitMask, webkitMaskProperties, WTF_ARRAY_LENGTH(webkitMaskProperties));
  return webkitMaskLonghands;
}

const StylePropertyShorthand& gridColumnShorthand() {
  static constexpr CSSPropertyID gridColumnProperties[] = {
    CSSPropertyGridColumnStart,
    CSSPropertyGridColumnEnd,
  };

  static constexpr StylePropertyShorthand gridColumnLonghands (CSSPropertyGridColumn, gridColumnProperties, WTF_ARRAY_LENGTH(gridColumnProperties));
  return gridColumnLonghands;
}

const StylePropertyShorthand& webkitBorderStartShorthand() {
  static constexpr CSSPropertyID webkitBorderStartProperties[] = {
    CSSPropertyWebkitBorderStartWidth,
    CSSPropertyWebkitBorderStartStyle,
    CSSPropertyWebkitBorderStartColor,
  };

  static constexpr StylePropertyShorthand webkitBorderStartLonghands (CSSPropertyWebkitBorderStart, webkitBorderStartProperties, WTF_ARRAY_LENGTH(webkitBorderStartProperties));
  return webkitBorderStartLonghands;
}

const StylePropertyShorthand& offsetShorthand() {
  static constexpr CSSPropertyID offsetProperties[] = {
    CSSPropertyOffsetPosition,
    CSSPropertyOffsetPath,
    CSSPropertyOffsetDistance,
    CSSPropertyOffsetRotate,
    CSSPropertyOffsetAnchor,
  };

  static constexpr StylePropertyShorthand offsetLonghands (CSSPropertyOffset, offsetProperties, WTF_ARRAY_LENGTH(offsetProperties));
  return offsetLonghands;
}

const StylePropertyShorthand& borderColorShorthand() {
  static constexpr CSSPropertyID borderColorProperties[] = {
    CSSPropertyBorderTopColor,
    CSSPropertyBorderRightColor,
    CSSPropertyBorderBottomColor,
    CSSPropertyBorderLeftColor,
  };

  static constexpr StylePropertyShorthand borderColorLonghands (CSSPropertyBorderColor, borderColorProperties, WTF_ARRAY_LENGTH(borderColorProperties));
  return borderColorLonghands;
}

const StylePropertyShorthand& webkitColumnBreakBeforeShorthand() {
  static constexpr CSSPropertyID webkitColumnBreakBeforeProperties[] = {
    CSSPropertyBreakBefore,
  };

  static constexpr StylePropertyShorthand webkitColumnBreakBeforeLonghands (CSSPropertyWebkitColumnBreakBefore, webkitColumnBreakBeforeProperties, WTF_ARRAY_LENGTH(webkitColumnBreakBeforeProperties));
  return webkitColumnBreakBeforeLonghands;
}

const StylePropertyShorthand& webkitColumnBreakInsideShorthand() {
  static constexpr CSSPropertyID webkitColumnBreakInsideProperties[] = {
    CSSPropertyBreakInside,
  };

  static constexpr StylePropertyShorthand webkitColumnBreakInsideLonghands (CSSPropertyWebkitColumnBreakInside, webkitColumnBreakInsideProperties, WTF_ARRAY_LENGTH(webkitColumnBreakInsideProperties));
  return webkitColumnBreakInsideLonghands;
}

const StylePropertyShorthand& borderWidthShorthand() {
  static constexpr CSSPropertyID borderWidthProperties[] = {
    CSSPropertyBorderTopWidth,
    CSSPropertyBorderRightWidth,
    CSSPropertyBorderBottomWidth,
    CSSPropertyBorderLeftWidth,
  };

  static constexpr StylePropertyShorthand borderWidthLonghands (CSSPropertyBorderWidth, borderWidthProperties, WTF_ARRAY_LENGTH(borderWidthProperties));
  return borderWidthLonghands;
}

const StylePropertyShorthand& gridTemplateShorthand() {
  static constexpr CSSPropertyID gridTemplateProperties[] = {
    CSSPropertyGridTemplateRows,
    CSSPropertyGridTemplateColumns,
    CSSPropertyGridTemplateAreas,
  };

  static constexpr StylePropertyShorthand gridTemplateLonghands (CSSPropertyGridTemplate, gridTemplateProperties, WTF_ARRAY_LENGTH(gridTemplateProperties));
  return gridTemplateLonghands;
}

const StylePropertyShorthand& placeContentShorthand() {
  static constexpr CSSPropertyID placeContentProperties[] = {
    CSSPropertyAlignContent,
    CSSPropertyJustifyContent,
  };

  static constexpr StylePropertyShorthand placeContentLonghands (CSSPropertyPlaceContent, placeContentProperties, WTF_ARRAY_LENGTH(placeContentProperties));
  return placeContentLonghands;
}

const StylePropertyShorthand& backgroundPositionShorthand() {
  static constexpr CSSPropertyID backgroundPositionProperties[] = {
    CSSPropertyBackgroundPositionX,
    CSSPropertyBackgroundPositionY,
  };

  static constexpr StylePropertyShorthand backgroundPositionLonghands (CSSPropertyBackgroundPosition, backgroundPositionProperties, WTF_ARRAY_LENGTH(backgroundPositionProperties));
  return backgroundPositionLonghands;
}

const StylePropertyShorthand& borderShorthand() {
  static constexpr CSSPropertyID borderProperties[] = {
    CSSPropertyBorderTopColor,
    CSSPropertyBorderTopStyle,
    CSSPropertyBorderTopWidth,
    CSSPropertyBorderRightColor,
    CSSPropertyBorderRightStyle,
    CSSPropertyBorderRightWidth,
    CSSPropertyBorderBottomColor,
    CSSPropertyBorderBottomStyle,
    CSSPropertyBorderBottomWidth,
    CSSPropertyBorderLeftColor,
    CSSPropertyBorderLeftStyle,
    CSSPropertyBorderLeftWidth,
    CSSPropertyBorderImageSource,
    CSSPropertyBorderImageSlice,
    CSSPropertyBorderImageWidth,
    CSSPropertyBorderImageOutset,
    CSSPropertyBorderImageRepeat,
  };

  static constexpr StylePropertyShorthand borderLonghands (CSSPropertyBorder, borderProperties, WTF_ARRAY_LENGTH(borderProperties));
  return borderLonghands;
}

const StylePropertyShorthand& borderBottomShorthand() {
  static constexpr CSSPropertyID borderBottomProperties[] = {
    CSSPropertyBorderBottomWidth,
    CSSPropertyBorderBottomStyle,
    CSSPropertyBorderBottomColor,
  };

  static constexpr StylePropertyShorthand borderBottomLonghands (CSSPropertyBorderBottom, borderBottomProperties, WTF_ARRAY_LENGTH(borderBottomProperties));
  return borderBottomLonghands;
}

const StylePropertyShorthand& gridRowShorthand() {
  static constexpr CSSPropertyID gridRowProperties[] = {
    CSSPropertyGridRowStart,
    CSSPropertyGridRowEnd,
  };

  static constexpr StylePropertyShorthand gridRowLonghands (CSSPropertyGridRow, gridRowProperties, WTF_ARRAY_LENGTH(gridRowProperties));
  return gridRowLonghands;
}

const StylePropertyShorthand& pageBreakAfterShorthand() {
  static constexpr CSSPropertyID pageBreakAfterProperties[] = {
    CSSPropertyBreakAfter,
  };

  static constexpr StylePropertyShorthand pageBreakAfterLonghands (CSSPropertyPageBreakAfter, pageBreakAfterProperties, WTF_ARRAY_LENGTH(pageBreakAfterProperties));
  return pageBreakAfterLonghands;
}

const StylePropertyShorthand& borderRadiusShorthand() {
  static constexpr CSSPropertyID borderRadiusProperties[] = {
    CSSPropertyBorderTopLeftRadius,
    CSSPropertyBorderTopRightRadius,
    CSSPropertyBorderBottomRightRadius,
    CSSPropertyBorderBottomLeftRadius,
  };

  static constexpr StylePropertyShorthand borderRadiusLonghands (CSSPropertyBorderRadius, borderRadiusProperties, WTF_ARRAY_LENGTH(borderRadiusProperties));
  return borderRadiusLonghands;
}

const StylePropertyShorthand& webkitTextEmphasisShorthand() {
  static constexpr CSSPropertyID webkitTextEmphasisProperties[] = {
    CSSPropertyWebkitTextEmphasisStyle,
    CSSPropertyWebkitTextEmphasisColor,
  };

  static constexpr StylePropertyShorthand webkitTextEmphasisLonghands (CSSPropertyWebkitTextEmphasis, webkitTextEmphasisProperties, WTF_ARRAY_LENGTH(webkitTextEmphasisProperties));
  return webkitTextEmphasisLonghands;
}

const StylePropertyShorthand& webkitMaskBoxImageShorthand() {
  static constexpr CSSPropertyID webkitMaskBoxImageProperties[] = {
    CSSPropertyWebkitMaskBoxImageSource,
    CSSPropertyWebkitMaskBoxImageSlice,
    CSSPropertyWebkitMaskBoxImageWidth,
    CSSPropertyWebkitMaskBoxImageOutset,
    CSSPropertyWebkitMaskBoxImageRepeat,
  };

  static constexpr StylePropertyShorthand webkitMaskBoxImageLonghands (CSSPropertyWebkitMaskBoxImage, webkitMaskBoxImageProperties, WTF_ARRAY_LENGTH(webkitMaskBoxImageProperties));
  return webkitMaskBoxImageLonghands;
}

const StylePropertyShorthand& borderImageShorthand() {
  static constexpr CSSPropertyID borderImageProperties[] = {
    CSSPropertyBorderImageSource,
    CSSPropertyBorderImageSlice,
    CSSPropertyBorderImageWidth,
    CSSPropertyBorderImageOutset,
    CSSPropertyBorderImageRepeat,
  };

  static constexpr StylePropertyShorthand borderImageLonghands (CSSPropertyBorderImage, borderImageProperties, WTF_ARRAY_LENGTH(borderImageProperties));
  return borderImageLonghands;
}

// TODO(ericwilligers): Retire this when offset-position and offset-anchor ship
const StylePropertyShorthand& offsetShorthandWithoutPositionAnchor() {
  static const CSSPropertyID offsetProperties[] = {
    CSSPropertyOffsetPath,
    CSSPropertyOffsetDistance,
    CSSPropertyOffsetRotate,
  };
  DEFINE_STATIC_LOCAL(StylePropertyShorthand, offsetLonghands, (CSSPropertyOffset, offsetProperties, WTF_ARRAY_LENGTH(offsetProperties)));
  return offsetLonghands;
}

// Returns an empty list if the property is not a shorthand
const StylePropertyShorthand& shorthandForProperty(CSSPropertyID propertyID) {
  // FIXME: We shouldn't switch between shorthand/not shorthand based on a runtime flag
  static constexpr StylePropertyShorthand emptyShorthand;

  if (propertyID == CSSPropertyTextDecoration &&
      !RuntimeEnabledFeatures::CSS3TextDecorationsEnabled())
    return emptyShorthand;
  if (propertyID == CSSPropertyOffset &&
      !RuntimeEnabledFeatures::CSSOffsetPositionAnchorEnabled())
    return offsetShorthandWithoutPositionAnchor();
  switch (propertyID) {
    case CSSPropertyBackgroundRepeat:
      return backgroundRepeatShorthand();
    case CSSPropertyScrollPaddingBlock:
      return scrollPaddingBlockShorthand();
    case CSSPropertyFlex:
      return flexShorthand();
    case CSSPropertyMarker:
      return markerShorthand();
    case CSSPropertyFlexFlow:
      return flexFlowShorthand();
    case CSSPropertyBorderRight:
      return borderRightShorthand();
    case CSSPropertyBorderStyle:
      return borderStyleShorthand();
    case CSSPropertyOverflow:
      return overflowShorthand();
    case CSSPropertyScrollPaddingInline:
      return scrollPaddingInlineShorthand();
    case CSSPropertyWebkitTextStroke:
      return webkitTextStrokeShorthand();
    case CSSPropertyScrollPadding:
      return scrollPaddingShorthand();
    case CSSPropertyPageBreakInside:
      return pageBreakInsideShorthand();
    case CSSPropertyListStyle:
      return listStyleShorthand();
    case CSSPropertyScrollSnapMarginInline:
      return scrollSnapMarginInlineShorthand();
    case CSSPropertyScrollSnapMarginBlock:
      return scrollSnapMarginBlockShorthand();
    case CSSPropertyPlaceSelf:
      return placeSelfShorthand();
    case CSSPropertyWebkitMarginCollapse:
      return webkitMarginCollapseShorthand();
    case CSSPropertyMargin:
      return marginShorthand();
    case CSSPropertyScrollBoundaryBehavior:
      return scrollBoundaryBehaviorShorthand();
    case CSSPropertyPageBreakBefore:
      return pageBreakBeforeShorthand();
    case CSSPropertyBorderSpacing:
      return borderSpacingShorthand();
    case CSSPropertyTextDecoration:
      return textDecorationShorthand();
    case CSSPropertyGridGap:
      return gridGapShorthand();
    case CSSPropertyBackground:
      return backgroundShorthand();
    case CSSPropertyWebkitMaskRepeat:
      return webkitMaskRepeatShorthand();
    case CSSPropertyColumnRule:
      return columnRuleShorthand();
    case CSSPropertyGrid:
      return gridShorthand();
    case CSSPropertyWebkitBorderEnd:
      return webkitBorderEndShorthand();
    case CSSPropertyScrollSnapMargin:
      return scrollSnapMarginShorthand();
    case CSSPropertyBorderTop:
      return borderTopShorthand();
    case CSSPropertyWebkitColumnBreakAfter:
      return webkitColumnBreakAfterShorthand();
    case CSSPropertyWebkitBorderAfter:
      return webkitBorderAfterShorthand();
    case CSSPropertyOutline:
      return outlineShorthand();
    case CSSPropertyGridArea:
      return gridAreaShorthand();
    case CSSPropertyFont:
      return fontShorthand();
    case CSSPropertyPlaceItems:
      return placeItemsShorthand();
    case CSSPropertyTransition:
      return transitionShorthand();
    case CSSPropertyAnimation:
      return animationShorthand();
    case CSSPropertyWebkitBorderBefore:
      return webkitBorderBeforeShorthand();
    case CSSPropertyColumns:
      return columnsShorthand();
    case CSSPropertyFontVariant:
      return fontVariantShorthand();
    case CSSPropertyPadding:
      return paddingShorthand();
    case CSSPropertyWebkitMaskPosition:
      return webkitMaskPositionShorthand();
    case CSSPropertyBorderLeft:
      return borderLeftShorthand();
    case CSSPropertyWebkitMask:
      return webkitMaskShorthand();
    case CSSPropertyGridColumn:
      return gridColumnShorthand();
    case CSSPropertyWebkitBorderStart:
      return webkitBorderStartShorthand();
    case CSSPropertyOffset:
      return offsetShorthand();
    case CSSPropertyBorderColor:
      return borderColorShorthand();
    case CSSPropertyWebkitColumnBreakBefore:
      return webkitColumnBreakBeforeShorthand();
    case CSSPropertyWebkitColumnBreakInside:
      return webkitColumnBreakInsideShorthand();
    case CSSPropertyBorderWidth:
      return borderWidthShorthand();
    case CSSPropertyGridTemplate:
      return gridTemplateShorthand();
    case CSSPropertyPlaceContent:
      return placeContentShorthand();
    case CSSPropertyBackgroundPosition:
      return backgroundPositionShorthand();
    case CSSPropertyBorder:
      return borderShorthand();
    case CSSPropertyBorderBottom:
      return borderBottomShorthand();
    case CSSPropertyGridRow:
      return gridRowShorthand();
    case CSSPropertyPageBreakAfter:
      return pageBreakAfterShorthand();
    case CSSPropertyBorderRadius:
      return borderRadiusShorthand();
    case CSSPropertyWebkitTextEmphasis:
      return webkitTextEmphasisShorthand();
    case CSSPropertyWebkitMaskBoxImage:
      return webkitMaskBoxImageShorthand();
    case CSSPropertyBorderImage:
      return borderImageShorthand();
    default: {
      return emptyShorthand;
    }
  }
}

void getMatchingShorthandsForLonghand(
    CSSPropertyID propertyID, Vector<StylePropertyShorthand, 4>* result) {
  DCHECK(!result->size());
  switch (propertyID) {
    case CSSPropertyScrollSnapMarginLeft: {
      result->UncheckedAppend(scrollSnapMarginShorthand());
      break;
    }
    case CSSPropertyWebkitMaskBoxImageWidth: {
      result->UncheckedAppend(webkitMaskBoxImageShorthand());
      break;
    }
    case CSSPropertyWebkitMaskBoxImageRepeat: {
      result->UncheckedAppend(webkitMaskBoxImageShorthand());
      break;
    }
    case CSSPropertyWebkitBorderEndColor: {
      result->UncheckedAppend(webkitBorderEndShorthand());
      break;
    }
    case CSSPropertyFlexWrap: {
      result->UncheckedAppend(flexFlowShorthand());
      break;
    }
    case CSSPropertyWebkitBorderHorizontalSpacing: {
      result->UncheckedAppend(borderSpacingShorthand());
      break;
    }
    case CSSPropertyScrollPaddingInlineStart: {
      result->UncheckedAppend(scrollPaddingInlineShorthand());
      break;
    }
    case CSSPropertyScrollSnapMarginBottom: {
      result->UncheckedAppend(scrollSnapMarginShorthand());
      break;
    }
    case CSSPropertyWebkitBorderEndWidth: {
      result->UncheckedAppend(webkitBorderEndShorthand());
      break;
    }
    case CSSPropertyFlexBasis: {
      result->UncheckedAppend(flexShorthand());
      break;
    }
    case CSSPropertyBorderLeftColor: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderColorShorthand());
      result->UncheckedAppend(borderLeftShorthand());
      break;
    }
    case CSSPropertyColumnCount: {
      result->UncheckedAppend(columnsShorthand());
      break;
    }
    case CSSPropertyWebkitBorderBeforeStyle: {
      result->UncheckedAppend(webkitBorderBeforeShorthand());
      break;
    }
    case CSSPropertyGridRowGap: {
      result->UncheckedAppend(gridShorthand());
      result->UncheckedAppend(gridGapShorthand());
      break;
    }
    case CSSPropertyFontStretch: {
      result->UncheckedAppend(fontShorthand());
      break;
    }
    case CSSPropertyScrollPaddingLeft: {
      result->UncheckedAppend(scrollPaddingShorthand());
      break;
    }
    case CSSPropertyWebkitBorderStartColor: {
      result->UncheckedAppend(webkitBorderStartShorthand());
      break;
    }
    case CSSPropertyFlexShrink: {
      result->UncheckedAppend(flexShorthand());
      break;
    }
    case CSSPropertyTransitionDelay: {
      result->UncheckedAppend(transitionShorthand());
      break;
    }
    case CSSPropertyGridTemplateRows: {
      result->UncheckedAppend(gridShorthand());
      result->UncheckedAppend(gridTemplateShorthand());
      break;
    }
    case CSSPropertyBackgroundClip: {
      result->UncheckedAppend(backgroundShorthand());
      break;
    }
    case CSSPropertyWebkitBorderEndStyle: {
      result->UncheckedAppend(webkitBorderEndShorthand());
      break;
    }
    case CSSPropertyTextDecorationColor: {
      result->UncheckedAppend(textDecorationShorthand());
      break;
    }
    case CSSPropertyAnimationName: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyBackgroundPositionY: {
      result->UncheckedAppend(backgroundShorthand());
      result->UncheckedAppend(backgroundPositionShorthand());
      break;
    }
    case CSSPropertyBackgroundPositionX: {
      result->UncheckedAppend(backgroundShorthand());
      result->UncheckedAppend(backgroundPositionShorthand());
      break;
    }
    case CSSPropertyScrollSnapMarginRight: {
      result->UncheckedAppend(scrollSnapMarginShorthand());
      break;
    }
    case CSSPropertyWebkitMarginAfterCollapse: {
      result->UncheckedAppend(webkitMarginCollapseShorthand());
      break;
    }
    case CSSPropertyOutlineWidth: {
      result->UncheckedAppend(outlineShorthand());
      break;
    }
    case CSSPropertyColumnWidth: {
      result->UncheckedAppend(columnsShorthand());
      break;
    }
    case CSSPropertyColumnRuleWidth: {
      result->UncheckedAppend(columnRuleShorthand());
      break;
    }
    case CSSPropertyWebkitMaskOrigin: {
      result->UncheckedAppend(webkitMaskShorthand());
      break;
    }
    case CSSPropertyBreakAfter: {
      result->UncheckedAppend(webkitColumnBreakAfterShorthand());
      result->UncheckedAppend(pageBreakAfterShorthand());
      break;
    }
    case CSSPropertyScrollPaddingBlockStart: {
      result->UncheckedAppend(scrollPaddingBlockShorthand());
      break;
    }
    case CSSPropertyScrollSnapMarginInlineStart: {
      result->UncheckedAppend(scrollSnapMarginInlineShorthand());
      break;
    }
    case CSSPropertyGridAutoFlow: {
      result->UncheckedAppend(gridShorthand());
      break;
    }
    case CSSPropertyMarkerEnd: {
      result->UncheckedAppend(markerShorthand());
      break;
    }
    case CSSPropertyBackgroundOrigin: {
      result->UncheckedAppend(backgroundShorthand());
      break;
    }
    case CSSPropertyOffsetRotate: {
      result->UncheckedAppend(offsetShorthand());
      break;
    }
    case CSSPropertyScrollSnapMarginBlockStart: {
      result->UncheckedAppend(scrollSnapMarginBlockShorthand());
      break;
    }
    case CSSPropertyBorderRightColor: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderColorShorthand());
      result->UncheckedAppend(borderRightShorthand());
      break;
    }
    case CSSPropertyJustifySelf: {
      result->UncheckedAppend(placeSelfShorthand());
      break;
    }
    case CSSPropertyAnimationDuration: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyAnimationPlayState: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyOffsetAnchor: {
      result->UncheckedAppend(offsetShorthand());
      break;
    }
    case CSSPropertyFontVariantNumeric: {
      result->UncheckedAppend(fontShorthand());
      result->UncheckedAppend(fontVariantShorthand());
      break;
    }
    case CSSPropertyBorderBottomStyle: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderStyleShorthand());
      result->UncheckedAppend(borderBottomShorthand());
      break;
    }
    case CSSPropertyBorderBottomWidth: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderWidthShorthand());
      result->UncheckedAppend(borderBottomShorthand());
      break;
    }
    case CSSPropertyBorderTopRightRadius: {
      result->UncheckedAppend(borderRadiusShorthand());
      break;
    }
    case CSSPropertyScrollSnapMarginInlineEnd: {
      result->UncheckedAppend(scrollSnapMarginInlineShorthand());
      break;
    }
    case CSSPropertyFontStyle: {
      result->UncheckedAppend(fontShorthand());
      break;
    }
    case CSSPropertyBorderRightStyle: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderStyleShorthand());
      result->UncheckedAppend(borderRightShorthand());
      break;
    }
    case CSSPropertyBorderBottomRightRadius: {
      result->UncheckedAppend(borderRadiusShorthand());
      break;
    }
    case CSSPropertyBorderRightWidth: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderWidthShorthand());
      result->UncheckedAppend(borderRightShorthand());
      break;
    }
    case CSSPropertyWebkitBorderAfterStyle: {
      result->UncheckedAppend(webkitBorderAfterShorthand());
      break;
    }
    case CSSPropertyBorderImageSource: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderImageShorthand());
      break;
    }
    case CSSPropertyScrollSnapMarginBlockEnd: {
      result->UncheckedAppend(scrollSnapMarginBlockShorthand());
      break;
    }
    case CSSPropertyBorderBottomLeftRadius: {
      result->UncheckedAppend(borderRadiusShorthand());
      break;
    }
    case CSSPropertyGridRowEnd: {
      result->UncheckedAppend(gridAreaShorthand());
      result->UncheckedAppend(gridRowShorthand());
      break;
    }
    case CSSPropertyOutlineStyle: {
      result->UncheckedAppend(outlineShorthand());
      break;
    }
    case CSSPropertyBackgroundAttachment: {
      result->UncheckedAppend(backgroundShorthand());
      break;
    }
    case CSSPropertyWebkitMaskBoxImageOutset: {
      result->UncheckedAppend(webkitMaskBoxImageShorthand());
      break;
    }
    case CSSPropertyJustifyContent: {
      result->UncheckedAppend(placeContentShorthand());
      break;
    }
    case CSSPropertyScrollPaddingBottom: {
      result->UncheckedAppend(scrollPaddingShorthand());
      break;
    }
    case CSSPropertyBorderLeftWidth: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderWidthShorthand());
      result->UncheckedAppend(borderLeftShorthand());
      break;
    }
    case CSSPropertyPaddingBottom: {
      result->UncheckedAppend(paddingShorthand());
      break;
    }
    case CSSPropertyWebkitMaskPositionX: {
      result->UncheckedAppend(webkitMaskShorthand());
      result->UncheckedAppend(webkitMaskPositionShorthand());
      break;
    }
    case CSSPropertyMarkerStart: {
      result->UncheckedAppend(markerShorthand());
      break;
    }
    case CSSPropertyBackgroundImage: {
      result->UncheckedAppend(backgroundShorthand());
      break;
    }
    case CSSPropertyWebkitBorderAfterColor: {
      result->UncheckedAppend(webkitBorderAfterShorthand());
      break;
    }
    case CSSPropertyTextDecorationLine: {
      result->UncheckedAppend(textDecorationShorthand());
      break;
    }
    case CSSPropertyWebkitMaskClip: {
      result->UncheckedAppend(webkitMaskShorthand());
      break;
    }
    case CSSPropertyTextDecorationStyle: {
      result->UncheckedAppend(textDecorationShorthand());
      break;
    }
    case CSSPropertyScrollPaddingBlockEnd: {
      result->UncheckedAppend(scrollPaddingBlockShorthand());
      break;
    }
    case CSSPropertyColumnRuleStyle: {
      result->UncheckedAppend(columnRuleShorthand());
      break;
    }
    case CSSPropertyOffsetPath: {
      result->UncheckedAppend(offsetShorthand());
      break;
    }
    case CSSPropertyBorderImageSlice: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderImageShorthand());
      break;
    }
    case CSSPropertyWebkitBorderAfterWidth: {
      result->UncheckedAppend(webkitBorderAfterShorthand());
      break;
    }
    case CSSPropertyBreakInside: {
      result->UncheckedAppend(webkitColumnBreakInsideShorthand());
      result->UncheckedAppend(pageBreakInsideShorthand());
      break;
    }
    case CSSPropertyBorderTopLeftRadius: {
      result->UncheckedAppend(borderRadiusShorthand());
      break;
    }
    case CSSPropertyBorderTopColor: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderColorShorthand());
      result->UncheckedAppend(borderTopShorthand());
      break;
    }
    case CSSPropertyBackgroundColor: {
      result->UncheckedAppend(backgroundShorthand());
      break;
    }
    case CSSPropertyOutlineColor: {
      result->UncheckedAppend(outlineShorthand());
      break;
    }
    case CSSPropertyPaddingTop: {
      result->UncheckedAppend(paddingShorthand());
      break;
    }
    case CSSPropertyBorderTopWidth: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderWidthShorthand());
      result->UncheckedAppend(borderTopShorthand());
      break;
    }
    case CSSPropertyGridColumnStart: {
      result->UncheckedAppend(gridAreaShorthand());
      result->UncheckedAppend(gridColumnShorthand());
      break;
    }
    case CSSPropertyMarginBottom: {
      result->UncheckedAppend(marginShorthand());
      break;
    }
    case CSSPropertyScrollBoundaryBehaviorY: {
      result->UncheckedAppend(scrollBoundaryBehaviorShorthand());
      break;
    }
    case CSSPropertyScrollBoundaryBehaviorX: {
      result->UncheckedAppend(scrollBoundaryBehaviorShorthand());
      break;
    }
    case CSSPropertyListStylePosition: {
      result->UncheckedAppend(listStyleShorthand());
      break;
    }
    case CSSPropertyOffsetPosition: {
      result->UncheckedAppend(offsetShorthand());
      break;
    }
    case CSSPropertyAnimationIterationCount: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyWebkitTextEmphasisColor: {
      result->UncheckedAppend(webkitTextEmphasisShorthand());
      break;
    }
    case CSSPropertyWebkitTextEmphasisStyle: {
      result->UncheckedAppend(webkitTextEmphasisShorthand());
      break;
    }
    case CSSPropertyWebkitMaskPositionY: {
      result->UncheckedAppend(webkitMaskShorthand());
      result->UncheckedAppend(webkitMaskPositionShorthand());
      break;
    }
    case CSSPropertyWebkitMaskSize: {
      result->UncheckedAppend(webkitMaskShorthand());
      break;
    }
    case CSSPropertyGridAutoRows: {
      result->UncheckedAppend(gridShorthand());
      break;
    }
    case CSSPropertyMarginTop: {
      result->UncheckedAppend(marginShorthand());
      break;
    }
    case CSSPropertyBorderTopStyle: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderStyleShorthand());
      result->UncheckedAppend(borderTopShorthand());
      break;
    }
    case CSSPropertyBreakBefore: {
      result->UncheckedAppend(webkitColumnBreakBeforeShorthand());
      result->UncheckedAppend(pageBreakBeforeShorthand());
      break;
    }
    case CSSPropertyWebkitBorderBeforeWidth: {
      result->UncheckedAppend(webkitBorderBeforeShorthand());
      break;
    }
    case CSSPropertyBorderImageOutset: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderImageShorthand());
      break;
    }
    case CSSPropertyFontFamily: {
      result->UncheckedAppend(fontShorthand());
      break;
    }
    case CSSPropertyBorderImageWidth: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderImageShorthand());
      break;
    }
    case CSSPropertyWebkitMaskBoxImageSource: {
      result->UncheckedAppend(webkitMaskBoxImageShorthand());
      break;
    }
    case CSSPropertyScrollSnapMarginTop: {
      result->UncheckedAppend(scrollSnapMarginShorthand());
      break;
    }
    case CSSPropertyFlexGrow: {
      result->UncheckedAppend(flexShorthand());
      break;
    }
    case CSSPropertyWebkitBorderBeforeColor: {
      result->UncheckedAppend(webkitBorderBeforeShorthand());
      break;
    }
    case CSSPropertyAlignContent: {
      result->UncheckedAppend(placeContentShorthand());
      break;
    }
    case CSSPropertyListStyleImage: {
      result->UncheckedAppend(listStyleShorthand());
      break;
    }
    case CSSPropertyWebkitMarginBeforeCollapse: {
      result->UncheckedAppend(webkitMarginCollapseShorthand());
      break;
    }
    case CSSPropertyWebkitMaskImage: {
      result->UncheckedAppend(webkitMaskShorthand());
      break;
    }
    case CSSPropertyAnimationTimingFunction: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyFontVariantCaps: {
      result->UncheckedAppend(fontShorthand());
      result->UncheckedAppend(fontVariantShorthand());
      break;
    }
    case CSSPropertyFlexDirection: {
      result->UncheckedAppend(flexFlowShorthand());
      break;
    }
    case CSSPropertyWebkitMaskRepeatY: {
      result->UncheckedAppend(webkitMaskShorthand());
      result->UncheckedAppend(webkitMaskRepeatShorthand());
      break;
    }
    case CSSPropertyWebkitMaskRepeatX: {
      result->UncheckedAppend(webkitMaskShorthand());
      result->UncheckedAppend(webkitMaskRepeatShorthand());
      break;
    }
    case CSSPropertyScrollPaddingTop: {
      result->UncheckedAppend(scrollPaddingShorthand());
      break;
    }
    case CSSPropertyBorderImageRepeat: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderImageShorthand());
      break;
    }
    case CSSPropertyScrollPaddingRight: {
      result->UncheckedAppend(scrollPaddingShorthand());
      break;
    }
    case CSSPropertyWebkitBorderStartStyle: {
      result->UncheckedAppend(webkitBorderStartShorthand());
      break;
    }
    case CSSPropertyListStyleType: {
      result->UncheckedAppend(listStyleShorthand());
      break;
    }
    case CSSPropertyOverflowX: {
      result->UncheckedAppend(overflowShorthand());
      break;
    }
    case CSSPropertyFontWeight: {
      result->UncheckedAppend(fontShorthand());
      break;
    }
    case CSSPropertyGridTemplateAreas: {
      result->UncheckedAppend(gridShorthand());
      result->UncheckedAppend(gridTemplateShorthand());
      break;
    }
    case CSSPropertyBorderLeftStyle: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderStyleShorthand());
      result->UncheckedAppend(borderLeftShorthand());
      break;
    }
    case CSSPropertyOverflowY: {
      result->UncheckedAppend(overflowShorthand());
      break;
    }
    case CSSPropertyColumnRuleColor: {
      result->UncheckedAppend(columnRuleShorthand());
      break;
    }
    case CSSPropertyMarkerMid: {
      result->UncheckedAppend(markerShorthand());
      break;
    }
    case CSSPropertyMarginRight: {
      result->UncheckedAppend(marginShorthand());
      break;
    }
    case CSSPropertyWebkitBorderStartWidth: {
      result->UncheckedAppend(webkitBorderStartShorthand());
      break;
    }
    case CSSPropertyAlignItems: {
      result->UncheckedAppend(placeItemsShorthand());
      break;
    }
    case CSSPropertyAnimationDelay: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyAnimationFillMode: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyFontVariantLigatures: {
      result->UncheckedAppend(fontShorthand());
      result->UncheckedAppend(fontVariantShorthand());
      break;
    }
    case CSSPropertyGridColumnGap: {
      result->UncheckedAppend(gridShorthand());
      result->UncheckedAppend(gridGapShorthand());
      break;
    }
    case CSSPropertyOffsetDistance: {
      result->UncheckedAppend(offsetShorthand());
      break;
    }
    case CSSPropertyGridTemplateColumns: {
      result->UncheckedAppend(gridShorthand());
      result->UncheckedAppend(gridTemplateShorthand());
      break;
    }
    case CSSPropertyAnimationDirection: {
      result->UncheckedAppend(animationShorthand());
      break;
    }
    case CSSPropertyWebkitMaskBoxImageSlice: {
      result->UncheckedAppend(webkitMaskBoxImageShorthand());
      break;
    }
    case CSSPropertyWebkitTextStrokeColor: {
      result->UncheckedAppend(webkitTextStrokeShorthand());
      break;
    }
    case CSSPropertyLineHeight: {
      result->UncheckedAppend(fontShorthand());
      break;
    }
    case CSSPropertyPaddingLeft: {
      result->UncheckedAppend(paddingShorthand());
      break;
    }
    case CSSPropertyBackgroundRepeatY: {
      result->UncheckedAppend(backgroundShorthand());
      result->UncheckedAppend(backgroundRepeatShorthand());
      break;
    }
    case CSSPropertyBackgroundRepeatX: {
      result->UncheckedAppend(backgroundShorthand());
      result->UncheckedAppend(backgroundRepeatShorthand());
      break;
    }
    case CSSPropertyBorderBottomColor: {
      result->UncheckedAppend(borderShorthand());
      result->UncheckedAppend(borderColorShorthand());
      result->UncheckedAppend(borderBottomShorthand());
      break;
    }
    case CSSPropertyAlignSelf: {
      result->UncheckedAppend(placeSelfShorthand());
      break;
    }
    case CSSPropertyFontSize: {
      result->UncheckedAppend(fontShorthand());
      break;
    }
    case CSSPropertyGridColumnEnd: {
      result->UncheckedAppend(gridAreaShorthand());
      result->UncheckedAppend(gridColumnShorthand());
      break;
    }
    case CSSPropertyBackgroundSize: {
      result->UncheckedAppend(backgroundShorthand());
      break;
    }
    case CSSPropertyPaddingRight: {
      result->UncheckedAppend(paddingShorthand());
      break;
    }
    case CSSPropertyJustifyItems: {
      result->UncheckedAppend(placeItemsShorthand());
      break;
    }
    case CSSPropertyTransitionProperty: {
      result->UncheckedAppend(transitionShorthand());
      break;
    }
    case CSSPropertyGridAutoColumns: {
      result->UncheckedAppend(gridShorthand());
      break;
    }
    case CSSPropertyScrollPaddingInlineEnd: {
      result->UncheckedAppend(scrollPaddingInlineShorthand());
      break;
    }
    case CSSPropertyTransitionTimingFunction: {
      result->UncheckedAppend(transitionShorthand());
      break;
    }
    case CSSPropertyGridRowStart: {
      result->UncheckedAppend(gridAreaShorthand());
      result->UncheckedAppend(gridRowShorthand());
      break;
    }
    case CSSPropertyTransitionDuration: {
      result->UncheckedAppend(transitionShorthand());
      break;
    }
    case CSSPropertyWebkitBorderVerticalSpacing: {
      result->UncheckedAppend(borderSpacingShorthand());
      break;
    }
    case CSSPropertyMarginLeft: {
      result->UncheckedAppend(marginShorthand());
      break;
    }
    case CSSPropertyWebkitTextStrokeWidth: {
      result->UncheckedAppend(webkitTextStrokeShorthand());
      break;
    }
    default:
      break;
  }
}

} // namespace blink
