// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_BEGIN_FRAME_ARGS_MOJOM_H_
#define CC_IPC_BEGIN_FRAME_ARGS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/begin_frame_args.mojom-shared.h"
#include "mojo/common/time.mojom.h"
#include <string>
#include <vector>
#include "cc/output/begin_frame_args.h"


namespace cc {
namespace mojom {
class BeginFrameArgs;
using BeginFrameArgsPtr = mojo::StructPtr<BeginFrameArgs>;

class BeginFrameAck;
using BeginFrameAckPtr = mojo::InlinedStructPtr<BeginFrameAck>;




class  BeginFrameAck {
 public:
  using DataView = BeginFrameAckDataView;
  using Data_ = internal::BeginFrameAck_Data;

  template <typename... Args>
  static BeginFrameAckPtr New(Args&&... args) {
    return BeginFrameAckPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BeginFrameAckPtr From(const U& u) {
    return mojo::TypeConverter<BeginFrameAckPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BeginFrameAck>::Convert(*this);
  }


  BeginFrameAck();

  BeginFrameAck(
      uint64_t sequence_number,
      uint32_t source_id);

  ~BeginFrameAck();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BeginFrameAckPtr>
  BeginFrameAckPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BeginFrameAck>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BeginFrameAck::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BeginFrameAck::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint64_t sequence_number;
  uint32_t source_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  BeginFrameArgs {
 public:
  using DataView = BeginFrameArgsDataView;
  using Data_ = internal::BeginFrameArgs_Data;

  template <typename... Args>
  static BeginFrameArgsPtr New(Args&&... args) {
    return BeginFrameArgsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BeginFrameArgsPtr From(const U& u) {
    return mojo::TypeConverter<BeginFrameArgsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BeginFrameArgs>::Convert(*this);
  }


  BeginFrameArgs();

  BeginFrameArgs(
      base::TimeTicks frame_time,
      base::TimeTicks deadline,
      base::TimeDelta interval,
      uint64_t sequence_number,
      uint32_t source_id,
      BeginFrameArgsType type,
      bool on_critical_path);

  ~BeginFrameArgs();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BeginFrameArgsPtr>
  BeginFrameArgsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BeginFrameArgs>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BeginFrameArgs::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BeginFrameArgs::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::TimeTicks frame_time;
  base::TimeTicks deadline;
  base::TimeDelta interval;
  uint64_t sequence_number;
  uint32_t source_id;
  BeginFrameArgsType type;
  bool on_critical_path;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



template <typename StructPtrType>
BeginFrameArgsPtr BeginFrameArgs::Clone() const {
  return New(
      mojo::Clone(frame_time),
      mojo::Clone(deadline),
      mojo::Clone(interval),
      mojo::Clone(sequence_number),
      mojo::Clone(source_id),
      mojo::Clone(type),
      mojo::Clone(on_critical_path)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BeginFrameArgs>::value>::type*>
bool BeginFrameArgs::Equals(const T& other) const {
  if (!mojo::Equals(this->frame_time, other.frame_time))
    return false;
  if (!mojo::Equals(this->deadline, other.deadline))
    return false;
  if (!mojo::Equals(this->interval, other.interval))
    return false;
  if (!mojo::Equals(this->sequence_number, other.sequence_number))
    return false;
  if (!mojo::Equals(this->source_id, other.source_id))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->on_critical_path, other.on_critical_path))
    return false;
  return true;
}
template <typename StructPtrType>
BeginFrameAckPtr BeginFrameAck::Clone() const {
  return New(
      mojo::Clone(sequence_number),
      mojo::Clone(source_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BeginFrameAck>::value>::type*>
bool BeginFrameAck::Equals(const T& other) const {
  if (!mojo::Equals(this->sequence_number, other.sequence_number))
    return false;
  if (!mojo::Equals(this->source_id, other.source_id))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::BeginFrameArgs::DataView,
                                         ::cc::mojom::BeginFrameArgsPtr> {
  static bool IsNull(const ::cc::mojom::BeginFrameArgsPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::BeginFrameArgsPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::BeginFrameArgs::frame_time)& frame_time(
      const ::cc::mojom::BeginFrameArgsPtr& input) {
    return input->frame_time;
  }

  static const decltype(::cc::mojom::BeginFrameArgs::deadline)& deadline(
      const ::cc::mojom::BeginFrameArgsPtr& input) {
    return input->deadline;
  }

  static const decltype(::cc::mojom::BeginFrameArgs::interval)& interval(
      const ::cc::mojom::BeginFrameArgsPtr& input) {
    return input->interval;
  }

  static decltype(::cc::mojom::BeginFrameArgs::sequence_number) sequence_number(
      const ::cc::mojom::BeginFrameArgsPtr& input) {
    return input->sequence_number;
  }

  static decltype(::cc::mojom::BeginFrameArgs::source_id) source_id(
      const ::cc::mojom::BeginFrameArgsPtr& input) {
    return input->source_id;
  }

  static decltype(::cc::mojom::BeginFrameArgs::type) type(
      const ::cc::mojom::BeginFrameArgsPtr& input) {
    return input->type;
  }

  static decltype(::cc::mojom::BeginFrameArgs::on_critical_path) on_critical_path(
      const ::cc::mojom::BeginFrameArgsPtr& input) {
    return input->on_critical_path;
  }

  static bool Read(::cc::mojom::BeginFrameArgs::DataView input, ::cc::mojom::BeginFrameArgsPtr* output);
};


template <>
struct  StructTraits<::cc::mojom::BeginFrameAck::DataView,
                                         ::cc::mojom::BeginFrameAckPtr> {
  static bool IsNull(const ::cc::mojom::BeginFrameAckPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::BeginFrameAckPtr* output) { output->reset(); }

  static decltype(::cc::mojom::BeginFrameAck::sequence_number) sequence_number(
      const ::cc::mojom::BeginFrameAckPtr& input) {
    return input->sequence_number;
  }

  static decltype(::cc::mojom::BeginFrameAck::source_id) source_id(
      const ::cc::mojom::BeginFrameAckPtr& input) {
    return input->source_id;
  }

  static bool Read(::cc::mojom::BeginFrameAck::DataView input, ::cc::mojom::BeginFrameAckPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_BEGIN_FRAME_ARGS_MOJOM_H_