// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_COMPOSITOR_FRAME_MOJOM_BLINK_H_
#define CC_IPC_COMPOSITOR_FRAME_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/compositor_frame.mojom-shared.h"
#include "cc/ipc/compositor_frame_metadata.mojom-blink.h"
#include "cc/ipc/render_pass.mojom-blink.h"
#include "cc/ipc/transferable_resource.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "cc/output/compositor_frame.h"


namespace cc {
namespace mojom {
namespace blink {
class CompositorFrame;
using CompositorFramePtr = mojo::StructPtr<CompositorFrame>;






class  CompositorFrame {
 public:
  using DataView = CompositorFrameDataView;
  using Data_ = internal::CompositorFrame_Data;

  template <typename... Args>
  static CompositorFramePtr New(Args&&... args) {
    return CompositorFramePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CompositorFramePtr From(const U& u) {
    return mojo::TypeConverter<CompositorFramePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CompositorFrame>::Convert(*this);
  }


  CompositorFrame();

  CompositorFrame(
      ::cc::mojom::blink::CompositorFrameMetadataPtr metadata,
      WTF::Vector<::cc::mojom::blink::TransferableResourcePtr> resources,
      WTF::Vector<::cc::mojom::blink::RenderPassPtr> passes);

  ~CompositorFrame();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CompositorFramePtr>
  CompositorFramePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CompositorFrame>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CompositorFrame::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CompositorFrame::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  ::cc::mojom::blink::CompositorFrameMetadataPtr metadata;
  WTF::Vector<::cc::mojom::blink::TransferableResourcePtr> resources;
  WTF::Vector<::cc::mojom::blink::RenderPassPtr> passes;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(CompositorFrame);
};


template <typename StructPtrType>
CompositorFramePtr CompositorFrame::Clone() const {
  return New(
      mojo::Clone(metadata),
      mojo::Clone(resources),
      mojo::Clone(passes)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CompositorFrame>::value>::type*>
bool CompositorFrame::Equals(const T& other) const {
  if (!mojo::Equals(this->metadata, other.metadata))
    return false;
  if (!mojo::Equals(this->resources, other.resources))
    return false;
  if (!mojo::Equals(this->passes, other.passes))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::blink::CompositorFrame::DataView,
                                         ::cc::mojom::blink::CompositorFramePtr> {
  static bool IsNull(const ::cc::mojom::blink::CompositorFramePtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::CompositorFramePtr* output) { output->reset(); }

  static const decltype(::cc::mojom::blink::CompositorFrame::metadata)& metadata(
      const ::cc::mojom::blink::CompositorFramePtr& input) {
    return input->metadata;
  }

  static const decltype(::cc::mojom::blink::CompositorFrame::resources)& resources(
      const ::cc::mojom::blink::CompositorFramePtr& input) {
    return input->resources;
  }

  static const decltype(::cc::mojom::blink::CompositorFrame::passes)& passes(
      const ::cc::mojom::blink::CompositorFramePtr& input) {
    return input->passes;
  }

  static bool Read(::cc::mojom::blink::CompositorFrame::DataView input, ::cc::mojom::blink::CompositorFramePtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_COMPOSITOR_FRAME_MOJOM_BLINK_H_