// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_COMPOSITOR_FRAME_SINK_MOJOM_BLINK_H_
#define CC_IPC_COMPOSITOR_FRAME_SINK_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/compositor_frame_sink.mojom-shared.h"
#include "cc/ipc/begin_frame_args.mojom-blink.h"
#include "cc/ipc/compositor_frame.mojom-blink.h"
#include "cc/ipc/copy_output_request.mojom-blink.h"
#include "cc/ipc/frame_sink_id.mojom-blink.h"
#include "cc/ipc/local_surface_id.mojom-blink.h"
#include "cc/ipc/surface_id.mojom-blink.h"
#include "cc/ipc/returned_resource.mojom-blink.h"
#include "cc/ipc/surface_sequence.mojom-blink.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


namespace cc {
namespace mojom {
namespace blink {
class CompositorFrameSink;
using CompositorFrameSinkPtr = mojo::InterfacePtr<CompositorFrameSink>;
using CompositorFrameSinkPtrInfo = mojo::InterfacePtrInfo<CompositorFrameSink>;
using ThreadSafeCompositorFrameSinkPtr =
    mojo::ThreadSafeInterfacePtr<CompositorFrameSink>;
using CompositorFrameSinkRequest = mojo::InterfaceRequest<CompositorFrameSink>;
using CompositorFrameSinkAssociatedPtr =
    mojo::AssociatedInterfacePtr<CompositorFrameSink>;
using ThreadSafeCompositorFrameSinkAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CompositorFrameSink>;
using CompositorFrameSinkAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CompositorFrameSink>;
using CompositorFrameSinkAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CompositorFrameSink>;

class CompositorFrameSinkClient;
using CompositorFrameSinkClientPtr = mojo::InterfacePtr<CompositorFrameSinkClient>;
using CompositorFrameSinkClientPtrInfo = mojo::InterfacePtrInfo<CompositorFrameSinkClient>;
using ThreadSafeCompositorFrameSinkClientPtr =
    mojo::ThreadSafeInterfacePtr<CompositorFrameSinkClient>;
using CompositorFrameSinkClientRequest = mojo::InterfaceRequest<CompositorFrameSinkClient>;
using CompositorFrameSinkClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<CompositorFrameSinkClient>;
using ThreadSafeCompositorFrameSinkClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CompositorFrameSinkClient>;
using CompositorFrameSinkClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CompositorFrameSinkClient>;
using CompositorFrameSinkClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CompositorFrameSinkClient>;

class CompositorFrameSinkPrivate;
using CompositorFrameSinkPrivatePtr = mojo::InterfacePtr<CompositorFrameSinkPrivate>;
using CompositorFrameSinkPrivatePtrInfo = mojo::InterfacePtrInfo<CompositorFrameSinkPrivate>;
using ThreadSafeCompositorFrameSinkPrivatePtr =
    mojo::ThreadSafeInterfacePtr<CompositorFrameSinkPrivate>;
using CompositorFrameSinkPrivateRequest = mojo::InterfaceRequest<CompositorFrameSinkPrivate>;
using CompositorFrameSinkPrivateAssociatedPtr =
    mojo::AssociatedInterfacePtr<CompositorFrameSinkPrivate>;
using ThreadSafeCompositorFrameSinkPrivateAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CompositorFrameSinkPrivate>;
using CompositorFrameSinkPrivateAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CompositorFrameSinkPrivate>;
using CompositorFrameSinkPrivateAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CompositorFrameSinkPrivate>;


class CompositorFrameSinkProxy;

template <typename ImplRefTraits>
class CompositorFrameSinkStub;

class CompositorFrameSinkRequestValidator;

class  CompositorFrameSink
    : public CompositorFrameSinkInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CompositorFrameSinkProxy;

  template <typename ImplRefTraits>
  using Stub_ = CompositorFrameSinkStub<ImplRefTraits>;

  using RequestValidator_ = CompositorFrameSinkRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetNeedsBeginFrameMinVersion = 0,
    kSubmitCompositorFrameMinVersion = 0,
    kDidNotProduceFrameMinVersion = 0,
  };
  virtual ~CompositorFrameSink() {}

  virtual void SetNeedsBeginFrame(bool needs_begin_frame) = 0;

  virtual void SubmitCompositorFrame(const viz::LocalSurfaceId& local_surface_id, cc::CompositorFrame frame) = 0;

  virtual void DidNotProduceFrame(const cc::BeginFrameAck& ack) = 0;
};

class CompositorFrameSinkClientProxy;

template <typename ImplRefTraits>
class CompositorFrameSinkClientStub;

class CompositorFrameSinkClientRequestValidator;

class  CompositorFrameSinkClient
    : public CompositorFrameSinkClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CompositorFrameSinkClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = CompositorFrameSinkClientStub<ImplRefTraits>;

  using RequestValidator_ = CompositorFrameSinkClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kDidReceiveCompositorFrameAckMinVersion = 0,
    kOnBeginFrameMinVersion = 0,
    kOnBeginFramePausedChangedMinVersion = 0,
    kReclaimResourcesMinVersion = 0,
  };
  virtual ~CompositorFrameSinkClient() {}

  virtual void DidReceiveCompositorFrameAck(const WTF::Vector<cc::ReturnedResource>& resources) = 0;

  virtual void OnBeginFrame(const cc::BeginFrameArgs& args) = 0;

  virtual void OnBeginFramePausedChanged(bool paused) = 0;

  virtual void ReclaimResources(const WTF::Vector<cc::ReturnedResource>& resources) = 0;
};

class CompositorFrameSinkPrivateProxy;

template <typename ImplRefTraits>
class CompositorFrameSinkPrivateStub;

class CompositorFrameSinkPrivateRequestValidator;

class  CompositorFrameSinkPrivate
    : public CompositorFrameSinkPrivateInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CompositorFrameSinkPrivateProxy;

  template <typename ImplRefTraits>
  using Stub_ = CompositorFrameSinkPrivateStub<ImplRefTraits>;

  using RequestValidator_ = CompositorFrameSinkPrivateRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kClaimTemporaryReferenceMinVersion = 0,
    kRequestCopyOfSurfaceMinVersion = 0,
  };
  virtual ~CompositorFrameSinkPrivate() {}

  virtual void ClaimTemporaryReference(const viz::SurfaceId& surface_id) = 0;

  virtual void RequestCopyOfSurface(::cc::mojom::blink::CopyOutputRequestPtr request) = 0;
};

class  CompositorFrameSinkProxy
    : public CompositorFrameSink {
 public:
  explicit CompositorFrameSinkProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetNeedsBeginFrame(bool needs_begin_frame) override;
  void SubmitCompositorFrame(const viz::LocalSurfaceId& local_surface_id, cc::CompositorFrame frame) override;
  void DidNotProduceFrame(const cc::BeginFrameAck& ack) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  CompositorFrameSinkClientProxy
    : public CompositorFrameSinkClient {
 public:
  explicit CompositorFrameSinkClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void DidReceiveCompositorFrameAck(const WTF::Vector<cc::ReturnedResource>& resources) override;
  void OnBeginFrame(const cc::BeginFrameArgs& args) override;
  void OnBeginFramePausedChanged(bool paused) override;
  void ReclaimResources(const WTF::Vector<cc::ReturnedResource>& resources) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  CompositorFrameSinkPrivateProxy
    : public CompositorFrameSinkPrivate {
 public:
  explicit CompositorFrameSinkPrivateProxy(mojo::MessageReceiverWithResponder* receiver);
  void ClaimTemporaryReference(const viz::SurfaceId& surface_id) override;
  void RequestCopyOfSurface(::cc::mojom::blink::CopyOutputRequestPtr request) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  CompositorFrameSinkStubDispatch {
 public:
  static bool Accept(CompositorFrameSink* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CompositorFrameSink* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CompositorFrameSink>>
class CompositorFrameSinkStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CompositorFrameSinkStub() {}
  ~CompositorFrameSinkStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CompositorFrameSinkStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CompositorFrameSinkStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  CompositorFrameSinkClientStubDispatch {
 public:
  static bool Accept(CompositorFrameSinkClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CompositorFrameSinkClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CompositorFrameSinkClient>>
class CompositorFrameSinkClientStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CompositorFrameSinkClientStub() {}
  ~CompositorFrameSinkClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CompositorFrameSinkClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CompositorFrameSinkClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  CompositorFrameSinkPrivateStubDispatch {
 public:
  static bool Accept(CompositorFrameSinkPrivate* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CompositorFrameSinkPrivate* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CompositorFrameSinkPrivate>>
class CompositorFrameSinkPrivateStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CompositorFrameSinkPrivateStub() {}
  ~CompositorFrameSinkPrivateStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CompositorFrameSinkPrivateStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CompositorFrameSinkPrivateStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  CompositorFrameSinkRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  CompositorFrameSinkClientRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  CompositorFrameSinkPrivateRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {

}  // namespace mojo

#endif  // CC_IPC_COMPOSITOR_FRAME_SINK_MOJOM_BLINK_H_