// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_COPY_OUTPUT_REQUEST_MOJOM_H_
#define CC_IPC_COPY_OUTPUT_REQUEST_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/copy_output_request.mojom-shared.h"
#include "cc/ipc/copy_output_result.mojom.h"
#include "cc/ipc/texture_mailbox.mojom.h"
#include "mojo/common/unguessable_token.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>
#include "cc/output/copy_output_request.h"


namespace cc {
namespace mojom {
class CopyOutputResultSender;
using CopyOutputResultSenderPtr = mojo::InterfacePtr<CopyOutputResultSender>;
using CopyOutputResultSenderPtrInfo = mojo::InterfacePtrInfo<CopyOutputResultSender>;
using ThreadSafeCopyOutputResultSenderPtr =
    mojo::ThreadSafeInterfacePtr<CopyOutputResultSender>;
using CopyOutputResultSenderRequest = mojo::InterfaceRequest<CopyOutputResultSender>;
using CopyOutputResultSenderAssociatedPtr =
    mojo::AssociatedInterfacePtr<CopyOutputResultSender>;
using ThreadSafeCopyOutputResultSenderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CopyOutputResultSender>;
using CopyOutputResultSenderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CopyOutputResultSender>;
using CopyOutputResultSenderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CopyOutputResultSender>;

class CopyOutputRequest;
using CopyOutputRequestPtr = mojo::StructPtr<CopyOutputRequest>;


class CopyOutputResultSenderProxy;

template <typename ImplRefTraits>
class CopyOutputResultSenderStub;

class CopyOutputResultSenderRequestValidator;

class  CopyOutputResultSender
    : public CopyOutputResultSenderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CopyOutputResultSenderProxy;

  template <typename ImplRefTraits>
  using Stub_ = CopyOutputResultSenderStub<ImplRefTraits>;

  using RequestValidator_ = CopyOutputResultSenderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSendResultMinVersion = 0,
  };
  virtual ~CopyOutputResultSender() {}

  virtual void SendResult(std::unique_ptr<cc::CopyOutputResult> result) = 0;
};

class  CopyOutputResultSenderProxy
    : public CopyOutputResultSender {
 public:
  explicit CopyOutputResultSenderProxy(mojo::MessageReceiverWithResponder* receiver);
  void SendResult(std::unique_ptr<cc::CopyOutputResult> result) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  CopyOutputResultSenderStubDispatch {
 public:
  static bool Accept(CopyOutputResultSender* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CopyOutputResultSender* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CopyOutputResultSender>>
class CopyOutputResultSenderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CopyOutputResultSenderStub() {}
  ~CopyOutputResultSenderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CopyOutputResultSenderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CopyOutputResultSenderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  CopyOutputResultSenderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





class  CopyOutputRequest {
 public:
  using DataView = CopyOutputRequestDataView;
  using Data_ = internal::CopyOutputRequest_Data;

  template <typename... Args>
  static CopyOutputRequestPtr New(Args&&... args) {
    return CopyOutputRequestPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CopyOutputRequestPtr From(const U& u) {
    return mojo::TypeConverter<CopyOutputRequestPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CopyOutputRequest>::Convert(*this);
  }


  CopyOutputRequest();

  CopyOutputRequest(
      const base::Optional<base::UnguessableToken>& source,
      bool force_bitmap_result,
      const base::Optional<gfx::Rect>& area,
      const base::Optional<viz::TextureMailbox>& texture_mailbox,
      CopyOutputResultSenderPtr result_sender);

  ~CopyOutputRequest();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CopyOutputRequestPtr>
  CopyOutputRequestPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CopyOutputRequest>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CopyOutputRequest::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CopyOutputRequest::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::Optional<base::UnguessableToken> source;
  bool force_bitmap_result;
  base::Optional<gfx::Rect> area;
  base::Optional<viz::TextureMailbox> texture_mailbox;
  CopyOutputResultSenderPtr result_sender;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(CopyOutputRequest);
};


template <typename StructPtrType>
CopyOutputRequestPtr CopyOutputRequest::Clone() const {
  return New(
      mojo::Clone(source),
      mojo::Clone(force_bitmap_result),
      mojo::Clone(area),
      mojo::Clone(texture_mailbox),
      mojo::Clone(result_sender)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CopyOutputRequest>::value>::type*>
bool CopyOutputRequest::Equals(const T& other) const {
  if (!mojo::Equals(this->source, other.source))
    return false;
  if (!mojo::Equals(this->force_bitmap_result, other.force_bitmap_result))
    return false;
  if (!mojo::Equals(this->area, other.area))
    return false;
  if (!mojo::Equals(this->texture_mailbox, other.texture_mailbox))
    return false;
  if (!mojo::Equals(this->result_sender, other.result_sender))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::CopyOutputRequest::DataView,
                                         ::cc::mojom::CopyOutputRequestPtr> {
  static bool IsNull(const ::cc::mojom::CopyOutputRequestPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::CopyOutputRequestPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::CopyOutputRequest::source)& source(
      const ::cc::mojom::CopyOutputRequestPtr& input) {
    return input->source;
  }

  static decltype(::cc::mojom::CopyOutputRequest::force_bitmap_result) force_bitmap_result(
      const ::cc::mojom::CopyOutputRequestPtr& input) {
    return input->force_bitmap_result;
  }

  static const decltype(::cc::mojom::CopyOutputRequest::area)& area(
      const ::cc::mojom::CopyOutputRequestPtr& input) {
    return input->area;
  }

  static const decltype(::cc::mojom::CopyOutputRequest::texture_mailbox)& texture_mailbox(
      const ::cc::mojom::CopyOutputRequestPtr& input) {
    return input->texture_mailbox;
  }

  static  decltype(::cc::mojom::CopyOutputRequest::result_sender)& result_sender(
       ::cc::mojom::CopyOutputRequestPtr& input) {
    return input->result_sender;
  }

  static bool Read(::cc::mojom::CopyOutputRequest::DataView input, ::cc::mojom::CopyOutputRequestPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_COPY_OUTPUT_REQUEST_MOJOM_H_