// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_FILTER_OPERATIONS_MOJOM_H_
#define CC_IPC_FILTER_OPERATIONS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/filter_operations.mojom-shared.h"
#include "cc/ipc/filter_operation.mojom.h"
#include <string>
#include <vector>
#include "cc/base/filter_operations.h"


namespace cc {
namespace mojom {
class FilterOperations;
using FilterOperationsPtr = mojo::StructPtr<FilterOperations>;






class  FilterOperations {
 public:
  using DataView = FilterOperationsDataView;
  using Data_ = internal::FilterOperations_Data;

  template <typename... Args>
  static FilterOperationsPtr New(Args&&... args) {
    return FilterOperationsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static FilterOperationsPtr From(const U& u) {
    return mojo::TypeConverter<FilterOperationsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, FilterOperations>::Convert(*this);
  }


  FilterOperations();

  explicit FilterOperations(
      const std::vector<cc::FilterOperation>& operations);

  ~FilterOperations();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = FilterOperationsPtr>
  FilterOperationsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, FilterOperations>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        FilterOperations::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        FilterOperations::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::vector<cc::FilterOperation> operations;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
FilterOperationsPtr FilterOperations::Clone() const {
  return New(
      mojo::Clone(operations)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, FilterOperations>::value>::type*>
bool FilterOperations::Equals(const T& other) const {
  if (!mojo::Equals(this->operations, other.operations))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::FilterOperations::DataView,
                                         ::cc::mojom::FilterOperationsPtr> {
  static bool IsNull(const ::cc::mojom::FilterOperationsPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::FilterOperationsPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::FilterOperations::operations)& operations(
      const ::cc::mojom::FilterOperationsPtr& input) {
    return input->operations;
  }

  static bool Read(::cc::mojom::FilterOperations::DataView input, ::cc::mojom::FilterOperationsPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_FILTER_OPERATIONS_MOJOM_H_