// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "cc/ipc/quads.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "cc/ipc/frame_sink_id_struct_traits.h"
#include "cc/ipc/local_surface_id_struct_traits.h"
#include "cc/ipc/surface_id_struct_traits.h"
#include "third_party/WebKit/Source/platform/mojo/GeometryStructTraits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace cc {
namespace mojom {
namespace blink {
DebugBorderQuadState::DebugBorderQuadState()
    : color(),
      width() {}

DebugBorderQuadState::DebugBorderQuadState(
    uint32_t color_in,
    int32_t width_in)
    : color(std::move(color_in)),
      width(std::move(width_in)) {}

DebugBorderQuadState::~DebugBorderQuadState() = default;
size_t DebugBorderQuadState::Hash(size_t seed) const {
  seed = mojo::internal::WTFHash(seed, this->color);
  seed = mojo::internal::WTFHash(seed, this->width);
  return seed;
}

bool DebugBorderQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
RenderPassQuadState::RenderPassQuadState()
    : render_pass_id(),
      mask_resource_id(),
      mask_uv_rect(),
      mask_texture_size(),
      filters_scale(),
      filters_origin(),
      tex_coord_rect() {}

RenderPassQuadState::RenderPassQuadState(
    uint64_t render_pass_id_in,
    uint32_t mask_resource_id_in,
    const ::blink::WebFloatRect& mask_uv_rect_in,
    const ::blink::WebSize& mask_texture_size_in,
    ::gfx::mojom::blink::Vector2dFPtr filters_scale_in,
    const ::blink::WebFloatPoint& filters_origin_in,
    const ::blink::WebFloatRect& tex_coord_rect_in)
    : render_pass_id(std::move(render_pass_id_in)),
      mask_resource_id(std::move(mask_resource_id_in)),
      mask_uv_rect(std::move(mask_uv_rect_in)),
      mask_texture_size(std::move(mask_texture_size_in)),
      filters_scale(std::move(filters_scale_in)),
      filters_origin(std::move(filters_origin_in)),
      tex_coord_rect(std::move(tex_coord_rect_in)) {}

RenderPassQuadState::~RenderPassQuadState() = default;

bool RenderPassQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
SolidColorQuadState::SolidColorQuadState()
    : color(),
      force_anti_aliasing_off() {}

SolidColorQuadState::SolidColorQuadState(
    uint32_t color_in,
    bool force_anti_aliasing_off_in)
    : color(std::move(color_in)),
      force_anti_aliasing_off(std::move(force_anti_aliasing_off_in)) {}

SolidColorQuadState::~SolidColorQuadState() = default;
size_t SolidColorQuadState::Hash(size_t seed) const {
  seed = mojo::internal::WTFHash(seed, this->color);
  seed = mojo::internal::WTFHash(seed, this->force_anti_aliasing_off);
  return seed;
}

bool SolidColorQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
StreamVideoQuadState::StreamVideoQuadState()
    : resource_id(),
      resource_size_in_pixels(),
      matrix() {}

StreamVideoQuadState::StreamVideoQuadState(
    uint32_t resource_id_in,
    const ::blink::WebSize& resource_size_in_pixels_in,
    ::gfx::mojom::blink::TransformPtr matrix_in)
    : resource_id(std::move(resource_id_in)),
      resource_size_in_pixels(std::move(resource_size_in_pixels_in)),
      matrix(std::move(matrix_in)) {}

StreamVideoQuadState::~StreamVideoQuadState() = default;

bool StreamVideoQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
SurfaceQuadState::SurfaceQuadState()
    : surface(),
      surface_draw_quad_type() {}

SurfaceQuadState::SurfaceQuadState(
    const viz::SurfaceId& surface_in,
    SurfaceDrawQuadType surface_draw_quad_type_in)
    : surface(std::move(surface_in)),
      surface_draw_quad_type(std::move(surface_draw_quad_type_in)) {}

SurfaceQuadState::~SurfaceQuadState() = default;

bool SurfaceQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
TextureQuadState::TextureQuadState()
    : resource_id(),
      resource_size_in_pixels(),
      premultiplied_alpha(),
      uv_top_left(),
      uv_bottom_right(),
      background_color(),
      vertex_opacity(),
      y_flipped(),
      nearest_neighbor(),
      secure_output_only() {}

TextureQuadState::TextureQuadState(
    uint32_t resource_id_in,
    const ::blink::WebSize& resource_size_in_pixels_in,
    bool premultiplied_alpha_in,
    const ::blink::WebFloatPoint& uv_top_left_in,
    const ::blink::WebFloatPoint& uv_bottom_right_in,
    uint32_t background_color_in,
    const WTF::Vector<float>& vertex_opacity_in,
    bool y_flipped_in,
    bool nearest_neighbor_in,
    bool secure_output_only_in)
    : resource_id(std::move(resource_id_in)),
      resource_size_in_pixels(std::move(resource_size_in_pixels_in)),
      premultiplied_alpha(std::move(premultiplied_alpha_in)),
      uv_top_left(std::move(uv_top_left_in)),
      uv_bottom_right(std::move(uv_bottom_right_in)),
      background_color(std::move(background_color_in)),
      vertex_opacity(std::move(vertex_opacity_in)),
      y_flipped(std::move(y_flipped_in)),
      nearest_neighbor(std::move(nearest_neighbor_in)),
      secure_output_only(std::move(secure_output_only_in)) {}

TextureQuadState::~TextureQuadState() = default;

bool TextureQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
TileQuadState::TileQuadState()
    : tex_coord_rect(),
      texture_size(),
      swizzle_contents(),
      resource_id(),
      nearest_neighbor() {}

TileQuadState::TileQuadState(
    const ::blink::WebFloatRect& tex_coord_rect_in,
    const ::blink::WebSize& texture_size_in,
    bool swizzle_contents_in,
    uint32_t resource_id_in,
    bool nearest_neighbor_in)
    : tex_coord_rect(std::move(tex_coord_rect_in)),
      texture_size(std::move(texture_size_in)),
      swizzle_contents(std::move(swizzle_contents_in)),
      resource_id(std::move(resource_id_in)),
      nearest_neighbor(std::move(nearest_neighbor_in)) {}

TileQuadState::~TileQuadState() = default;

bool TileQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
YUVVideoQuadState::YUVVideoQuadState()
    : ya_tex_coord_rect(),
      uv_tex_coord_rect(),
      ya_tex_size(),
      uv_tex_size(),
      y_plane_resource_id(),
      u_plane_resource_id(),
      v_plane_resource_id(),
      a_plane_resource_id(),
      color_space(),
      resource_offset(),
      resource_multiplier(),
      bits_per_channel(),
      video_color_space(),
      require_overlay() {}

YUVVideoQuadState::YUVVideoQuadState(
    const ::blink::WebFloatRect& ya_tex_coord_rect_in,
    const ::blink::WebFloatRect& uv_tex_coord_rect_in,
    const ::blink::WebSize& ya_tex_size_in,
    const ::blink::WebSize& uv_tex_size_in,
    uint32_t y_plane_resource_id_in,
    uint32_t u_plane_resource_id_in,
    uint32_t v_plane_resource_id_in,
    uint32_t a_plane_resource_id_in,
    YUVColorSpace color_space_in,
    float resource_offset_in,
    float resource_multiplier_in,
    uint32_t bits_per_channel_in,
    ::gfx::mojom::blink::ColorSpacePtr video_color_space_in,
    bool require_overlay_in)
    : ya_tex_coord_rect(std::move(ya_tex_coord_rect_in)),
      uv_tex_coord_rect(std::move(uv_tex_coord_rect_in)),
      ya_tex_size(std::move(ya_tex_size_in)),
      uv_tex_size(std::move(uv_tex_size_in)),
      y_plane_resource_id(std::move(y_plane_resource_id_in)),
      u_plane_resource_id(std::move(u_plane_resource_id_in)),
      v_plane_resource_id(std::move(v_plane_resource_id_in)),
      a_plane_resource_id(std::move(a_plane_resource_id_in)),
      color_space(std::move(color_space_in)),
      resource_offset(std::move(resource_offset_in)),
      resource_multiplier(std::move(resource_multiplier_in)),
      bits_per_channel(std::move(bits_per_channel_in)),
      video_color_space(std::move(video_color_space_in)),
      require_overlay(std::move(require_overlay_in)) {}

YUVVideoQuadState::~YUVVideoQuadState() = default;

bool YUVVideoQuadState::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DrawQuad::DrawQuad()
    : rect(),
      opaque_rect(),
      visible_rect(),
      needs_blending(),
      sqs(),
      draw_quad_state() {}

DrawQuad::DrawQuad(
    ::gfx::mojom::blink::RectPtr rect_in,
    ::gfx::mojom::blink::RectPtr opaque_rect_in,
    ::gfx::mojom::blink::RectPtr visible_rect_in,
    bool needs_blending_in,
    ::cc::mojom::blink::SharedQuadStatePtr sqs_in,
    DrawQuadStatePtr draw_quad_state_in)
    : rect(std::move(rect_in)),
      opaque_rect(std::move(opaque_rect_in)),
      visible_rect(std::move(visible_rect_in)),
      needs_blending(std::move(needs_blending_in)),
      sqs(std::move(sqs_in)),
      draw_quad_state(std::move(draw_quad_state_in)) {}

DrawQuad::~DrawQuad() = default;

bool DrawQuad::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DrawQuadState::DrawQuadState() : tag_(Tag::DEBUG_BORDER_QUAD_STATE) {
  data_.debug_border_quad_state = new DebugBorderQuadStatePtr;
}

DrawQuadState::~DrawQuadState() {
  DestroyActive();
}


void DrawQuadState::set_debug_border_quad_state(
    DebugBorderQuadStatePtr debug_border_quad_state) {
  if (tag_ == Tag::DEBUG_BORDER_QUAD_STATE) {
    *(data_.debug_border_quad_state) = std::move(debug_border_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::DEBUG_BORDER_QUAD_STATE;
    data_.debug_border_quad_state = new DebugBorderQuadStatePtr(
        std::move(debug_border_quad_state));
  }
}
void DrawQuadState::set_render_pass_quad_state(
    RenderPassQuadStatePtr render_pass_quad_state) {
  if (tag_ == Tag::RENDER_PASS_QUAD_STATE) {
    *(data_.render_pass_quad_state) = std::move(render_pass_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::RENDER_PASS_QUAD_STATE;
    data_.render_pass_quad_state = new RenderPassQuadStatePtr(
        std::move(render_pass_quad_state));
  }
}
void DrawQuadState::set_solid_color_quad_state(
    SolidColorQuadStatePtr solid_color_quad_state) {
  if (tag_ == Tag::SOLID_COLOR_QUAD_STATE) {
    *(data_.solid_color_quad_state) = std::move(solid_color_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::SOLID_COLOR_QUAD_STATE;
    data_.solid_color_quad_state = new SolidColorQuadStatePtr(
        std::move(solid_color_quad_state));
  }
}
void DrawQuadState::set_stream_video_quad_state(
    StreamVideoQuadStatePtr stream_video_quad_state) {
  if (tag_ == Tag::STREAM_VIDEO_QUAD_STATE) {
    *(data_.stream_video_quad_state) = std::move(stream_video_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::STREAM_VIDEO_QUAD_STATE;
    data_.stream_video_quad_state = new StreamVideoQuadStatePtr(
        std::move(stream_video_quad_state));
  }
}
void DrawQuadState::set_surface_quad_state(
    SurfaceQuadStatePtr surface_quad_state) {
  if (tag_ == Tag::SURFACE_QUAD_STATE) {
    *(data_.surface_quad_state) = std::move(surface_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::SURFACE_QUAD_STATE;
    data_.surface_quad_state = new SurfaceQuadStatePtr(
        std::move(surface_quad_state));
  }
}
void DrawQuadState::set_texture_quad_state(
    TextureQuadStatePtr texture_quad_state) {
  if (tag_ == Tag::TEXTURE_QUAD_STATE) {
    *(data_.texture_quad_state) = std::move(texture_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::TEXTURE_QUAD_STATE;
    data_.texture_quad_state = new TextureQuadStatePtr(
        std::move(texture_quad_state));
  }
}
void DrawQuadState::set_tile_quad_state(
    TileQuadStatePtr tile_quad_state) {
  if (tag_ == Tag::TILE_QUAD_STATE) {
    *(data_.tile_quad_state) = std::move(tile_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::TILE_QUAD_STATE;
    data_.tile_quad_state = new TileQuadStatePtr(
        std::move(tile_quad_state));
  }
}
void DrawQuadState::set_yuv_video_quad_state(
    YUVVideoQuadStatePtr yuv_video_quad_state) {
  if (tag_ == Tag::YUV_VIDEO_QUAD_STATE) {
    *(data_.yuv_video_quad_state) = std::move(yuv_video_quad_state);
  } else {
    DestroyActive();
    tag_ = Tag::YUV_VIDEO_QUAD_STATE;
    data_.yuv_video_quad_state = new YUVVideoQuadStatePtr(
        std::move(yuv_video_quad_state));
  }
}

void DrawQuadState::DestroyActive() {
  switch (tag_) {

    case Tag::DEBUG_BORDER_QUAD_STATE:

      delete data_.debug_border_quad_state;
      break;
    case Tag::RENDER_PASS_QUAD_STATE:

      delete data_.render_pass_quad_state;
      break;
    case Tag::SOLID_COLOR_QUAD_STATE:

      delete data_.solid_color_quad_state;
      break;
    case Tag::STREAM_VIDEO_QUAD_STATE:

      delete data_.stream_video_quad_state;
      break;
    case Tag::SURFACE_QUAD_STATE:

      delete data_.surface_quad_state;
      break;
    case Tag::TEXTURE_QUAD_STATE:

      delete data_.texture_quad_state;
      break;
    case Tag::TILE_QUAD_STATE:

      delete data_.tile_quad_state;
      break;
    case Tag::YUV_VIDEO_QUAD_STATE:

      delete data_.yuv_video_quad_state;
      break;
  }
}
}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


// static
bool StructTraits<::cc::mojom::blink::DebugBorderQuadState::DataView, ::cc::mojom::blink::DebugBorderQuadStatePtr>::Read(
    ::cc::mojom::blink::DebugBorderQuadState::DataView input,
    ::cc::mojom::blink::DebugBorderQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::DebugBorderQuadStatePtr result(::cc::mojom::blink::DebugBorderQuadState::New());
  
      result->color = input.color();
      result->width = input.width();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::RenderPassQuadState::DataView, ::cc::mojom::blink::RenderPassQuadStatePtr>::Read(
    ::cc::mojom::blink::RenderPassQuadState::DataView input,
    ::cc::mojom::blink::RenderPassQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::RenderPassQuadStatePtr result(::cc::mojom::blink::RenderPassQuadState::New());
  
      result->render_pass_id = input.render_pass_id();
      result->mask_resource_id = input.mask_resource_id();
      if (!input.ReadMaskUvRect(&result->mask_uv_rect))
        success = false;
      if (!input.ReadMaskTextureSize(&result->mask_texture_size))
        success = false;
      if (!input.ReadFiltersScale(&result->filters_scale))
        success = false;
      if (!input.ReadFiltersOrigin(&result->filters_origin))
        success = false;
      if (!input.ReadTexCoordRect(&result->tex_coord_rect))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::SolidColorQuadState::DataView, ::cc::mojom::blink::SolidColorQuadStatePtr>::Read(
    ::cc::mojom::blink::SolidColorQuadState::DataView input,
    ::cc::mojom::blink::SolidColorQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::SolidColorQuadStatePtr result(::cc::mojom::blink::SolidColorQuadState::New());
  
      result->color = input.color();
      result->force_anti_aliasing_off = input.force_anti_aliasing_off();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::StreamVideoQuadState::DataView, ::cc::mojom::blink::StreamVideoQuadStatePtr>::Read(
    ::cc::mojom::blink::StreamVideoQuadState::DataView input,
    ::cc::mojom::blink::StreamVideoQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::StreamVideoQuadStatePtr result(::cc::mojom::blink::StreamVideoQuadState::New());
  
      result->resource_id = input.resource_id();
      if (!input.ReadResourceSizeInPixels(&result->resource_size_in_pixels))
        success = false;
      if (!input.ReadMatrix(&result->matrix))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::SurfaceQuadState::DataView, ::cc::mojom::blink::SurfaceQuadStatePtr>::Read(
    ::cc::mojom::blink::SurfaceQuadState::DataView input,
    ::cc::mojom::blink::SurfaceQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::SurfaceQuadStatePtr result(::cc::mojom::blink::SurfaceQuadState::New());
  
      if (!input.ReadSurface(&result->surface))
        success = false;
      if (!input.ReadSurfaceDrawQuadType(&result->surface_draw_quad_type))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::TextureQuadState::DataView, ::cc::mojom::blink::TextureQuadStatePtr>::Read(
    ::cc::mojom::blink::TextureQuadState::DataView input,
    ::cc::mojom::blink::TextureQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::TextureQuadStatePtr result(::cc::mojom::blink::TextureQuadState::New());
  
      result->resource_id = input.resource_id();
      if (!input.ReadResourceSizeInPixels(&result->resource_size_in_pixels))
        success = false;
      result->premultiplied_alpha = input.premultiplied_alpha();
      if (!input.ReadUvTopLeft(&result->uv_top_left))
        success = false;
      if (!input.ReadUvBottomRight(&result->uv_bottom_right))
        success = false;
      result->background_color = input.background_color();
      if (!input.ReadVertexOpacity(&result->vertex_opacity))
        success = false;
      result->y_flipped = input.y_flipped();
      result->nearest_neighbor = input.nearest_neighbor();
      result->secure_output_only = input.secure_output_only();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::TileQuadState::DataView, ::cc::mojom::blink::TileQuadStatePtr>::Read(
    ::cc::mojom::blink::TileQuadState::DataView input,
    ::cc::mojom::blink::TileQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::TileQuadStatePtr result(::cc::mojom::blink::TileQuadState::New());
  
      if (!input.ReadTexCoordRect(&result->tex_coord_rect))
        success = false;
      if (!input.ReadTextureSize(&result->texture_size))
        success = false;
      result->swizzle_contents = input.swizzle_contents();
      result->resource_id = input.resource_id();
      result->nearest_neighbor = input.nearest_neighbor();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::YUVVideoQuadState::DataView, ::cc::mojom::blink::YUVVideoQuadStatePtr>::Read(
    ::cc::mojom::blink::YUVVideoQuadState::DataView input,
    ::cc::mojom::blink::YUVVideoQuadStatePtr* output) {
  bool success = true;
  ::cc::mojom::blink::YUVVideoQuadStatePtr result(::cc::mojom::blink::YUVVideoQuadState::New());
  
      if (!input.ReadYaTexCoordRect(&result->ya_tex_coord_rect))
        success = false;
      if (!input.ReadUvTexCoordRect(&result->uv_tex_coord_rect))
        success = false;
      if (!input.ReadYaTexSize(&result->ya_tex_size))
        success = false;
      if (!input.ReadUvTexSize(&result->uv_tex_size))
        success = false;
      result->y_plane_resource_id = input.y_plane_resource_id();
      result->u_plane_resource_id = input.u_plane_resource_id();
      result->v_plane_resource_id = input.v_plane_resource_id();
      result->a_plane_resource_id = input.a_plane_resource_id();
      if (!input.ReadColorSpace(&result->color_space))
        success = false;
      result->resource_offset = input.resource_offset();
      result->resource_multiplier = input.resource_multiplier();
      result->bits_per_channel = input.bits_per_channel();
      if (!input.ReadVideoColorSpace(&result->video_color_space))
        success = false;
      result->require_overlay = input.require_overlay();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::cc::mojom::blink::DrawQuad::DataView, ::cc::mojom::blink::DrawQuadPtr>::Read(
    ::cc::mojom::blink::DrawQuad::DataView input,
    ::cc::mojom::blink::DrawQuadPtr* output) {
  bool success = true;
  ::cc::mojom::blink::DrawQuadPtr result(::cc::mojom::blink::DrawQuad::New());
  
      if (!input.ReadRect(&result->rect))
        success = false;
      if (!input.ReadOpaqueRect(&result->opaque_rect))
        success = false;
      if (!input.ReadVisibleRect(&result->visible_rect))
        success = false;
      result->needs_blending = input.needs_blending();
      if (!input.ReadSqs(&result->sqs))
        success = false;
      if (!input.ReadDrawQuadState(&result->draw_quad_state))
        success = false;
  *output = std::move(result);
  return success;
}

// static
bool UnionTraits<::cc::mojom::blink::DrawQuadState::DataView, ::cc::mojom::blink::DrawQuadStatePtr>::Read(
    ::cc::mojom::blink::DrawQuadState::DataView input,
    ::cc::mojom::blink::DrawQuadStatePtr* output) {
  using UnionType = ::cc::mojom::blink::DrawQuadState;
  using Tag = UnionType::Tag;

  switch (input.tag()) {
    case Tag::DEBUG_BORDER_QUAD_STATE: {
      ::cc::mojom::blink::DebugBorderQuadStatePtr result_debug_border_quad_state;
      if (!input.ReadDebugBorderQuadState(&result_debug_border_quad_state))
        return false;

      *output = UnionType::NewDebugBorderQuadState(
          std::move(result_debug_border_quad_state));
      break;
    }
    case Tag::RENDER_PASS_QUAD_STATE: {
      ::cc::mojom::blink::RenderPassQuadStatePtr result_render_pass_quad_state;
      if (!input.ReadRenderPassQuadState(&result_render_pass_quad_state))
        return false;

      *output = UnionType::NewRenderPassQuadState(
          std::move(result_render_pass_quad_state));
      break;
    }
    case Tag::SOLID_COLOR_QUAD_STATE: {
      ::cc::mojom::blink::SolidColorQuadStatePtr result_solid_color_quad_state;
      if (!input.ReadSolidColorQuadState(&result_solid_color_quad_state))
        return false;

      *output = UnionType::NewSolidColorQuadState(
          std::move(result_solid_color_quad_state));
      break;
    }
    case Tag::STREAM_VIDEO_QUAD_STATE: {
      ::cc::mojom::blink::StreamVideoQuadStatePtr result_stream_video_quad_state;
      if (!input.ReadStreamVideoQuadState(&result_stream_video_quad_state))
        return false;

      *output = UnionType::NewStreamVideoQuadState(
          std::move(result_stream_video_quad_state));
      break;
    }
    case Tag::SURFACE_QUAD_STATE: {
      ::cc::mojom::blink::SurfaceQuadStatePtr result_surface_quad_state;
      if (!input.ReadSurfaceQuadState(&result_surface_quad_state))
        return false;

      *output = UnionType::NewSurfaceQuadState(
          std::move(result_surface_quad_state));
      break;
    }
    case Tag::TEXTURE_QUAD_STATE: {
      ::cc::mojom::blink::TextureQuadStatePtr result_texture_quad_state;
      if (!input.ReadTextureQuadState(&result_texture_quad_state))
        return false;

      *output = UnionType::NewTextureQuadState(
          std::move(result_texture_quad_state));
      break;
    }
    case Tag::TILE_QUAD_STATE: {
      ::cc::mojom::blink::TileQuadStatePtr result_tile_quad_state;
      if (!input.ReadTileQuadState(&result_tile_quad_state))
        return false;

      *output = UnionType::NewTileQuadState(
          std::move(result_tile_quad_state));
      break;
    }
    case Tag::YUV_VIDEO_QUAD_STATE: {
      ::cc::mojom::blink::YUVVideoQuadStatePtr result_yuv_video_quad_state;
      if (!input.ReadYuvVideoQuadState(&result_yuv_video_quad_state))
        return false;

      *output = UnionType::NewYuvVideoQuadState(
          std::move(result_yuv_video_quad_state));
      break;
    }
    default:
      return false;
  }
  return true;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif