// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_RENDER_PASS_MOJOM_H_
#define CC_IPC_RENDER_PASS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/render_pass.mojom-shared.h"
#include "cc/ipc/filter_operations.mojom.h"
#include "cc/ipc/quads.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/color_space.mojom.h"
#include "ui/gfx/mojo/transform.mojom.h"
#include <string>
#include <vector>
#include "cc/quads/draw_quad.h"
#include "cc/quads/render_pass.h"


namespace cc {
namespace mojom {
class RenderPass;
using RenderPassPtr = mojo::StructPtr<RenderPass>;






class  RenderPass {
 public:
  using DataView = RenderPassDataView;
  using Data_ = internal::RenderPass_Data;

  template <typename... Args>
  static RenderPassPtr New(Args&&... args) {
    return RenderPassPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static RenderPassPtr From(const U& u) {
    return mojo::TypeConverter<RenderPassPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, RenderPass>::Convert(*this);
  }


  RenderPass();

  RenderPass(
      uint64_t id,
      const gfx::Rect& output_rect,
      const gfx::Rect& damage_rect,
      const gfx::Transform& transform_to_root_target,
      const cc::FilterOperations& filters,
      const cc::FilterOperations& background_filters,
      const gfx::ColorSpace& color_space,
      bool has_transparent_background,
      bool cache_render_pass,
      bool has_damage_from_contributing_content,
      std::vector<::cc::mojom::DrawQuadPtr> quad_list);

  ~RenderPass();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = RenderPassPtr>
  RenderPassPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, RenderPass>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        RenderPass::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        RenderPass::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint64_t id;
  gfx::Rect output_rect;
  gfx::Rect damage_rect;
  gfx::Transform transform_to_root_target;
  cc::FilterOperations filters;
  cc::FilterOperations background_filters;
  gfx::ColorSpace color_space;
  bool has_transparent_background;
  bool cache_render_pass;
  bool has_damage_from_contributing_content;
  std::vector<::cc::mojom::DrawQuadPtr> quad_list;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(RenderPass);
};


template <typename StructPtrType>
RenderPassPtr RenderPass::Clone() const {
  return New(
      mojo::Clone(id),
      mojo::Clone(output_rect),
      mojo::Clone(damage_rect),
      mojo::Clone(transform_to_root_target),
      mojo::Clone(filters),
      mojo::Clone(background_filters),
      mojo::Clone(color_space),
      mojo::Clone(has_transparent_background),
      mojo::Clone(cache_render_pass),
      mojo::Clone(has_damage_from_contributing_content),
      mojo::Clone(quad_list)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, RenderPass>::value>::type*>
bool RenderPass::Equals(const T& other) const {
  if (!mojo::Equals(this->id, other.id))
    return false;
  if (!mojo::Equals(this->output_rect, other.output_rect))
    return false;
  if (!mojo::Equals(this->damage_rect, other.damage_rect))
    return false;
  if (!mojo::Equals(this->transform_to_root_target, other.transform_to_root_target))
    return false;
  if (!mojo::Equals(this->filters, other.filters))
    return false;
  if (!mojo::Equals(this->background_filters, other.background_filters))
    return false;
  if (!mojo::Equals(this->color_space, other.color_space))
    return false;
  if (!mojo::Equals(this->has_transparent_background, other.has_transparent_background))
    return false;
  if (!mojo::Equals(this->cache_render_pass, other.cache_render_pass))
    return false;
  if (!mojo::Equals(this->has_damage_from_contributing_content, other.has_damage_from_contributing_content))
    return false;
  if (!mojo::Equals(this->quad_list, other.quad_list))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::RenderPass::DataView,
                                         ::cc::mojom::RenderPassPtr> {
  static bool IsNull(const ::cc::mojom::RenderPassPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::RenderPassPtr* output) { output->reset(); }

  static decltype(::cc::mojom::RenderPass::id) id(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->id;
  }

  static const decltype(::cc::mojom::RenderPass::output_rect)& output_rect(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->output_rect;
  }

  static const decltype(::cc::mojom::RenderPass::damage_rect)& damage_rect(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->damage_rect;
  }

  static const decltype(::cc::mojom::RenderPass::transform_to_root_target)& transform_to_root_target(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->transform_to_root_target;
  }

  static const decltype(::cc::mojom::RenderPass::filters)& filters(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->filters;
  }

  static const decltype(::cc::mojom::RenderPass::background_filters)& background_filters(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->background_filters;
  }

  static const decltype(::cc::mojom::RenderPass::color_space)& color_space(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->color_space;
  }

  static decltype(::cc::mojom::RenderPass::has_transparent_background) has_transparent_background(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->has_transparent_background;
  }

  static decltype(::cc::mojom::RenderPass::cache_render_pass) cache_render_pass(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->cache_render_pass;
  }

  static decltype(::cc::mojom::RenderPass::has_damage_from_contributing_content) has_damage_from_contributing_content(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->has_damage_from_contributing_content;
  }

  static const decltype(::cc::mojom::RenderPass::quad_list)& quad_list(
      const ::cc::mojom::RenderPassPtr& input) {
    return input->quad_list;
  }

  static bool Read(::cc::mojom::RenderPass::DataView input, ::cc::mojom::RenderPassPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_RENDER_PASS_MOJOM_H_