// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_SHARED_QUAD_STATE_MOJOM_BLINK_H_
#define CC_IPC_SHARED_QUAD_STATE_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/shared_quad_state.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-blink.h"
#include "ui/gfx/mojo/transform.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"


namespace cc {
namespace mojom {
namespace blink {
class SharedQuadState;
using SharedQuadStatePtr = mojo::StructPtr<SharedQuadState>;






class  SharedQuadState {
 public:
  using DataView = SharedQuadStateDataView;
  using Data_ = internal::SharedQuadState_Data;

  template <typename... Args>
  static SharedQuadStatePtr New(Args&&... args) {
    return SharedQuadStatePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static SharedQuadStatePtr From(const U& u) {
    return mojo::TypeConverter<SharedQuadStatePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SharedQuadState>::Convert(*this);
  }


  SharedQuadState();

  SharedQuadState(
      ::gfx::mojom::blink::TransformPtr quad_to_target_transform,
      ::gfx::mojom::blink::RectPtr quad_layer_rect,
      ::gfx::mojom::blink::RectPtr visible_quad_layer_rect,
      ::gfx::mojom::blink::RectPtr clip_rect,
      bool is_clipped,
      float opacity,
      uint32_t blend_mode,
      int32_t sorting_context_id);

  ~SharedQuadState();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SharedQuadStatePtr>
  SharedQuadStatePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SharedQuadState>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        SharedQuadState::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        SharedQuadState::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  ::gfx::mojom::blink::TransformPtr quad_to_target_transform;
  ::gfx::mojom::blink::RectPtr quad_layer_rect;
  ::gfx::mojom::blink::RectPtr visible_quad_layer_rect;
  ::gfx::mojom::blink::RectPtr clip_rect;
  bool is_clipped;
  float opacity;
  uint32_t blend_mode;
  int32_t sorting_context_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(SharedQuadState);
};


template <typename StructPtrType>
SharedQuadStatePtr SharedQuadState::Clone() const {
  return New(
      mojo::Clone(quad_to_target_transform),
      mojo::Clone(quad_layer_rect),
      mojo::Clone(visible_quad_layer_rect),
      mojo::Clone(clip_rect),
      mojo::Clone(is_clipped),
      mojo::Clone(opacity),
      mojo::Clone(blend_mode),
      mojo::Clone(sorting_context_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SharedQuadState>::value>::type*>
bool SharedQuadState::Equals(const T& other) const {
  if (!mojo::Equals(this->quad_to_target_transform, other.quad_to_target_transform))
    return false;
  if (!mojo::Equals(this->quad_layer_rect, other.quad_layer_rect))
    return false;
  if (!mojo::Equals(this->visible_quad_layer_rect, other.visible_quad_layer_rect))
    return false;
  if (!mojo::Equals(this->clip_rect, other.clip_rect))
    return false;
  if (!mojo::Equals(this->is_clipped, other.is_clipped))
    return false;
  if (!mojo::Equals(this->opacity, other.opacity))
    return false;
  if (!mojo::Equals(this->blend_mode, other.blend_mode))
    return false;
  if (!mojo::Equals(this->sorting_context_id, other.sorting_context_id))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::blink::SharedQuadState::DataView,
                                         ::cc::mojom::blink::SharedQuadStatePtr> {
  static bool IsNull(const ::cc::mojom::blink::SharedQuadStatePtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::SharedQuadStatePtr* output) { output->reset(); }

  static const decltype(::cc::mojom::blink::SharedQuadState::quad_to_target_transform)& quad_to_target_transform(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->quad_to_target_transform;
  }

  static const decltype(::cc::mojom::blink::SharedQuadState::quad_layer_rect)& quad_layer_rect(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->quad_layer_rect;
  }

  static const decltype(::cc::mojom::blink::SharedQuadState::visible_quad_layer_rect)& visible_quad_layer_rect(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->visible_quad_layer_rect;
  }

  static const decltype(::cc::mojom::blink::SharedQuadState::clip_rect)& clip_rect(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->clip_rect;
  }

  static decltype(::cc::mojom::blink::SharedQuadState::is_clipped) is_clipped(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->is_clipped;
  }

  static decltype(::cc::mojom::blink::SharedQuadState::opacity) opacity(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->opacity;
  }

  static decltype(::cc::mojom::blink::SharedQuadState::blend_mode) blend_mode(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->blend_mode;
  }

  static decltype(::cc::mojom::blink::SharedQuadState::sorting_context_id) sorting_context_id(
      const ::cc::mojom::blink::SharedQuadStatePtr& input) {
    return input->sorting_context_id;
  }

  static bool Read(::cc::mojom::blink::SharedQuadState::DataView input, ::cc::mojom::blink::SharedQuadStatePtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_SHARED_QUAD_STATE_MOJOM_BLINK_H_