// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_SURFACE_ID_MOJOM_BLINK_H_
#define CC_IPC_SURFACE_ID_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/surface_id.mojom-shared.h"
#include "cc/ipc/frame_sink_id.mojom-blink.h"
#include "cc/ipc/local_surface_id.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "components/viz/common/surfaces/surface_id.h"


namespace cc {
namespace mojom {
namespace blink {
class SurfaceId;
using SurfaceIdPtr = mojo::StructPtr<SurfaceId>;






class  SurfaceId {
 public:
  using DataView = SurfaceIdDataView;
  using Data_ = internal::SurfaceId_Data;

  template <typename... Args>
  static SurfaceIdPtr New(Args&&... args) {
    return SurfaceIdPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static SurfaceIdPtr From(const U& u) {
    return mojo::TypeConverter<SurfaceIdPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SurfaceId>::Convert(*this);
  }


  SurfaceId();

  SurfaceId(
      const viz::FrameSinkId& frame_sink_id,
      const viz::LocalSurfaceId& local_surface_id);

  ~SurfaceId();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SurfaceIdPtr>
  SurfaceIdPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SurfaceId>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        SurfaceId::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        SurfaceId::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  viz::FrameSinkId frame_sink_id;
  viz::LocalSurfaceId local_surface_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
SurfaceIdPtr SurfaceId::Clone() const {
  return New(
      mojo::Clone(frame_sink_id),
      mojo::Clone(local_surface_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SurfaceId>::value>::type*>
bool SurfaceId::Equals(const T& other) const {
  if (!mojo::Equals(this->frame_sink_id, other.frame_sink_id))
    return false;
  if (!mojo::Equals(this->local_surface_id, other.local_surface_id))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::blink::SurfaceId::DataView,
                                         ::cc::mojom::blink::SurfaceIdPtr> {
  static bool IsNull(const ::cc::mojom::blink::SurfaceIdPtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::SurfaceIdPtr* output) { output->reset(); }

  static const decltype(::cc::mojom::blink::SurfaceId::frame_sink_id)& frame_sink_id(
      const ::cc::mojom::blink::SurfaceIdPtr& input) {
    return input->frame_sink_id;
  }

  static const decltype(::cc::mojom::blink::SurfaceId::local_surface_id)& local_surface_id(
      const ::cc::mojom::blink::SurfaceIdPtr& input) {
    return input->local_surface_id;
  }

  static bool Read(::cc::mojom::blink::SurfaceId::DataView input, ::cc::mojom::blink::SurfaceIdPtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_SURFACE_ID_MOJOM_BLINK_H_