// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CC_IPC_SURFACE_SEQUENCE_MOJOM_BLINK_H_
#define CC_IPC_SURFACE_SEQUENCE_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "cc/ipc/surface_sequence.mojom-shared.h"
#include "cc/ipc/frame_sink_id.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "components/viz/common/surfaces/surface_sequence.h"


namespace cc {
namespace mojom {
namespace blink {
class SurfaceSequence;
using SurfaceSequencePtr = mojo::StructPtr<SurfaceSequence>;






class  SurfaceSequence {
 public:
  using DataView = SurfaceSequenceDataView;
  using Data_ = internal::SurfaceSequence_Data;

  template <typename... Args>
  static SurfaceSequencePtr New(Args&&... args) {
    return SurfaceSequencePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static SurfaceSequencePtr From(const U& u) {
    return mojo::TypeConverter<SurfaceSequencePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SurfaceSequence>::Convert(*this);
  }


  SurfaceSequence();

  SurfaceSequence(
      const viz::FrameSinkId& frame_sink_id,
      uint32_t sequence);

  ~SurfaceSequence();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SurfaceSequencePtr>
  SurfaceSequencePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SurfaceSequence>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        SurfaceSequence::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        SurfaceSequence::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  viz::FrameSinkId frame_sink_id;
  uint32_t sequence;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
SurfaceSequencePtr SurfaceSequence::Clone() const {
  return New(
      mojo::Clone(frame_sink_id),
      mojo::Clone(sequence)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SurfaceSequence>::value>::type*>
bool SurfaceSequence::Equals(const T& other) const {
  if (!mojo::Equals(this->frame_sink_id, other.frame_sink_id))
    return false;
  if (!mojo::Equals(this->sequence, other.sequence))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace cc

namespace mojo {


template <>
struct  StructTraits<::cc::mojom::blink::SurfaceSequence::DataView,
                                         ::cc::mojom::blink::SurfaceSequencePtr> {
  static bool IsNull(const ::cc::mojom::blink::SurfaceSequencePtr& input) { return !input; }
  static void SetToNull(::cc::mojom::blink::SurfaceSequencePtr* output) { output->reset(); }

  static const decltype(::cc::mojom::blink::SurfaceSequence::frame_sink_id)& frame_sink_id(
      const ::cc::mojom::blink::SurfaceSequencePtr& input) {
    return input->frame_sink_id;
  }

  static decltype(::cc::mojom::blink::SurfaceSequence::sequence) sequence(
      const ::cc::mojom::blink::SurfaceSequencePtr& input) {
    return input->sequence;
  }

  static bool Read(::cc::mojom::blink::SurfaceSequence::DataView input, ::cc::mojom::blink::SurfaceSequencePtr* output);
};

}  // namespace mojo

#endif  // CC_IPC_SURFACE_SEQUENCE_MOJOM_BLINK_H_