// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_FRAME_MOJOM_H_
#define CONTENT_COMMON_FRAME_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/frame.mojom-shared.h"
#include "content/public/common/window_container_type.mojom.h"
#include "services/service_manager/public/interfaces/interface_provider.mojom.h"
#include "third_party/WebKit/public/platform/referrer.mojom.h"
#include "third_party/WebKit/public/web/window_features.mojom.h"
#include "ui/base/mojo/window_open_disposition.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
CONTENT_EXPORT extern const char kNavigation_FrameSpec[];
class Frame;
using FramePtr = mojo::InterfacePtr<Frame>;
using FramePtrInfo = mojo::InterfacePtrInfo<Frame>;
using ThreadSafeFramePtr =
    mojo::ThreadSafeInterfacePtr<Frame>;
using FrameRequest = mojo::InterfaceRequest<Frame>;
using FrameAssociatedPtr =
    mojo::AssociatedInterfacePtr<Frame>;
using ThreadSafeFrameAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Frame>;
using FrameAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Frame>;
using FrameAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Frame>;

class FrameBindingsControl;
using FrameBindingsControlPtr = mojo::InterfacePtr<FrameBindingsControl>;
using FrameBindingsControlPtrInfo = mojo::InterfacePtrInfo<FrameBindingsControl>;
using ThreadSafeFrameBindingsControlPtr =
    mojo::ThreadSafeInterfacePtr<FrameBindingsControl>;
using FrameBindingsControlRequest = mojo::InterfaceRequest<FrameBindingsControl>;
using FrameBindingsControlAssociatedPtr =
    mojo::AssociatedInterfacePtr<FrameBindingsControl>;
using ThreadSafeFrameBindingsControlAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<FrameBindingsControl>;
using FrameBindingsControlAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<FrameBindingsControl>;
using FrameBindingsControlAssociatedRequest =
    mojo::AssociatedInterfaceRequest<FrameBindingsControl>;

class FrameHostInterfaceBroker;
using FrameHostInterfaceBrokerPtr = mojo::InterfacePtr<FrameHostInterfaceBroker>;
using FrameHostInterfaceBrokerPtrInfo = mojo::InterfacePtrInfo<FrameHostInterfaceBroker>;
using ThreadSafeFrameHostInterfaceBrokerPtr =
    mojo::ThreadSafeInterfacePtr<FrameHostInterfaceBroker>;
using FrameHostInterfaceBrokerRequest = mojo::InterfaceRequest<FrameHostInterfaceBroker>;
using FrameHostInterfaceBrokerAssociatedPtr =
    mojo::AssociatedInterfacePtr<FrameHostInterfaceBroker>;
using ThreadSafeFrameHostInterfaceBrokerAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<FrameHostInterfaceBroker>;
using FrameHostInterfaceBrokerAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<FrameHostInterfaceBroker>;
using FrameHostInterfaceBrokerAssociatedRequest =
    mojo::AssociatedInterfaceRequest<FrameHostInterfaceBroker>;

class FrameFactory;
using FrameFactoryPtr = mojo::InterfacePtr<FrameFactory>;
using FrameFactoryPtrInfo = mojo::InterfacePtrInfo<FrameFactory>;
using ThreadSafeFrameFactoryPtr =
    mojo::ThreadSafeInterfacePtr<FrameFactory>;
using FrameFactoryRequest = mojo::InterfaceRequest<FrameFactory>;
using FrameFactoryAssociatedPtr =
    mojo::AssociatedInterfacePtr<FrameFactory>;
using ThreadSafeFrameFactoryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<FrameFactory>;
using FrameFactoryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<FrameFactory>;
using FrameFactoryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<FrameFactory>;

class FrameHost;
using FrameHostPtr = mojo::InterfacePtr<FrameHost>;
using FrameHostPtrInfo = mojo::InterfacePtrInfo<FrameHost>;
using ThreadSafeFrameHostPtr =
    mojo::ThreadSafeInterfacePtr<FrameHost>;
using FrameHostRequest = mojo::InterfaceRequest<FrameHost>;
using FrameHostAssociatedPtr =
    mojo::AssociatedInterfacePtr<FrameHost>;
using ThreadSafeFrameHostAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<FrameHost>;
using FrameHostAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<FrameHost>;
using FrameHostAssociatedRequest =
    mojo::AssociatedInterfaceRequest<FrameHost>;

class CreateNewWindowParams;
using CreateNewWindowParamsPtr = mojo::StructPtr<CreateNewWindowParams>;

class CreateNewWindowReply;
using CreateNewWindowReplyPtr = mojo::InlinedStructPtr<CreateNewWindowReply>;


class FrameProxy;

template <typename ImplRefTraits>
class FrameStub;

class FrameRequestValidator;

class CONTENT_EXPORT Frame
    : public FrameInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameStub<ImplRefTraits>;

  using RequestValidator_ = FrameRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetInterfaceProviderMinVersion = 0,
  };
  virtual ~Frame() {}

  virtual void GetInterfaceProvider(::service_manager::mojom::InterfaceProviderRequest interfaces) = 0;
};

class FrameBindingsControlProxy;

template <typename ImplRefTraits>
class FrameBindingsControlStub;

class FrameBindingsControlRequestValidator;

class CONTENT_EXPORT FrameBindingsControl
    : public FrameBindingsControlInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameBindingsControlProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameBindingsControlStub<ImplRefTraits>;

  using RequestValidator_ = FrameBindingsControlRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAllowBindingsMinVersion = 0,
  };
  virtual ~FrameBindingsControl() {}

  virtual void AllowBindings(int32_t enabled_bindings_flags) = 0;
};

class FrameHostInterfaceBrokerProxy;

template <typename ImplRefTraits>
class FrameHostInterfaceBrokerStub;

class FrameHostInterfaceBrokerRequestValidator;

class CONTENT_EXPORT FrameHostInterfaceBroker
    : public FrameHostInterfaceBrokerInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameHostInterfaceBrokerProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameHostInterfaceBrokerStub<ImplRefTraits>;

  using RequestValidator_ = FrameHostInterfaceBrokerRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kGetInterfaceProviderMinVersion = 0,
  };
  virtual ~FrameHostInterfaceBroker() {}

  virtual void GetInterfaceProvider(::service_manager::mojom::InterfaceProviderRequest interfaces) = 0;
};

class FrameFactoryProxy;

template <typename ImplRefTraits>
class FrameFactoryStub;

class FrameFactoryRequestValidator;

class CONTENT_EXPORT FrameFactory
    : public FrameFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = FrameFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameFactoryStub<ImplRefTraits>;

  using RequestValidator_ = FrameFactoryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateFrameMinVersion = 0,
  };
  virtual ~FrameFactory() {}

  virtual void CreateFrame(int32_t frame_routing_id, FrameRequest frame, FrameHostInterfaceBrokerPtr host) = 0;
};

class FrameHostProxy;

template <typename ImplRefTraits>
class FrameHostStub;

class FrameHostRequestValidator;
class FrameHostResponseValidator;

class CONTENT_EXPORT FrameHost
    : public FrameHostInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = FrameHostProxy;

  template <typename ImplRefTraits>
  using Stub_ = FrameHostStub<ImplRefTraits>;

  using RequestValidator_ = FrameHostRequestValidator;
  using ResponseValidator_ = FrameHostResponseValidator;
  enum MethodMinVersions : uint32_t {
    kCreateNewWindowMinVersion = 0,
  };
  virtual ~FrameHost() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool CreateNewWindow(CreateNewWindowParamsPtr params, CreateNewWindowReplyPtr* reply);

  using CreateNewWindowCallback = base::OnceCallback<void(CreateNewWindowReplyPtr)>;
  virtual void CreateNewWindow(CreateNewWindowParamsPtr params, CreateNewWindowCallback callback) = 0;
};

class CONTENT_EXPORT FrameProxy
    : public Frame {
 public:
  explicit FrameProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetInterfaceProvider(::service_manager::mojom::InterfaceProviderRequest interfaces) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT FrameBindingsControlProxy
    : public FrameBindingsControl {
 public:
  explicit FrameBindingsControlProxy(mojo::MessageReceiverWithResponder* receiver);
  void AllowBindings(int32_t enabled_bindings_flags) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT FrameHostInterfaceBrokerProxy
    : public FrameHostInterfaceBroker {
 public:
  explicit FrameHostInterfaceBrokerProxy(mojo::MessageReceiverWithResponder* receiver);
  void GetInterfaceProvider(::service_manager::mojom::InterfaceProviderRequest interfaces) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT FrameFactoryProxy
    : public FrameFactory {
 public:
  explicit FrameFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateFrame(int32_t frame_routing_id, FrameRequest frame, FrameHostInterfaceBrokerPtr host) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT FrameHostProxy
    : public FrameHost {
 public:
  explicit FrameHostProxy(mojo::MessageReceiverWithResponder* receiver);
  bool CreateNewWindow(CreateNewWindowParamsPtr params, CreateNewWindowReplyPtr* reply) override;
  void CreateNewWindow(CreateNewWindowParamsPtr params, CreateNewWindowCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT FrameStubDispatch {
 public:
  static bool Accept(Frame* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Frame* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Frame>>
class FrameStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameStub() {}
  ~FrameStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT FrameBindingsControlStubDispatch {
 public:
  static bool Accept(FrameBindingsControl* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameBindingsControl* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameBindingsControl>>
class FrameBindingsControlStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameBindingsControlStub() {}
  ~FrameBindingsControlStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameBindingsControlStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameBindingsControlStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT FrameHostInterfaceBrokerStubDispatch {
 public:
  static bool Accept(FrameHostInterfaceBroker* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameHostInterfaceBroker* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameHostInterfaceBroker>>
class FrameHostInterfaceBrokerStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameHostInterfaceBrokerStub() {}
  ~FrameHostInterfaceBrokerStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameHostInterfaceBrokerStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameHostInterfaceBrokerStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT FrameFactoryStubDispatch {
 public:
  static bool Accept(FrameFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameFactory>>
class FrameFactoryStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameFactoryStub() {}
  ~FrameFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT FrameHostStubDispatch {
 public:
  static bool Accept(FrameHost* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      FrameHost* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<FrameHost>>
class FrameHostStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  FrameHostStub() {}
  ~FrameHostStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameHostStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return FrameHostStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT FrameRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT FrameBindingsControlRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT FrameHostInterfaceBrokerRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT FrameFactoryRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT FrameHostRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT FrameHostResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};



class CONTENT_EXPORT CreateNewWindowReply {
 public:
  using DataView = CreateNewWindowReplyDataView;
  using Data_ = internal::CreateNewWindowReply_Data;

  template <typename... Args>
  static CreateNewWindowReplyPtr New(Args&&... args) {
    return CreateNewWindowReplyPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CreateNewWindowReplyPtr From(const U& u) {
    return mojo::TypeConverter<CreateNewWindowReplyPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CreateNewWindowReply>::Convert(*this);
  }


  CreateNewWindowReply();

  CreateNewWindowReply(
      int32_t route_id,
      int32_t main_frame_route_id,
      int32_t main_frame_widget_route_id,
      int64_t cloned_session_storage_namespace_id);

  ~CreateNewWindowReply();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CreateNewWindowReplyPtr>
  CreateNewWindowReplyPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CreateNewWindowReply>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CreateNewWindowReply::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CreateNewWindowReply::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  int32_t route_id;
  int32_t main_frame_route_id;
  int32_t main_frame_widget_route_id;
  int64_t cloned_session_storage_namespace_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class CONTENT_EXPORT CreateNewWindowParams {
 public:
  using DataView = CreateNewWindowParamsDataView;
  using Data_ = internal::CreateNewWindowParams_Data;

  template <typename... Args>
  static CreateNewWindowParamsPtr New(Args&&... args) {
    return CreateNewWindowParamsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CreateNewWindowParamsPtr From(const U& u) {
    return mojo::TypeConverter<CreateNewWindowParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CreateNewWindowParams>::Convert(*this);
  }


  CreateNewWindowParams();

  CreateNewWindowParams(
      bool user_gesture,
      ::content::mojom::WindowContainerType window_container_type,
      int64_t session_storage_namespace_id,
      const std::string& frame_name,
      bool opener_suppressed,
      WindowOpenDisposition disposition,
      const GURL& target_url,
      const content::Referrer& referrer,
      ::blink::mojom::WindowFeaturesPtr features);

  ~CreateNewWindowParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CreateNewWindowParamsPtr>
  CreateNewWindowParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CreateNewWindowParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CreateNewWindowParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CreateNewWindowParams::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  bool user_gesture;
  ::content::mojom::WindowContainerType window_container_type;
  int64_t session_storage_namespace_id;
  std::string frame_name;
  bool opener_suppressed;
  WindowOpenDisposition disposition;
  GURL target_url;
  content::Referrer referrer;
  ::blink::mojom::WindowFeaturesPtr features;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(CreateNewWindowParams);
};



template <typename StructPtrType>
CreateNewWindowParamsPtr CreateNewWindowParams::Clone() const {
  return New(
      mojo::Clone(user_gesture),
      mojo::Clone(window_container_type),
      mojo::Clone(session_storage_namespace_id),
      mojo::Clone(frame_name),
      mojo::Clone(opener_suppressed),
      mojo::Clone(disposition),
      mojo::Clone(target_url),
      mojo::Clone(referrer),
      mojo::Clone(features)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CreateNewWindowParams>::value>::type*>
bool CreateNewWindowParams::Equals(const T& other) const {
  if (!mojo::Equals(this->user_gesture, other.user_gesture))
    return false;
  if (!mojo::Equals(this->window_container_type, other.window_container_type))
    return false;
  if (!mojo::Equals(this->session_storage_namespace_id, other.session_storage_namespace_id))
    return false;
  if (!mojo::Equals(this->frame_name, other.frame_name))
    return false;
  if (!mojo::Equals(this->opener_suppressed, other.opener_suppressed))
    return false;
  if (!mojo::Equals(this->disposition, other.disposition))
    return false;
  if (!mojo::Equals(this->target_url, other.target_url))
    return false;
  if (!mojo::Equals(this->referrer, other.referrer))
    return false;
  if (!mojo::Equals(this->features, other.features))
    return false;
  return true;
}
template <typename StructPtrType>
CreateNewWindowReplyPtr CreateNewWindowReply::Clone() const {
  return New(
      mojo::Clone(route_id),
      mojo::Clone(main_frame_route_id),
      mojo::Clone(main_frame_widget_route_id),
      mojo::Clone(cloned_session_storage_namespace_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CreateNewWindowReply>::value>::type*>
bool CreateNewWindowReply::Equals(const T& other) const {
  if (!mojo::Equals(this->route_id, other.route_id))
    return false;
  if (!mojo::Equals(this->main_frame_route_id, other.main_frame_route_id))
    return false;
  if (!mojo::Equals(this->main_frame_widget_route_id, other.main_frame_widget_route_id))
    return false;
  if (!mojo::Equals(this->cloned_session_storage_namespace_id, other.cloned_session_storage_namespace_id))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::CreateNewWindowParams::DataView,
                                         ::content::mojom::CreateNewWindowParamsPtr> {
  static bool IsNull(const ::content::mojom::CreateNewWindowParamsPtr& input) { return !input; }
  static void SetToNull(::content::mojom::CreateNewWindowParamsPtr* output) { output->reset(); }

  static decltype(::content::mojom::CreateNewWindowParams::user_gesture) user_gesture(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->user_gesture;
  }

  static decltype(::content::mojom::CreateNewWindowParams::window_container_type) window_container_type(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->window_container_type;
  }

  static decltype(::content::mojom::CreateNewWindowParams::session_storage_namespace_id) session_storage_namespace_id(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->session_storage_namespace_id;
  }

  static const decltype(::content::mojom::CreateNewWindowParams::frame_name)& frame_name(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->frame_name;
  }

  static decltype(::content::mojom::CreateNewWindowParams::opener_suppressed) opener_suppressed(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->opener_suppressed;
  }

  static decltype(::content::mojom::CreateNewWindowParams::disposition) disposition(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->disposition;
  }

  static const decltype(::content::mojom::CreateNewWindowParams::target_url)& target_url(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->target_url;
  }

  static const decltype(::content::mojom::CreateNewWindowParams::referrer)& referrer(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->referrer;
  }

  static const decltype(::content::mojom::CreateNewWindowParams::features)& features(
      const ::content::mojom::CreateNewWindowParamsPtr& input) {
    return input->features;
  }

  static bool Read(::content::mojom::CreateNewWindowParams::DataView input, ::content::mojom::CreateNewWindowParamsPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::CreateNewWindowReply::DataView,
                                         ::content::mojom::CreateNewWindowReplyPtr> {
  static bool IsNull(const ::content::mojom::CreateNewWindowReplyPtr& input) { return !input; }
  static void SetToNull(::content::mojom::CreateNewWindowReplyPtr* output) { output->reset(); }

  static decltype(::content::mojom::CreateNewWindowReply::route_id) route_id(
      const ::content::mojom::CreateNewWindowReplyPtr& input) {
    return input->route_id;
  }

  static decltype(::content::mojom::CreateNewWindowReply::main_frame_route_id) main_frame_route_id(
      const ::content::mojom::CreateNewWindowReplyPtr& input) {
    return input->main_frame_route_id;
  }

  static decltype(::content::mojom::CreateNewWindowReply::main_frame_widget_route_id) main_frame_widget_route_id(
      const ::content::mojom::CreateNewWindowReplyPtr& input) {
    return input->main_frame_widget_route_id;
  }

  static decltype(::content::mojom::CreateNewWindowReply::cloned_session_storage_namespace_id) cloned_session_storage_namespace_id(
      const ::content::mojom::CreateNewWindowReplyPtr& input) {
    return input->cloned_session_storage_namespace_id;
  }

  static bool Read(::content::mojom::CreateNewWindowReply::DataView input, ::content::mojom::CreateNewWindowReplyPtr* output);
};

}  // namespace mojo

#endif  // CONTENT_COMMON_FRAME_MOJOM_H_