// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_MEDIA_MEDIA_DEVICES_MOJOM_SHARED_INTERNAL_H_
#define CONTENT_COMMON_MEDIA_MEDIA_DEVICES_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "media/capture/mojo/video_capture_types.mojom-shared-internal.h"
#include "media/mojo/interfaces/audio_parameters.mojom-shared-internal.h"
#include "content/common/mojo_bindings_shared_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace mojom {
namespace internal {
using MediaDeviceInfo_Data = mojo::internal::NativeStruct_Data;
class VideoInputDeviceCapabilities_Data;
class AudioInputDeviceCapabilities_Data;
using MediaDeviceType_Data =
    mojo::internal::NativeEnum_Data;

struct FacingMode_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class MOJOM_SHARED_CONTENT_EXPORT VideoInputDeviceCapabilities_Data {
 public:
  static VideoInputDeviceCapabilities_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> device_id;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::media::mojom::internal::VideoCaptureFormat_Data>>> formats;
  int32_t facing_mode;
  uint8_t padfinal_[4];

 private:
  VideoInputDeviceCapabilities_Data();
  ~VideoInputDeviceCapabilities_Data() = delete;
};
static_assert(sizeof(VideoInputDeviceCapabilities_Data) == 32,
              "Bad sizeof(VideoInputDeviceCapabilities_Data)");
class MOJOM_SHARED_CONTENT_EXPORT AudioInputDeviceCapabilities_Data {
 public:
  static AudioInputDeviceCapabilities_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> device_id;
  mojo::internal::Pointer<::media::mojom::internal::AudioParameters_Data> parameters;

 private:
  AudioInputDeviceCapabilities_Data();
  ~AudioInputDeviceCapabilities_Data() = delete;
};
static_assert(sizeof(AudioInputDeviceCapabilities_Data) == 24,
              "Bad sizeof(AudioInputDeviceCapabilities_Data)");
constexpr uint32_t kMediaDevicesDispatcherHost_EnumerateDevices_Name = 0;
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_EnumerateDevices_Params_Data {
 public:
  static MediaDevicesDispatcherHost_EnumerateDevices_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t request_audio_input : 1;
  uint8_t request_video_input : 1;
  uint8_t request_audio_output : 1;
  uint8_t padfinal_[7];

 private:
  MediaDevicesDispatcherHost_EnumerateDevices_Params_Data();
  ~MediaDevicesDispatcherHost_EnumerateDevices_Params_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_EnumerateDevices_Params_Data) == 16,
              "Bad sizeof(MediaDevicesDispatcherHost_EnumerateDevices_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data {
 public:
  static MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::MediaDeviceInfo_Data>>>>> enumeration;

 private:
  MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data();
  ~MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data) == 16,
              "Bad sizeof(MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data)");
constexpr uint32_t kMediaDevicesDispatcherHost_GetVideoInputCapabilities_Name = 1;
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data {
 public:
  static MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data();
  ~MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data) == 8,
              "Bad sizeof(MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data {
 public:
  static MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::VideoInputDeviceCapabilities_Data>>> video_input_device_capabilities;

 private:
  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data();
  ~MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data) == 16,
              "Bad sizeof(MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data)");
constexpr uint32_t kMediaDevicesDispatcherHost_GetAudioInputCapabilities_Name = 2;
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data {
 public:
  static MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data();
  ~MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data) == 8,
              "Bad sizeof(MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data {
 public:
  static MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::AudioInputDeviceCapabilities_Data>>> audio_input_device_capabilities;

 private:
  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data();
  ~MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data) == 16,
              "Bad sizeof(MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data)");
constexpr uint32_t kMediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Name = 3;
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data {
 public:
  static MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t type;
  uint32_t subscription_id;

 private:
  MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data();
  ~MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data) == 16,
              "Bad sizeof(MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data)");
constexpr uint32_t kMediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Name = 4;
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data {
 public:
  static MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t type;
  uint32_t subscription_id;

 private:
  MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data();
  ~MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data() = delete;
};
static_assert(sizeof(MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data) == 16,
              "Bad sizeof(MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data)");
constexpr uint32_t kMediaDevicesListener_OnDevicesChanged_Name = 0;
class MOJOM_SHARED_CONTENT_EXPORT MediaDevicesListener_OnDevicesChanged_Params_Data {
 public:
  static MediaDevicesListener_OnDevicesChanged_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t type;
  uint32_t subscription_id;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::MediaDeviceInfo_Data>>> device_infos;

 private:
  MediaDevicesListener_OnDevicesChanged_Params_Data();
  ~MediaDevicesListener_OnDevicesChanged_Params_Data() = delete;
};
static_assert(sizeof(MediaDevicesListener_OnDevicesChanged_Params_Data) == 24,
              "Bad sizeof(MediaDevicesListener_OnDevicesChanged_Params_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom

#endif  // CONTENT_COMMON_MEDIA_MEDIA_DEVICES_MOJOM_SHARED_INTERNAL_H_