// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_MEDIA_MEDIA_DEVICES_MOJOM_SHARED_H_
#define CONTENT_COMMON_MEDIA_MEDIA_DEVICES_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "content/common/media/media_devices.mojom-shared-internal.h"
#include "media/capture/mojo/video_capture_types.mojom-shared.h"
#include "media/mojo/interfaces/audio_parameters.mojom-shared.h"
#include "content/common/mojo_bindings_shared_export.h"


namespace mojom {
using MediaDeviceInfoDataView = mojo::NativeStructDataView;

class VideoInputDeviceCapabilitiesDataView;

class AudioInputDeviceCapabilitiesDataView;



}  // namespace mojom

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::mojom::VideoInputDeviceCapabilitiesDataView> {
  using Data = ::mojom::internal::VideoInputDeviceCapabilities_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::mojom::AudioInputDeviceCapabilitiesDataView> {
  using Data = ::mojom::internal::AudioInputDeviceCapabilities_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace mojom {
using MediaDeviceType = mojo::NativeEnum;

enum class FacingMode : int32_t {
  NONE,
  USER,
  ENVIRONMENT,
  LEFT,
  RIGHT,
};

inline std::ostream& operator<<(std::ostream& os, FacingMode value) {
  switch(value) {
    case FacingMode::NONE:
      return os << "FacingMode::NONE";
    case FacingMode::USER:
      return os << "FacingMode::USER";
    case FacingMode::ENVIRONMENT:
      return os << "FacingMode::ENVIRONMENT";
    case FacingMode::LEFT:
      return os << "FacingMode::LEFT";
    case FacingMode::RIGHT:
      return os << "FacingMode::RIGHT";
    default:
      return os << "Unknown FacingMode value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(FacingMode value) {
  return internal::FacingMode_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class MediaDevicesDispatcherHostInterfaceBase {};

using MediaDevicesDispatcherHostPtrDataView =
    mojo::InterfacePtrDataView<MediaDevicesDispatcherHostInterfaceBase>;
using MediaDevicesDispatcherHostRequestDataView =
    mojo::InterfaceRequestDataView<MediaDevicesDispatcherHostInterfaceBase>;
using MediaDevicesDispatcherHostAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<MediaDevicesDispatcherHostInterfaceBase>;
using MediaDevicesDispatcherHostAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<MediaDevicesDispatcherHostInterfaceBase>;
class MediaDevicesListenerInterfaceBase {};

using MediaDevicesListenerPtrDataView =
    mojo::InterfacePtrDataView<MediaDevicesListenerInterfaceBase>;
using MediaDevicesListenerRequestDataView =
    mojo::InterfaceRequestDataView<MediaDevicesListenerInterfaceBase>;
using MediaDevicesListenerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<MediaDevicesListenerInterfaceBase>;
using MediaDevicesListenerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<MediaDevicesListenerInterfaceBase>;
class VideoInputDeviceCapabilitiesDataView {
 public:
  VideoInputDeviceCapabilitiesDataView() {}

  VideoInputDeviceCapabilitiesDataView(
      internal::VideoInputDeviceCapabilities_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDeviceIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDeviceId(UserType* output) {
    auto* pointer = data_->device_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetFormatsDataView(
      mojo::ArrayDataView<::media::mojom::VideoCaptureFormatDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFormats(UserType* output) {
    auto* pointer = data_->formats.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::media::mojom::VideoCaptureFormatDataView>>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFacingMode(UserType* output) const {
    auto data_value = data_->facing_mode;
    return mojo::internal::Deserialize<::mojom::FacingMode>(
        data_value, output);
  }

  FacingMode facing_mode() const {
    return static_cast<FacingMode>(data_->facing_mode);
  }
 private:
  internal::VideoInputDeviceCapabilities_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AudioInputDeviceCapabilitiesDataView {
 public:
  AudioInputDeviceCapabilitiesDataView() {}

  AudioInputDeviceCapabilitiesDataView(
      internal::AudioInputDeviceCapabilities_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDeviceIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDeviceId(UserType* output) {
    auto* pointer = data_->device_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetParametersDataView(
      ::media::mojom::AudioParametersDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadParameters(UserType* output) {
    auto* pointer = data_->parameters.Get();
    return mojo::internal::Deserialize<::media::mojom::AudioParametersDataView>(
        pointer, output, context_);
  }
 private:
  internal::AudioInputDeviceCapabilities_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaDevicesDispatcherHost_EnumerateDevices_ParamsDataView {
 public:
  MediaDevicesDispatcherHost_EnumerateDevices_ParamsDataView() {}

  MediaDevicesDispatcherHost_EnumerateDevices_ParamsDataView(
      internal::MediaDevicesDispatcherHost_EnumerateDevices_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool request_audio_input() const {
    return data_->request_audio_input;
  }
  bool request_video_input() const {
    return data_->request_video_input;
  }
  bool request_audio_output() const {
    return data_->request_audio_output;
  }
 private:
  internal::MediaDevicesDispatcherHost_EnumerateDevices_Params_Data* data_ = nullptr;
};

class MediaDevicesDispatcherHost_EnumerateDevices_ResponseParamsDataView {
 public:
  MediaDevicesDispatcherHost_EnumerateDevices_ResponseParamsDataView() {}

  MediaDevicesDispatcherHost_EnumerateDevices_ResponseParamsDataView(
      internal::MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetEnumerationDataView(
      mojo::ArrayDataView<mojo::ArrayDataView<MediaDeviceInfoDataView>>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEnumeration(UserType* output) {
    auto* pointer = data_->enumeration.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::ArrayDataView<::mojom::MediaDeviceInfoDataView>>>(
        pointer, output, context_);
  }
 private:
  internal::MediaDevicesDispatcherHost_EnumerateDevices_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaDevicesDispatcherHost_GetVideoInputCapabilities_ParamsDataView {
 public:
  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ParamsDataView() {}

  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ParamsDataView(
      internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_Params_Data* data_ = nullptr;
};

class MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParamsDataView {
 public:
  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParamsDataView() {}

  MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParamsDataView(
      internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetVideoInputDeviceCapabilitiesDataView(
      mojo::ArrayDataView<VideoInputDeviceCapabilitiesDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadVideoInputDeviceCapabilities(UserType* output) {
    auto* pointer = data_->video_input_device_capabilities.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::mojom::VideoInputDeviceCapabilitiesDataView>>(
        pointer, output, context_);
  }
 private:
  internal::MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaDevicesDispatcherHost_GetAudioInputCapabilities_ParamsDataView {
 public:
  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ParamsDataView() {}

  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ParamsDataView(
      internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_Params_Data* data_ = nullptr;
};

class MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParamsDataView {
 public:
  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParamsDataView() {}

  MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParamsDataView(
      internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetAudioInputDeviceCapabilitiesDataView(
      mojo::ArrayDataView<AudioInputDeviceCapabilitiesDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAudioInputDeviceCapabilities(UserType* output) {
    auto* pointer = data_->audio_input_device_capabilities.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::mojom::AudioInputDeviceCapabilitiesDataView>>(
        pointer, output, context_);
  }
 private:
  internal::MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_ParamsDataView {
 public:
  MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_ParamsDataView() {}

  MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_ParamsDataView(
      internal::MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::mojom::MediaDeviceType>(
        data_value, output);
  }

  MediaDeviceType type() const {
    return static_cast<MediaDeviceType>(data_->type);
  }
  uint32_t subscription_id() const {
    return data_->subscription_id;
  }
 private:
  internal::MediaDevicesDispatcherHost_SubscribeDeviceChangeNotifications_Params_Data* data_ = nullptr;
};

class MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_ParamsDataView {
 public:
  MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_ParamsDataView() {}

  MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_ParamsDataView(
      internal::MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::mojom::MediaDeviceType>(
        data_value, output);
  }

  MediaDeviceType type() const {
    return static_cast<MediaDeviceType>(data_->type);
  }
  uint32_t subscription_id() const {
    return data_->subscription_id;
  }
 private:
  internal::MediaDevicesDispatcherHost_UnsubscribeDeviceChangeNotifications_Params_Data* data_ = nullptr;
};

class MediaDevicesListener_OnDevicesChanged_ParamsDataView {
 public:
  MediaDevicesListener_OnDevicesChanged_ParamsDataView() {}

  MediaDevicesListener_OnDevicesChanged_ParamsDataView(
      internal::MediaDevicesListener_OnDevicesChanged_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::mojom::MediaDeviceType>(
        data_value, output);
  }

  MediaDeviceType type() const {
    return static_cast<MediaDeviceType>(data_->type);
  }
  uint32_t subscription_id() const {
    return data_->subscription_id;
  }
  inline void GetDeviceInfosDataView(
      mojo::ArrayDataView<MediaDeviceInfoDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDeviceInfos(UserType* output) {
    auto* pointer = data_->device_infos.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::mojom::MediaDeviceInfoDataView>>(
        pointer, output, context_);
  }
 private:
  internal::MediaDevicesListener_OnDevicesChanged_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom

namespace std {

template <>
struct hash<::mojom::FacingMode>
    : public mojo::internal::EnumHashImpl<::mojom::FacingMode> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::mojom::FacingMode, ::mojom::FacingMode> {
  static ::mojom::FacingMode ToMojom(::mojom::FacingMode input) { return input; }
  static bool FromMojom(::mojom::FacingMode input, ::mojom::FacingMode* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojom::FacingMode, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::mojom::FacingMode, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::mojom::FacingMode>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojom::VideoInputDeviceCapabilitiesDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mojom::VideoInputDeviceCapabilitiesDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::mojom::internal::VideoInputDeviceCapabilities_Data);
    decltype(CallWithContext(Traits::device_id, input, custom_context)) in_device_id = CallWithContext(Traits::device_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_device_id, context);
    decltype(CallWithContext(Traits::formats, input, custom_context)) in_formats = CallWithContext(Traits::formats, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::media::mojom::VideoCaptureFormatDataView>>(
        in_formats, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mojom::internal::VideoInputDeviceCapabilities_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::mojom::internal::VideoInputDeviceCapabilities_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::device_id, input, custom_context)) in_device_id = CallWithContext(Traits::device_id, input, custom_context);
    typename decltype(result->device_id)::BaseType* device_id_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_device_id, buffer, &device_id_ptr, context);
    result->device_id.Set(device_id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->device_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null device_id in VideoInputDeviceCapabilities struct");
    decltype(CallWithContext(Traits::formats, input, custom_context)) in_formats = CallWithContext(Traits::formats, input, custom_context);
    typename decltype(result->formats)::BaseType* formats_ptr;
    const mojo::internal::ContainerValidateParams formats_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::media::mojom::VideoCaptureFormatDataView>>(
        in_formats, buffer, &formats_ptr, &formats_validate_params,
        context);
    result->formats.Set(formats_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->formats.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null formats in VideoInputDeviceCapabilities struct");
    mojo::internal::Serialize<::mojom::FacingMode>(
        CallWithContext(Traits::facing_mode, input, custom_context), &result->facing_mode);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::mojom::internal::VideoInputDeviceCapabilities_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mojom::VideoInputDeviceCapabilitiesDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::mojom::AudioInputDeviceCapabilitiesDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::mojom::AudioInputDeviceCapabilitiesDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::mojom::internal::AudioInputDeviceCapabilities_Data);
    decltype(CallWithContext(Traits::device_id, input, custom_context)) in_device_id = CallWithContext(Traits::device_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_device_id, context);
    decltype(CallWithContext(Traits::parameters, input, custom_context)) in_parameters = CallWithContext(Traits::parameters, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::media::mojom::AudioParametersDataView>(
        in_parameters, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::mojom::internal::AudioInputDeviceCapabilities_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::mojom::internal::AudioInputDeviceCapabilities_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::device_id, input, custom_context)) in_device_id = CallWithContext(Traits::device_id, input, custom_context);
    typename decltype(result->device_id)::BaseType* device_id_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_device_id, buffer, &device_id_ptr, context);
    result->device_id.Set(device_id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->device_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null device_id in AudioInputDeviceCapabilities struct");
    decltype(CallWithContext(Traits::parameters, input, custom_context)) in_parameters = CallWithContext(Traits::parameters, input, custom_context);
    typename decltype(result->parameters)::BaseType* parameters_ptr;
    mojo::internal::Serialize<::media::mojom::AudioParametersDataView>(
        in_parameters, buffer, &parameters_ptr, context);
    result->parameters.Set(parameters_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->parameters.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null parameters in AudioInputDeviceCapabilities struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::mojom::internal::AudioInputDeviceCapabilities_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::mojom::AudioInputDeviceCapabilitiesDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace mojom {

inline void VideoInputDeviceCapabilitiesDataView::GetDeviceIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->device_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void VideoInputDeviceCapabilitiesDataView::GetFormatsDataView(
    mojo::ArrayDataView<::media::mojom::VideoCaptureFormatDataView>* output) {
  auto pointer = data_->formats.Get();
  *output = mojo::ArrayDataView<::media::mojom::VideoCaptureFormatDataView>(pointer, context_);
}


inline void AudioInputDeviceCapabilitiesDataView::GetDeviceIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->device_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void AudioInputDeviceCapabilitiesDataView::GetParametersDataView(
    ::media::mojom::AudioParametersDataView* output) {
  auto pointer = data_->parameters.Get();
  *output = ::media::mojom::AudioParametersDataView(pointer, context_);
}




inline void MediaDevicesDispatcherHost_EnumerateDevices_ResponseParamsDataView::GetEnumerationDataView(
    mojo::ArrayDataView<mojo::ArrayDataView<MediaDeviceInfoDataView>>* output) {
  auto pointer = data_->enumeration.Get();
  *output = mojo::ArrayDataView<mojo::ArrayDataView<MediaDeviceInfoDataView>>(pointer, context_);
}




inline void MediaDevicesDispatcherHost_GetVideoInputCapabilities_ResponseParamsDataView::GetVideoInputDeviceCapabilitiesDataView(
    mojo::ArrayDataView<VideoInputDeviceCapabilitiesDataView>* output) {
  auto pointer = data_->video_input_device_capabilities.Get();
  *output = mojo::ArrayDataView<VideoInputDeviceCapabilitiesDataView>(pointer, context_);
}




inline void MediaDevicesDispatcherHost_GetAudioInputCapabilities_ResponseParamsDataView::GetAudioInputDeviceCapabilitiesDataView(
    mojo::ArrayDataView<AudioInputDeviceCapabilitiesDataView>* output) {
  auto pointer = data_->audio_input_device_capabilities.Get();
  *output = mojo::ArrayDataView<AudioInputDeviceCapabilitiesDataView>(pointer, context_);
}






inline void MediaDevicesListener_OnDevicesChanged_ParamsDataView::GetDeviceInfosDataView(
    mojo::ArrayDataView<MediaDeviceInfoDataView>* output) {
  auto pointer = data_->device_infos.Get();
  *output = mojo::ArrayDataView<MediaDeviceInfoDataView>(pointer, context_);
}



}  // namespace mojom

#endif  // CONTENT_COMMON_MEDIA_MEDIA_DEVICES_MOJOM_SHARED_H_
