// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_PUSH_MESSAGING_MOJOM_SHARED_INTERNAL_H_
#define CONTENT_COMMON_PUSH_MESSAGING_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "content/public/common/push_messaging_status.mojom-shared-internal.h"
#include "url/mojo/url.mojom-shared-internal.h"
#include "content/common/mojo_bindings_shared_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace content {
namespace mojom {
namespace internal {
class PushSubscriptionOptions_Data;

struct PushErrorType_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
      case 4:
      case 5:
      case 6:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct PushPermissionStatus_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class MOJOM_SHARED_CONTENT_EXPORT PushSubscriptionOptions_Data {
 public:
  static PushSubscriptionOptions_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t user_visible_only : 1;
  uint8_t pad0_[7];
  mojo::internal::Pointer<mojo::internal::String_Data> sender_info;

 private:
  PushSubscriptionOptions_Data();
  ~PushSubscriptionOptions_Data() = delete;
};
static_assert(sizeof(PushSubscriptionOptions_Data) == 24,
              "Bad sizeof(PushSubscriptionOptions_Data)");
constexpr uint32_t kPushMessaging_Subscribe_Name = 0;
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_Subscribe_Params_Data {
 public:
  static PushMessaging_Subscribe_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t render_frame_id;
  uint8_t user_gesture : 1;
  uint8_t pad1_[3];
  int64_t service_worker_registration_id;
  mojo::internal::Pointer<internal::PushSubscriptionOptions_Data> options;

 private:
  PushMessaging_Subscribe_Params_Data();
  ~PushMessaging_Subscribe_Params_Data() = delete;
};
static_assert(sizeof(PushMessaging_Subscribe_Params_Data) == 32,
              "Bad sizeof(PushMessaging_Subscribe_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_Subscribe_ResponseParams_Data {
 public:
  static PushMessaging_Subscribe_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t status;
  uint8_t pad0_[4];
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> endpoint;
  mojo::internal::Pointer<internal::PushSubscriptionOptions_Data> options;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> p256dh;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> auth;

 private:
  PushMessaging_Subscribe_ResponseParams_Data();
  ~PushMessaging_Subscribe_ResponseParams_Data() = delete;
};
static_assert(sizeof(PushMessaging_Subscribe_ResponseParams_Data) == 48,
              "Bad sizeof(PushMessaging_Subscribe_ResponseParams_Data)");
constexpr uint32_t kPushMessaging_Unsubscribe_Name = 1;
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_Unsubscribe_Params_Data {
 public:
  static PushMessaging_Unsubscribe_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t service_worker_registration_id;

 private:
  PushMessaging_Unsubscribe_Params_Data();
  ~PushMessaging_Unsubscribe_Params_Data() = delete;
};
static_assert(sizeof(PushMessaging_Unsubscribe_Params_Data) == 16,
              "Bad sizeof(PushMessaging_Unsubscribe_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_Unsubscribe_ResponseParams_Data {
 public:
  static PushMessaging_Unsubscribe_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_type;
  uint8_t did_unsubscribe : 1;
  uint8_t pad1_[3];
  mojo::internal::Pointer<mojo::internal::String_Data> error_message;

 private:
  PushMessaging_Unsubscribe_ResponseParams_Data();
  ~PushMessaging_Unsubscribe_ResponseParams_Data() = delete;
};
static_assert(sizeof(PushMessaging_Unsubscribe_ResponseParams_Data) == 24,
              "Bad sizeof(PushMessaging_Unsubscribe_ResponseParams_Data)");
constexpr uint32_t kPushMessaging_GetSubscription_Name = 2;
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_GetSubscription_Params_Data {
 public:
  static PushMessaging_GetSubscription_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t service_worker_registration_id;

 private:
  PushMessaging_GetSubscription_Params_Data();
  ~PushMessaging_GetSubscription_Params_Data() = delete;
};
static_assert(sizeof(PushMessaging_GetSubscription_Params_Data) == 16,
              "Bad sizeof(PushMessaging_GetSubscription_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_GetSubscription_ResponseParams_Data {
 public:
  static PushMessaging_GetSubscription_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t status;
  uint8_t pad0_[4];
  mojo::internal::Pointer<::url::mojom::internal::Url_Data> endpoint;
  mojo::internal::Pointer<internal::PushSubscriptionOptions_Data> options;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> p256dh;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> auth;

 private:
  PushMessaging_GetSubscription_ResponseParams_Data();
  ~PushMessaging_GetSubscription_ResponseParams_Data() = delete;
};
static_assert(sizeof(PushMessaging_GetSubscription_ResponseParams_Data) == 48,
              "Bad sizeof(PushMessaging_GetSubscription_ResponseParams_Data)");
constexpr uint32_t kPushMessaging_GetPermissionStatus_Name = 3;
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_GetPermissionStatus_Params_Data {
 public:
  static PushMessaging_GetPermissionStatus_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int64_t service_worker_registration_id;
  uint8_t user_visible : 1;
  uint8_t padfinal_[7];

 private:
  PushMessaging_GetPermissionStatus_Params_Data();
  ~PushMessaging_GetPermissionStatus_Params_Data() = delete;
};
static_assert(sizeof(PushMessaging_GetPermissionStatus_Params_Data) == 24,
              "Bad sizeof(PushMessaging_GetPermissionStatus_Params_Data)");
class MOJOM_SHARED_CONTENT_EXPORT PushMessaging_GetPermissionStatus_ResponseParams_Data {
 public:
  static PushMessaging_GetPermissionStatus_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t error_type;
  int32_t status;

 private:
  PushMessaging_GetPermissionStatus_ResponseParams_Data();
  ~PushMessaging_GetPermissionStatus_ResponseParams_Data() = delete;
};
static_assert(sizeof(PushMessaging_GetPermissionStatus_ResponseParams_Data) == 16,
              "Bad sizeof(PushMessaging_GetPermissionStatus_ResponseParams_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_PUSH_MESSAGING_MOJOM_SHARED_INTERNAL_H_