// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "content/common/push_messaging.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "content/common/push_messaging_param_traits.h"
#include "url/mojo/url_gurl_struct_traits.h"
namespace content {
namespace mojom {
PushSubscriptionOptions::PushSubscriptionOptions()
    : user_visible_only(),
      sender_info() {}

PushSubscriptionOptions::PushSubscriptionOptions(
    bool user_visible_only_in,
    const std::string& sender_info_in)
    : user_visible_only(std::move(user_visible_only_in)),
      sender_info(std::move(sender_info_in)) {}

PushSubscriptionOptions::~PushSubscriptionOptions() = default;

bool PushSubscriptionOptions::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char PushMessaging::Name_[] = "content::mojom::PushMessaging";

class PushMessaging_Subscribe_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PushMessaging_Subscribe_ForwardToCallback(
      PushMessaging::SubscribeCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PushMessaging::SubscribeCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PushMessaging_Subscribe_ForwardToCallback);
};

class PushMessaging_Unsubscribe_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PushMessaging_Unsubscribe_ForwardToCallback(
      PushMessaging::UnsubscribeCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PushMessaging::UnsubscribeCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PushMessaging_Unsubscribe_ForwardToCallback);
};

class PushMessaging_GetSubscription_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PushMessaging_GetSubscription_ForwardToCallback(
      PushMessaging::GetSubscriptionCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PushMessaging::GetSubscriptionCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PushMessaging_GetSubscription_ForwardToCallback);
};

class PushMessaging_GetPermissionStatus_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PushMessaging_GetPermissionStatus_ForwardToCallback(
      PushMessaging::GetPermissionStatusCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PushMessaging::GetPermissionStatusCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PushMessaging_GetPermissionStatus_ForwardToCallback);
};

PushMessagingProxy::PushMessagingProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PushMessagingProxy::Subscribe(
    int32_t in_render_frame_id, int64_t in_service_worker_registration_id, const content::PushSubscriptionOptions& in_options, bool in_user_gesture, SubscribeCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_Subscribe_Params_Data);
  size += mojo::internal::PrepareToSerialize<::content::mojom::PushSubscriptionOptionsDataView>(
      in_options, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_Subscribe_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_Subscribe_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->render_frame_id = in_render_frame_id;
  params->service_worker_registration_id = in_service_worker_registration_id;
  typename decltype(params->options)::BaseType* options_ptr;
  mojo::internal::Serialize<::content::mojom::PushSubscriptionOptionsDataView>(
      in_options, buffer, &options_ptr, &serialization_context);
  params->options.Set(options_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->options.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null options in PushMessaging.Subscribe request");
  params->user_gesture = in_user_gesture;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PushMessaging_Subscribe_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PushMessagingProxy::Unsubscribe(
    int64_t in_service_worker_registration_id, UnsubscribeCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_Unsubscribe_Params_Data);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_Unsubscribe_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_Unsubscribe_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PushMessaging_Unsubscribe_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PushMessagingProxy::GetSubscription(
    int64_t in_service_worker_registration_id, GetSubscriptionCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_GetSubscription_Params_Data);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_GetSubscription_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_GetSubscription_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PushMessaging_GetSubscription_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PushMessagingProxy::GetPermissionStatus(
    int64_t in_service_worker_registration_id, bool in_user_visible, GetPermissionStatusCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_GetPermissionStatus_Params_Data);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_GetPermissionStatus_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_GetPermissionStatus_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->service_worker_registration_id = in_service_worker_registration_id;
  params->user_visible = in_user_visible;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PushMessaging_GetPermissionStatus_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class PushMessaging_Subscribe_ProxyToResponder {
 public:
  static PushMessaging::SubscribeCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PushMessaging_Subscribe_ProxyToResponder> proxy(
        new PushMessaging_Subscribe_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PushMessaging_Subscribe_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PushMessaging_Subscribe_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PushMessaging::Subscribe() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PushMessaging_Subscribe_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      ::content::mojom::PushRegistrationStatus in_status, const base::Optional<GURL>& in_endpoint, const base::Optional<content::PushSubscriptionOptions>& in_options, const base::Optional<std::vector<uint8_t>>& in_p256dh, const base::Optional<std::vector<uint8_t>>& in_auth);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PushMessaging_Subscribe_ProxyToResponder);
};

bool PushMessaging_Subscribe_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PushMessaging_Subscribe_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PushMessaging_Subscribe_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  ::content::mojom::PushRegistrationStatus p_status{};
  base::Optional<GURL> p_endpoint{};
  base::Optional<content::PushSubscriptionOptions> p_options{};
  base::Optional<std::vector<uint8_t>> p_p256dh{};
  base::Optional<std::vector<uint8_t>> p_auth{};
  PushMessaging_Subscribe_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!input_data_view.ReadEndpoint(&p_endpoint))
    success = false;
  if (!input_data_view.ReadOptions(&p_options))
    success = false;
  if (!input_data_view.ReadP256dh(&p_p256dh))
    success = false;
  if (!input_data_view.ReadAuth(&p_auth))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PushMessaging::Subscribe response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_status), 
std::move(p_endpoint), 
std::move(p_options), 
std::move(p_p256dh), 
std::move(p_auth));
  return true;
}

void PushMessaging_Subscribe_ProxyToResponder::Run(
    ::content::mojom::PushRegistrationStatus in_status, const base::Optional<GURL>& in_endpoint, const base::Optional<content::PushSubscriptionOptions>& in_options, const base::Optional<std::vector<uint8_t>>& in_p256dh, const base::Optional<std::vector<uint8_t>>& in_auth) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_Subscribe_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_endpoint, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::content::mojom::PushSubscriptionOptionsDataView>(
      in_options, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
      in_p256dh, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
      in_auth, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_Subscribe_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_Subscribe_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::content::mojom::PushRegistrationStatus>(
      in_status, &params->status);
  typename decltype(params->endpoint)::BaseType* endpoint_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_endpoint, buffer, &endpoint_ptr, &serialization_context);
  params->endpoint.Set(endpoint_ptr);
  typename decltype(params->options)::BaseType* options_ptr;
  mojo::internal::Serialize<::content::mojom::PushSubscriptionOptionsDataView>(
      in_options, buffer, &options_ptr, &serialization_context);
  params->options.Set(options_ptr);
  typename decltype(params->p256dh)::BaseType* p256dh_ptr;
  const mojo::internal::ContainerValidateParams p256dh_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      in_p256dh, buffer, &p256dh_ptr, &p256dh_validate_params,
      &serialization_context);
  params->p256dh.Set(p256dh_ptr);
  typename decltype(params->auth)::BaseType* auth_ptr;
  const mojo::internal::ContainerValidateParams auth_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      in_auth, buffer, &auth_ptr, &auth_validate_params,
      &serialization_context);
  params->auth.Set(auth_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PushMessaging_Unsubscribe_ProxyToResponder {
 public:
  static PushMessaging::UnsubscribeCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PushMessaging_Unsubscribe_ProxyToResponder> proxy(
        new PushMessaging_Unsubscribe_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PushMessaging_Unsubscribe_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PushMessaging_Unsubscribe_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PushMessaging::Unsubscribe() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PushMessaging_Unsubscribe_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      blink::WebPushError::ErrorType in_error_type, bool in_did_unsubscribe, const base::Optional<std::string>& in_error_message);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PushMessaging_Unsubscribe_ProxyToResponder);
};

bool PushMessaging_Unsubscribe_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PushMessaging_Unsubscribe_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PushMessaging_Unsubscribe_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  blink::WebPushError::ErrorType p_error_type{};
  bool p_did_unsubscribe{};
  base::Optional<std::string> p_error_message{};
  PushMessaging_Unsubscribe_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadErrorType(&p_error_type))
    success = false;
  p_did_unsubscribe = input_data_view.did_unsubscribe();
  if (!input_data_view.ReadErrorMessage(&p_error_message))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PushMessaging::Unsubscribe response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error_type), 
std::move(p_did_unsubscribe), 
std::move(p_error_message));
  return true;
}

void PushMessaging_Unsubscribe_ProxyToResponder::Run(
    blink::WebPushError::ErrorType in_error_type, bool in_did_unsubscribe, const base::Optional<std::string>& in_error_message) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_Unsubscribe_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_error_message, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_Unsubscribe_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_Unsubscribe_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::content::mojom::PushErrorType>(
      in_error_type, &params->error_type);
  params->did_unsubscribe = in_did_unsubscribe;
  typename decltype(params->error_message)::BaseType* error_message_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_error_message, buffer, &error_message_ptr, &serialization_context);
  params->error_message.Set(error_message_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PushMessaging_GetSubscription_ProxyToResponder {
 public:
  static PushMessaging::GetSubscriptionCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PushMessaging_GetSubscription_ProxyToResponder> proxy(
        new PushMessaging_GetSubscription_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PushMessaging_GetSubscription_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PushMessaging_GetSubscription_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PushMessaging::GetSubscription() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PushMessaging_GetSubscription_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      ::content::mojom::PushGetRegistrationStatus in_status, const base::Optional<GURL>& in_endpoint, const base::Optional<content::PushSubscriptionOptions>& in_options, const base::Optional<std::vector<uint8_t>>& in_p256dh, const base::Optional<std::vector<uint8_t>>& in_auth);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PushMessaging_GetSubscription_ProxyToResponder);
};

bool PushMessaging_GetSubscription_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PushMessaging_GetSubscription_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PushMessaging_GetSubscription_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  ::content::mojom::PushGetRegistrationStatus p_status{};
  base::Optional<GURL> p_endpoint{};
  base::Optional<content::PushSubscriptionOptions> p_options{};
  base::Optional<std::vector<uint8_t>> p_p256dh{};
  base::Optional<std::vector<uint8_t>> p_auth{};
  PushMessaging_GetSubscription_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!input_data_view.ReadEndpoint(&p_endpoint))
    success = false;
  if (!input_data_view.ReadOptions(&p_options))
    success = false;
  if (!input_data_view.ReadP256dh(&p_p256dh))
    success = false;
  if (!input_data_view.ReadAuth(&p_auth))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PushMessaging::GetSubscription response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_status), 
std::move(p_endpoint), 
std::move(p_options), 
std::move(p_p256dh), 
std::move(p_auth));
  return true;
}

void PushMessaging_GetSubscription_ProxyToResponder::Run(
    ::content::mojom::PushGetRegistrationStatus in_status, const base::Optional<GURL>& in_endpoint, const base::Optional<content::PushSubscriptionOptions>& in_options, const base::Optional<std::vector<uint8_t>>& in_p256dh, const base::Optional<std::vector<uint8_t>>& in_auth) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_GetSubscription_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_endpoint, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::content::mojom::PushSubscriptionOptionsDataView>(
      in_options, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
      in_p256dh, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
      in_auth, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_GetSubscription_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_GetSubscription_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::content::mojom::PushGetRegistrationStatus>(
      in_status, &params->status);
  typename decltype(params->endpoint)::BaseType* endpoint_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_endpoint, buffer, &endpoint_ptr, &serialization_context);
  params->endpoint.Set(endpoint_ptr);
  typename decltype(params->options)::BaseType* options_ptr;
  mojo::internal::Serialize<::content::mojom::PushSubscriptionOptionsDataView>(
      in_options, buffer, &options_ptr, &serialization_context);
  params->options.Set(options_ptr);
  typename decltype(params->p256dh)::BaseType* p256dh_ptr;
  const mojo::internal::ContainerValidateParams p256dh_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      in_p256dh, buffer, &p256dh_ptr, &p256dh_validate_params,
      &serialization_context);
  params->p256dh.Set(p256dh_ptr);
  typename decltype(params->auth)::BaseType* auth_ptr;
  const mojo::internal::ContainerValidateParams auth_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      in_auth, buffer, &auth_ptr, &auth_validate_params,
      &serialization_context);
  params->auth.Set(auth_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PushMessaging_GetPermissionStatus_ProxyToResponder {
 public:
  static PushMessaging::GetPermissionStatusCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PushMessaging_GetPermissionStatus_ProxyToResponder> proxy(
        new PushMessaging_GetPermissionStatus_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PushMessaging_GetPermissionStatus_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PushMessaging_GetPermissionStatus_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PushMessaging::GetPermissionStatus() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PushMessaging_GetPermissionStatus_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      blink::WebPushError::ErrorType in_error_type, blink::WebPushPermissionStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PushMessaging_GetPermissionStatus_ProxyToResponder);
};

bool PushMessaging_GetPermissionStatus_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PushMessaging_GetPermissionStatus_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PushMessaging_GetPermissionStatus_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  blink::WebPushError::ErrorType p_error_type{};
  blink::WebPushPermissionStatus p_status{};
  PushMessaging_GetPermissionStatus_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadErrorType(&p_error_type))
    success = false;
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PushMessaging::GetPermissionStatus response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_error_type), 
std::move(p_status));
  return true;
}

void PushMessaging_GetPermissionStatus_ProxyToResponder::Run(
    blink::WebPushError::ErrorType in_error_type, blink::WebPushPermissionStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::PushMessaging_GetPermissionStatus_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kPushMessaging_GetPermissionStatus_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::PushMessaging_GetPermissionStatus_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::content::mojom::PushErrorType>(
      in_error_type, &params->error_type);
  mojo::internal::Serialize<::content::mojom::PushPermissionStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool PushMessagingStubDispatch::Accept(
    PushMessaging* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPushMessaging_Subscribe_Name: {
      break;
    }
    case internal::kPushMessaging_Unsubscribe_Name: {
      break;
    }
    case internal::kPushMessaging_GetSubscription_Name: {
      break;
    }
    case internal::kPushMessaging_GetPermissionStatus_Name: {
      break;
    }
  }
  return false;
}

// static
bool PushMessagingStubDispatch::AcceptWithResponder(
    PushMessaging* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPushMessaging_Subscribe_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PushMessaging_Subscribe_Params_Data* params =
          reinterpret_cast<
              internal::PushMessaging_Subscribe_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int32_t p_render_frame_id{};
      int64_t p_service_worker_registration_id{};
      content::PushSubscriptionOptions p_options{};
      bool p_user_gesture{};
      PushMessaging_Subscribe_ParamsDataView input_data_view(params, &serialization_context);
      
      p_render_frame_id = input_data_view.render_frame_id();
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!input_data_view.ReadOptions(&p_options))
        success = false;
      p_user_gesture = input_data_view.user_gesture();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PushMessaging::Subscribe deserializer");
        return false;
      }
      PushMessaging::SubscribeCallback callback =
          PushMessaging_Subscribe_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Subscribe(
std::move(p_render_frame_id), 
std::move(p_service_worker_registration_id), 
std::move(p_options), 
std::move(p_user_gesture), std::move(callback));
      return true;
    }
    case internal::kPushMessaging_Unsubscribe_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PushMessaging_Unsubscribe_Params_Data* params =
          reinterpret_cast<
              internal::PushMessaging_Unsubscribe_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      PushMessaging_Unsubscribe_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PushMessaging::Unsubscribe deserializer");
        return false;
      }
      PushMessaging::UnsubscribeCallback callback =
          PushMessaging_Unsubscribe_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Unsubscribe(
std::move(p_service_worker_registration_id), std::move(callback));
      return true;
    }
    case internal::kPushMessaging_GetSubscription_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PushMessaging_GetSubscription_Params_Data* params =
          reinterpret_cast<
              internal::PushMessaging_GetSubscription_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      PushMessaging_GetSubscription_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PushMessaging::GetSubscription deserializer");
        return false;
      }
      PushMessaging::GetSubscriptionCallback callback =
          PushMessaging_GetSubscription_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetSubscription(
std::move(p_service_worker_registration_id), std::move(callback));
      return true;
    }
    case internal::kPushMessaging_GetPermissionStatus_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PushMessaging_GetPermissionStatus_Params_Data* params =
          reinterpret_cast<
              internal::PushMessaging_GetPermissionStatus_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      int64_t p_service_worker_registration_id{};
      bool p_user_visible{};
      PushMessaging_GetPermissionStatus_ParamsDataView input_data_view(params, &serialization_context);
      
      p_service_worker_registration_id = input_data_view.service_worker_registration_id();
      p_user_visible = input_data_view.user_visible();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PushMessaging::GetPermissionStatus deserializer");
        return false;
      }
      PushMessaging::GetPermissionStatusCallback callback =
          PushMessaging_GetPermissionStatus_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetPermissionStatus(
std::move(p_service_worker_registration_id), 
std::move(p_user_visible), std::move(callback));
      return true;
    }
  }
  return false;
}

bool PushMessagingRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PushMessaging RequestValidator");

  switch (message->header()->name) {
    case internal::kPushMessaging_Subscribe_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_Subscribe_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPushMessaging_Unsubscribe_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_Unsubscribe_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPushMessaging_GetSubscription_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_GetSubscription_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPushMessaging_GetPermissionStatus_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_GetPermissionStatus_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool PushMessagingResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PushMessaging ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kPushMessaging_Subscribe_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_Subscribe_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPushMessaging_Unsubscribe_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_Unsubscribe_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPushMessaging_GetSubscription_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_GetSubscription_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPushMessaging_GetPermissionStatus_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PushMessaging_GetPermissionStatus_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace mojom
}  // namespace content

namespace mojo {


// static
bool StructTraits<::content::mojom::PushSubscriptionOptions::DataView, ::content::mojom::PushSubscriptionOptionsPtr>::Read(
    ::content::mojom::PushSubscriptionOptions::DataView input,
    ::content::mojom::PushSubscriptionOptionsPtr* output) {
  bool success = true;
  ::content::mojom::PushSubscriptionOptionsPtr result(::content::mojom::PushSubscriptionOptions::New());
  
      result->user_visible_only = input.user_visible_only();
      if (!input.ReadSenderInfo(&result->sender_info))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif