// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_RENDER_MESSAGE_FILTER_MOJOM_H_
#define CONTENT_COMMON_RENDER_MESSAGE_FILTER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/render_message_filter.mojom-shared.h"
#include "cc/ipc/shared_bitmap_allocation_notifier.mojom.h"
#include "content/common/input/input_handler.mojom.h"
#include "content/common/native_types.mojom.h"
#include "content/common/widget.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class RenderMessageFilter;
using RenderMessageFilterPtr = mojo::InterfacePtr<RenderMessageFilter>;
using RenderMessageFilterPtrInfo = mojo::InterfacePtrInfo<RenderMessageFilter>;
using ThreadSafeRenderMessageFilterPtr =
    mojo::ThreadSafeInterfacePtr<RenderMessageFilter>;
using RenderMessageFilterRequest = mojo::InterfaceRequest<RenderMessageFilter>;
using RenderMessageFilterAssociatedPtr =
    mojo::AssociatedInterfacePtr<RenderMessageFilter>;
using ThreadSafeRenderMessageFilterAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<RenderMessageFilter>;
using RenderMessageFilterAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<RenderMessageFilter>;
using RenderMessageFilterAssociatedRequest =
    mojo::AssociatedInterfaceRequest<RenderMessageFilter>;


class RenderMessageFilterProxy;

template <typename ImplRefTraits>
class RenderMessageFilterStub;

class RenderMessageFilterRequestValidator;
class RenderMessageFilterResponseValidator;

class CONTENT_EXPORT RenderMessageFilter
    : public RenderMessageFilterInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = RenderMessageFilterProxy;

  template <typename ImplRefTraits>
  using Stub_ = RenderMessageFilterStub<ImplRefTraits>;

  using RequestValidator_ = RenderMessageFilterRequestValidator;
  using ResponseValidator_ = RenderMessageFilterResponseValidator;
  enum MethodMinVersions : uint32_t {
    kGenerateRoutingIDMinVersion = 0,
    kCreateNewWidgetMinVersion = 0,
    kCreateFullscreenWidgetMinVersion = 0,
  };
  virtual ~RenderMessageFilter() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool GenerateRoutingID(int32_t* routing_id);

  using GenerateRoutingIDCallback = base::OnceCallback<void(int32_t)>;
  virtual void GenerateRoutingID(GenerateRoutingIDCallback callback) = 0;

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool CreateNewWidget(int32_t opener_id, blink::WebPopupType popup_type, ::content::mojom::WidgetPtr widget, int32_t* route_id);

  using CreateNewWidgetCallback = base::OnceCallback<void(int32_t)>;
  virtual void CreateNewWidget(int32_t opener_id, blink::WebPopupType popup_type, ::content::mojom::WidgetPtr widget, CreateNewWidgetCallback callback) = 0;

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool CreateFullscreenWidget(int32_t opener_id, ::content::mojom::WidgetPtr widget, int32_t* route_id);

  using CreateFullscreenWidgetCallback = base::OnceCallback<void(int32_t)>;
  virtual void CreateFullscreenWidget(int32_t opener_id, ::content::mojom::WidgetPtr widget, CreateFullscreenWidgetCallback callback) = 0;
};

class CONTENT_EXPORT RenderMessageFilterProxy
    : public RenderMessageFilter {
 public:
  explicit RenderMessageFilterProxy(mojo::MessageReceiverWithResponder* receiver);
  bool GenerateRoutingID(int32_t* routing_id) override;
  void GenerateRoutingID(GenerateRoutingIDCallback callback) override;
  bool CreateNewWidget(int32_t opener_id, blink::WebPopupType popup_type, ::content::mojom::WidgetPtr widget, int32_t* route_id) override;
  void CreateNewWidget(int32_t opener_id, blink::WebPopupType popup_type, ::content::mojom::WidgetPtr widget, CreateNewWidgetCallback callback) override;
  bool CreateFullscreenWidget(int32_t opener_id, ::content::mojom::WidgetPtr widget, int32_t* route_id) override;
  void CreateFullscreenWidget(int32_t opener_id, ::content::mojom::WidgetPtr widget, CreateFullscreenWidgetCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT RenderMessageFilterStubDispatch {
 public:
  static bool Accept(RenderMessageFilter* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      RenderMessageFilter* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<RenderMessageFilter>>
class RenderMessageFilterStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RenderMessageFilterStub() {}
  ~RenderMessageFilterStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RenderMessageFilterStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RenderMessageFilterStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT RenderMessageFilterRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT RenderMessageFilterResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_COMMON_RENDER_MESSAGE_FILTER_MOJOM_H_