// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_RENDER_WIDGET_WINDOW_TREE_CLIENT_FACTORY_MOJOM_H_
#define CONTENT_COMMON_RENDER_WIDGET_WINDOW_TREE_CLIENT_FACTORY_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/render_widget_window_tree_client_factory.mojom-shared.h"
#include "services/ui/public/interfaces/window_tree.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class RenderWidgetWindowTreeClientFactory;
using RenderWidgetWindowTreeClientFactoryPtr = mojo::InterfacePtr<RenderWidgetWindowTreeClientFactory>;
using RenderWidgetWindowTreeClientFactoryPtrInfo = mojo::InterfacePtrInfo<RenderWidgetWindowTreeClientFactory>;
using ThreadSafeRenderWidgetWindowTreeClientFactoryPtr =
    mojo::ThreadSafeInterfacePtr<RenderWidgetWindowTreeClientFactory>;
using RenderWidgetWindowTreeClientFactoryRequest = mojo::InterfaceRequest<RenderWidgetWindowTreeClientFactory>;
using RenderWidgetWindowTreeClientFactoryAssociatedPtr =
    mojo::AssociatedInterfacePtr<RenderWidgetWindowTreeClientFactory>;
using ThreadSafeRenderWidgetWindowTreeClientFactoryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<RenderWidgetWindowTreeClientFactory>;
using RenderWidgetWindowTreeClientFactoryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<RenderWidgetWindowTreeClientFactory>;
using RenderWidgetWindowTreeClientFactoryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<RenderWidgetWindowTreeClientFactory>;


class RenderWidgetWindowTreeClientFactoryProxy;

template <typename ImplRefTraits>
class RenderWidgetWindowTreeClientFactoryStub;

class RenderWidgetWindowTreeClientFactoryRequestValidator;

class CONTENT_EXPORT RenderWidgetWindowTreeClientFactory
    : public RenderWidgetWindowTreeClientFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RenderWidgetWindowTreeClientFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = RenderWidgetWindowTreeClientFactoryStub<ImplRefTraits>;

  using RequestValidator_ = RenderWidgetWindowTreeClientFactoryRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateWindowTreeClientForRenderWidgetMinVersion = 0,
  };
  virtual ~RenderWidgetWindowTreeClientFactory() {}

  virtual void CreateWindowTreeClientForRenderWidget(uint32_t routing_id, ::ui::mojom::WindowTreeClientRequest window_tree_client) = 0;
};

class CONTENT_EXPORT RenderWidgetWindowTreeClientFactoryProxy
    : public RenderWidgetWindowTreeClientFactory {
 public:
  explicit RenderWidgetWindowTreeClientFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateWindowTreeClientForRenderWidget(uint32_t routing_id, ::ui::mojom::WindowTreeClientRequest window_tree_client) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT RenderWidgetWindowTreeClientFactoryStubDispatch {
 public:
  static bool Accept(RenderWidgetWindowTreeClientFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      RenderWidgetWindowTreeClientFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<RenderWidgetWindowTreeClientFactory>>
class RenderWidgetWindowTreeClientFactoryStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RenderWidgetWindowTreeClientFactoryStub() {}
  ~RenderWidgetWindowTreeClientFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RenderWidgetWindowTreeClientFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RenderWidgetWindowTreeClientFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT RenderWidgetWindowTreeClientFactoryRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_COMMON_RENDER_WIDGET_WINDOW_TREE_CLIENT_FACTORY_MOJOM_H_