// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_RENDERER_MOJOM_H_
#define CONTENT_COMMON_RENDERER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/renderer.mojom-shared.h"
#include "content/common/native_types.mojom.h"
#include "ipc/ipc.mojom.h"
#include "mojo/common/time.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/icc_profile.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class Renderer;
using RendererPtr = mojo::InterfacePtr<Renderer>;
using RendererPtrInfo = mojo::InterfacePtrInfo<Renderer>;
using ThreadSafeRendererPtr =
    mojo::ThreadSafeInterfacePtr<Renderer>;
using RendererRequest = mojo::InterfaceRequest<Renderer>;
using RendererAssociatedPtr =
    mojo::AssociatedInterfacePtr<Renderer>;
using ThreadSafeRendererAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Renderer>;
using RendererAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Renderer>;
using RendererAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Renderer>;

class CreateViewParams;
using CreateViewParamsPtr = mojo::StructPtr<CreateViewParams>;

class CreateFrameWidgetParams;
using CreateFrameWidgetParamsPtr = mojo::InlinedStructPtr<CreateFrameWidgetParams>;

class CreateFrameParams;
using CreateFrameParamsPtr = mojo::StructPtr<CreateFrameParams>;

class UpdateScrollbarThemeParams;
using UpdateScrollbarThemeParamsPtr = mojo::StructPtr<UpdateScrollbarThemeParams>;


class RendererProxy;

template <typename ImplRefTraits>
class RendererStub;

class RendererRequestValidator;

class CONTENT_EXPORT Renderer
    : public RendererInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = RendererProxy;

  template <typename ImplRefTraits>
  using Stub_ = RendererStub<ImplRefTraits>;

  using RequestValidator_ = RendererRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateViewMinVersion = 0,
    kCreateFrameMinVersion = 0,
    kCreateFrameProxyMinVersion = 0,
    kOnNetworkConnectionChangedMinVersion = 0,
    kOnNetworkQualityChangedMinVersion = 0,
    kSetWebKitSharedTimersSuspendedMinVersion = 0,
    kUpdateScrollbarThemeMinVersion = 0,
    kOnSystemColorsChangedMinVersion = 0,
    kPurgePluginListCacheMinVersion = 0,
  };
  virtual ~Renderer() {}

  virtual void CreateView(CreateViewParamsPtr params) = 0;

  virtual void CreateFrame(CreateFrameParamsPtr params) = 0;

  virtual void CreateFrameProxy(int32_t routing_id, int32_t render_view_routing_id, int32_t opener_routing_id, int32_t parent_routing_id, const content::FrameReplicationState& replication_state) = 0;

  virtual void OnNetworkConnectionChanged(net::NetworkChangeNotifier::ConnectionType connection_type, double max_bandwidth_mbps) = 0;

  virtual void OnNetworkQualityChanged(net::EffectiveConnectionType effective_connection_type, base::TimeDelta http_rtt, base::TimeDelta transport_rtt, double bandwidth_kbps) = 0;

  virtual void SetWebKitSharedTimersSuspended(bool suspend) = 0;

  virtual void UpdateScrollbarTheme(UpdateScrollbarThemeParamsPtr params) = 0;

  virtual void OnSystemColorsChanged(int32_t aqua_color_variant, const std::string& highlight_text_color, const std::string& highlight_color) = 0;

  virtual void PurgePluginListCache(bool reload_pages) = 0;
};

class CONTENT_EXPORT RendererProxy
    : public Renderer {
 public:
  explicit RendererProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateView(CreateViewParamsPtr params) override;
  void CreateFrame(CreateFrameParamsPtr params) override;
  void CreateFrameProxy(int32_t routing_id, int32_t render_view_routing_id, int32_t opener_routing_id, int32_t parent_routing_id, const content::FrameReplicationState& replication_state) override;
  void OnNetworkConnectionChanged(net::NetworkChangeNotifier::ConnectionType connection_type, double max_bandwidth_mbps) override;
  void OnNetworkQualityChanged(net::EffectiveConnectionType effective_connection_type, base::TimeDelta http_rtt, base::TimeDelta transport_rtt, double bandwidth_kbps) override;
  void SetWebKitSharedTimersSuspended(bool suspend) override;
  void UpdateScrollbarTheme(UpdateScrollbarThemeParamsPtr params) override;
  void OnSystemColorsChanged(int32_t aqua_color_variant, const std::string& highlight_text_color, const std::string& highlight_color) override;
  void PurgePluginListCache(bool reload_pages) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT RendererStubDispatch {
 public:
  static bool Accept(Renderer* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Renderer* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Renderer>>
class RendererStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  RendererStub() {}
  ~RendererStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RendererStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return RendererStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT RendererRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};



class CONTENT_EXPORT CreateFrameWidgetParams {
 public:
  using DataView = CreateFrameWidgetParamsDataView;
  using Data_ = internal::CreateFrameWidgetParams_Data;

  template <typename... Args>
  static CreateFrameWidgetParamsPtr New(Args&&... args) {
    return CreateFrameWidgetParamsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CreateFrameWidgetParamsPtr From(const U& u) {
    return mojo::TypeConverter<CreateFrameWidgetParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CreateFrameWidgetParams>::Convert(*this);
  }


  CreateFrameWidgetParams();

  CreateFrameWidgetParams(
      int32_t routing_id,
      bool hidden);

  ~CreateFrameWidgetParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CreateFrameWidgetParamsPtr>
  CreateFrameWidgetParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CreateFrameWidgetParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CreateFrameWidgetParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CreateFrameWidgetParams::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  int32_t routing_id;
  bool hidden;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};







class CONTENT_EXPORT CreateViewParams {
 public:
  using DataView = CreateViewParamsDataView;
  using Data_ = internal::CreateViewParams_Data;

  template <typename... Args>
  static CreateViewParamsPtr New(Args&&... args) {
    return CreateViewParamsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CreateViewParamsPtr From(const U& u) {
    return mojo::TypeConverter<CreateViewParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CreateViewParams>::Convert(*this);
  }


  CreateViewParams();

  CreateViewParams(
      const content::RendererPreferences& renderer_preferences,
      const content::WebPreferences& web_preferences,
      int32_t view_id,
      int32_t main_frame_routing_id,
      int32_t main_frame_widget_routing_id,
      int64_t session_storage_namespace_id,
      int32_t opener_frame_route_id,
      bool swapped_out,
      const content::FrameReplicationState& replicated_frame_state,
      int32_t proxy_routing_id,
      bool hidden,
      bool never_visible,
      bool window_was_created_with_opener,
      const content::ResizeParams& initial_size,
      bool enable_auto_resize,
      const gfx::Size& min_size,
      const gfx::Size& max_size,
      double page_zoom_level,
      const gfx::ICCProfile& image_decode_color_space);

  ~CreateViewParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CreateViewParamsPtr>
  CreateViewParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CreateViewParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CreateViewParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CreateViewParams::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  content::RendererPreferences renderer_preferences;
  content::WebPreferences web_preferences;
  int32_t view_id;
  int32_t main_frame_routing_id;
  int32_t main_frame_widget_routing_id;
  int64_t session_storage_namespace_id;
  int32_t opener_frame_route_id;
  bool swapped_out;
  content::FrameReplicationState replicated_frame_state;
  int32_t proxy_routing_id;
  bool hidden;
  bool never_visible;
  bool window_was_created_with_opener;
  content::ResizeParams initial_size;
  bool enable_auto_resize;
  gfx::Size min_size;
  gfx::Size max_size;
  double page_zoom_level;
  gfx::ICCProfile image_decode_color_space;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




class CONTENT_EXPORT CreateFrameParams {
 public:
  using DataView = CreateFrameParamsDataView;
  using Data_ = internal::CreateFrameParams_Data;

  template <typename... Args>
  static CreateFrameParamsPtr New(Args&&... args) {
    return CreateFrameParamsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CreateFrameParamsPtr From(const U& u) {
    return mojo::TypeConverter<CreateFrameParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CreateFrameParams>::Convert(*this);
  }


  CreateFrameParams();

  CreateFrameParams(
      int32_t routing_id,
      int32_t proxy_routing_id,
      int32_t opener_routing_id,
      int32_t parent_routing_id,
      int32_t previous_sibling_routing_id,
      const content::FrameReplicationState& replication_state,
      const content::FrameOwnerProperties& frame_owner_properties,
      CreateFrameWidgetParamsPtr widget_params);

  ~CreateFrameParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CreateFrameParamsPtr>
  CreateFrameParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CreateFrameParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CreateFrameParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CreateFrameParams::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  int32_t routing_id;
  int32_t proxy_routing_id;
  int32_t opener_routing_id;
  int32_t parent_routing_id;
  int32_t previous_sibling_routing_id;
  content::FrameReplicationState replication_state;
  content::FrameOwnerProperties frame_owner_properties;
  CreateFrameWidgetParamsPtr widget_params;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(CreateFrameParams);
};



class CONTENT_EXPORT UpdateScrollbarThemeParams {
 public:
  using DataView = UpdateScrollbarThemeParamsDataView;
  using Data_ = internal::UpdateScrollbarThemeParams_Data;

  template <typename... Args>
  static UpdateScrollbarThemeParamsPtr New(Args&&... args) {
    return UpdateScrollbarThemeParamsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static UpdateScrollbarThemeParamsPtr From(const U& u) {
    return mojo::TypeConverter<UpdateScrollbarThemeParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, UpdateScrollbarThemeParams>::Convert(*this);
  }


  UpdateScrollbarThemeParams();

  UpdateScrollbarThemeParams(
      float initial_button_delay,
      float autoscroll_button_delay,
      bool jump_on_track_click,
      ::content::mojom::ScrollerStyle preferred_scroller_style,
      bool redraw,
      ::content::mojom::ScrollbarButtonsPlacement button_placement);

  ~UpdateScrollbarThemeParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = UpdateScrollbarThemeParamsPtr>
  UpdateScrollbarThemeParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, UpdateScrollbarThemeParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        UpdateScrollbarThemeParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        UpdateScrollbarThemeParams::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  float initial_button_delay;
  float autoscroll_button_delay;
  bool jump_on_track_click;
  ::content::mojom::ScrollerStyle preferred_scroller_style;
  bool redraw;
  ::content::mojom::ScrollbarButtonsPlacement button_placement;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
CreateViewParamsPtr CreateViewParams::Clone() const {
  return New(
      mojo::Clone(renderer_preferences),
      mojo::Clone(web_preferences),
      mojo::Clone(view_id),
      mojo::Clone(main_frame_routing_id),
      mojo::Clone(main_frame_widget_routing_id),
      mojo::Clone(session_storage_namespace_id),
      mojo::Clone(opener_frame_route_id),
      mojo::Clone(swapped_out),
      mojo::Clone(replicated_frame_state),
      mojo::Clone(proxy_routing_id),
      mojo::Clone(hidden),
      mojo::Clone(never_visible),
      mojo::Clone(window_was_created_with_opener),
      mojo::Clone(initial_size),
      mojo::Clone(enable_auto_resize),
      mojo::Clone(min_size),
      mojo::Clone(max_size),
      mojo::Clone(page_zoom_level),
      mojo::Clone(image_decode_color_space)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CreateViewParams>::value>::type*>
bool CreateViewParams::Equals(const T& other) const {
  if (!mojo::Equals(this->renderer_preferences, other.renderer_preferences))
    return false;
  if (!mojo::Equals(this->web_preferences, other.web_preferences))
    return false;
  if (!mojo::Equals(this->view_id, other.view_id))
    return false;
  if (!mojo::Equals(this->main_frame_routing_id, other.main_frame_routing_id))
    return false;
  if (!mojo::Equals(this->main_frame_widget_routing_id, other.main_frame_widget_routing_id))
    return false;
  if (!mojo::Equals(this->session_storage_namespace_id, other.session_storage_namespace_id))
    return false;
  if (!mojo::Equals(this->opener_frame_route_id, other.opener_frame_route_id))
    return false;
  if (!mojo::Equals(this->swapped_out, other.swapped_out))
    return false;
  if (!mojo::Equals(this->replicated_frame_state, other.replicated_frame_state))
    return false;
  if (!mojo::Equals(this->proxy_routing_id, other.proxy_routing_id))
    return false;
  if (!mojo::Equals(this->hidden, other.hidden))
    return false;
  if (!mojo::Equals(this->never_visible, other.never_visible))
    return false;
  if (!mojo::Equals(this->window_was_created_with_opener, other.window_was_created_with_opener))
    return false;
  if (!mojo::Equals(this->initial_size, other.initial_size))
    return false;
  if (!mojo::Equals(this->enable_auto_resize, other.enable_auto_resize))
    return false;
  if (!mojo::Equals(this->min_size, other.min_size))
    return false;
  if (!mojo::Equals(this->max_size, other.max_size))
    return false;
  if (!mojo::Equals(this->page_zoom_level, other.page_zoom_level))
    return false;
  if (!mojo::Equals(this->image_decode_color_space, other.image_decode_color_space))
    return false;
  return true;
}
template <typename StructPtrType>
CreateFrameWidgetParamsPtr CreateFrameWidgetParams::Clone() const {
  return New(
      mojo::Clone(routing_id),
      mojo::Clone(hidden)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CreateFrameWidgetParams>::value>::type*>
bool CreateFrameWidgetParams::Equals(const T& other) const {
  if (!mojo::Equals(this->routing_id, other.routing_id))
    return false;
  if (!mojo::Equals(this->hidden, other.hidden))
    return false;
  return true;
}
template <typename StructPtrType>
CreateFrameParamsPtr CreateFrameParams::Clone() const {
  return New(
      mojo::Clone(routing_id),
      mojo::Clone(proxy_routing_id),
      mojo::Clone(opener_routing_id),
      mojo::Clone(parent_routing_id),
      mojo::Clone(previous_sibling_routing_id),
      mojo::Clone(replication_state),
      mojo::Clone(frame_owner_properties),
      mojo::Clone(widget_params)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CreateFrameParams>::value>::type*>
bool CreateFrameParams::Equals(const T& other) const {
  if (!mojo::Equals(this->routing_id, other.routing_id))
    return false;
  if (!mojo::Equals(this->proxy_routing_id, other.proxy_routing_id))
    return false;
  if (!mojo::Equals(this->opener_routing_id, other.opener_routing_id))
    return false;
  if (!mojo::Equals(this->parent_routing_id, other.parent_routing_id))
    return false;
  if (!mojo::Equals(this->previous_sibling_routing_id, other.previous_sibling_routing_id))
    return false;
  if (!mojo::Equals(this->replication_state, other.replication_state))
    return false;
  if (!mojo::Equals(this->frame_owner_properties, other.frame_owner_properties))
    return false;
  if (!mojo::Equals(this->widget_params, other.widget_params))
    return false;
  return true;
}
template <typename StructPtrType>
UpdateScrollbarThemeParamsPtr UpdateScrollbarThemeParams::Clone() const {
  return New(
      mojo::Clone(initial_button_delay),
      mojo::Clone(autoscroll_button_delay),
      mojo::Clone(jump_on_track_click),
      mojo::Clone(preferred_scroller_style),
      mojo::Clone(redraw),
      mojo::Clone(button_placement)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, UpdateScrollbarThemeParams>::value>::type*>
bool UpdateScrollbarThemeParams::Equals(const T& other) const {
  if (!mojo::Equals(this->initial_button_delay, other.initial_button_delay))
    return false;
  if (!mojo::Equals(this->autoscroll_button_delay, other.autoscroll_button_delay))
    return false;
  if (!mojo::Equals(this->jump_on_track_click, other.jump_on_track_click))
    return false;
  if (!mojo::Equals(this->preferred_scroller_style, other.preferred_scroller_style))
    return false;
  if (!mojo::Equals(this->redraw, other.redraw))
    return false;
  if (!mojo::Equals(this->button_placement, other.button_placement))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::CreateViewParams::DataView,
                                         ::content::mojom::CreateViewParamsPtr> {
  static bool IsNull(const ::content::mojom::CreateViewParamsPtr& input) { return !input; }
  static void SetToNull(::content::mojom::CreateViewParamsPtr* output) { output->reset(); }

  static const decltype(::content::mojom::CreateViewParams::renderer_preferences)& renderer_preferences(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->renderer_preferences;
  }

  static const decltype(::content::mojom::CreateViewParams::web_preferences)& web_preferences(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->web_preferences;
  }

  static decltype(::content::mojom::CreateViewParams::view_id) view_id(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->view_id;
  }

  static decltype(::content::mojom::CreateViewParams::main_frame_routing_id) main_frame_routing_id(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->main_frame_routing_id;
  }

  static decltype(::content::mojom::CreateViewParams::main_frame_widget_routing_id) main_frame_widget_routing_id(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->main_frame_widget_routing_id;
  }

  static decltype(::content::mojom::CreateViewParams::session_storage_namespace_id) session_storage_namespace_id(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->session_storage_namespace_id;
  }

  static decltype(::content::mojom::CreateViewParams::opener_frame_route_id) opener_frame_route_id(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->opener_frame_route_id;
  }

  static decltype(::content::mojom::CreateViewParams::swapped_out) swapped_out(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->swapped_out;
  }

  static const decltype(::content::mojom::CreateViewParams::replicated_frame_state)& replicated_frame_state(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->replicated_frame_state;
  }

  static decltype(::content::mojom::CreateViewParams::proxy_routing_id) proxy_routing_id(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->proxy_routing_id;
  }

  static decltype(::content::mojom::CreateViewParams::hidden) hidden(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->hidden;
  }

  static decltype(::content::mojom::CreateViewParams::never_visible) never_visible(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->never_visible;
  }

  static decltype(::content::mojom::CreateViewParams::window_was_created_with_opener) window_was_created_with_opener(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->window_was_created_with_opener;
  }

  static const decltype(::content::mojom::CreateViewParams::initial_size)& initial_size(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->initial_size;
  }

  static decltype(::content::mojom::CreateViewParams::enable_auto_resize) enable_auto_resize(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->enable_auto_resize;
  }

  static const decltype(::content::mojom::CreateViewParams::min_size)& min_size(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->min_size;
  }

  static const decltype(::content::mojom::CreateViewParams::max_size)& max_size(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->max_size;
  }

  static decltype(::content::mojom::CreateViewParams::page_zoom_level) page_zoom_level(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->page_zoom_level;
  }

  static const decltype(::content::mojom::CreateViewParams::image_decode_color_space)& image_decode_color_space(
      const ::content::mojom::CreateViewParamsPtr& input) {
    return input->image_decode_color_space;
  }

  static bool Read(::content::mojom::CreateViewParams::DataView input, ::content::mojom::CreateViewParamsPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::CreateFrameWidgetParams::DataView,
                                         ::content::mojom::CreateFrameWidgetParamsPtr> {
  static bool IsNull(const ::content::mojom::CreateFrameWidgetParamsPtr& input) { return !input; }
  static void SetToNull(::content::mojom::CreateFrameWidgetParamsPtr* output) { output->reset(); }

  static decltype(::content::mojom::CreateFrameWidgetParams::routing_id) routing_id(
      const ::content::mojom::CreateFrameWidgetParamsPtr& input) {
    return input->routing_id;
  }

  static decltype(::content::mojom::CreateFrameWidgetParams::hidden) hidden(
      const ::content::mojom::CreateFrameWidgetParamsPtr& input) {
    return input->hidden;
  }

  static bool Read(::content::mojom::CreateFrameWidgetParams::DataView input, ::content::mojom::CreateFrameWidgetParamsPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::CreateFrameParams::DataView,
                                         ::content::mojom::CreateFrameParamsPtr> {
  static bool IsNull(const ::content::mojom::CreateFrameParamsPtr& input) { return !input; }
  static void SetToNull(::content::mojom::CreateFrameParamsPtr* output) { output->reset(); }

  static decltype(::content::mojom::CreateFrameParams::routing_id) routing_id(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->routing_id;
  }

  static decltype(::content::mojom::CreateFrameParams::proxy_routing_id) proxy_routing_id(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->proxy_routing_id;
  }

  static decltype(::content::mojom::CreateFrameParams::opener_routing_id) opener_routing_id(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->opener_routing_id;
  }

  static decltype(::content::mojom::CreateFrameParams::parent_routing_id) parent_routing_id(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->parent_routing_id;
  }

  static decltype(::content::mojom::CreateFrameParams::previous_sibling_routing_id) previous_sibling_routing_id(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->previous_sibling_routing_id;
  }

  static const decltype(::content::mojom::CreateFrameParams::replication_state)& replication_state(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->replication_state;
  }

  static const decltype(::content::mojom::CreateFrameParams::frame_owner_properties)& frame_owner_properties(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->frame_owner_properties;
  }

  static const decltype(::content::mojom::CreateFrameParams::widget_params)& widget_params(
      const ::content::mojom::CreateFrameParamsPtr& input) {
    return input->widget_params;
  }

  static bool Read(::content::mojom::CreateFrameParams::DataView input, ::content::mojom::CreateFrameParamsPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::UpdateScrollbarThemeParams::DataView,
                                         ::content::mojom::UpdateScrollbarThemeParamsPtr> {
  static bool IsNull(const ::content::mojom::UpdateScrollbarThemeParamsPtr& input) { return !input; }
  static void SetToNull(::content::mojom::UpdateScrollbarThemeParamsPtr* output) { output->reset(); }

  static decltype(::content::mojom::UpdateScrollbarThemeParams::initial_button_delay) initial_button_delay(
      const ::content::mojom::UpdateScrollbarThemeParamsPtr& input) {
    return input->initial_button_delay;
  }

  static decltype(::content::mojom::UpdateScrollbarThemeParams::autoscroll_button_delay) autoscroll_button_delay(
      const ::content::mojom::UpdateScrollbarThemeParamsPtr& input) {
    return input->autoscroll_button_delay;
  }

  static decltype(::content::mojom::UpdateScrollbarThemeParams::jump_on_track_click) jump_on_track_click(
      const ::content::mojom::UpdateScrollbarThemeParamsPtr& input) {
    return input->jump_on_track_click;
  }

  static decltype(::content::mojom::UpdateScrollbarThemeParams::preferred_scroller_style) preferred_scroller_style(
      const ::content::mojom::UpdateScrollbarThemeParamsPtr& input) {
    return input->preferred_scroller_style;
  }

  static decltype(::content::mojom::UpdateScrollbarThemeParams::redraw) redraw(
      const ::content::mojom::UpdateScrollbarThemeParamsPtr& input) {
    return input->redraw;
  }

  static decltype(::content::mojom::UpdateScrollbarThemeParams::button_placement) button_placement(
      const ::content::mojom::UpdateScrollbarThemeParamsPtr& input) {
    return input->button_placement;
  }

  static bool Read(::content::mojom::UpdateScrollbarThemeParams::DataView input, ::content::mojom::UpdateScrollbarThemeParamsPtr* output);
};

}  // namespace mojo

#endif  // CONTENT_COMMON_RENDERER_MOJOM_H_