// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "content/common/service_worker/embedded_worker.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace content {
namespace mojom {

namespace internal {


// static
EmbeddedWorkerStartTiming_Data* EmbeddedWorkerStartTiming_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerStartTiming_Data))) EmbeddedWorkerStartTiming_Data();
}

// static
bool EmbeddedWorkerStartTiming_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerStartTiming_Data* object = static_cast<const EmbeddedWorkerStartTiming_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->blink_initialized_time, "null blink_initialized_time field in EmbeddedWorkerStartTiming",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->blink_initialized_time, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->start_worker_received_time, "null start_worker_received_time field in EmbeddedWorkerStartTiming",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->start_worker_received_time, validation_context))
    return false;

  return true;
}

EmbeddedWorkerStartTiming_Data::EmbeddedWorkerStartTiming_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceClient_StartWorker_Params_Data* EmbeddedWorkerInstanceClient_StartWorker_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceClient_StartWorker_Params_Data))) EmbeddedWorkerInstanceClient_StartWorker_Params_Data();
}

// static
bool EmbeddedWorkerInstanceClient_StartWorker_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceClient_StartWorker_Params_Data* object = static_cast<const EmbeddedWorkerInstanceClient_StartWorker_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->params, "null params field in EmbeddedWorkerInstanceClient_StartWorker_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->params, validation_context))
    return false;

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->dispatcher_request,
          "invalid dispatcher_request field in EmbeddedWorkerInstanceClient_StartWorker_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->dispatcher_request,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->installed_scripts_info, validation_context))
    return false;

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->instance_host,
          "invalid instance_host field in EmbeddedWorkerInstanceClient_StartWorker_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->instance_host,
                                                 validation_context)) {
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceClient_StartWorker_Params_Data::EmbeddedWorkerInstanceClient_StartWorker_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceClient_StopWorker_Params_Data* EmbeddedWorkerInstanceClient_StopWorker_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceClient_StopWorker_Params_Data))) EmbeddedWorkerInstanceClient_StopWorker_Params_Data();
}

// static
bool EmbeddedWorkerInstanceClient_StopWorker_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceClient_StopWorker_Params_Data* object = static_cast<const EmbeddedWorkerInstanceClient_StopWorker_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceClient_StopWorker_Params_Data::EmbeddedWorkerInstanceClient_StopWorker_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data* EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data))) EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data();
}

// static
bool EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data* object = static_cast<const EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data::EmbeddedWorkerInstanceClient_ResumeAfterDownload_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data* EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data))) EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data();
}

// static
bool EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data* object = static_cast<const EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::ConsoleMessageLevel_Data
        ::Validate(object->level, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->message, "null message field in EmbeddedWorkerInstanceClient_AddMessageToConsole_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams message_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->message, validation_context,
                                         &message_validate_params)) {
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data::EmbeddedWorkerInstanceClient_AddMessageToConsole_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data* EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data))) EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data::EmbeddedWorkerInstanceHost_OnReadyForInspection_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data* EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data))) EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data::EmbeddedWorkerInstanceHost_OnScriptLoaded_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data* EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data))) EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data::EmbeddedWorkerInstanceHost_OnScriptLoadFailed_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data* EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data))) EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data::EmbeddedWorkerInstanceHost_OnThreadStarted_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data* EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data))) EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data::EmbeddedWorkerInstanceHost_OnScriptEvaluated_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnStarted_Params_Data* EmbeddedWorkerInstanceHost_OnStarted_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnStarted_Params_Data))) EmbeddedWorkerInstanceHost_OnStarted_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnStarted_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnStarted_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnStarted_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->start_timing, "null start_timing field in EmbeddedWorkerInstanceHost_OnStarted_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->start_timing, validation_context))
    return false;

  return true;
}

EmbeddedWorkerInstanceHost_OnStarted_Params_Data::EmbeddedWorkerInstanceHost_OnStarted_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnReportException_Params_Data* EmbeddedWorkerInstanceHost_OnReportException_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnReportException_Params_Data))) EmbeddedWorkerInstanceHost_OnReportException_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnReportException_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnReportException_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnReportException_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->error_message, "null error_message field in EmbeddedWorkerInstanceHost_OnReportException_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->error_message, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->source_url, "null source_url field in EmbeddedWorkerInstanceHost_OnReportException_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->source_url, validation_context))
    return false;

  return true;
}

EmbeddedWorkerInstanceHost_OnReportException_Params_Data::EmbeddedWorkerInstanceHost_OnReportException_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data* EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data))) EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->message, "null message field in EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->message, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->source_url, "null source_url field in EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->source_url, validation_context))
    return false;

  return true;
}

EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data::EmbeddedWorkerInstanceHost_OnReportConsoleMessage_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
EmbeddedWorkerInstanceHost_OnStopped_Params_Data* EmbeddedWorkerInstanceHost_OnStopped_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(EmbeddedWorkerInstanceHost_OnStopped_Params_Data))) EmbeddedWorkerInstanceHost_OnStopped_Params_Data();
}

// static
bool EmbeddedWorkerInstanceHost_OnStopped_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const EmbeddedWorkerInstanceHost_OnStopped_Params_Data* object = static_cast<const EmbeddedWorkerInstanceHost_OnStopped_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

EmbeddedWorkerInstanceHost_OnStopped_Params_Data::EmbeddedWorkerInstanceHost_OnStopped_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace content

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
