// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_WORKER_EMBEDDED_WORKER_MOJOM_H_
#define CONTENT_COMMON_SERVICE_WORKER_EMBEDDED_WORKER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/service_worker/embedded_worker.mojom-shared.h"
#include "content/common/service_worker/service_worker_event_dispatcher.mojom.h"
#include "content/common/service_worker/service_worker_installed_scripts_manager.mojom.h"
#include "mojo/common/string16.mojom.h"
#include "mojo/common/time.mojom.h"
#include "services/service_manager/public/interfaces/interface_provider.mojom.h"
#include "third_party/WebKit/public/web/console_message.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/common/service_worker/embedded_worker_start_params.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class EmbeddedWorkerInstanceClient;
using EmbeddedWorkerInstanceClientPtr = mojo::InterfacePtr<EmbeddedWorkerInstanceClient>;
using EmbeddedWorkerInstanceClientPtrInfo = mojo::InterfacePtrInfo<EmbeddedWorkerInstanceClient>;
using ThreadSafeEmbeddedWorkerInstanceClientPtr =
    mojo::ThreadSafeInterfacePtr<EmbeddedWorkerInstanceClient>;
using EmbeddedWorkerInstanceClientRequest = mojo::InterfaceRequest<EmbeddedWorkerInstanceClient>;
using EmbeddedWorkerInstanceClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<EmbeddedWorkerInstanceClient>;
using ThreadSafeEmbeddedWorkerInstanceClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<EmbeddedWorkerInstanceClient>;
using EmbeddedWorkerInstanceClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<EmbeddedWorkerInstanceClient>;
using EmbeddedWorkerInstanceClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<EmbeddedWorkerInstanceClient>;

class EmbeddedWorkerInstanceHost;
using EmbeddedWorkerInstanceHostPtr = mojo::InterfacePtr<EmbeddedWorkerInstanceHost>;
using EmbeddedWorkerInstanceHostPtrInfo = mojo::InterfacePtrInfo<EmbeddedWorkerInstanceHost>;
using ThreadSafeEmbeddedWorkerInstanceHostPtr =
    mojo::ThreadSafeInterfacePtr<EmbeddedWorkerInstanceHost>;
using EmbeddedWorkerInstanceHostRequest = mojo::InterfaceRequest<EmbeddedWorkerInstanceHost>;
using EmbeddedWorkerInstanceHostAssociatedPtr =
    mojo::AssociatedInterfacePtr<EmbeddedWorkerInstanceHost>;
using ThreadSafeEmbeddedWorkerInstanceHostAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<EmbeddedWorkerInstanceHost>;
using EmbeddedWorkerInstanceHostAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<EmbeddedWorkerInstanceHost>;
using EmbeddedWorkerInstanceHostAssociatedRequest =
    mojo::AssociatedInterfaceRequest<EmbeddedWorkerInstanceHost>;

using EmbeddedWorkerStartParams = mojo::NativeStruct;
using EmbeddedWorkerStartParamsPtr = mojo::NativeStructPtr;

class EmbeddedWorkerStartTiming;
using EmbeddedWorkerStartTimingPtr = mojo::StructPtr<EmbeddedWorkerStartTiming>;


class EmbeddedWorkerInstanceClientProxy;

template <typename ImplRefTraits>
class EmbeddedWorkerInstanceClientStub;

class EmbeddedWorkerInstanceClientRequestValidator;

class CONTENT_EXPORT EmbeddedWorkerInstanceClient
    : public EmbeddedWorkerInstanceClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = EmbeddedWorkerInstanceClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = EmbeddedWorkerInstanceClientStub<ImplRefTraits>;

  using RequestValidator_ = EmbeddedWorkerInstanceClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kStartWorkerMinVersion = 0,
    kStopWorkerMinVersion = 0,
    kResumeAfterDownloadMinVersion = 0,
    kAddMessageToConsoleMinVersion = 0,
  };
  virtual ~EmbeddedWorkerInstanceClient() {}

  virtual void StartWorker(const ::content::EmbeddedWorkerStartParams& params, ::content::mojom::ServiceWorkerEventDispatcherRequest dispatcher_request, ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr installed_scripts_info, EmbeddedWorkerInstanceHostAssociatedPtrInfo instance_host) = 0;

  virtual void StopWorker() = 0;

  virtual void ResumeAfterDownload() = 0;

  virtual void AddMessageToConsole(::blink::WebConsoleMessage::Level level, const std::string& message) = 0;
};

class EmbeddedWorkerInstanceHostProxy;

template <typename ImplRefTraits>
class EmbeddedWorkerInstanceHostStub;

class EmbeddedWorkerInstanceHostRequestValidator;

class CONTENT_EXPORT EmbeddedWorkerInstanceHost
    : public EmbeddedWorkerInstanceHostInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = EmbeddedWorkerInstanceHostProxy;

  template <typename ImplRefTraits>
  using Stub_ = EmbeddedWorkerInstanceHostStub<ImplRefTraits>;

  using RequestValidator_ = EmbeddedWorkerInstanceHostRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnReadyForInspectionMinVersion = 0,
    kOnScriptLoadedMinVersion = 0,
    kOnScriptLoadFailedMinVersion = 0,
    kOnThreadStartedMinVersion = 0,
    kOnScriptEvaluatedMinVersion = 0,
    kOnStartedMinVersion = 0,
    kOnReportExceptionMinVersion = 0,
    kOnReportConsoleMessageMinVersion = 0,
    kOnStoppedMinVersion = 0,
  };
  virtual ~EmbeddedWorkerInstanceHost() {}

  virtual void OnReadyForInspection() = 0;

  virtual void OnScriptLoaded() = 0;

  virtual void OnScriptLoadFailed() = 0;

  virtual void OnThreadStarted(int32_t thread_id, int32_t provider_id) = 0;

  virtual void OnScriptEvaluated(bool success) = 0;

  virtual void OnStarted(EmbeddedWorkerStartTimingPtr start_timing) = 0;

  virtual void OnReportException(const base::string16& error_message, int32_t line_number, int32_t column_number, const GURL& source_url) = 0;

  virtual void OnReportConsoleMessage(int32_t source_identifier, int32_t message_level, const base::string16& message, int32_t line_number, const GURL& source_url) = 0;

  virtual void OnStopped() = 0;
};

class CONTENT_EXPORT EmbeddedWorkerInstanceClientProxy
    : public EmbeddedWorkerInstanceClient {
 public:
  explicit EmbeddedWorkerInstanceClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void StartWorker(const ::content::EmbeddedWorkerStartParams& params, ::content::mojom::ServiceWorkerEventDispatcherRequest dispatcher_request, ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr installed_scripts_info, EmbeddedWorkerInstanceHostAssociatedPtrInfo instance_host) override;
  void StopWorker() override;
  void ResumeAfterDownload() override;
  void AddMessageToConsole(::blink::WebConsoleMessage::Level level, const std::string& message) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT EmbeddedWorkerInstanceHostProxy
    : public EmbeddedWorkerInstanceHost {
 public:
  explicit EmbeddedWorkerInstanceHostProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnReadyForInspection() override;
  void OnScriptLoaded() override;
  void OnScriptLoadFailed() override;
  void OnThreadStarted(int32_t thread_id, int32_t provider_id) override;
  void OnScriptEvaluated(bool success) override;
  void OnStarted(EmbeddedWorkerStartTimingPtr start_timing) override;
  void OnReportException(const base::string16& error_message, int32_t line_number, int32_t column_number, const GURL& source_url) override;
  void OnReportConsoleMessage(int32_t source_identifier, int32_t message_level, const base::string16& message, int32_t line_number, const GURL& source_url) override;
  void OnStopped() override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT EmbeddedWorkerInstanceClientStubDispatch {
 public:
  static bool Accept(EmbeddedWorkerInstanceClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      EmbeddedWorkerInstanceClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<EmbeddedWorkerInstanceClient>>
class EmbeddedWorkerInstanceClientStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  EmbeddedWorkerInstanceClientStub() {}
  ~EmbeddedWorkerInstanceClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return EmbeddedWorkerInstanceClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return EmbeddedWorkerInstanceClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT EmbeddedWorkerInstanceHostStubDispatch {
 public:
  static bool Accept(EmbeddedWorkerInstanceHost* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      EmbeddedWorkerInstanceHost* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<EmbeddedWorkerInstanceHost>>
class EmbeddedWorkerInstanceHostStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  EmbeddedWorkerInstanceHostStub() {}
  ~EmbeddedWorkerInstanceHostStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return EmbeddedWorkerInstanceHostStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return EmbeddedWorkerInstanceHostStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT EmbeddedWorkerInstanceClientRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT EmbeddedWorkerInstanceHostRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};







class CONTENT_EXPORT EmbeddedWorkerStartTiming {
 public:
  using DataView = EmbeddedWorkerStartTimingDataView;
  using Data_ = internal::EmbeddedWorkerStartTiming_Data;

  template <typename... Args>
  static EmbeddedWorkerStartTimingPtr New(Args&&... args) {
    return EmbeddedWorkerStartTimingPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static EmbeddedWorkerStartTimingPtr From(const U& u) {
    return mojo::TypeConverter<EmbeddedWorkerStartTimingPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, EmbeddedWorkerStartTiming>::Convert(*this);
  }


  EmbeddedWorkerStartTiming();

  EmbeddedWorkerStartTiming(
      base::TimeTicks blink_initialized_time,
      base::TimeTicks start_worker_received_time);

  ~EmbeddedWorkerStartTiming();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = EmbeddedWorkerStartTimingPtr>
  EmbeddedWorkerStartTimingPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, EmbeddedWorkerStartTiming>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        EmbeddedWorkerStartTiming::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        EmbeddedWorkerStartTiming::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::TimeTicks blink_initialized_time;
  base::TimeTicks start_worker_received_time;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
EmbeddedWorkerStartTimingPtr EmbeddedWorkerStartTiming::Clone() const {
  return New(
      mojo::Clone(blink_initialized_time),
      mojo::Clone(start_worker_received_time)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, EmbeddedWorkerStartTiming>::value>::type*>
bool EmbeddedWorkerStartTiming::Equals(const T& other) const {
  if (!mojo::Equals(this->blink_initialized_time, other.blink_initialized_time))
    return false;
  if (!mojo::Equals(this->start_worker_received_time, other.start_worker_received_time))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::EmbeddedWorkerStartTiming::DataView,
                                         ::content::mojom::EmbeddedWorkerStartTimingPtr> {
  static bool IsNull(const ::content::mojom::EmbeddedWorkerStartTimingPtr& input) { return !input; }
  static void SetToNull(::content::mojom::EmbeddedWorkerStartTimingPtr* output) { output->reset(); }

  static const decltype(::content::mojom::EmbeddedWorkerStartTiming::blink_initialized_time)& blink_initialized_time(
      const ::content::mojom::EmbeddedWorkerStartTimingPtr& input) {
    return input->blink_initialized_time;
  }

  static const decltype(::content::mojom::EmbeddedWorkerStartTiming::start_worker_received_time)& start_worker_received_time(
      const ::content::mojom::EmbeddedWorkerStartTimingPtr& input) {
    return input->start_worker_received_time;
  }

  static bool Read(::content::mojom::EmbeddedWorkerStartTiming::DataView input, ::content::mojom::EmbeddedWorkerStartTimingPtr* output);
};

}  // namespace mojo

#endif  // CONTENT_COMMON_SERVICE_WORKER_EMBEDDED_WORKER_MOJOM_H_