// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_INSTALLED_SCRIPTS_MANAGER_MOJOM_SHARED_H_
#define CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_INSTALLED_SCRIPTS_MANAGER_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "content/common/service_worker/service_worker_installed_scripts_manager.mojom-shared-internal.h"
#include "url/mojo/url.mojom-shared.h"
#include "content/common/mojo_bindings_shared_export.h"


namespace content {
namespace mojom {
class ServiceWorkerInstalledScriptsInfoDataView;

class ServiceWorkerScriptInfoDataView;



}  // namespace mojom
}  // namespace content

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::content::mojom::ServiceWorkerInstalledScriptsInfoDataView> {
  using Data = ::content::mojom::internal::ServiceWorkerInstalledScriptsInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::content::mojom::ServiceWorkerScriptInfoDataView> {
  using Data = ::content::mojom::internal::ServiceWorkerScriptInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace content {
namespace mojom {
// Interface base classes. They are used for type safety check.
class ServiceWorkerInstalledScriptsManagerInterfaceBase {};

using ServiceWorkerInstalledScriptsManagerPtrDataView =
    mojo::InterfacePtrDataView<ServiceWorkerInstalledScriptsManagerInterfaceBase>;
using ServiceWorkerInstalledScriptsManagerRequestDataView =
    mojo::InterfaceRequestDataView<ServiceWorkerInstalledScriptsManagerInterfaceBase>;
using ServiceWorkerInstalledScriptsManagerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ServiceWorkerInstalledScriptsManagerInterfaceBase>;
using ServiceWorkerInstalledScriptsManagerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ServiceWorkerInstalledScriptsManagerInterfaceBase>;
class ServiceWorkerInstalledScriptsInfoDataView {
 public:
  ServiceWorkerInstalledScriptsInfoDataView() {}

  ServiceWorkerInstalledScriptsInfoDataView(
      internal::ServiceWorkerInstalledScriptsInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeManagerRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::content::mojom::ServiceWorkerInstalledScriptsManagerRequestDataView>(
            &data_->manager_request, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetInstalledUrlsDataView(
      mojo::ArrayDataView<::url::mojom::UrlDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstalledUrls(UserType* output) {
    auto* pointer = data_->installed_urls.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
        pointer, output, context_);
  }
 private:
  internal::ServiceWorkerInstalledScriptsInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ServiceWorkerScriptInfoDataView {
 public:
  ServiceWorkerScriptInfoDataView() {}

  ServiceWorkerScriptInfoDataView(
      internal::ServiceWorkerScriptInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetScriptUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadScriptUrl(UserType* output) {
    auto* pointer = data_->script_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetEncodingDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEncoding(UserType* output) {
    auto* pointer = data_->encoding.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetHeadersDataView(
      mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadHeaders(UserType* output) {
    auto* pointer = data_->headers.Get();
    return mojo::internal::Deserialize<mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>>(
        pointer, output, context_);
  }
  mojo::ScopedDataPipeConsumerHandle TakeBody() {
    mojo::ScopedDataPipeConsumerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeConsumerHandle>(
            &data_->body, &result, context_);
    DCHECK(ret);
    return result;
  }
  mojo::ScopedDataPipeConsumerHandle TakeMetaData() {
    mojo::ScopedDataPipeConsumerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeConsumerHandle>(
            &data_->meta_data, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::ServiceWorkerScriptInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ServiceWorkerInstalledScriptsManager_TransferInstalledScript_ParamsDataView {
 public:
  ServiceWorkerInstalledScriptsManager_TransferInstalledScript_ParamsDataView() {}

  ServiceWorkerInstalledScriptsManager_TransferInstalledScript_ParamsDataView(
      internal::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetScriptInfoDataView(
      ServiceWorkerScriptInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadScriptInfo(UserType* output) {
    auto* pointer = data_->script_info.Get();
    return mojo::internal::Deserialize<::content::mojom::ServiceWorkerScriptInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace content

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::content::mojom::ServiceWorkerInstalledScriptsInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::content::mojom::ServiceWorkerInstalledScriptsInfoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::content::mojom::internal::ServiceWorkerInstalledScriptsInfo_Data);
    decltype(CallWithContext(Traits::manager_request, input, custom_context)) in_manager_request = CallWithContext(Traits::manager_request, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::content::mojom::ServiceWorkerInstalledScriptsManagerRequestDataView>(
        in_manager_request, context);
    decltype(CallWithContext(Traits::installed_urls, input, custom_context)) in_installed_urls = CallWithContext(Traits::installed_urls, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
        in_installed_urls, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::content::mojom::internal::ServiceWorkerInstalledScriptsInfo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::content::mojom::internal::ServiceWorkerInstalledScriptsInfo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::manager_request, input, custom_context))>::type
        in_manager_request;
    mojo::internal::Serialize<::content::mojom::ServiceWorkerInstalledScriptsManagerRequestDataView>(
        in_manager_request, &result->manager_request, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->manager_request),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid manager_request in ServiceWorkerInstalledScriptsInfo struct");
    decltype(CallWithContext(Traits::installed_urls, input, custom_context)) in_installed_urls = CallWithContext(Traits::installed_urls, input, custom_context);
    typename decltype(result->installed_urls)::BaseType* installed_urls_ptr;
    const mojo::internal::ContainerValidateParams installed_urls_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
        in_installed_urls, buffer, &installed_urls_ptr, &installed_urls_validate_params,
        context);
    result->installed_urls.Set(installed_urls_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->installed_urls.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null installed_urls in ServiceWorkerInstalledScriptsInfo struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::content::mojom::internal::ServiceWorkerInstalledScriptsInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::content::mojom::ServiceWorkerInstalledScriptsInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::content::mojom::ServiceWorkerScriptInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::content::mojom::ServiceWorkerScriptInfoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::content::mojom::internal::ServiceWorkerScriptInfo_Data);
    decltype(CallWithContext(Traits::script_url, input, custom_context)) in_script_url = CallWithContext(Traits::script_url, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_script_url, context);
    decltype(CallWithContext(Traits::encoding, input, custom_context)) in_encoding = CallWithContext(Traits::encoding, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_encoding, context);
    decltype(CallWithContext(Traits::headers, input, custom_context)) in_headers = CallWithContext(Traits::headers, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>>(
        in_headers, context);
    decltype(CallWithContext(Traits::body, input, custom_context)) in_body = CallWithContext(Traits::body, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ScopedDataPipeConsumerHandle>(
        in_body, context);
    decltype(CallWithContext(Traits::meta_data, input, custom_context)) in_meta_data = CallWithContext(Traits::meta_data, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ScopedDataPipeConsumerHandle>(
        in_meta_data, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::content::mojom::internal::ServiceWorkerScriptInfo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::content::mojom::internal::ServiceWorkerScriptInfo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::script_url, input, custom_context)) in_script_url = CallWithContext(Traits::script_url, input, custom_context);
    typename decltype(result->script_url)::BaseType* script_url_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_script_url, buffer, &script_url_ptr, context);
    result->script_url.Set(script_url_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->script_url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null script_url in ServiceWorkerScriptInfo struct");
    decltype(CallWithContext(Traits::encoding, input, custom_context)) in_encoding = CallWithContext(Traits::encoding, input, custom_context);
    typename decltype(result->encoding)::BaseType* encoding_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_encoding, buffer, &encoding_ptr, context);
    result->encoding.Set(encoding_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->encoding.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null encoding in ServiceWorkerScriptInfo struct");
    decltype(CallWithContext(Traits::headers, input, custom_context)) in_headers = CallWithContext(Traits::headers, input, custom_context);
    typename decltype(result->headers)::BaseType* headers_ptr;
    const mojo::internal::ContainerValidateParams headers_validate_params(
        new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
    mojo::internal::Serialize<mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>>(
        in_headers, buffer, &headers_ptr, &headers_validate_params,
        context);
    result->headers.Set(headers_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->headers.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null headers in ServiceWorkerScriptInfo struct");
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::body, input, custom_context))>::type
        in_body;
    mojo::internal::Serialize<mojo::ScopedDataPipeConsumerHandle>(
        in_body, &result->body, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->body),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid body in ServiceWorkerScriptInfo struct");
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::meta_data, input, custom_context))>::type
        in_meta_data;
    mojo::internal::Serialize<mojo::ScopedDataPipeConsumerHandle>(
        in_meta_data, &result->meta_data, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->meta_data),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid meta_data in ServiceWorkerScriptInfo struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::content::mojom::internal::ServiceWorkerScriptInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::content::mojom::ServiceWorkerScriptInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace content {
namespace mojom {

inline void ServiceWorkerInstalledScriptsInfoDataView::GetInstalledUrlsDataView(
    mojo::ArrayDataView<::url::mojom::UrlDataView>* output) {
  auto pointer = data_->installed_urls.Get();
  *output = mojo::ArrayDataView<::url::mojom::UrlDataView>(pointer, context_);
}


inline void ServiceWorkerScriptInfoDataView::GetScriptUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->script_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void ServiceWorkerScriptInfoDataView::GetEncodingDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->encoding.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void ServiceWorkerScriptInfoDataView::GetHeadersDataView(
    mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>* output) {
  auto pointer = data_->headers.Get();
  *output = mojo::MapDataView<mojo::StringDataView, mojo::StringDataView>(pointer, context_);
}


inline void ServiceWorkerInstalledScriptsManager_TransferInstalledScript_ParamsDataView::GetScriptInfoDataView(
    ServiceWorkerScriptInfoDataView* output) {
  auto pointer = data_->script_info.Get();
  *output = ServiceWorkerScriptInfoDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace content

#endif  // CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_INSTALLED_SCRIPTS_MANAGER_MOJOM_SHARED_H_
