// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "content/common/service_worker/service_worker_installed_scripts_manager.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "url/mojo/url_gurl_struct_traits.h"
namespace content {
namespace mojom {
ServiceWorkerInstalledScriptsInfo::ServiceWorkerInstalledScriptsInfo()
    : manager_request(),
      installed_urls() {}

ServiceWorkerInstalledScriptsInfo::ServiceWorkerInstalledScriptsInfo(
    ServiceWorkerInstalledScriptsManagerRequest manager_request_in,
    const std::vector<GURL>& installed_urls_in)
    : manager_request(std::move(manager_request_in)),
      installed_urls(std::move(installed_urls_in)) {}

ServiceWorkerInstalledScriptsInfo::~ServiceWorkerInstalledScriptsInfo() = default;

bool ServiceWorkerInstalledScriptsInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ServiceWorkerScriptInfo::ServiceWorkerScriptInfo()
    : script_url(),
      encoding(),
      headers(),
      body(),
      meta_data() {}

ServiceWorkerScriptInfo::ServiceWorkerScriptInfo(
    const GURL& script_url_in,
    const std::string& encoding_in,
    const std::unordered_map<std::string, std::string>& headers_in,
    mojo::ScopedDataPipeConsumerHandle body_in,
    mojo::ScopedDataPipeConsumerHandle meta_data_in)
    : script_url(std::move(script_url_in)),
      encoding(std::move(encoding_in)),
      headers(std::move(headers_in)),
      body(std::move(body_in)),
      meta_data(std::move(meta_data_in)) {}

ServiceWorkerScriptInfo::~ServiceWorkerScriptInfo() = default;

bool ServiceWorkerScriptInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char ServiceWorkerInstalledScriptsManager::Name_[] = "content::mojom::ServiceWorkerInstalledScriptsManager";

ServiceWorkerInstalledScriptsManagerProxy::ServiceWorkerInstalledScriptsManagerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void ServiceWorkerInstalledScriptsManagerProxy::TransferInstalledScript(
    ServiceWorkerScriptInfoPtr in_script_info) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::content::mojom::internal::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data);
  size += mojo::internal::PrepareToSerialize<::content::mojom::ServiceWorkerScriptInfoDataView>(
      in_script_info, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kServiceWorkerInstalledScriptsManager_TransferInstalledScript_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::content::mojom::internal::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->script_info)::BaseType* script_info_ptr;
  mojo::internal::Serialize<::content::mojom::ServiceWorkerScriptInfoDataView>(
      in_script_info, buffer, &script_info_ptr, &serialization_context);
  params->script_info.Set(script_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->script_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null script_info in ServiceWorkerInstalledScriptsManager.TransferInstalledScript request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool ServiceWorkerInstalledScriptsManagerStubDispatch::Accept(
    ServiceWorkerInstalledScriptsManager* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kServiceWorkerInstalledScriptsManager_TransferInstalledScript_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data* params =
          reinterpret_cast<internal::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ServiceWorkerScriptInfoPtr p_script_info{};
      ServiceWorkerInstalledScriptsManager_TransferInstalledScript_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadScriptInfo(&p_script_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ServiceWorkerInstalledScriptsManager::TransferInstalledScript deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->TransferInstalledScript(
std::move(p_script_info));
      return true;
    }
  }
  return false;
}

// static
bool ServiceWorkerInstalledScriptsManagerStubDispatch::AcceptWithResponder(
    ServiceWorkerInstalledScriptsManager* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kServiceWorkerInstalledScriptsManager_TransferInstalledScript_Name: {
      break;
    }
  }
  return false;
}

bool ServiceWorkerInstalledScriptsManagerRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ServiceWorkerInstalledScriptsManager RequestValidator");

  switch (message->header()->name) {
    case internal::kServiceWorkerInstalledScriptsManager_TransferInstalledScript_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ServiceWorkerInstalledScriptsManager_TransferInstalledScript_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace content

namespace mojo {


// static
bool StructTraits<::content::mojom::ServiceWorkerInstalledScriptsInfo::DataView, ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr>::Read(
    ::content::mojom::ServiceWorkerInstalledScriptsInfo::DataView input,
    ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr* output) {
  bool success = true;
  ::content::mojom::ServiceWorkerInstalledScriptsInfoPtr result(::content::mojom::ServiceWorkerInstalledScriptsInfo::New());
  
      result->manager_request =
          input.TakeManagerRequest<decltype(result->manager_request)>();
      if (!input.ReadInstalledUrls(&result->installed_urls))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::content::mojom::ServiceWorkerScriptInfo::DataView, ::content::mojom::ServiceWorkerScriptInfoPtr>::Read(
    ::content::mojom::ServiceWorkerScriptInfo::DataView input,
    ::content::mojom::ServiceWorkerScriptInfoPtr* output) {
  bool success = true;
  ::content::mojom::ServiceWorkerScriptInfoPtr result(::content::mojom::ServiceWorkerScriptInfo::New());
  
      if (!input.ReadScriptUrl(&result->script_url))
        success = false;
      if (!input.ReadEncoding(&result->encoding))
        success = false;
      if (!input.ReadHeaders(&result->headers))
        success = false;
      result->body = input.TakeBody();
      result->meta_data = input.TakeMetaData();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif