// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_MOJOM_H_
#define CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/common/service_worker/service_worker_provider.mojom-shared.h"
#include "content/common/service_worker/service_worker_provider_interfaces.mojom.h"
#include "content/common/service_worker/service_worker_types.mojom.h"
#include <string>
#include <vector>
#include "content/common/service_worker/service_worker_provider_host_info.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class ServiceWorkerProviderHostInfo;
using ServiceWorkerProviderHostInfoPtr = mojo::StructPtr<ServiceWorkerProviderHostInfo>;






class CONTENT_EXPORT ServiceWorkerProviderHostInfo {
 public:
  using DataView = ServiceWorkerProviderHostInfoDataView;
  using Data_ = internal::ServiceWorkerProviderHostInfo_Data;

  template <typename... Args>
  static ServiceWorkerProviderHostInfoPtr New(Args&&... args) {
    return ServiceWorkerProviderHostInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ServiceWorkerProviderHostInfoPtr From(const U& u) {
    return mojo::TypeConverter<ServiceWorkerProviderHostInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ServiceWorkerProviderHostInfo>::Convert(*this);
  }


  ServiceWorkerProviderHostInfo();

  ServiceWorkerProviderHostInfo(
      int32_t provider_id,
      int32_t route_id,
      ::content::ServiceWorkerProviderType type,
      bool is_parent_frame_secure,
      ::content::mojom::ServiceWorkerProviderHostAssociatedRequest host_request,
      ::content::mojom::ServiceWorkerProviderAssociatedPtrInfo client_ptr_info);

  ~ServiceWorkerProviderHostInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ServiceWorkerProviderHostInfoPtr>
  ServiceWorkerProviderHostInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ServiceWorkerProviderHostInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ServiceWorkerProviderHostInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ServiceWorkerProviderHostInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  int32_t provider_id;
  int32_t route_id;
  ::content::ServiceWorkerProviderType type;
  bool is_parent_frame_secure;
  ::content::mojom::ServiceWorkerProviderHostAssociatedRequest host_request;
  ::content::mojom::ServiceWorkerProviderAssociatedPtrInfo client_ptr_info;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(ServiceWorkerProviderHostInfo);
};


template <typename StructPtrType>
ServiceWorkerProviderHostInfoPtr ServiceWorkerProviderHostInfo::Clone() const {
  return New(
      mojo::Clone(provider_id),
      mojo::Clone(route_id),
      mojo::Clone(type),
      mojo::Clone(is_parent_frame_secure),
      mojo::Clone(host_request),
      mojo::Clone(client_ptr_info)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ServiceWorkerProviderHostInfo>::value>::type*>
bool ServiceWorkerProviderHostInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->provider_id, other.provider_id))
    return false;
  if (!mojo::Equals(this->route_id, other.route_id))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->is_parent_frame_secure, other.is_parent_frame_secure))
    return false;
  if (!mojo::Equals(this->host_request, other.host_request))
    return false;
  if (!mojo::Equals(this->client_ptr_info, other.client_ptr_info))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::ServiceWorkerProviderHostInfo::DataView,
                                         ::content::mojom::ServiceWorkerProviderHostInfoPtr> {
  static bool IsNull(const ::content::mojom::ServiceWorkerProviderHostInfoPtr& input) { return !input; }
  static void SetToNull(::content::mojom::ServiceWorkerProviderHostInfoPtr* output) { output->reset(); }

  static decltype(::content::mojom::ServiceWorkerProviderHostInfo::provider_id) provider_id(
      const ::content::mojom::ServiceWorkerProviderHostInfoPtr& input) {
    return input->provider_id;
  }

  static decltype(::content::mojom::ServiceWorkerProviderHostInfo::route_id) route_id(
      const ::content::mojom::ServiceWorkerProviderHostInfoPtr& input) {
    return input->route_id;
  }

  static decltype(::content::mojom::ServiceWorkerProviderHostInfo::type) type(
      const ::content::mojom::ServiceWorkerProviderHostInfoPtr& input) {
    return input->type;
  }

  static decltype(::content::mojom::ServiceWorkerProviderHostInfo::is_parent_frame_secure) is_parent_frame_secure(
      const ::content::mojom::ServiceWorkerProviderHostInfoPtr& input) {
    return input->is_parent_frame_secure;
  }

  static  decltype(::content::mojom::ServiceWorkerProviderHostInfo::host_request)& host_request(
       ::content::mojom::ServiceWorkerProviderHostInfoPtr& input) {
    return input->host_request;
  }

  static  decltype(::content::mojom::ServiceWorkerProviderHostInfo::client_ptr_info)& client_ptr_info(
       ::content::mojom::ServiceWorkerProviderHostInfoPtr& input) {
    return input->client_ptr_info;
  }

  static bool Read(::content::mojom::ServiceWorkerProviderHostInfo::DataView input, ::content::mojom::ServiceWorkerProviderHostInfoPtr* output);
};

}  // namespace mojo

#endif  // CONTENT_COMMON_SERVICE_WORKER_SERVICE_WORKER_PROVIDER_MOJOM_H_