// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_PUBLIC_COMMON_NETWORK_SERVICE_MOJOM_H_
#define CONTENT_PUBLIC_COMMON_NETWORK_SERVICE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/public/common/network_service.mojom-shared.h"
#include "mojo/common/file_path.mojom.h"
#include "content/public/common/url_loader.mojom.h"
#include "content/public/common/url_loader_factory.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace content {
namespace mojom {
class NetworkContext;
using NetworkContextPtr = mojo::InterfacePtr<NetworkContext>;
using NetworkContextPtrInfo = mojo::InterfacePtrInfo<NetworkContext>;
using ThreadSafeNetworkContextPtr =
    mojo::ThreadSafeInterfacePtr<NetworkContext>;
using NetworkContextRequest = mojo::InterfaceRequest<NetworkContext>;
using NetworkContextAssociatedPtr =
    mojo::AssociatedInterfacePtr<NetworkContext>;
using ThreadSafeNetworkContextAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NetworkContext>;
using NetworkContextAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NetworkContext>;
using NetworkContextAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NetworkContext>;

class NetworkService;
using NetworkServicePtr = mojo::InterfacePtr<NetworkService>;
using NetworkServicePtrInfo = mojo::InterfacePtrInfo<NetworkService>;
using ThreadSafeNetworkServicePtr =
    mojo::ThreadSafeInterfacePtr<NetworkService>;
using NetworkServiceRequest = mojo::InterfaceRequest<NetworkService>;
using NetworkServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<NetworkService>;
using ThreadSafeNetworkServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NetworkService>;
using NetworkServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NetworkService>;
using NetworkServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NetworkService>;

class NetworkContextParams;
using NetworkContextParamsPtr = mojo::StructPtr<NetworkContextParams>;


class NetworkContextProxy;

template <typename ImplRefTraits>
class NetworkContextStub;

class NetworkContextRequestValidator;

class CONTENT_EXPORT NetworkContext
    : public NetworkContextInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NetworkContextProxy;

  template <typename ImplRefTraits>
  using Stub_ = NetworkContextStub<ImplRefTraits>;

  using RequestValidator_ = NetworkContextRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateURLLoaderFactoryMinVersion = 0,
    kHandleViewCacheRequestMinVersion = 0,
  };
  virtual ~NetworkContext() {}

  virtual void CreateURLLoaderFactory(::content::mojom::URLLoaderFactoryRequest url_loader_factory, uint32_t process_id) = 0;

  virtual void HandleViewCacheRequest(const GURL& url, ::content::mojom::URLLoaderClientPtr client) = 0;
};

class NetworkServiceProxy;

template <typename ImplRefTraits>
class NetworkServiceStub;

class NetworkServiceRequestValidator;

class CONTENT_EXPORT NetworkService
    : public NetworkServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NetworkServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = NetworkServiceStub<ImplRefTraits>;

  using RequestValidator_ = NetworkServiceRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateNetworkContextMinVersion = 0,
  };
  virtual ~NetworkService() {}

  virtual void CreateNetworkContext(NetworkContextRequest context, NetworkContextParamsPtr params) = 0;
};

class CONTENT_EXPORT NetworkContextProxy
    : public NetworkContext {
 public:
  explicit NetworkContextProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateURLLoaderFactory(::content::mojom::URLLoaderFactoryRequest url_loader_factory, uint32_t process_id) override;
  void HandleViewCacheRequest(const GURL& url, ::content::mojom::URLLoaderClientPtr client) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class CONTENT_EXPORT NetworkServiceProxy
    : public NetworkService {
 public:
  explicit NetworkServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateNetworkContext(NetworkContextRequest context, NetworkContextParamsPtr params) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT NetworkContextStubDispatch {
 public:
  static bool Accept(NetworkContext* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NetworkContext* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NetworkContext>>
class NetworkContextStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NetworkContextStub() {}
  ~NetworkContextStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkContextStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkContextStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT NetworkServiceStubDispatch {
 public:
  static bool Accept(NetworkService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NetworkService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NetworkService>>
class NetworkServiceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NetworkServiceStub() {}
  ~NetworkServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NetworkServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT NetworkContextRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT NetworkServiceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





class CONTENT_EXPORT NetworkContextParams {
 public:
  using DataView = NetworkContextParamsDataView;
  using Data_ = internal::NetworkContextParams_Data;

  template <typename... Args>
  static NetworkContextParamsPtr New(Args&&... args) {
    return NetworkContextParamsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static NetworkContextParamsPtr From(const U& u) {
    return mojo::TypeConverter<NetworkContextParamsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, NetworkContextParams>::Convert(*this);
  }


  NetworkContextParams();

  NetworkContextParams(
      const base::Optional<base::FilePath>& cookie_path,
      const base::Optional<base::FilePath>& cache_dir);

  ~NetworkContextParams();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = NetworkContextParamsPtr>
  NetworkContextParamsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, NetworkContextParams>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        NetworkContextParams::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        NetworkContextParams::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::Optional<base::FilePath> cookie_path;
  base::Optional<base::FilePath> cache_dir;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
NetworkContextParamsPtr NetworkContextParams::Clone() const {
  return New(
      mojo::Clone(cookie_path),
      mojo::Clone(cache_dir)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, NetworkContextParams>::value>::type*>
bool NetworkContextParams::Equals(const T& other) const {
  if (!mojo::Equals(this->cookie_path, other.cookie_path))
    return false;
  if (!mojo::Equals(this->cache_dir, other.cache_dir))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace content

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::content::mojom::NetworkContextParams::DataView,
                                         ::content::mojom::NetworkContextParamsPtr> {
  static bool IsNull(const ::content::mojom::NetworkContextParamsPtr& input) { return !input; }
  static void SetToNull(::content::mojom::NetworkContextParamsPtr* output) { output->reset(); }

  static const decltype(::content::mojom::NetworkContextParams::cookie_path)& cookie_path(
      const ::content::mojom::NetworkContextParamsPtr& input) {
    return input->cookie_path;
  }

  static const decltype(::content::mojom::NetworkContextParams::cache_dir)& cache_dir(
      const ::content::mojom::NetworkContextParamsPtr& input) {
    return input->cache_dir;
  }

  static bool Read(::content::mojom::NetworkContextParams::DataView input, ::content::mojom::NetworkContextParamsPtr* output);
};

}  // namespace mojo

#endif  // CONTENT_PUBLIC_COMMON_NETWORK_SERVICE_MOJOM_H_