// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef CONTENT_PUBLIC_COMMON_URL_LOADER_FACTORY_MOJOM_H_
#define CONTENT_PUBLIC_COMMON_URL_LOADER_FACTORY_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "content/public/common/url_loader_factory.mojom-shared.h"
#include "content/public/common/mutable_network_traffic_annotation_tag.mojom.h"
#include "content/public/common/url_loader.mojom.h"
#include <string>
#include <vector>
#include "content/public/common/resource_response.h"
#include "content/common/content_export.h"


namespace content {
namespace mojom {
constexpr uint32_t kURLLoadOptionNone = 0U;
constexpr uint32_t kURLLoadOptionSendSSLInfo = 1U;
constexpr uint32_t kURLLoadOptionSniffMimeType = 2U;
class URLLoaderFactory;
using URLLoaderFactoryPtr = mojo::InterfacePtr<URLLoaderFactory>;
using URLLoaderFactoryPtrInfo = mojo::InterfacePtrInfo<URLLoaderFactory>;
using ThreadSafeURLLoaderFactoryPtr =
    mojo::ThreadSafeInterfacePtr<URLLoaderFactory>;
using URLLoaderFactoryRequest = mojo::InterfaceRequest<URLLoaderFactory>;
using URLLoaderFactoryAssociatedPtr =
    mojo::AssociatedInterfacePtr<URLLoaderFactory>;
using ThreadSafeURLLoaderFactoryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<URLLoaderFactory>;
using URLLoaderFactoryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<URLLoaderFactory>;
using URLLoaderFactoryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<URLLoaderFactory>;

using URLSyncLoadResult = mojo::NativeStruct;
using URLSyncLoadResultPtr = mojo::NativeStructPtr;


class URLLoaderFactoryProxy;

template <typename ImplRefTraits>
class URLLoaderFactoryStub;

class URLLoaderFactoryRequestValidator;
class URLLoaderFactoryResponseValidator;

class CONTENT_EXPORT URLLoaderFactory
    : public URLLoaderFactoryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = URLLoaderFactoryProxy;

  template <typename ImplRefTraits>
  using Stub_ = URLLoaderFactoryStub<ImplRefTraits>;

  using RequestValidator_ = URLLoaderFactoryRequestValidator;
  using ResponseValidator_ = URLLoaderFactoryResponseValidator;
  enum MethodMinVersions : uint32_t {
    kCreateLoaderAndStartMinVersion = 0,
    kSyncLoadMinVersion = 0,
  };
  virtual ~URLLoaderFactory() {}

  virtual void CreateLoaderAndStart(::content::mojom::URLLoaderAssociatedRequest loader, int32_t routing_id, int32_t request_id, uint32_t options, const content::ResourceRequest& request, ::content::mojom::URLLoaderClientPtr client, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation) = 0;

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool SyncLoad(int32_t routing_id, int32_t request_id, const content::ResourceRequest& request, content::SyncLoadResult* result);

  using SyncLoadCallback = base::OnceCallback<void(const content::SyncLoadResult&)>;
  virtual void SyncLoad(int32_t routing_id, int32_t request_id, const content::ResourceRequest& request, SyncLoadCallback callback) = 0;
};

class CONTENT_EXPORT URLLoaderFactoryProxy
    : public URLLoaderFactory {
 public:
  explicit URLLoaderFactoryProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateLoaderAndStart(::content::mojom::URLLoaderAssociatedRequest loader, int32_t routing_id, int32_t request_id, uint32_t options, const content::ResourceRequest& request, ::content::mojom::URLLoaderClientPtr client, const net::MutableNetworkTrafficAnnotationTag& traffic_annotation) override;
  bool SyncLoad(int32_t routing_id, int32_t request_id, const content::ResourceRequest& request, content::SyncLoadResult* result) override;
  void SyncLoad(int32_t routing_id, int32_t request_id, const content::ResourceRequest& request, SyncLoadCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT URLLoaderFactoryStubDispatch {
 public:
  static bool Accept(URLLoaderFactory* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      URLLoaderFactory* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<URLLoaderFactory>>
class URLLoaderFactoryStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  URLLoaderFactoryStub() {}
  ~URLLoaderFactoryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return URLLoaderFactoryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return URLLoaderFactoryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT URLLoaderFactoryRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT URLLoaderFactoryResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};







}  // namespace mojom
}  // namespace content

namespace mojo {

}  // namespace mojo

#endif  // CONTENT_PUBLIC_COMMON_URL_LOADER_FACTORY_MOJOM_H_