// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "device/serial/serial.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
namespace device {
namespace serial {
DeviceInfo::DeviceInfo()
    : path(),
      vendor_id(),
      has_vendor_id(false),
      product_id(),
      has_product_id(false),
      display_name() {}

DeviceInfo::DeviceInfo(
    const std::string& path_in,
    uint16_t vendor_id_in,
    bool has_vendor_id_in,
    uint16_t product_id_in,
    bool has_product_id_in,
    const base::Optional<std::string>& display_name_in)
    : path(std::move(path_in)),
      vendor_id(std::move(vendor_id_in)),
      has_vendor_id(std::move(has_vendor_id_in)),
      product_id(std::move(product_id_in)),
      has_product_id(std::move(has_product_id_in)),
      display_name(std::move(display_name_in)) {}

DeviceInfo::~DeviceInfo() = default;

bool DeviceInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ConnectionOptions::ConnectionOptions()
    : bitrate(0U),
      data_bits(DataBits::NONE),
      parity_bit(ParityBit::NONE),
      stop_bits(StopBits::NONE),
      cts_flow_control(),
      has_cts_flow_control(false) {}

ConnectionOptions::ConnectionOptions(
    uint32_t bitrate_in,
    DataBits data_bits_in,
    ParityBit parity_bit_in,
    StopBits stop_bits_in,
    bool cts_flow_control_in,
    bool has_cts_flow_control_in)
    : bitrate(std::move(bitrate_in)),
      data_bits(std::move(data_bits_in)),
      parity_bit(std::move(parity_bit_in)),
      stop_bits(std::move(stop_bits_in)),
      cts_flow_control(std::move(cts_flow_control_in)),
      has_cts_flow_control(std::move(has_cts_flow_control_in)) {}

ConnectionOptions::~ConnectionOptions() = default;
size_t ConnectionOptions::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->bitrate);
  seed = mojo::internal::Hash(seed, this->data_bits);
  seed = mojo::internal::Hash(seed, this->parity_bit);
  seed = mojo::internal::Hash(seed, this->stop_bits);
  seed = mojo::internal::Hash(seed, this->cts_flow_control);
  seed = mojo::internal::Hash(seed, this->has_cts_flow_control);
  return seed;
}

bool ConnectionOptions::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
ConnectionInfo::ConnectionInfo()
    : bitrate(0U),
      data_bits(DataBits::NONE),
      parity_bit(ParityBit::NONE),
      stop_bits(StopBits::NONE),
      cts_flow_control() {}

ConnectionInfo::ConnectionInfo(
    uint32_t bitrate_in,
    DataBits data_bits_in,
    ParityBit parity_bit_in,
    StopBits stop_bits_in,
    bool cts_flow_control_in)
    : bitrate(std::move(bitrate_in)),
      data_bits(std::move(data_bits_in)),
      parity_bit(std::move(parity_bit_in)),
      stop_bits(std::move(stop_bits_in)),
      cts_flow_control(std::move(cts_flow_control_in)) {}

ConnectionInfo::~ConnectionInfo() = default;
size_t ConnectionInfo::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->bitrate);
  seed = mojo::internal::Hash(seed, this->data_bits);
  seed = mojo::internal::Hash(seed, this->parity_bit);
  seed = mojo::internal::Hash(seed, this->stop_bits);
  seed = mojo::internal::Hash(seed, this->cts_flow_control);
  return seed;
}

bool ConnectionInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
HostControlSignals::HostControlSignals()
    : dtr(),
      has_dtr(false),
      rts(),
      has_rts(false) {}

HostControlSignals::HostControlSignals(
    bool dtr_in,
    bool has_dtr_in,
    bool rts_in,
    bool has_rts_in)
    : dtr(std::move(dtr_in)),
      has_dtr(std::move(has_dtr_in)),
      rts(std::move(rts_in)),
      has_rts(std::move(has_rts_in)) {}

HostControlSignals::~HostControlSignals() = default;
size_t HostControlSignals::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->dtr);
  seed = mojo::internal::Hash(seed, this->has_dtr);
  seed = mojo::internal::Hash(seed, this->rts);
  seed = mojo::internal::Hash(seed, this->has_rts);
  return seed;
}

bool HostControlSignals::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DeviceControlSignals::DeviceControlSignals()
    : dcd(),
      cts(),
      ri(),
      dsr() {}

DeviceControlSignals::DeviceControlSignals(
    bool dcd_in,
    bool cts_in,
    bool ri_in,
    bool dsr_in)
    : dcd(std::move(dcd_in)),
      cts(std::move(cts_in)),
      ri(std::move(ri_in)),
      dsr(std::move(dsr_in)) {}

DeviceControlSignals::~DeviceControlSignals() = default;
size_t DeviceControlSignals::Hash(size_t seed) const {
  seed = mojo::internal::Hash(seed, this->dcd);
  seed = mojo::internal::Hash(seed, this->cts);
  seed = mojo::internal::Hash(seed, this->ri);
  seed = mojo::internal::Hash(seed, this->dsr);
  return seed;
}

bool DeviceControlSignals::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
}  // namespace serial
}  // namespace device

namespace mojo {


// static
bool StructTraits<::device::serial::DeviceInfo::DataView, ::device::serial::DeviceInfoPtr>::Read(
    ::device::serial::DeviceInfo::DataView input,
    ::device::serial::DeviceInfoPtr* output) {
  bool success = true;
  ::device::serial::DeviceInfoPtr result(::device::serial::DeviceInfo::New());
  
      if (!input.ReadPath(&result->path))
        success = false;
      result->vendor_id = input.vendor_id();
      result->has_vendor_id = input.has_vendor_id();
      result->product_id = input.product_id();
      result->has_product_id = input.has_product_id();
      if (!input.ReadDisplayName(&result->display_name))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::serial::ConnectionOptions::DataView, ::device::serial::ConnectionOptionsPtr>::Read(
    ::device::serial::ConnectionOptions::DataView input,
    ::device::serial::ConnectionOptionsPtr* output) {
  bool success = true;
  ::device::serial::ConnectionOptionsPtr result(::device::serial::ConnectionOptions::New());
  
      result->bitrate = input.bitrate();
      if (!input.ReadDataBits(&result->data_bits))
        success = false;
      if (!input.ReadParityBit(&result->parity_bit))
        success = false;
      if (!input.ReadStopBits(&result->stop_bits))
        success = false;
      result->cts_flow_control = input.cts_flow_control();
      result->has_cts_flow_control = input.has_cts_flow_control();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::serial::ConnectionInfo::DataView, ::device::serial::ConnectionInfoPtr>::Read(
    ::device::serial::ConnectionInfo::DataView input,
    ::device::serial::ConnectionInfoPtr* output) {
  bool success = true;
  ::device::serial::ConnectionInfoPtr result(::device::serial::ConnectionInfo::New());
  
      result->bitrate = input.bitrate();
      if (!input.ReadDataBits(&result->data_bits))
        success = false;
      if (!input.ReadParityBit(&result->parity_bit))
        success = false;
      if (!input.ReadStopBits(&result->stop_bits))
        success = false;
      result->cts_flow_control = input.cts_flow_control();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::serial::HostControlSignals::DataView, ::device::serial::HostControlSignalsPtr>::Read(
    ::device::serial::HostControlSignals::DataView input,
    ::device::serial::HostControlSignalsPtr* output) {
  bool success = true;
  ::device::serial::HostControlSignalsPtr result(::device::serial::HostControlSignals::New());
  
      result->dtr = input.dtr();
      result->has_dtr = input.has_dtr();
      result->rts = input.rts();
      result->has_rts = input.has_rts();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::device::serial::DeviceControlSignals::DataView, ::device::serial::DeviceControlSignalsPtr>::Read(
    ::device::serial::DeviceControlSignals::DataView input,
    ::device::serial::DeviceControlSignalsPtr* output) {
  bool success = true;
  ::device::serial::DeviceControlSignalsPtr result(::device::serial::DeviceControlSignals::New());
  
      result->dcd = input.dcd();
      result->cts = input.cts();
      result->ri = input.ri();
      result->dsr = input.dsr();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif