// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef DEVICE_SERIAL_SERIAL_MOJOM_H_
#define DEVICE_SERIAL_SERIAL_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "device/serial/serial.mojom-shared.h"
#include <string>
#include <vector>


namespace device {
namespace serial {
class DeviceInfo;
using DeviceInfoPtr = mojo::StructPtr<DeviceInfo>;

class ConnectionOptions;
using ConnectionOptionsPtr = mojo::StructPtr<ConnectionOptions>;

class ConnectionInfo;
using ConnectionInfoPtr = mojo::StructPtr<ConnectionInfo>;

class HostControlSignals;
using HostControlSignalsPtr = mojo::InlinedStructPtr<HostControlSignals>;

class DeviceControlSignals;
using DeviceControlSignalsPtr = mojo::InlinedStructPtr<DeviceControlSignals>;






class  HostControlSignals {
 public:
  using DataView = HostControlSignalsDataView;
  using Data_ = internal::HostControlSignals_Data;

  template <typename... Args>
  static HostControlSignalsPtr New(Args&&... args) {
    return HostControlSignalsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static HostControlSignalsPtr From(const U& u) {
    return mojo::TypeConverter<HostControlSignalsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, HostControlSignals>::Convert(*this);
  }


  HostControlSignals();

  HostControlSignals(
      bool dtr,
      bool has_dtr,
      bool rts,
      bool has_rts);

  ~HostControlSignals();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = HostControlSignalsPtr>
  HostControlSignalsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, HostControlSignals>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        HostControlSignals::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        HostControlSignals::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  bool dtr;
  bool has_dtr;
  bool rts;
  bool has_rts;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  DeviceControlSignals {
 public:
  using DataView = DeviceControlSignalsDataView;
  using Data_ = internal::DeviceControlSignals_Data;

  template <typename... Args>
  static DeviceControlSignalsPtr New(Args&&... args) {
    return DeviceControlSignalsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DeviceControlSignalsPtr From(const U& u) {
    return mojo::TypeConverter<DeviceControlSignalsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DeviceControlSignals>::Convert(*this);
  }


  DeviceControlSignals();

  DeviceControlSignals(
      bool dcd,
      bool cts,
      bool ri,
      bool dsr);

  ~DeviceControlSignals();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DeviceControlSignalsPtr>
  DeviceControlSignalsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DeviceControlSignals>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DeviceControlSignals::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DeviceControlSignals::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  bool dcd;
  bool cts;
  bool ri;
  bool dsr;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};





class  DeviceInfo {
 public:
  using DataView = DeviceInfoDataView;
  using Data_ = internal::DeviceInfo_Data;

  template <typename... Args>
  static DeviceInfoPtr New(Args&&... args) {
    return DeviceInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DeviceInfoPtr From(const U& u) {
    return mojo::TypeConverter<DeviceInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DeviceInfo>::Convert(*this);
  }


  DeviceInfo();

  DeviceInfo(
      const std::string& path,
      uint16_t vendor_id,
      bool has_vendor_id,
      uint16_t product_id,
      bool has_product_id,
      const base::Optional<std::string>& display_name);

  ~DeviceInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DeviceInfoPtr>
  DeviceInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DeviceInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DeviceInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DeviceInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string path;
  uint16_t vendor_id;
  bool has_vendor_id;
  uint16_t product_id;
  bool has_product_id;
  base::Optional<std::string> display_name;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  ConnectionOptions {
 public:
  using DataView = ConnectionOptionsDataView;
  using Data_ = internal::ConnectionOptions_Data;

  template <typename... Args>
  static ConnectionOptionsPtr New(Args&&... args) {
    return ConnectionOptionsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ConnectionOptionsPtr From(const U& u) {
    return mojo::TypeConverter<ConnectionOptionsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ConnectionOptions>::Convert(*this);
  }


  ConnectionOptions();

  ConnectionOptions(
      uint32_t bitrate,
      DataBits data_bits,
      ParityBit parity_bit,
      StopBits stop_bits,
      bool cts_flow_control,
      bool has_cts_flow_control);

  ~ConnectionOptions();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ConnectionOptionsPtr>
  ConnectionOptionsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ConnectionOptions>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ConnectionOptions::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ConnectionOptions::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint32_t bitrate;
  DataBits data_bits;
  ParityBit parity_bit;
  StopBits stop_bits;
  bool cts_flow_control;
  bool has_cts_flow_control;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class  ConnectionInfo {
 public:
  using DataView = ConnectionInfoDataView;
  using Data_ = internal::ConnectionInfo_Data;

  template <typename... Args>
  static ConnectionInfoPtr New(Args&&... args) {
    return ConnectionInfoPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ConnectionInfoPtr From(const U& u) {
    return mojo::TypeConverter<ConnectionInfoPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ConnectionInfo>::Convert(*this);
  }


  ConnectionInfo();

  ConnectionInfo(
      uint32_t bitrate,
      DataBits data_bits,
      ParityBit parity_bit,
      StopBits stop_bits,
      bool cts_flow_control);

  ~ConnectionInfo();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ConnectionInfoPtr>
  ConnectionInfoPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ConnectionInfo>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ConnectionInfo::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ConnectionInfo::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  uint32_t bitrate;
  DataBits data_bits;
  ParityBit parity_bit;
  StopBits stop_bits;
  bool cts_flow_control;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};




template <typename StructPtrType>
DeviceInfoPtr DeviceInfo::Clone() const {
  return New(
      mojo::Clone(path),
      mojo::Clone(vendor_id),
      mojo::Clone(has_vendor_id),
      mojo::Clone(product_id),
      mojo::Clone(has_product_id),
      mojo::Clone(display_name)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DeviceInfo>::value>::type*>
bool DeviceInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->path, other.path))
    return false;
  if (!mojo::Equals(this->vendor_id, other.vendor_id))
    return false;
  if (!mojo::Equals(this->has_vendor_id, other.has_vendor_id))
    return false;
  if (!mojo::Equals(this->product_id, other.product_id))
    return false;
  if (!mojo::Equals(this->has_product_id, other.has_product_id))
    return false;
  if (!mojo::Equals(this->display_name, other.display_name))
    return false;
  return true;
}
template <typename StructPtrType>
ConnectionOptionsPtr ConnectionOptions::Clone() const {
  return New(
      mojo::Clone(bitrate),
      mojo::Clone(data_bits),
      mojo::Clone(parity_bit),
      mojo::Clone(stop_bits),
      mojo::Clone(cts_flow_control),
      mojo::Clone(has_cts_flow_control)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ConnectionOptions>::value>::type*>
bool ConnectionOptions::Equals(const T& other) const {
  if (!mojo::Equals(this->bitrate, other.bitrate))
    return false;
  if (!mojo::Equals(this->data_bits, other.data_bits))
    return false;
  if (!mojo::Equals(this->parity_bit, other.parity_bit))
    return false;
  if (!mojo::Equals(this->stop_bits, other.stop_bits))
    return false;
  if (!mojo::Equals(this->cts_flow_control, other.cts_flow_control))
    return false;
  if (!mojo::Equals(this->has_cts_flow_control, other.has_cts_flow_control))
    return false;
  return true;
}
template <typename StructPtrType>
ConnectionInfoPtr ConnectionInfo::Clone() const {
  return New(
      mojo::Clone(bitrate),
      mojo::Clone(data_bits),
      mojo::Clone(parity_bit),
      mojo::Clone(stop_bits),
      mojo::Clone(cts_flow_control)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ConnectionInfo>::value>::type*>
bool ConnectionInfo::Equals(const T& other) const {
  if (!mojo::Equals(this->bitrate, other.bitrate))
    return false;
  if (!mojo::Equals(this->data_bits, other.data_bits))
    return false;
  if (!mojo::Equals(this->parity_bit, other.parity_bit))
    return false;
  if (!mojo::Equals(this->stop_bits, other.stop_bits))
    return false;
  if (!mojo::Equals(this->cts_flow_control, other.cts_flow_control))
    return false;
  return true;
}
template <typename StructPtrType>
HostControlSignalsPtr HostControlSignals::Clone() const {
  return New(
      mojo::Clone(dtr),
      mojo::Clone(has_dtr),
      mojo::Clone(rts),
      mojo::Clone(has_rts)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, HostControlSignals>::value>::type*>
bool HostControlSignals::Equals(const T& other) const {
  if (!mojo::Equals(this->dtr, other.dtr))
    return false;
  if (!mojo::Equals(this->has_dtr, other.has_dtr))
    return false;
  if (!mojo::Equals(this->rts, other.rts))
    return false;
  if (!mojo::Equals(this->has_rts, other.has_rts))
    return false;
  return true;
}
template <typename StructPtrType>
DeviceControlSignalsPtr DeviceControlSignals::Clone() const {
  return New(
      mojo::Clone(dcd),
      mojo::Clone(cts),
      mojo::Clone(ri),
      mojo::Clone(dsr)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DeviceControlSignals>::value>::type*>
bool DeviceControlSignals::Equals(const T& other) const {
  if (!mojo::Equals(this->dcd, other.dcd))
    return false;
  if (!mojo::Equals(this->cts, other.cts))
    return false;
  if (!mojo::Equals(this->ri, other.ri))
    return false;
  if (!mojo::Equals(this->dsr, other.dsr))
    return false;
  return true;
}


}  // namespace serial
}  // namespace device

namespace mojo {


template <>
struct  StructTraits<::device::serial::DeviceInfo::DataView,
                                         ::device::serial::DeviceInfoPtr> {
  static bool IsNull(const ::device::serial::DeviceInfoPtr& input) { return !input; }
  static void SetToNull(::device::serial::DeviceInfoPtr* output) { output->reset(); }

  static const decltype(::device::serial::DeviceInfo::path)& path(
      const ::device::serial::DeviceInfoPtr& input) {
    return input->path;
  }

  static decltype(::device::serial::DeviceInfo::vendor_id) vendor_id(
      const ::device::serial::DeviceInfoPtr& input) {
    return input->vendor_id;
  }

  static decltype(::device::serial::DeviceInfo::has_vendor_id) has_vendor_id(
      const ::device::serial::DeviceInfoPtr& input) {
    return input->has_vendor_id;
  }

  static decltype(::device::serial::DeviceInfo::product_id) product_id(
      const ::device::serial::DeviceInfoPtr& input) {
    return input->product_id;
  }

  static decltype(::device::serial::DeviceInfo::has_product_id) has_product_id(
      const ::device::serial::DeviceInfoPtr& input) {
    return input->has_product_id;
  }

  static const decltype(::device::serial::DeviceInfo::display_name)& display_name(
      const ::device::serial::DeviceInfoPtr& input) {
    return input->display_name;
  }

  static bool Read(::device::serial::DeviceInfo::DataView input, ::device::serial::DeviceInfoPtr* output);
};


template <>
struct  StructTraits<::device::serial::ConnectionOptions::DataView,
                                         ::device::serial::ConnectionOptionsPtr> {
  static bool IsNull(const ::device::serial::ConnectionOptionsPtr& input) { return !input; }
  static void SetToNull(::device::serial::ConnectionOptionsPtr* output) { output->reset(); }

  static decltype(::device::serial::ConnectionOptions::bitrate) bitrate(
      const ::device::serial::ConnectionOptionsPtr& input) {
    return input->bitrate;
  }

  static decltype(::device::serial::ConnectionOptions::data_bits) data_bits(
      const ::device::serial::ConnectionOptionsPtr& input) {
    return input->data_bits;
  }

  static decltype(::device::serial::ConnectionOptions::parity_bit) parity_bit(
      const ::device::serial::ConnectionOptionsPtr& input) {
    return input->parity_bit;
  }

  static decltype(::device::serial::ConnectionOptions::stop_bits) stop_bits(
      const ::device::serial::ConnectionOptionsPtr& input) {
    return input->stop_bits;
  }

  static decltype(::device::serial::ConnectionOptions::cts_flow_control) cts_flow_control(
      const ::device::serial::ConnectionOptionsPtr& input) {
    return input->cts_flow_control;
  }

  static decltype(::device::serial::ConnectionOptions::has_cts_flow_control) has_cts_flow_control(
      const ::device::serial::ConnectionOptionsPtr& input) {
    return input->has_cts_flow_control;
  }

  static bool Read(::device::serial::ConnectionOptions::DataView input, ::device::serial::ConnectionOptionsPtr* output);
};


template <>
struct  StructTraits<::device::serial::ConnectionInfo::DataView,
                                         ::device::serial::ConnectionInfoPtr> {
  static bool IsNull(const ::device::serial::ConnectionInfoPtr& input) { return !input; }
  static void SetToNull(::device::serial::ConnectionInfoPtr* output) { output->reset(); }

  static decltype(::device::serial::ConnectionInfo::bitrate) bitrate(
      const ::device::serial::ConnectionInfoPtr& input) {
    return input->bitrate;
  }

  static decltype(::device::serial::ConnectionInfo::data_bits) data_bits(
      const ::device::serial::ConnectionInfoPtr& input) {
    return input->data_bits;
  }

  static decltype(::device::serial::ConnectionInfo::parity_bit) parity_bit(
      const ::device::serial::ConnectionInfoPtr& input) {
    return input->parity_bit;
  }

  static decltype(::device::serial::ConnectionInfo::stop_bits) stop_bits(
      const ::device::serial::ConnectionInfoPtr& input) {
    return input->stop_bits;
  }

  static decltype(::device::serial::ConnectionInfo::cts_flow_control) cts_flow_control(
      const ::device::serial::ConnectionInfoPtr& input) {
    return input->cts_flow_control;
  }

  static bool Read(::device::serial::ConnectionInfo::DataView input, ::device::serial::ConnectionInfoPtr* output);
};


template <>
struct  StructTraits<::device::serial::HostControlSignals::DataView,
                                         ::device::serial::HostControlSignalsPtr> {
  static bool IsNull(const ::device::serial::HostControlSignalsPtr& input) { return !input; }
  static void SetToNull(::device::serial::HostControlSignalsPtr* output) { output->reset(); }

  static decltype(::device::serial::HostControlSignals::dtr) dtr(
      const ::device::serial::HostControlSignalsPtr& input) {
    return input->dtr;
  }

  static decltype(::device::serial::HostControlSignals::has_dtr) has_dtr(
      const ::device::serial::HostControlSignalsPtr& input) {
    return input->has_dtr;
  }

  static decltype(::device::serial::HostControlSignals::rts) rts(
      const ::device::serial::HostControlSignalsPtr& input) {
    return input->rts;
  }

  static decltype(::device::serial::HostControlSignals::has_rts) has_rts(
      const ::device::serial::HostControlSignalsPtr& input) {
    return input->has_rts;
  }

  static bool Read(::device::serial::HostControlSignals::DataView input, ::device::serial::HostControlSignalsPtr* output);
};


template <>
struct  StructTraits<::device::serial::DeviceControlSignals::DataView,
                                         ::device::serial::DeviceControlSignalsPtr> {
  static bool IsNull(const ::device::serial::DeviceControlSignalsPtr& input) { return !input; }
  static void SetToNull(::device::serial::DeviceControlSignalsPtr* output) { output->reset(); }

  static decltype(::device::serial::DeviceControlSignals::dcd) dcd(
      const ::device::serial::DeviceControlSignalsPtr& input) {
    return input->dcd;
  }

  static decltype(::device::serial::DeviceControlSignals::cts) cts(
      const ::device::serial::DeviceControlSignalsPtr& input) {
    return input->cts;
  }

  static decltype(::device::serial::DeviceControlSignals::ri) ri(
      const ::device::serial::DeviceControlSignalsPtr& input) {
    return input->ri;
  }

  static decltype(::device::serial::DeviceControlSignals::dsr) dsr(
      const ::device::serial::DeviceControlSignalsPtr& input) {
    return input->dsr;
  }

  static bool Read(::device::serial::DeviceControlSignals::DataView input, ::device::serial::DeviceControlSignalsPtr* output);
};

}  // namespace mojo

#endif  // DEVICE_SERIAL_SERIAL_MOJOM_H_