// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef DEVICE_VR_VR_SERVICE_MOJOM_SHARED_INTERNAL_H_
#define DEVICE_VR_VR_SERVICE_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "mojo/common/time.mojom-shared-internal.h"
#include "gpu/ipc/common/mailbox_holder.mojom-shared-internal.h"
#include "gpu/ipc/common/sync_token.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared-internal.h"
#include "device/vr/mojo_bindings_shared_export.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace device {
namespace mojom {
namespace internal {
class VRFieldOfView_Data;
class VRPose_Data;
class VRDisplayCapabilities_Data;
class VREyeParameters_Data;
class VRStageParameters_Data;
class VRDisplayInfo_Data;

struct VRDisplayEventReason_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
      case 2:
      case 3:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

struct VRPresentationProvider_VSyncStatus_Data {
 public:
  static bool constexpr kIsExtensible = false;

  static bool IsKnownValue(int32_t value) {
    switch (value) {
      case 0:
      case 1:
        return true;
    }
    return false;
  }

  static bool Validate(int32_t value,
                       mojo::internal::ValidationContext* validation_context) {
    if (kIsExtensible || IsKnownValue(value))
      return true;

    ReportValidationError(validation_context,
                          mojo::internal::VALIDATION_ERROR_UNKNOWN_ENUM_VALUE);
    return false;
  }
};

#pragma pack(push, 1)
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRFieldOfView_Data {
 public:
  static VRFieldOfView_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  float upDegrees;
  float downDegrees;
  float leftDegrees;
  float rightDegrees;

 private:
  VRFieldOfView_Data();
  ~VRFieldOfView_Data() = delete;
};
static_assert(sizeof(VRFieldOfView_Data) == 24,
              "Bad sizeof(VRFieldOfView_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRPose_Data {
 public:
  static VRPose_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> orientation;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> position;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> angularVelocity;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> linearVelocity;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> angularAcceleration;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> linearAcceleration;

 private:
  VRPose_Data();
  ~VRPose_Data() = delete;
};
static_assert(sizeof(VRPose_Data) == 56,
              "Bad sizeof(VRPose_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayCapabilities_Data {
 public:
  static VRDisplayCapabilities_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t hasPosition : 1;
  uint8_t hasExternalDisplay : 1;
  uint8_t canPresent : 1;
  uint8_t padfinal_[7];

 private:
  VRDisplayCapabilities_Data();
  ~VRDisplayCapabilities_Data() = delete;
};
static_assert(sizeof(VRDisplayCapabilities_Data) == 16,
              "Bad sizeof(VRDisplayCapabilities_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VREyeParameters_Data {
 public:
  static VREyeParameters_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::VRFieldOfView_Data> fieldOfView;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> offset;
  uint32_t renderWidth;
  uint32_t renderHeight;

 private:
  VREyeParameters_Data();
  ~VREyeParameters_Data() = delete;
};
static_assert(sizeof(VREyeParameters_Data) == 32,
              "Bad sizeof(VREyeParameters_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRStageParameters_Data {
 public:
  static VRStageParameters_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<float>> standingTransform;
  float sizeX;
  float sizeZ;

 private:
  VRStageParameters_Data();
  ~VRStageParameters_Data() = delete;
};
static_assert(sizeof(VRStageParameters_Data) == 24,
              "Bad sizeof(VRStageParameters_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayInfo_Data {
 public:
  static VRDisplayInfo_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t index;
  uint8_t pad0_[4];
  mojo::internal::Pointer<mojo::internal::String_Data> displayName;
  mojo::internal::Pointer<internal::VRDisplayCapabilities_Data> capabilities;
  mojo::internal::Pointer<internal::VRStageParameters_Data> stageParameters;
  mojo::internal::Pointer<internal::VREyeParameters_Data> leftEye;
  mojo::internal::Pointer<internal::VREyeParameters_Data> rightEye;

 private:
  VRDisplayInfo_Data();
  ~VRDisplayInfo_Data() = delete;
};
static_assert(sizeof(VRDisplayInfo_Data) == 56,
              "Bad sizeof(VRDisplayInfo_Data)");
constexpr uint32_t kVRService_SetClient_Name = 0;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRService_SetClient_Params_Data {
 public:
  static VRService_SetClient_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Interface_Data client;

 private:
  VRService_SetClient_Params_Data();
  ~VRService_SetClient_Params_Data() = delete;
};
static_assert(sizeof(VRService_SetClient_Params_Data) == 16,
              "Bad sizeof(VRService_SetClient_Params_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRService_SetClient_ResponseParams_Data {
 public:
  static VRService_SetClient_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t number_of_connected_devices;
  uint8_t padfinal_[4];

 private:
  VRService_SetClient_ResponseParams_Data();
  ~VRService_SetClient_ResponseParams_Data() = delete;
};
static_assert(sizeof(VRService_SetClient_ResponseParams_Data) == 16,
              "Bad sizeof(VRService_SetClient_ResponseParams_Data)");
constexpr uint32_t kVRService_SetListeningForActivate_Name = 1;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRService_SetListeningForActivate_Params_Data {
 public:
  static VRService_SetListeningForActivate_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t listening : 1;
  uint8_t padfinal_[7];

 private:
  VRService_SetListeningForActivate_Params_Data();
  ~VRService_SetListeningForActivate_Params_Data() = delete;
};
static_assert(sizeof(VRService_SetListeningForActivate_Params_Data) == 16,
              "Bad sizeof(VRService_SetListeningForActivate_Params_Data)");
constexpr uint32_t kVRServiceClient_OnDisplayConnected_Name = 0;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRServiceClient_OnDisplayConnected_Params_Data {
 public:
  static VRServiceClient_OnDisplayConnected_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Interface_Data display;
  mojo::internal::Handle_Data request;
  uint8_t pad1_[4];
  mojo::internal::Pointer<internal::VRDisplayInfo_Data> display_info;

 private:
  VRServiceClient_OnDisplayConnected_Params_Data();
  ~VRServiceClient_OnDisplayConnected_Params_Data() = delete;
};
static_assert(sizeof(VRServiceClient_OnDisplayConnected_Params_Data) == 32,
              "Bad sizeof(VRServiceClient_OnDisplayConnected_Params_Data)");
constexpr uint32_t kVRSubmitFrameClient_OnSubmitFrameTransferred_Name = 0;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data {
 public:
  static VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data();
  ~VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data() = delete;
};
static_assert(sizeof(VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data) == 8,
              "Bad sizeof(VRSubmitFrameClient_OnSubmitFrameTransferred_Params_Data)");
constexpr uint32_t kVRSubmitFrameClient_OnSubmitFrameRendered_Name = 1;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data {
 public:
  static VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data();
  ~VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data() = delete;
};
static_assert(sizeof(VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data) == 8,
              "Bad sizeof(VRSubmitFrameClient_OnSubmitFrameRendered_Params_Data)");
constexpr uint32_t kVRDisplay_RequestPresent_Name = 0;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplay_RequestPresent_Params_Data {
 public:
  static VRDisplay_RequestPresent_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t secure_origin : 1;
  uint8_t pad0_[3];
  mojo::internal::Interface_Data client;
  mojo::internal::Handle_Data request;

 private:
  VRDisplay_RequestPresent_Params_Data();
  ~VRDisplay_RequestPresent_Params_Data() = delete;
};
static_assert(sizeof(VRDisplay_RequestPresent_Params_Data) == 24,
              "Bad sizeof(VRDisplay_RequestPresent_Params_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplay_RequestPresent_ResponseParams_Data {
 public:
  static VRDisplay_RequestPresent_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t success : 1;
  uint8_t padfinal_[7];

 private:
  VRDisplay_RequestPresent_ResponseParams_Data();
  ~VRDisplay_RequestPresent_ResponseParams_Data() = delete;
};
static_assert(sizeof(VRDisplay_RequestPresent_ResponseParams_Data) == 16,
              "Bad sizeof(VRDisplay_RequestPresent_ResponseParams_Data)");
constexpr uint32_t kVRDisplay_ExitPresent_Name = 1;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplay_ExitPresent_Params_Data {
 public:
  static VRDisplay_ExitPresent_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRDisplay_ExitPresent_Params_Data();
  ~VRDisplay_ExitPresent_Params_Data() = delete;
};
static_assert(sizeof(VRDisplay_ExitPresent_Params_Data) == 8,
              "Bad sizeof(VRDisplay_ExitPresent_Params_Data)");
constexpr uint32_t kVRDisplay_GetNextMagicWindowPose_Name = 2;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplay_GetNextMagicWindowPose_Params_Data {
 public:
  static VRDisplay_GetNextMagicWindowPose_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRDisplay_GetNextMagicWindowPose_Params_Data();
  ~VRDisplay_GetNextMagicWindowPose_Params_Data() = delete;
};
static_assert(sizeof(VRDisplay_GetNextMagicWindowPose_Params_Data) == 8,
              "Bad sizeof(VRDisplay_GetNextMagicWindowPose_Params_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplay_GetNextMagicWindowPose_ResponseParams_Data {
 public:
  static VRDisplay_GetNextMagicWindowPose_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::VRPose_Data> pose;

 private:
  VRDisplay_GetNextMagicWindowPose_ResponseParams_Data();
  ~VRDisplay_GetNextMagicWindowPose_ResponseParams_Data() = delete;
};
static_assert(sizeof(VRDisplay_GetNextMagicWindowPose_ResponseParams_Data) == 16,
              "Bad sizeof(VRDisplay_GetNextMagicWindowPose_ResponseParams_Data)");
constexpr uint32_t kVRPresentationProvider_GetVSync_Name = 0;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRPresentationProvider_GetVSync_Params_Data {
 public:
  static VRPresentationProvider_GetVSync_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRPresentationProvider_GetVSync_Params_Data();
  ~VRPresentationProvider_GetVSync_Params_Data() = delete;
};
static_assert(sizeof(VRPresentationProvider_GetVSync_Params_Data) == 8,
              "Bad sizeof(VRPresentationProvider_GetVSync_Params_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRPresentationProvider_GetVSync_ResponseParams_Data {
 public:
  static VRPresentationProvider_GetVSync_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::VRPose_Data> pose;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> time;
  int16_t frame_id;
  uint8_t pad2_[2];
  int32_t status;

 private:
  VRPresentationProvider_GetVSync_ResponseParams_Data();
  ~VRPresentationProvider_GetVSync_ResponseParams_Data() = delete;
};
static_assert(sizeof(VRPresentationProvider_GetVSync_ResponseParams_Data) == 32,
              "Bad sizeof(VRPresentationProvider_GetVSync_ResponseParams_Data)");
constexpr uint32_t kVRPresentationProvider_SubmitFrame_Name = 1;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRPresentationProvider_SubmitFrame_Params_Data {
 public:
  static VRPresentationProvider_SubmitFrame_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int16_t frame_id;
  uint8_t pad0_[6];
  mojo::internal::Pointer<::gpu::mojom::internal::MailboxHolder_Data> mailbox_holder;

 private:
  VRPresentationProvider_SubmitFrame_Params_Data();
  ~VRPresentationProvider_SubmitFrame_Params_Data() = delete;
};
static_assert(sizeof(VRPresentationProvider_SubmitFrame_Params_Data) == 24,
              "Bad sizeof(VRPresentationProvider_SubmitFrame_Params_Data)");
constexpr uint32_t kVRPresentationProvider_UpdateLayerBounds_Name = 2;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRPresentationProvider_UpdateLayerBounds_Params_Data {
 public:
  static VRPresentationProvider_UpdateLayerBounds_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int16_t frame_id;
  uint8_t pad0_[6];
  mojo::internal::Pointer<::gfx::mojom::internal::RectF_Data> left_bounds;
  mojo::internal::Pointer<::gfx::mojom::internal::RectF_Data> right_bounds;
  mojo::internal::Pointer<::gfx::mojom::internal::Size_Data> source_size;

 private:
  VRPresentationProvider_UpdateLayerBounds_Params_Data();
  ~VRPresentationProvider_UpdateLayerBounds_Params_Data() = delete;
};
static_assert(sizeof(VRPresentationProvider_UpdateLayerBounds_Params_Data) == 40,
              "Bad sizeof(VRPresentationProvider_UpdateLayerBounds_Params_Data)");
constexpr uint32_t kVRDisplayClient_OnChanged_Name = 0;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayClient_OnChanged_Params_Data {
 public:
  static VRDisplayClient_OnChanged_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::VRDisplayInfo_Data> display;

 private:
  VRDisplayClient_OnChanged_Params_Data();
  ~VRDisplayClient_OnChanged_Params_Data() = delete;
};
static_assert(sizeof(VRDisplayClient_OnChanged_Params_Data) == 16,
              "Bad sizeof(VRDisplayClient_OnChanged_Params_Data)");
constexpr uint32_t kVRDisplayClient_OnExitPresent_Name = 1;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayClient_OnExitPresent_Params_Data {
 public:
  static VRDisplayClient_OnExitPresent_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRDisplayClient_OnExitPresent_Params_Data();
  ~VRDisplayClient_OnExitPresent_Params_Data() = delete;
};
static_assert(sizeof(VRDisplayClient_OnExitPresent_Params_Data) == 8,
              "Bad sizeof(VRDisplayClient_OnExitPresent_Params_Data)");
constexpr uint32_t kVRDisplayClient_OnBlur_Name = 2;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayClient_OnBlur_Params_Data {
 public:
  static VRDisplayClient_OnBlur_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRDisplayClient_OnBlur_Params_Data();
  ~VRDisplayClient_OnBlur_Params_Data() = delete;
};
static_assert(sizeof(VRDisplayClient_OnBlur_Params_Data) == 8,
              "Bad sizeof(VRDisplayClient_OnBlur_Params_Data)");
constexpr uint32_t kVRDisplayClient_OnFocus_Name = 3;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayClient_OnFocus_Params_Data {
 public:
  static VRDisplayClient_OnFocus_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  VRDisplayClient_OnFocus_Params_Data();
  ~VRDisplayClient_OnFocus_Params_Data() = delete;
};
static_assert(sizeof(VRDisplayClient_OnFocus_Params_Data) == 8,
              "Bad sizeof(VRDisplayClient_OnFocus_Params_Data)");
constexpr uint32_t kVRDisplayClient_OnActivate_Name = 4;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayClient_OnActivate_Params_Data {
 public:
  static VRDisplayClient_OnActivate_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t reason;
  uint8_t padfinal_[4];

 private:
  VRDisplayClient_OnActivate_Params_Data();
  ~VRDisplayClient_OnActivate_Params_Data() = delete;
};
static_assert(sizeof(VRDisplayClient_OnActivate_Params_Data) == 16,
              "Bad sizeof(VRDisplayClient_OnActivate_Params_Data)");
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayClient_OnActivate_ResponseParams_Data {
 public:
  static VRDisplayClient_OnActivate_ResponseParams_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint8_t will_not_present : 1;
  uint8_t padfinal_[7];

 private:
  VRDisplayClient_OnActivate_ResponseParams_Data();
  ~VRDisplayClient_OnActivate_ResponseParams_Data() = delete;
};
static_assert(sizeof(VRDisplayClient_OnActivate_ResponseParams_Data) == 16,
              "Bad sizeof(VRDisplayClient_OnActivate_ResponseParams_Data)");
constexpr uint32_t kVRDisplayClient_OnDeactivate_Name = 5;
class DEVICE_VR_MOJO_BINDINGS_SHARED_EXPORT VRDisplayClient_OnDeactivate_Params_Data {
 public:
  static VRDisplayClient_OnDeactivate_Params_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t reason;
  uint8_t padfinal_[4];

 private:
  VRDisplayClient_OnDeactivate_Params_Data();
  ~VRDisplayClient_OnDeactivate_Params_Data() = delete;
};
static_assert(sizeof(VRDisplayClient_OnDeactivate_Params_Data) == 16,
              "Bad sizeof(VRDisplayClient_OnDeactivate_Params_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace device

#endif  // DEVICE_VR_VR_SERVICE_MOJOM_SHARED_INTERNAL_H_