// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef GPU_IPC_COMMON_GPU_FEATURE_INFO_MOJOM_SHARED_H_
#define GPU_IPC_COMMON_GPU_FEATURE_INFO_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "gpu/ipc/common/gpu_feature_info.mojom-shared-internal.h"


namespace gpu {
namespace mojom {
class GpuFeatureInfoDataView;



}  // namespace mojom
}  // namespace gpu

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::gpu::mojom::GpuFeatureInfoDataView> {
  using Data = ::gpu::mojom::internal::GpuFeatureInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace gpu {
namespace mojom {

enum class GpuFeatureStatus : int32_t {
  Enabled,
  Blacklisted,
  Disabled,
  Undefined,
  Max,
};

inline std::ostream& operator<<(std::ostream& os, GpuFeatureStatus value) {
  switch(value) {
    case GpuFeatureStatus::Enabled:
      return os << "GpuFeatureStatus::Enabled";
    case GpuFeatureStatus::Blacklisted:
      return os << "GpuFeatureStatus::Blacklisted";
    case GpuFeatureStatus::Disabled:
      return os << "GpuFeatureStatus::Disabled";
    case GpuFeatureStatus::Undefined:
      return os << "GpuFeatureStatus::Undefined";
    case GpuFeatureStatus::Max:
      return os << "GpuFeatureStatus::Max";
    default:
      return os << "Unknown GpuFeatureStatus value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(GpuFeatureStatus value) {
  return internal::GpuFeatureStatus_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class GpuFeatureInfoDataView {
 public:
  GpuFeatureInfoDataView() {}

  GpuFeatureInfoDataView(
      internal::GpuFeatureInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetStatusValuesDataView(
      mojo::ArrayDataView<GpuFeatureStatus>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatusValues(UserType* output) {
    auto* pointer = data_->status_values.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::gpu::mojom::GpuFeatureStatus>>(
        pointer, output, context_);
  }
 private:
  internal::GpuFeatureInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace gpu

namespace std {

template <>
struct hash<::gpu::mojom::GpuFeatureStatus>
    : public mojo::internal::EnumHashImpl<::gpu::mojom::GpuFeatureStatus> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::gpu::mojom::GpuFeatureStatus, ::gpu::mojom::GpuFeatureStatus> {
  static ::gpu::mojom::GpuFeatureStatus ToMojom(::gpu::mojom::GpuFeatureStatus input) { return input; }
  static bool FromMojom(::gpu::mojom::GpuFeatureStatus input, ::gpu::mojom::GpuFeatureStatus* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::gpu::mojom::GpuFeatureStatus, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::gpu::mojom::GpuFeatureStatus, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::gpu::mojom::GpuFeatureStatus>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::gpu::mojom::GpuFeatureInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::gpu::mojom::GpuFeatureInfoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::gpu::mojom::internal::GpuFeatureInfo_Data);
    decltype(CallWithContext(Traits::status_values, input, custom_context)) in_status_values = CallWithContext(Traits::status_values, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::gpu::mojom::GpuFeatureStatus>>(
        in_status_values, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::gpu::mojom::internal::GpuFeatureInfo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::gpu::mojom::internal::GpuFeatureInfo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::status_values, input, custom_context)) in_status_values = CallWithContext(Traits::status_values, input, custom_context);
    typename decltype(result->status_values)::BaseType* status_values_ptr;
    const mojo::internal::ContainerValidateParams status_values_validate_params(
        0, ::gpu::mojom::internal::GpuFeatureStatus_Data::Validate);
    mojo::internal::Serialize<mojo::ArrayDataView<::gpu::mojom::GpuFeatureStatus>>(
        in_status_values, buffer, &status_values_ptr, &status_values_validate_params,
        context);
    result->status_values.Set(status_values_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->status_values.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null status_values in GpuFeatureInfo struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::gpu::mojom::internal::GpuFeatureInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::gpu::mojom::GpuFeatureInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace gpu {
namespace mojom {

inline void GpuFeatureInfoDataView::GetStatusValuesDataView(
    mojo::ArrayDataView<GpuFeatureStatus>* output) {
  auto pointer = data_->status_values.Get();
  *output = mojo::ArrayDataView<GpuFeatureStatus>(pointer, context_);
}



}  // namespace mojom
}  // namespace gpu

#endif  // GPU_IPC_COMMON_GPU_FEATURE_INFO_MOJOM_SHARED_H_
