// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_MOJO_INTERFACES_MEDIA_TYPES_MOJOM_SHARED_INTERNAL_H_
#define MEDIA_MOJO_INTERFACES_MEDIA_TYPES_MOJOM_SHARED_INTERNAL_H_

#include <stdint.h>

#include "mojo/public/cpp/bindings/lib/array_internal.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/map_data_internal.h"
#include "mojo/public/cpp/bindings/lib/native_enum_data.h"
#include "mojo/public/cpp/bindings/lib/native_struct_data.h"
#include "mojo/public/cpp/bindings/lib/buffer.h"
#include "gpu/ipc/common/mailbox_holder.mojom-shared-internal.h"
#include "mojo/common/time.mojom-shared-internal.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared-internal.h"

namespace mojo {
namespace internal {
class ValidationContext;
}
}
namespace media {
namespace mojom {
namespace internal {
using MediaLogEvent_Data = mojo::internal::NativeStruct_Data;
class Pattern_Data;
class EncryptionScheme_Data;
class VideoColorSpace_Data;
class MasteringMetadata_Data;
class HDRMetadata_Data;
class AudioDecoderConfig_Data;
class VideoDecoderConfig_Data;
using SubsampleEntry_Data = mojo::internal::NativeStruct_Data;
class DecryptConfig_Data;
class DecoderBuffer_Data;
class AudioBuffer_Data;
class VideoFrame_Data;
class EosVideoFrameData_Data;
class SharedBufferVideoFrameData_Data;
class MailboxVideoFrameData_Data;
class PipelineStatistics_Data;
class VideoFrameData_Data;
using AudioCodec_Data =
    mojo::internal::NativeEnum_Data;
using BufferingState_Data =
    mojo::internal::NativeEnum_Data;
using ChannelLayout_Data =
    mojo::internal::NativeEnum_Data;
using ColorSpace_Data =
    mojo::internal::NativeEnum_Data;
using DecodeStatus_Data =
    mojo::internal::NativeEnum_Data;
using OutputDeviceStatus_Data =
    mojo::internal::NativeEnum_Data;
using SampleFormat_Data =
    mojo::internal::NativeEnum_Data;
using VideoCodec_Data =
    mojo::internal::NativeEnum_Data;
using VideoCodecProfile_Data =
    mojo::internal::NativeEnum_Data;
using VideoPixelFormat_Data =
    mojo::internal::NativeEnum_Data;
using EncryptionScheme_CipherMode_Data =
    mojo::internal::NativeEnum_Data;
using VideoColorSpace_PrimaryID_Data =
    mojo::internal::NativeEnum_Data;
using VideoColorSpace_TransferID_Data =
    mojo::internal::NativeEnum_Data;
using VideoColorSpace_MatrixID_Data =
    mojo::internal::NativeEnum_Data;
using VideoColorSpace_RangeID_Data =
    mojo::internal::NativeEnum_Data;

#pragma pack(push, 1)


class  VideoFrameData_Data {
 public:
  // Used to identify Mojom Union Data Classes.
  typedef void MojomUnionDataType;

  VideoFrameData_Data() {}
  // Do nothing in the destructor since it won't be called when it is a
  // non-inlined union.
  ~VideoFrameData_Data() {}

  static VideoFrameData_Data* New(mojo::internal::Buffer* buf) {
    return new (buf->Allocate(sizeof(VideoFrameData_Data))) VideoFrameData_Data();
  }

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context,
                       bool inlined);

  bool is_null() const { return size == 0; }

  void set_null() {
    size = 0U;
    tag = static_cast<VideoFrameData_Tag>(0);
    data.unknown = 0U;
  }

  enum class VideoFrameData_Tag : uint32_t {

    EOS_DATA,
    SHARED_BUFFER_DATA,
    MAILBOX_DATA,
  };

  // A note on layout:
  // "Each non-static data member is allocated as if it were the sole member of
  // a struct." - Section 9.5.2 ISO/IEC 14882:2011 (The C++ Spec)
  union MOJO_ALIGNAS(8) Union_ {
    mojo::internal::Pointer<internal::EosVideoFrameData_Data> f_eos_data;
    mojo::internal::Pointer<internal::SharedBufferVideoFrameData_Data> f_shared_buffer_data;
    mojo::internal::Pointer<internal::MailboxVideoFrameData_Data> f_mailbox_data;
    uint64_t unknown;
  };

  uint32_t size;
  VideoFrameData_Tag tag;
  Union_ data;
};
static_assert(sizeof(VideoFrameData_Data) == mojo::internal::kUnionDataSize,
              "Bad sizeof(VideoFrameData_Data)");
class  Pattern_Data {
 public:
  static Pattern_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint32_t encrypt_blocks;
  uint32_t skip_blocks;

 private:
  Pattern_Data();
  ~Pattern_Data() = delete;
};
static_assert(sizeof(Pattern_Data) == 16,
              "Bad sizeof(Pattern_Data)");
class  EncryptionScheme_Data {
 public:
  static EncryptionScheme_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t mode;
  uint8_t pad0_[4];
  mojo::internal::Pointer<internal::Pattern_Data> pattern;

 private:
  EncryptionScheme_Data();
  ~EncryptionScheme_Data() = delete;
};
static_assert(sizeof(EncryptionScheme_Data) == 24,
              "Bad sizeof(EncryptionScheme_Data)");
class  VideoColorSpace_Data {
 public:
  static VideoColorSpace_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t primaries;
  int32_t transfer;
  int32_t matrix;
  int32_t range;

 private:
  VideoColorSpace_Data();
  ~VideoColorSpace_Data() = delete;
};
static_assert(sizeof(VideoColorSpace_Data) == 24,
              "Bad sizeof(VideoColorSpace_Data)");
class  MasteringMetadata_Data {
 public:
  static MasteringMetadata_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> primary_r;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> primary_g;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> primary_b;
  mojo::internal::Pointer<::gfx::mojom::internal::PointF_Data> white_point;
  float luminance_max;
  float luminance_min;

 private:
  MasteringMetadata_Data();
  ~MasteringMetadata_Data() = delete;
};
static_assert(sizeof(MasteringMetadata_Data) == 48,
              "Bad sizeof(MasteringMetadata_Data)");
class  HDRMetadata_Data {
 public:
  static HDRMetadata_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<internal::MasteringMetadata_Data> mastering_metadata;
  uint32_t max_content_light_level;
  uint32_t max_frame_average_light_level;

 private:
  HDRMetadata_Data();
  ~HDRMetadata_Data() = delete;
};
static_assert(sizeof(HDRMetadata_Data) == 24,
              "Bad sizeof(HDRMetadata_Data)");
class  AudioDecoderConfig_Data {
 public:
  static AudioDecoderConfig_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t codec;
  int32_t sample_format;
  int32_t channel_layout;
  int32_t samples_per_second;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> extra_data;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> seek_preroll;
  int32_t codec_delay;
  uint8_t pad6_[4];
  mojo::internal::Pointer<internal::EncryptionScheme_Data> encryption_scheme;

 private:
  AudioDecoderConfig_Data();
  ~AudioDecoderConfig_Data() = delete;
};
static_assert(sizeof(AudioDecoderConfig_Data) == 56,
              "Bad sizeof(AudioDecoderConfig_Data)");
class  VideoDecoderConfig_Data {
 public:
  static VideoDecoderConfig_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t codec;
  int32_t profile;
  int32_t format;
  int32_t color_space;
  mojo::internal::Pointer<::gfx::mojom::internal::Size_Data> coded_size;
  mojo::internal::Pointer<::gfx::mojom::internal::Rect_Data> visible_rect;
  mojo::internal::Pointer<::gfx::mojom::internal::Size_Data> natural_size;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> extra_data;
  mojo::internal::Pointer<internal::EncryptionScheme_Data> encryption_scheme;
  mojo::internal::Pointer<internal::VideoColorSpace_Data> color_space_info;
  mojo::internal::Pointer<internal::HDRMetadata_Data> hdr_metadata;

 private:
  VideoDecoderConfig_Data();
  ~VideoDecoderConfig_Data() = delete;
};
static_assert(sizeof(VideoDecoderConfig_Data) == 80,
              "Bad sizeof(VideoDecoderConfig_Data)");
class  DecryptConfig_Data {
 public:
  static DecryptConfig_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::String_Data> key_id;
  mojo::internal::Pointer<mojo::internal::String_Data> iv;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<internal::SubsampleEntry_Data>>> subsamples;

 private:
  DecryptConfig_Data();
  ~DecryptConfig_Data() = delete;
};
static_assert(sizeof(DecryptConfig_Data) == 32,
              "Bad sizeof(DecryptConfig_Data)");
class  DecoderBuffer_Data {
 public:
  static DecoderBuffer_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> timestamp;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> duration;
  uint8_t is_end_of_stream : 1;
  uint8_t is_key_frame : 1;
  uint8_t pad3_[3];
  uint32_t data_size;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> side_data;
  mojo::internal::Pointer<internal::DecryptConfig_Data> decrypt_config;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> front_discard;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> back_discard;

 private:
  DecoderBuffer_Data();
  ~DecoderBuffer_Data() = delete;
};
static_assert(sizeof(DecoderBuffer_Data) == 64,
              "Bad sizeof(DecoderBuffer_Data)");
class  AudioBuffer_Data {
 public:
  static AudioBuffer_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t sample_format;
  int32_t channel_layout;
  int32_t channel_count;
  int32_t sample_rate;
  int32_t frame_count;
  uint8_t end_of_stream : 1;
  uint8_t pad5_[3];
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> timestamp;
  mojo::internal::Pointer<mojo::internal::Array_Data<uint8_t>> data;

 private:
  AudioBuffer_Data();
  ~AudioBuffer_Data() = delete;
};
static_assert(sizeof(AudioBuffer_Data) == 48,
              "Bad sizeof(AudioBuffer_Data)");
class  VideoFrame_Data {
 public:
  static VideoFrame_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  int32_t format;
  uint8_t pad0_[4];
  mojo::internal::Pointer<::gfx::mojom::internal::Size_Data> coded_size;
  mojo::internal::Pointer<::gfx::mojom::internal::Rect_Data> visible_rect;
  mojo::internal::Pointer<::gfx::mojom::internal::Size_Data> natural_size;
  mojo::internal::Pointer<::mojo::common::mojom::internal::TimeDelta_Data> timestamp;
  internal::VideoFrameData_Data data;

 private:
  VideoFrame_Data();
  ~VideoFrame_Data() = delete;
};
static_assert(sizeof(VideoFrame_Data) == 64,
              "Bad sizeof(VideoFrame_Data)");
class  EosVideoFrameData_Data {
 public:
  static EosVideoFrameData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;

 private:
  EosVideoFrameData_Data();
  ~EosVideoFrameData_Data() = delete;
};
static_assert(sizeof(EosVideoFrameData_Data) == 8,
              "Bad sizeof(EosVideoFrameData_Data)");
class  SharedBufferVideoFrameData_Data {
 public:
  static SharedBufferVideoFrameData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Handle_Data frame_data;
  int32_t y_stride;
  uint64_t frame_data_size;
  int32_t u_stride;
  int32_t v_stride;
  uint64_t y_offset;
  uint64_t u_offset;
  uint64_t v_offset;

 private:
  SharedBufferVideoFrameData_Data();
  ~SharedBufferVideoFrameData_Data() = delete;
};
static_assert(sizeof(SharedBufferVideoFrameData_Data) == 56,
              "Bad sizeof(SharedBufferVideoFrameData_Data)");
class  MailboxVideoFrameData_Data {
 public:
  static MailboxVideoFrameData_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  mojo::internal::Pointer<mojo::internal::Array_Data<mojo::internal::Pointer<::gpu::mojom::internal::MailboxHolder_Data>>> mailbox_holder;

 private:
  MailboxVideoFrameData_Data();
  ~MailboxVideoFrameData_Data() = delete;
};
static_assert(sizeof(MailboxVideoFrameData_Data) == 16,
              "Bad sizeof(MailboxVideoFrameData_Data)");
class  PipelineStatistics_Data {
 public:
  static PipelineStatistics_Data* New(mojo::internal::Buffer* buf);

  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);

  mojo::internal::StructHeader header_;
  uint64_t audio_bytes_decoded;
  uint64_t video_bytes_decoded;
  uint32_t video_frames_decoded;
  uint32_t video_frames_dropped;
  int64_t audio_memory_usage;
  int64_t video_memory_usage;

 private:
  PipelineStatistics_Data();
  ~PipelineStatistics_Data() = delete;
};
static_assert(sizeof(PipelineStatistics_Data) == 48,
              "Bad sizeof(PipelineStatistics_Data)");

#pragma pack(pop)

}  // namespace internal
}  // namespace mojom
}  // namespace media

#endif  // MEDIA_MOJO_INTERFACES_MEDIA_TYPES_MOJOM_SHARED_INTERNAL_H_