// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_MOJO_INTERFACES_REMOTING_COMMON_MOJOM_H_
#define MEDIA_MOJO_INTERFACES_REMOTING_COMMON_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "media/mojo/interfaces/remoting_common.mojom-shared.h"
#include <string>
#include <vector>


namespace media {
namespace mojom {
class SinkCapabilities;
using SinkCapabilitiesPtr = mojo::StructPtr<SinkCapabilities>;






class  SinkCapabilities {
 public:
  using DataView = SinkCapabilitiesDataView;
  using Data_ = internal::SinkCapabilities_Data;

  template <typename... Args>
  static SinkCapabilitiesPtr New(Args&&... args) {
    return SinkCapabilitiesPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static SinkCapabilitiesPtr From(const U& u) {
    return mojo::TypeConverter<SinkCapabilitiesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, SinkCapabilities>::Convert(*this);
  }


  SinkCapabilities();

  SinkCapabilities(
      const std::vector<RemotingSinkFeatures>& features,
      const std::vector<RemotingSinkAudioCapabilities>& audio,
      const std::vector<RemotingSinkVideoCapabilities>& video);

  ~SinkCapabilities();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = SinkCapabilitiesPtr>
  SinkCapabilitiesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, SinkCapabilities>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        SinkCapabilities::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        SinkCapabilities::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::vector<RemotingSinkFeatures> features;
  std::vector<RemotingSinkAudioCapabilities> audio;
  std::vector<RemotingSinkVideoCapabilities> video;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
SinkCapabilitiesPtr SinkCapabilities::Clone() const {
  return New(
      mojo::Clone(features),
      mojo::Clone(audio),
      mojo::Clone(video)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, SinkCapabilities>::value>::type*>
bool SinkCapabilities::Equals(const T& other) const {
  if (!mojo::Equals(this->features, other.features))
    return false;
  if (!mojo::Equals(this->audio, other.audio))
    return false;
  if (!mojo::Equals(this->video, other.video))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace media

namespace mojo {


template <>
struct  StructTraits<::media::mojom::SinkCapabilities::DataView,
                                         ::media::mojom::SinkCapabilitiesPtr> {
  static bool IsNull(const ::media::mojom::SinkCapabilitiesPtr& input) { return !input; }
  static void SetToNull(::media::mojom::SinkCapabilitiesPtr* output) { output->reset(); }

  static const decltype(::media::mojom::SinkCapabilities::features)& features(
      const ::media::mojom::SinkCapabilitiesPtr& input) {
    return input->features;
  }

  static const decltype(::media::mojom::SinkCapabilities::audio)& audio(
      const ::media::mojom::SinkCapabilitiesPtr& input) {
    return input->audio;
  }

  static const decltype(::media::mojom::SinkCapabilities::video)& video(
      const ::media::mojom::SinkCapabilitiesPtr& input) {
    return input->video;
  }

  static bool Read(::media::mojom::SinkCapabilities::DataView input, ::media::mojom::SinkCapabilitiesPtr* output);
};

}  // namespace mojo

#endif  // MEDIA_MOJO_INTERFACES_REMOTING_COMMON_MOJOM_H_