// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef MEDIA_MOJO_INTERFACES_VIDEO_DECODER_MOJOM_H_
#define MEDIA_MOJO_INTERFACES_VIDEO_DECODER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "media/mojo/interfaces/video_decoder.mojom-shared.h"
#include "gpu/ipc/common/sync_token.mojom.h"
#include "media/mojo/interfaces/media_log.mojom.h"
#include "media/mojo/interfaces/media_types.mojom.h"
#include "mojo/common/unguessable_token.mojom.h"
#include <string>
#include <vector>


namespace media {
namespace mojom {
class VideoDecoder;
using VideoDecoderPtr = mojo::InterfacePtr<VideoDecoder>;
using VideoDecoderPtrInfo = mojo::InterfacePtrInfo<VideoDecoder>;
using ThreadSafeVideoDecoderPtr =
    mojo::ThreadSafeInterfacePtr<VideoDecoder>;
using VideoDecoderRequest = mojo::InterfaceRequest<VideoDecoder>;
using VideoDecoderAssociatedPtr =
    mojo::AssociatedInterfacePtr<VideoDecoder>;
using ThreadSafeVideoDecoderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<VideoDecoder>;
using VideoDecoderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<VideoDecoder>;
using VideoDecoderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<VideoDecoder>;

class VideoDecoderClient;
using VideoDecoderClientPtr = mojo::InterfacePtr<VideoDecoderClient>;
using VideoDecoderClientPtrInfo = mojo::InterfacePtrInfo<VideoDecoderClient>;
using ThreadSafeVideoDecoderClientPtr =
    mojo::ThreadSafeInterfacePtr<VideoDecoderClient>;
using VideoDecoderClientRequest = mojo::InterfaceRequest<VideoDecoderClient>;
using VideoDecoderClientAssociatedPtr =
    mojo::AssociatedInterfacePtr<VideoDecoderClient>;
using ThreadSafeVideoDecoderClientAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<VideoDecoderClient>;
using VideoDecoderClientAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<VideoDecoderClient>;
using VideoDecoderClientAssociatedRequest =
    mojo::AssociatedInterfaceRequest<VideoDecoderClient>;

class CommandBufferId;
using CommandBufferIdPtr = mojo::StructPtr<CommandBufferId>;


class VideoDecoderProxy;

template <typename ImplRefTraits>
class VideoDecoderStub;

class VideoDecoderRequestValidator;
class VideoDecoderResponseValidator;

class  VideoDecoder
    : public VideoDecoderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = true;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = VideoDecoderProxy;

  template <typename ImplRefTraits>
  using Stub_ = VideoDecoderStub<ImplRefTraits>;

  using RequestValidator_ = VideoDecoderRequestValidator;
  using ResponseValidator_ = VideoDecoderResponseValidator;
  enum MethodMinVersions : uint32_t {
    kConstructMinVersion = 0,
    kInitializeMinVersion = 0,
    kDecodeMinVersion = 0,
    kResetMinVersion = 0,
    kOnReleaseMailboxMinVersion = 0,
  };
  virtual ~VideoDecoder() {}

  virtual void Construct(VideoDecoderClientAssociatedPtrInfo client, ::media::mojom::MediaLogAssociatedPtrInfo media_log, mojo::ScopedDataPipeConsumerHandle decoder_buffer_pipe, CommandBufferIdPtr command_buffer_id) = 0;


  using InitializeCallback = base::OnceCallback<void(bool, bool, int32_t)>;
  virtual void Initialize(const media::VideoDecoderConfig& config, bool low_delay, InitializeCallback callback) = 0;


  using DecodeCallback = base::OnceCallback<void(media::DecodeStatus)>;
  virtual void Decode(::media::mojom::DecoderBufferPtr buffer, DecodeCallback callback) = 0;


  using ResetCallback = base::OnceCallback<void()>;
  virtual void Reset(ResetCallback callback) = 0;

  virtual void OnReleaseMailbox(const base::UnguessableToken& release_token, const ::gpu::SyncToken& release_sync_token) = 0;
};

class VideoDecoderClientProxy;

template <typename ImplRefTraits>
class VideoDecoderClientStub;

class VideoDecoderClientRequestValidator;

class  VideoDecoderClient
    : public VideoDecoderClientInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = VideoDecoderClientProxy;

  template <typename ImplRefTraits>
  using Stub_ = VideoDecoderClientStub<ImplRefTraits>;

  using RequestValidator_ = VideoDecoderClientRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnVideoFrameDecodedMinVersion = 0,
  };
  virtual ~VideoDecoderClient() {}

  virtual void OnVideoFrameDecoded(const scoped_refptr<media::VideoFrame>& frame, bool can_read_without_stalling, const base::Optional<base::UnguessableToken>& release_token) = 0;
};

class  VideoDecoderProxy
    : public VideoDecoder {
 public:
  explicit VideoDecoderProxy(mojo::MessageReceiverWithResponder* receiver);
  void Construct(VideoDecoderClientAssociatedPtrInfo client, ::media::mojom::MediaLogAssociatedPtrInfo media_log, mojo::ScopedDataPipeConsumerHandle decoder_buffer_pipe, CommandBufferIdPtr command_buffer_id) override;
  void Initialize(const media::VideoDecoderConfig& config, bool low_delay, InitializeCallback callback) override;
  void Decode(::media::mojom::DecoderBufferPtr buffer, DecodeCallback callback) override;
  void Reset(ResetCallback callback) override;
  void OnReleaseMailbox(const base::UnguessableToken& release_token, const ::gpu::SyncToken& release_sync_token) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  VideoDecoderClientProxy
    : public VideoDecoderClient {
 public:
  explicit VideoDecoderClientProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnVideoFrameDecoded(const scoped_refptr<media::VideoFrame>& frame, bool can_read_without_stalling, const base::Optional<base::UnguessableToken>& release_token) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  VideoDecoderStubDispatch {
 public:
  static bool Accept(VideoDecoder* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      VideoDecoder* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<VideoDecoder>>
class VideoDecoderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  VideoDecoderStub() {}
  ~VideoDecoderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return VideoDecoderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return VideoDecoderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  VideoDecoderClientStubDispatch {
 public:
  static bool Accept(VideoDecoderClient* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      VideoDecoderClient* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<VideoDecoderClient>>
class VideoDecoderClientStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  VideoDecoderClientStub() {}
  ~VideoDecoderClientStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return VideoDecoderClientStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return VideoDecoderClientStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  VideoDecoderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  VideoDecoderClientRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  VideoDecoderResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





class  CommandBufferId {
 public:
  using DataView = CommandBufferIdDataView;
  using Data_ = internal::CommandBufferId_Data;

  template <typename... Args>
  static CommandBufferIdPtr New(Args&&... args) {
    return CommandBufferIdPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CommandBufferIdPtr From(const U& u) {
    return mojo::TypeConverter<CommandBufferIdPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CommandBufferId>::Convert(*this);
  }


  CommandBufferId();

  CommandBufferId(
      const base::UnguessableToken& channel_token,
      int32_t route_id);

  ~CommandBufferId();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CommandBufferIdPtr>
  CommandBufferIdPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CommandBufferId>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CommandBufferId::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CommandBufferId::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::UnguessableToken channel_token;
  int32_t route_id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
CommandBufferIdPtr CommandBufferId::Clone() const {
  return New(
      mojo::Clone(channel_token),
      mojo::Clone(route_id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CommandBufferId>::value>::type*>
bool CommandBufferId::Equals(const T& other) const {
  if (!mojo::Equals(this->channel_token, other.channel_token))
    return false;
  if (!mojo::Equals(this->route_id, other.route_id))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace media

namespace mojo {


template <>
struct  StructTraits<::media::mojom::CommandBufferId::DataView,
                                         ::media::mojom::CommandBufferIdPtr> {
  static bool IsNull(const ::media::mojom::CommandBufferIdPtr& input) { return !input; }
  static void SetToNull(::media::mojom::CommandBufferIdPtr* output) { output->reset(); }

  static const decltype(::media::mojom::CommandBufferId::channel_token)& channel_token(
      const ::media::mojom::CommandBufferIdPtr& input) {
    return input->channel_token;
  }

  static decltype(::media::mojom::CommandBufferId::route_id) route_id(
      const ::media::mojom::CommandBufferIdPtr& input) {
    return input->route_id;
  }

  static bool Read(::media::mojom::CommandBufferId::DataView input, ::media::mojom::CommandBufferIdPtr* output);
};

}  // namespace mojo

#endif  // MEDIA_MOJO_INTERFACES_VIDEO_DECODER_MOJOM_H_