// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef NET_INTERFACES_IP_ADDRESS_MOJOM_H_
#define NET_INTERFACES_IP_ADDRESS_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "net/interfaces/ip_address.mojom-shared.h"
#include <string>
#include <vector>
#include "net/base/ip_address.h"


namespace net {
namespace interfaces {
class IPAddress;
using IPAddressPtr = mojo::StructPtr<IPAddress>;






class  IPAddress {
 public:
  using DataView = IPAddressDataView;
  using Data_ = internal::IPAddress_Data;

  template <typename... Args>
  static IPAddressPtr New(Args&&... args) {
    return IPAddressPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static IPAddressPtr From(const U& u) {
    return mojo::TypeConverter<IPAddressPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, IPAddress>::Convert(*this);
  }


  IPAddress();

  explicit IPAddress(
      const std::vector<uint8_t>& address);

  ~IPAddress();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = IPAddressPtr>
  IPAddressPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, IPAddress>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        IPAddress::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        IPAddress::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::vector<uint8_t> address;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
IPAddressPtr IPAddress::Clone() const {
  return New(
      mojo::Clone(address)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, IPAddress>::value>::type*>
bool IPAddress::Equals(const T& other) const {
  if (!mojo::Equals(this->address, other.address))
    return false;
  return true;
}


}  // namespace interfaces
}  // namespace net

namespace mojo {


template <>
struct  StructTraits<::net::interfaces::IPAddress::DataView,
                                         ::net::interfaces::IPAddressPtr> {
  static bool IsNull(const ::net::interfaces::IPAddressPtr& input) { return !input; }
  static void SetToNull(::net::interfaces::IPAddressPtr* output) { output->reset(); }

  static const decltype(::net::interfaces::IPAddress::address)& address(
      const ::net::interfaces::IPAddressPtr& input) {
    return input->address;
  }

  static bool Read(::net::interfaces::IPAddress::DataView input, ::net::interfaces::IPAddressPtr* output);
};

}  // namespace mojo

#endif  // NET_INTERFACES_IP_ADDRESS_MOJOM_H_