// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_DATA_DECODER_PUBLIC_INTERFACES_IMAGE_DECODER_MOJOM_H_
#define SERVICES_DATA_DECODER_PUBLIC_INTERFACES_IMAGE_DECODER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/data_decoder/public/interfaces/image_decoder.mojom-shared.h"
#include "skia/public/interfaces/bitmap.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>


namespace data_decoder {
namespace mojom {
class ImageDecoder;
using ImageDecoderPtr = mojo::InterfacePtr<ImageDecoder>;
using ImageDecoderPtrInfo = mojo::InterfacePtrInfo<ImageDecoder>;
using ThreadSafeImageDecoderPtr =
    mojo::ThreadSafeInterfacePtr<ImageDecoder>;
using ImageDecoderRequest = mojo::InterfaceRequest<ImageDecoder>;
using ImageDecoderAssociatedPtr =
    mojo::AssociatedInterfacePtr<ImageDecoder>;
using ThreadSafeImageDecoderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ImageDecoder>;
using ImageDecoderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ImageDecoder>;
using ImageDecoderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ImageDecoder>;


class ImageDecoderProxy;

template <typename ImplRefTraits>
class ImageDecoderStub;

class ImageDecoderRequestValidator;
class ImageDecoderResponseValidator;

class  ImageDecoder
    : public ImageDecoderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ImageDecoderProxy;

  template <typename ImplRefTraits>
  using Stub_ = ImageDecoderStub<ImplRefTraits>;

  using RequestValidator_ = ImageDecoderRequestValidator;
  using ResponseValidator_ = ImageDecoderResponseValidator;
  enum MethodMinVersions : uint32_t {
    kDecodeImageMinVersion = 0,
  };
  virtual ~ImageDecoder() {}


  using DecodeImageCallback = base::OnceCallback<void(const SkBitmap&)>;
  virtual void DecodeImage(const std::vector<uint8_t>& encoded_data, ImageCodec codec, bool shrink_to_fit, int64_t max_size_in_bytes, const gfx::Size& desired_image_frame_size, DecodeImageCallback callback) = 0;
};

class  ImageDecoderProxy
    : public ImageDecoder {
 public:
  explicit ImageDecoderProxy(mojo::MessageReceiverWithResponder* receiver);
  void DecodeImage(const std::vector<uint8_t>& encoded_data, ImageCodec codec, bool shrink_to_fit, int64_t max_size_in_bytes, const gfx::Size& desired_image_frame_size, DecodeImageCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  ImageDecoderStubDispatch {
 public:
  static bool Accept(ImageDecoder* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ImageDecoder* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ImageDecoder>>
class ImageDecoderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ImageDecoderStub() {}
  ~ImageDecoderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ImageDecoderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ImageDecoderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  ImageDecoderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  ImageDecoderResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace data_decoder

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_DATA_DECODER_PUBLIC_INTERFACES_IMAGE_DECODER_MOJOM_H_