// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/device/public/interfaces/sensor_provider.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
namespace device {
namespace mojom {
namespace blink {
constexpr uint64_t SensorInitParams::kReadBufferSizeForTests;
SensorInitParams::SensorInitParams()
    : memory(),
      buffer_offset(),
      mode(),
      default_configuration(),
      maximum_frequency(),
      minimum_frequency() {}

SensorInitParams::SensorInitParams(
    mojo::ScopedSharedBufferHandle memory_in,
    uint64_t buffer_offset_in,
    ::device::mojom::blink::ReportingMode mode_in,
    ::device::mojom::blink::SensorConfigurationPtr default_configuration_in,
    double maximum_frequency_in,
    double minimum_frequency_in)
    : memory(std::move(memory_in)),
      buffer_offset(std::move(buffer_offset_in)),
      mode(std::move(mode_in)),
      default_configuration(std::move(default_configuration_in)),
      maximum_frequency(std::move(maximum_frequency_in)),
      minimum_frequency(std::move(minimum_frequency_in)) {}

SensorInitParams::~SensorInitParams() = default;

bool SensorInitParams::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char SensorProvider::Name_[] = "device::mojom::SensorProvider";

class SensorProvider_GetSensor_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  SensorProvider_GetSensor_ForwardToCallback(
      SensorProvider::GetSensorCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  SensorProvider::GetSensorCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(SensorProvider_GetSensor_ForwardToCallback);
};

SensorProviderProxy::SensorProviderProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void SensorProviderProxy::GetSensor(
    ::device::mojom::blink::SensorType in_type, ::device::mojom::blink::SensorRequest in_sensor_request, GetSensorCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::SensorProvider_GetSensor_Params_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::SensorRequestDataView>(
      in_sensor_request, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kSensorProvider_GetSensor_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::SensorProvider_GetSensor_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::device::mojom::SensorType>(
      in_type, &params->type);
  mojo::internal::Serialize<::device::mojom::SensorRequestDataView>(
      in_sensor_request, &params->sensor_request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->sensor_request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid sensor_request in SensorProvider.GetSensor request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new SensorProvider_GetSensor_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class SensorProvider_GetSensor_ProxyToResponder {
 public:
  static SensorProvider::GetSensorCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<SensorProvider_GetSensor_ProxyToResponder> proxy(
        new SensorProvider_GetSensor_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&SensorProvider_GetSensor_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~SensorProvider_GetSensor_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "SensorProvider::GetSensor() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  SensorProvider_GetSensor_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      SensorInitParamsPtr in_init_params, ::device::mojom::blink::SensorClientRequest in_client_request);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(SensorProvider_GetSensor_ProxyToResponder);
};

bool SensorProvider_GetSensor_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::SensorProvider_GetSensor_ResponseParams_Data* params =
      reinterpret_cast<
          internal::SensorProvider_GetSensor_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  SensorInitParamsPtr p_init_params{};
  ::device::mojom::blink::SensorClientRequest p_client_request{};
  SensorProvider_GetSensor_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadInitParams(&p_init_params))
    success = false;
  p_client_request =
      input_data_view.TakeClientRequest<decltype(p_client_request)>();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "SensorProvider::GetSensor response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_init_params), 
std::move(p_client_request));
  return true;
}

void SensorProvider_GetSensor_ProxyToResponder::Run(
    SensorInitParamsPtr in_init_params, ::device::mojom::blink::SensorClientRequest in_client_request) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::device::mojom::internal::SensorProvider_GetSensor_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::device::mojom::SensorInitParamsDataView>(
      in_init_params, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::device::mojom::SensorClientRequestDataView>(
      in_client_request, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kSensorProvider_GetSensor_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::device::mojom::internal::SensorProvider_GetSensor_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->init_params)::BaseType* init_params_ptr;
  mojo::internal::Serialize<::device::mojom::SensorInitParamsDataView>(
      in_init_params, buffer, &init_params_ptr, &serialization_context);
  params->init_params.Set(init_params_ptr);
  mojo::internal::Serialize<::device::mojom::SensorClientRequestDataView>(
      in_client_request, &params->client_request, &serialization_context);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool SensorProviderStubDispatch::Accept(
    SensorProvider* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kSensorProvider_GetSensor_Name: {
      break;
    }
  }
  return false;
}

// static
bool SensorProviderStubDispatch::AcceptWithResponder(
    SensorProvider* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kSensorProvider_GetSensor_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::SensorProvider_GetSensor_Params_Data* params =
          reinterpret_cast<
              internal::SensorProvider_GetSensor_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::device::mojom::blink::SensorType p_type{};
      ::device::mojom::blink::SensorRequest p_sensor_request{};
      SensorProvider_GetSensor_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadType(&p_type))
        success = false;
      p_sensor_request =
          input_data_view.TakeSensorRequest<decltype(p_sensor_request)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "SensorProvider::GetSensor deserializer");
        return false;
      }
      SensorProvider::GetSensorCallback callback =
          SensorProvider_GetSensor_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetSensor(
std::move(p_type), 
std::move(p_sensor_request), std::move(callback));
      return true;
    }
  }
  return false;
}

bool SensorProviderRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SensorProvider RequestValidator");

  switch (message->header()->name) {
    case internal::kSensorProvider_GetSensor_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::SensorProvider_GetSensor_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool SensorProviderResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "SensorProvider ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kSensorProvider_GetSensor_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::SensorProvider_GetSensor_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace blink
}  // namespace mojom
}  // namespace device

namespace mojo {


// static
bool StructTraits<::device::mojom::blink::SensorInitParams::DataView, ::device::mojom::blink::SensorInitParamsPtr>::Read(
    ::device::mojom::blink::SensorInitParams::DataView input,
    ::device::mojom::blink::SensorInitParamsPtr* output) {
  bool success = true;
  ::device::mojom::blink::SensorInitParamsPtr result(::device::mojom::blink::SensorInitParams::New());
  
      result->memory = input.TakeMemory();
      result->buffer_offset = input.buffer_offset();
      if (!input.ReadMode(&result->mode))
        success = false;
      if (!input.ReadDefaultConfiguration(&result->default_configuration))
        success = false;
      result->maximum_frequency = input.maximum_frequency();
      result->minimum_frequency = input.minimum_frequency();
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif