// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_SHARED_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/resource_coordinator/public/interfaces/coordination_unit.mojom-shared-internal.h"
#include "services/resource_coordinator/public/interfaces/events.mojom-shared.h"
#include "mojo/common/values.mojom-shared.h"
#include "services/resource_coordinator/public/interfaces/interfaces_internal_shared_export.h"


namespace resource_coordinator {
namespace mojom {
class CoordinationUnitIDDataView;

class CoordinationPolicyDataView;



}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::CoordinationUnitIDDataView> {
  using Data = ::resource_coordinator::mojom::internal::CoordinationUnitID_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::resource_coordinator::mojom::CoordinationPolicyDataView> {
  using Data = ::resource_coordinator::mojom::internal::CoordinationPolicy_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace resource_coordinator {
namespace mojom {

enum class CoordinationUnitType : int32_t {
  kWebContents,
  kFrame,
  kNavigation,
  kProcess,
};

inline std::ostream& operator<<(std::ostream& os, CoordinationUnitType value) {
  switch(value) {
    case CoordinationUnitType::kWebContents:
      return os << "CoordinationUnitType::kWebContents";
    case CoordinationUnitType::kFrame:
      return os << "CoordinationUnitType::kFrame";
    case CoordinationUnitType::kNavigation:
      return os << "CoordinationUnitType::kNavigation";
    case CoordinationUnitType::kProcess:
      return os << "CoordinationUnitType::kProcess";
    default:
      return os << "Unknown CoordinationUnitType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(CoordinationUnitType value) {
  return internal::CoordinationUnitType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class PropertyType : int32_t {
  kTest,
  kCPUUsage,
  kNetworkIdle,
  kVisible,
};

inline std::ostream& operator<<(std::ostream& os, PropertyType value) {
  switch(value) {
    case PropertyType::kTest:
      return os << "PropertyType::kTest";
    case PropertyType::kCPUUsage:
      return os << "PropertyType::kCPUUsage";
    case PropertyType::kNetworkIdle:
      return os << "PropertyType::kNetworkIdle";
    case PropertyType::kVisible:
      return os << "PropertyType::kVisible";
    default:
      return os << "Unknown PropertyType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(PropertyType value) {
  return internal::PropertyType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class CoordinationPolicyCallbackInterfaceBase {};

using CoordinationPolicyCallbackPtrDataView =
    mojo::InterfacePtrDataView<CoordinationPolicyCallbackInterfaceBase>;
using CoordinationPolicyCallbackRequestDataView =
    mojo::InterfaceRequestDataView<CoordinationPolicyCallbackInterfaceBase>;
using CoordinationPolicyCallbackAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CoordinationPolicyCallbackInterfaceBase>;
using CoordinationPolicyCallbackAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CoordinationPolicyCallbackInterfaceBase>;
class CoordinationUnitInterfaceBase {};

using CoordinationUnitPtrDataView =
    mojo::InterfacePtrDataView<CoordinationUnitInterfaceBase>;
using CoordinationUnitRequestDataView =
    mojo::InterfaceRequestDataView<CoordinationUnitInterfaceBase>;
using CoordinationUnitAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CoordinationUnitInterfaceBase>;
using CoordinationUnitAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CoordinationUnitInterfaceBase>;
class CoordinationUnitIDDataView {
 public:
  CoordinationUnitIDDataView() {}

  CoordinationUnitIDDataView(
      internal::CoordinationUnitID_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitType>(
        data_value, output);
  }

  CoordinationUnitType type() const {
    return static_cast<CoordinationUnitType>(data_->type);
  }
  int64_t id() const {
    return data_->id;
  }
 private:
  internal::CoordinationUnitID_Data* data_ = nullptr;
};

class CoordinationPolicyDataView {
 public:
  CoordinationPolicyDataView() {}

  CoordinationPolicyDataView(
      internal::CoordinationPolicy_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  bool use_background_priority() const {
    return data_->use_background_priority;
  }
 private:
  internal::CoordinationPolicy_Data* data_ = nullptr;
};

class CoordinationPolicyCallback_SetCoordinationPolicy_ParamsDataView {
 public:
  CoordinationPolicyCallback_SetCoordinationPolicy_ParamsDataView() {}

  CoordinationPolicyCallback_SetCoordinationPolicy_ParamsDataView(
      internal::CoordinationPolicyCallback_SetCoordinationPolicy_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPolicyDataView(
      CoordinationPolicyDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPolicy(UserType* output) {
    auto* pointer = data_->policy.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationPolicyDataView>(
        pointer, output, context_);
  }
 private:
  internal::CoordinationPolicyCallback_SetCoordinationPolicy_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CoordinationUnit_SendEvent_ParamsDataView {
 public:
  CoordinationUnit_SendEvent_ParamsDataView() {}

  CoordinationUnit_SendEvent_ParamsDataView(
      internal::CoordinationUnit_SendEvent_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetEventDataView(
      ::resource_coordinator::mojom::EventDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEvent(UserType* output) {
    auto* pointer = data_->event.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::EventDataView>(
        pointer, output, context_);
  }
 private:
  internal::CoordinationUnit_SendEvent_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CoordinationUnit_GetID_ParamsDataView {
 public:
  CoordinationUnit_GetID_ParamsDataView() {}

  CoordinationUnit_GetID_ParamsDataView(
      internal::CoordinationUnit_GetID_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::CoordinationUnit_GetID_Params_Data* data_ = nullptr;
};

class CoordinationUnit_GetID_ResponseParamsDataView {
 public:
  CoordinationUnit_GetID_ResponseParamsDataView() {}

  CoordinationUnit_GetID_ResponseParamsDataView(
      internal::CoordinationUnit_GetID_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::CoordinationUnit_GetID_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CoordinationUnit_AddBinding_ParamsDataView {
 public:
  CoordinationUnit_AddBinding_ParamsDataView() {}

  CoordinationUnit_AddBinding_ParamsDataView(
      internal::CoordinationUnit_AddBinding_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeRequest() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitRequestDataView>(
            &data_->request, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::CoordinationUnit_AddBinding_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CoordinationUnit_AddChild_ParamsDataView {
 public:
  CoordinationUnit_AddChild_ParamsDataView() {}

  CoordinationUnit_AddChild_ParamsDataView(
      internal::CoordinationUnit_AddChild_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetChildIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadChildId(UserType* output) {
    auto* pointer = data_->child_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::CoordinationUnit_AddChild_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CoordinationUnit_RemoveChild_ParamsDataView {
 public:
  CoordinationUnit_RemoveChild_ParamsDataView() {}

  CoordinationUnit_RemoveChild_ParamsDataView(
      internal::CoordinationUnit_RemoveChild_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetChildIdDataView(
      CoordinationUnitIDDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadChildId(UserType* output) {
    auto* pointer = data_->child_id.Get();
    return mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationUnitIDDataView>(
        pointer, output, context_);
  }
 private:
  internal::CoordinationUnit_RemoveChild_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CoordinationUnit_SetCoordinationPolicyCallback_ParamsDataView {
 public:
  CoordinationUnit_SetCoordinationPolicyCallback_ParamsDataView() {}

  CoordinationUnit_SetCoordinationPolicyCallback_ParamsDataView(
      internal::CoordinationUnit_SetCoordinationPolicyCallback_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeCallback() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::resource_coordinator::mojom::CoordinationPolicyCallbackPtrDataView>(
            &data_->callback, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::CoordinationUnit_SetCoordinationPolicyCallback_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class CoordinationUnit_SetProperty_ParamsDataView {
 public:
  CoordinationUnit_SetProperty_ParamsDataView() {}

  CoordinationUnit_SetProperty_ParamsDataView(
      internal::CoordinationUnit_SetProperty_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPropertyType(UserType* output) const {
    auto data_value = data_->property_type;
    return mojo::internal::Deserialize<::resource_coordinator::mojom::PropertyType>(
        data_value, output);
  }

  PropertyType property_type() const {
    return static_cast<PropertyType>(data_->property_type);
  }
  inline void GetValueDataView(
      ::mojo::common::mojom::ValueDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadValue(UserType* output) {
    auto* pointer = &data_->value;
    return mojo::internal::Deserialize<::mojo::common::mojom::ValueDataView>(
        pointer, output, context_);
  }
 private:
  internal::CoordinationUnit_SetProperty_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace resource_coordinator

namespace std {

template <>
struct hash<::resource_coordinator::mojom::CoordinationUnitType>
    : public mojo::internal::EnumHashImpl<::resource_coordinator::mojom::CoordinationUnitType> {};

template <>
struct hash<::resource_coordinator::mojom::PropertyType>
    : public mojo::internal::EnumHashImpl<::resource_coordinator::mojom::PropertyType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::resource_coordinator::mojom::CoordinationUnitType, ::resource_coordinator::mojom::CoordinationUnitType> {
  static ::resource_coordinator::mojom::CoordinationUnitType ToMojom(::resource_coordinator::mojom::CoordinationUnitType input) { return input; }
  static bool FromMojom(::resource_coordinator::mojom::CoordinationUnitType input, ::resource_coordinator::mojom::CoordinationUnitType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::CoordinationUnitType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::resource_coordinator::mojom::CoordinationUnitType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::resource_coordinator::mojom::CoordinationUnitType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::resource_coordinator::mojom::PropertyType, ::resource_coordinator::mojom::PropertyType> {
  static ::resource_coordinator::mojom::PropertyType ToMojom(::resource_coordinator::mojom::PropertyType input) { return input; }
  static bool FromMojom(::resource_coordinator::mojom::PropertyType input, ::resource_coordinator::mojom::PropertyType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::PropertyType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::resource_coordinator::mojom::PropertyType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::resource_coordinator::mojom::PropertyType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::CoordinationUnitIDDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::CoordinationUnitIDDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::resource_coordinator::mojom::internal::CoordinationUnitID_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::CoordinationUnitID_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::resource_coordinator::mojom::internal::CoordinationUnitID_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    mojo::internal::Serialize<::resource_coordinator::mojom::CoordinationUnitType>(
        CallWithContext(Traits::type, input, custom_context), &result->type);
    result->id = CallWithContext(Traits::id, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::CoordinationUnitID_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::CoordinationUnitIDDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::resource_coordinator::mojom::CoordinationPolicyDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::resource_coordinator::mojom::CoordinationPolicyDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::resource_coordinator::mojom::internal::CoordinationPolicy_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::resource_coordinator::mojom::internal::CoordinationPolicy_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::resource_coordinator::mojom::internal::CoordinationPolicy_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->use_background_priority = CallWithContext(Traits::use_background_priority, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::resource_coordinator::mojom::internal::CoordinationPolicy_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::resource_coordinator::mojom::CoordinationPolicyDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace resource_coordinator {
namespace mojom {





inline void CoordinationPolicyCallback_SetCoordinationPolicy_ParamsDataView::GetPolicyDataView(
    CoordinationPolicyDataView* output) {
  auto pointer = data_->policy.Get();
  *output = CoordinationPolicyDataView(pointer, context_);
}


inline void CoordinationUnit_SendEvent_ParamsDataView::GetEventDataView(
    ::resource_coordinator::mojom::EventDataView* output) {
  auto pointer = data_->event.Get();
  *output = ::resource_coordinator::mojom::EventDataView(pointer, context_);
}




inline void CoordinationUnit_GetID_ResponseParamsDataView::GetIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}




inline void CoordinationUnit_AddChild_ParamsDataView::GetChildIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->child_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}


inline void CoordinationUnit_RemoveChild_ParamsDataView::GetChildIdDataView(
    CoordinationUnitIDDataView* output) {
  auto pointer = data_->child_id.Get();
  *output = CoordinationUnitIDDataView(pointer, context_);
}




inline void CoordinationUnit_SetProperty_ParamsDataView::GetValueDataView(
    ::mojo::common::mojom::ValueDataView* output) {
  auto pointer = &data_->value;
  *output = ::mojo::common::mojom::ValueDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace resource_coordinator

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_SHARED_H_
