// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/coordination_unit.mojom-shared.h"
#include "services/resource_coordinator/public/interfaces/events.mojom.h"
#include "mojo/common/values.mojom.h"
#include <string>
#include <vector>
#include "services/resource_coordinator/public/cpp/coordination_unit_id.h"
#include "services/resource_coordinator/public/cpp/coordination_unit_types.h"
#include "services/resource_coordinator/public/cpp/resource_coordinator_export.h"


namespace resource_coordinator {
namespace mojom {
class CoordinationPolicyCallback;
using CoordinationPolicyCallbackPtr = mojo::InterfacePtr<CoordinationPolicyCallback>;
using CoordinationPolicyCallbackPtrInfo = mojo::InterfacePtrInfo<CoordinationPolicyCallback>;
using ThreadSafeCoordinationPolicyCallbackPtr =
    mojo::ThreadSafeInterfacePtr<CoordinationPolicyCallback>;
using CoordinationPolicyCallbackRequest = mojo::InterfaceRequest<CoordinationPolicyCallback>;
using CoordinationPolicyCallbackAssociatedPtr =
    mojo::AssociatedInterfacePtr<CoordinationPolicyCallback>;
using ThreadSafeCoordinationPolicyCallbackAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CoordinationPolicyCallback>;
using CoordinationPolicyCallbackAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CoordinationPolicyCallback>;
using CoordinationPolicyCallbackAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CoordinationPolicyCallback>;

class CoordinationUnit;
using CoordinationUnitPtr = mojo::InterfacePtr<CoordinationUnit>;
using CoordinationUnitPtrInfo = mojo::InterfacePtrInfo<CoordinationUnit>;
using ThreadSafeCoordinationUnitPtr =
    mojo::ThreadSafeInterfacePtr<CoordinationUnit>;
using CoordinationUnitRequest = mojo::InterfaceRequest<CoordinationUnit>;
using CoordinationUnitAssociatedPtr =
    mojo::AssociatedInterfacePtr<CoordinationUnit>;
using ThreadSafeCoordinationUnitAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CoordinationUnit>;
using CoordinationUnitAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CoordinationUnit>;
using CoordinationUnitAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CoordinationUnit>;

class CoordinationUnitID;
using CoordinationUnitIDPtr = mojo::InlinedStructPtr<CoordinationUnitID>;

class CoordinationPolicy;
using CoordinationPolicyPtr = mojo::InlinedStructPtr<CoordinationPolicy>;


class CoordinationPolicyCallbackProxy;

template <typename ImplRefTraits>
class CoordinationPolicyCallbackStub;

class CoordinationPolicyCallbackRequestValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationPolicyCallback
    : public CoordinationPolicyCallbackInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CoordinationPolicyCallbackProxy;

  template <typename ImplRefTraits>
  using Stub_ = CoordinationPolicyCallbackStub<ImplRefTraits>;

  using RequestValidator_ = CoordinationPolicyCallbackRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kSetCoordinationPolicyMinVersion = 0,
  };
  virtual ~CoordinationPolicyCallback() {}

  virtual void SetCoordinationPolicy(CoordinationPolicyPtr policy) = 0;
};

class CoordinationUnitProxy;

template <typename ImplRefTraits>
class CoordinationUnitStub;

class CoordinationUnitRequestValidator;
class CoordinationUnitResponseValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnit
    : public CoordinationUnitInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CoordinationUnitProxy;

  template <typename ImplRefTraits>
  using Stub_ = CoordinationUnitStub<ImplRefTraits>;

  using RequestValidator_ = CoordinationUnitRequestValidator;
  using ResponseValidator_ = CoordinationUnitResponseValidator;
  enum MethodMinVersions : uint32_t {
    kSendEventMinVersion = 0,
    kGetIDMinVersion = 0,
    kAddBindingMinVersion = 0,
    kAddChildMinVersion = 0,
    kRemoveChildMinVersion = 0,
    kSetCoordinationPolicyCallbackMinVersion = 0,
    kSetPropertyMinVersion = 0,
  };
  virtual ~CoordinationUnit() {}

  virtual void SendEvent(::resource_coordinator::mojom::EventPtr event) = 0;


  using GetIDCallback = base::Callback<void(const ::resource_coordinator::CoordinationUnitID&)>;
  virtual void GetID(const GetIDCallback& callback) = 0;

  virtual void AddBinding(CoordinationUnitRequest request) = 0;

  virtual void AddChild(const ::resource_coordinator::CoordinationUnitID& child_id) = 0;

  virtual void RemoveChild(const ::resource_coordinator::CoordinationUnitID& child_id) = 0;

  virtual void SetCoordinationPolicyCallback(CoordinationPolicyCallbackPtr callback) = 0;

  virtual void SetProperty(PropertyType property_type, std::unique_ptr<base::Value> value) = 0;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationPolicyCallbackProxy
    : public CoordinationPolicyCallback {
 public:
  explicit CoordinationPolicyCallbackProxy(mojo::MessageReceiverWithResponder* receiver);
  void SetCoordinationPolicy(CoordinationPolicyPtr policy) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitProxy
    : public CoordinationUnit {
 public:
  explicit CoordinationUnitProxy(mojo::MessageReceiverWithResponder* receiver);
  void SendEvent(::resource_coordinator::mojom::EventPtr event) override;
  void GetID(const GetIDCallback& callback) override;
  void AddBinding(CoordinationUnitRequest request) override;
  void AddChild(const ::resource_coordinator::CoordinationUnitID& child_id) override;
  void RemoveChild(const ::resource_coordinator::CoordinationUnitID& child_id) override;
  void SetCoordinationPolicyCallback(CoordinationPolicyCallbackPtr callback) override;
  void SetProperty(PropertyType property_type, std::unique_ptr<base::Value> value) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationPolicyCallbackStubDispatch {
 public:
  static bool Accept(CoordinationPolicyCallback* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CoordinationPolicyCallback* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CoordinationPolicyCallback>>
class CoordinationPolicyCallbackStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CoordinationPolicyCallbackStub() {}
  ~CoordinationPolicyCallbackStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinationPolicyCallbackStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinationPolicyCallbackStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitStubDispatch {
 public:
  static bool Accept(CoordinationUnit* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CoordinationUnit* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CoordinationUnit>>
class CoordinationUnitStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CoordinationUnitStub() {}
  ~CoordinationUnitStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinationUnitStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinationUnitStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationPolicyCallbackRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};


class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitID {
 public:
  using DataView = CoordinationUnitIDDataView;
  using Data_ = internal::CoordinationUnitID_Data;

  template <typename... Args>
  static CoordinationUnitIDPtr New(Args&&... args) {
    return CoordinationUnitIDPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CoordinationUnitIDPtr From(const U& u) {
    return mojo::TypeConverter<CoordinationUnitIDPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CoordinationUnitID>::Convert(*this);
  }


  CoordinationUnitID();

  CoordinationUnitID(
      ::resource_coordinator::CoordinationUnitType type,
      int64_t id);

  ~CoordinationUnitID();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CoordinationUnitIDPtr>
  CoordinationUnitIDPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CoordinationUnitID>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CoordinationUnitID::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CoordinationUnitID::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  ::resource_coordinator::CoordinationUnitType type;
  int64_t id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationPolicy {
 public:
  using DataView = CoordinationPolicyDataView;
  using Data_ = internal::CoordinationPolicy_Data;

  template <typename... Args>
  static CoordinationPolicyPtr New(Args&&... args) {
    return CoordinationPolicyPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static CoordinationPolicyPtr From(const U& u) {
    return mojo::TypeConverter<CoordinationPolicyPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, CoordinationPolicy>::Convert(*this);
  }


  CoordinationPolicy();

  explicit CoordinationPolicy(
      bool use_background_priority);

  ~CoordinationPolicy();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = CoordinationPolicyPtr>
  CoordinationPolicyPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, CoordinationPolicy>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        CoordinationPolicy::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        CoordinationPolicy::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  bool use_background_priority;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};






template <typename StructPtrType>
CoordinationUnitIDPtr CoordinationUnitID::Clone() const {
  return New(
      mojo::Clone(type),
      mojo::Clone(id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CoordinationUnitID>::value>::type*>
bool CoordinationUnitID::Equals(const T& other) const {
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->id, other.id))
    return false;
  return true;
}
template <typename StructPtrType>
CoordinationPolicyPtr CoordinationPolicy::Clone() const {
  return New(
      mojo::Clone(use_background_priority)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, CoordinationPolicy>::value>::type*>
bool CoordinationPolicy::Equals(const T& other) const {
  if (!mojo::Equals(this->use_background_priority, other.use_background_priority))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::resource_coordinator::mojom::CoordinationUnitID::DataView,
                                         ::resource_coordinator::mojom::CoordinationUnitIDPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::CoordinationUnitIDPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::CoordinationUnitID::type) type(
      const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) {
    return input->type;
  }

  static decltype(::resource_coordinator::mojom::CoordinationUnitID::id) id(
      const ::resource_coordinator::mojom::CoordinationUnitIDPtr& input) {
    return input->id;
  }

  static bool Read(::resource_coordinator::mojom::CoordinationUnitID::DataView input, ::resource_coordinator::mojom::CoordinationUnitIDPtr* output);
};


template <>
struct SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT StructTraits<::resource_coordinator::mojom::CoordinationPolicy::DataView,
                                         ::resource_coordinator::mojom::CoordinationPolicyPtr> {
  static bool IsNull(const ::resource_coordinator::mojom::CoordinationPolicyPtr& input) { return !input; }
  static void SetToNull(::resource_coordinator::mojom::CoordinationPolicyPtr* output) { output->reset(); }

  static decltype(::resource_coordinator::mojom::CoordinationPolicy::use_background_priority) use_background_priority(
      const ::resource_coordinator::mojom::CoordinationPolicyPtr& input) {
    return input->use_background_priority;
  }

  static bool Read(::resource_coordinator::mojom::CoordinationPolicy::DataView input, ::resource_coordinator::mojom::CoordinationPolicyPtr* output);
};

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_MOJOM_H_