// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_PROVIDER_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_PROVIDER_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/coordination_unit_provider.mojom-shared.h"
#include "services/resource_coordinator/public/interfaces/coordination_unit.mojom.h"
#include <string>
#include <vector>
#include "services/resource_coordinator/public/cpp/resource_coordinator_export.h"


namespace resource_coordinator {
namespace mojom {
class CoordinationUnitProvider;
using CoordinationUnitProviderPtr = mojo::InterfacePtr<CoordinationUnitProvider>;
using CoordinationUnitProviderPtrInfo = mojo::InterfacePtrInfo<CoordinationUnitProvider>;
using ThreadSafeCoordinationUnitProviderPtr =
    mojo::ThreadSafeInterfacePtr<CoordinationUnitProvider>;
using CoordinationUnitProviderRequest = mojo::InterfaceRequest<CoordinationUnitProvider>;
using CoordinationUnitProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<CoordinationUnitProvider>;
using ThreadSafeCoordinationUnitProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<CoordinationUnitProvider>;
using CoordinationUnitProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<CoordinationUnitProvider>;
using CoordinationUnitProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<CoordinationUnitProvider>;


class CoordinationUnitProviderProxy;

template <typename ImplRefTraits>
class CoordinationUnitProviderStub;

class CoordinationUnitProviderRequestValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitProvider
    : public CoordinationUnitProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = CoordinationUnitProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = CoordinationUnitProviderStub<ImplRefTraits>;

  using RequestValidator_ = CoordinationUnitProviderRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kCreateCoordinationUnitMinVersion = 0,
  };
  virtual ~CoordinationUnitProvider() {}

  virtual void CreateCoordinationUnit(::resource_coordinator::mojom::CoordinationUnitRequest request, const ::resource_coordinator::CoordinationUnitID& id) = 0;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitProviderProxy
    : public CoordinationUnitProvider {
 public:
  explicit CoordinationUnitProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void CreateCoordinationUnit(::resource_coordinator::mojom::CoordinationUnitRequest request, const ::resource_coordinator::CoordinationUnitID& id) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitProviderStubDispatch {
 public:
  static bool Accept(CoordinationUnitProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      CoordinationUnitProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<CoordinationUnitProvider>>
class CoordinationUnitProviderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  CoordinationUnitProviderStub() {}
  ~CoordinationUnitProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinationUnitProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return CoordinationUnitProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT CoordinationUnitProviderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_COORDINATION_UNIT_PROVIDER_MOJOM_H_