// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "services/resource_coordinator/public/interfaces/memory_instrumentation/memory_instrumentation.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace memory_instrumentation {
namespace mojom {

namespace internal {


// static
RequestArgs_Data* RequestArgs_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(RequestArgs_Data))) RequestArgs_Data();
}

// static
bool RequestArgs_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const RequestArgs_Data* object = static_cast<const RequestArgs_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::memory_instrumentation::mojom::internal::DumpType_Data
        ::Validate(object->dump_type, validation_context))
    return false;


  if (!::memory_instrumentation::mojom::internal::LevelOfDetail_Data
        ::Validate(object->level_of_detail, validation_context))
    return false;

  return true;
}

RequestArgs_Data::RequestArgs_Data()
    : header_({sizeof(*this), 0}) {}


// static
ChromeMemDump_Data* ChromeMemDump_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ChromeMemDump_Data))) ChromeMemDump_Data();
}

// static
bool ChromeMemDump_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ChromeMemDump_Data* object = static_cast<const ChromeMemDump_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

ChromeMemDump_Data::ChromeMemDump_Data()
    : header_({sizeof(*this), 0}) {}


// static
PlatformPrivateFootprint_Data* PlatformPrivateFootprint_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PlatformPrivateFootprint_Data))) PlatformPrivateFootprint_Data();
}

// static
bool PlatformPrivateFootprint_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PlatformPrivateFootprint_Data* object = static_cast<const PlatformPrivateFootprint_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 56 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

PlatformPrivateFootprint_Data::PlatformPrivateFootprint_Data()
    : header_({sizeof(*this), 0}) {}


// static
RawOSMemDump_Data* RawOSMemDump_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(RawOSMemDump_Data))) RawOSMemDump_Data();
}

// static
bool RawOSMemDump_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const RawOSMemDump_Data* object = static_cast<const RawOSMemDump_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->platform_private_footprint, "null platform_private_footprint field in RawOSMemDump",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->platform_private_footprint, validation_context))
    return false;

  return true;
}

RawOSMemDump_Data::RawOSMemDump_Data()
    : header_({sizeof(*this), 0}) {}


// static
RawProcessMemoryDump_Data* RawProcessMemoryDump_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(RawProcessMemoryDump_Data))) RawProcessMemoryDump_Data();
}

// static
bool RawProcessMemoryDump_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const RawProcessMemoryDump_Data* object = static_cast<const RawProcessMemoryDump_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->os_dump, "null os_dump field in RawProcessMemoryDump",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->os_dump, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->chrome_dump, "null chrome_dump field in RawProcessMemoryDump",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->chrome_dump, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->extra_processes_dumps, "null extra_processes_dumps field in RawProcessMemoryDump",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams extra_processes_dumps_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, nullptr), new mojo::internal::ContainerValidateParams(0, false, nullptr));
  if (!mojo::internal::ValidateContainer(object->extra_processes_dumps, validation_context,
                                         &extra_processes_dumps_validate_params)) {
    return false;
  }

  return true;
}

RawProcessMemoryDump_Data::RawProcessMemoryDump_Data()
    : header_({sizeof(*this), 0}) {}


// static
OSMemDump_Data* OSMemDump_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(OSMemDump_Data))) OSMemDump_Data();
}

// static
bool OSMemDump_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const OSMemDump_Data* object = static_cast<const OSMemDump_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

OSMemDump_Data::OSMemDump_Data()
    : header_({sizeof(*this), 0}) {}


// static
ProcessMemoryDump_Data* ProcessMemoryDump_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ProcessMemoryDump_Data))) ProcessMemoryDump_Data();
}

// static
bool ProcessMemoryDump_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ProcessMemoryDump_Data* object = static_cast<const ProcessMemoryDump_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::memory_instrumentation::mojom::internal::ProcessType_Data
        ::Validate(object->process_type, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->os_dump, "null os_dump field in ProcessMemoryDump",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->os_dump, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->chrome_dump, "null chrome_dump field in ProcessMemoryDump",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->chrome_dump, validation_context))
    return false;

  return true;
}

ProcessMemoryDump_Data::ProcessMemoryDump_Data()
    : header_({sizeof(*this), 0}) {}


// static
GlobalMemoryDump_Data* GlobalMemoryDump_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(GlobalMemoryDump_Data))) GlobalMemoryDump_Data();
}

// static
bool GlobalMemoryDump_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const GlobalMemoryDump_Data* object = static_cast<const GlobalMemoryDump_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->process_dumps, "null process_dumps field in GlobalMemoryDump",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams process_dumps_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->process_dumps, validation_context,
                                         &process_dumps_validate_params)) {
    return false;
  }

  return true;
}

GlobalMemoryDump_Data::GlobalMemoryDump_Data()
    : header_({sizeof(*this), 0}) {}


// static
ClientProcess_RequestProcessMemoryDump_Params_Data* ClientProcess_RequestProcessMemoryDump_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ClientProcess_RequestProcessMemoryDump_Params_Data))) ClientProcess_RequestProcessMemoryDump_Params_Data();
}

// static
bool ClientProcess_RequestProcessMemoryDump_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ClientProcess_RequestProcessMemoryDump_Params_Data* object = static_cast<const ClientProcess_RequestProcessMemoryDump_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->args, "null args field in ClientProcess_RequestProcessMemoryDump_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->args, validation_context))
    return false;

  return true;
}

ClientProcess_RequestProcessMemoryDump_Params_Data::ClientProcess_RequestProcessMemoryDump_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ClientProcess_RequestProcessMemoryDump_ResponseParams_Data* ClientProcess_RequestProcessMemoryDump_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ClientProcess_RequestProcessMemoryDump_ResponseParams_Data))) ClientProcess_RequestProcessMemoryDump_ResponseParams_Data();
}

// static
bool ClientProcess_RequestProcessMemoryDump_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ClientProcess_RequestProcessMemoryDump_ResponseParams_Data* object = static_cast<const ClientProcess_RequestProcessMemoryDump_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->process_memory_dump, validation_context))
    return false;

  return true;
}

ClientProcess_RequestProcessMemoryDump_ResponseParams_Data::ClientProcess_RequestProcessMemoryDump_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
ClientProcess_RequestOSMemoryDump_Params_Data* ClientProcess_RequestOSMemoryDump_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ClientProcess_RequestOSMemoryDump_Params_Data))) ClientProcess_RequestOSMemoryDump_Params_Data();
}

// static
bool ClientProcess_RequestOSMemoryDump_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ClientProcess_RequestOSMemoryDump_Params_Data* object = static_cast<const ClientProcess_RequestOSMemoryDump_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->pids, "null pids field in ClientProcess_RequestOSMemoryDump_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams pids_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->pids, validation_context,
                                         &pids_validate_params)) {
    return false;
  }

  return true;
}

ClientProcess_RequestOSMemoryDump_Params_Data::ClientProcess_RequestOSMemoryDump_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
ClientProcess_RequestOSMemoryDump_ResponseParams_Data* ClientProcess_RequestOSMemoryDump_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(ClientProcess_RequestOSMemoryDump_ResponseParams_Data))) ClientProcess_RequestOSMemoryDump_ResponseParams_Data();
}

// static
bool ClientProcess_RequestOSMemoryDump_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const ClientProcess_RequestOSMemoryDump_ResponseParams_Data* object = static_cast<const ClientProcess_RequestOSMemoryDump_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dumps, "null dumps field in ClientProcess_RequestOSMemoryDump_ResponseParams",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams dumps_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, nullptr), new mojo::internal::ContainerValidateParams(0, false, nullptr));
  if (!mojo::internal::ValidateContainer(object->dumps, validation_context,
                                         &dumps_validate_params)) {
    return false;
  }

  return true;
}

ClientProcess_RequestOSMemoryDump_ResponseParams_Data::ClientProcess_RequestOSMemoryDump_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_RegisterClientProcess_Params_Data* Coordinator_RegisterClientProcess_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_RegisterClientProcess_Params_Data))) Coordinator_RegisterClientProcess_Params_Data();
}

// static
bool Coordinator_RegisterClientProcess_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_RegisterClientProcess_Params_Data* object = static_cast<const Coordinator_RegisterClientProcess_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->client_process,
          "invalid client_process field in Coordinator_RegisterClientProcess_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->client_process,
                                                 validation_context)) {
    return false;
  }


  if (!::memory_instrumentation::mojom::internal::ProcessType_Data
        ::Validate(object->process_type, validation_context))
    return false;

  return true;
}

Coordinator_RegisterClientProcess_Params_Data::Coordinator_RegisterClientProcess_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_RequestGlobalMemoryDump_Params_Data* Coordinator_RequestGlobalMemoryDump_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_RequestGlobalMemoryDump_Params_Data))) Coordinator_RequestGlobalMemoryDump_Params_Data();
}

// static
bool Coordinator_RequestGlobalMemoryDump_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_RequestGlobalMemoryDump_Params_Data* object = static_cast<const Coordinator_RequestGlobalMemoryDump_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->args, "null args field in Coordinator_RequestGlobalMemoryDump_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->args, validation_context))
    return false;

  return true;
}

Coordinator_RequestGlobalMemoryDump_Params_Data::Coordinator_RequestGlobalMemoryDump_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
Coordinator_RequestGlobalMemoryDump_ResponseParams_Data* Coordinator_RequestGlobalMemoryDump_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(Coordinator_RequestGlobalMemoryDump_ResponseParams_Data))) Coordinator_RequestGlobalMemoryDump_ResponseParams_Data();
}

// static
bool Coordinator_RequestGlobalMemoryDump_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const Coordinator_RequestGlobalMemoryDump_ResponseParams_Data* object = static_cast<const Coordinator_RequestGlobalMemoryDump_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->global_memory_dump, validation_context))
    return false;

  return true;
}

Coordinator_RequestGlobalMemoryDump_ResponseParams_Data::Coordinator_RequestGlobalMemoryDump_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace memory_instrumentation

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
