// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_SHARED_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/resource_coordinator/public/interfaces/memory_instrumentation/memory_instrumentation.mojom-shared-internal.h"
#include "mojo/common/process_id.mojom-shared.h"
#include "services/resource_coordinator/public/interfaces/interfaces_internal_shared_export.h"


namespace memory_instrumentation {
namespace mojom {
class RequestArgsDataView;

class ChromeMemDumpDataView;

class PlatformPrivateFootprintDataView;

class RawOSMemDumpDataView;

class RawProcessMemoryDumpDataView;

class OSMemDumpDataView;

class ProcessMemoryDumpDataView;

class GlobalMemoryDumpDataView;



}  // namespace mojom
}  // namespace memory_instrumentation

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::RequestArgsDataView> {
  using Data = ::memory_instrumentation::mojom::internal::RequestArgs_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::ChromeMemDumpDataView> {
  using Data = ::memory_instrumentation::mojom::internal::ChromeMemDump_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::PlatformPrivateFootprintDataView> {
  using Data = ::memory_instrumentation::mojom::internal::PlatformPrivateFootprint_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::RawOSMemDumpDataView> {
  using Data = ::memory_instrumentation::mojom::internal::RawOSMemDump_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::RawProcessMemoryDumpDataView> {
  using Data = ::memory_instrumentation::mojom::internal::RawProcessMemoryDump_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::OSMemDumpDataView> {
  using Data = ::memory_instrumentation::mojom::internal::OSMemDump_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::ProcessMemoryDumpDataView> {
  using Data = ::memory_instrumentation::mojom::internal::ProcessMemoryDump_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::memory_instrumentation::mojom::GlobalMemoryDumpDataView> {
  using Data = ::memory_instrumentation::mojom::internal::GlobalMemoryDump_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace memory_instrumentation {
namespace mojom {

enum class DumpType : int32_t {
  PERIODIC_INTERVAL,
  EXPLICITLY_TRIGGERED,
  PEAK_MEMORY_USAGE,
  SUMMARY_ONLY,
};

inline std::ostream& operator<<(std::ostream& os, DumpType value) {
  switch(value) {
    case DumpType::PERIODIC_INTERVAL:
      return os << "DumpType::PERIODIC_INTERVAL";
    case DumpType::EXPLICITLY_TRIGGERED:
      return os << "DumpType::EXPLICITLY_TRIGGERED";
    case DumpType::PEAK_MEMORY_USAGE:
      return os << "DumpType::PEAK_MEMORY_USAGE";
    case DumpType::SUMMARY_ONLY:
      return os << "DumpType::SUMMARY_ONLY";
    default:
      return os << "Unknown DumpType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(DumpType value) {
  return internal::DumpType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class LevelOfDetail : int32_t {
  BACKGROUND,
  LIGHT,
  DETAILED,
};

inline std::ostream& operator<<(std::ostream& os, LevelOfDetail value) {
  switch(value) {
    case LevelOfDetail::BACKGROUND:
      return os << "LevelOfDetail::BACKGROUND";
    case LevelOfDetail::LIGHT:
      return os << "LevelOfDetail::LIGHT";
    case LevelOfDetail::DETAILED:
      return os << "LevelOfDetail::DETAILED";
    default:
      return os << "Unknown LevelOfDetail value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(LevelOfDetail value) {
  return internal::LevelOfDetail_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class ProcessType : int32_t {
  OTHER,
  BROWSER,
  RENDERER,
  GPU,
  UTILITY,
  PLUGIN,
};

inline std::ostream& operator<<(std::ostream& os, ProcessType value) {
  switch(value) {
    case ProcessType::OTHER:
      return os << "ProcessType::OTHER";
    case ProcessType::BROWSER:
      return os << "ProcessType::BROWSER";
    case ProcessType::RENDERER:
      return os << "ProcessType::RENDERER";
    case ProcessType::GPU:
      return os << "ProcessType::GPU";
    case ProcessType::UTILITY:
      return os << "ProcessType::UTILITY";
    case ProcessType::PLUGIN:
      return os << "ProcessType::PLUGIN";
    default:
      return os << "Unknown ProcessType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(ProcessType value) {
  return internal::ProcessType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class ClientProcessInterfaceBase {};

using ClientProcessPtrDataView =
    mojo::InterfacePtrDataView<ClientProcessInterfaceBase>;
using ClientProcessRequestDataView =
    mojo::InterfaceRequestDataView<ClientProcessInterfaceBase>;
using ClientProcessAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<ClientProcessInterfaceBase>;
using ClientProcessAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<ClientProcessInterfaceBase>;
class CoordinatorInterfaceBase {};

using CoordinatorPtrDataView =
    mojo::InterfacePtrDataView<CoordinatorInterfaceBase>;
using CoordinatorRequestDataView =
    mojo::InterfaceRequestDataView<CoordinatorInterfaceBase>;
using CoordinatorAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<CoordinatorInterfaceBase>;
using CoordinatorAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<CoordinatorInterfaceBase>;
class RequestArgsDataView {
 public:
  RequestArgsDataView() {}

  RequestArgsDataView(
      internal::RequestArgs_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint64_t dump_guid() const {
    return data_->dump_guid;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDumpType(UserType* output) const {
    auto data_value = data_->dump_type;
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::DumpType>(
        data_value, output);
  }

  DumpType dump_type() const {
    return static_cast<DumpType>(data_->dump_type);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadLevelOfDetail(UserType* output) const {
    auto data_value = data_->level_of_detail;
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::LevelOfDetail>(
        data_value, output);
  }

  LevelOfDetail level_of_detail() const {
    return static_cast<LevelOfDetail>(data_->level_of_detail);
  }
 private:
  internal::RequestArgs_Data* data_ = nullptr;
};

class ChromeMemDumpDataView {
 public:
  ChromeMemDumpDataView() {}

  ChromeMemDumpDataView(
      internal::ChromeMemDump_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t malloc_total_kb() const {
    return data_->malloc_total_kb;
  }
  uint32_t command_buffer_total_kb() const {
    return data_->command_buffer_total_kb;
  }
  uint32_t partition_alloc_total_kb() const {
    return data_->partition_alloc_total_kb;
  }
  uint32_t blink_gc_total_kb() const {
    return data_->blink_gc_total_kb;
  }
  uint32_t v8_total_kb() const {
    return data_->v8_total_kb;
  }
 private:
  internal::ChromeMemDump_Data* data_ = nullptr;
};

class PlatformPrivateFootprintDataView {
 public:
  PlatformPrivateFootprintDataView() {}

  PlatformPrivateFootprintDataView(
      internal::PlatformPrivateFootprint_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint64_t phys_footprint_bytes() const {
    return data_->phys_footprint_bytes;
  }
  uint64_t internal_bytes() const {
    return data_->internal_bytes;
  }
  uint64_t compressed_bytes() const {
    return data_->compressed_bytes;
  }
  uint64_t rss_anon_bytes() const {
    return data_->rss_anon_bytes;
  }
  uint64_t vm_swap_bytes() const {
    return data_->vm_swap_bytes;
  }
  uint64_t private_bytes() const {
    return data_->private_bytes;
  }
 private:
  internal::PlatformPrivateFootprint_Data* data_ = nullptr;
};

class RawOSMemDumpDataView {
 public:
  RawOSMemDumpDataView() {}

  RawOSMemDumpDataView(
      internal::RawOSMemDump_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t resident_set_kb() const {
    return data_->resident_set_kb;
  }
  inline void GetPlatformPrivateFootprintDataView(
      PlatformPrivateFootprintDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPlatformPrivateFootprint(UserType* output) {
    auto* pointer = data_->platform_private_footprint.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::PlatformPrivateFootprintDataView>(
        pointer, output, context_);
  }
 private:
  internal::RawOSMemDump_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class RawProcessMemoryDumpDataView {
 public:
  RawProcessMemoryDumpDataView() {}

  RawProcessMemoryDumpDataView(
      internal::RawProcessMemoryDump_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOsDumpDataView(
      RawOSMemDumpDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOsDump(UserType* output) {
    auto* pointer = data_->os_dump.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::RawOSMemDumpDataView>(
        pointer, output, context_);
  }
  inline void GetChromeDumpDataView(
      ChromeMemDumpDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadChromeDump(UserType* output) {
    auto* pointer = data_->chrome_dump.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::ChromeMemDumpDataView>(
        pointer, output, context_);
  }
  inline void GetExtraProcessesDumpsDataView(
      mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, RawOSMemDumpDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExtraProcessesDumps(UserType* output) {
    auto* pointer = data_->extra_processes_dumps.Get();
    return mojo::internal::Deserialize<mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, ::memory_instrumentation::mojom::RawOSMemDumpDataView>>(
        pointer, output, context_);
  }
 private:
  internal::RawProcessMemoryDump_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class OSMemDumpDataView {
 public:
  OSMemDumpDataView() {}

  OSMemDumpDataView(
      internal::OSMemDump_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  uint32_t resident_set_kb() const {
    return data_->resident_set_kb;
  }
  uint32_t private_footprint_kb() const {
    return data_->private_footprint_kb;
  }
 private:
  internal::OSMemDump_Data* data_ = nullptr;
};

class ProcessMemoryDumpDataView {
 public:
  ProcessMemoryDumpDataView() {}

  ProcessMemoryDumpDataView(
      internal::ProcessMemoryDump_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadProcessType(UserType* output) const {
    auto data_value = data_->process_type;
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::ProcessType>(
        data_value, output);
  }

  ProcessType process_type() const {
    return static_cast<ProcessType>(data_->process_type);
  }
  inline void GetOsDumpDataView(
      OSMemDumpDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOsDump(UserType* output) {
    auto* pointer = data_->os_dump.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::OSMemDumpDataView>(
        pointer, output, context_);
  }
  inline void GetChromeDumpDataView(
      ChromeMemDumpDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadChromeDump(UserType* output) {
    auto* pointer = data_->chrome_dump.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::ChromeMemDumpDataView>(
        pointer, output, context_);
  }
 private:
  internal::ProcessMemoryDump_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class GlobalMemoryDumpDataView {
 public:
  GlobalMemoryDumpDataView() {}

  GlobalMemoryDumpDataView(
      internal::GlobalMemoryDump_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetProcessDumpsDataView(
      mojo::ArrayDataView<ProcessMemoryDumpDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadProcessDumps(UserType* output) {
    auto* pointer = data_->process_dumps.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::memory_instrumentation::mojom::ProcessMemoryDumpDataView>>(
        pointer, output, context_);
  }
 private:
  internal::GlobalMemoryDump_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClientProcess_RequestProcessMemoryDump_ParamsDataView {
 public:
  ClientProcess_RequestProcessMemoryDump_ParamsDataView() {}

  ClientProcess_RequestProcessMemoryDump_ParamsDataView(
      internal::ClientProcess_RequestProcessMemoryDump_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetArgsDataView(
      RequestArgsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadArgs(UserType* output) {
    auto* pointer = data_->args.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::RequestArgsDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClientProcess_RequestProcessMemoryDump_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClientProcess_RequestProcessMemoryDump_ResponseParamsDataView {
 public:
  ClientProcess_RequestProcessMemoryDump_ResponseParamsDataView() {}

  ClientProcess_RequestProcessMemoryDump_ResponseParamsDataView(
      internal::ClientProcess_RequestProcessMemoryDump_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
  uint64_t dump_id() const {
    return data_->dump_id;
  }
  inline void GetProcessMemoryDumpDataView(
      RawProcessMemoryDumpDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadProcessMemoryDump(UserType* output) {
    auto* pointer = data_->process_memory_dump.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::RawProcessMemoryDumpDataView>(
        pointer, output, context_);
  }
 private:
  internal::ClientProcess_RequestProcessMemoryDump_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClientProcess_RequestOSMemoryDump_ParamsDataView {
 public:
  ClientProcess_RequestOSMemoryDump_ParamsDataView() {}

  ClientProcess_RequestOSMemoryDump_ParamsDataView(
      internal::ClientProcess_RequestOSMemoryDump_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPidsDataView(
      mojo::ArrayDataView<::mojo::common::mojom::ProcessIdDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPids(UserType* output) {
    auto* pointer = data_->pids.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::mojo::common::mojom::ProcessIdDataView>>(
        pointer, output, context_);
  }
 private:
  internal::ClientProcess_RequestOSMemoryDump_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ClientProcess_RequestOSMemoryDump_ResponseParamsDataView {
 public:
  ClientProcess_RequestOSMemoryDump_ResponseParamsDataView() {}

  ClientProcess_RequestOSMemoryDump_ResponseParamsDataView(
      internal::ClientProcess_RequestOSMemoryDump_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
  inline void GetDumpsDataView(
      mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, RawOSMemDumpDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDumps(UserType* output) {
    auto* pointer = data_->dumps.Get();
    return mojo::internal::Deserialize<mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, ::memory_instrumentation::mojom::RawOSMemDumpDataView>>(
        pointer, output, context_);
  }
 private:
  internal::ClientProcess_RequestOSMemoryDump_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_RegisterClientProcess_ParamsDataView {
 public:
  Coordinator_RegisterClientProcess_ParamsDataView() {}

  Coordinator_RegisterClientProcess_ParamsDataView(
      internal::Coordinator_RegisterClientProcess_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeClientProcess() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::memory_instrumentation::mojom::ClientProcessPtrDataView>(
            &data_->client_process, &result, context_);
    DCHECK(ret);
    return result;
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadProcessType(UserType* output) const {
    auto data_value = data_->process_type;
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::ProcessType>(
        data_value, output);
  }

  ProcessType process_type() const {
    return static_cast<ProcessType>(data_->process_type);
  }
 private:
  internal::Coordinator_RegisterClientProcess_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_RequestGlobalMemoryDump_ParamsDataView {
 public:
  Coordinator_RequestGlobalMemoryDump_ParamsDataView() {}

  Coordinator_RequestGlobalMemoryDump_ParamsDataView(
      internal::Coordinator_RequestGlobalMemoryDump_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetArgsDataView(
      RequestArgsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadArgs(UserType* output) {
    auto* pointer = data_->args.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::RequestArgsDataView>(
        pointer, output, context_);
  }
 private:
  internal::Coordinator_RequestGlobalMemoryDump_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Coordinator_RequestGlobalMemoryDump_ResponseParamsDataView {
 public:
  Coordinator_RequestGlobalMemoryDump_ResponseParamsDataView() {}

  Coordinator_RequestGlobalMemoryDump_ResponseParamsDataView(
      internal::Coordinator_RequestGlobalMemoryDump_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  bool success() const {
    return data_->success;
  }
  uint64_t dump_id() const {
    return data_->dump_id;
  }
  inline void GetGlobalMemoryDumpDataView(
      GlobalMemoryDumpDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGlobalMemoryDump(UserType* output) {
    auto* pointer = data_->global_memory_dump.Get();
    return mojo::internal::Deserialize<::memory_instrumentation::mojom::GlobalMemoryDumpDataView>(
        pointer, output, context_);
  }
 private:
  internal::Coordinator_RequestGlobalMemoryDump_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace memory_instrumentation

namespace std {

template <>
struct hash<::memory_instrumentation::mojom::DumpType>
    : public mojo::internal::EnumHashImpl<::memory_instrumentation::mojom::DumpType> {};

template <>
struct hash<::memory_instrumentation::mojom::LevelOfDetail>
    : public mojo::internal::EnumHashImpl<::memory_instrumentation::mojom::LevelOfDetail> {};

template <>
struct hash<::memory_instrumentation::mojom::ProcessType>
    : public mojo::internal::EnumHashImpl<::memory_instrumentation::mojom::ProcessType> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::memory_instrumentation::mojom::DumpType, ::memory_instrumentation::mojom::DumpType> {
  static ::memory_instrumentation::mojom::DumpType ToMojom(::memory_instrumentation::mojom::DumpType input) { return input; }
  static bool FromMojom(::memory_instrumentation::mojom::DumpType input, ::memory_instrumentation::mojom::DumpType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::DumpType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::memory_instrumentation::mojom::DumpType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::memory_instrumentation::mojom::DumpType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::memory_instrumentation::mojom::LevelOfDetail, ::memory_instrumentation::mojom::LevelOfDetail> {
  static ::memory_instrumentation::mojom::LevelOfDetail ToMojom(::memory_instrumentation::mojom::LevelOfDetail input) { return input; }
  static bool FromMojom(::memory_instrumentation::mojom::LevelOfDetail input, ::memory_instrumentation::mojom::LevelOfDetail* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::LevelOfDetail, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::memory_instrumentation::mojom::LevelOfDetail, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::memory_instrumentation::mojom::LevelOfDetail>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::memory_instrumentation::mojom::ProcessType, ::memory_instrumentation::mojom::ProcessType> {
  static ::memory_instrumentation::mojom::ProcessType ToMojom(::memory_instrumentation::mojom::ProcessType input) { return input; }
  static bool FromMojom(::memory_instrumentation::mojom::ProcessType input, ::memory_instrumentation::mojom::ProcessType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::ProcessType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::memory_instrumentation::mojom::ProcessType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::memory_instrumentation::mojom::ProcessType>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::RequestArgsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::RequestArgsDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::RequestArgs_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::RequestArgs_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::RequestArgs_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->dump_guid = CallWithContext(Traits::dump_guid, input, custom_context);
    mojo::internal::Serialize<::memory_instrumentation::mojom::DumpType>(
        CallWithContext(Traits::dump_type, input, custom_context), &result->dump_type);
    mojo::internal::Serialize<::memory_instrumentation::mojom::LevelOfDetail>(
        CallWithContext(Traits::level_of_detail, input, custom_context), &result->level_of_detail);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::RequestArgs_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::RequestArgsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::ChromeMemDumpDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::ChromeMemDumpDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::ChromeMemDump_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::ChromeMemDump_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::ChromeMemDump_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->malloc_total_kb = CallWithContext(Traits::malloc_total_kb, input, custom_context);
    result->command_buffer_total_kb = CallWithContext(Traits::command_buffer_total_kb, input, custom_context);
    result->partition_alloc_total_kb = CallWithContext(Traits::partition_alloc_total_kb, input, custom_context);
    result->blink_gc_total_kb = CallWithContext(Traits::blink_gc_total_kb, input, custom_context);
    result->v8_total_kb = CallWithContext(Traits::v8_total_kb, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::ChromeMemDump_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::ChromeMemDumpDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::PlatformPrivateFootprintDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::PlatformPrivateFootprintDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::PlatformPrivateFootprint_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::PlatformPrivateFootprint_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::PlatformPrivateFootprint_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->phys_footprint_bytes = CallWithContext(Traits::phys_footprint_bytes, input, custom_context);
    result->internal_bytes = CallWithContext(Traits::internal_bytes, input, custom_context);
    result->compressed_bytes = CallWithContext(Traits::compressed_bytes, input, custom_context);
    result->rss_anon_bytes = CallWithContext(Traits::rss_anon_bytes, input, custom_context);
    result->vm_swap_bytes = CallWithContext(Traits::vm_swap_bytes, input, custom_context);
    result->private_bytes = CallWithContext(Traits::private_bytes, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::PlatformPrivateFootprint_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::PlatformPrivateFootprintDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::RawOSMemDumpDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::RawOSMemDumpDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::RawOSMemDump_Data);
    decltype(CallWithContext(Traits::platform_private_footprint, input, custom_context)) in_platform_private_footprint = CallWithContext(Traits::platform_private_footprint, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::memory_instrumentation::mojom::PlatformPrivateFootprintDataView>(
        in_platform_private_footprint, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::RawOSMemDump_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::RawOSMemDump_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->resident_set_kb = CallWithContext(Traits::resident_set_kb, input, custom_context);
    decltype(CallWithContext(Traits::platform_private_footprint, input, custom_context)) in_platform_private_footprint = CallWithContext(Traits::platform_private_footprint, input, custom_context);
    typename decltype(result->platform_private_footprint)::BaseType* platform_private_footprint_ptr;
    mojo::internal::Serialize<::memory_instrumentation::mojom::PlatformPrivateFootprintDataView>(
        in_platform_private_footprint, buffer, &platform_private_footprint_ptr, context);
    result->platform_private_footprint.Set(platform_private_footprint_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->platform_private_footprint.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null platform_private_footprint in RawOSMemDump struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::RawOSMemDump_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::RawOSMemDumpDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::RawProcessMemoryDumpDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::RawProcessMemoryDumpDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::RawProcessMemoryDump_Data);
    decltype(CallWithContext(Traits::os_dump, input, custom_context)) in_os_dump = CallWithContext(Traits::os_dump, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::memory_instrumentation::mojom::RawOSMemDumpDataView>(
        in_os_dump, context);
    decltype(CallWithContext(Traits::chrome_dump, input, custom_context)) in_chrome_dump = CallWithContext(Traits::chrome_dump, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::memory_instrumentation::mojom::ChromeMemDumpDataView>(
        in_chrome_dump, context);
    decltype(CallWithContext(Traits::extra_processes_dumps, input, custom_context)) in_extra_processes_dumps = CallWithContext(Traits::extra_processes_dumps, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, ::memory_instrumentation::mojom::RawOSMemDumpDataView>>(
        in_extra_processes_dumps, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::RawProcessMemoryDump_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::RawProcessMemoryDump_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::os_dump, input, custom_context)) in_os_dump = CallWithContext(Traits::os_dump, input, custom_context);
    typename decltype(result->os_dump)::BaseType* os_dump_ptr;
    mojo::internal::Serialize<::memory_instrumentation::mojom::RawOSMemDumpDataView>(
        in_os_dump, buffer, &os_dump_ptr, context);
    result->os_dump.Set(os_dump_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->os_dump.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null os_dump in RawProcessMemoryDump struct");
    decltype(CallWithContext(Traits::chrome_dump, input, custom_context)) in_chrome_dump = CallWithContext(Traits::chrome_dump, input, custom_context);
    typename decltype(result->chrome_dump)::BaseType* chrome_dump_ptr;
    mojo::internal::Serialize<::memory_instrumentation::mojom::ChromeMemDumpDataView>(
        in_chrome_dump, buffer, &chrome_dump_ptr, context);
    result->chrome_dump.Set(chrome_dump_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->chrome_dump.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null chrome_dump in RawProcessMemoryDump struct");
    decltype(CallWithContext(Traits::extra_processes_dumps, input, custom_context)) in_extra_processes_dumps = CallWithContext(Traits::extra_processes_dumps, input, custom_context);
    typename decltype(result->extra_processes_dumps)::BaseType* extra_processes_dumps_ptr;
    const mojo::internal::ContainerValidateParams extra_processes_dumps_validate_params(
        new mojo::internal::ContainerValidateParams(0, false, nullptr), new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, ::memory_instrumentation::mojom::RawOSMemDumpDataView>>(
        in_extra_processes_dumps, buffer, &extra_processes_dumps_ptr, &extra_processes_dumps_validate_params,
        context);
    result->extra_processes_dumps.Set(extra_processes_dumps_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->extra_processes_dumps.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null extra_processes_dumps in RawProcessMemoryDump struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::RawProcessMemoryDump_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::RawProcessMemoryDumpDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::OSMemDumpDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::OSMemDumpDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::OSMemDump_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::OSMemDump_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::OSMemDump_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->resident_set_kb = CallWithContext(Traits::resident_set_kb, input, custom_context);
    result->private_footprint_kb = CallWithContext(Traits::private_footprint_kb, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::OSMemDump_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::OSMemDumpDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::ProcessMemoryDumpDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::ProcessMemoryDumpDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::ProcessMemoryDump_Data);
    decltype(CallWithContext(Traits::os_dump, input, custom_context)) in_os_dump = CallWithContext(Traits::os_dump, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::memory_instrumentation::mojom::OSMemDumpDataView>(
        in_os_dump, context);
    decltype(CallWithContext(Traits::chrome_dump, input, custom_context)) in_chrome_dump = CallWithContext(Traits::chrome_dump, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::memory_instrumentation::mojom::ChromeMemDumpDataView>(
        in_chrome_dump, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::ProcessMemoryDump_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::ProcessMemoryDump_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    mojo::internal::Serialize<::memory_instrumentation::mojom::ProcessType>(
        CallWithContext(Traits::process_type, input, custom_context), &result->process_type);
    decltype(CallWithContext(Traits::os_dump, input, custom_context)) in_os_dump = CallWithContext(Traits::os_dump, input, custom_context);
    typename decltype(result->os_dump)::BaseType* os_dump_ptr;
    mojo::internal::Serialize<::memory_instrumentation::mojom::OSMemDumpDataView>(
        in_os_dump, buffer, &os_dump_ptr, context);
    result->os_dump.Set(os_dump_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->os_dump.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null os_dump in ProcessMemoryDump struct");
    decltype(CallWithContext(Traits::chrome_dump, input, custom_context)) in_chrome_dump = CallWithContext(Traits::chrome_dump, input, custom_context);
    typename decltype(result->chrome_dump)::BaseType* chrome_dump_ptr;
    mojo::internal::Serialize<::memory_instrumentation::mojom::ChromeMemDumpDataView>(
        in_chrome_dump, buffer, &chrome_dump_ptr, context);
    result->chrome_dump.Set(chrome_dump_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->chrome_dump.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null chrome_dump in ProcessMemoryDump struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::ProcessMemoryDump_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::ProcessMemoryDumpDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::memory_instrumentation::mojom::GlobalMemoryDumpDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::memory_instrumentation::mojom::GlobalMemoryDumpDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::memory_instrumentation::mojom::internal::GlobalMemoryDump_Data);
    decltype(CallWithContext(Traits::process_dumps, input, custom_context)) in_process_dumps = CallWithContext(Traits::process_dumps, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::memory_instrumentation::mojom::ProcessMemoryDumpDataView>>(
        in_process_dumps, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::memory_instrumentation::mojom::internal::GlobalMemoryDump_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::memory_instrumentation::mojom::internal::GlobalMemoryDump_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::process_dumps, input, custom_context)) in_process_dumps = CallWithContext(Traits::process_dumps, input, custom_context);
    typename decltype(result->process_dumps)::BaseType* process_dumps_ptr;
    const mojo::internal::ContainerValidateParams process_dumps_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::memory_instrumentation::mojom::ProcessMemoryDumpDataView>>(
        in_process_dumps, buffer, &process_dumps_ptr, &process_dumps_validate_params,
        context);
    result->process_dumps.Set(process_dumps_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->process_dumps.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null process_dumps in GlobalMemoryDump struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::memory_instrumentation::mojom::internal::GlobalMemoryDump_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::memory_instrumentation::mojom::GlobalMemoryDumpDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace memory_instrumentation {
namespace mojom {







inline void RawOSMemDumpDataView::GetPlatformPrivateFootprintDataView(
    PlatformPrivateFootprintDataView* output) {
  auto pointer = data_->platform_private_footprint.Get();
  *output = PlatformPrivateFootprintDataView(pointer, context_);
}


inline void RawProcessMemoryDumpDataView::GetOsDumpDataView(
    RawOSMemDumpDataView* output) {
  auto pointer = data_->os_dump.Get();
  *output = RawOSMemDumpDataView(pointer, context_);
}
inline void RawProcessMemoryDumpDataView::GetChromeDumpDataView(
    ChromeMemDumpDataView* output) {
  auto pointer = data_->chrome_dump.Get();
  *output = ChromeMemDumpDataView(pointer, context_);
}
inline void RawProcessMemoryDumpDataView::GetExtraProcessesDumpsDataView(
    mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, RawOSMemDumpDataView>* output) {
  auto pointer = data_->extra_processes_dumps.Get();
  *output = mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, RawOSMemDumpDataView>(pointer, context_);
}




inline void ProcessMemoryDumpDataView::GetOsDumpDataView(
    OSMemDumpDataView* output) {
  auto pointer = data_->os_dump.Get();
  *output = OSMemDumpDataView(pointer, context_);
}
inline void ProcessMemoryDumpDataView::GetChromeDumpDataView(
    ChromeMemDumpDataView* output) {
  auto pointer = data_->chrome_dump.Get();
  *output = ChromeMemDumpDataView(pointer, context_);
}


inline void GlobalMemoryDumpDataView::GetProcessDumpsDataView(
    mojo::ArrayDataView<ProcessMemoryDumpDataView>* output) {
  auto pointer = data_->process_dumps.Get();
  *output = mojo::ArrayDataView<ProcessMemoryDumpDataView>(pointer, context_);
}


inline void ClientProcess_RequestProcessMemoryDump_ParamsDataView::GetArgsDataView(
    RequestArgsDataView* output) {
  auto pointer = data_->args.Get();
  *output = RequestArgsDataView(pointer, context_);
}


inline void ClientProcess_RequestProcessMemoryDump_ResponseParamsDataView::GetProcessMemoryDumpDataView(
    RawProcessMemoryDumpDataView* output) {
  auto pointer = data_->process_memory_dump.Get();
  *output = RawProcessMemoryDumpDataView(pointer, context_);
}


inline void ClientProcess_RequestOSMemoryDump_ParamsDataView::GetPidsDataView(
    mojo::ArrayDataView<::mojo::common::mojom::ProcessIdDataView>* output) {
  auto pointer = data_->pids.Get();
  *output = mojo::ArrayDataView<::mojo::common::mojom::ProcessIdDataView>(pointer, context_);
}


inline void ClientProcess_RequestOSMemoryDump_ResponseParamsDataView::GetDumpsDataView(
    mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, RawOSMemDumpDataView>* output) {
  auto pointer = data_->dumps.Get();
  *output = mojo::MapDataView<::mojo::common::mojom::ProcessIdDataView, RawOSMemDumpDataView>(pointer, context_);
}




inline void Coordinator_RequestGlobalMemoryDump_ParamsDataView::GetArgsDataView(
    RequestArgsDataView* output) {
  auto pointer = data_->args.Get();
  *output = RequestArgsDataView(pointer, context_);
}


inline void Coordinator_RequestGlobalMemoryDump_ResponseParamsDataView::GetGlobalMemoryDumpDataView(
    GlobalMemoryDumpDataView* output) {
  auto pointer = data_->global_memory_dump.Get();
  *output = GlobalMemoryDumpDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace memory_instrumentation

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_MEMORY_INSTRUMENTATION_MEMORY_INSTRUMENTATION_MOJOM_SHARED_H_
