// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_SERVICE_CALLBACKS_MOJOM_BLINK_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_SERVICE_CALLBACKS_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/service_callbacks.mojom-shared.h"
#include "services/metrics/public/interfaces/ukm_interface.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "services/resource_coordinator/public/cpp/resource_coordinator_export.h"


namespace resource_coordinator {
namespace mojom {
namespace blink {
class ServiceCallbacks;
using ServiceCallbacksPtr = mojo::InterfacePtr<ServiceCallbacks>;
using ServiceCallbacksPtrInfo = mojo::InterfacePtrInfo<ServiceCallbacks>;
using ThreadSafeServiceCallbacksPtr =
    mojo::ThreadSafeInterfacePtr<ServiceCallbacks>;
using ServiceCallbacksRequest = mojo::InterfaceRequest<ServiceCallbacks>;
using ServiceCallbacksAssociatedPtr =
    mojo::AssociatedInterfacePtr<ServiceCallbacks>;
using ThreadSafeServiceCallbacksAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<ServiceCallbacks>;
using ServiceCallbacksAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<ServiceCallbacks>;
using ServiceCallbacksAssociatedRequest =
    mojo::AssociatedInterfaceRequest<ServiceCallbacks>;


class ServiceCallbacksProxy;

template <typename ImplRefTraits>
class ServiceCallbacksStub;

class ServiceCallbacksRequestValidator;
class ServiceCallbacksResponseValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ServiceCallbacks
    : public ServiceCallbacksInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = ServiceCallbacksProxy;

  template <typename ImplRefTraits>
  using Stub_ = ServiceCallbacksStub<ImplRefTraits>;

  using RequestValidator_ = ServiceCallbacksRequestValidator;
  using ResponseValidator_ = ServiceCallbacksResponseValidator;
  enum MethodMinVersions : uint32_t {
    kIsUkmRecorderInterfaceInitializedMinVersion = 0,
    kSetUkmRecorderInterfaceMinVersion = 0,
  };
  virtual ~ServiceCallbacks() {}


  using IsUkmRecorderInterfaceInitializedCallback = base::Callback<void(bool)>;
  virtual void IsUkmRecorderInterfaceInitialized(const IsUkmRecorderInterfaceInitializedCallback& callback) = 0;

  virtual void SetUkmRecorderInterface(::ukm::mojom::blink::UkmRecorderInterfacePtr ukm_recorder_interface) = 0;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ServiceCallbacksProxy
    : public ServiceCallbacks {
 public:
  explicit ServiceCallbacksProxy(mojo::MessageReceiverWithResponder* receiver);
  void IsUkmRecorderInterfaceInitialized(const IsUkmRecorderInterfaceInitializedCallback& callback) override;
  void SetUkmRecorderInterface(::ukm::mojom::blink::UkmRecorderInterfacePtr ukm_recorder_interface) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ServiceCallbacksStubDispatch {
 public:
  static bool Accept(ServiceCallbacks* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      ServiceCallbacks* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<ServiceCallbacks>>
class ServiceCallbacksStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  ServiceCallbacksStub() {}
  ~ServiceCallbacksStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceCallbacksStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return ServiceCallbacksStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ServiceCallbacksRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT ServiceCallbacksResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace blink
}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_SERVICE_CALLBACKS_MOJOM_BLINK_H_