// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TAB_SIGNAL_MOJOM_H_
#define SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TAB_SIGNAL_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/resource_coordinator/public/interfaces/tab_signal.mojom-shared.h"
#include "services/resource_coordinator/public/interfaces/coordination_unit.mojom.h"
#include "mojo/common/values.mojom.h"
#include <string>
#include <vector>
#include "services/resource_coordinator/public/cpp/resource_coordinator_export.h"


namespace resource_coordinator {
namespace mojom {
class TabSignalObserver;
using TabSignalObserverPtr = mojo::InterfacePtr<TabSignalObserver>;
using TabSignalObserverPtrInfo = mojo::InterfacePtrInfo<TabSignalObserver>;
using ThreadSafeTabSignalObserverPtr =
    mojo::ThreadSafeInterfacePtr<TabSignalObserver>;
using TabSignalObserverRequest = mojo::InterfaceRequest<TabSignalObserver>;
using TabSignalObserverAssociatedPtr =
    mojo::AssociatedInterfacePtr<TabSignalObserver>;
using ThreadSafeTabSignalObserverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<TabSignalObserver>;
using TabSignalObserverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<TabSignalObserver>;
using TabSignalObserverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<TabSignalObserver>;

class TabSignalGenerator;
using TabSignalGeneratorPtr = mojo::InterfacePtr<TabSignalGenerator>;
using TabSignalGeneratorPtrInfo = mojo::InterfacePtrInfo<TabSignalGenerator>;
using ThreadSafeTabSignalGeneratorPtr =
    mojo::ThreadSafeInterfacePtr<TabSignalGenerator>;
using TabSignalGeneratorRequest = mojo::InterfaceRequest<TabSignalGenerator>;
using TabSignalGeneratorAssociatedPtr =
    mojo::AssociatedInterfacePtr<TabSignalGenerator>;
using ThreadSafeTabSignalGeneratorAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<TabSignalGenerator>;
using TabSignalGeneratorAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<TabSignalGenerator>;
using TabSignalGeneratorAssociatedRequest =
    mojo::AssociatedInterfaceRequest<TabSignalGenerator>;


class TabSignalObserverProxy;

template <typename ImplRefTraits>
class TabSignalObserverStub;

class TabSignalObserverRequestValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalObserver
    : public TabSignalObserverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = TabSignalObserverProxy;

  template <typename ImplRefTraits>
  using Stub_ = TabSignalObserverStub<ImplRefTraits>;

  using RequestValidator_ = TabSignalObserverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnEventReceivedMinVersion = 0,
  };
  virtual ~TabSignalObserver() {}

  virtual void OnEventReceived(const ::resource_coordinator::CoordinationUnitID& cu_id, TabEvent event) = 0;
};

class TabSignalGeneratorProxy;

template <typename ImplRefTraits>
class TabSignalGeneratorStub;

class TabSignalGeneratorRequestValidator;

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalGenerator
    : public TabSignalGeneratorInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = TabSignalGeneratorProxy;

  template <typename ImplRefTraits>
  using Stub_ = TabSignalGeneratorStub<ImplRefTraits>;

  using RequestValidator_ = TabSignalGeneratorRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kAddObserverMinVersion = 0,
  };
  virtual ~TabSignalGenerator() {}

  virtual void AddObserver(TabSignalObserverPtr observer) = 0;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalObserverProxy
    : public TabSignalObserver {
 public:
  explicit TabSignalObserverProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnEventReceived(const ::resource_coordinator::CoordinationUnitID& cu_id, TabEvent event) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalGeneratorProxy
    : public TabSignalGenerator {
 public:
  explicit TabSignalGeneratorProxy(mojo::MessageReceiverWithResponder* receiver);
  void AddObserver(TabSignalObserverPtr observer) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalObserverStubDispatch {
 public:
  static bool Accept(TabSignalObserver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TabSignalObserver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TabSignalObserver>>
class TabSignalObserverStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TabSignalObserverStub() {}
  ~TabSignalObserverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TabSignalObserverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TabSignalObserverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalGeneratorStubDispatch {
 public:
  static bool Accept(TabSignalGenerator* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      TabSignalGenerator* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<TabSignalGenerator>>
class TabSignalGeneratorStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  TabSignalGeneratorStub() {}
  ~TabSignalGeneratorStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TabSignalGeneratorStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return TabSignalGeneratorStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalObserverRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class SERVICES_RESOURCE_COORDINATOR_PUBLIC_CPP_EXPORT TabSignalGeneratorRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace resource_coordinator

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_RESOURCE_COORDINATOR_PUBLIC_INTERFACES_TAB_SIGNAL_MOJOM_H_