// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_UI_GPU_INTERFACES_GPU_SERVICE_MOJOM_H_
#define SERVICES_UI_GPU_INTERFACES_GPU_SERVICE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "services/ui/gpu/interfaces/gpu_service.mojom-shared.h"
#include "gpu/ipc/common/gpu_info.mojom.h"
#include "gpu/ipc/common/memory_stats.mojom.h"
#include "gpu/ipc/common/surface_handle.mojom.h"
#include "gpu/ipc/common/sync_token.mojom.h"
#include "media/gpu/mojo/jpeg_decoder.mojom.h"
#include "media/mojo/interfaces/video_encode_accelerator.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "ui/gfx/mojo/buffer_types.mojom.h"
#include <string>
#include <vector>


namespace ui {
namespace mojom {
class GpuService;
using GpuServicePtr = mojo::InterfacePtr<GpuService>;
using GpuServicePtrInfo = mojo::InterfacePtrInfo<GpuService>;
using ThreadSafeGpuServicePtr =
    mojo::ThreadSafeInterfacePtr<GpuService>;
using GpuServiceRequest = mojo::InterfaceRequest<GpuService>;
using GpuServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<GpuService>;
using ThreadSafeGpuServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<GpuService>;
using GpuServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<GpuService>;
using GpuServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<GpuService>;


class GpuServiceProxy;

template <typename ImplRefTraits>
class GpuServiceStub;

class GpuServiceRequestValidator;
class GpuServiceResponseValidator;

class  GpuService
    : public GpuServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = GpuServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = GpuServiceStub<ImplRefTraits>;

  using RequestValidator_ = GpuServiceRequestValidator;
  using ResponseValidator_ = GpuServiceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kEstablishGpuChannelMinVersion = 0,
    kCloseChannelMinVersion = 0,
    kCreateJpegDecodeAcceleratorMinVersion = 0,
    kCreateVideoEncodeAcceleratorMinVersion = 0,
    kCreateGpuMemoryBufferMinVersion = 0,
    kDestroyGpuMemoryBufferMinVersion = 0,
    kGetVideoMemoryUsageStatsMinVersion = 0,
    kRequestCompleteGpuInfoMinVersion = 0,
    kLoadedShaderMinVersion = 0,
    kDestroyingVideoSurfaceMinVersion = 0,
    kWakeUpGpuMinVersion = 0,
    kGpuSwitchedMinVersion = 0,
    kDestroyAllChannelsMinVersion = 0,
    kCrashMinVersion = 0,
    kHangMinVersion = 0,
    kThrowJavaExceptionMinVersion = 0,
    kStopMinVersion = 0,
  };
  virtual ~GpuService() {}


  using EstablishGpuChannelCallback = base::Callback<void(mojo::ScopedMessagePipeHandle)>;
  virtual void EstablishGpuChannel(int32_t client_id, uint64_t client_tracing_id, bool is_gpu_host, const EstablishGpuChannelCallback& callback) = 0;

  virtual void CloseChannel(int32_t client_id) = 0;

  virtual void CreateJpegDecodeAccelerator(::media::mojom::GpuJpegDecodeAcceleratorRequest jda) = 0;

  virtual void CreateVideoEncodeAccelerator(::media::mojom::VideoEncodeAcceleratorRequest vea) = 0;


  using CreateGpuMemoryBufferCallback = base::Callback<void(const gfx::GpuMemoryBufferHandle&)>;
  virtual void CreateGpuMemoryBuffer(gfx::GpuMemoryBufferId id, const gfx::Size& size, gfx::BufferFormat format, gfx::BufferUsage usage, int32_t client_id, ::gpu::SurfaceHandle surface_handle, const CreateGpuMemoryBufferCallback& callback) = 0;

  virtual void DestroyGpuMemoryBuffer(gfx::GpuMemoryBufferId id, int32_t client_id, const ::gpu::SyncToken& sync_token) = 0;


  using GetVideoMemoryUsageStatsCallback = base::Callback<void(const gpu::VideoMemoryUsageStats&)>;
  virtual void GetVideoMemoryUsageStats(const GetVideoMemoryUsageStatsCallback& callback) = 0;


  using RequestCompleteGpuInfoCallback = base::Callback<void(const gpu::GPUInfo&)>;
  virtual void RequestCompleteGpuInfo(const RequestCompleteGpuInfoCallback& callback) = 0;

  virtual void LoadedShader(const std::string& key, const std::string& data) = 0;


  using DestroyingVideoSurfaceCallback = base::Callback<void()>;
  virtual void DestroyingVideoSurface(int32_t surface_id, const DestroyingVideoSurfaceCallback& callback) = 0;

  virtual void WakeUpGpu() = 0;

  virtual void GpuSwitched() = 0;

  virtual void DestroyAllChannels() = 0;

  virtual void Crash() = 0;

  virtual void Hang() = 0;

  virtual void ThrowJavaException() = 0;


  using StopCallback = base::Callback<void()>;
  virtual void Stop(const StopCallback& callback) = 0;
};

class  GpuServiceProxy
    : public GpuService {
 public:
  explicit GpuServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void EstablishGpuChannel(int32_t client_id, uint64_t client_tracing_id, bool is_gpu_host, const EstablishGpuChannelCallback& callback) override;
  void CloseChannel(int32_t client_id) override;
  void CreateJpegDecodeAccelerator(::media::mojom::GpuJpegDecodeAcceleratorRequest jda) override;
  void CreateVideoEncodeAccelerator(::media::mojom::VideoEncodeAcceleratorRequest vea) override;
  void CreateGpuMemoryBuffer(gfx::GpuMemoryBufferId id, const gfx::Size& size, gfx::BufferFormat format, gfx::BufferUsage usage, int32_t client_id, ::gpu::SurfaceHandle surface_handle, const CreateGpuMemoryBufferCallback& callback) override;
  void DestroyGpuMemoryBuffer(gfx::GpuMemoryBufferId id, int32_t client_id, const ::gpu::SyncToken& sync_token) override;
  void GetVideoMemoryUsageStats(const GetVideoMemoryUsageStatsCallback& callback) override;
  void RequestCompleteGpuInfo(const RequestCompleteGpuInfoCallback& callback) override;
  void LoadedShader(const std::string& key, const std::string& data) override;
  void DestroyingVideoSurface(int32_t surface_id, const DestroyingVideoSurfaceCallback& callback) override;
  void WakeUpGpu() override;
  void GpuSwitched() override;
  void DestroyAllChannels() override;
  void Crash() override;
  void Hang() override;
  void ThrowJavaException() override;
  void Stop(const StopCallback& callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  GpuServiceStubDispatch {
 public:
  static bool Accept(GpuService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      GpuService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<GpuService>>
class GpuServiceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  GpuServiceStub() {}
  ~GpuServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return GpuServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return GpuServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  GpuServiceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  GpuServiceResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#endif  // SERVICES_UI_GPU_INTERFACES_GPU_SERVICE_MOJOM_H_