// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "services/ui/public/interfaces/ime/ime.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace ui {
namespace mojom {

namespace internal {


// static
CandidateWindowProperties_Data* CandidateWindowProperties_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(CandidateWindowProperties_Data))) CandidateWindowProperties_Data();
}

// static
bool CandidateWindowProperties_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const CandidateWindowProperties_Data* object = static_cast<const CandidateWindowProperties_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->auxiliary_text, "null auxiliary_text field in CandidateWindowProperties",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams auxiliary_text_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->auxiliary_text, validation_context,
                                         &auxiliary_text_validate_params)) {
    return false;
  }


  if (!::ui::mojom::internal::CandidateWindowPosition_Data
        ::Validate(object->window_position, validation_context))
    return false;

  return true;
}

CandidateWindowProperties_Data::CandidateWindowProperties_Data()
    : header_({sizeof(*this), 0}) {}


// static
CandidateWindowEntry_Data* CandidateWindowEntry_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(CandidateWindowEntry_Data))) CandidateWindowEntry_Data();
}

// static
bool CandidateWindowEntry_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const CandidateWindowEntry_Data* object = static_cast<const CandidateWindowEntry_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 48 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->value, "null value field in CandidateWindowEntry",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams value_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->value, validation_context,
                                         &value_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->label, "null label field in CandidateWindowEntry",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams label_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->label, validation_context,
                                         &label_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->annotation, "null annotation field in CandidateWindowEntry",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams annotation_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->annotation, validation_context,
                                         &annotation_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->description_title, "null description_title field in CandidateWindowEntry",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams description_title_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->description_title, validation_context,
                                         &description_title_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->description_body, "null description_body field in CandidateWindowEntry",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams description_body_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->description_body, validation_context,
                                         &description_body_validate_params)) {
    return false;
  }

  return true;
}

CandidateWindowEntry_Data::CandidateWindowEntry_Data()
    : header_({sizeof(*this), 0}) {}


// static
CompositionUnderline_Data* CompositionUnderline_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(CompositionUnderline_Data))) CompositionUnderline_Data();
}

// static
bool CompositionUnderline_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const CompositionUnderline_Data* object = static_cast<const CompositionUnderline_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

CompositionUnderline_Data::CompositionUnderline_Data()
    : header_({sizeof(*this), 0}) {}


// static
CompositionText_Data* CompositionText_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(CompositionText_Data))) CompositionText_Data();
}

// static
bool CompositionText_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const CompositionText_Data* object = static_cast<const CompositionText_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->text, "null text field in CompositionText",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams text_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->text, validation_context,
                                         &text_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->underlines, "null underlines field in CompositionText",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams underlines_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->underlines, validation_context,
                                         &underlines_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->selection, "null selection field in CompositionText",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->selection, validation_context))
    return false;

  return true;
}

CompositionText_Data::CompositionText_Data()
    : header_({sizeof(*this), 0}) {}


// static
StartSessionDetails_Data* StartSessionDetails_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(StartSessionDetails_Data))) StartSessionDetails_Data();
}

// static
bool StartSessionDetails_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const StartSessionDetails_Data* object = static_cast<const StartSessionDetails_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 48 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->client,
          "invalid client field in StartSessionDetails", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->client,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->input_method_request,
          "invalid input_method_request field in StartSessionDetails", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->input_method_request,
                                                 validation_context)) {
    return false;
  }


  if (!::ui::mojom::internal::TextInputType_Data
        ::Validate(object->text_input_type, validation_context))
    return false;


  if (!::ui::mojom::internal::TextInputMode_Data
        ::Validate(object->text_input_mode, validation_context))
    return false;


  if (!::mojo::common::mojom::internal::TextDirection_Data
        ::Validate(object->text_direction, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->caret_bounds, "null caret_bounds field in StartSessionDetails",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->caret_bounds, validation_context))
    return false;

  return true;
}

StartSessionDetails_Data::StartSessionDetails_Data()
    : header_({sizeof(*this), 0}) {}


// static
IMEDriver_StartSession_Params_Data* IMEDriver_StartSession_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(IMEDriver_StartSession_Params_Data))) IMEDriver_StartSession_Params_Data();
}

// static
bool IMEDriver_StartSession_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const IMEDriver_StartSession_Params_Data* object = static_cast<const IMEDriver_StartSession_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->details, "null details field in IMEDriver_StartSession_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->details, validation_context))
    return false;

  return true;
}

IMEDriver_StartSession_Params_Data::IMEDriver_StartSession_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
IMERegistrar_RegisterDriver_Params_Data* IMERegistrar_RegisterDriver_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(IMERegistrar_RegisterDriver_Params_Data))) IMERegistrar_RegisterDriver_Params_Data();
}

// static
bool IMERegistrar_RegisterDriver_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const IMERegistrar_RegisterDriver_Params_Data* object = static_cast<const IMERegistrar_RegisterDriver_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->driver,
          "invalid driver field in IMERegistrar_RegisterDriver_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->driver,
                                                 validation_context)) {
    return false;
  }

  return true;
}

IMERegistrar_RegisterDriver_Params_Data::IMERegistrar_RegisterDriver_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
InputMethod_OnTextInputTypeChanged_Params_Data* InputMethod_OnTextInputTypeChanged_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(InputMethod_OnTextInputTypeChanged_Params_Data))) InputMethod_OnTextInputTypeChanged_Params_Data();
}

// static
bool InputMethod_OnTextInputTypeChanged_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const InputMethod_OnTextInputTypeChanged_Params_Data* object = static_cast<const InputMethod_OnTextInputTypeChanged_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::ui::mojom::internal::TextInputType_Data
        ::Validate(object->text_input_type, validation_context))
    return false;

  return true;
}

InputMethod_OnTextInputTypeChanged_Params_Data::InputMethod_OnTextInputTypeChanged_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
InputMethod_OnCaretBoundsChanged_Params_Data* InputMethod_OnCaretBoundsChanged_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(InputMethod_OnCaretBoundsChanged_Params_Data))) InputMethod_OnCaretBoundsChanged_Params_Data();
}

// static
bool InputMethod_OnCaretBoundsChanged_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const InputMethod_OnCaretBoundsChanged_Params_Data* object = static_cast<const InputMethod_OnCaretBoundsChanged_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->caret_bounds, "null caret_bounds field in InputMethod_OnCaretBoundsChanged_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->caret_bounds, validation_context))
    return false;

  return true;
}

InputMethod_OnCaretBoundsChanged_Params_Data::InputMethod_OnCaretBoundsChanged_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
InputMethod_ProcessKeyEvent_Params_Data* InputMethod_ProcessKeyEvent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(InputMethod_ProcessKeyEvent_Params_Data))) InputMethod_ProcessKeyEvent_Params_Data();
}

// static
bool InputMethod_ProcessKeyEvent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const InputMethod_ProcessKeyEvent_Params_Data* object = static_cast<const InputMethod_ProcessKeyEvent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->key_event, "null key_event field in InputMethod_ProcessKeyEvent_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->key_event, validation_context))
    return false;

  return true;
}

InputMethod_ProcessKeyEvent_Params_Data::InputMethod_ProcessKeyEvent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
InputMethod_ProcessKeyEvent_ResponseParams_Data* InputMethod_ProcessKeyEvent_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(InputMethod_ProcessKeyEvent_ResponseParams_Data))) InputMethod_ProcessKeyEvent_ResponseParams_Data();
}

// static
bool InputMethod_ProcessKeyEvent_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const InputMethod_ProcessKeyEvent_ResponseParams_Data* object = static_cast<const InputMethod_ProcessKeyEvent_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

InputMethod_ProcessKeyEvent_ResponseParams_Data::InputMethod_ProcessKeyEvent_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
InputMethod_CancelComposition_Params_Data* InputMethod_CancelComposition_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(InputMethod_CancelComposition_Params_Data))) InputMethod_CancelComposition_Params_Data();
}

// static
bool InputMethod_CancelComposition_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const InputMethod_CancelComposition_Params_Data* object = static_cast<const InputMethod_CancelComposition_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

InputMethod_CancelComposition_Params_Data::InputMethod_CancelComposition_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
TextInputClient_SetCompositionText_Params_Data* TextInputClient_SetCompositionText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TextInputClient_SetCompositionText_Params_Data))) TextInputClient_SetCompositionText_Params_Data();
}

// static
bool TextInputClient_SetCompositionText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TextInputClient_SetCompositionText_Params_Data* object = static_cast<const TextInputClient_SetCompositionText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->composition, "null composition field in TextInputClient_SetCompositionText_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->composition, validation_context))
    return false;

  return true;
}

TextInputClient_SetCompositionText_Params_Data::TextInputClient_SetCompositionText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
TextInputClient_ConfirmCompositionText_Params_Data* TextInputClient_ConfirmCompositionText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TextInputClient_ConfirmCompositionText_Params_Data))) TextInputClient_ConfirmCompositionText_Params_Data();
}

// static
bool TextInputClient_ConfirmCompositionText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TextInputClient_ConfirmCompositionText_Params_Data* object = static_cast<const TextInputClient_ConfirmCompositionText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

TextInputClient_ConfirmCompositionText_Params_Data::TextInputClient_ConfirmCompositionText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
TextInputClient_ClearCompositionText_Params_Data* TextInputClient_ClearCompositionText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TextInputClient_ClearCompositionText_Params_Data))) TextInputClient_ClearCompositionText_Params_Data();
}

// static
bool TextInputClient_ClearCompositionText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TextInputClient_ClearCompositionText_Params_Data* object = static_cast<const TextInputClient_ClearCompositionText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

TextInputClient_ClearCompositionText_Params_Data::TextInputClient_ClearCompositionText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
TextInputClient_InsertText_Params_Data* TextInputClient_InsertText_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TextInputClient_InsertText_Params_Data))) TextInputClient_InsertText_Params_Data();
}

// static
bool TextInputClient_InsertText_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TextInputClient_InsertText_Params_Data* object = static_cast<const TextInputClient_InsertText_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->text, "null text field in TextInputClient_InsertText_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams text_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->text, validation_context,
                                         &text_validate_params)) {
    return false;
  }

  return true;
}

TextInputClient_InsertText_Params_Data::TextInputClient_InsertText_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
TextInputClient_InsertChar_Params_Data* TextInputClient_InsertChar_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(TextInputClient_InsertChar_Params_Data))) TextInputClient_InsertChar_Params_Data();
}

// static
bool TextInputClient_InsertChar_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const TextInputClient_InsertChar_Params_Data* object = static_cast<const TextInputClient_InsertChar_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->event, "null event field in TextInputClient_InsertChar_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->event, validation_context))
    return false;

  return true;
}

TextInputClient_InsertChar_Params_Data::TextInputClient_InsertChar_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace ui

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
