// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "services/ui/public/interfaces/window_manager.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace ui {
namespace mojom {

namespace internal {


// static
WmAccelerator_Data* WmAccelerator_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WmAccelerator_Data))) WmAccelerator_Data();
}

// static
bool WmAccelerator_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WmAccelerator_Data* object = static_cast<const WmAccelerator_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->event_matcher, "null event_matcher field in WmAccelerator",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->event_matcher, validation_context))
    return false;

  return true;
}

WmAccelerator_Data::WmAccelerator_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_OnConnect_Params_Data* WindowManager_OnConnect_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_OnConnect_Params_Data))) WindowManager_OnConnect_Params_Data();
}

// static
bool WindowManager_OnConnect_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_OnConnect_Params_Data* object = static_cast<const WindowManager_OnConnect_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_OnConnect_Params_Data::WindowManager_OnConnect_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmNewDisplayAdded_Params_Data* WindowManager_WmNewDisplayAdded_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmNewDisplayAdded_Params_Data))) WindowManager_WmNewDisplayAdded_Params_Data();
}

// static
bool WindowManager_WmNewDisplayAdded_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmNewDisplayAdded_Params_Data* object = static_cast<const WindowManager_WmNewDisplayAdded_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->display, "null display field in WindowManager_WmNewDisplayAdded_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->display, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->root, "null root field in WindowManager_WmNewDisplayAdded_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->root, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->local_surface_id, validation_context))
    return false;

  return true;
}

WindowManager_WmNewDisplayAdded_Params_Data::WindowManager_WmNewDisplayAdded_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmDisplayRemoved_Params_Data* WindowManager_WmDisplayRemoved_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmDisplayRemoved_Params_Data))) WindowManager_WmDisplayRemoved_Params_Data();
}

// static
bool WindowManager_WmDisplayRemoved_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmDisplayRemoved_Params_Data* object = static_cast<const WindowManager_WmDisplayRemoved_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmDisplayRemoved_Params_Data::WindowManager_WmDisplayRemoved_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmDisplayModified_Params_Data* WindowManager_WmDisplayModified_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmDisplayModified_Params_Data))) WindowManager_WmDisplayModified_Params_Data();
}

// static
bool WindowManager_WmDisplayModified_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmDisplayModified_Params_Data* object = static_cast<const WindowManager_WmDisplayModified_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->display, "null display field in WindowManager_WmDisplayModified_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->display, validation_context))
    return false;

  return true;
}

WindowManager_WmDisplayModified_Params_Data::WindowManager_WmDisplayModified_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmSetBounds_Params_Data* WindowManager_WmSetBounds_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmSetBounds_Params_Data))) WindowManager_WmSetBounds_Params_Data();
}

// static
bool WindowManager_WmSetBounds_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmSetBounds_Params_Data* object = static_cast<const WindowManager_WmSetBounds_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->bounds, "null bounds field in WindowManager_WmSetBounds_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->bounds, validation_context))
    return false;

  return true;
}

WindowManager_WmSetBounds_Params_Data::WindowManager_WmSetBounds_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmSetProperty_Params_Data* WindowManager_WmSetProperty_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmSetProperty_Params_Data))) WindowManager_WmSetProperty_Params_Data();
}

// static
bool WindowManager_WmSetProperty_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmSetProperty_Params_Data* object = static_cast<const WindowManager_WmSetProperty_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->name, "null name field in WindowManager_WmSetProperty_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams name_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->name, validation_context,
                                         &name_validate_params)) {
    return false;
  }

  const mojo::internal::ContainerValidateParams value_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->value, validation_context,
                                         &value_validate_params)) {
    return false;
  }

  return true;
}

WindowManager_WmSetProperty_Params_Data::WindowManager_WmSetProperty_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmSetModalType_Params_Data* WindowManager_WmSetModalType_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmSetModalType_Params_Data))) WindowManager_WmSetModalType_Params_Data();
}

// static
bool WindowManager_WmSetModalType_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmSetModalType_Params_Data* object = static_cast<const WindowManager_WmSetModalType_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::ui::mojom::internal::ModalType_Data
        ::Validate(object->type, validation_context))
    return false;

  return true;
}

WindowManager_WmSetModalType_Params_Data::WindowManager_WmSetModalType_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmSetCanFocus_Params_Data* WindowManager_WmSetCanFocus_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmSetCanFocus_Params_Data))) WindowManager_WmSetCanFocus_Params_Data();
}

// static
bool WindowManager_WmSetCanFocus_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmSetCanFocus_Params_Data* object = static_cast<const WindowManager_WmSetCanFocus_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmSetCanFocus_Params_Data::WindowManager_WmSetCanFocus_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmCreateTopLevelWindow_Params_Data* WindowManager_WmCreateTopLevelWindow_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmCreateTopLevelWindow_Params_Data))) WindowManager_WmCreateTopLevelWindow_Params_Data();
}

// static
bool WindowManager_WmCreateTopLevelWindow_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmCreateTopLevelWindow_Params_Data* object = static_cast<const WindowManager_WmCreateTopLevelWindow_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->properties, "null properties field in WindowManager_WmCreateTopLevelWindow_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams properties_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
  if (!mojo::internal::ValidateContainer(object->properties, validation_context,
                                         &properties_validate_params)) {
    return false;
  }

  return true;
}

WindowManager_WmCreateTopLevelWindow_Params_Data::WindowManager_WmCreateTopLevelWindow_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmClientJankinessChanged_Params_Data* WindowManager_WmClientJankinessChanged_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmClientJankinessChanged_Params_Data))) WindowManager_WmClientJankinessChanged_Params_Data();
}

// static
bool WindowManager_WmClientJankinessChanged_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmClientJankinessChanged_Params_Data* object = static_cast<const WindowManager_WmClientJankinessChanged_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmClientJankinessChanged_Params_Data::WindowManager_WmClientJankinessChanged_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmBuildDragImage_Params_Data* WindowManager_WmBuildDragImage_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmBuildDragImage_Params_Data))) WindowManager_WmBuildDragImage_Params_Data();
}

// static
bool WindowManager_WmBuildDragImage_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmBuildDragImage_Params_Data* object = static_cast<const WindowManager_WmBuildDragImage_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->screen_location, "null screen_location field in WindowManager_WmBuildDragImage_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->screen_location, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->drag_image, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->drag_image_offset, "null drag_image_offset field in WindowManager_WmBuildDragImage_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->drag_image_offset, validation_context))
    return false;


  if (!::ui::mojom::internal::PointerKind_Data
        ::Validate(object->source, validation_context))
    return false;

  return true;
}

WindowManager_WmBuildDragImage_Params_Data::WindowManager_WmBuildDragImage_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmMoveDragImage_Params_Data* WindowManager_WmMoveDragImage_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmMoveDragImage_Params_Data))) WindowManager_WmMoveDragImage_Params_Data();
}

// static
bool WindowManager_WmMoveDragImage_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmMoveDragImage_Params_Data* object = static_cast<const WindowManager_WmMoveDragImage_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->screen_location, "null screen_location field in WindowManager_WmMoveDragImage_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->screen_location, validation_context))
    return false;

  return true;
}

WindowManager_WmMoveDragImage_Params_Data::WindowManager_WmMoveDragImage_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmMoveDragImage_ResponseParams_Data* WindowManager_WmMoveDragImage_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmMoveDragImage_ResponseParams_Data))) WindowManager_WmMoveDragImage_ResponseParams_Data();
}

// static
bool WindowManager_WmMoveDragImage_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmMoveDragImage_ResponseParams_Data* object = static_cast<const WindowManager_WmMoveDragImage_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmMoveDragImage_ResponseParams_Data::WindowManager_WmMoveDragImage_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmDestroyDragImage_Params_Data* WindowManager_WmDestroyDragImage_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmDestroyDragImage_Params_Data))) WindowManager_WmDestroyDragImage_Params_Data();
}

// static
bool WindowManager_WmDestroyDragImage_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmDestroyDragImage_Params_Data* object = static_cast<const WindowManager_WmDestroyDragImage_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmDestroyDragImage_Params_Data::WindowManager_WmDestroyDragImage_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmPerformMoveLoop_Params_Data* WindowManager_WmPerformMoveLoop_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmPerformMoveLoop_Params_Data))) WindowManager_WmPerformMoveLoop_Params_Data();
}

// static
bool WindowManager_WmPerformMoveLoop_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmPerformMoveLoop_Params_Data* object = static_cast<const WindowManager_WmPerformMoveLoop_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::ui::mojom::internal::MoveLoopSource_Data
        ::Validate(object->source, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->cursor_location, "null cursor_location field in WindowManager_WmPerformMoveLoop_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->cursor_location, validation_context))
    return false;

  return true;
}

WindowManager_WmPerformMoveLoop_Params_Data::WindowManager_WmPerformMoveLoop_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmCancelMoveLoop_Params_Data* WindowManager_WmCancelMoveLoop_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmCancelMoveLoop_Params_Data))) WindowManager_WmCancelMoveLoop_Params_Data();
}

// static
bool WindowManager_WmCancelMoveLoop_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmCancelMoveLoop_Params_Data* object = static_cast<const WindowManager_WmCancelMoveLoop_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmCancelMoveLoop_Params_Data::WindowManager_WmCancelMoveLoop_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmDeactivateWindow_Params_Data* WindowManager_WmDeactivateWindow_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmDeactivateWindow_Params_Data))) WindowManager_WmDeactivateWindow_Params_Data();
}

// static
bool WindowManager_WmDeactivateWindow_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmDeactivateWindow_Params_Data* object = static_cast<const WindowManager_WmDeactivateWindow_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmDeactivateWindow_Params_Data::WindowManager_WmDeactivateWindow_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmStackAbove_Params_Data* WindowManager_WmStackAbove_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmStackAbove_Params_Data))) WindowManager_WmStackAbove_Params_Data();
}

// static
bool WindowManager_WmStackAbove_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmStackAbove_Params_Data* object = static_cast<const WindowManager_WmStackAbove_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmStackAbove_Params_Data::WindowManager_WmStackAbove_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_WmStackAtTop_Params_Data* WindowManager_WmStackAtTop_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_WmStackAtTop_Params_Data))) WindowManager_WmStackAtTop_Params_Data();
}

// static
bool WindowManager_WmStackAtTop_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_WmStackAtTop_Params_Data* object = static_cast<const WindowManager_WmStackAtTop_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManager_WmStackAtTop_Params_Data::WindowManager_WmStackAtTop_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManager_OnAccelerator_Params_Data* WindowManager_OnAccelerator_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManager_OnAccelerator_Params_Data))) WindowManager_OnAccelerator_Params_Data();
}

// static
bool WindowManager_OnAccelerator_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManager_OnAccelerator_Params_Data* object = static_cast<const WindowManager_OnAccelerator_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->event, "null event field in WindowManager_OnAccelerator_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->event, validation_context))
    return false;

  return true;
}

WindowManager_OnAccelerator_Params_Data::WindowManager_OnAccelerator_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_AddActivationParent_Params_Data* WindowManagerClient_AddActivationParent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_AddActivationParent_Params_Data))) WindowManagerClient_AddActivationParent_Params_Data();
}

// static
bool WindowManagerClient_AddActivationParent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_AddActivationParent_Params_Data* object = static_cast<const WindowManagerClient_AddActivationParent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_AddActivationParent_Params_Data::WindowManagerClient_AddActivationParent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_RemoveActivationParent_Params_Data* WindowManagerClient_RemoveActivationParent_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_RemoveActivationParent_Params_Data))) WindowManagerClient_RemoveActivationParent_Params_Data();
}

// static
bool WindowManagerClient_RemoveActivationParent_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_RemoveActivationParent_Params_Data* object = static_cast<const WindowManagerClient_RemoveActivationParent_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_RemoveActivationParent_Params_Data::WindowManagerClient_RemoveActivationParent_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_ActivateNextWindow_Params_Data* WindowManagerClient_ActivateNextWindow_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_ActivateNextWindow_Params_Data))) WindowManagerClient_ActivateNextWindow_Params_Data();
}

// static
bool WindowManagerClient_ActivateNextWindow_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_ActivateNextWindow_Params_Data* object = static_cast<const WindowManagerClient_ActivateNextWindow_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_ActivateNextWindow_Params_Data::WindowManagerClient_ActivateNextWindow_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data* WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data))) WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data();
}

// static
bool WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data* object = static_cast<const WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->mouse_insets, "null mouse_insets field in WindowManagerClient_SetExtendedHitRegionForChildren_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->mouse_insets, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->touch_insets, "null touch_insets field in WindowManagerClient_SetExtendedHitRegionForChildren_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->touch_insets, validation_context))
    return false;

  return true;
}

WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data::WindowManagerClient_SetExtendedHitRegionForChildren_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_AddAccelerators_Params_Data* WindowManagerClient_AddAccelerators_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_AddAccelerators_Params_Data))) WindowManagerClient_AddAccelerators_Params_Data();
}

// static
bool WindowManagerClient_AddAccelerators_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_AddAccelerators_Params_Data* object = static_cast<const WindowManagerClient_AddAccelerators_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->accelerators, "null accelerators field in WindowManagerClient_AddAccelerators_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams accelerators_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->accelerators, validation_context,
                                         &accelerators_validate_params)) {
    return false;
  }

  return true;
}

WindowManagerClient_AddAccelerators_Params_Data::WindowManagerClient_AddAccelerators_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_AddAccelerators_ResponseParams_Data* WindowManagerClient_AddAccelerators_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_AddAccelerators_ResponseParams_Data))) WindowManagerClient_AddAccelerators_ResponseParams_Data();
}

// static
bool WindowManagerClient_AddAccelerators_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_AddAccelerators_ResponseParams_Data* object = static_cast<const WindowManagerClient_AddAccelerators_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_AddAccelerators_ResponseParams_Data::WindowManagerClient_AddAccelerators_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_RemoveAccelerator_Params_Data* WindowManagerClient_RemoveAccelerator_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_RemoveAccelerator_Params_Data))) WindowManagerClient_RemoveAccelerator_Params_Data();
}

// static
bool WindowManagerClient_RemoveAccelerator_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_RemoveAccelerator_Params_Data* object = static_cast<const WindowManagerClient_RemoveAccelerator_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_RemoveAccelerator_Params_Data::WindowManagerClient_RemoveAccelerator_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data* WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data))) WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data();
}

// static
bool WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data* object = static_cast<const WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->dont_hide_cursor_list, "null dont_hide_cursor_list field in WindowManagerClient_SetKeyEventsThatDontHideCursor_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams dont_hide_cursor_list_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->dont_hide_cursor_list, validation_context,
                                         &dont_hide_cursor_list_validate_params)) {
    return false;
  }

  return true;
}

WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data::WindowManagerClient_SetKeyEventsThatDontHideCursor_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SetDisplayRoot_Params_Data* WindowManagerClient_SetDisplayRoot_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SetDisplayRoot_Params_Data))) WindowManagerClient_SetDisplayRoot_Params_Data();
}

// static
bool WindowManagerClient_SetDisplayRoot_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SetDisplayRoot_Params_Data* object = static_cast<const WindowManagerClient_SetDisplayRoot_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->display, "null display field in WindowManagerClient_SetDisplayRoot_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->display, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->viewport_metrics, "null viewport_metrics field in WindowManagerClient_SetDisplayRoot_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->viewport_metrics, validation_context))
    return false;

  return true;
}

WindowManagerClient_SetDisplayRoot_Params_Data::WindowManagerClient_SetDisplayRoot_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SetDisplayRoot_ResponseParams_Data* WindowManagerClient_SetDisplayRoot_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SetDisplayRoot_ResponseParams_Data))) WindowManagerClient_SetDisplayRoot_ResponseParams_Data();
}

// static
bool WindowManagerClient_SetDisplayRoot_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SetDisplayRoot_ResponseParams_Data* object = static_cast<const WindowManagerClient_SetDisplayRoot_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->local_surface_id, validation_context))
    return false;

  return true;
}

WindowManagerClient_SetDisplayRoot_ResponseParams_Data::WindowManagerClient_SetDisplayRoot_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SetDisplayConfiguration_Params_Data* WindowManagerClient_SetDisplayConfiguration_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SetDisplayConfiguration_Params_Data))) WindowManagerClient_SetDisplayConfiguration_Params_Data();
}

// static
bool WindowManagerClient_SetDisplayConfiguration_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SetDisplayConfiguration_Params_Data* object = static_cast<const WindowManagerClient_SetDisplayConfiguration_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 40 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->displays, "null displays field in WindowManagerClient_SetDisplayConfiguration_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams displays_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->displays, validation_context,
                                         &displays_validate_params)) {
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->viewport_metrics, "null viewport_metrics field in WindowManagerClient_SetDisplayConfiguration_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams viewport_metrics_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->viewport_metrics, validation_context,
                                         &viewport_metrics_validate_params)) {
    return false;
  }

  return true;
}

WindowManagerClient_SetDisplayConfiguration_Params_Data::WindowManagerClient_SetDisplayConfiguration_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data* WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data))) WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data();
}

// static
bool WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data* object = static_cast<const WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data::WindowManagerClient_SetDisplayConfiguration_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SwapDisplayRoots_Params_Data* WindowManagerClient_SwapDisplayRoots_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SwapDisplayRoots_Params_Data))) WindowManagerClient_SwapDisplayRoots_Params_Data();
}

// static
bool WindowManagerClient_SwapDisplayRoots_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SwapDisplayRoots_Params_Data* object = static_cast<const WindowManagerClient_SwapDisplayRoots_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_SwapDisplayRoots_Params_Data::WindowManagerClient_SwapDisplayRoots_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_SwapDisplayRoots_ResponseParams_Data* WindowManagerClient_SwapDisplayRoots_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_SwapDisplayRoots_ResponseParams_Data))) WindowManagerClient_SwapDisplayRoots_ResponseParams_Data();
}

// static
bool WindowManagerClient_SwapDisplayRoots_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_SwapDisplayRoots_ResponseParams_Data* object = static_cast<const WindowManagerClient_SwapDisplayRoots_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_SwapDisplayRoots_ResponseParams_Data::WindowManagerClient_SwapDisplayRoots_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmResponse_Params_Data* WindowManagerClient_WmResponse_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmResponse_Params_Data))) WindowManagerClient_WmResponse_Params_Data();
}

// static
bool WindowManagerClient_WmResponse_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmResponse_Params_Data* object = static_cast<const WindowManagerClient_WmResponse_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_WmResponse_Params_Data::WindowManagerClient_WmResponse_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmSetBoundsResponse_Params_Data* WindowManagerClient_WmSetBoundsResponse_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmSetBoundsResponse_Params_Data))) WindowManagerClient_WmSetBoundsResponse_Params_Data();
}

// static
bool WindowManagerClient_WmSetBoundsResponse_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmSetBoundsResponse_Params_Data* object = static_cast<const WindowManagerClient_WmSetBoundsResponse_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_WmSetBoundsResponse_Params_Data::WindowManagerClient_WmSetBoundsResponse_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmRequestClose_Params_Data* WindowManagerClient_WmRequestClose_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmRequestClose_Params_Data))) WindowManagerClient_WmRequestClose_Params_Data();
}

// static
bool WindowManagerClient_WmRequestClose_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmRequestClose_Params_Data* object = static_cast<const WindowManagerClient_WmRequestClose_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_WmRequestClose_Params_Data::WindowManagerClient_WmRequestClose_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmSetFrameDecorationValues_Params_Data* WindowManagerClient_WmSetFrameDecorationValues_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmSetFrameDecorationValues_Params_Data))) WindowManagerClient_WmSetFrameDecorationValues_Params_Data();
}

// static
bool WindowManagerClient_WmSetFrameDecorationValues_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmSetFrameDecorationValues_Params_Data* object = static_cast<const WindowManagerClient_WmSetFrameDecorationValues_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->values, "null values field in WindowManagerClient_WmSetFrameDecorationValues_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->values, validation_context))
    return false;

  return true;
}

WindowManagerClient_WmSetFrameDecorationValues_Params_Data::WindowManagerClient_WmSetFrameDecorationValues_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmSetNonClientCursor_Params_Data* WindowManagerClient_WmSetNonClientCursor_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmSetNonClientCursor_Params_Data))) WindowManagerClient_WmSetNonClientCursor_Params_Data();
}

// static
bool WindowManagerClient_WmSetNonClientCursor_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmSetNonClientCursor_Params_Data* object = static_cast<const WindowManagerClient_WmSetNonClientCursor_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->cursor, "null cursor field in WindowManagerClient_WmSetNonClientCursor_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->cursor, validation_context))
    return false;

  return true;
}

WindowManagerClient_WmSetNonClientCursor_Params_Data::WindowManagerClient_WmSetNonClientCursor_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmLockCursor_Params_Data* WindowManagerClient_WmLockCursor_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmLockCursor_Params_Data))) WindowManagerClient_WmLockCursor_Params_Data();
}

// static
bool WindowManagerClient_WmLockCursor_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmLockCursor_Params_Data* object = static_cast<const WindowManagerClient_WmLockCursor_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_WmLockCursor_Params_Data::WindowManagerClient_WmLockCursor_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmUnlockCursor_Params_Data* WindowManagerClient_WmUnlockCursor_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmUnlockCursor_Params_Data))) WindowManagerClient_WmUnlockCursor_Params_Data();
}

// static
bool WindowManagerClient_WmUnlockCursor_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmUnlockCursor_Params_Data* object = static_cast<const WindowManagerClient_WmUnlockCursor_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_WmUnlockCursor_Params_Data::WindowManagerClient_WmUnlockCursor_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmSetCursorVisible_Params_Data* WindowManagerClient_WmSetCursorVisible_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmSetCursorVisible_Params_Data))) WindowManagerClient_WmSetCursorVisible_Params_Data();
}

// static
bool WindowManagerClient_WmSetCursorVisible_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmSetCursorVisible_Params_Data* object = static_cast<const WindowManagerClient_WmSetCursorVisible_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_WmSetCursorVisible_Params_Data::WindowManagerClient_WmSetCursorVisible_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmSetCursorSize_Params_Data* WindowManagerClient_WmSetCursorSize_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmSetCursorSize_Params_Data))) WindowManagerClient_WmSetCursorSize_Params_Data();
}

// static
bool WindowManagerClient_WmSetCursorSize_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmSetCursorSize_Params_Data* object = static_cast<const WindowManagerClient_WmSetCursorSize_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::ui::mojom::internal::CursorSize_Data
        ::Validate(object->cursor_size, validation_context))
    return false;

  return true;
}

WindowManagerClient_WmSetCursorSize_Params_Data::WindowManagerClient_WmSetCursorSize_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data* WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data))) WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data();
}

// static
bool WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data* object = static_cast<const WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->cursor, validation_context))
    return false;

  return true;
}

WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data::WindowManagerClient_WmSetGlobalOverrideCursor_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data* WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data))) WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data();
}

// static
bool WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data* object = static_cast<const WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->display_pixels, "null display_pixels field in WindowManagerClient_WmMoveCursorToDisplayLocation_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->display_pixels, validation_context))
    return false;

  return true;
}

WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data::WindowManagerClient_WmMoveCursorToDisplayLocation_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data* WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data))) WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data();
}

// static
bool WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data* object = static_cast<const WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data::WindowManagerClient_OnWmCreatedTopLevelWindow_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
WindowManagerClient_OnAcceleratorAck_Params_Data* WindowManagerClient_OnAcceleratorAck_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(WindowManagerClient_OnAcceleratorAck_Params_Data))) WindowManagerClient_OnAcceleratorAck_Params_Data();
}

// static
bool WindowManagerClient_OnAcceleratorAck_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const WindowManagerClient_OnAcceleratorAck_Params_Data* object = static_cast<const WindowManagerClient_OnAcceleratorAck_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::ui::mojom::internal::EventResult_Data
        ::Validate(object->event_result, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->properties, "null properties field in WindowManagerClient_OnAcceleratorAck_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams properties_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
  if (!mojo::internal::ValidateContainer(object->properties, validation_context,
                                         &properties_validate_params)) {
    return false;
  }

  return true;
}

WindowManagerClient_OnAcceleratorAck_Params_Data::WindowManagerClient_OnAcceleratorAck_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace ui

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
