// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/ui/public/interfaces/window_tree_host.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace ui {
namespace mojom {
const char WindowTreeHost::Name_[] = "ui::mojom::WindowTreeHost";

WindowTreeHostProxy::WindowTreeHostProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WindowTreeHostProxy::SetSize(
    const gfx::Size& in_size) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeHost_SetSize_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::SizeDataView>(
      in_size, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeHost_SetSize_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeHost_SetSize_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->size)::BaseType* size_ptr;
  mojo::internal::Serialize<::gfx::mojom::SizeDataView>(
      in_size, buffer, &size_ptr, &serialization_context);
  params->size.Set(size_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->size.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null size in WindowTreeHost.SetSize request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeHostProxy::SetTitle(
    const std::string& in_title) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeHost_SetTitle_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_title, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeHost_SetTitle_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeHost_SetTitle_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->title)::BaseType* title_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_title, buffer, &title_ptr, &serialization_context);
  params->title.Set(title_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->title.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null title in WindowTreeHost.SetTitle request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool WindowTreeHostStubDispatch::Accept(
    WindowTreeHost* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWindowTreeHost_SetSize_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeHost_SetSize_Params_Data* params =
          reinterpret_cast<internal::WindowTreeHost_SetSize_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      gfx::Size p_size{};
      WindowTreeHost_SetSize_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadSize(&p_size))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeHost::SetSize deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetSize(
std::move(p_size));
      return true;
    }
    case internal::kWindowTreeHost_SetTitle_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeHost_SetTitle_Params_Data* params =
          reinterpret_cast<internal::WindowTreeHost_SetTitle_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::string p_title{};
      WindowTreeHost_SetTitle_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadTitle(&p_title))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeHost::SetTitle deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetTitle(
std::move(p_title));
      return true;
    }
  }
  return false;
}

// static
bool WindowTreeHostStubDispatch::AcceptWithResponder(
    WindowTreeHost* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWindowTreeHost_SetSize_Name: {
      break;
    }
    case internal::kWindowTreeHost_SetTitle_Name: {
      break;
    }
  }
  return false;
}

bool WindowTreeHostRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WindowTreeHost RequestValidator");

  switch (message->header()->name) {
    case internal::kWindowTreeHost_SetSize_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeHost_SetSize_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeHost_SetTitle_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeHost_SetTitle_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

const char WindowTreeHostFactory::Name_[] = "ui::mojom::WindowTreeHostFactory";

WindowTreeHostFactoryProxy::WindowTreeHostFactoryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WindowTreeHostFactoryProxy::CreateWindowTreeHost(
    WindowTreeHostRequest in_window_tree_host, ::ui::mojom::WindowTreeClientPtr in_tree_client) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeHostFactory_CreateWindowTreeHost_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowTreeHostRequestDataView>(
      in_window_tree_host, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowTreeClientPtrDataView>(
      in_tree_client, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeHostFactory_CreateWindowTreeHost_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeHostFactory_CreateWindowTreeHost_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::ui::mojom::WindowTreeHostRequestDataView>(
      in_window_tree_host, &params->window_tree_host, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->window_tree_host),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid window_tree_host in WindowTreeHostFactory.CreateWindowTreeHost request");
  mojo::internal::Serialize<::ui::mojom::WindowTreeClientPtrDataView>(
      in_tree_client, &params->tree_client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->tree_client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid tree_client in WindowTreeHostFactory.CreateWindowTreeHost request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool WindowTreeHostFactoryStubDispatch::Accept(
    WindowTreeHostFactory* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWindowTreeHostFactory_CreateWindowTreeHost_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeHostFactory_CreateWindowTreeHost_Params_Data* params =
          reinterpret_cast<internal::WindowTreeHostFactory_CreateWindowTreeHost_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WindowTreeHostRequest p_window_tree_host{};
      ::ui::mojom::WindowTreeClientPtr p_tree_client{};
      WindowTreeHostFactory_CreateWindowTreeHost_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_tree_host =
          input_data_view.TakeWindowTreeHost<decltype(p_window_tree_host)>();
      p_tree_client =
          input_data_view.TakeTreeClient<decltype(p_tree_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeHostFactory::CreateWindowTreeHost deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateWindowTreeHost(
std::move(p_window_tree_host), 
std::move(p_tree_client));
      return true;
    }
  }
  return false;
}

// static
bool WindowTreeHostFactoryStubDispatch::AcceptWithResponder(
    WindowTreeHostFactory* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWindowTreeHostFactory_CreateWindowTreeHost_Name: {
      break;
    }
  }
  return false;
}

bool WindowTreeHostFactoryRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WindowTreeHostFactory RequestValidator");

  switch (message->header()->name) {
    case internal::kWindowTreeHostFactory_CreateWindowTreeHost_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeHostFactory_CreateWindowTreeHost_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif