// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef SERVICES_VIZ_HIT_TEST_PUBLIC_INTERFACES_HIT_TEST_REGION_LIST_MOJOM_SHARED_H_
#define SERVICES_VIZ_HIT_TEST_PUBLIC_INTERFACES_HIT_TEST_REGION_LIST_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "services/viz/hit_test/public/interfaces/hit_test_region_list.mojom-shared-internal.h"
#include "cc/ipc/surface_id.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "ui/gfx/mojo/transform.mojom-shared.h"


namespace viz {
namespace mojom {
class HitTestRegionDataView;

class HitTestRegionListDataView;



}  // namespace mojom
}  // namespace viz

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::viz::mojom::HitTestRegionDataView> {
  using Data = ::viz::mojom::internal::HitTestRegion_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::viz::mojom::HitTestRegionListDataView> {
  using Data = ::viz::mojom::internal::HitTestRegionList_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace viz {
namespace mojom {
class HitTestRegionDataView {
 public:
  HitTestRegionDataView() {}

  HitTestRegionDataView(
      internal::HitTestRegion_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint32_t flags() const {
    return data_->flags;
  }
  inline void GetSurfaceIdDataView(
      ::cc::mojom::SurfaceIdDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSurfaceId(UserType* output) {
    auto* pointer = data_->surface_id.Get();
    return mojo::internal::Deserialize<::cc::mojom::SurfaceIdDataView>(
        pointer, output, context_);
  }
  inline void GetRectDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRect(UserType* output) {
    auto* pointer = data_->rect.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetTransformDataView(
      ::gfx::mojom::TransformDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTransform(UserType* output) {
    auto* pointer = data_->transform.Get();
    return mojo::internal::Deserialize<::gfx::mojom::TransformDataView>(
        pointer, output, context_);
  }
 private:
  internal::HitTestRegion_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class HitTestRegionListDataView {
 public:
  HitTestRegionListDataView() {}

  HitTestRegionListDataView(
      internal::HitTestRegionList_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSurfaceIdDataView(
      ::cc::mojom::SurfaceIdDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSurfaceId(UserType* output) {
    auto* pointer = data_->surface_id.Get();
    return mojo::internal::Deserialize<::cc::mojom::SurfaceIdDataView>(
        pointer, output, context_);
  }
  uint32_t flags() const {
    return data_->flags;
  }
  inline void GetBoundsDataView(
      ::gfx::mojom::RectDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBounds(UserType* output) {
    auto* pointer = data_->bounds.Get();
    return mojo::internal::Deserialize<::gfx::mojom::RectDataView>(
        pointer, output, context_);
  }
  inline void GetTransformDataView(
      ::gfx::mojom::TransformDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTransform(UserType* output) {
    auto* pointer = data_->transform.Get();
    return mojo::internal::Deserialize<::gfx::mojom::TransformDataView>(
        pointer, output, context_);
  }
  inline void GetRegionsDataView(
      mojo::ArrayDataView<HitTestRegionDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRegions(UserType* output) {
    auto* pointer = data_->regions.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::viz::mojom::HitTestRegionDataView>>(
        pointer, output, context_);
  }
 private:
  internal::HitTestRegionList_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace viz

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::viz::mojom::HitTestRegionDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::viz::mojom::HitTestRegionDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::viz::mojom::internal::HitTestRegion_Data);
    decltype(CallWithContext(Traits::surface_id, input, custom_context)) in_surface_id = CallWithContext(Traits::surface_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::cc::mojom::SurfaceIdDataView>(
        in_surface_id, context);
    decltype(CallWithContext(Traits::rect, input, custom_context)) in_rect = CallWithContext(Traits::rect, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
        in_rect, context);
    decltype(CallWithContext(Traits::transform, input, custom_context)) in_transform = CallWithContext(Traits::transform, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::TransformDataView>(
        in_transform, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::viz::mojom::internal::HitTestRegion_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::viz::mojom::internal::HitTestRegion_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->flags = CallWithContext(Traits::flags, input, custom_context);
    decltype(CallWithContext(Traits::surface_id, input, custom_context)) in_surface_id = CallWithContext(Traits::surface_id, input, custom_context);
    typename decltype(result->surface_id)::BaseType* surface_id_ptr;
    mojo::internal::Serialize<::cc::mojom::SurfaceIdDataView>(
        in_surface_id, buffer, &surface_id_ptr, context);
    result->surface_id.Set(surface_id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->surface_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null surface_id in HitTestRegion struct");
    decltype(CallWithContext(Traits::rect, input, custom_context)) in_rect = CallWithContext(Traits::rect, input, custom_context);
    typename decltype(result->rect)::BaseType* rect_ptr;
    mojo::internal::Serialize<::gfx::mojom::RectDataView>(
        in_rect, buffer, &rect_ptr, context);
    result->rect.Set(rect_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->rect.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null rect in HitTestRegion struct");
    decltype(CallWithContext(Traits::transform, input, custom_context)) in_transform = CallWithContext(Traits::transform, input, custom_context);
    typename decltype(result->transform)::BaseType* transform_ptr;
    mojo::internal::Serialize<::gfx::mojom::TransformDataView>(
        in_transform, buffer, &transform_ptr, context);
    result->transform.Set(transform_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->transform.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null transform in HitTestRegion struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::viz::mojom::internal::HitTestRegion_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::viz::mojom::HitTestRegionDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::viz::mojom::HitTestRegionListDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::viz::mojom::HitTestRegionListDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::viz::mojom::internal::HitTestRegionList_Data);
    decltype(CallWithContext(Traits::surface_id, input, custom_context)) in_surface_id = CallWithContext(Traits::surface_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::cc::mojom::SurfaceIdDataView>(
        in_surface_id, context);
    decltype(CallWithContext(Traits::bounds, input, custom_context)) in_bounds = CallWithContext(Traits::bounds, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
        in_bounds, context);
    decltype(CallWithContext(Traits::transform, input, custom_context)) in_transform = CallWithContext(Traits::transform, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::gfx::mojom::TransformDataView>(
        in_transform, context);
    decltype(CallWithContext(Traits::regions, input, custom_context)) in_regions = CallWithContext(Traits::regions, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::viz::mojom::HitTestRegionDataView>>(
        in_regions, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::viz::mojom::internal::HitTestRegionList_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::viz::mojom::internal::HitTestRegionList_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::surface_id, input, custom_context)) in_surface_id = CallWithContext(Traits::surface_id, input, custom_context);
    typename decltype(result->surface_id)::BaseType* surface_id_ptr;
    mojo::internal::Serialize<::cc::mojom::SurfaceIdDataView>(
        in_surface_id, buffer, &surface_id_ptr, context);
    result->surface_id.Set(surface_id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->surface_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null surface_id in HitTestRegionList struct");
    result->flags = CallWithContext(Traits::flags, input, custom_context);
    decltype(CallWithContext(Traits::bounds, input, custom_context)) in_bounds = CallWithContext(Traits::bounds, input, custom_context);
    typename decltype(result->bounds)::BaseType* bounds_ptr;
    mojo::internal::Serialize<::gfx::mojom::RectDataView>(
        in_bounds, buffer, &bounds_ptr, context);
    result->bounds.Set(bounds_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->bounds.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null bounds in HitTestRegionList struct");
    decltype(CallWithContext(Traits::transform, input, custom_context)) in_transform = CallWithContext(Traits::transform, input, custom_context);
    typename decltype(result->transform)::BaseType* transform_ptr;
    mojo::internal::Serialize<::gfx::mojom::TransformDataView>(
        in_transform, buffer, &transform_ptr, context);
    result->transform.Set(transform_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->transform.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null transform in HitTestRegionList struct");
    decltype(CallWithContext(Traits::regions, input, custom_context)) in_regions = CallWithContext(Traits::regions, input, custom_context);
    typename decltype(result->regions)::BaseType* regions_ptr;
    const mojo::internal::ContainerValidateParams regions_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::viz::mojom::HitTestRegionDataView>>(
        in_regions, buffer, &regions_ptr, &regions_validate_params,
        context);
    result->regions.Set(regions_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->regions.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null regions in HitTestRegionList struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::viz::mojom::internal::HitTestRegionList_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::viz::mojom::HitTestRegionListDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace viz {
namespace mojom {

inline void HitTestRegionDataView::GetSurfaceIdDataView(
    ::cc::mojom::SurfaceIdDataView* output) {
  auto pointer = data_->surface_id.Get();
  *output = ::cc::mojom::SurfaceIdDataView(pointer, context_);
}
inline void HitTestRegionDataView::GetRectDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->rect.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void HitTestRegionDataView::GetTransformDataView(
    ::gfx::mojom::TransformDataView* output) {
  auto pointer = data_->transform.Get();
  *output = ::gfx::mojom::TransformDataView(pointer, context_);
}


inline void HitTestRegionListDataView::GetSurfaceIdDataView(
    ::cc::mojom::SurfaceIdDataView* output) {
  auto pointer = data_->surface_id.Get();
  *output = ::cc::mojom::SurfaceIdDataView(pointer, context_);
}
inline void HitTestRegionListDataView::GetBoundsDataView(
    ::gfx::mojom::RectDataView* output) {
  auto pointer = data_->bounds.Get();
  *output = ::gfx::mojom::RectDataView(pointer, context_);
}
inline void HitTestRegionListDataView::GetTransformDataView(
    ::gfx::mojom::TransformDataView* output) {
  auto pointer = data_->transform.Get();
  *output = ::gfx::mojom::TransformDataView(pointer, context_);
}
inline void HitTestRegionListDataView::GetRegionsDataView(
    mojo::ArrayDataView<HitTestRegionDataView>* output) {
  auto pointer = data_->regions.Get();
  *output = mojo::ArrayDataView<HitTestRegionDataView>(pointer, context_);
}



}  // namespace mojom
}  // namespace viz

#endif  // SERVICES_VIZ_HIT_TEST_PUBLIC_INTERFACES_HIT_TEST_REGION_LIST_MOJOM_SHARED_H_
