// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_BLINK_H_
#define STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "storage/public/interfaces/blobs.mojom-shared.h"
#include "mojo/common/file.mojom-blink.h"
#include "mojo/common/file_path.mojom-blink.h"
#include "mojo/common/time.mojom-blink.h"
#include "url/mojo/url.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "third_party/WebKit/public/platform/WebCommon.h"


namespace storage {
namespace mojom {
namespace blink {
class Blob;
using BlobPtr = mojo::InterfacePtr<Blob>;
using BlobPtrInfo = mojo::InterfacePtrInfo<Blob>;
using ThreadSafeBlobPtr =
    mojo::ThreadSafeInterfacePtr<Blob>;
using BlobRequest = mojo::InterfaceRequest<Blob>;
using BlobAssociatedPtr =
    mojo::AssociatedInterfacePtr<Blob>;
using ThreadSafeBlobAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<Blob>;
using BlobAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<Blob>;
using BlobAssociatedRequest =
    mojo::AssociatedInterfaceRequest<Blob>;

class BlobRegistry;
using BlobRegistryPtr = mojo::InterfacePtr<BlobRegistry>;
using BlobRegistryPtrInfo = mojo::InterfacePtrInfo<BlobRegistry>;
using ThreadSafeBlobRegistryPtr =
    mojo::ThreadSafeInterfacePtr<BlobRegistry>;
using BlobRegistryRequest = mojo::InterfaceRequest<BlobRegistry>;
using BlobRegistryAssociatedPtr =
    mojo::AssociatedInterfacePtr<BlobRegistry>;
using ThreadSafeBlobRegistryAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BlobRegistry>;
using BlobRegistryAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BlobRegistry>;
using BlobRegistryAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BlobRegistry>;

class BytesProvider;
using BytesProviderPtr = mojo::InterfacePtr<BytesProvider>;
using BytesProviderPtrInfo = mojo::InterfacePtrInfo<BytesProvider>;
using ThreadSafeBytesProviderPtr =
    mojo::ThreadSafeInterfacePtr<BytesProvider>;
using BytesProviderRequest = mojo::InterfaceRequest<BytesProvider>;
using BytesProviderAssociatedPtr =
    mojo::AssociatedInterfacePtr<BytesProvider>;
using ThreadSafeBytesProviderAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BytesProvider>;
using BytesProviderAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BytesProvider>;
using BytesProviderAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BytesProvider>;

class DataElementBytes;
using DataElementBytesPtr = mojo::StructPtr<DataElementBytes>;

class DataElementFile;
using DataElementFilePtr = mojo::StructPtr<DataElementFile>;

class DataElementFilesystemURL;
using DataElementFilesystemURLPtr = mojo::StructPtr<DataElementFilesystemURL>;

class DataElementBlob;
using DataElementBlobPtr = mojo::StructPtr<DataElementBlob>;

class DataElement;

typedef mojo::StructPtr<DataElement> DataElementPtr;


class BlobProxy;

template <typename ImplRefTraits>
class BlobStub;

class BlobRequestValidator;
class BlobResponseValidator;

class BLINK_PLATFORM_EXPORT Blob
    : public BlobInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = BlobProxy;

  template <typename ImplRefTraits>
  using Stub_ = BlobStub<ImplRefTraits>;

  using RequestValidator_ = BlobRequestValidator;
  using ResponseValidator_ = BlobResponseValidator;
  enum MethodMinVersions : uint32_t {
    kCloneMinVersion = 0,
    kGetInternalUUIDMinVersion = 0,
  };
  virtual ~Blob() {}

  virtual void Clone(BlobRequest blob) = 0;


  using GetInternalUUIDCallback = base::OnceCallback<void(const WTF::String&)>;
  virtual void GetInternalUUID(GetInternalUUIDCallback callback) = 0;
};

class BlobRegistryProxy;

template <typename ImplRefTraits>
class BlobRegistryStub;

class BlobRegistryRequestValidator;
class BlobRegistryResponseValidator;

class BLINK_PLATFORM_EXPORT BlobRegistry
    : public BlobRegistryInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = BlobRegistryProxy;

  template <typename ImplRefTraits>
  using Stub_ = BlobRegistryStub<ImplRefTraits>;

  using RequestValidator_ = BlobRegistryRequestValidator;
  using ResponseValidator_ = BlobRegistryResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRegisterMinVersion = 0,
    kGetBlobFromUUIDMinVersion = 0,
  };
  virtual ~BlobRegistry() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool Register(BlobRequest blob, const WTF::String& uuid, const WTF::String& content_type, const WTF::String& content_disposition, WTF::Vector<DataElementPtr> elements);

  using RegisterCallback = base::OnceCallback<void()>;
  virtual void Register(BlobRequest blob, const WTF::String& uuid, const WTF::String& content_type, const WTF::String& content_disposition, WTF::Vector<DataElementPtr> elements, RegisterCallback callback) = 0;

  virtual void GetBlobFromUUID(BlobRequest blob, const WTF::String& uuid) = 0;
};

class BytesProviderProxy;

template <typename ImplRefTraits>
class BytesProviderStub;

class BytesProviderRequestValidator;
class BytesProviderResponseValidator;

class BLINK_PLATFORM_EXPORT BytesProvider
    : public BytesProviderInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = BytesProviderProxy;

  template <typename ImplRefTraits>
  using Stub_ = BytesProviderStub<ImplRefTraits>;

  using RequestValidator_ = BytesProviderRequestValidator;
  using ResponseValidator_ = BytesProviderResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRequestAsReplyMinVersion = 0,
    kRequestAsStreamMinVersion = 0,
    kRequestAsFileMinVersion = 0,
  };
  virtual ~BytesProvider() {}


  using RequestAsReplyCallback = base::OnceCallback<void(const WTF::Vector<uint8_t>&)>;
  virtual void RequestAsReply(RequestAsReplyCallback callback) = 0;

  virtual void RequestAsStream(mojo::ScopedDataPipeProducerHandle pipe) = 0;


  using RequestAsFileCallback = base::OnceCallback<void(WTF::Optional<WTF::Time>)>;
  virtual void RequestAsFile(uint64_t source_offset, uint64_t source_size, base::File file, uint64_t file_offset, RequestAsFileCallback callback) = 0;
};

class BLINK_PLATFORM_EXPORT BlobProxy
    : public Blob {
 public:
  explicit BlobProxy(mojo::MessageReceiverWithResponder* receiver);
  void Clone(BlobRequest blob) override;
  void GetInternalUUID(GetInternalUUIDCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class BLINK_PLATFORM_EXPORT BlobRegistryProxy
    : public BlobRegistry {
 public:
  explicit BlobRegistryProxy(mojo::MessageReceiverWithResponder* receiver);
  bool Register(BlobRequest blob, const WTF::String& uuid, const WTF::String& content_type, const WTF::String& content_disposition, WTF::Vector<DataElementPtr> elements) override;
  void Register(BlobRequest blob, const WTF::String& uuid, const WTF::String& content_type, const WTF::String& content_disposition, WTF::Vector<DataElementPtr> elements, RegisterCallback callback) override;
  void GetBlobFromUUID(BlobRequest blob, const WTF::String& uuid) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class BLINK_PLATFORM_EXPORT BytesProviderProxy
    : public BytesProvider {
 public:
  explicit BytesProviderProxy(mojo::MessageReceiverWithResponder* receiver);
  void RequestAsReply(RequestAsReplyCallback callback) override;
  void RequestAsStream(mojo::ScopedDataPipeProducerHandle pipe) override;
  void RequestAsFile(uint64_t source_offset, uint64_t source_size, base::File file, uint64_t file_offset, RequestAsFileCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT BlobStubDispatch {
 public:
  static bool Accept(Blob* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      Blob* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<Blob>>
class BlobStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BlobStub() {}
  ~BlobStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT BlobRegistryStubDispatch {
 public:
  static bool Accept(BlobRegistry* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BlobRegistry* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BlobRegistry>>
class BlobRegistryStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BlobRegistryStub() {}
  ~BlobRegistryStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobRegistryStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BlobRegistryStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT BytesProviderStubDispatch {
 public:
  static bool Accept(BytesProvider* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BytesProvider* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BytesProvider>>
class BytesProviderStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BytesProviderStub() {}
  ~BytesProviderStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BytesProviderStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BytesProviderStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT BlobRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT BlobRegistryRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT BytesProviderRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT BlobResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT BlobRegistryResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT BytesProviderResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};






class BLINK_PLATFORM_EXPORT DataElement {
 public:
  using DataView = DataElementDataView;
  using Data_ = internal::DataElement_Data;
  using Tag = Data_::DataElement_Tag;

  static DataElementPtr New() {
    return DataElementPtr(base::in_place);
  }
  // Construct an instance holding |bytes|.
  static DataElementPtr
  NewBytes(
      DataElementBytesPtr bytes) {
    auto result = DataElementPtr(base::in_place);
    result->set_bytes(std::move(bytes));
    return result;
  }
  // Construct an instance holding |file|.
  static DataElementPtr
  NewFile(
      DataElementFilePtr file) {
    auto result = DataElementPtr(base::in_place);
    result->set_file(std::move(file));
    return result;
  }
  // Construct an instance holding |file_filesystem|.
  static DataElementPtr
  NewFileFilesystem(
      DataElementFilesystemURLPtr file_filesystem) {
    auto result = DataElementPtr(base::in_place);
    result->set_file_filesystem(std::move(file_filesystem));
    return result;
  }
  // Construct an instance holding |blob|.
  static DataElementPtr
  NewBlob(
      DataElementBlobPtr blob) {
    auto result = DataElementPtr(base::in_place);
    result->set_blob(std::move(blob));
    return result;
  }

  template <typename U>
  static DataElementPtr From(const U& u) {
    return mojo::TypeConverter<DataElementPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElement>::Convert(*this);
  }

  DataElement();
  ~DataElement();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename UnionPtrType = DataElementPtr>
  DataElementPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElement>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  Tag which() const {
    return tag_;
  }


  bool is_bytes() const { return tag_ == Tag::BYTES; }

  DataElementBytesPtr& get_bytes() const {
    DCHECK(tag_ == Tag::BYTES);
    return *(data_.bytes);
  }

  void set_bytes(DataElementBytesPtr bytes);
  bool is_file() const { return tag_ == Tag::FILE; }

  DataElementFilePtr& get_file() const {
    DCHECK(tag_ == Tag::FILE);
    return *(data_.file);
  }

  void set_file(DataElementFilePtr file);
  bool is_file_filesystem() const { return tag_ == Tag::FILE_FILESYSTEM; }

  DataElementFilesystemURLPtr& get_file_filesystem() const {
    DCHECK(tag_ == Tag::FILE_FILESYSTEM);
    return *(data_.file_filesystem);
  }

  void set_file_filesystem(DataElementFilesystemURLPtr file_filesystem);
  bool is_blob() const { return tag_ == Tag::BLOB; }

  DataElementBlobPtr& get_blob() const {
    DCHECK(tag_ == Tag::BLOB);
    return *(data_.blob);
  }

  void set_blob(DataElementBlobPtr blob);

 private:
  union Union_ {
    Union_() {}
    ~Union_() {}
    DataElementBytesPtr* bytes;
    DataElementFilePtr* file;
    DataElementFilesystemURLPtr* file_filesystem;
    DataElementBlobPtr* blob;
  };
  void DestroyActive();
  Tag tag_;
  Union_ data_;
};


class BLINK_PLATFORM_EXPORT DataElementBytes {
 public:
  using DataView = DataElementBytesDataView;
  using Data_ = internal::DataElementBytes_Data;
  static constexpr uint64_t kMaximumEmbeddedDataSize = 256000ULL;

  template <typename... Args>
  static DataElementBytesPtr New(Args&&... args) {
    return DataElementBytesPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementBytesPtr From(const U& u) {
    return mojo::TypeConverter<DataElementBytesPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementBytes>::Convert(*this);
  }


  DataElementBytes();

  DataElementBytes(
      uint64_t length,
      const WTF::Optional<WTF::Vector<uint8_t>>& embedded_data,
      BytesProviderPtr data);

  ~DataElementBytes();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementBytesPtr>
  DataElementBytesPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementBytes>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DataElementBytes::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DataElementBytes::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  uint64_t length;
  WTF::Optional<WTF::Vector<uint8_t>> embedded_data;
  BytesProviderPtr data;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(DataElementBytes);
};



class BLINK_PLATFORM_EXPORT DataElementFile {
 public:
  using DataView = DataElementFileDataView;
  using Data_ = internal::DataElementFile_Data;

  template <typename... Args>
  static DataElementFilePtr New(Args&&... args) {
    return DataElementFilePtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementFilePtr From(const U& u) {
    return mojo::TypeConverter<DataElementFilePtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementFile>::Convert(*this);
  }


  DataElementFile();

  DataElementFile(
      const base::FilePath& path,
      uint64_t offset,
      uint64_t length,
      WTF::Optional<WTF::Time> expected_modification_time);

  ~DataElementFile();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementFilePtr>
  DataElementFilePtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementFile>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DataElementFile::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DataElementFile::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  base::FilePath path;
  uint64_t offset;
  uint64_t length;
  WTF::Optional<WTF::Time> expected_modification_time;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class BLINK_PLATFORM_EXPORT DataElementFilesystemURL {
 public:
  using DataView = DataElementFilesystemURLDataView;
  using Data_ = internal::DataElementFilesystemURL_Data;

  template <typename... Args>
  static DataElementFilesystemURLPtr New(Args&&... args) {
    return DataElementFilesystemURLPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementFilesystemURLPtr From(const U& u) {
    return mojo::TypeConverter<DataElementFilesystemURLPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementFilesystemURL>::Convert(*this);
  }


  DataElementFilesystemURL();

  DataElementFilesystemURL(
      const ::blink::KURL& url,
      uint64_t offset,
      uint64_t length,
      WTF::Optional<WTF::Time> expected_modification_time);

  ~DataElementFilesystemURL();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementFilesystemURLPtr>
  DataElementFilesystemURLPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementFilesystemURL>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DataElementFilesystemURL::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DataElementFilesystemURL::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  ::blink::KURL url;
  uint64_t offset;
  uint64_t length;
  WTF::Optional<WTF::Time> expected_modification_time;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class BLINK_PLATFORM_EXPORT DataElementBlob {
 public:
  using DataView = DataElementBlobDataView;
  using Data_ = internal::DataElementBlob_Data;

  template <typename... Args>
  static DataElementBlobPtr New(Args&&... args) {
    return DataElementBlobPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static DataElementBlobPtr From(const U& u) {
    return mojo::TypeConverter<DataElementBlobPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, DataElementBlob>::Convert(*this);
  }


  DataElementBlob();

  DataElementBlob(
      BlobPtr blob,
      uint64_t offset,
      uint64_t length);

  ~DataElementBlob();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = DataElementBlobPtr>
  DataElementBlobPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, DataElementBlob>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        DataElementBlob::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        DataElementBlob::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  BlobPtr blob;
  uint64_t offset;
  uint64_t length;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(DataElementBlob);
};


template <typename UnionPtrType>
DataElementPtr DataElement::Clone() const {
  // Use UnionPtrType to prevent the compiler from trying to compile this
  // without being asked.
  UnionPtrType rv(New());
  switch (tag_) {
    case Tag::BYTES:
      rv->set_bytes(mojo::Clone(*data_.bytes));
      break;
    case Tag::FILE:
      rv->set_file(mojo::Clone(*data_.file));
      break;
    case Tag::FILE_FILESYSTEM:
      rv->set_file_filesystem(mojo::Clone(*data_.file_filesystem));
      break;
    case Tag::BLOB:
      rv->set_blob(mojo::Clone(*data_.blob));
      break;
  };
  return rv;
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElement>::value>::type*>
bool DataElement::Equals(const T& other) const {
  if (tag_ != other.which())
    return false;

  switch (tag_) {
    case Tag::BYTES:
      return mojo::Equals(*(data_.bytes), *(other.data_.bytes));
    case Tag::FILE:
      return mojo::Equals(*(data_.file), *(other.data_.file));
    case Tag::FILE_FILESYSTEM:
      return mojo::Equals(*(data_.file_filesystem), *(other.data_.file_filesystem));
    case Tag::BLOB:
      return mojo::Equals(*(data_.blob), *(other.data_.blob));
  };

  return false;
}
template <typename StructPtrType>
DataElementBytesPtr DataElementBytes::Clone() const {
  return New(
      mojo::Clone(length),
      mojo::Clone(embedded_data),
      mojo::Clone(data)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementBytes>::value>::type*>
bool DataElementBytes::Equals(const T& other) const {
  if (!mojo::Equals(this->length, other.length))
    return false;
  if (!mojo::Equals(this->embedded_data, other.embedded_data))
    return false;
  if (!mojo::Equals(this->data, other.data))
    return false;
  return true;
}
template <typename StructPtrType>
DataElementFilePtr DataElementFile::Clone() const {
  return New(
      mojo::Clone(path),
      mojo::Clone(offset),
      mojo::Clone(length),
      mojo::Clone(expected_modification_time)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementFile>::value>::type*>
bool DataElementFile::Equals(const T& other) const {
  if (!mojo::Equals(this->path, other.path))
    return false;
  if (!mojo::Equals(this->offset, other.offset))
    return false;
  if (!mojo::Equals(this->length, other.length))
    return false;
  if (!mojo::Equals(this->expected_modification_time, other.expected_modification_time))
    return false;
  return true;
}
template <typename StructPtrType>
DataElementFilesystemURLPtr DataElementFilesystemURL::Clone() const {
  return New(
      mojo::Clone(url),
      mojo::Clone(offset),
      mojo::Clone(length),
      mojo::Clone(expected_modification_time)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementFilesystemURL>::value>::type*>
bool DataElementFilesystemURL::Equals(const T& other) const {
  if (!mojo::Equals(this->url, other.url))
    return false;
  if (!mojo::Equals(this->offset, other.offset))
    return false;
  if (!mojo::Equals(this->length, other.length))
    return false;
  if (!mojo::Equals(this->expected_modification_time, other.expected_modification_time))
    return false;
  return true;
}
template <typename StructPtrType>
DataElementBlobPtr DataElementBlob::Clone() const {
  return New(
      mojo::Clone(blob),
      mojo::Clone(offset),
      mojo::Clone(length)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, DataElementBlob>::value>::type*>
bool DataElementBlob::Equals(const T& other) const {
  if (!mojo::Equals(this->blob, other.blob))
    return false;
  if (!mojo::Equals(this->offset, other.offset))
    return false;
  if (!mojo::Equals(this->length, other.length))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace storage

namespace mojo {


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::storage::mojom::blink::DataElementBytes::DataView,
                                         ::storage::mojom::blink::DataElementBytesPtr> {
  static bool IsNull(const ::storage::mojom::blink::DataElementBytesPtr& input) { return !input; }
  static void SetToNull(::storage::mojom::blink::DataElementBytesPtr* output) { output->reset(); }

  static decltype(::storage::mojom::blink::DataElementBytes::length) length(
      const ::storage::mojom::blink::DataElementBytesPtr& input) {
    return input->length;
  }

  static const decltype(::storage::mojom::blink::DataElementBytes::embedded_data)& embedded_data(
      const ::storage::mojom::blink::DataElementBytesPtr& input) {
    return input->embedded_data;
  }

  static  decltype(::storage::mojom::blink::DataElementBytes::data)& data(
       ::storage::mojom::blink::DataElementBytesPtr& input) {
    return input->data;
  }

  static bool Read(::storage::mojom::blink::DataElementBytes::DataView input, ::storage::mojom::blink::DataElementBytesPtr* output);
};


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::storage::mojom::blink::DataElementFile::DataView,
                                         ::storage::mojom::blink::DataElementFilePtr> {
  static bool IsNull(const ::storage::mojom::blink::DataElementFilePtr& input) { return !input; }
  static void SetToNull(::storage::mojom::blink::DataElementFilePtr* output) { output->reset(); }

  static const decltype(::storage::mojom::blink::DataElementFile::path)& path(
      const ::storage::mojom::blink::DataElementFilePtr& input) {
    return input->path;
  }

  static decltype(::storage::mojom::blink::DataElementFile::offset) offset(
      const ::storage::mojom::blink::DataElementFilePtr& input) {
    return input->offset;
  }

  static decltype(::storage::mojom::blink::DataElementFile::length) length(
      const ::storage::mojom::blink::DataElementFilePtr& input) {
    return input->length;
  }

  static const decltype(::storage::mojom::blink::DataElementFile::expected_modification_time)& expected_modification_time(
      const ::storage::mojom::blink::DataElementFilePtr& input) {
    return input->expected_modification_time;
  }

  static bool Read(::storage::mojom::blink::DataElementFile::DataView input, ::storage::mojom::blink::DataElementFilePtr* output);
};


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::storage::mojom::blink::DataElementFilesystemURL::DataView,
                                         ::storage::mojom::blink::DataElementFilesystemURLPtr> {
  static bool IsNull(const ::storage::mojom::blink::DataElementFilesystemURLPtr& input) { return !input; }
  static void SetToNull(::storage::mojom::blink::DataElementFilesystemURLPtr* output) { output->reset(); }

  static const decltype(::storage::mojom::blink::DataElementFilesystemURL::url)& url(
      const ::storage::mojom::blink::DataElementFilesystemURLPtr& input) {
    return input->url;
  }

  static decltype(::storage::mojom::blink::DataElementFilesystemURL::offset) offset(
      const ::storage::mojom::blink::DataElementFilesystemURLPtr& input) {
    return input->offset;
  }

  static decltype(::storage::mojom::blink::DataElementFilesystemURL::length) length(
      const ::storage::mojom::blink::DataElementFilesystemURLPtr& input) {
    return input->length;
  }

  static const decltype(::storage::mojom::blink::DataElementFilesystemURL::expected_modification_time)& expected_modification_time(
      const ::storage::mojom::blink::DataElementFilesystemURLPtr& input) {
    return input->expected_modification_time;
  }

  static bool Read(::storage::mojom::blink::DataElementFilesystemURL::DataView input, ::storage::mojom::blink::DataElementFilesystemURLPtr* output);
};


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::storage::mojom::blink::DataElementBlob::DataView,
                                         ::storage::mojom::blink::DataElementBlobPtr> {
  static bool IsNull(const ::storage::mojom::blink::DataElementBlobPtr& input) { return !input; }
  static void SetToNull(::storage::mojom::blink::DataElementBlobPtr* output) { output->reset(); }

  static  decltype(::storage::mojom::blink::DataElementBlob::blob)& blob(
       ::storage::mojom::blink::DataElementBlobPtr& input) {
    return input->blob;
  }

  static decltype(::storage::mojom::blink::DataElementBlob::offset) offset(
      const ::storage::mojom::blink::DataElementBlobPtr& input) {
    return input->offset;
  }

  static decltype(::storage::mojom::blink::DataElementBlob::length) length(
      const ::storage::mojom::blink::DataElementBlobPtr& input) {
    return input->length;
  }

  static bool Read(::storage::mojom::blink::DataElementBlob::DataView input, ::storage::mojom::blink::DataElementBlobPtr* output);
};


template <>
struct BLINK_PLATFORM_EXPORT UnionTraits<::storage::mojom::blink::DataElement::DataView,
                                        ::storage::mojom::blink::DataElementPtr> {
  static bool IsNull(const ::storage::mojom::blink::DataElementPtr& input) { return !input; }
  static void SetToNull(::storage::mojom::blink::DataElementPtr* output) { output->reset(); }

  static ::storage::mojom::blink::DataElement::Tag GetTag(const ::storage::mojom::blink::DataElementPtr& input) {
    return input->which();
  }

  static  ::storage::mojom::blink::DataElementBytesPtr& bytes( ::storage::mojom::blink::DataElementPtr& input) {
    return input->get_bytes();
  }

  static const ::storage::mojom::blink::DataElementFilePtr& file(const ::storage::mojom::blink::DataElementPtr& input) {
    return input->get_file();
  }

  static const ::storage::mojom::blink::DataElementFilesystemURLPtr& file_filesystem(const ::storage::mojom::blink::DataElementPtr& input) {
    return input->get_file_filesystem();
  }

  static  ::storage::mojom::blink::DataElementBlobPtr& blob( ::storage::mojom::blink::DataElementPtr& input) {
    return input->get_blob();
  }

  static bool Read(::storage::mojom::blink::DataElement::DataView input, ::storage::mojom::blink::DataElementPtr* output);
};

}  // namespace mojo

#endif  // STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_BLINK_H_