// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_SHARED_H_
#define STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "storage/public/interfaces/blobs.mojom-shared-internal.h"
#include "mojo/common/file.mojom-shared.h"
#include "mojo/common/file_path.mojom-shared.h"
#include "mojo/common/time.mojom-shared.h"
#include "url/mojo/url.mojom-shared.h"
#include "storage/public/interfaces/interfaces_shared_export.h"


namespace storage {
namespace mojom {
class DataElementBytesDataView;

class DataElementFileDataView;

class DataElementFilesystemURLDataView;

class DataElementBlobDataView;

class DataElementDataView;


}  // namespace mojom
}  // namespace storage

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::storage::mojom::DataElementBytesDataView> {
  using Data = ::storage::mojom::internal::DataElementBytes_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::storage::mojom::DataElementFileDataView> {
  using Data = ::storage::mojom::internal::DataElementFile_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::storage::mojom::DataElementFilesystemURLDataView> {
  using Data = ::storage::mojom::internal::DataElementFilesystemURL_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::storage::mojom::DataElementBlobDataView> {
  using Data = ::storage::mojom::internal::DataElementBlob_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::storage::mojom::DataElementDataView> {
  using Data = ::storage::mojom::internal::DataElement_Data;
  using DataAsArrayElement = Data;
  static constexpr MojomTypeCategory category = MojomTypeCategory::UNION;
};

}  // namespace internal
}  // namespace mojo


namespace storage {
namespace mojom {
// Interface base classes. They are used for type safety check.
class BlobInterfaceBase {};

using BlobPtrDataView =
    mojo::InterfacePtrDataView<BlobInterfaceBase>;
using BlobRequestDataView =
    mojo::InterfaceRequestDataView<BlobInterfaceBase>;
using BlobAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BlobInterfaceBase>;
using BlobAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BlobInterfaceBase>;
class BlobRegistryInterfaceBase {};

using BlobRegistryPtrDataView =
    mojo::InterfacePtrDataView<BlobRegistryInterfaceBase>;
using BlobRegistryRequestDataView =
    mojo::InterfaceRequestDataView<BlobRegistryInterfaceBase>;
using BlobRegistryAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BlobRegistryInterfaceBase>;
using BlobRegistryAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BlobRegistryInterfaceBase>;
class BytesProviderInterfaceBase {};

using BytesProviderPtrDataView =
    mojo::InterfacePtrDataView<BytesProviderInterfaceBase>;
using BytesProviderRequestDataView =
    mojo::InterfaceRequestDataView<BytesProviderInterfaceBase>;
using BytesProviderAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<BytesProviderInterfaceBase>;
using BytesProviderAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<BytesProviderInterfaceBase>;
class DataElementBytesDataView {
 public:
  DataElementBytesDataView() {}

  DataElementBytesDataView(
      internal::DataElementBytes_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint64_t length() const {
    return data_->length;
  }
  inline void GetEmbeddedDataDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEmbeddedData(UserType* output) {
    auto* pointer = data_->embedded_data.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  template <typename UserType>
  UserType TakeData() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::storage::mojom::BytesProviderPtrDataView>(
            &data_->data, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::DataElementBytes_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class DataElementFileDataView {
 public:
  DataElementFileDataView() {}

  DataElementFileDataView(
      internal::DataElementFile_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPathDataView(
      ::mojo::common::mojom::FilePathDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPath(UserType* output) {
    auto* pointer = data_->path.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::FilePathDataView>(
        pointer, output, context_);
  }
  uint64_t offset() const {
    return data_->offset;
  }
  uint64_t length() const {
    return data_->length;
  }
  inline void GetExpectedModificationTimeDataView(
      ::mojo::common::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExpectedModificationTime(UserType* output) {
    auto* pointer = data_->expected_modification_time.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::DataElementFile_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class DataElementFilesystemURLDataView {
 public:
  DataElementFilesystemURLDataView() {}

  DataElementFilesystemURLDataView(
      internal::DataElementFilesystemURL_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  uint64_t offset() const {
    return data_->offset;
  }
  uint64_t length() const {
    return data_->length;
  }
  inline void GetExpectedModificationTimeDataView(
      ::mojo::common::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExpectedModificationTime(UserType* output) {
    auto* pointer = data_->expected_modification_time.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::DataElementFilesystemURL_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class DataElementBlobDataView {
 public:
  DataElementBlobDataView() {}

  DataElementBlobDataView(
      internal::DataElementBlob_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeBlob() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::storage::mojom::BlobPtrDataView>(
            &data_->blob, &result, context_);
    DCHECK(ret);
    return result;
  }
  uint64_t offset() const {
    return data_->offset;
  }
  uint64_t length() const {
    return data_->length;
  }
 private:
  internal::DataElementBlob_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Blob_Clone_ParamsDataView {
 public:
  Blob_Clone_ParamsDataView() {}

  Blob_Clone_ParamsDataView(
      internal::Blob_Clone_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeBlob() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::storage::mojom::BlobRequestDataView>(
            &data_->blob, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::Blob_Clone_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Blob_GetInternalUUID_ParamsDataView {
 public:
  Blob_GetInternalUUID_ParamsDataView() {}

  Blob_GetInternalUUID_ParamsDataView(
      internal::Blob_GetInternalUUID_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::Blob_GetInternalUUID_Params_Data* data_ = nullptr;
};

class Blob_GetInternalUUID_ResponseParamsDataView {
 public:
  Blob_GetInternalUUID_ResponseParamsDataView() {}

  Blob_GetInternalUUID_ResponseParamsDataView(
      internal::Blob_GetInternalUUID_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUuidDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUuid(UserType* output) {
    auto* pointer = data_->uuid.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::Blob_GetInternalUUID_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BlobRegistry_Register_ParamsDataView {
 public:
  BlobRegistry_Register_ParamsDataView() {}

  BlobRegistry_Register_ParamsDataView(
      internal::BlobRegistry_Register_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeBlob() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::storage::mojom::BlobRequestDataView>(
            &data_->blob, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetUuidDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUuid(UserType* output) {
    auto* pointer = data_->uuid.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetContentTypeDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadContentType(UserType* output) {
    auto* pointer = data_->content_type.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetContentDispositionDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadContentDisposition(UserType* output) {
    auto* pointer = data_->content_disposition.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetElementsDataView(
      mojo::ArrayDataView<DataElementDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadElements(UserType* output) {
    auto* pointer = data_->elements.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::storage::mojom::DataElementDataView>>(
        pointer, output, context_);
  }
 private:
  internal::BlobRegistry_Register_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BlobRegistry_Register_ResponseParamsDataView {
 public:
  BlobRegistry_Register_ResponseParamsDataView() {}

  BlobRegistry_Register_ResponseParamsDataView(
      internal::BlobRegistry_Register_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BlobRegistry_Register_ResponseParams_Data* data_ = nullptr;
};

class BlobRegistry_GetBlobFromUUID_ParamsDataView {
 public:
  BlobRegistry_GetBlobFromUUID_ParamsDataView() {}

  BlobRegistry_GetBlobFromUUID_ParamsDataView(
      internal::BlobRegistry_GetBlobFromUUID_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  UserType TakeBlob() {
    UserType result;
    bool ret =
        mojo::internal::Deserialize<::storage::mojom::BlobRequestDataView>(
            &data_->blob, &result, context_);
    DCHECK(ret);
    return result;
  }
  inline void GetUuidDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUuid(UserType* output) {
    auto* pointer = data_->uuid.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::BlobRegistry_GetBlobFromUUID_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BytesProvider_RequestAsReply_ParamsDataView {
 public:
  BytesProvider_RequestAsReply_ParamsDataView() {}

  BytesProvider_RequestAsReply_ParamsDataView(
      internal::BytesProvider_RequestAsReply_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::BytesProvider_RequestAsReply_Params_Data* data_ = nullptr;
};

class BytesProvider_RequestAsReply_ResponseParamsDataView {
 public:
  BytesProvider_RequestAsReply_ResponseParamsDataView() {}

  BytesProvider_RequestAsReply_ResponseParamsDataView(
      internal::BytesProvider_RequestAsReply_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetDataDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadData(UserType* output) {
    auto* pointer = data_->data.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
 private:
  internal::BytesProvider_RequestAsReply_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BytesProvider_RequestAsStream_ParamsDataView {
 public:
  BytesProvider_RequestAsStream_ParamsDataView() {}

  BytesProvider_RequestAsStream_ParamsDataView(
      internal::BytesProvider_RequestAsStream_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  mojo::ScopedDataPipeProducerHandle TakePipe() {
    mojo::ScopedDataPipeProducerHandle result;
    bool ret =
        mojo::internal::Deserialize<mojo::ScopedDataPipeProducerHandle>(
            &data_->pipe, &result, context_);
    DCHECK(ret);
    return result;
  }
 private:
  internal::BytesProvider_RequestAsStream_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BytesProvider_RequestAsFile_ParamsDataView {
 public:
  BytesProvider_RequestAsFile_ParamsDataView() {}

  BytesProvider_RequestAsFile_ParamsDataView(
      internal::BytesProvider_RequestAsFile_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  uint64_t source_offset() const {
    return data_->source_offset;
  }
  uint64_t source_size() const {
    return data_->source_size;
  }
  inline void GetFileDataView(
      ::mojo::common::mojom::FileDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFile(UserType* output) {
    auto* pointer = data_->file.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::FileDataView>(
        pointer, output, context_);
  }
  uint64_t file_offset() const {
    return data_->file_offset;
  }
 private:
  internal::BytesProvider_RequestAsFile_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class BytesProvider_RequestAsFile_ResponseParamsDataView {
 public:
  BytesProvider_RequestAsFile_ResponseParamsDataView() {}

  BytesProvider_RequestAsFile_ResponseParamsDataView(
      internal::BytesProvider_RequestAsFile_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTimeFileModifiedDataView(
      ::mojo::common::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTimeFileModified(UserType* output) {
    auto* pointer = data_->time_file_modified.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::BytesProvider_RequestAsFile_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class DataElementDataView {
 public:
  using Tag = internal::DataElement_Data::DataElement_Tag;

  DataElementDataView() {}

  DataElementDataView(
      internal::DataElement_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const {
    // For inlined unions, |data_| is always non-null. In that case we need to
    // check |data_->is_null()|.
    return !data_ || data_->is_null();
  }

  Tag tag() const { return data_->tag; }
  bool is_bytes() const { return data_->tag == Tag::BYTES; }
  inline void GetBytesDataView(
      DataElementBytesDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBytes(UserType* output) {
    DCHECK(is_bytes());
    return mojo::internal::Deserialize<::storage::mojom::DataElementBytesDataView>(
        data_->data.f_bytes.Get(), output, context_);
  }
  bool is_file() const { return data_->tag == Tag::FILE; }
  inline void GetFileDataView(
      DataElementFileDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFile(UserType* output) {
    DCHECK(is_file());
    return mojo::internal::Deserialize<::storage::mojom::DataElementFileDataView>(
        data_->data.f_file.Get(), output, context_);
  }
  bool is_file_filesystem() const { return data_->tag == Tag::FILE_FILESYSTEM; }
  inline void GetFileFilesystemDataView(
      DataElementFilesystemURLDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadFileFilesystem(UserType* output) {
    DCHECK(is_file_filesystem());
    return mojo::internal::Deserialize<::storage::mojom::DataElementFilesystemURLDataView>(
        data_->data.f_file_filesystem.Get(), output, context_);
  }
  bool is_blob() const { return data_->tag == Tag::BLOB; }
  inline void GetBlobDataView(
      DataElementBlobDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadBlob(UserType* output) {
    DCHECK(is_blob());
    return mojo::internal::Deserialize<::storage::mojom::DataElementBlobDataView>(
        data_->data.f_blob.Get(), output, context_);
  }

 private:
  internal::DataElement_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace storage

namespace std {

}  // namespace std

namespace mojo {


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::storage::mojom::DataElementBytesDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::storage::mojom::DataElementBytesDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::storage::mojom::internal::DataElementBytes_Data);
    decltype(CallWithContext(Traits::embedded_data, input, custom_context)) in_embedded_data = CallWithContext(Traits::embedded_data, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_embedded_data, context);
    decltype(CallWithContext(Traits::data, input, custom_context)) in_data = CallWithContext(Traits::data, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::storage::mojom::BytesProviderPtrDataView>(
        in_data, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::storage::mojom::internal::DataElementBytes_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::storage::mojom::internal::DataElementBytes_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    result->length = CallWithContext(Traits::length, input, custom_context);
    decltype(CallWithContext(Traits::embedded_data, input, custom_context)) in_embedded_data = CallWithContext(Traits::embedded_data, input, custom_context);
    typename decltype(result->embedded_data)::BaseType* embedded_data_ptr;
    const mojo::internal::ContainerValidateParams embedded_data_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_embedded_data, buffer, &embedded_data_ptr, &embedded_data_validate_params,
        context);
    result->embedded_data.Set(embedded_data_ptr);
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::data, input, custom_context))>::type
        in_data;
    mojo::internal::Serialize<::storage::mojom::BytesProviderPtrDataView>(
        in_data, &result->data, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->data),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid data in DataElementBytes struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::storage::mojom::internal::DataElementBytes_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::storage::mojom::DataElementBytesDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::storage::mojom::DataElementFileDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::storage::mojom::DataElementFileDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::storage::mojom::internal::DataElementFile_Data);
    decltype(CallWithContext(Traits::path, input, custom_context)) in_path = CallWithContext(Traits::path, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::FilePathDataView>(
        in_path, context);
    decltype(CallWithContext(Traits::expected_modification_time, input, custom_context)) in_expected_modification_time = CallWithContext(Traits::expected_modification_time, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDataView>(
        in_expected_modification_time, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::storage::mojom::internal::DataElementFile_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::storage::mojom::internal::DataElementFile_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::path, input, custom_context)) in_path = CallWithContext(Traits::path, input, custom_context);
    typename decltype(result->path)::BaseType* path_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::FilePathDataView>(
        in_path, buffer, &path_ptr, context);
    result->path.Set(path_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->path.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null path in DataElementFile struct");
    result->offset = CallWithContext(Traits::offset, input, custom_context);
    result->length = CallWithContext(Traits::length, input, custom_context);
    decltype(CallWithContext(Traits::expected_modification_time, input, custom_context)) in_expected_modification_time = CallWithContext(Traits::expected_modification_time, input, custom_context);
    typename decltype(result->expected_modification_time)::BaseType* expected_modification_time_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::TimeDataView>(
        in_expected_modification_time, buffer, &expected_modification_time_ptr, context);
    result->expected_modification_time.Set(expected_modification_time_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::storage::mojom::internal::DataElementFile_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::storage::mojom::DataElementFileDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::storage::mojom::DataElementFilesystemURLDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::storage::mojom::DataElementFilesystemURLDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::storage::mojom::internal::DataElementFilesystemURL_Data);
    decltype(CallWithContext(Traits::url, input, custom_context)) in_url = CallWithContext(Traits::url, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_url, context);
    decltype(CallWithContext(Traits::expected_modification_time, input, custom_context)) in_expected_modification_time = CallWithContext(Traits::expected_modification_time, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDataView>(
        in_expected_modification_time, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::storage::mojom::internal::DataElementFilesystemURL_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::storage::mojom::internal::DataElementFilesystemURL_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::url, input, custom_context)) in_url = CallWithContext(Traits::url, input, custom_context);
    typename decltype(result->url)::BaseType* url_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_url, buffer, &url_ptr, context);
    result->url.Set(url_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->url.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null url in DataElementFilesystemURL struct");
    result->offset = CallWithContext(Traits::offset, input, custom_context);
    result->length = CallWithContext(Traits::length, input, custom_context);
    decltype(CallWithContext(Traits::expected_modification_time, input, custom_context)) in_expected_modification_time = CallWithContext(Traits::expected_modification_time, input, custom_context);
    typename decltype(result->expected_modification_time)::BaseType* expected_modification_time_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::TimeDataView>(
        in_expected_modification_time, buffer, &expected_modification_time_ptr, context);
    result->expected_modification_time.Set(expected_modification_time_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::storage::mojom::internal::DataElementFilesystemURL_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::storage::mojom::DataElementFilesystemURLDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::storage::mojom::DataElementBlobDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::storage::mojom::DataElementBlobDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::storage::mojom::internal::DataElementBlob_Data);
    decltype(CallWithContext(Traits::blob, input, custom_context)) in_blob = CallWithContext(Traits::blob, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::storage::mojom::BlobPtrDataView>(
        in_blob, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::storage::mojom::internal::DataElementBlob_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::storage::mojom::internal::DataElementBlob_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    // Dummy input value. The serialized value is already stored in the
    // SerializationContext.
    typename std::remove_reference<decltype(CallWithContext(Traits::blob, input, custom_context))>::type
        in_blob;
    mojo::internal::Serialize<::storage::mojom::BlobPtrDataView>(
        in_blob, &result->blob, context);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        !mojo::internal::IsHandleOrInterfaceValid(result->blob),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
        "invalid blob in DataElementBlob struct");
    result->offset = CallWithContext(Traits::offset, input, custom_context);
    result->length = CallWithContext(Traits::length, input, custom_context);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::storage::mojom::internal::DataElementBlob_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::storage::mojom::DataElementBlobDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::storage::mojom::DataElementDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = UnionTraits<::storage::mojom::DataElementDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   bool inlined,
                                   SerializationContext* context) {
    size_t size = inlined ? 0 : sizeof(::storage::mojom::internal::DataElement_Data);
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return size;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    switch (CallWithContext(Traits::GetTag, input, custom_context)) {
      case ::storage::mojom::DataElementDataView::Tag::BYTES: {
        decltype(CallWithContext(Traits::bytes, input, custom_context))
            in_bytes = CallWithContext(Traits::bytes, input,
                                          custom_context);
        size += mojo::internal::PrepareToSerialize<::storage::mojom::DataElementBytesDataView>(
            in_bytes, context);
        break;
      }
      case ::storage::mojom::DataElementDataView::Tag::FILE: {
        decltype(CallWithContext(Traits::file, input, custom_context))
            in_file = CallWithContext(Traits::file, input,
                                          custom_context);
        size += mojo::internal::PrepareToSerialize<::storage::mojom::DataElementFileDataView>(
            in_file, context);
        break;
      }
      case ::storage::mojom::DataElementDataView::Tag::FILE_FILESYSTEM: {
        decltype(CallWithContext(Traits::file_filesystem, input, custom_context))
            in_file_filesystem = CallWithContext(Traits::file_filesystem, input,
                                          custom_context);
        size += mojo::internal::PrepareToSerialize<::storage::mojom::DataElementFilesystemURLDataView>(
            in_file_filesystem, context);
        break;
      }
      case ::storage::mojom::DataElementDataView::Tag::BLOB: {
        decltype(CallWithContext(Traits::blob, input, custom_context))
            in_blob = CallWithContext(Traits::blob, input,
                                          custom_context);
        size += mojo::internal::PrepareToSerialize<::storage::mojom::DataElementBlobDataView>(
            in_blob, context);
        break;
      }
    }
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::storage::mojom::internal::DataElement_Data** output,
                        bool inlined,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      if (inlined)
        (*output)->set_null();
      else
        *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    if (!inlined)
      *output = ::storage::mojom::internal::DataElement_Data::New(buffer);

    ::storage::mojom::internal::DataElement_Data* result = *output;
    ALLOW_UNUSED_LOCAL(result);
    // TODO(azani): Handle unknown and objects.
    // Set the not-null flag.
    result->size = kUnionDataSize;
    result->tag = CallWithContext(Traits::GetTag, input, custom_context);
    switch (result->tag) {
      case ::storage::mojom::DataElementDataView::Tag::BYTES: {
        decltype(CallWithContext(Traits::bytes, input, custom_context))
            in_bytes = CallWithContext(Traits::bytes, input,
                                          custom_context);
        typename decltype(result->data.f_bytes)::BaseType* ptr;
        mojo::internal::Serialize<::storage::mojom::DataElementBytesDataView>(
            in_bytes, buffer, &ptr, context);
        result->data.f_bytes.Set(ptr);
        MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
            !ptr, mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
            "null bytes in DataElement union");
        break;
      }
      case ::storage::mojom::DataElementDataView::Tag::FILE: {
        decltype(CallWithContext(Traits::file, input, custom_context))
            in_file = CallWithContext(Traits::file, input,
                                          custom_context);
        typename decltype(result->data.f_file)::BaseType* ptr;
        mojo::internal::Serialize<::storage::mojom::DataElementFileDataView>(
            in_file, buffer, &ptr, context);
        result->data.f_file.Set(ptr);
        MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
            !ptr, mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
            "null file in DataElement union");
        break;
      }
      case ::storage::mojom::DataElementDataView::Tag::FILE_FILESYSTEM: {
        decltype(CallWithContext(Traits::file_filesystem, input, custom_context))
            in_file_filesystem = CallWithContext(Traits::file_filesystem, input,
                                          custom_context);
        typename decltype(result->data.f_file_filesystem)::BaseType* ptr;
        mojo::internal::Serialize<::storage::mojom::DataElementFilesystemURLDataView>(
            in_file_filesystem, buffer, &ptr, context);
        result->data.f_file_filesystem.Set(ptr);
        MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
            !ptr, mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
            "null file_filesystem in DataElement union");
        break;
      }
      case ::storage::mojom::DataElementDataView::Tag::BLOB: {
        decltype(CallWithContext(Traits::blob, input, custom_context))
            in_blob = CallWithContext(Traits::blob, input,
                                          custom_context);
        typename decltype(result->data.f_blob)::BaseType* ptr;
        mojo::internal::Serialize<::storage::mojom::DataElementBlobDataView>(
            in_blob, buffer, &ptr, context);
        result->data.f_blob.Set(ptr);
        MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
            !ptr, mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
            "null blob in DataElement union");
        break;
      }
    }

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::storage::mojom::internal::DataElement_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input || input->is_null())
      return CallSetToNullIfExists<Traits>(output);

    ::storage::mojom::DataElementDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace storage {
namespace mojom {

inline void DataElementBytesDataView::GetEmbeddedDataDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->embedded_data.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}


inline void DataElementFileDataView::GetPathDataView(
    ::mojo::common::mojom::FilePathDataView* output) {
  auto pointer = data_->path.Get();
  *output = ::mojo::common::mojom::FilePathDataView(pointer, context_);
}
inline void DataElementFileDataView::GetExpectedModificationTimeDataView(
    ::mojo::common::mojom::TimeDataView* output) {
  auto pointer = data_->expected_modification_time.Get();
  *output = ::mojo::common::mojom::TimeDataView(pointer, context_);
}


inline void DataElementFilesystemURLDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void DataElementFilesystemURLDataView::GetExpectedModificationTimeDataView(
    ::mojo::common::mojom::TimeDataView* output) {
  auto pointer = data_->expected_modification_time.Get();
  *output = ::mojo::common::mojom::TimeDataView(pointer, context_);
}








inline void Blob_GetInternalUUID_ResponseParamsDataView::GetUuidDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->uuid.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void BlobRegistry_Register_ParamsDataView::GetUuidDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->uuid.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void BlobRegistry_Register_ParamsDataView::GetContentTypeDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->content_type.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void BlobRegistry_Register_ParamsDataView::GetContentDispositionDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->content_disposition.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void BlobRegistry_Register_ParamsDataView::GetElementsDataView(
    mojo::ArrayDataView<DataElementDataView>* output) {
  auto pointer = data_->elements.Get();
  *output = mojo::ArrayDataView<DataElementDataView>(pointer, context_);
}




inline void BlobRegistry_GetBlobFromUUID_ParamsDataView::GetUuidDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->uuid.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void BytesProvider_RequestAsReply_ResponseParamsDataView::GetDataDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->data.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}




inline void BytesProvider_RequestAsFile_ParamsDataView::GetFileDataView(
    ::mojo::common::mojom::FileDataView* output) {
  auto pointer = data_->file.Get();
  *output = ::mojo::common::mojom::FileDataView(pointer, context_);
}


inline void BytesProvider_RequestAsFile_ResponseParamsDataView::GetTimeFileModifiedDataView(
    ::mojo::common::mojom::TimeDataView* output) {
  auto pointer = data_->time_file_modified.Get();
  *output = ::mojo::common::mojom::TimeDataView(pointer, context_);
}


inline void DataElementDataView::GetBytesDataView(
    DataElementBytesDataView* output) {
  DCHECK(is_bytes());
  *output = DataElementBytesDataView(data_->data.f_bytes.Get(), context_);
}
inline void DataElementDataView::GetFileDataView(
    DataElementFileDataView* output) {
  DCHECK(is_file());
  *output = DataElementFileDataView(data_->data.f_file.Get(), context_);
}
inline void DataElementDataView::GetFileFilesystemDataView(
    DataElementFilesystemURLDataView* output) {
  DCHECK(is_file_filesystem());
  *output = DataElementFilesystemURLDataView(data_->data.f_file_filesystem.Get(), context_);
}
inline void DataElementDataView::GetBlobDataView(
    DataElementBlobDataView* output) {
  DCHECK(is_blob());
  *output = DataElementBlobDataView(data_->data.f_blob.Get(), context_);
}


}  // namespace mojom
}  // namespace storage

#endif  // STORAGE_PUBLIC_INTERFACES_BLOBS_MOJOM_SHARED_H_
