// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "storage/public/interfaces/blobs.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/common/common_custom_types_struct_traits.h"
#include "url/mojo/url_gurl_struct_traits.h"
namespace storage {
namespace mojom {
constexpr uint64_t DataElementBytes::kMaximumEmbeddedDataSize;
DataElementBytes::DataElementBytes()
    : length(),
      embedded_data(),
      data() {}

DataElementBytes::DataElementBytes(
    uint64_t length_in,
    const base::Optional<std::vector<uint8_t>>& embedded_data_in,
    BytesProviderPtr data_in)
    : length(std::move(length_in)),
      embedded_data(std::move(embedded_data_in)),
      data(std::move(data_in)) {}

DataElementBytes::~DataElementBytes() = default;

bool DataElementBytes::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DataElementFile::DataElementFile()
    : path(),
      offset(),
      length(),
      expected_modification_time() {}

DataElementFile::DataElementFile(
    const base::FilePath& path_in,
    uint64_t offset_in,
    uint64_t length_in,
    base::Optional<base::Time> expected_modification_time_in)
    : path(std::move(path_in)),
      offset(std::move(offset_in)),
      length(std::move(length_in)),
      expected_modification_time(std::move(expected_modification_time_in)) {}

DataElementFile::~DataElementFile() = default;

bool DataElementFile::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DataElementFilesystemURL::DataElementFilesystemURL()
    : url(),
      offset(),
      length(),
      expected_modification_time() {}

DataElementFilesystemURL::DataElementFilesystemURL(
    const GURL& url_in,
    uint64_t offset_in,
    uint64_t length_in,
    base::Optional<base::Time> expected_modification_time_in)
    : url(std::move(url_in)),
      offset(std::move(offset_in)),
      length(std::move(length_in)),
      expected_modification_time(std::move(expected_modification_time_in)) {}

DataElementFilesystemURL::~DataElementFilesystemURL() = default;

bool DataElementFilesystemURL::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DataElementBlob::DataElementBlob()
    : blob(),
      offset(),
      length() {}

DataElementBlob::DataElementBlob(
    BlobPtr blob_in,
    uint64_t offset_in,
    uint64_t length_in)
    : blob(std::move(blob_in)),
      offset(std::move(offset_in)),
      length(std::move(length_in)) {}

DataElementBlob::~DataElementBlob() = default;

bool DataElementBlob::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
DataElement::DataElement() : tag_(Tag::BYTES) {
  data_.bytes = new DataElementBytesPtr;
}

DataElement::~DataElement() {
  DestroyActive();
}


void DataElement::set_bytes(
    DataElementBytesPtr bytes) {
  if (tag_ == Tag::BYTES) {
    *(data_.bytes) = std::move(bytes);
  } else {
    DestroyActive();
    tag_ = Tag::BYTES;
    data_.bytes = new DataElementBytesPtr(
        std::move(bytes));
  }
}
void DataElement::set_file(
    DataElementFilePtr file) {
  if (tag_ == Tag::FILE) {
    *(data_.file) = std::move(file);
  } else {
    DestroyActive();
    tag_ = Tag::FILE;
    data_.file = new DataElementFilePtr(
        std::move(file));
  }
}
void DataElement::set_file_filesystem(
    DataElementFilesystemURLPtr file_filesystem) {
  if (tag_ == Tag::FILE_FILESYSTEM) {
    *(data_.file_filesystem) = std::move(file_filesystem);
  } else {
    DestroyActive();
    tag_ = Tag::FILE_FILESYSTEM;
    data_.file_filesystem = new DataElementFilesystemURLPtr(
        std::move(file_filesystem));
  }
}
void DataElement::set_blob(
    DataElementBlobPtr blob) {
  if (tag_ == Tag::BLOB) {
    *(data_.blob) = std::move(blob);
  } else {
    DestroyActive();
    tag_ = Tag::BLOB;
    data_.blob = new DataElementBlobPtr(
        std::move(blob));
  }
}

void DataElement::DestroyActive() {
  switch (tag_) {

    case Tag::BYTES:

      delete data_.bytes;
      break;
    case Tag::FILE:

      delete data_.file;
      break;
    case Tag::FILE_FILESYSTEM:

      delete data_.file_filesystem;
      break;
    case Tag::BLOB:

      delete data_.blob;
      break;
  }
}
const char Blob::Name_[] = "storage::mojom::Blob";

class Blob_GetInternalUUID_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  Blob_GetInternalUUID_ForwardToCallback(
      Blob::GetInternalUUIDCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  Blob::GetInternalUUIDCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(Blob_GetInternalUUID_ForwardToCallback);
};

BlobProxy::BlobProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void BlobProxy::Clone(
    BlobRequest in_blob) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::Blob_Clone_Params_Data);
  size += mojo::internal::PrepareToSerialize<::storage::mojom::BlobRequestDataView>(
      in_blob, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBlob_Clone_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::Blob_Clone_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::storage::mojom::BlobRequestDataView>(
      in_blob, &params->blob, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->blob),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid blob in Blob.Clone request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void BlobProxy::GetInternalUUID(
    GetInternalUUIDCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::Blob_GetInternalUUID_Params_Data);
  serialization_context.PrepareMessage(
      internal::kBlob_GetInternalUUID_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::Blob_GetInternalUUID_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new Blob_GetInternalUUID_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class Blob_GetInternalUUID_ProxyToResponder {
 public:
  static Blob::GetInternalUUIDCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<Blob_GetInternalUUID_ProxyToResponder> proxy(
        new Blob_GetInternalUUID_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&Blob_GetInternalUUID_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~Blob_GetInternalUUID_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "Blob::GetInternalUUID() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  Blob_GetInternalUUID_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const std::string& in_uuid);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(Blob_GetInternalUUID_ProxyToResponder);
};

bool Blob_GetInternalUUID_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::Blob_GetInternalUUID_ResponseParams_Data* params =
      reinterpret_cast<
          internal::Blob_GetInternalUUID_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::string p_uuid{};
  Blob_GetInternalUUID_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadUuid(&p_uuid))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "Blob::GetInternalUUID response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_uuid));
  return true;
}

void Blob_GetInternalUUID_ProxyToResponder::Run(
    const std::string& in_uuid) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::Blob_GetInternalUUID_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_uuid, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBlob_GetInternalUUID_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::Blob_GetInternalUUID_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->uuid)::BaseType* uuid_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_uuid, buffer, &uuid_ptr, &serialization_context);
  params->uuid.Set(uuid_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->uuid.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null uuid in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool BlobStubDispatch::Accept(
    Blob* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kBlob_Clone_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::Blob_Clone_Params_Data* params =
          reinterpret_cast<internal::Blob_Clone_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      BlobRequest p_blob{};
      Blob_Clone_ParamsDataView input_data_view(params, &serialization_context);
      
      p_blob =
          input_data_view.TakeBlob<decltype(p_blob)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Blob::Clone deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Clone(
std::move(p_blob));
      return true;
    }
    case internal::kBlob_GetInternalUUID_Name: {
      break;
    }
  }
  return false;
}

// static
bool BlobStubDispatch::AcceptWithResponder(
    Blob* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kBlob_Clone_Name: {
      break;
    }
    case internal::kBlob_GetInternalUUID_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::Blob_GetInternalUUID_Params_Data* params =
          reinterpret_cast<
              internal::Blob_GetInternalUUID_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      Blob_GetInternalUUID_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "Blob::GetInternalUUID deserializer");
        return false;
      }
      Blob::GetInternalUUIDCallback callback =
          Blob_GetInternalUUID_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetInternalUUID(std::move(callback));
      return true;
    }
  }
  return false;
}

bool BlobRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "Blob RequestValidator");

  switch (message->header()->name) {
    case internal::kBlob_Clone_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Blob_Clone_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBlob_GetInternalUUID_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::Blob_GetInternalUUID_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool BlobResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "Blob ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kBlob_GetInternalUUID_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::Blob_GetInternalUUID_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char BlobRegistry::Name_[] = "storage::mojom::BlobRegistry";
bool BlobRegistry::Register(BlobRequest blob, const std::string& uuid, const std::string& content_type, const std::string& content_disposition, std::vector<DataElementPtr> elements) {
  NOTREACHED();
  return false;
}
class BlobRegistry_Register_HandleSyncResponse
    : public mojo::MessageReceiver {
 public:
  BlobRegistry_Register_HandleSyncResponse(
      bool* result)
      : result_(result) {
    DCHECK(!*result_);
  }
  bool Accept(mojo::Message* message) override;
 private:
  bool* result_;DISALLOW_COPY_AND_ASSIGN(BlobRegistry_Register_HandleSyncResponse);
};

class BlobRegistry_Register_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BlobRegistry_Register_ForwardToCallback(
      BlobRegistry::RegisterCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BlobRegistry::RegisterCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BlobRegistry_Register_ForwardToCallback);
};

BlobRegistryProxy::BlobRegistryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}
bool BlobRegistryProxy::Register(
    BlobRequest param_blob, const std::string& param_uuid, const std::string& param_content_type, const std::string& param_content_disposition, std::vector<DataElementPtr> param_elements) {
  const bool kExpectsResponse = true;
  const bool kIsSync = true;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BlobRegistry_Register_Params_Data);
  size += mojo::internal::PrepareToSerialize<::storage::mojom::BlobRequestDataView>(
      param_blob, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      param_uuid, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      param_content_type, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      param_content_disposition, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::storage::mojom::DataElementDataView>>(
      param_elements, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBlobRegistry_Register_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BlobRegistry_Register_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::storage::mojom::BlobRequestDataView>(
      param_blob, &params->blob, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->blob),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid blob in BlobRegistry.Register request");
  typename decltype(params->uuid)::BaseType* uuid_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_uuid, buffer, &uuid_ptr, &serialization_context);
  params->uuid.Set(uuid_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->uuid.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null uuid in BlobRegistry.Register request");
  typename decltype(params->content_type)::BaseType* content_type_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_content_type, buffer, &content_type_ptr, &serialization_context);
  params->content_type.Set(content_type_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->content_type.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null content_type in BlobRegistry.Register request");
  typename decltype(params->content_disposition)::BaseType* content_disposition_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      param_content_disposition, buffer, &content_disposition_ptr, &serialization_context);
  params->content_disposition.Set(content_disposition_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->content_disposition.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null content_disposition in BlobRegistry.Register request");
  typename decltype(params->elements)::BaseType* elements_ptr;
  const mojo::internal::ContainerValidateParams elements_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::storage::mojom::DataElementDataView>>(
      param_elements, buffer, &elements_ptr, &elements_validate_params,
      &serialization_context);
  params->elements.Set(elements_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->elements.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null elements in BlobRegistry.Register request");

  bool result = false;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BlobRegistry_Register_HandleSyncResponse(
          &result));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
  return result;
}

void BlobRegistryProxy::Register(
    BlobRequest in_blob, const std::string& in_uuid, const std::string& in_content_type, const std::string& in_content_disposition, std::vector<DataElementPtr> in_elements, RegisterCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BlobRegistry_Register_Params_Data);
  size += mojo::internal::PrepareToSerialize<::storage::mojom::BlobRequestDataView>(
      in_blob, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_uuid, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_content_type, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_content_disposition, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::storage::mojom::DataElementDataView>>(
      in_elements, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBlobRegistry_Register_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BlobRegistry_Register_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::storage::mojom::BlobRequestDataView>(
      in_blob, &params->blob, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->blob),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid blob in BlobRegistry.Register request");
  typename decltype(params->uuid)::BaseType* uuid_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_uuid, buffer, &uuid_ptr, &serialization_context);
  params->uuid.Set(uuid_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->uuid.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null uuid in BlobRegistry.Register request");
  typename decltype(params->content_type)::BaseType* content_type_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_content_type, buffer, &content_type_ptr, &serialization_context);
  params->content_type.Set(content_type_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->content_type.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null content_type in BlobRegistry.Register request");
  typename decltype(params->content_disposition)::BaseType* content_disposition_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_content_disposition, buffer, &content_disposition_ptr, &serialization_context);
  params->content_disposition.Set(content_disposition_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->content_disposition.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null content_disposition in BlobRegistry.Register request");
  typename decltype(params->elements)::BaseType* elements_ptr;
  const mojo::internal::ContainerValidateParams elements_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::storage::mojom::DataElementDataView>>(
      in_elements, buffer, &elements_ptr, &elements_validate_params,
      &serialization_context);
  params->elements.Set(elements_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->elements.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null elements in BlobRegistry.Register request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BlobRegistry_Register_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void BlobRegistryProxy::GetBlobFromUUID(
    BlobRequest in_blob, const std::string& in_uuid) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BlobRegistry_GetBlobFromUUID_Params_Data);
  size += mojo::internal::PrepareToSerialize<::storage::mojom::BlobRequestDataView>(
      in_blob, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_uuid, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBlobRegistry_GetBlobFromUUID_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BlobRegistry_GetBlobFromUUID_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::storage::mojom::BlobRequestDataView>(
      in_blob, &params->blob, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->blob),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid blob in BlobRegistry.GetBlobFromUUID request");
  typename decltype(params->uuid)::BaseType* uuid_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_uuid, buffer, &uuid_ptr, &serialization_context);
  params->uuid.Set(uuid_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->uuid.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null uuid in BlobRegistry.GetBlobFromUUID request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class BlobRegistry_Register_ProxyToResponder {
 public:
  static BlobRegistry::RegisterCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BlobRegistry_Register_ProxyToResponder> proxy(
        new BlobRegistry_Register_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BlobRegistry_Register_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BlobRegistry_Register_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BlobRegistry::Register() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BlobRegistry_Register_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      );

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BlobRegistry_Register_ProxyToResponder);
};

bool BlobRegistry_Register_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BlobRegistry_Register_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BlobRegistry_Register_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  BlobRegistry_Register_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BlobRegistry::Register response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run();
  return true;
}

void BlobRegistry_Register_ProxyToResponder::Run(
    ) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BlobRegistry_Register_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kBlobRegistry_Register_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BlobRegistry_Register_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
bool BlobRegistry_Register_HandleSyncResponse::Accept(
    mojo::Message* message) {

  DCHECK(message->is_serialized());
  internal::BlobRegistry_Register_ResponseParams_Data* params =
      reinterpret_cast<internal::BlobRegistry_Register_ResponseParams_Data*>(
          message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  BlobRegistry_Register_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BlobRegistry::Register response deserializer");
    return false;
  }
  mojo::internal::SyncMessageResponseSetup::SetCurrentSyncResponseMessage(
      message);
  *result_ = true;
  return true;
}

// static
bool BlobRegistryStubDispatch::Accept(
    BlobRegistry* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kBlobRegistry_Register_Name: {
      break;
    }
    case internal::kBlobRegistry_GetBlobFromUUID_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::BlobRegistry_GetBlobFromUUID_Params_Data* params =
          reinterpret_cast<internal::BlobRegistry_GetBlobFromUUID_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      BlobRequest p_blob{};
      std::string p_uuid{};
      BlobRegistry_GetBlobFromUUID_ParamsDataView input_data_view(params, &serialization_context);
      
      p_blob =
          input_data_view.TakeBlob<decltype(p_blob)>();
      if (!input_data_view.ReadUuid(&p_uuid))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BlobRegistry::GetBlobFromUUID deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetBlobFromUUID(
std::move(p_blob), 
std::move(p_uuid));
      return true;
    }
  }
  return false;
}

// static
bool BlobRegistryStubDispatch::AcceptWithResponder(
    BlobRegistry* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kBlobRegistry_Register_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BlobRegistry_Register_Params_Data* params =
          reinterpret_cast<
              internal::BlobRegistry_Register_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      BlobRequest p_blob{};
      std::string p_uuid{};
      std::string p_content_type{};
      std::string p_content_disposition{};
      std::vector<DataElementPtr> p_elements{};
      BlobRegistry_Register_ParamsDataView input_data_view(params, &serialization_context);
      
      p_blob =
          input_data_view.TakeBlob<decltype(p_blob)>();
      if (!input_data_view.ReadUuid(&p_uuid))
        success = false;
      if (!input_data_view.ReadContentType(&p_content_type))
        success = false;
      if (!input_data_view.ReadContentDisposition(&p_content_disposition))
        success = false;
      if (!input_data_view.ReadElements(&p_elements))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BlobRegistry::Register deserializer");
        return false;
      }
      BlobRegistry::RegisterCallback callback =
          BlobRegistry_Register_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Register(
std::move(p_blob), 
std::move(p_uuid), 
std::move(p_content_type), 
std::move(p_content_disposition), 
std::move(p_elements), std::move(callback));
      return true;
    }
    case internal::kBlobRegistry_GetBlobFromUUID_Name: {
      break;
    }
  }
  return false;
}

bool BlobRegistryRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "BlobRegistry RequestValidator");

  switch (message->header()->name) {
    case internal::kBlobRegistry_Register_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BlobRegistry_Register_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBlobRegistry_GetBlobFromUUID_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BlobRegistry_GetBlobFromUUID_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool BlobRegistryResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "BlobRegistry ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kBlobRegistry_Register_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BlobRegistry_Register_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char BytesProvider::Name_[] = "storage::mojom::BytesProvider";

class BytesProvider_RequestAsReply_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BytesProvider_RequestAsReply_ForwardToCallback(
      BytesProvider::RequestAsReplyCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BytesProvider::RequestAsReplyCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BytesProvider_RequestAsReply_ForwardToCallback);
};

class BytesProvider_RequestAsFile_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  BytesProvider_RequestAsFile_ForwardToCallback(
      BytesProvider::RequestAsFileCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  BytesProvider::RequestAsFileCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(BytesProvider_RequestAsFile_ForwardToCallback);
};

BytesProviderProxy::BytesProviderProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void BytesProviderProxy::RequestAsReply(
    RequestAsReplyCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BytesProvider_RequestAsReply_Params_Data);
  serialization_context.PrepareMessage(
      internal::kBytesProvider_RequestAsReply_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BytesProvider_RequestAsReply_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BytesProvider_RequestAsReply_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void BytesProviderProxy::RequestAsStream(
    mojo::ScopedDataPipeProducerHandle in_pipe) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BytesProvider_RequestAsStream_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ScopedDataPipeProducerHandle>(
      in_pipe, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBytesProvider_RequestAsStream_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BytesProvider_RequestAsStream_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<mojo::ScopedDataPipeProducerHandle>(
      in_pipe, &params->pipe, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->pipe),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid pipe in BytesProvider.RequestAsStream request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void BytesProviderProxy::RequestAsFile(
    uint64_t in_source_offset, uint64_t in_source_size, base::File in_file, uint64_t in_file_offset, RequestAsFileCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BytesProvider_RequestAsFile_Params_Data);
  size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::FileDataView>(
      in_file, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBytesProvider_RequestAsFile_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BytesProvider_RequestAsFile_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->source_offset = in_source_offset;
  params->source_size = in_source_size;
  typename decltype(params->file)::BaseType* file_ptr;
  mojo::internal::Serialize<::mojo::common::mojom::FileDataView>(
      in_file, buffer, &file_ptr, &serialization_context);
  params->file.Set(file_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->file.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null file in BytesProvider.RequestAsFile request");
  params->file_offset = in_file_offset;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new BytesProvider_RequestAsFile_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class BytesProvider_RequestAsReply_ProxyToResponder {
 public:
  static BytesProvider::RequestAsReplyCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BytesProvider_RequestAsReply_ProxyToResponder> proxy(
        new BytesProvider_RequestAsReply_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BytesProvider_RequestAsReply_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BytesProvider_RequestAsReply_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BytesProvider::RequestAsReply() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BytesProvider_RequestAsReply_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const std::vector<uint8_t>& in_data);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BytesProvider_RequestAsReply_ProxyToResponder);
};

bool BytesProvider_RequestAsReply_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BytesProvider_RequestAsReply_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BytesProvider_RequestAsReply_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<uint8_t> p_data{};
  BytesProvider_RequestAsReply_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadData(&p_data))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BytesProvider::RequestAsReply response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_data));
  return true;
}

void BytesProvider_RequestAsReply_ProxyToResponder::Run(
    const std::vector<uint8_t>& in_data) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BytesProvider_RequestAsReply_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
      in_data, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBytesProvider_RequestAsReply_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BytesProvider_RequestAsReply_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->data)::BaseType* data_ptr;
  const mojo::internal::ContainerValidateParams data_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      in_data, buffer, &data_ptr, &data_validate_params,
      &serialization_context);
  params->data.Set(data_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null data in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class BytesProvider_RequestAsFile_ProxyToResponder {
 public:
  static BytesProvider::RequestAsFileCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<BytesProvider_RequestAsFile_ProxyToResponder> proxy(
        new BytesProvider_RequestAsFile_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&BytesProvider_RequestAsFile_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~BytesProvider_RequestAsFile_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "BytesProvider::RequestAsFile() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  BytesProvider_RequestAsFile_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      base::Optional<base::Time> in_time_file_modified);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(BytesProvider_RequestAsFile_ProxyToResponder);
};

bool BytesProvider_RequestAsFile_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::BytesProvider_RequestAsFile_ResponseParams_Data* params =
      reinterpret_cast<
          internal::BytesProvider_RequestAsFile_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Optional<base::Time> p_time_file_modified{};
  BytesProvider_RequestAsFile_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadTimeFileModified(&p_time_file_modified))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "BytesProvider::RequestAsFile response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_time_file_modified));
  return true;
}

void BytesProvider_RequestAsFile_ProxyToResponder::Run(
    base::Optional<base::Time> in_time_file_modified) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::storage::mojom::internal::BytesProvider_RequestAsFile_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDataView>(
      in_time_file_modified, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kBytesProvider_RequestAsFile_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::storage::mojom::internal::BytesProvider_RequestAsFile_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->time_file_modified)::BaseType* time_file_modified_ptr;
  mojo::internal::Serialize<::mojo::common::mojom::TimeDataView>(
      in_time_file_modified, buffer, &time_file_modified_ptr, &serialization_context);
  params->time_file_modified.Set(time_file_modified_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool BytesProviderStubDispatch::Accept(
    BytesProvider* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kBytesProvider_RequestAsReply_Name: {
      break;
    }
    case internal::kBytesProvider_RequestAsStream_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::BytesProvider_RequestAsStream_Params_Data* params =
          reinterpret_cast<internal::BytesProvider_RequestAsStream_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      mojo::ScopedDataPipeProducerHandle p_pipe{};
      BytesProvider_RequestAsStream_ParamsDataView input_data_view(params, &serialization_context);
      
      p_pipe = input_data_view.TakePipe();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BytesProvider::RequestAsStream deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestAsStream(
std::move(p_pipe));
      return true;
    }
    case internal::kBytesProvider_RequestAsFile_Name: {
      break;
    }
  }
  return false;
}

// static
bool BytesProviderStubDispatch::AcceptWithResponder(
    BytesProvider* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kBytesProvider_RequestAsReply_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BytesProvider_RequestAsReply_Params_Data* params =
          reinterpret_cast<
              internal::BytesProvider_RequestAsReply_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      BytesProvider_RequestAsReply_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BytesProvider::RequestAsReply deserializer");
        return false;
      }
      BytesProvider::RequestAsReplyCallback callback =
          BytesProvider_RequestAsReply_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestAsReply(std::move(callback));
      return true;
    }
    case internal::kBytesProvider_RequestAsStream_Name: {
      break;
    }
    case internal::kBytesProvider_RequestAsFile_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::BytesProvider_RequestAsFile_Params_Data* params =
          reinterpret_cast<
              internal::BytesProvider_RequestAsFile_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint64_t p_source_offset{};
      uint64_t p_source_size{};
      base::File p_file{};
      uint64_t p_file_offset{};
      BytesProvider_RequestAsFile_ParamsDataView input_data_view(params, &serialization_context);
      
      p_source_offset = input_data_view.source_offset();
      p_source_size = input_data_view.source_size();
      if (!input_data_view.ReadFile(&p_file))
        success = false;
      p_file_offset = input_data_view.file_offset();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "BytesProvider::RequestAsFile deserializer");
        return false;
      }
      BytesProvider::RequestAsFileCallback callback =
          BytesProvider_RequestAsFile_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestAsFile(
std::move(p_source_offset), 
std::move(p_source_size), 
std::move(p_file), 
std::move(p_file_offset), std::move(callback));
      return true;
    }
  }
  return false;
}

bool BytesProviderRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "BytesProvider RequestValidator");

  switch (message->header()->name) {
    case internal::kBytesProvider_RequestAsReply_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BytesProvider_RequestAsReply_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBytesProvider_RequestAsStream_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BytesProvider_RequestAsStream_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBytesProvider_RequestAsFile_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::BytesProvider_RequestAsFile_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool BytesProviderResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "BytesProvider ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kBytesProvider_RequestAsReply_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BytesProvider_RequestAsReply_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kBytesProvider_RequestAsFile_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::BytesProvider_RequestAsFile_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace mojom
}  // namespace storage

namespace mojo {


// static
bool StructTraits<::storage::mojom::DataElementBytes::DataView, ::storage::mojom::DataElementBytesPtr>::Read(
    ::storage::mojom::DataElementBytes::DataView input,
    ::storage::mojom::DataElementBytesPtr* output) {
  bool success = true;
  ::storage::mojom::DataElementBytesPtr result(::storage::mojom::DataElementBytes::New());
  
      result->length = input.length();
      if (!input.ReadEmbeddedData(&result->embedded_data))
        success = false;
      result->data =
          input.TakeData<decltype(result->data)>();
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::storage::mojom::DataElementFile::DataView, ::storage::mojom::DataElementFilePtr>::Read(
    ::storage::mojom::DataElementFile::DataView input,
    ::storage::mojom::DataElementFilePtr* output) {
  bool success = true;
  ::storage::mojom::DataElementFilePtr result(::storage::mojom::DataElementFile::New());
  
      if (!input.ReadPath(&result->path))
        success = false;
      result->offset = input.offset();
      result->length = input.length();
      if (!input.ReadExpectedModificationTime(&result->expected_modification_time))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::storage::mojom::DataElementFilesystemURL::DataView, ::storage::mojom::DataElementFilesystemURLPtr>::Read(
    ::storage::mojom::DataElementFilesystemURL::DataView input,
    ::storage::mojom::DataElementFilesystemURLPtr* output) {
  bool success = true;
  ::storage::mojom::DataElementFilesystemURLPtr result(::storage::mojom::DataElementFilesystemURL::New());
  
      if (!input.ReadUrl(&result->url))
        success = false;
      result->offset = input.offset();
      result->length = input.length();
      if (!input.ReadExpectedModificationTime(&result->expected_modification_time))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::storage::mojom::DataElementBlob::DataView, ::storage::mojom::DataElementBlobPtr>::Read(
    ::storage::mojom::DataElementBlob::DataView input,
    ::storage::mojom::DataElementBlobPtr* output) {
  bool success = true;
  ::storage::mojom::DataElementBlobPtr result(::storage::mojom::DataElementBlob::New());
  
      result->blob =
          input.TakeBlob<decltype(result->blob)>();
      result->offset = input.offset();
      result->length = input.length();
  *output = std::move(result);
  return success;
}

// static
bool UnionTraits<::storage::mojom::DataElement::DataView, ::storage::mojom::DataElementPtr>::Read(
    ::storage::mojom::DataElement::DataView input,
    ::storage::mojom::DataElementPtr* output) {
  using UnionType = ::storage::mojom::DataElement;
  using Tag = UnionType::Tag;

  switch (input.tag()) {
    case Tag::BYTES: {
      ::storage::mojom::DataElementBytesPtr result_bytes;
      if (!input.ReadBytes(&result_bytes))
        return false;

      *output = UnionType::NewBytes(
          std::move(result_bytes));
      break;
    }
    case Tag::FILE: {
      ::storage::mojom::DataElementFilePtr result_file;
      if (!input.ReadFile(&result_file))
        return false;

      *output = UnionType::NewFile(
          std::move(result_file));
      break;
    }
    case Tag::FILE_FILESYSTEM: {
      ::storage::mojom::DataElementFilesystemURLPtr result_file_filesystem;
      if (!input.ReadFileFilesystem(&result_file_filesystem))
        return false;

      *output = UnionType::NewFileFilesystem(
          std::move(result_file_filesystem));
      break;
    }
    case Tag::BLOB: {
      ::storage::mojom::DataElementBlobPtr result_blob;
      if (!input.ReadBlob(&result_blob))
        return false;

      *output = UnionType::NewBlob(
          std::move(result_blob));
      break;
    }
    default:
      return false;
  }
  return true;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif