// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_BACKGROUND_FETCH_BACKGROUND_FETCH_MOJOM_BLINK_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_BACKGROUND_FETCH_BACKGROUND_FETCH_MOJOM_BLINK_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/background_fetch/background_fetch.mojom-shared.h"
#include "third_party/WebKit/public/platform/modules/fetch/fetch_api_request.mojom-blink.h"
#include "url/mojo/origin.mojom-blink.h"

#include "mojo/public/cpp/bindings/lib/wtf_hash_util.h"
#include "third_party/WebKit/Source/platform/wtf/HashFunctions.h"
#include "third_party/WebKit/Source/platform/wtf/Optional.h"
#include "third_party/WebKit/Source/platform/wtf/text/WTFString.h"
#include "third_party/WebKit/public/platform/WebCommon.h"


namespace WTF {
struct blink_mojom_internal_BackgroundFetchError_DataHashFn {
  static unsigned GetHash(const ::blink::mojom::BackgroundFetchError& value) {
    using utype = std::underlying_type<::blink::mojom::BackgroundFetchError>::type;
    return DefaultHash<utype>::Hash().GetHash(static_cast<utype>(value));
  }
  static bool Equal(const ::blink::mojom::BackgroundFetchError& left, const ::blink::mojom::BackgroundFetchError& right) {
    return left == right;
  }
  static const bool safe_to_compare_to_empty_or_deleted = true;
};

template <>
struct DefaultHash<::blink::mojom::BackgroundFetchError> {
  using Hash = blink_mojom_internal_BackgroundFetchError_DataHashFn;
};

template <>
struct HashTraits<::blink::mojom::BackgroundFetchError>
    : public GenericHashTraits<::blink::mojom::BackgroundFetchError> {
  static_assert(true,
                "-1000000 is a reserved enum value");
  static_assert(true,
                "-1000001 is a reserved enum value");
  static const bool hasIsEmptyValueFunction = true;
  static bool IsEmptyValue(const ::blink::mojom::BackgroundFetchError& value) {
    return value == static_cast<::blink::mojom::BackgroundFetchError>(-1000000);
  }
  static void ConstructDeletedValue(::blink::mojom::BackgroundFetchError& slot, bool) {
    slot = static_cast<::blink::mojom::BackgroundFetchError>(-1000001);
  }
  static bool IsDeletedValue(const ::blink::mojom::BackgroundFetchError& value) {
    return value == static_cast<::blink::mojom::BackgroundFetchError>(-1000001);
  }
};
}  // namespace WTF


namespace blink {
namespace mojom {
namespace blink {
using BackgroundFetchError = BackgroundFetchError;  // Alias for definition in the parent namespace.
class BackgroundFetchService;
using BackgroundFetchServicePtr = mojo::InterfacePtr<BackgroundFetchService>;
using BackgroundFetchServicePtrInfo = mojo::InterfacePtrInfo<BackgroundFetchService>;
using ThreadSafeBackgroundFetchServicePtr =
    mojo::ThreadSafeInterfacePtr<BackgroundFetchService>;
using BackgroundFetchServiceRequest = mojo::InterfaceRequest<BackgroundFetchService>;
using BackgroundFetchServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<BackgroundFetchService>;
using ThreadSafeBackgroundFetchServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BackgroundFetchService>;
using BackgroundFetchServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BackgroundFetchService>;
using BackgroundFetchServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BackgroundFetchService>;

class IconDefinition;
using IconDefinitionPtr = mojo::InlinedStructPtr<IconDefinition>;

class BackgroundFetchOptions;
using BackgroundFetchOptionsPtr = mojo::StructPtr<BackgroundFetchOptions>;

class BackgroundFetchRegistration;
using BackgroundFetchRegistrationPtr = mojo::StructPtr<BackgroundFetchRegistration>;


class BackgroundFetchServiceProxy;

template <typename ImplRefTraits>
class BackgroundFetchServiceStub;

class BackgroundFetchServiceRequestValidator;
class BackgroundFetchServiceResponseValidator;

class BLINK_PLATFORM_EXPORT BackgroundFetchService
    : public BackgroundFetchServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = BackgroundFetchServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = BackgroundFetchServiceStub<ImplRefTraits>;

  using RequestValidator_ = BackgroundFetchServiceRequestValidator;
  using ResponseValidator_ = BackgroundFetchServiceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kFetchMinVersion = 0,
    kUpdateUIMinVersion = 0,
    kAbortMinVersion = 0,
    kGetRegistrationMinVersion = 0,
    kGetTagsMinVersion = 0,
  };
  virtual ~BackgroundFetchService() {}


  using FetchCallback = base::OnceCallback<void(BackgroundFetchError, BackgroundFetchRegistrationPtr)>;
  virtual void Fetch(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, const WTF::Vector<::blink::WebServiceWorkerRequest>& requests, BackgroundFetchOptionsPtr options, FetchCallback callback) = 0;


  using UpdateUICallback = base::OnceCallback<void(BackgroundFetchError)>;
  virtual void UpdateUI(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, const WTF::String& title, UpdateUICallback callback) = 0;


  using AbortCallback = base::OnceCallback<void(BackgroundFetchError)>;
  virtual void Abort(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, AbortCallback callback) = 0;


  using GetRegistrationCallback = base::OnceCallback<void(BackgroundFetchError, BackgroundFetchRegistrationPtr)>;
  virtual void GetRegistration(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, GetRegistrationCallback callback) = 0;


  using GetTagsCallback = base::OnceCallback<void(BackgroundFetchError, const WTF::Vector<WTF::String>&)>;
  virtual void GetTags(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, GetTagsCallback callback) = 0;
};

class BLINK_PLATFORM_EXPORT BackgroundFetchServiceProxy
    : public BackgroundFetchService {
 public:
  explicit BackgroundFetchServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void Fetch(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, const WTF::Vector<::blink::WebServiceWorkerRequest>& requests, BackgroundFetchOptionsPtr options, FetchCallback callback) override;
  void UpdateUI(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, const WTF::String& title, UpdateUICallback callback) override;
  void Abort(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, AbortCallback callback) override;
  void GetRegistration(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, const WTF::String& tag, GetRegistrationCallback callback) override;
  void GetTags(int64_t service_worker_registration_id, const RefPtr<::blink::SecurityOrigin>& origin, GetTagsCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class BLINK_PLATFORM_EXPORT BackgroundFetchServiceStubDispatch {
 public:
  static bool Accept(BackgroundFetchService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BackgroundFetchService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BackgroundFetchService>>
class BackgroundFetchServiceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BackgroundFetchServiceStub() {}
  ~BackgroundFetchServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BackgroundFetchServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BackgroundFetchServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class BLINK_PLATFORM_EXPORT BackgroundFetchServiceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class BLINK_PLATFORM_EXPORT BackgroundFetchServiceResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};


class BLINK_PLATFORM_EXPORT IconDefinition {
 public:
  using DataView = IconDefinitionDataView;
  using Data_ = internal::IconDefinition_Data;

  template <typename... Args>
  static IconDefinitionPtr New(Args&&... args) {
    return IconDefinitionPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static IconDefinitionPtr From(const U& u) {
    return mojo::TypeConverter<IconDefinitionPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, IconDefinition>::Convert(*this);
  }


  IconDefinition();

  IconDefinition(
      const WTF::String& src,
      const WTF::String& sizes,
      const WTF::String& type);

  ~IconDefinition();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = IconDefinitionPtr>
  IconDefinitionPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, IconDefinition>::value>::type* = nullptr>
  bool Equals(const T& other) const;
  size_t Hash(size_t seed) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        IconDefinition::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        IconDefinition::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  WTF::String src;
  WTF::String sizes;
  WTF::String type;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};








class BLINK_PLATFORM_EXPORT BackgroundFetchOptions {
 public:
  using DataView = BackgroundFetchOptionsDataView;
  using Data_ = internal::BackgroundFetchOptions_Data;

  template <typename... Args>
  static BackgroundFetchOptionsPtr New(Args&&... args) {
    return BackgroundFetchOptionsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BackgroundFetchOptionsPtr From(const U& u) {
    return mojo::TypeConverter<BackgroundFetchOptionsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BackgroundFetchOptions>::Convert(*this);
  }


  BackgroundFetchOptions();

  BackgroundFetchOptions(
      WTF::Vector<IconDefinitionPtr> icons,
      int64_t total_download_size,
      const WTF::String& title);

  ~BackgroundFetchOptions();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BackgroundFetchOptionsPtr>
  BackgroundFetchOptionsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BackgroundFetchOptions>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BackgroundFetchOptions::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BackgroundFetchOptions::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  WTF::Vector<IconDefinitionPtr> icons;
  int64_t total_download_size;
  WTF::String title;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchOptions);
};



class BLINK_PLATFORM_EXPORT BackgroundFetchRegistration {
 public:
  using DataView = BackgroundFetchRegistrationDataView;
  using Data_ = internal::BackgroundFetchRegistration_Data;

  template <typename... Args>
  static BackgroundFetchRegistrationPtr New(Args&&... args) {
    return BackgroundFetchRegistrationPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BackgroundFetchRegistrationPtr From(const U& u) {
    return mojo::TypeConverter<BackgroundFetchRegistrationPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BackgroundFetchRegistration>::Convert(*this);
  }


  BackgroundFetchRegistration();

  BackgroundFetchRegistration(
      const WTF::String& tag,
      WTF::Vector<IconDefinitionPtr> icons,
      int64_t total_download_size,
      const WTF::String& title);

  ~BackgroundFetchRegistration();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BackgroundFetchRegistrationPtr>
  BackgroundFetchRegistrationPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BackgroundFetchRegistration>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static WTF::Vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BackgroundFetchRegistration::DataView, WTF::Vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const WTF::Vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BackgroundFetchRegistration::DataView, WTF::Vector<uint8_t>>(
            input, output, Validate);
  }

  WTF::String tag;
  WTF::Vector<IconDefinitionPtr> icons;
  int64_t total_download_size;
  WTF::String title;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
  DISALLOW_COPY_AND_ASSIGN(BackgroundFetchRegistration);
};


template <typename StructPtrType>
IconDefinitionPtr IconDefinition::Clone() const {
  return New(
      mojo::Clone(src),
      mojo::Clone(sizes),
      mojo::Clone(type)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, IconDefinition>::value>::type*>
bool IconDefinition::Equals(const T& other) const {
  if (!mojo::Equals(this->src, other.src))
    return false;
  if (!mojo::Equals(this->sizes, other.sizes))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  return true;
}
template <typename StructPtrType>
BackgroundFetchOptionsPtr BackgroundFetchOptions::Clone() const {
  return New(
      mojo::Clone(icons),
      mojo::Clone(total_download_size),
      mojo::Clone(title)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BackgroundFetchOptions>::value>::type*>
bool BackgroundFetchOptions::Equals(const T& other) const {
  if (!mojo::Equals(this->icons, other.icons))
    return false;
  if (!mojo::Equals(this->total_download_size, other.total_download_size))
    return false;
  if (!mojo::Equals(this->title, other.title))
    return false;
  return true;
}
template <typename StructPtrType>
BackgroundFetchRegistrationPtr BackgroundFetchRegistration::Clone() const {
  return New(
      mojo::Clone(tag),
      mojo::Clone(icons),
      mojo::Clone(total_download_size),
      mojo::Clone(title)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BackgroundFetchRegistration>::value>::type*>
bool BackgroundFetchRegistration::Equals(const T& other) const {
  if (!mojo::Equals(this->tag, other.tag))
    return false;
  if (!mojo::Equals(this->icons, other.icons))
    return false;
  if (!mojo::Equals(this->total_download_size, other.total_download_size))
    return false;
  if (!mojo::Equals(this->title, other.title))
    return false;
  return true;
}


}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::blink::mojom::blink::IconDefinition::DataView,
                                         ::blink::mojom::blink::IconDefinitionPtr> {
  static bool IsNull(const ::blink::mojom::blink::IconDefinitionPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::blink::IconDefinitionPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::blink::IconDefinition::src)& src(
      const ::blink::mojom::blink::IconDefinitionPtr& input) {
    return input->src;
  }

  static const decltype(::blink::mojom::blink::IconDefinition::sizes)& sizes(
      const ::blink::mojom::blink::IconDefinitionPtr& input) {
    return input->sizes;
  }

  static const decltype(::blink::mojom::blink::IconDefinition::type)& type(
      const ::blink::mojom::blink::IconDefinitionPtr& input) {
    return input->type;
  }

  static bool Read(::blink::mojom::blink::IconDefinition::DataView input, ::blink::mojom::blink::IconDefinitionPtr* output);
};


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::blink::mojom::blink::BackgroundFetchOptions::DataView,
                                         ::blink::mojom::blink::BackgroundFetchOptionsPtr> {
  static bool IsNull(const ::blink::mojom::blink::BackgroundFetchOptionsPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::blink::BackgroundFetchOptionsPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::blink::BackgroundFetchOptions::icons)& icons(
      const ::blink::mojom::blink::BackgroundFetchOptionsPtr& input) {
    return input->icons;
  }

  static decltype(::blink::mojom::blink::BackgroundFetchOptions::total_download_size) total_download_size(
      const ::blink::mojom::blink::BackgroundFetchOptionsPtr& input) {
    return input->total_download_size;
  }

  static const decltype(::blink::mojom::blink::BackgroundFetchOptions::title)& title(
      const ::blink::mojom::blink::BackgroundFetchOptionsPtr& input) {
    return input->title;
  }

  static bool Read(::blink::mojom::blink::BackgroundFetchOptions::DataView input, ::blink::mojom::blink::BackgroundFetchOptionsPtr* output);
};


template <>
struct BLINK_PLATFORM_EXPORT StructTraits<::blink::mojom::blink::BackgroundFetchRegistration::DataView,
                                         ::blink::mojom::blink::BackgroundFetchRegistrationPtr> {
  static bool IsNull(const ::blink::mojom::blink::BackgroundFetchRegistrationPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::blink::BackgroundFetchRegistrationPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::blink::BackgroundFetchRegistration::tag)& tag(
      const ::blink::mojom::blink::BackgroundFetchRegistrationPtr& input) {
    return input->tag;
  }

  static const decltype(::blink::mojom::blink::BackgroundFetchRegistration::icons)& icons(
      const ::blink::mojom::blink::BackgroundFetchRegistrationPtr& input) {
    return input->icons;
  }

  static decltype(::blink::mojom::blink::BackgroundFetchRegistration::total_download_size) total_download_size(
      const ::blink::mojom::blink::BackgroundFetchRegistrationPtr& input) {
    return input->total_download_size;
  }

  static const decltype(::blink::mojom::blink::BackgroundFetchRegistration::title)& title(
      const ::blink::mojom::blink::BackgroundFetchRegistrationPtr& input) {
    return input->title;
  }

  static bool Read(::blink::mojom::blink::BackgroundFetchRegistration::DataView input, ::blink::mojom::blink::BackgroundFetchRegistrationPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_BACKGROUND_FETCH_BACKGROUND_FETCH_MOJOM_BLINK_H_