// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_BACKGROUND_FETCH_BACKGROUND_FETCH_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_BACKGROUND_FETCH_BACKGROUND_FETCH_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/background_fetch/background_fetch.mojom-shared.h"
#include "third_party/WebKit/public/platform/modules/fetch/fetch_api_request.mojom.h"
#include "url/mojo/origin.mojom.h"
#include <string>
#include <vector>
#include "content/common/background_fetch/background_fetch_types.h"
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class BackgroundFetchService;
using BackgroundFetchServicePtr = mojo::InterfacePtr<BackgroundFetchService>;
using BackgroundFetchServicePtrInfo = mojo::InterfacePtrInfo<BackgroundFetchService>;
using ThreadSafeBackgroundFetchServicePtr =
    mojo::ThreadSafeInterfacePtr<BackgroundFetchService>;
using BackgroundFetchServiceRequest = mojo::InterfaceRequest<BackgroundFetchService>;
using BackgroundFetchServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<BackgroundFetchService>;
using ThreadSafeBackgroundFetchServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<BackgroundFetchService>;
using BackgroundFetchServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<BackgroundFetchService>;
using BackgroundFetchServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<BackgroundFetchService>;

class IconDefinition;
using IconDefinitionPtr = mojo::InlinedStructPtr<IconDefinition>;

class BackgroundFetchOptions;
using BackgroundFetchOptionsPtr = mojo::StructPtr<BackgroundFetchOptions>;

class BackgroundFetchRegistration;
using BackgroundFetchRegistrationPtr = mojo::StructPtr<BackgroundFetchRegistration>;


class BackgroundFetchServiceProxy;

template <typename ImplRefTraits>
class BackgroundFetchServiceStub;

class BackgroundFetchServiceRequestValidator;
class BackgroundFetchServiceResponseValidator;

class CONTENT_EXPORT BackgroundFetchService
    : public BackgroundFetchServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = BackgroundFetchServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = BackgroundFetchServiceStub<ImplRefTraits>;

  using RequestValidator_ = BackgroundFetchServiceRequestValidator;
  using ResponseValidator_ = BackgroundFetchServiceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kFetchMinVersion = 0,
    kUpdateUIMinVersion = 0,
    kAbortMinVersion = 0,
    kGetRegistrationMinVersion = 0,
    kGetTagsMinVersion = 0,
  };
  virtual ~BackgroundFetchService() {}


  using FetchCallback = base::OnceCallback<void(BackgroundFetchError, const base::Optional<content::BackgroundFetchRegistration>&)>;
  virtual void Fetch(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, const std::vector<::content::ServiceWorkerFetchRequest>& requests, const content::BackgroundFetchOptions& options, FetchCallback callback) = 0;


  using UpdateUICallback = base::OnceCallback<void(BackgroundFetchError)>;
  virtual void UpdateUI(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, const std::string& title, UpdateUICallback callback) = 0;


  using AbortCallback = base::OnceCallback<void(BackgroundFetchError)>;
  virtual void Abort(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, AbortCallback callback) = 0;


  using GetRegistrationCallback = base::OnceCallback<void(BackgroundFetchError, const base::Optional<content::BackgroundFetchRegistration>&)>;
  virtual void GetRegistration(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, GetRegistrationCallback callback) = 0;


  using GetTagsCallback = base::OnceCallback<void(BackgroundFetchError, const std::vector<std::string>&)>;
  virtual void GetTags(int64_t service_worker_registration_id, const url::Origin& origin, GetTagsCallback callback) = 0;
};

class CONTENT_EXPORT BackgroundFetchServiceProxy
    : public BackgroundFetchService {
 public:
  explicit BackgroundFetchServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void Fetch(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, const std::vector<::content::ServiceWorkerFetchRequest>& requests, const content::BackgroundFetchOptions& options, FetchCallback callback) override;
  void UpdateUI(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, const std::string& title, UpdateUICallback callback) override;
  void Abort(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, AbortCallback callback) override;
  void GetRegistration(int64_t service_worker_registration_id, const url::Origin& origin, const std::string& tag, GetRegistrationCallback callback) override;
  void GetTags(int64_t service_worker_registration_id, const url::Origin& origin, GetTagsCallback callback) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT BackgroundFetchServiceStubDispatch {
 public:
  static bool Accept(BackgroundFetchService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      BackgroundFetchService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<BackgroundFetchService>>
class BackgroundFetchServiceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  BackgroundFetchServiceStub() {}
  ~BackgroundFetchServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BackgroundFetchServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return BackgroundFetchServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT BackgroundFetchServiceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT BackgroundFetchServiceResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};


class CONTENT_EXPORT IconDefinition {
 public:
  using DataView = IconDefinitionDataView;
  using Data_ = internal::IconDefinition_Data;

  template <typename... Args>
  static IconDefinitionPtr New(Args&&... args) {
    return IconDefinitionPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static IconDefinitionPtr From(const U& u) {
    return mojo::TypeConverter<IconDefinitionPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, IconDefinition>::Convert(*this);
  }


  IconDefinition();

  IconDefinition(
      const std::string& src,
      const std::string& sizes,
      const std::string& type);

  ~IconDefinition();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = IconDefinitionPtr>
  IconDefinitionPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, IconDefinition>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        IconDefinition::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        IconDefinition::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string src;
  std::string sizes;
  std::string type;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};








class CONTENT_EXPORT BackgroundFetchOptions {
 public:
  using DataView = BackgroundFetchOptionsDataView;
  using Data_ = internal::BackgroundFetchOptions_Data;

  template <typename... Args>
  static BackgroundFetchOptionsPtr New(Args&&... args) {
    return BackgroundFetchOptionsPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BackgroundFetchOptionsPtr From(const U& u) {
    return mojo::TypeConverter<BackgroundFetchOptionsPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BackgroundFetchOptions>::Convert(*this);
  }


  BackgroundFetchOptions();

  BackgroundFetchOptions(
      const std::vector<content::IconDefinition>& icons,
      int64_t total_download_size,
      const std::string& title);

  ~BackgroundFetchOptions();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BackgroundFetchOptionsPtr>
  BackgroundFetchOptionsPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BackgroundFetchOptions>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BackgroundFetchOptions::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BackgroundFetchOptions::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::vector<content::IconDefinition> icons;
  int64_t total_download_size;
  std::string title;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class CONTENT_EXPORT BackgroundFetchRegistration {
 public:
  using DataView = BackgroundFetchRegistrationDataView;
  using Data_ = internal::BackgroundFetchRegistration_Data;

  template <typename... Args>
  static BackgroundFetchRegistrationPtr New(Args&&... args) {
    return BackgroundFetchRegistrationPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static BackgroundFetchRegistrationPtr From(const U& u) {
    return mojo::TypeConverter<BackgroundFetchRegistrationPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, BackgroundFetchRegistration>::Convert(*this);
  }


  BackgroundFetchRegistration();

  BackgroundFetchRegistration(
      const std::string& tag,
      const std::vector<content::IconDefinition>& icons,
      int64_t total_download_size,
      const std::string& title);

  ~BackgroundFetchRegistration();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = BackgroundFetchRegistrationPtr>
  BackgroundFetchRegistrationPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, BackgroundFetchRegistration>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        BackgroundFetchRegistration::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        BackgroundFetchRegistration::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  std::string tag;
  std::vector<content::IconDefinition> icons;
  int64_t total_download_size;
  std::string title;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
IconDefinitionPtr IconDefinition::Clone() const {
  return New(
      mojo::Clone(src),
      mojo::Clone(sizes),
      mojo::Clone(type)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, IconDefinition>::value>::type*>
bool IconDefinition::Equals(const T& other) const {
  if (!mojo::Equals(this->src, other.src))
    return false;
  if (!mojo::Equals(this->sizes, other.sizes))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  return true;
}
template <typename StructPtrType>
BackgroundFetchOptionsPtr BackgroundFetchOptions::Clone() const {
  return New(
      mojo::Clone(icons),
      mojo::Clone(total_download_size),
      mojo::Clone(title)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BackgroundFetchOptions>::value>::type*>
bool BackgroundFetchOptions::Equals(const T& other) const {
  if (!mojo::Equals(this->icons, other.icons))
    return false;
  if (!mojo::Equals(this->total_download_size, other.total_download_size))
    return false;
  if (!mojo::Equals(this->title, other.title))
    return false;
  return true;
}
template <typename StructPtrType>
BackgroundFetchRegistrationPtr BackgroundFetchRegistration::Clone() const {
  return New(
      mojo::Clone(tag),
      mojo::Clone(icons),
      mojo::Clone(total_download_size),
      mojo::Clone(title)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, BackgroundFetchRegistration>::value>::type*>
bool BackgroundFetchRegistration::Equals(const T& other) const {
  if (!mojo::Equals(this->tag, other.tag))
    return false;
  if (!mojo::Equals(this->icons, other.icons))
    return false;
  if (!mojo::Equals(this->total_download_size, other.total_download_size))
    return false;
  if (!mojo::Equals(this->title, other.title))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::IconDefinition::DataView,
                                         ::blink::mojom::IconDefinitionPtr> {
  static bool IsNull(const ::blink::mojom::IconDefinitionPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::IconDefinitionPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::IconDefinition::src)& src(
      const ::blink::mojom::IconDefinitionPtr& input) {
    return input->src;
  }

  static const decltype(::blink::mojom::IconDefinition::sizes)& sizes(
      const ::blink::mojom::IconDefinitionPtr& input) {
    return input->sizes;
  }

  static const decltype(::blink::mojom::IconDefinition::type)& type(
      const ::blink::mojom::IconDefinitionPtr& input) {
    return input->type;
  }

  static bool Read(::blink::mojom::IconDefinition::DataView input, ::blink::mojom::IconDefinitionPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::BackgroundFetchOptions::DataView,
                                         ::blink::mojom::BackgroundFetchOptionsPtr> {
  static bool IsNull(const ::blink::mojom::BackgroundFetchOptionsPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::BackgroundFetchOptionsPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::BackgroundFetchOptions::icons)& icons(
      const ::blink::mojom::BackgroundFetchOptionsPtr& input) {
    return input->icons;
  }

  static decltype(::blink::mojom::BackgroundFetchOptions::total_download_size) total_download_size(
      const ::blink::mojom::BackgroundFetchOptionsPtr& input) {
    return input->total_download_size;
  }

  static const decltype(::blink::mojom::BackgroundFetchOptions::title)& title(
      const ::blink::mojom::BackgroundFetchOptionsPtr& input) {
    return input->title;
  }

  static bool Read(::blink::mojom::BackgroundFetchOptions::DataView input, ::blink::mojom::BackgroundFetchOptionsPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::BackgroundFetchRegistration::DataView,
                                         ::blink::mojom::BackgroundFetchRegistrationPtr> {
  static bool IsNull(const ::blink::mojom::BackgroundFetchRegistrationPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::BackgroundFetchRegistrationPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::BackgroundFetchRegistration::tag)& tag(
      const ::blink::mojom::BackgroundFetchRegistrationPtr& input) {
    return input->tag;
  }

  static const decltype(::blink::mojom::BackgroundFetchRegistration::icons)& icons(
      const ::blink::mojom::BackgroundFetchRegistrationPtr& input) {
    return input->icons;
  }

  static decltype(::blink::mojom::BackgroundFetchRegistration::total_download_size) total_download_size(
      const ::blink::mojom::BackgroundFetchRegistrationPtr& input) {
    return input->total_download_size;
  }

  static const decltype(::blink::mojom::BackgroundFetchRegistration::title)& title(
      const ::blink::mojom::BackgroundFetchRegistrationPtr& input) {
    return input->title;
  }

  static bool Read(::blink::mojom::BackgroundFetchRegistration::DataView input, ::blink::mojom::BackgroundFetchRegistrationPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_BACKGROUND_FETCH_BACKGROUND_FETCH_MOJOM_H_