// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_KEYBOARD_LOCK_KEYBOARD_LOCK_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_KEYBOARD_LOCK_KEYBOARD_LOCK_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/keyboard_lock/keyboard_lock.mojom-shared.h"
#include <string>
#include <vector>
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class KeyboardLockService;
using KeyboardLockServicePtr = mojo::InterfacePtr<KeyboardLockService>;
using KeyboardLockServicePtrInfo = mojo::InterfacePtrInfo<KeyboardLockService>;
using ThreadSafeKeyboardLockServicePtr =
    mojo::ThreadSafeInterfacePtr<KeyboardLockService>;
using KeyboardLockServiceRequest = mojo::InterfaceRequest<KeyboardLockService>;
using KeyboardLockServiceAssociatedPtr =
    mojo::AssociatedInterfacePtr<KeyboardLockService>;
using ThreadSafeKeyboardLockServiceAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<KeyboardLockService>;
using KeyboardLockServiceAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<KeyboardLockService>;
using KeyboardLockServiceAssociatedRequest =
    mojo::AssociatedInterfaceRequest<KeyboardLockService>;


class KeyboardLockServiceProxy;

template <typename ImplRefTraits>
class KeyboardLockServiceStub;

class KeyboardLockServiceRequestValidator;
class KeyboardLockServiceResponseValidator;

class CONTENT_EXPORT KeyboardLockService
    : public KeyboardLockServiceInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = KeyboardLockServiceProxy;

  template <typename ImplRefTraits>
  using Stub_ = KeyboardLockServiceStub<ImplRefTraits>;

  using RequestValidator_ = KeyboardLockServiceRequestValidator;
  using ResponseValidator_ = KeyboardLockServiceResponseValidator;
  enum MethodMinVersions : uint32_t {
    kRequestKeyboardLockMinVersion = 0,
    kCancelKeyboardLockMinVersion = 0,
  };
  virtual ~KeyboardLockService() {}


  using RequestKeyboardLockCallback = base::OnceCallback<void(KeyboardLockRequestResult)>;
  virtual void RequestKeyboardLock(const std::vector<std::string>& key_codes, RequestKeyboardLockCallback callback) = 0;

  virtual void CancelKeyboardLock() = 0;
};

class CONTENT_EXPORT KeyboardLockServiceProxy
    : public KeyboardLockService {
 public:
  explicit KeyboardLockServiceProxy(mojo::MessageReceiverWithResponder* receiver);
  void RequestKeyboardLock(const std::vector<std::string>& key_codes, RequestKeyboardLockCallback callback) override;
  void CancelKeyboardLock() override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class CONTENT_EXPORT KeyboardLockServiceStubDispatch {
 public:
  static bool Accept(KeyboardLockService* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      KeyboardLockService* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<KeyboardLockService>>
class KeyboardLockServiceStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  KeyboardLockServiceStub() {}
  ~KeyboardLockServiceStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return KeyboardLockServiceStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return KeyboardLockServiceStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class CONTENT_EXPORT KeyboardLockServiceRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class CONTENT_EXPORT KeyboardLockServiceResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_KEYBOARD_LOCK_KEYBOARD_LOCK_MOJOM_H_