// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_MANIFEST_MANIFEST_MOJOM_SHARED_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_MANIFEST_MANIFEST_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/WebKit/public/platform/modules/manifest/manifest.mojom-shared-internal.h"
#include "device/screen_orientation/public/interfaces/screen_orientation_lock_types.mojom-shared.h"
#include "mojo/common/string16.mojom-shared.h"
#include "third_party/WebKit/public/platform/display_mode.mojom-shared.h"
#include "ui/gfx/geometry/mojo/geometry.mojom-shared.h"
#include "url/mojo/url.mojom-shared.h"
#include "third_party/WebKit/public/mojo_bindings_shared_export.h"


namespace blink {
namespace mojom {
class ManifestDataView;

class ManifestIconDataView;

class ManifestRelatedApplicationDataView;

class ManifestShareTargetDataView;



}  // namespace mojom
}  // namespace blink

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::blink::mojom::ManifestDataView> {
  using Data = ::blink::mojom::internal::Manifest_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::blink::mojom::ManifestIconDataView> {
  using Data = ::blink::mojom::internal::ManifestIcon_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::blink::mojom::ManifestRelatedApplicationDataView> {
  using Data = ::blink::mojom::internal::ManifestRelatedApplication_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::blink::mojom::ManifestShareTargetDataView> {
  using Data = ::blink::mojom::internal::ManifestShareTarget_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace blink {
namespace mojom {

enum class ManifestIcon_Purpose : int32_t {
  ANY = 0,
  BADGE,
};

inline std::ostream& operator<<(std::ostream& os, ManifestIcon_Purpose value) {
  switch(value) {
    case ManifestIcon_Purpose::ANY:
      return os << "ManifestIcon_Purpose::ANY";
    case ManifestIcon_Purpose::BADGE:
      return os << "ManifestIcon_Purpose::BADGE";
    default:
      return os << "Unknown ManifestIcon_Purpose value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(ManifestIcon_Purpose value) {
  return internal::ManifestIcon_Purpose_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
class ManifestDataView {
 public:
  ManifestDataView() {}

  ManifestDataView(
      internal::Manifest_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNameDataView(
      ::mojo::common::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::String16DataView>(
        pointer, output, context_);
  }
  inline void GetShortNameDataView(
      ::mojo::common::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadShortName(UserType* output) {
    auto* pointer = data_->short_name.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::String16DataView>(
        pointer, output, context_);
  }
  inline void GetStartUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStartUrl(UserType* output) {
    auto* pointer = data_->start_url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDisplay(UserType* output) const {
    auto data_value = data_->display;
    return mojo::internal::Deserialize<::blink::mojom::DisplayMode>(
        data_value, output);
  }

  ::blink::mojom::DisplayMode display() const {
    return static_cast<::blink::mojom::DisplayMode>(data_->display);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOrientation(UserType* output) const {
    auto data_value = data_->orientation;
    return mojo::internal::Deserialize<::device::mojom::ScreenOrientationLockType>(
        data_value, output);
  }

  ::device::mojom::ScreenOrientationLockType orientation() const {
    return static_cast<::device::mojom::ScreenOrientationLockType>(data_->orientation);
  }
  inline void GetIconsDataView(
      mojo::ArrayDataView<ManifestIconDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIcons(UserType* output) {
    auto* pointer = data_->icons.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::blink::mojom::ManifestIconDataView>>(
        pointer, output, context_);
  }
  inline void GetShareTargetDataView(
      ManifestShareTargetDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadShareTarget(UserType* output) {
    auto* pointer = data_->share_target.Get();
    return mojo::internal::Deserialize<::blink::mojom::ManifestShareTargetDataView>(
        pointer, output, context_);
  }
  inline void GetRelatedApplicationsDataView(
      mojo::ArrayDataView<ManifestRelatedApplicationDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRelatedApplications(UserType* output) {
    auto* pointer = data_->related_applications.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::blink::mojom::ManifestRelatedApplicationDataView>>(
        pointer, output, context_);
  }
  bool prefer_related_applications() const {
    return data_->prefer_related_applications;
  }
  int64_t theme_color() const {
    return data_->theme_color;
  }
  int64_t background_color() const {
    return data_->background_color;
  }
  inline void GetGcmSenderIdDataView(
      ::mojo::common::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadGcmSenderId(UserType* output) {
    auto* pointer = data_->gcm_sender_id.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::String16DataView>(
        pointer, output, context_);
  }
  inline void GetScopeDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadScope(UserType* output) {
    auto* pointer = data_->scope.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::Manifest_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ManifestIconDataView {
 public:
  ManifestIconDataView() {}

  ManifestIconDataView(
      internal::ManifestIcon_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSrcDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSrc(UserType* output) {
    auto* pointer = data_->src.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetTypeDataView(
      ::mojo::common::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) {
    auto* pointer = data_->type.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::String16DataView>(
        pointer, output, context_);
  }
  inline void GetSizesDataView(
      mojo::ArrayDataView<::gfx::mojom::SizeDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSizes(UserType* output) {
    auto* pointer = data_->sizes.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::gfx::mojom::SizeDataView>>(
        pointer, output, context_);
  }
  inline void GetPurposeDataView(
      mojo::ArrayDataView<ManifestIcon_Purpose>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPurpose(UserType* output) {
    auto* pointer = data_->purpose.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::blink::mojom::ManifestIcon_Purpose>>(
        pointer, output, context_);
  }
 private:
  internal::ManifestIcon_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ManifestRelatedApplicationDataView {
 public:
  ManifestRelatedApplicationDataView() {}

  ManifestRelatedApplicationDataView(
      internal::ManifestRelatedApplication_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetPlatformDataView(
      ::mojo::common::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPlatform(UserType* output) {
    auto* pointer = data_->platform.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::String16DataView>(
        pointer, output, context_);
  }
  inline void GetUrlDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrl(UserType* output) {
    auto* pointer = data_->url.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetIdDataView(
      ::mojo::common::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::String16DataView>(
        pointer, output, context_);
  }
 private:
  internal::ManifestRelatedApplication_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class ManifestShareTargetDataView {
 public:
  ManifestShareTargetDataView() {}

  ManifestShareTargetDataView(
      internal::ManifestShareTarget_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetUrlTemplateDataView(
      ::mojo::common::mojom::String16DataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUrlTemplate(UserType* output) {
    auto* pointer = data_->url_template.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::String16DataView>(
        pointer, output, context_);
  }
 private:
  internal::ManifestShareTarget_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace blink

namespace std {

template <>
struct hash<::blink::mojom::ManifestIcon_Purpose>
    : public mojo::internal::EnumHashImpl<::blink::mojom::ManifestIcon_Purpose> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::blink::mojom::ManifestIcon_Purpose, ::blink::mojom::ManifestIcon_Purpose> {
  static ::blink::mojom::ManifestIcon_Purpose ToMojom(::blink::mojom::ManifestIcon_Purpose input) { return input; }
  static bool FromMojom(::blink::mojom::ManifestIcon_Purpose input, ::blink::mojom::ManifestIcon_Purpose* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ManifestIcon_Purpose, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::blink::mojom::ManifestIcon_Purpose, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::blink::mojom::ManifestIcon_Purpose>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ManifestDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::ManifestDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::blink::mojom::internal::Manifest_Data);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::String16DataView>(
        in_name, context);
    decltype(CallWithContext(Traits::short_name, input, custom_context)) in_short_name = CallWithContext(Traits::short_name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::String16DataView>(
        in_short_name, context);
    decltype(CallWithContext(Traits::start_url, input, custom_context)) in_start_url = CallWithContext(Traits::start_url, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_start_url, context);
    decltype(CallWithContext(Traits::icons, input, custom_context)) in_icons = CallWithContext(Traits::icons, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::blink::mojom::ManifestIconDataView>>(
        in_icons, context);
    decltype(CallWithContext(Traits::share_target, input, custom_context)) in_share_target = CallWithContext(Traits::share_target, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::blink::mojom::ManifestShareTargetDataView>(
        in_share_target, context);
    decltype(CallWithContext(Traits::related_applications, input, custom_context)) in_related_applications = CallWithContext(Traits::related_applications, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::blink::mojom::ManifestRelatedApplicationDataView>>(
        in_related_applications, context);
    decltype(CallWithContext(Traits::gcm_sender_id, input, custom_context)) in_gcm_sender_id = CallWithContext(Traits::gcm_sender_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::String16DataView>(
        in_gcm_sender_id, context);
    decltype(CallWithContext(Traits::scope, input, custom_context)) in_scope = CallWithContext(Traits::scope, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_scope, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::Manifest_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::blink::mojom::internal::Manifest_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    typename decltype(result->name)::BaseType* name_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::String16DataView>(
        in_name, buffer, &name_ptr, context);
    result->name.Set(name_ptr);
    decltype(CallWithContext(Traits::short_name, input, custom_context)) in_short_name = CallWithContext(Traits::short_name, input, custom_context);
    typename decltype(result->short_name)::BaseType* short_name_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::String16DataView>(
        in_short_name, buffer, &short_name_ptr, context);
    result->short_name.Set(short_name_ptr);
    decltype(CallWithContext(Traits::start_url, input, custom_context)) in_start_url = CallWithContext(Traits::start_url, input, custom_context);
    typename decltype(result->start_url)::BaseType* start_url_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_start_url, buffer, &start_url_ptr, context);
    result->start_url.Set(start_url_ptr);
    mojo::internal::Serialize<::blink::mojom::DisplayMode>(
        CallWithContext(Traits::display, input, custom_context), &result->display);
    mojo::internal::Serialize<::device::mojom::ScreenOrientationLockType>(
        CallWithContext(Traits::orientation, input, custom_context), &result->orientation);
    decltype(CallWithContext(Traits::icons, input, custom_context)) in_icons = CallWithContext(Traits::icons, input, custom_context);
    typename decltype(result->icons)::BaseType* icons_ptr;
    const mojo::internal::ContainerValidateParams icons_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::blink::mojom::ManifestIconDataView>>(
        in_icons, buffer, &icons_ptr, &icons_validate_params,
        context);
    result->icons.Set(icons_ptr);
    decltype(CallWithContext(Traits::share_target, input, custom_context)) in_share_target = CallWithContext(Traits::share_target, input, custom_context);
    typename decltype(result->share_target)::BaseType* share_target_ptr;
    mojo::internal::Serialize<::blink::mojom::ManifestShareTargetDataView>(
        in_share_target, buffer, &share_target_ptr, context);
    result->share_target.Set(share_target_ptr);
    decltype(CallWithContext(Traits::related_applications, input, custom_context)) in_related_applications = CallWithContext(Traits::related_applications, input, custom_context);
    typename decltype(result->related_applications)::BaseType* related_applications_ptr;
    const mojo::internal::ContainerValidateParams related_applications_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::blink::mojom::ManifestRelatedApplicationDataView>>(
        in_related_applications, buffer, &related_applications_ptr, &related_applications_validate_params,
        context);
    result->related_applications.Set(related_applications_ptr);
    result->prefer_related_applications = CallWithContext(Traits::prefer_related_applications, input, custom_context);
    result->theme_color = CallWithContext(Traits::theme_color, input, custom_context);
    result->background_color = CallWithContext(Traits::background_color, input, custom_context);
    decltype(CallWithContext(Traits::gcm_sender_id, input, custom_context)) in_gcm_sender_id = CallWithContext(Traits::gcm_sender_id, input, custom_context);
    typename decltype(result->gcm_sender_id)::BaseType* gcm_sender_id_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::String16DataView>(
        in_gcm_sender_id, buffer, &gcm_sender_id_ptr, context);
    result->gcm_sender_id.Set(gcm_sender_id_ptr);
    decltype(CallWithContext(Traits::scope, input, custom_context)) in_scope = CallWithContext(Traits::scope, input, custom_context);
    typename decltype(result->scope)::BaseType* scope_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_scope, buffer, &scope_ptr, context);
    result->scope.Set(scope_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::Manifest_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::ManifestDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ManifestIconDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::ManifestIconDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::blink::mojom::internal::ManifestIcon_Data);
    decltype(CallWithContext(Traits::src, input, custom_context)) in_src = CallWithContext(Traits::src, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_src, context);
    decltype(CallWithContext(Traits::type, input, custom_context)) in_type = CallWithContext(Traits::type, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::String16DataView>(
        in_type, context);
    decltype(CallWithContext(Traits::sizes, input, custom_context)) in_sizes = CallWithContext(Traits::sizes, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::gfx::mojom::SizeDataView>>(
        in_sizes, context);
    decltype(CallWithContext(Traits::purpose, input, custom_context)) in_purpose = CallWithContext(Traits::purpose, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::blink::mojom::ManifestIcon_Purpose>>(
        in_purpose, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::ManifestIcon_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::blink::mojom::internal::ManifestIcon_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::src, input, custom_context)) in_src = CallWithContext(Traits::src, input, custom_context);
    typename decltype(result->src)::BaseType* src_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_src, buffer, &src_ptr, context);
    result->src.Set(src_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->src.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null src in ManifestIcon struct");
    decltype(CallWithContext(Traits::type, input, custom_context)) in_type = CallWithContext(Traits::type, input, custom_context);
    typename decltype(result->type)::BaseType* type_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::String16DataView>(
        in_type, buffer, &type_ptr, context);
    result->type.Set(type_ptr);
    decltype(CallWithContext(Traits::sizes, input, custom_context)) in_sizes = CallWithContext(Traits::sizes, input, custom_context);
    typename decltype(result->sizes)::BaseType* sizes_ptr;
    const mojo::internal::ContainerValidateParams sizes_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::gfx::mojom::SizeDataView>>(
        in_sizes, buffer, &sizes_ptr, &sizes_validate_params,
        context);
    result->sizes.Set(sizes_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->sizes.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null sizes in ManifestIcon struct");
    decltype(CallWithContext(Traits::purpose, input, custom_context)) in_purpose = CallWithContext(Traits::purpose, input, custom_context);
    typename decltype(result->purpose)::BaseType* purpose_ptr;
    const mojo::internal::ContainerValidateParams purpose_validate_params(
        0, ::blink::mojom::internal::ManifestIcon_Purpose_Data::Validate);
    mojo::internal::Serialize<mojo::ArrayDataView<::blink::mojom::ManifestIcon_Purpose>>(
        in_purpose, buffer, &purpose_ptr, &purpose_validate_params,
        context);
    result->purpose.Set(purpose_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->purpose.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null purpose in ManifestIcon struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::ManifestIcon_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::ManifestIconDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ManifestRelatedApplicationDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::ManifestRelatedApplicationDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::blink::mojom::internal::ManifestRelatedApplication_Data);
    decltype(CallWithContext(Traits::platform, input, custom_context)) in_platform = CallWithContext(Traits::platform, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::String16DataView>(
        in_platform, context);
    decltype(CallWithContext(Traits::url, input, custom_context)) in_url = CallWithContext(Traits::url, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_url, context);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::String16DataView>(
        in_id, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::ManifestRelatedApplication_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::blink::mojom::internal::ManifestRelatedApplication_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::platform, input, custom_context)) in_platform = CallWithContext(Traits::platform, input, custom_context);
    typename decltype(result->platform)::BaseType* platform_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::String16DataView>(
        in_platform, buffer, &platform_ptr, context);
    result->platform.Set(platform_ptr);
    decltype(CallWithContext(Traits::url, input, custom_context)) in_url = CallWithContext(Traits::url, input, custom_context);
    typename decltype(result->url)::BaseType* url_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_url, buffer, &url_ptr, context);
    result->url.Set(url_ptr);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    typename decltype(result->id)::BaseType* id_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::String16DataView>(
        in_id, buffer, &id_ptr, context);
    result->id.Set(id_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::ManifestRelatedApplication_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::ManifestRelatedApplicationDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::blink::mojom::ManifestShareTargetDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::blink::mojom::ManifestShareTargetDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::blink::mojom::internal::ManifestShareTarget_Data);
    decltype(CallWithContext(Traits::url_template, input, custom_context)) in_url_template = CallWithContext(Traits::url_template, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::String16DataView>(
        in_url_template, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::blink::mojom::internal::ManifestShareTarget_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::blink::mojom::internal::ManifestShareTarget_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::url_template, input, custom_context)) in_url_template = CallWithContext(Traits::url_template, input, custom_context);
    typename decltype(result->url_template)::BaseType* url_template_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::String16DataView>(
        in_url_template, buffer, &url_template_ptr, context);
    result->url_template.Set(url_template_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::blink::mojom::internal::ManifestShareTarget_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::blink::mojom::ManifestShareTargetDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace blink {
namespace mojom {

inline void ManifestDataView::GetNameDataView(
    ::mojo::common::mojom::String16DataView* output) {
  auto pointer = data_->name.Get();
  *output = ::mojo::common::mojom::String16DataView(pointer, context_);
}
inline void ManifestDataView::GetShortNameDataView(
    ::mojo::common::mojom::String16DataView* output) {
  auto pointer = data_->short_name.Get();
  *output = ::mojo::common::mojom::String16DataView(pointer, context_);
}
inline void ManifestDataView::GetStartUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->start_url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void ManifestDataView::GetIconsDataView(
    mojo::ArrayDataView<ManifestIconDataView>* output) {
  auto pointer = data_->icons.Get();
  *output = mojo::ArrayDataView<ManifestIconDataView>(pointer, context_);
}
inline void ManifestDataView::GetShareTargetDataView(
    ManifestShareTargetDataView* output) {
  auto pointer = data_->share_target.Get();
  *output = ManifestShareTargetDataView(pointer, context_);
}
inline void ManifestDataView::GetRelatedApplicationsDataView(
    mojo::ArrayDataView<ManifestRelatedApplicationDataView>* output) {
  auto pointer = data_->related_applications.Get();
  *output = mojo::ArrayDataView<ManifestRelatedApplicationDataView>(pointer, context_);
}
inline void ManifestDataView::GetGcmSenderIdDataView(
    ::mojo::common::mojom::String16DataView* output) {
  auto pointer = data_->gcm_sender_id.Get();
  *output = ::mojo::common::mojom::String16DataView(pointer, context_);
}
inline void ManifestDataView::GetScopeDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->scope.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void ManifestIconDataView::GetSrcDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->src.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void ManifestIconDataView::GetTypeDataView(
    ::mojo::common::mojom::String16DataView* output) {
  auto pointer = data_->type.Get();
  *output = ::mojo::common::mojom::String16DataView(pointer, context_);
}
inline void ManifestIconDataView::GetSizesDataView(
    mojo::ArrayDataView<::gfx::mojom::SizeDataView>* output) {
  auto pointer = data_->sizes.Get();
  *output = mojo::ArrayDataView<::gfx::mojom::SizeDataView>(pointer, context_);
}
inline void ManifestIconDataView::GetPurposeDataView(
    mojo::ArrayDataView<ManifestIcon_Purpose>* output) {
  auto pointer = data_->purpose.Get();
  *output = mojo::ArrayDataView<ManifestIcon_Purpose>(pointer, context_);
}


inline void ManifestRelatedApplicationDataView::GetPlatformDataView(
    ::mojo::common::mojom::String16DataView* output) {
  auto pointer = data_->platform.Get();
  *output = ::mojo::common::mojom::String16DataView(pointer, context_);
}
inline void ManifestRelatedApplicationDataView::GetUrlDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->url.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void ManifestRelatedApplicationDataView::GetIdDataView(
    ::mojo::common::mojom::String16DataView* output) {
  auto pointer = data_->id.Get();
  *output = ::mojo::common::mojom::String16DataView(pointer, context_);
}


inline void ManifestShareTargetDataView::GetUrlTemplateDataView(
    ::mojo::common::mojom::String16DataView* output) {
  auto pointer = data_->url_template.Get();
  *output = ::mojo::common::mojom::String16DataView(pointer, context_);
}



}  // namespace mojom
}  // namespace blink

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_MANIFEST_MANIFEST_MOJOM_SHARED_H_
