// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_MANIFEST_MANIFEST_MOJOM_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_MANIFEST_MANIFEST_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "third_party/WebKit/public/platform/modules/manifest/manifest.mojom-shared.h"
#include "device/screen_orientation/public/interfaces/screen_orientation_lock_types.mojom.h"
#include "mojo/common/string16.mojom.h"
#include "third_party/WebKit/public/platform/display_mode.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include "url/mojo/url.mojom.h"
#include <string>
#include <vector>
#include "content/public/common/manifest.h"
#include "content/common/content_export.h"


namespace blink {
namespace mojom {
class Manifest;
using ManifestPtr = mojo::StructPtr<Manifest>;

class ManifestIcon;
using ManifestIconPtr = mojo::StructPtr<ManifestIcon>;

class ManifestRelatedApplication;
using ManifestRelatedApplicationPtr = mojo::StructPtr<ManifestRelatedApplication>;

class ManifestShareTarget;
using ManifestShareTargetPtr = mojo::StructPtr<ManifestShareTarget>;









class CONTENT_EXPORT Manifest {
 public:
  using DataView = ManifestDataView;
  using Data_ = internal::Manifest_Data;

  template <typename... Args>
  static ManifestPtr New(Args&&... args) {
    return ManifestPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ManifestPtr From(const U& u) {
    return mojo::TypeConverter<ManifestPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, Manifest>::Convert(*this);
  }


  Manifest();

  Manifest(
      const base::Optional<base::string16>& name,
      const base::Optional<base::string16>& short_name,
      const base::Optional<GURL>& start_url,
      ::blink::WebDisplayMode display,
      ::blink::WebScreenOrientationLockType orientation,
      const base::Optional<std::vector<content::Manifest::Icon>>& icons,
      const base::Optional<content::Manifest::ShareTarget>& share_target,
      const base::Optional<std::vector<content::Manifest::RelatedApplication>>& related_applications,
      bool prefer_related_applications,
      int64_t theme_color,
      int64_t background_color,
      const base::Optional<base::string16>& gcm_sender_id,
      const base::Optional<GURL>& scope);

  ~Manifest();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ManifestPtr>
  ManifestPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, Manifest>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        Manifest::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        Manifest::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::Optional<base::string16> name;
  base::Optional<base::string16> short_name;
  base::Optional<GURL> start_url;
  ::blink::WebDisplayMode display;
  ::blink::WebScreenOrientationLockType orientation;
  base::Optional<std::vector<content::Manifest::Icon>> icons;
  base::Optional<content::Manifest::ShareTarget> share_target;
  base::Optional<std::vector<content::Manifest::RelatedApplication>> related_applications;
  bool prefer_related_applications;
  int64_t theme_color;
  int64_t background_color;
  base::Optional<base::string16> gcm_sender_id;
  base::Optional<GURL> scope;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class CONTENT_EXPORT ManifestIcon {
 public:
  using DataView = ManifestIconDataView;
  using Data_ = internal::ManifestIcon_Data;using Purpose = ManifestIcon_Purpose;

  template <typename... Args>
  static ManifestIconPtr New(Args&&... args) {
    return ManifestIconPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ManifestIconPtr From(const U& u) {
    return mojo::TypeConverter<ManifestIconPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ManifestIcon>::Convert(*this);
  }


  ManifestIcon();

  ManifestIcon(
      const GURL& src,
      const base::Optional<base::string16>& type,
      const std::vector<gfx::Size>& sizes,
      const std::vector<ManifestIcon::Purpose>& purpose);

  ~ManifestIcon();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ManifestIconPtr>
  ManifestIconPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ManifestIcon>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ManifestIcon::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ManifestIcon::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  GURL src;
  base::Optional<base::string16> type;
  std::vector<gfx::Size> sizes;
  std::vector<ManifestIcon::Purpose> purpose;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class CONTENT_EXPORT ManifestRelatedApplication {
 public:
  using DataView = ManifestRelatedApplicationDataView;
  using Data_ = internal::ManifestRelatedApplication_Data;

  template <typename... Args>
  static ManifestRelatedApplicationPtr New(Args&&... args) {
    return ManifestRelatedApplicationPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ManifestRelatedApplicationPtr From(const U& u) {
    return mojo::TypeConverter<ManifestRelatedApplicationPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ManifestRelatedApplication>::Convert(*this);
  }


  ManifestRelatedApplication();

  ManifestRelatedApplication(
      const base::Optional<base::string16>& platform,
      const base::Optional<GURL>& url,
      const base::Optional<base::string16>& id);

  ~ManifestRelatedApplication();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ManifestRelatedApplicationPtr>
  ManifestRelatedApplicationPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ManifestRelatedApplication>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ManifestRelatedApplication::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ManifestRelatedApplication::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::Optional<base::string16> platform;
  base::Optional<GURL> url;
  base::Optional<base::string16> id;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};



class CONTENT_EXPORT ManifestShareTarget {
 public:
  using DataView = ManifestShareTargetDataView;
  using Data_ = internal::ManifestShareTarget_Data;

  template <typename... Args>
  static ManifestShareTargetPtr New(Args&&... args) {
    return ManifestShareTargetPtr(
        base::in_place,
        std::forward<Args>(args)...);
  }

  template <typename U>
  static ManifestShareTargetPtr From(const U& u) {
    return mojo::TypeConverter<ManifestShareTargetPtr, U>::Convert(u);
  }

  template <typename U>
  U To() const {
    return mojo::TypeConverter<U, ManifestShareTarget>::Convert(*this);
  }


  ManifestShareTarget();

  explicit ManifestShareTarget(
      const base::Optional<base::string16>& url_template);

  ~ManifestShareTarget();

  // Clone() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Clone() or copy
  // constructor/assignment are available for members.
  template <typename StructPtrType = ManifestShareTargetPtr>
  ManifestShareTargetPtr Clone() const;

  // Equals() is a template so it is only instantiated if it is used. Thus, the
  // bindings generator does not need to know whether Equals() or == operator
  // are available for members.
  template <typename T,
            typename std::enable_if<std::is_same<
                T, ManifestShareTarget>::value>::type* = nullptr>
  bool Equals(const T& other) const;

  template <typename UserType>
  static std::vector<uint8_t> Serialize(UserType* input) {
    return mojo::internal::StructSerializeImpl<
        ManifestShareTarget::DataView, std::vector<uint8_t>>(input);
  }

  template <typename UserType>
  static bool Deserialize(const std::vector<uint8_t>& input,
                          UserType* output) {
    return mojo::internal::StructDeserializeImpl<
        ManifestShareTarget::DataView, std::vector<uint8_t>>(
            input, output, Validate);
  }

  base::Optional<base::string16> url_template;

 private:
  static bool Validate(const void* data,
                       mojo::internal::ValidationContext* validation_context);
};


template <typename StructPtrType>
ManifestPtr Manifest::Clone() const {
  return New(
      mojo::Clone(name),
      mojo::Clone(short_name),
      mojo::Clone(start_url),
      mojo::Clone(display),
      mojo::Clone(orientation),
      mojo::Clone(icons),
      mojo::Clone(share_target),
      mojo::Clone(related_applications),
      mojo::Clone(prefer_related_applications),
      mojo::Clone(theme_color),
      mojo::Clone(background_color),
      mojo::Clone(gcm_sender_id),
      mojo::Clone(scope)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, Manifest>::value>::type*>
bool Manifest::Equals(const T& other) const {
  if (!mojo::Equals(this->name, other.name))
    return false;
  if (!mojo::Equals(this->short_name, other.short_name))
    return false;
  if (!mojo::Equals(this->start_url, other.start_url))
    return false;
  if (!mojo::Equals(this->display, other.display))
    return false;
  if (!mojo::Equals(this->orientation, other.orientation))
    return false;
  if (!mojo::Equals(this->icons, other.icons))
    return false;
  if (!mojo::Equals(this->share_target, other.share_target))
    return false;
  if (!mojo::Equals(this->related_applications, other.related_applications))
    return false;
  if (!mojo::Equals(this->prefer_related_applications, other.prefer_related_applications))
    return false;
  if (!mojo::Equals(this->theme_color, other.theme_color))
    return false;
  if (!mojo::Equals(this->background_color, other.background_color))
    return false;
  if (!mojo::Equals(this->gcm_sender_id, other.gcm_sender_id))
    return false;
  if (!mojo::Equals(this->scope, other.scope))
    return false;
  return true;
}
template <typename StructPtrType>
ManifestIconPtr ManifestIcon::Clone() const {
  return New(
      mojo::Clone(src),
      mojo::Clone(type),
      mojo::Clone(sizes),
      mojo::Clone(purpose)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ManifestIcon>::value>::type*>
bool ManifestIcon::Equals(const T& other) const {
  if (!mojo::Equals(this->src, other.src))
    return false;
  if (!mojo::Equals(this->type, other.type))
    return false;
  if (!mojo::Equals(this->sizes, other.sizes))
    return false;
  if (!mojo::Equals(this->purpose, other.purpose))
    return false;
  return true;
}
template <typename StructPtrType>
ManifestRelatedApplicationPtr ManifestRelatedApplication::Clone() const {
  return New(
      mojo::Clone(platform),
      mojo::Clone(url),
      mojo::Clone(id)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ManifestRelatedApplication>::value>::type*>
bool ManifestRelatedApplication::Equals(const T& other) const {
  if (!mojo::Equals(this->platform, other.platform))
    return false;
  if (!mojo::Equals(this->url, other.url))
    return false;
  if (!mojo::Equals(this->id, other.id))
    return false;
  return true;
}
template <typename StructPtrType>
ManifestShareTargetPtr ManifestShareTarget::Clone() const {
  return New(
      mojo::Clone(url_template)
  );
}

template <typename T,
          typename std::enable_if<std::is_same<
              T, ManifestShareTarget>::value>::type*>
bool ManifestShareTarget::Equals(const T& other) const {
  if (!mojo::Equals(this->url_template, other.url_template))
    return false;
  return true;
}


}  // namespace mojom
}  // namespace blink

namespace mojo {


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::Manifest::DataView,
                                         ::blink::mojom::ManifestPtr> {
  static bool IsNull(const ::blink::mojom::ManifestPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::ManifestPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::Manifest::name)& name(
      const ::blink::mojom::ManifestPtr& input) {
    return input->name;
  }

  static const decltype(::blink::mojom::Manifest::short_name)& short_name(
      const ::blink::mojom::ManifestPtr& input) {
    return input->short_name;
  }

  static const decltype(::blink::mojom::Manifest::start_url)& start_url(
      const ::blink::mojom::ManifestPtr& input) {
    return input->start_url;
  }

  static decltype(::blink::mojom::Manifest::display) display(
      const ::blink::mojom::ManifestPtr& input) {
    return input->display;
  }

  static decltype(::blink::mojom::Manifest::orientation) orientation(
      const ::blink::mojom::ManifestPtr& input) {
    return input->orientation;
  }

  static const decltype(::blink::mojom::Manifest::icons)& icons(
      const ::blink::mojom::ManifestPtr& input) {
    return input->icons;
  }

  static const decltype(::blink::mojom::Manifest::share_target)& share_target(
      const ::blink::mojom::ManifestPtr& input) {
    return input->share_target;
  }

  static const decltype(::blink::mojom::Manifest::related_applications)& related_applications(
      const ::blink::mojom::ManifestPtr& input) {
    return input->related_applications;
  }

  static decltype(::blink::mojom::Manifest::prefer_related_applications) prefer_related_applications(
      const ::blink::mojom::ManifestPtr& input) {
    return input->prefer_related_applications;
  }

  static decltype(::blink::mojom::Manifest::theme_color) theme_color(
      const ::blink::mojom::ManifestPtr& input) {
    return input->theme_color;
  }

  static decltype(::blink::mojom::Manifest::background_color) background_color(
      const ::blink::mojom::ManifestPtr& input) {
    return input->background_color;
  }

  static const decltype(::blink::mojom::Manifest::gcm_sender_id)& gcm_sender_id(
      const ::blink::mojom::ManifestPtr& input) {
    return input->gcm_sender_id;
  }

  static const decltype(::blink::mojom::Manifest::scope)& scope(
      const ::blink::mojom::ManifestPtr& input) {
    return input->scope;
  }

  static bool Read(::blink::mojom::Manifest::DataView input, ::blink::mojom::ManifestPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::ManifestIcon::DataView,
                                         ::blink::mojom::ManifestIconPtr> {
  static bool IsNull(const ::blink::mojom::ManifestIconPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::ManifestIconPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::ManifestIcon::src)& src(
      const ::blink::mojom::ManifestIconPtr& input) {
    return input->src;
  }

  static const decltype(::blink::mojom::ManifestIcon::type)& type(
      const ::blink::mojom::ManifestIconPtr& input) {
    return input->type;
  }

  static const decltype(::blink::mojom::ManifestIcon::sizes)& sizes(
      const ::blink::mojom::ManifestIconPtr& input) {
    return input->sizes;
  }

  static const decltype(::blink::mojom::ManifestIcon::purpose)& purpose(
      const ::blink::mojom::ManifestIconPtr& input) {
    return input->purpose;
  }

  static bool Read(::blink::mojom::ManifestIcon::DataView input, ::blink::mojom::ManifestIconPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::ManifestRelatedApplication::DataView,
                                         ::blink::mojom::ManifestRelatedApplicationPtr> {
  static bool IsNull(const ::blink::mojom::ManifestRelatedApplicationPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::ManifestRelatedApplicationPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::ManifestRelatedApplication::platform)& platform(
      const ::blink::mojom::ManifestRelatedApplicationPtr& input) {
    return input->platform;
  }

  static const decltype(::blink::mojom::ManifestRelatedApplication::url)& url(
      const ::blink::mojom::ManifestRelatedApplicationPtr& input) {
    return input->url;
  }

  static const decltype(::blink::mojom::ManifestRelatedApplication::id)& id(
      const ::blink::mojom::ManifestRelatedApplicationPtr& input) {
    return input->id;
  }

  static bool Read(::blink::mojom::ManifestRelatedApplication::DataView input, ::blink::mojom::ManifestRelatedApplicationPtr* output);
};


template <>
struct CONTENT_EXPORT StructTraits<::blink::mojom::ManifestShareTarget::DataView,
                                         ::blink::mojom::ManifestShareTargetPtr> {
  static bool IsNull(const ::blink::mojom::ManifestShareTargetPtr& input) { return !input; }
  static void SetToNull(::blink::mojom::ManifestShareTargetPtr* output) { output->reset(); }

  static const decltype(::blink::mojom::ManifestShareTarget::url_template)& url_template(
      const ::blink::mojom::ManifestShareTargetPtr& input) {
    return input->url_template;
  }

  static bool Read(::blink::mojom::ManifestShareTarget::DataView input, ::blink::mojom::ManifestShareTargetPtr* output);
};

}  // namespace mojo

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_MANIFEST_MANIFEST_MOJOM_H_