// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "third_party/WebKit/public/platform/modules/manifest/manifest_manager.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "third_party/WebKit/Source/platform/mojo/CommonCustomTypesStructTraits.h"
#include "third_party/WebKit/Source/platform/mojo/GeometryStructTraits.h"
#include "third_party/WebKit/Source/platform/mojo/KURLStructTraits.h"
#include "third_party/WebKit/public/platform/WebDisplayModeStructTraits.h"
#include "third_party/WebKit/public/platform/modules/screen_orientation/WebScreenOrientationEnumTraits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
namespace blink {
namespace mojom {
namespace blink {
const char ManifestManager::Name_[] = "blink::mojom::ManifestManager";

class ManifestManager_RequestManifest_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  ManifestManager_RequestManifest_ForwardToCallback(
      ManifestManager::RequestManifestCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  ManifestManager::RequestManifestCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(ManifestManager_RequestManifest_ForwardToCallback);
};

ManifestManagerProxy::ManifestManagerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void ManifestManagerProxy::RequestManifest(
    RequestManifestCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::ManifestManager_RequestManifest_Params_Data);
  serialization_context.PrepareMessage(
      internal::kManifestManager_RequestManifest_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::ManifestManager_RequestManifest_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new ManifestManager_RequestManifest_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class ManifestManager_RequestManifest_ProxyToResponder {
 public:
  static ManifestManager::RequestManifestCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<ManifestManager_RequestManifest_ProxyToResponder> proxy(
        new ManifestManager_RequestManifest_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&ManifestManager_RequestManifest_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~ManifestManager_RequestManifest_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "ManifestManager::RequestManifest() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  ManifestManager_RequestManifest_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const ::blink::KURL& in_url, ::blink::mojom::blink::ManifestPtr in_manifest);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(ManifestManager_RequestManifest_ProxyToResponder);
};

bool ManifestManager_RequestManifest_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::ManifestManager_RequestManifest_ResponseParams_Data* params =
      reinterpret_cast<
          internal::ManifestManager_RequestManifest_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  ::blink::KURL p_url{};
  ::blink::mojom::blink::ManifestPtr p_manifest{};
  ManifestManager_RequestManifest_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadUrl(&p_url))
    success = false;
  if (!input_data_view.ReadManifest(&p_manifest))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "ManifestManager::RequestManifest response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_url), 
std::move(p_manifest));
  return true;
}

void ManifestManager_RequestManifest_ProxyToResponder::Run(
    const ::blink::KURL& in_url, ::blink::mojom::blink::ManifestPtr in_manifest) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::ManifestManager_RequestManifest_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_url, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::ManifestDataView>(
      in_manifest, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kManifestManager_RequestManifest_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::ManifestManager_RequestManifest_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->url)::BaseType* url_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_url, buffer, &url_ptr, &serialization_context);
  params->url.Set(url_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in ");
  typename decltype(params->manifest)::BaseType* manifest_ptr;
  mojo::internal::Serialize<::blink::mojom::ManifestDataView>(
      in_manifest, buffer, &manifest_ptr, &serialization_context);
  params->manifest.Set(manifest_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool ManifestManagerStubDispatch::Accept(
    ManifestManager* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kManifestManager_RequestManifest_Name: {
      break;
    }
  }
  return false;
}

// static
bool ManifestManagerStubDispatch::AcceptWithResponder(
    ManifestManager* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kManifestManager_RequestManifest_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::ManifestManager_RequestManifest_Params_Data* params =
          reinterpret_cast<
              internal::ManifestManager_RequestManifest_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ManifestManager_RequestManifest_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "ManifestManager::RequestManifest deserializer");
        return false;
      }
      ManifestManager::RequestManifestCallback callback =
          ManifestManager_RequestManifest_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestManifest(std::move(callback));
      return true;
    }
  }
  return false;
}

bool ManifestManagerRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ManifestManager RequestValidator");

  switch (message->header()->name) {
    case internal::kManifestManager_RequestManifest_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::ManifestManager_RequestManifest_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool ManifestManagerResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "ManifestManager ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kManifestManager_RequestManifest_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::ManifestManager_RequestManifest_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
}  // namespace blink
}  // namespace mojom
}  // namespace blink

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif