// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "third_party/WebKit/public/platform/modules/payments/payment_app.mojom-blink.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "mojo/public/cpp/bindings/lib/wtf_serialization.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/common/common_custom_types_struct_traits.h"
#include "third_party/WebKit/Source/platform/mojo/KURLStructTraits.h"
namespace payments {
namespace mojom {
namespace blink {
ImageObject::ImageObject()
    : src() {}

ImageObject::ImageObject(
    const ::blink::KURL& src_in)
    : src(std::move(src_in)) {}

ImageObject::~ImageObject() = default;

bool ImageObject::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PaymentInstrument::PaymentInstrument()
    : name(),
      icons(),
      enabled_methods(),
      stringified_capabilities() {}

PaymentInstrument::PaymentInstrument(
    const WTF::String& name_in,
    WTF::Vector<ImageObjectPtr> icons_in,
    const WTF::Vector<WTF::String>& enabled_methods_in,
    const WTF::String& stringified_capabilities_in)
    : name(std::move(name_in)),
      icons(std::move(icons_in)),
      enabled_methods(std::move(enabled_methods_in)),
      stringified_capabilities(std::move(stringified_capabilities_in)) {}

PaymentInstrument::~PaymentInstrument() = default;

bool PaymentInstrument::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PaymentRequestEventData::PaymentRequestEventData()
    : top_level_origin(),
      payment_request_origin(),
      payment_request_id(),
      method_data(),
      total(),
      modifiers(),
      instrument_key() {}

PaymentRequestEventData::PaymentRequestEventData(
    const ::blink::KURL& top_level_origin_in,
    const ::blink::KURL& payment_request_origin_in,
    const WTF::String& payment_request_id_in,
    WTF::Vector<::payments::mojom::blink::PaymentMethodDataPtr> method_data_in,
    ::payments::mojom::blink::PaymentCurrencyAmountPtr total_in,
    WTF::Vector<::payments::mojom::blink::PaymentDetailsModifierPtr> modifiers_in,
    const WTF::String& instrument_key_in)
    : top_level_origin(std::move(top_level_origin_in)),
      payment_request_origin(std::move(payment_request_origin_in)),
      payment_request_id(std::move(payment_request_id_in)),
      method_data(std::move(method_data_in)),
      total(std::move(total_in)),
      modifiers(std::move(modifiers_in)),
      instrument_key(std::move(instrument_key_in)) {}

PaymentRequestEventData::~PaymentRequestEventData() = default;

bool PaymentRequestEventData::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PaymentHandlerResponse::PaymentHandlerResponse()
    : method_name(),
      stringified_details() {}

PaymentHandlerResponse::PaymentHandlerResponse(
    const WTF::String& method_name_in,
    const WTF::String& stringified_details_in)
    : method_name(std::move(method_name_in)),
      stringified_details(std::move(stringified_details_in)) {}

PaymentHandlerResponse::~PaymentHandlerResponse() = default;
size_t PaymentHandlerResponse::Hash(size_t seed) const {
  seed = mojo::internal::WTFHash(seed, this->method_name);
  seed = mojo::internal::WTFHash(seed, this->stringified_details);
  return seed;
}

bool PaymentHandlerResponse::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
const char PaymentManager::Name_[] = "payments::mojom::PaymentManager";

class PaymentManager_DeletePaymentInstrument_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PaymentManager_DeletePaymentInstrument_ForwardToCallback(
      PaymentManager::DeletePaymentInstrumentCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PaymentManager::DeletePaymentInstrumentCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PaymentManager_DeletePaymentInstrument_ForwardToCallback);
};

class PaymentManager_GetPaymentInstrument_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PaymentManager_GetPaymentInstrument_ForwardToCallback(
      PaymentManager::GetPaymentInstrumentCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PaymentManager::GetPaymentInstrumentCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PaymentManager_GetPaymentInstrument_ForwardToCallback);
};

class PaymentManager_KeysOfPaymentInstruments_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PaymentManager_KeysOfPaymentInstruments_ForwardToCallback(
      PaymentManager::KeysOfPaymentInstrumentsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PaymentManager::KeysOfPaymentInstrumentsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PaymentManager_KeysOfPaymentInstruments_ForwardToCallback);
};

class PaymentManager_HasPaymentInstrument_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PaymentManager_HasPaymentInstrument_ForwardToCallback(
      PaymentManager::HasPaymentInstrumentCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PaymentManager::HasPaymentInstrumentCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PaymentManager_HasPaymentInstrument_ForwardToCallback);
};

class PaymentManager_SetPaymentInstrument_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PaymentManager_SetPaymentInstrument_ForwardToCallback(
      PaymentManager::SetPaymentInstrumentCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PaymentManager::SetPaymentInstrumentCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PaymentManager_SetPaymentInstrument_ForwardToCallback);
};

class PaymentManager_ClearPaymentInstruments_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PaymentManager_ClearPaymentInstruments_ForwardToCallback(
      PaymentManager::ClearPaymentInstrumentsCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PaymentManager::ClearPaymentInstrumentsCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PaymentManager_ClearPaymentInstruments_ForwardToCallback);
};

PaymentManagerProxy::PaymentManagerProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PaymentManagerProxy::Init(
    const WTF::String& in_service_worker_script_url, const WTF::String& in_service_worker_scope) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_Init_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_service_worker_script_url, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_service_worker_scope, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_Init_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_Init_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->service_worker_script_url)::BaseType* service_worker_script_url_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_service_worker_script_url, buffer, &service_worker_script_url_ptr, &serialization_context);
  params->service_worker_script_url.Set(service_worker_script_url_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->service_worker_script_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null service_worker_script_url in PaymentManager.Init request");
  typename decltype(params->service_worker_scope)::BaseType* service_worker_scope_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_service_worker_scope, buffer, &service_worker_scope_ptr, &serialization_context);
  params->service_worker_scope.Set(service_worker_scope_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->service_worker_scope.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null service_worker_scope in PaymentManager.Init request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PaymentManagerProxy::DeletePaymentInstrument(
    const WTF::String& in_instrument_key, DeletePaymentInstrumentCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_DeletePaymentInstrument_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_instrument_key, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_DeletePaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_DeletePaymentInstrument_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->instrument_key)::BaseType* instrument_key_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_instrument_key, buffer, &instrument_key_ptr, &serialization_context);
  params->instrument_key.Set(instrument_key_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->instrument_key.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null instrument_key in PaymentManager.DeletePaymentInstrument request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PaymentManager_DeletePaymentInstrument_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PaymentManagerProxy::GetPaymentInstrument(
    const WTF::String& in_instrument_key, GetPaymentInstrumentCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_GetPaymentInstrument_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_instrument_key, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_GetPaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_GetPaymentInstrument_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->instrument_key)::BaseType* instrument_key_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_instrument_key, buffer, &instrument_key_ptr, &serialization_context);
  params->instrument_key.Set(instrument_key_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->instrument_key.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null instrument_key in PaymentManager.GetPaymentInstrument request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PaymentManager_GetPaymentInstrument_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PaymentManagerProxy::KeysOfPaymentInstruments(
    KeysOfPaymentInstrumentsCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_KeysOfPaymentInstruments_Params_Data);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_KeysOfPaymentInstruments_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_KeysOfPaymentInstruments_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PaymentManager_KeysOfPaymentInstruments_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PaymentManagerProxy::HasPaymentInstrument(
    const WTF::String& in_instrument_key, HasPaymentInstrumentCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_HasPaymentInstrument_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_instrument_key, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_HasPaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_HasPaymentInstrument_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->instrument_key)::BaseType* instrument_key_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_instrument_key, buffer, &instrument_key_ptr, &serialization_context);
  params->instrument_key.Set(instrument_key_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->instrument_key.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null instrument_key in PaymentManager.HasPaymentInstrument request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PaymentManager_HasPaymentInstrument_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PaymentManagerProxy::SetPaymentInstrument(
    const WTF::String& in_instrument_key, PaymentInstrumentPtr in_instrument, SetPaymentInstrumentCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_SetPaymentInstrument_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_instrument_key, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::payments::mojom::PaymentInstrumentDataView>(
      in_instrument, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_SetPaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_SetPaymentInstrument_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->instrument_key)::BaseType* instrument_key_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_instrument_key, buffer, &instrument_key_ptr, &serialization_context);
  params->instrument_key.Set(instrument_key_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->instrument_key.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null instrument_key in PaymentManager.SetPaymentInstrument request");
  typename decltype(params->instrument)::BaseType* instrument_ptr;
  mojo::internal::Serialize<::payments::mojom::PaymentInstrumentDataView>(
      in_instrument, buffer, &instrument_ptr, &serialization_context);
  params->instrument.Set(instrument_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->instrument.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null instrument in PaymentManager.SetPaymentInstrument request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PaymentManager_SetPaymentInstrument_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PaymentManagerProxy::ClearPaymentInstruments(
    ClearPaymentInstrumentsCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_ClearPaymentInstruments_Params_Data);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_ClearPaymentInstruments_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_ClearPaymentInstruments_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PaymentManager_ClearPaymentInstruments_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}
class PaymentManager_DeletePaymentInstrument_ProxyToResponder {
 public:
  static PaymentManager::DeletePaymentInstrumentCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PaymentManager_DeletePaymentInstrument_ProxyToResponder> proxy(
        new PaymentManager_DeletePaymentInstrument_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PaymentManager_DeletePaymentInstrument_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PaymentManager_DeletePaymentInstrument_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PaymentManager::DeletePaymentInstrument() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PaymentManager_DeletePaymentInstrument_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      PaymentHandlerStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PaymentManager_DeletePaymentInstrument_ProxyToResponder);
};

bool PaymentManager_DeletePaymentInstrument_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PaymentManager_DeletePaymentInstrument_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PaymentManager_DeletePaymentInstrument_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  PaymentHandlerStatus p_status{};
  PaymentManager_DeletePaymentInstrument_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PaymentManager::DeletePaymentInstrument response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_status));
  return true;
}

void PaymentManager_DeletePaymentInstrument_ProxyToResponder::Run(
    PaymentHandlerStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_DeletePaymentInstrument_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_DeletePaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_DeletePaymentInstrument_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::payments::mojom::PaymentHandlerStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PaymentManager_GetPaymentInstrument_ProxyToResponder {
 public:
  static PaymentManager::GetPaymentInstrumentCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PaymentManager_GetPaymentInstrument_ProxyToResponder> proxy(
        new PaymentManager_GetPaymentInstrument_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PaymentManager_GetPaymentInstrument_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PaymentManager_GetPaymentInstrument_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PaymentManager::GetPaymentInstrument() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PaymentManager_GetPaymentInstrument_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      PaymentInstrumentPtr in_instrument, PaymentHandlerStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PaymentManager_GetPaymentInstrument_ProxyToResponder);
};

bool PaymentManager_GetPaymentInstrument_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PaymentManager_GetPaymentInstrument_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PaymentManager_GetPaymentInstrument_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  PaymentInstrumentPtr p_instrument{};
  PaymentHandlerStatus p_status{};
  PaymentManager_GetPaymentInstrument_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadInstrument(&p_instrument))
    success = false;
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PaymentManager::GetPaymentInstrument response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_instrument), 
std::move(p_status));
  return true;
}

void PaymentManager_GetPaymentInstrument_ProxyToResponder::Run(
    PaymentInstrumentPtr in_instrument, PaymentHandlerStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_GetPaymentInstrument_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::payments::mojom::PaymentInstrumentDataView>(
      in_instrument, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_GetPaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_GetPaymentInstrument_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->instrument)::BaseType* instrument_ptr;
  mojo::internal::Serialize<::payments::mojom::PaymentInstrumentDataView>(
      in_instrument, buffer, &instrument_ptr, &serialization_context);
  params->instrument.Set(instrument_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->instrument.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null instrument in ");
  mojo::internal::Serialize<::payments::mojom::PaymentHandlerStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PaymentManager_KeysOfPaymentInstruments_ProxyToResponder {
 public:
  static PaymentManager::KeysOfPaymentInstrumentsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PaymentManager_KeysOfPaymentInstruments_ProxyToResponder> proxy(
        new PaymentManager_KeysOfPaymentInstruments_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PaymentManager_KeysOfPaymentInstruments_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PaymentManager_KeysOfPaymentInstruments_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PaymentManager::KeysOfPaymentInstruments() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PaymentManager_KeysOfPaymentInstruments_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const WTF::Vector<WTF::String>& in_keys, PaymentHandlerStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PaymentManager_KeysOfPaymentInstruments_ProxyToResponder);
};

bool PaymentManager_KeysOfPaymentInstruments_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  WTF::Vector<WTF::String> p_keys{};
  PaymentHandlerStatus p_status{};
  PaymentManager_KeysOfPaymentInstruments_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadKeys(&p_keys))
    success = false;
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PaymentManager::KeysOfPaymentInstruments response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_keys), 
std::move(p_status));
  return true;
}

void PaymentManager_KeysOfPaymentInstruments_ProxyToResponder::Run(
    const WTF::Vector<WTF::String>& in_keys, PaymentHandlerStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<mojo::StringDataView>>(
      in_keys, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_KeysOfPaymentInstruments_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->keys)::BaseType* keys_ptr;
  const mojo::internal::ContainerValidateParams keys_validate_params(
      0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
  mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
      in_keys, buffer, &keys_ptr, &keys_validate_params,
      &serialization_context);
  params->keys.Set(keys_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->keys.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null keys in ");
  mojo::internal::Serialize<::payments::mojom::PaymentHandlerStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PaymentManager_HasPaymentInstrument_ProxyToResponder {
 public:
  static PaymentManager::HasPaymentInstrumentCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PaymentManager_HasPaymentInstrument_ProxyToResponder> proxy(
        new PaymentManager_HasPaymentInstrument_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PaymentManager_HasPaymentInstrument_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PaymentManager_HasPaymentInstrument_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PaymentManager::HasPaymentInstrument() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PaymentManager_HasPaymentInstrument_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      PaymentHandlerStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PaymentManager_HasPaymentInstrument_ProxyToResponder);
};

bool PaymentManager_HasPaymentInstrument_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PaymentManager_HasPaymentInstrument_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PaymentManager_HasPaymentInstrument_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  PaymentHandlerStatus p_status{};
  PaymentManager_HasPaymentInstrument_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PaymentManager::HasPaymentInstrument response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_status));
  return true;
}

void PaymentManager_HasPaymentInstrument_ProxyToResponder::Run(
    PaymentHandlerStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_HasPaymentInstrument_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_HasPaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_HasPaymentInstrument_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::payments::mojom::PaymentHandlerStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PaymentManager_SetPaymentInstrument_ProxyToResponder {
 public:
  static PaymentManager::SetPaymentInstrumentCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PaymentManager_SetPaymentInstrument_ProxyToResponder> proxy(
        new PaymentManager_SetPaymentInstrument_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PaymentManager_SetPaymentInstrument_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PaymentManager_SetPaymentInstrument_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PaymentManager::SetPaymentInstrument() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PaymentManager_SetPaymentInstrument_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      PaymentHandlerStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PaymentManager_SetPaymentInstrument_ProxyToResponder);
};

bool PaymentManager_SetPaymentInstrument_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PaymentManager_SetPaymentInstrument_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PaymentManager_SetPaymentInstrument_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  PaymentHandlerStatus p_status{};
  PaymentManager_SetPaymentInstrument_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PaymentManager::SetPaymentInstrument response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_status));
  return true;
}

void PaymentManager_SetPaymentInstrument_ProxyToResponder::Run(
    PaymentHandlerStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_SetPaymentInstrument_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_SetPaymentInstrument_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_SetPaymentInstrument_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::payments::mojom::PaymentHandlerStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PaymentManager_ClearPaymentInstruments_ProxyToResponder {
 public:
  static PaymentManager::ClearPaymentInstrumentsCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PaymentManager_ClearPaymentInstruments_ProxyToResponder> proxy(
        new PaymentManager_ClearPaymentInstruments_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PaymentManager_ClearPaymentInstruments_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PaymentManager_ClearPaymentInstruments_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PaymentManager::ClearPaymentInstruments() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PaymentManager_ClearPaymentInstruments_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      PaymentHandlerStatus in_status);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PaymentManager_ClearPaymentInstruments_ProxyToResponder);
};

bool PaymentManager_ClearPaymentInstruments_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PaymentManager_ClearPaymentInstruments_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PaymentManager_ClearPaymentInstruments_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  PaymentHandlerStatus p_status{};
  PaymentManager_ClearPaymentInstruments_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadStatus(&p_status))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PaymentManager::ClearPaymentInstruments response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_status));
  return true;
}

void PaymentManager_ClearPaymentInstruments_ProxyToResponder::Run(
    PaymentHandlerStatus in_status) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentManager_ClearPaymentInstruments_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kPaymentManager_ClearPaymentInstruments_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentManager_ClearPaymentInstruments_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::payments::mojom::PaymentHandlerStatus>(
      in_status, &params->status);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool PaymentManagerStubDispatch::Accept(
    PaymentManager* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPaymentManager_Init_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PaymentManager_Init_Params_Data* params =
          reinterpret_cast<internal::PaymentManager_Init_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WTF::String p_service_worker_script_url{};
      WTF::String p_service_worker_scope{};
      PaymentManager_Init_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadServiceWorkerScriptUrl(&p_service_worker_script_url))
        success = false;
      if (!input_data_view.ReadServiceWorkerScope(&p_service_worker_scope))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentManager::Init deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Init(
std::move(p_service_worker_script_url), 
std::move(p_service_worker_scope));
      return true;
    }
    case internal::kPaymentManager_DeletePaymentInstrument_Name: {
      break;
    }
    case internal::kPaymentManager_GetPaymentInstrument_Name: {
      break;
    }
    case internal::kPaymentManager_KeysOfPaymentInstruments_Name: {
      break;
    }
    case internal::kPaymentManager_HasPaymentInstrument_Name: {
      break;
    }
    case internal::kPaymentManager_SetPaymentInstrument_Name: {
      break;
    }
    case internal::kPaymentManager_ClearPaymentInstruments_Name: {
      break;
    }
  }
  return false;
}

// static
bool PaymentManagerStubDispatch::AcceptWithResponder(
    PaymentManager* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPaymentManager_Init_Name: {
      break;
    }
    case internal::kPaymentManager_DeletePaymentInstrument_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PaymentManager_DeletePaymentInstrument_Params_Data* params =
          reinterpret_cast<
              internal::PaymentManager_DeletePaymentInstrument_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WTF::String p_instrument_key{};
      PaymentManager_DeletePaymentInstrument_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadInstrumentKey(&p_instrument_key))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentManager::DeletePaymentInstrument deserializer");
        return false;
      }
      PaymentManager::DeletePaymentInstrumentCallback callback =
          PaymentManager_DeletePaymentInstrument_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DeletePaymentInstrument(
std::move(p_instrument_key), std::move(callback));
      return true;
    }
    case internal::kPaymentManager_GetPaymentInstrument_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PaymentManager_GetPaymentInstrument_Params_Data* params =
          reinterpret_cast<
              internal::PaymentManager_GetPaymentInstrument_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WTF::String p_instrument_key{};
      PaymentManager_GetPaymentInstrument_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadInstrumentKey(&p_instrument_key))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentManager::GetPaymentInstrument deserializer");
        return false;
      }
      PaymentManager::GetPaymentInstrumentCallback callback =
          PaymentManager_GetPaymentInstrument_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetPaymentInstrument(
std::move(p_instrument_key), std::move(callback));
      return true;
    }
    case internal::kPaymentManager_KeysOfPaymentInstruments_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PaymentManager_KeysOfPaymentInstruments_Params_Data* params =
          reinterpret_cast<
              internal::PaymentManager_KeysOfPaymentInstruments_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PaymentManager_KeysOfPaymentInstruments_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentManager::KeysOfPaymentInstruments deserializer");
        return false;
      }
      PaymentManager::KeysOfPaymentInstrumentsCallback callback =
          PaymentManager_KeysOfPaymentInstruments_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->KeysOfPaymentInstruments(std::move(callback));
      return true;
    }
    case internal::kPaymentManager_HasPaymentInstrument_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PaymentManager_HasPaymentInstrument_Params_Data* params =
          reinterpret_cast<
              internal::PaymentManager_HasPaymentInstrument_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WTF::String p_instrument_key{};
      PaymentManager_HasPaymentInstrument_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadInstrumentKey(&p_instrument_key))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentManager::HasPaymentInstrument deserializer");
        return false;
      }
      PaymentManager::HasPaymentInstrumentCallback callback =
          PaymentManager_HasPaymentInstrument_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->HasPaymentInstrument(
std::move(p_instrument_key), std::move(callback));
      return true;
    }
    case internal::kPaymentManager_SetPaymentInstrument_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PaymentManager_SetPaymentInstrument_Params_Data* params =
          reinterpret_cast<
              internal::PaymentManager_SetPaymentInstrument_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WTF::String p_instrument_key{};
      PaymentInstrumentPtr p_instrument{};
      PaymentManager_SetPaymentInstrument_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadInstrumentKey(&p_instrument_key))
        success = false;
      if (!input_data_view.ReadInstrument(&p_instrument))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentManager::SetPaymentInstrument deserializer");
        return false;
      }
      PaymentManager::SetPaymentInstrumentCallback callback =
          PaymentManager_SetPaymentInstrument_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetPaymentInstrument(
std::move(p_instrument_key), 
std::move(p_instrument), std::move(callback));
      return true;
    }
    case internal::kPaymentManager_ClearPaymentInstruments_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PaymentManager_ClearPaymentInstruments_Params_Data* params =
          reinterpret_cast<
              internal::PaymentManager_ClearPaymentInstruments_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PaymentManager_ClearPaymentInstruments_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentManager::ClearPaymentInstruments deserializer");
        return false;
      }
      PaymentManager::ClearPaymentInstrumentsCallback callback =
          PaymentManager_ClearPaymentInstruments_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ClearPaymentInstruments(std::move(callback));
      return true;
    }
  }
  return false;
}

bool PaymentManagerRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PaymentManager RequestValidator");

  switch (message->header()->name) {
    case internal::kPaymentManager_Init_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_Init_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_DeletePaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_DeletePaymentInstrument_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_GetPaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_GetPaymentInstrument_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_KeysOfPaymentInstruments_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_KeysOfPaymentInstruments_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_HasPaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_HasPaymentInstrument_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_SetPaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_SetPaymentInstrument_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_ClearPaymentInstruments_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_ClearPaymentInstruments_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool PaymentManagerResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PaymentManager ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kPaymentManager_DeletePaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_DeletePaymentInstrument_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_GetPaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_GetPaymentInstrument_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_KeysOfPaymentInstruments_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_HasPaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_HasPaymentInstrument_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_SetPaymentInstrument_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_SetPaymentInstrument_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPaymentManager_ClearPaymentInstruments_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentManager_ClearPaymentInstruments_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char PaymentHandlerResponseCallback::Name_[] = "payments::mojom::PaymentHandlerResponseCallback";

PaymentHandlerResponseCallbackProxy::PaymentHandlerResponseCallbackProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PaymentHandlerResponseCallbackProxy::OnPaymentHandlerResponse(
    PaymentHandlerResponsePtr in_response, WTF::Time in_dispatch_event_time) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::payments::mojom::internal::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data);
  size += mojo::internal::PrepareToSerialize<::payments::mojom::PaymentHandlerResponseDataView>(
      in_response, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDataView>(
      in_dispatch_event_time, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPaymentHandlerResponseCallback_OnPaymentHandlerResponse_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::payments::mojom::internal::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->response)::BaseType* response_ptr;
  mojo::internal::Serialize<::payments::mojom::PaymentHandlerResponseDataView>(
      in_response, buffer, &response_ptr, &serialization_context);
  params->response.Set(response_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->response.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null response in PaymentHandlerResponseCallback.OnPaymentHandlerResponse request");
  typename decltype(params->dispatch_event_time)::BaseType* dispatch_event_time_ptr;
  mojo::internal::Serialize<::mojo::common::mojom::TimeDataView>(
      in_dispatch_event_time, buffer, &dispatch_event_time_ptr, &serialization_context);
  params->dispatch_event_time.Set(dispatch_event_time_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->dispatch_event_time.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null dispatch_event_time in PaymentHandlerResponseCallback.OnPaymentHandlerResponse request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool PaymentHandlerResponseCallbackStubDispatch::Accept(
    PaymentHandlerResponseCallback* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPaymentHandlerResponseCallback_OnPaymentHandlerResponse_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data* params =
          reinterpret_cast<internal::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PaymentHandlerResponsePtr p_response{};
      WTF::Time p_dispatch_event_time{};
      PaymentHandlerResponseCallback_OnPaymentHandlerResponse_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadResponse(&p_response))
        success = false;
      if (!input_data_view.ReadDispatchEventTime(&p_dispatch_event_time))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PaymentHandlerResponseCallback::OnPaymentHandlerResponse deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnPaymentHandlerResponse(
std::move(p_response), 
std::move(p_dispatch_event_time));
      return true;
    }
  }
  return false;
}

// static
bool PaymentHandlerResponseCallbackStubDispatch::AcceptWithResponder(
    PaymentHandlerResponseCallback* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPaymentHandlerResponseCallback_OnPaymentHandlerResponse_Name: {
      break;
    }
  }
  return false;
}

bool PaymentHandlerResponseCallbackRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PaymentHandlerResponseCallback RequestValidator");

  switch (message->header()->name) {
    case internal::kPaymentHandlerResponseCallback_OnPaymentHandlerResponse_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace blink
}  // namespace mojom
}  // namespace payments

namespace mojo {


// static
bool StructTraits<::payments::mojom::blink::ImageObject::DataView, ::payments::mojom::blink::ImageObjectPtr>::Read(
    ::payments::mojom::blink::ImageObject::DataView input,
    ::payments::mojom::blink::ImageObjectPtr* output) {
  bool success = true;
  ::payments::mojom::blink::ImageObjectPtr result(::payments::mojom::blink::ImageObject::New());
  
      if (!input.ReadSrc(&result->src))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::payments::mojom::blink::PaymentInstrument::DataView, ::payments::mojom::blink::PaymentInstrumentPtr>::Read(
    ::payments::mojom::blink::PaymentInstrument::DataView input,
    ::payments::mojom::blink::PaymentInstrumentPtr* output) {
  bool success = true;
  ::payments::mojom::blink::PaymentInstrumentPtr result(::payments::mojom::blink::PaymentInstrument::New());
  
      if (!input.ReadName(&result->name))
        success = false;
      if (!input.ReadIcons(&result->icons))
        success = false;
      if (!input.ReadEnabledMethods(&result->enabled_methods))
        success = false;
      if (!input.ReadStringifiedCapabilities(&result->stringified_capabilities))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::payments::mojom::blink::PaymentRequestEventData::DataView, ::payments::mojom::blink::PaymentRequestEventDataPtr>::Read(
    ::payments::mojom::blink::PaymentRequestEventData::DataView input,
    ::payments::mojom::blink::PaymentRequestEventDataPtr* output) {
  bool success = true;
  ::payments::mojom::blink::PaymentRequestEventDataPtr result(::payments::mojom::blink::PaymentRequestEventData::New());
  
      if (!input.ReadTopLevelOrigin(&result->top_level_origin))
        success = false;
      if (!input.ReadPaymentRequestOrigin(&result->payment_request_origin))
        success = false;
      if (!input.ReadPaymentRequestId(&result->payment_request_id))
        success = false;
      if (!input.ReadMethodData(&result->method_data))
        success = false;
      if (!input.ReadTotal(&result->total))
        success = false;
      if (!input.ReadModifiers(&result->modifiers))
        success = false;
      if (!input.ReadInstrumentKey(&result->instrument_key))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::payments::mojom::blink::PaymentHandlerResponse::DataView, ::payments::mojom::blink::PaymentHandlerResponsePtr>::Read(
    ::payments::mojom::blink::PaymentHandlerResponse::DataView input,
    ::payments::mojom::blink::PaymentHandlerResponsePtr* output) {
  bool success = true;
  ::payments::mojom::blink::PaymentHandlerResponsePtr result(::payments::mojom::blink::PaymentHandlerResponse::New());
  
      if (!input.ReadMethodName(&result->method_name))
        success = false;
      if (!input.ReadStringifiedDetails(&result->stringified_details))
        success = false;
  *output = std::move(result);
  return success;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif