// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PAYMENTS_PAYMENT_APP_MOJOM_SHARED_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PAYMENTS_PAYMENT_APP_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/WebKit/public/platform/modules/payments/payment_app.mojom-shared-internal.h"
#include "mojo/common/time.mojom-shared.h"
#include "third_party/WebKit/public/platform/modules/payments/payment_request.mojom-shared.h"
#include "url/mojo/url.mojom-shared.h"
#include "third_party/WebKit/public/mojo_bindings_shared_export.h"


namespace payments {
namespace mojom {
class ImageObjectDataView;

class PaymentInstrumentDataView;

class PaymentRequestEventDataDataView;

class PaymentHandlerResponseDataView;



}  // namespace mojom
}  // namespace payments

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::payments::mojom::ImageObjectDataView> {
  using Data = ::payments::mojom::internal::ImageObject_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::payments::mojom::PaymentInstrumentDataView> {
  using Data = ::payments::mojom::internal::PaymentInstrument_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::payments::mojom::PaymentRequestEventDataDataView> {
  using Data = ::payments::mojom::internal::PaymentRequestEventData_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::payments::mojom::PaymentHandlerResponseDataView> {
  using Data = ::payments::mojom::internal::PaymentHandlerResponse_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace payments {
namespace mojom {

enum class PaymentHandlerStatus : int32_t {
  SUCCESS,
  NOT_IMPLEMENTED,
  NOT_FOUND,
  NO_ACTIVE_WORKER,
  STORAGE_OPERATION_FAILED,
  FETCH_INSTRUMENT_ICON_FAILED,
  FETCH_PAYMENT_APP_INFO_FAILED,
};

inline std::ostream& operator<<(std::ostream& os, PaymentHandlerStatus value) {
  switch(value) {
    case PaymentHandlerStatus::SUCCESS:
      return os << "PaymentHandlerStatus::SUCCESS";
    case PaymentHandlerStatus::NOT_IMPLEMENTED:
      return os << "PaymentHandlerStatus::NOT_IMPLEMENTED";
    case PaymentHandlerStatus::NOT_FOUND:
      return os << "PaymentHandlerStatus::NOT_FOUND";
    case PaymentHandlerStatus::NO_ACTIVE_WORKER:
      return os << "PaymentHandlerStatus::NO_ACTIVE_WORKER";
    case PaymentHandlerStatus::STORAGE_OPERATION_FAILED:
      return os << "PaymentHandlerStatus::STORAGE_OPERATION_FAILED";
    case PaymentHandlerStatus::FETCH_INSTRUMENT_ICON_FAILED:
      return os << "PaymentHandlerStatus::FETCH_INSTRUMENT_ICON_FAILED";
    case PaymentHandlerStatus::FETCH_PAYMENT_APP_INFO_FAILED:
      return os << "PaymentHandlerStatus::FETCH_PAYMENT_APP_INFO_FAILED";
    default:
      return os << "Unknown PaymentHandlerStatus value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(PaymentHandlerStatus value) {
  return internal::PaymentHandlerStatus_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class PaymentManagerInterfaceBase {};

using PaymentManagerPtrDataView =
    mojo::InterfacePtrDataView<PaymentManagerInterfaceBase>;
using PaymentManagerRequestDataView =
    mojo::InterfaceRequestDataView<PaymentManagerInterfaceBase>;
using PaymentManagerAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PaymentManagerInterfaceBase>;
using PaymentManagerAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PaymentManagerInterfaceBase>;
class PaymentHandlerResponseCallbackInterfaceBase {};

using PaymentHandlerResponseCallbackPtrDataView =
    mojo::InterfacePtrDataView<PaymentHandlerResponseCallbackInterfaceBase>;
using PaymentHandlerResponseCallbackRequestDataView =
    mojo::InterfaceRequestDataView<PaymentHandlerResponseCallbackInterfaceBase>;
using PaymentHandlerResponseCallbackAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<PaymentHandlerResponseCallbackInterfaceBase>;
using PaymentHandlerResponseCallbackAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<PaymentHandlerResponseCallbackInterfaceBase>;
class ImageObjectDataView {
 public:
  ImageObjectDataView() {}

  ImageObjectDataView(
      internal::ImageObject_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetSrcDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSrc(UserType* output) {
    auto* pointer = data_->src.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
 private:
  internal::ImageObject_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentInstrumentDataView {
 public:
  PaymentInstrumentDataView() {}

  PaymentInstrumentDataView(
      internal::PaymentInstrument_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIconsDataView(
      mojo::ArrayDataView<ImageObjectDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIcons(UserType* output) {
    auto* pointer = data_->icons.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::payments::mojom::ImageObjectDataView>>(
        pointer, output, context_);
  }
  inline void GetEnabledMethodsDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadEnabledMethods(UserType* output) {
    auto* pointer = data_->enabled_methods.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  inline void GetStringifiedCapabilitiesDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStringifiedCapabilities(UserType* output) {
    auto* pointer = data_->stringified_capabilities.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentInstrument_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentRequestEventDataDataView {
 public:
  PaymentRequestEventDataDataView() {}

  PaymentRequestEventDataDataView(
      internal::PaymentRequestEventData_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetTopLevelOriginDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTopLevelOrigin(UserType* output) {
    auto* pointer = data_->top_level_origin.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetPaymentRequestOriginDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPaymentRequestOrigin(UserType* output) {
    auto* pointer = data_->payment_request_origin.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetPaymentRequestIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadPaymentRequestId(UserType* output) {
    auto* pointer = data_->payment_request_id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetMethodDataDataView(
      mojo::ArrayDataView<::payments::mojom::PaymentMethodDataDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMethodData(UserType* output) {
    auto* pointer = data_->method_data.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::payments::mojom::PaymentMethodDataDataView>>(
        pointer, output, context_);
  }
  inline void GetTotalDataView(
      ::payments::mojom::PaymentCurrencyAmountDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTotal(UserType* output) {
    auto* pointer = data_->total.Get();
    return mojo::internal::Deserialize<::payments::mojom::PaymentCurrencyAmountDataView>(
        pointer, output, context_);
  }
  inline void GetModifiersDataView(
      mojo::ArrayDataView<::payments::mojom::PaymentDetailsModifierDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadModifiers(UserType* output) {
    auto* pointer = data_->modifiers.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::payments::mojom::PaymentDetailsModifierDataView>>(
        pointer, output, context_);
  }
  inline void GetInstrumentKeyDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstrumentKey(UserType* output) {
    auto* pointer = data_->instrument_key.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentRequestEventData_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentHandlerResponseDataView {
 public:
  PaymentHandlerResponseDataView() {}

  PaymentHandlerResponseDataView(
      internal::PaymentHandlerResponse_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetMethodNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadMethodName(UserType* output) {
    auto* pointer = data_->method_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetStringifiedDetailsDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStringifiedDetails(UserType* output) {
    auto* pointer = data_->stringified_details.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentHandlerResponse_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_Init_ParamsDataView {
 public:
  PaymentManager_Init_ParamsDataView() {}

  PaymentManager_Init_ParamsDataView(
      internal::PaymentManager_Init_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetServiceWorkerScriptUrlDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadServiceWorkerScriptUrl(UserType* output) {
    auto* pointer = data_->service_worker_script_url.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetServiceWorkerScopeDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadServiceWorkerScope(UserType* output) {
    auto* pointer = data_->service_worker_scope.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentManager_Init_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_DeletePaymentInstrument_ParamsDataView {
 public:
  PaymentManager_DeletePaymentInstrument_ParamsDataView() {}

  PaymentManager_DeletePaymentInstrument_ParamsDataView(
      internal::PaymentManager_DeletePaymentInstrument_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInstrumentKeyDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstrumentKey(UserType* output) {
    auto* pointer = data_->instrument_key.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentManager_DeletePaymentInstrument_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_DeletePaymentInstrument_ResponseParamsDataView {
 public:
  PaymentManager_DeletePaymentInstrument_ResponseParamsDataView() {}

  PaymentManager_DeletePaymentInstrument_ResponseParamsDataView(
      internal::PaymentManager_DeletePaymentInstrument_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::payments::mojom::PaymentHandlerStatus>(
        data_value, output);
  }

  PaymentHandlerStatus status() const {
    return static_cast<PaymentHandlerStatus>(data_->status);
  }
 private:
  internal::PaymentManager_DeletePaymentInstrument_ResponseParams_Data* data_ = nullptr;
};

class PaymentManager_GetPaymentInstrument_ParamsDataView {
 public:
  PaymentManager_GetPaymentInstrument_ParamsDataView() {}

  PaymentManager_GetPaymentInstrument_ParamsDataView(
      internal::PaymentManager_GetPaymentInstrument_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInstrumentKeyDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstrumentKey(UserType* output) {
    auto* pointer = data_->instrument_key.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentManager_GetPaymentInstrument_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_GetPaymentInstrument_ResponseParamsDataView {
 public:
  PaymentManager_GetPaymentInstrument_ResponseParamsDataView() {}

  PaymentManager_GetPaymentInstrument_ResponseParamsDataView(
      internal::PaymentManager_GetPaymentInstrument_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInstrumentDataView(
      PaymentInstrumentDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstrument(UserType* output) {
    auto* pointer = data_->instrument.Get();
    return mojo::internal::Deserialize<::payments::mojom::PaymentInstrumentDataView>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::payments::mojom::PaymentHandlerStatus>(
        data_value, output);
  }

  PaymentHandlerStatus status() const {
    return static_cast<PaymentHandlerStatus>(data_->status);
  }
 private:
  internal::PaymentManager_GetPaymentInstrument_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_KeysOfPaymentInstruments_ParamsDataView {
 public:
  PaymentManager_KeysOfPaymentInstruments_ParamsDataView() {}

  PaymentManager_KeysOfPaymentInstruments_ParamsDataView(
      internal::PaymentManager_KeysOfPaymentInstruments_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::PaymentManager_KeysOfPaymentInstruments_Params_Data* data_ = nullptr;
};

class PaymentManager_KeysOfPaymentInstruments_ResponseParamsDataView {
 public:
  PaymentManager_KeysOfPaymentInstruments_ResponseParamsDataView() {}

  PaymentManager_KeysOfPaymentInstruments_ResponseParamsDataView(
      internal::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetKeysDataView(
      mojo::ArrayDataView<mojo::StringDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadKeys(UserType* output) {
    auto* pointer = data_->keys.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<mojo::StringDataView>>(
        pointer, output, context_);
  }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::payments::mojom::PaymentHandlerStatus>(
        data_value, output);
  }

  PaymentHandlerStatus status() const {
    return static_cast<PaymentHandlerStatus>(data_->status);
  }
 private:
  internal::PaymentManager_KeysOfPaymentInstruments_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_HasPaymentInstrument_ParamsDataView {
 public:
  PaymentManager_HasPaymentInstrument_ParamsDataView() {}

  PaymentManager_HasPaymentInstrument_ParamsDataView(
      internal::PaymentManager_HasPaymentInstrument_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInstrumentKeyDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstrumentKey(UserType* output) {
    auto* pointer = data_->instrument_key.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentManager_HasPaymentInstrument_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_HasPaymentInstrument_ResponseParamsDataView {
 public:
  PaymentManager_HasPaymentInstrument_ResponseParamsDataView() {}

  PaymentManager_HasPaymentInstrument_ResponseParamsDataView(
      internal::PaymentManager_HasPaymentInstrument_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::payments::mojom::PaymentHandlerStatus>(
        data_value, output);
  }

  PaymentHandlerStatus status() const {
    return static_cast<PaymentHandlerStatus>(data_->status);
  }
 private:
  internal::PaymentManager_HasPaymentInstrument_ResponseParams_Data* data_ = nullptr;
};

class PaymentManager_SetPaymentInstrument_ParamsDataView {
 public:
  PaymentManager_SetPaymentInstrument_ParamsDataView() {}

  PaymentManager_SetPaymentInstrument_ParamsDataView(
      internal::PaymentManager_SetPaymentInstrument_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetInstrumentKeyDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstrumentKey(UserType* output) {
    auto* pointer = data_->instrument_key.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetInstrumentDataView(
      PaymentInstrumentDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadInstrument(UserType* output) {
    auto* pointer = data_->instrument.Get();
    return mojo::internal::Deserialize<::payments::mojom::PaymentInstrumentDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentManager_SetPaymentInstrument_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PaymentManager_SetPaymentInstrument_ResponseParamsDataView {
 public:
  PaymentManager_SetPaymentInstrument_ResponseParamsDataView() {}

  PaymentManager_SetPaymentInstrument_ResponseParamsDataView(
      internal::PaymentManager_SetPaymentInstrument_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::payments::mojom::PaymentHandlerStatus>(
        data_value, output);
  }

  PaymentHandlerStatus status() const {
    return static_cast<PaymentHandlerStatus>(data_->status);
  }
 private:
  internal::PaymentManager_SetPaymentInstrument_ResponseParams_Data* data_ = nullptr;
};

class PaymentManager_ClearPaymentInstruments_ParamsDataView {
 public:
  PaymentManager_ClearPaymentInstruments_ParamsDataView() {}

  PaymentManager_ClearPaymentInstruments_ParamsDataView(
      internal::PaymentManager_ClearPaymentInstruments_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
 private:
  internal::PaymentManager_ClearPaymentInstruments_Params_Data* data_ = nullptr;
};

class PaymentManager_ClearPaymentInstruments_ResponseParamsDataView {
 public:
  PaymentManager_ClearPaymentInstruments_ResponseParamsDataView() {}

  PaymentManager_ClearPaymentInstruments_ResponseParamsDataView(
      internal::PaymentManager_ClearPaymentInstruments_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::payments::mojom::PaymentHandlerStatus>(
        data_value, output);
  }

  PaymentHandlerStatus status() const {
    return static_cast<PaymentHandlerStatus>(data_->status);
  }
 private:
  internal::PaymentManager_ClearPaymentInstruments_ResponseParams_Data* data_ = nullptr;
};

class PaymentHandlerResponseCallback_OnPaymentHandlerResponse_ParamsDataView {
 public:
  PaymentHandlerResponseCallback_OnPaymentHandlerResponse_ParamsDataView() {}

  PaymentHandlerResponseCallback_OnPaymentHandlerResponse_ParamsDataView(
      internal::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetResponseDataView(
      PaymentHandlerResponseDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResponse(UserType* output) {
    auto* pointer = data_->response.Get();
    return mojo::internal::Deserialize<::payments::mojom::PaymentHandlerResponseDataView>(
        pointer, output, context_);
  }
  inline void GetDispatchEventTimeDataView(
      ::mojo::common::mojom::TimeDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDispatchEventTime(UserType* output) {
    auto* pointer = data_->dispatch_event_time.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDataView>(
        pointer, output, context_);
  }
 private:
  internal::PaymentHandlerResponseCallback_OnPaymentHandlerResponse_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace payments

namespace std {

template <>
struct hash<::payments::mojom::PaymentHandlerStatus>
    : public mojo::internal::EnumHashImpl<::payments::mojom::PaymentHandlerStatus> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::payments::mojom::PaymentHandlerStatus, ::payments::mojom::PaymentHandlerStatus> {
  static ::payments::mojom::PaymentHandlerStatus ToMojom(::payments::mojom::PaymentHandlerStatus input) { return input; }
  static bool FromMojom(::payments::mojom::PaymentHandlerStatus input, ::payments::mojom::PaymentHandlerStatus* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::PaymentHandlerStatus, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::payments::mojom::PaymentHandlerStatus, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::payments::mojom::PaymentHandlerStatus>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::ImageObjectDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::payments::mojom::ImageObjectDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::payments::mojom::internal::ImageObject_Data);
    decltype(CallWithContext(Traits::src, input, custom_context)) in_src = CallWithContext(Traits::src, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_src, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::payments::mojom::internal::ImageObject_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::payments::mojom::internal::ImageObject_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::src, input, custom_context)) in_src = CallWithContext(Traits::src, input, custom_context);
    typename decltype(result->src)::BaseType* src_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_src, buffer, &src_ptr, context);
    result->src.Set(src_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->src.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null src in ImageObject struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::payments::mojom::internal::ImageObject_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::payments::mojom::ImageObjectDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::PaymentInstrumentDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::payments::mojom::PaymentInstrumentDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::payments::mojom::internal::PaymentInstrument_Data);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_name, context);
    decltype(CallWithContext(Traits::icons, input, custom_context)) in_icons = CallWithContext(Traits::icons, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::payments::mojom::ImageObjectDataView>>(
        in_icons, context);
    decltype(CallWithContext(Traits::enabled_methods, input, custom_context)) in_enabled_methods = CallWithContext(Traits::enabled_methods, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_enabled_methods, context);
    decltype(CallWithContext(Traits::stringified_capabilities, input, custom_context)) in_stringified_capabilities = CallWithContext(Traits::stringified_capabilities, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_stringified_capabilities, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::payments::mojom::internal::PaymentInstrument_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::payments::mojom::internal::PaymentInstrument_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    typename decltype(result->name)::BaseType* name_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_name, buffer, &name_ptr, context);
    result->name.Set(name_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null name in PaymentInstrument struct");
    decltype(CallWithContext(Traits::icons, input, custom_context)) in_icons = CallWithContext(Traits::icons, input, custom_context);
    typename decltype(result->icons)::BaseType* icons_ptr;
    const mojo::internal::ContainerValidateParams icons_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::payments::mojom::ImageObjectDataView>>(
        in_icons, buffer, &icons_ptr, &icons_validate_params,
        context);
    result->icons.Set(icons_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->icons.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null icons in PaymentInstrument struct");
    decltype(CallWithContext(Traits::enabled_methods, input, custom_context)) in_enabled_methods = CallWithContext(Traits::enabled_methods, input, custom_context);
    typename decltype(result->enabled_methods)::BaseType* enabled_methods_ptr;
    const mojo::internal::ContainerValidateParams enabled_methods_validate_params(
        0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr));
    mojo::internal::Serialize<mojo::ArrayDataView<mojo::StringDataView>>(
        in_enabled_methods, buffer, &enabled_methods_ptr, &enabled_methods_validate_params,
        context);
    result->enabled_methods.Set(enabled_methods_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->enabled_methods.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null enabled_methods in PaymentInstrument struct");
    decltype(CallWithContext(Traits::stringified_capabilities, input, custom_context)) in_stringified_capabilities = CallWithContext(Traits::stringified_capabilities, input, custom_context);
    typename decltype(result->stringified_capabilities)::BaseType* stringified_capabilities_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_stringified_capabilities, buffer, &stringified_capabilities_ptr, context);
    result->stringified_capabilities.Set(stringified_capabilities_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->stringified_capabilities.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null stringified_capabilities in PaymentInstrument struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::payments::mojom::internal::PaymentInstrument_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::payments::mojom::PaymentInstrumentDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::PaymentRequestEventDataDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::payments::mojom::PaymentRequestEventDataDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::payments::mojom::internal::PaymentRequestEventData_Data);
    decltype(CallWithContext(Traits::top_level_origin, input, custom_context)) in_top_level_origin = CallWithContext(Traits::top_level_origin, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_top_level_origin, context);
    decltype(CallWithContext(Traits::payment_request_origin, input, custom_context)) in_payment_request_origin = CallWithContext(Traits::payment_request_origin, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_payment_request_origin, context);
    decltype(CallWithContext(Traits::payment_request_id, input, custom_context)) in_payment_request_id = CallWithContext(Traits::payment_request_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_payment_request_id, context);
    decltype(CallWithContext(Traits::method_data, input, custom_context)) in_method_data = CallWithContext(Traits::method_data, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::payments::mojom::PaymentMethodDataDataView>>(
        in_method_data, context);
    decltype(CallWithContext(Traits::total, input, custom_context)) in_total = CallWithContext(Traits::total, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::payments::mojom::PaymentCurrencyAmountDataView>(
        in_total, context);
    decltype(CallWithContext(Traits::modifiers, input, custom_context)) in_modifiers = CallWithContext(Traits::modifiers, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::payments::mojom::PaymentDetailsModifierDataView>>(
        in_modifiers, context);
    decltype(CallWithContext(Traits::instrument_key, input, custom_context)) in_instrument_key = CallWithContext(Traits::instrument_key, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_instrument_key, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::payments::mojom::internal::PaymentRequestEventData_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::payments::mojom::internal::PaymentRequestEventData_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::top_level_origin, input, custom_context)) in_top_level_origin = CallWithContext(Traits::top_level_origin, input, custom_context);
    typename decltype(result->top_level_origin)::BaseType* top_level_origin_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_top_level_origin, buffer, &top_level_origin_ptr, context);
    result->top_level_origin.Set(top_level_origin_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->top_level_origin.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null top_level_origin in PaymentRequestEventData struct");
    decltype(CallWithContext(Traits::payment_request_origin, input, custom_context)) in_payment_request_origin = CallWithContext(Traits::payment_request_origin, input, custom_context);
    typename decltype(result->payment_request_origin)::BaseType* payment_request_origin_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_payment_request_origin, buffer, &payment_request_origin_ptr, context);
    result->payment_request_origin.Set(payment_request_origin_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->payment_request_origin.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null payment_request_origin in PaymentRequestEventData struct");
    decltype(CallWithContext(Traits::payment_request_id, input, custom_context)) in_payment_request_id = CallWithContext(Traits::payment_request_id, input, custom_context);
    typename decltype(result->payment_request_id)::BaseType* payment_request_id_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_payment_request_id, buffer, &payment_request_id_ptr, context);
    result->payment_request_id.Set(payment_request_id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->payment_request_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null payment_request_id in PaymentRequestEventData struct");
    decltype(CallWithContext(Traits::method_data, input, custom_context)) in_method_data = CallWithContext(Traits::method_data, input, custom_context);
    typename decltype(result->method_data)::BaseType* method_data_ptr;
    const mojo::internal::ContainerValidateParams method_data_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::payments::mojom::PaymentMethodDataDataView>>(
        in_method_data, buffer, &method_data_ptr, &method_data_validate_params,
        context);
    result->method_data.Set(method_data_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->method_data.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null method_data in PaymentRequestEventData struct");
    decltype(CallWithContext(Traits::total, input, custom_context)) in_total = CallWithContext(Traits::total, input, custom_context);
    typename decltype(result->total)::BaseType* total_ptr;
    mojo::internal::Serialize<::payments::mojom::PaymentCurrencyAmountDataView>(
        in_total, buffer, &total_ptr, context);
    result->total.Set(total_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->total.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null total in PaymentRequestEventData struct");
    decltype(CallWithContext(Traits::modifiers, input, custom_context)) in_modifiers = CallWithContext(Traits::modifiers, input, custom_context);
    typename decltype(result->modifiers)::BaseType* modifiers_ptr;
    const mojo::internal::ContainerValidateParams modifiers_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::payments::mojom::PaymentDetailsModifierDataView>>(
        in_modifiers, buffer, &modifiers_ptr, &modifiers_validate_params,
        context);
    result->modifiers.Set(modifiers_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->modifiers.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null modifiers in PaymentRequestEventData struct");
    decltype(CallWithContext(Traits::instrument_key, input, custom_context)) in_instrument_key = CallWithContext(Traits::instrument_key, input, custom_context);
    typename decltype(result->instrument_key)::BaseType* instrument_key_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_instrument_key, buffer, &instrument_key_ptr, context);
    result->instrument_key.Set(instrument_key_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->instrument_key.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null instrument_key in PaymentRequestEventData struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::payments::mojom::internal::PaymentRequestEventData_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::payments::mojom::PaymentRequestEventDataDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::payments::mojom::PaymentHandlerResponseDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::payments::mojom::PaymentHandlerResponseDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::payments::mojom::internal::PaymentHandlerResponse_Data);
    decltype(CallWithContext(Traits::method_name, input, custom_context)) in_method_name = CallWithContext(Traits::method_name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_method_name, context);
    decltype(CallWithContext(Traits::stringified_details, input, custom_context)) in_stringified_details = CallWithContext(Traits::stringified_details, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_stringified_details, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::payments::mojom::internal::PaymentHandlerResponse_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::payments::mojom::internal::PaymentHandlerResponse_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::method_name, input, custom_context)) in_method_name = CallWithContext(Traits::method_name, input, custom_context);
    typename decltype(result->method_name)::BaseType* method_name_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_method_name, buffer, &method_name_ptr, context);
    result->method_name.Set(method_name_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->method_name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null method_name in PaymentHandlerResponse struct");
    decltype(CallWithContext(Traits::stringified_details, input, custom_context)) in_stringified_details = CallWithContext(Traits::stringified_details, input, custom_context);
    typename decltype(result->stringified_details)::BaseType* stringified_details_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_stringified_details, buffer, &stringified_details_ptr, context);
    result->stringified_details.Set(stringified_details_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->stringified_details.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null stringified_details in PaymentHandlerResponse struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::payments::mojom::internal::PaymentHandlerResponse_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::payments::mojom::PaymentHandlerResponseDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace payments {
namespace mojom {

inline void ImageObjectDataView::GetSrcDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->src.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}


inline void PaymentInstrumentDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentInstrumentDataView::GetIconsDataView(
    mojo::ArrayDataView<ImageObjectDataView>* output) {
  auto pointer = data_->icons.Get();
  *output = mojo::ArrayDataView<ImageObjectDataView>(pointer, context_);
}
inline void PaymentInstrumentDataView::GetEnabledMethodsDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->enabled_methods.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}
inline void PaymentInstrumentDataView::GetStringifiedCapabilitiesDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->stringified_capabilities.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void PaymentRequestEventDataDataView::GetTopLevelOriginDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->top_level_origin.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void PaymentRequestEventDataDataView::GetPaymentRequestOriginDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->payment_request_origin.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void PaymentRequestEventDataDataView::GetPaymentRequestIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->payment_request_id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentRequestEventDataDataView::GetMethodDataDataView(
    mojo::ArrayDataView<::payments::mojom::PaymentMethodDataDataView>* output) {
  auto pointer = data_->method_data.Get();
  *output = mojo::ArrayDataView<::payments::mojom::PaymentMethodDataDataView>(pointer, context_);
}
inline void PaymentRequestEventDataDataView::GetTotalDataView(
    ::payments::mojom::PaymentCurrencyAmountDataView* output) {
  auto pointer = data_->total.Get();
  *output = ::payments::mojom::PaymentCurrencyAmountDataView(pointer, context_);
}
inline void PaymentRequestEventDataDataView::GetModifiersDataView(
    mojo::ArrayDataView<::payments::mojom::PaymentDetailsModifierDataView>* output) {
  auto pointer = data_->modifiers.Get();
  *output = mojo::ArrayDataView<::payments::mojom::PaymentDetailsModifierDataView>(pointer, context_);
}
inline void PaymentRequestEventDataDataView::GetInstrumentKeyDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->instrument_key.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void PaymentHandlerResponseDataView::GetMethodNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->method_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentHandlerResponseDataView::GetStringifiedDetailsDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->stringified_details.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void PaymentManager_Init_ParamsDataView::GetServiceWorkerScriptUrlDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->service_worker_script_url.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentManager_Init_ParamsDataView::GetServiceWorkerScopeDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->service_worker_scope.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void PaymentManager_DeletePaymentInstrument_ParamsDataView::GetInstrumentKeyDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->instrument_key.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void PaymentManager_GetPaymentInstrument_ParamsDataView::GetInstrumentKeyDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->instrument_key.Get();
  *output = mojo::StringDataView(pointer, context_);
}


inline void PaymentManager_GetPaymentInstrument_ResponseParamsDataView::GetInstrumentDataView(
    PaymentInstrumentDataView* output) {
  auto pointer = data_->instrument.Get();
  *output = PaymentInstrumentDataView(pointer, context_);
}




inline void PaymentManager_KeysOfPaymentInstruments_ResponseParamsDataView::GetKeysDataView(
    mojo::ArrayDataView<mojo::StringDataView>* output) {
  auto pointer = data_->keys.Get();
  *output = mojo::ArrayDataView<mojo::StringDataView>(pointer, context_);
}


inline void PaymentManager_HasPaymentInstrument_ParamsDataView::GetInstrumentKeyDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->instrument_key.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void PaymentManager_SetPaymentInstrument_ParamsDataView::GetInstrumentKeyDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->instrument_key.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PaymentManager_SetPaymentInstrument_ParamsDataView::GetInstrumentDataView(
    PaymentInstrumentDataView* output) {
  auto pointer = data_->instrument.Get();
  *output = PaymentInstrumentDataView(pointer, context_);
}








inline void PaymentHandlerResponseCallback_OnPaymentHandlerResponse_ParamsDataView::GetResponseDataView(
    PaymentHandlerResponseDataView* output) {
  auto pointer = data_->response.Get();
  *output = PaymentHandlerResponseDataView(pointer, context_);
}
inline void PaymentHandlerResponseCallback_OnPaymentHandlerResponse_ParamsDataView::GetDispatchEventTimeDataView(
    ::mojo::common::mojom::TimeDataView* output) {
  auto pointer = data_->dispatch_event_time.Get();
  *output = ::mojo::common::mojom::TimeDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace payments

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_PAYMENTS_PAYMENT_APP_MOJOM_SHARED_H_
