// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4065)
#endif

#include "third_party/WebKit/public/platform/modules/presentation/presentation.mojom-shared.h"

#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/cpp/bindings/lib/validation_util.h"
namespace blink {
namespace mojom {

namespace internal {
// static
bool PresentationConnectionMessage_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context,
    bool inlined) {
  if (!data) {
    DCHECK(!inlined);
    return true;
  }

  // If it is inlined, the alignment is already enforced by its enclosing
  // object. We don't have to validate that.
  DCHECK(!inlined || mojo::internal::IsAligned(data));

  if (!inlined &&
      !mojo::internal::ValidateNonInlinedUnionHeaderAndClaimMemory(
          data, validation_context)) {
    return false;
  }

  const PresentationConnectionMessage_Data* object = static_cast<const PresentationConnectionMessage_Data*>(data);
  ALLOW_UNUSED_LOCAL(object);

  if (inlined && object->is_null())
    return true;

  switch (object->tag) {

    case PresentationConnectionMessage_Tag::MESSAGE: {

      if (!mojo::internal::ValidatePointerNonNullable(
              object->data.f_message, "null message field in PresentationConnectionMessage",
              validation_context)) {
        return false;
      }
      const mojo::internal::ContainerValidateParams message_validate_params(
          0, false, nullptr);
      if (!mojo::internal::ValidateContainer(object->data.f_message, validation_context,
                                             &message_validate_params)) {
        return false;
      }
      return true;
    }
    case PresentationConnectionMessage_Tag::DATA: {

      if (!mojo::internal::ValidatePointerNonNullable(
              object->data.f_data, "null data field in PresentationConnectionMessage",
              validation_context)) {
        return false;
      }
      const mojo::internal::ContainerValidateParams data_validate_params(
          0, false, nullptr);
      if (!mojo::internal::ValidateContainer(object->data.f_data, validation_context,
                                             &data_validate_params)) {
        return false;
      }
      return true;
    }
    default: {
      ReportValidationError(
          validation_context,
          mojo::internal::VALIDATION_ERROR_UNKNOWN_UNION_TAG,
          "unknown tag in PresentationConnectionMessage");
      return false;
    }
  }
}


// static
PresentationInfo_Data* PresentationInfo_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationInfo_Data))) PresentationInfo_Data();
}

// static
bool PresentationInfo_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationInfo_Data* object = static_cast<const PresentationInfo_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->url, "null url field in PresentationInfo",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->url, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->id, "null id field in PresentationInfo",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->id, validation_context,
                                         &id_validate_params)) {
    return false;
  }

  return true;
}

PresentationInfo_Data::PresentationInfo_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationError_Data* PresentationError_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationError_Data))) PresentationError_Data();
}

// static
bool PresentationError_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationError_Data* object = static_cast<const PresentationError_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::PresentationErrorType_Data
        ::Validate(object->error_type, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->message, "null message field in PresentationError",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams message_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->message, validation_context,
                                         &message_validate_params)) {
    return false;
  }

  return true;
}

PresentationError_Data::PresentationError_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationConnection_OnMessage_Params_Data* PresentationConnection_OnMessage_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationConnection_OnMessage_Params_Data))) PresentationConnection_OnMessage_Params_Data();
}

// static
bool PresentationConnection_OnMessage_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationConnection_OnMessage_Params_Data* object = static_cast<const PresentationConnection_OnMessage_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateInlinedUnionNonNullable(
          object->message, "null message field in PresentationConnection_OnMessage_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateInlinedUnion(object->message, validation_context))
    return false;

  return true;
}

PresentationConnection_OnMessage_Params_Data::PresentationConnection_OnMessage_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationConnection_OnMessage_ResponseParams_Data* PresentationConnection_OnMessage_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationConnection_OnMessage_ResponseParams_Data))) PresentationConnection_OnMessage_ResponseParams_Data();
}

// static
bool PresentationConnection_OnMessage_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationConnection_OnMessage_ResponseParams_Data* object = static_cast<const PresentationConnection_OnMessage_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

PresentationConnection_OnMessage_ResponseParams_Data::PresentationConnection_OnMessage_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationConnection_DidChangeState_Params_Data* PresentationConnection_DidChangeState_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationConnection_DidChangeState_Params_Data))) PresentationConnection_DidChangeState_Params_Data();
}

// static
bool PresentationConnection_DidChangeState_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationConnection_DidChangeState_Params_Data* object = static_cast<const PresentationConnection_DidChangeState_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }


  if (!::blink::mojom::internal::PresentationConnectionState_Data
        ::Validate(object->state, validation_context))
    return false;

  return true;
}

PresentationConnection_DidChangeState_Params_Data::PresentationConnection_DidChangeState_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationConnection_OnClose_Params_Data* PresentationConnection_OnClose_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationConnection_OnClose_Params_Data))) PresentationConnection_OnClose_Params_Data();
}

// static
bool PresentationConnection_OnClose_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationConnection_OnClose_Params_Data* object = static_cast<const PresentationConnection_OnClose_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 8 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  return true;
}

PresentationConnection_OnClose_Params_Data::PresentationConnection_OnClose_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_SetClient_Params_Data* PresentationService_SetClient_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_SetClient_Params_Data))) PresentationService_SetClient_Params_Data();
}

// static
bool PresentationService_SetClient_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_SetClient_Params_Data* object = static_cast<const PresentationService_SetClient_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->client,
          "invalid client field in PresentationService_SetClient_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->client,
                                                 validation_context)) {
    return false;
  }

  return true;
}

PresentationService_SetClient_Params_Data::PresentationService_SetClient_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_SetDefaultPresentationUrls_Params_Data* PresentationService_SetDefaultPresentationUrls_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_SetDefaultPresentationUrls_Params_Data))) PresentationService_SetDefaultPresentationUrls_Params_Data();
}

// static
bool PresentationService_SetDefaultPresentationUrls_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_SetDefaultPresentationUrls_Params_Data* object = static_cast<const PresentationService_SetDefaultPresentationUrls_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_urls, "null presentation_urls field in PresentationService_SetDefaultPresentationUrls_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams presentation_urls_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->presentation_urls, validation_context,
                                         &presentation_urls_validate_params)) {
    return false;
  }

  return true;
}

PresentationService_SetDefaultPresentationUrls_Params_Data::PresentationService_SetDefaultPresentationUrls_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_ListenForScreenAvailability_Params_Data* PresentationService_ListenForScreenAvailability_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_ListenForScreenAvailability_Params_Data))) PresentationService_ListenForScreenAvailability_Params_Data();
}

// static
bool PresentationService_ListenForScreenAvailability_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_ListenForScreenAvailability_Params_Data* object = static_cast<const PresentationService_ListenForScreenAvailability_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->availability_url, "null availability_url field in PresentationService_ListenForScreenAvailability_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->availability_url, validation_context))
    return false;

  return true;
}

PresentationService_ListenForScreenAvailability_Params_Data::PresentationService_ListenForScreenAvailability_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_StopListeningForScreenAvailability_Params_Data* PresentationService_StopListeningForScreenAvailability_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_StopListeningForScreenAvailability_Params_Data))) PresentationService_StopListeningForScreenAvailability_Params_Data();
}

// static
bool PresentationService_StopListeningForScreenAvailability_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_StopListeningForScreenAvailability_Params_Data* object = static_cast<const PresentationService_StopListeningForScreenAvailability_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->availability_url, "null availability_url field in PresentationService_StopListeningForScreenAvailability_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->availability_url, validation_context))
    return false;

  return true;
}

PresentationService_StopListeningForScreenAvailability_Params_Data::PresentationService_StopListeningForScreenAvailability_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_StartPresentation_Params_Data* PresentationService_StartPresentation_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_StartPresentation_Params_Data))) PresentationService_StartPresentation_Params_Data();
}

// static
bool PresentationService_StartPresentation_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_StartPresentation_Params_Data* object = static_cast<const PresentationService_StartPresentation_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_urls, "null presentation_urls field in PresentationService_StartPresentation_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams presentation_urls_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->presentation_urls, validation_context,
                                         &presentation_urls_validate_params)) {
    return false;
  }

  return true;
}

PresentationService_StartPresentation_Params_Data::PresentationService_StartPresentation_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_StartPresentation_ResponseParams_Data* PresentationService_StartPresentation_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_StartPresentation_ResponseParams_Data))) PresentationService_StartPresentation_ResponseParams_Data();
}

// static
bool PresentationService_StartPresentation_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_StartPresentation_ResponseParams_Data* object = static_cast<const PresentationService_StartPresentation_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->presentation_info, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->error, validation_context))
    return false;

  return true;
}

PresentationService_StartPresentation_ResponseParams_Data::PresentationService_StartPresentation_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_ReconnectPresentation_Params_Data* PresentationService_ReconnectPresentation_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_ReconnectPresentation_Params_Data))) PresentationService_ReconnectPresentation_Params_Data();
}

// static
bool PresentationService_ReconnectPresentation_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_ReconnectPresentation_Params_Data* object = static_cast<const PresentationService_ReconnectPresentation_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_urls, "null presentation_urls field in PresentationService_ReconnectPresentation_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams presentation_urls_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->presentation_urls, validation_context,
                                         &presentation_urls_validate_params)) {
    return false;
  }

  const mojo::internal::ContainerValidateParams presentation_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->presentation_id, validation_context,
                                         &presentation_id_validate_params)) {
    return false;
  }

  return true;
}

PresentationService_ReconnectPresentation_Params_Data::PresentationService_ReconnectPresentation_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_ReconnectPresentation_ResponseParams_Data* PresentationService_ReconnectPresentation_ResponseParams_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_ReconnectPresentation_ResponseParams_Data))) PresentationService_ReconnectPresentation_ResponseParams_Data();
}

// static
bool PresentationService_ReconnectPresentation_ResponseParams_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_ReconnectPresentation_ResponseParams_Data* object = static_cast<const PresentationService_ReconnectPresentation_ResponseParams_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidateStruct(object->presentation_info, validation_context))
    return false;

  if (!mojo::internal::ValidateStruct(object->error, validation_context))
    return false;

  return true;
}

PresentationService_ReconnectPresentation_ResponseParams_Data::PresentationService_ReconnectPresentation_ResponseParams_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_SetPresentationConnection_Params_Data* PresentationService_SetPresentationConnection_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_SetPresentationConnection_Params_Data))) PresentationService_SetPresentationConnection_Params_Data();
}

// static
bool PresentationService_SetPresentationConnection_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_SetPresentationConnection_Params_Data* object = static_cast<const PresentationService_SetPresentationConnection_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_info, "null presentation_info field in PresentationService_SetPresentationConnection_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->presentation_info, validation_context))
    return false;

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->controller_connection_ptr,
          "invalid controller_connection_ptr field in PresentationService_SetPresentationConnection_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->controller_connection_ptr,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->receiver_connection_request,
          "invalid receiver_connection_request field in PresentationService_SetPresentationConnection_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->receiver_connection_request,
                                                 validation_context)) {
    return false;
  }

  return true;
}

PresentationService_SetPresentationConnection_Params_Data::PresentationService_SetPresentationConnection_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_CloseConnection_Params_Data* PresentationService_CloseConnection_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_CloseConnection_Params_Data))) PresentationService_CloseConnection_Params_Data();
}

// static
bool PresentationService_CloseConnection_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_CloseConnection_Params_Data* object = static_cast<const PresentationService_CloseConnection_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_url, "null presentation_url field in PresentationService_CloseConnection_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->presentation_url, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_id, "null presentation_id field in PresentationService_CloseConnection_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams presentation_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->presentation_id, validation_context,
                                         &presentation_id_validate_params)) {
    return false;
  }

  return true;
}

PresentationService_CloseConnection_Params_Data::PresentationService_CloseConnection_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationService_Terminate_Params_Data* PresentationService_Terminate_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationService_Terminate_Params_Data))) PresentationService_Terminate_Params_Data();
}

// static
bool PresentationService_Terminate_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationService_Terminate_Params_Data* object = static_cast<const PresentationService_Terminate_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_url, "null presentation_url field in PresentationService_Terminate_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->presentation_url, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_id, "null presentation_id field in PresentationService_Terminate_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams presentation_id_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->presentation_id, validation_context,
                                         &presentation_id_validate_params)) {
    return false;
  }

  return true;
}

PresentationService_Terminate_Params_Data::PresentationService_Terminate_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data* PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data))) PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data();
}

// static
bool PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data* object = static_cast<const PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->url, "null url field in PresentationServiceClient_OnScreenAvailabilityUpdated_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->url, validation_context))
    return false;


  if (!::blink::mojom::internal::ScreenAvailability_Data
        ::Validate(object->availability, validation_context))
    return false;

  return true;
}

PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data::PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationServiceClient_OnDefaultPresentationStarted_Params_Data* PresentationServiceClient_OnDefaultPresentationStarted_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationServiceClient_OnDefaultPresentationStarted_Params_Data))) PresentationServiceClient_OnDefaultPresentationStarted_Params_Data();
}

// static
bool PresentationServiceClient_OnDefaultPresentationStarted_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationServiceClient_OnDefaultPresentationStarted_Params_Data* object = static_cast<const PresentationServiceClient_OnDefaultPresentationStarted_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 16 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_info, "null presentation_info field in PresentationServiceClient_OnDefaultPresentationStarted_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->presentation_info, validation_context))
    return false;

  return true;
}

PresentationServiceClient_OnDefaultPresentationStarted_Params_Data::PresentationServiceClient_OnDefaultPresentationStarted_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data* PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data))) PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data();
}

// static
bool PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data* object = static_cast<const PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_info, "null presentation_info field in PresentationServiceClient_OnReceiverConnectionAvailable_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->presentation_info, validation_context))
    return false;

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->controller_connection_ptr,
          "invalid controller_connection_ptr field in PresentationServiceClient_OnReceiverConnectionAvailable_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->controller_connection_ptr,
                                                 validation_context)) {
    return false;
  }

  if (!mojo::internal::ValidateHandleOrInterfaceNonNullable(
          object->receiver_connection_request,
          "invalid receiver_connection_request field in PresentationServiceClient_OnReceiverConnectionAvailable_Params", validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateHandleOrInterface(object->receiver_connection_request,
                                                 validation_context)) {
    return false;
  }

  return true;
}

PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data::PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationServiceClient_OnConnectionStateChanged_Params_Data* PresentationServiceClient_OnConnectionStateChanged_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationServiceClient_OnConnectionStateChanged_Params_Data))) PresentationServiceClient_OnConnectionStateChanged_Params_Data();
}

// static
bool PresentationServiceClient_OnConnectionStateChanged_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationServiceClient_OnConnectionStateChanged_Params_Data* object = static_cast<const PresentationServiceClient_OnConnectionStateChanged_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 24 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_info, "null presentation_info field in PresentationServiceClient_OnConnectionStateChanged_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->presentation_info, validation_context))
    return false;


  if (!::blink::mojom::internal::PresentationConnectionState_Data
        ::Validate(object->newState, validation_context))
    return false;

  return true;
}

PresentationServiceClient_OnConnectionStateChanged_Params_Data::PresentationServiceClient_OnConnectionStateChanged_Params_Data()
    : header_({sizeof(*this), 0}) {}


// static
PresentationServiceClient_OnConnectionClosed_Params_Data* PresentationServiceClient_OnConnectionClosed_Params_Data::New(mojo::internal::Buffer* buf) {
  return new (buf->Allocate(sizeof(PresentationServiceClient_OnConnectionClosed_Params_Data))) PresentationServiceClient_OnConnectionClosed_Params_Data();
}

// static
bool PresentationServiceClient_OnConnectionClosed_Params_Data::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  if (!data)
    return true;

  if (!ValidateStructHeaderAndClaimMemory(data, validation_context))
    return false;

  // NOTE: The memory backing |object| may be smaller than |sizeof(*object)| if
  // the message comes from an older version.
  const PresentationServiceClient_OnConnectionClosed_Params_Data* object = static_cast<const PresentationServiceClient_OnConnectionClosed_Params_Data*>(data);

  static constexpr struct {
    uint32_t version;
    uint32_t num_bytes;
  } kVersionSizes[] = {{ 0, 32 }};

  if (object->header_.version <=
          kVersionSizes[arraysize(kVersionSizes) - 1].version) {
    // Scan in reverse order to optimize for more recent versions.
    for (int i = arraysize(kVersionSizes) - 1; i >= 0; --i) {
      if (object->header_.version >= kVersionSizes[i].version) {
        if (object->header_.num_bytes == kVersionSizes[i].num_bytes)
          break;

        ReportValidationError(
            validation_context,
            mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
        return false;
      }
    }
  } else if (object->header_.num_bytes <
                 kVersionSizes[arraysize(kVersionSizes) - 1].num_bytes) {
    ReportValidationError(
        validation_context,
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_STRUCT_HEADER);
    return false;
  }

  if (!mojo::internal::ValidatePointerNonNullable(
          object->presentation_info, "null presentation_info field in PresentationServiceClient_OnConnectionClosed_Params",
          validation_context)) {
    return false;
  }
  if (!mojo::internal::ValidateStruct(object->presentation_info, validation_context))
    return false;


  if (!::blink::mojom::internal::PresentationConnectionCloseReason_Data
        ::Validate(object->reason, validation_context))
    return false;

  if (!mojo::internal::ValidatePointerNonNullable(
          object->message, "null message field in PresentationServiceClient_OnConnectionClosed_Params",
          validation_context)) {
    return false;
  }
  const mojo::internal::ContainerValidateParams message_validate_params(
      0, false, nullptr);
  if (!mojo::internal::ValidateContainer(object->message, validation_context,
                                         &message_validate_params)) {
    return false;
  }

  return true;
}

PresentationServiceClient_OnConnectionClosed_Params_Data::PresentationServiceClient_OnConnectionClosed_Params_Data()
    : header_({sizeof(*this), 0}) {}

}  // namespace internal
}  // namespace mojom
}  // namespace blink

#if defined(_MSC_VER)
#pragma warning(pop)
#endif
