// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "third_party/WebKit/public/platform/modules/presentation/presentation.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "content/common/presentation/presentation_struct_traits.h"
#include "url/mojo/url_gurl_struct_traits.h"
namespace blink {
namespace mojom {
PresentationInfo::PresentationInfo()
    : url(),
      id() {}

PresentationInfo::PresentationInfo(
    const GURL& url_in,
    const std::string& id_in)
    : url(std::move(url_in)),
      id(std::move(id_in)) {}

PresentationInfo::~PresentationInfo() = default;

bool PresentationInfo::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PresentationError::PresentationError()
    : error_type(),
      message() {}

PresentationError::PresentationError(
    content::PresentationErrorType error_type_in,
    const std::string& message_in)
    : error_type(std::move(error_type_in)),
      message(std::move(message_in)) {}

PresentationError::~PresentationError() = default;

bool PresentationError::Validate(
    const void* data,
    mojo::internal::ValidationContext* validation_context) {
  return Data_::Validate(data, validation_context);
}
PresentationConnectionMessage::PresentationConnectionMessage() : tag_(Tag::MESSAGE) {
  data_.message = new std::string;
}

PresentationConnectionMessage::~PresentationConnectionMessage() {
  DestroyActive();
}


void PresentationConnectionMessage::set_message(
    const std::string& message) {
  if (tag_ == Tag::MESSAGE) {
    *(data_.message) = std::move(message);
  } else {
    DestroyActive();
    tag_ = Tag::MESSAGE;
    data_.message = new std::string(
        std::move(message));
  }
}
void PresentationConnectionMessage::set_data(
    const std::vector<uint8_t>& data) {
  if (tag_ == Tag::DATA) {
    *(data_.data) = std::move(data);
  } else {
    DestroyActive();
    tag_ = Tag::DATA;
    data_.data = new std::vector<uint8_t>(
        std::move(data));
  }
}

void PresentationConnectionMessage::DestroyActive() {
  switch (tag_) {

    case Tag::MESSAGE:

      delete data_.message;
      break;
    case Tag::DATA:

      delete data_.data;
      break;
  }
}
const char PresentationConnection::Name_[] = "blink::mojom::PresentationConnection";

class PresentationConnection_OnMessage_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PresentationConnection_OnMessage_ForwardToCallback(
      PresentationConnection::OnMessageCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PresentationConnection::OnMessageCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PresentationConnection_OnMessage_ForwardToCallback);
};

PresentationConnectionProxy::PresentationConnectionProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PresentationConnectionProxy::OnMessage(
    content::PresentationConnectionMessage in_message, OnMessageCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationConnection_OnMessage_Params_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationConnectionMessageDataView>(
      in_message, true, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationConnection_OnMessage_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationConnection_OnMessage_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  auto message_ptr = &params->message;
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionMessageDataView>(
      in_message, buffer, &message_ptr, true, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->message.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null message in PresentationConnection.OnMessage request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PresentationConnection_OnMessage_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PresentationConnectionProxy::DidChangeState(
    content::PresentationConnectionState in_state) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationConnection_DidChangeState_Params_Data);
  serialization_context.PrepareMessage(
      internal::kPresentationConnection_DidChangeState_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationConnection_DidChangeState_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionState>(
      in_state, &params->state);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationConnectionProxy::OnClose(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationConnection_OnClose_Params_Data);
  serialization_context.PrepareMessage(
      internal::kPresentationConnection_OnClose_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationConnection_OnClose_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class PresentationConnection_OnMessage_ProxyToResponder {
 public:
  static PresentationConnection::OnMessageCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PresentationConnection_OnMessage_ProxyToResponder> proxy(
        new PresentationConnection_OnMessage_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PresentationConnection_OnMessage_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PresentationConnection_OnMessage_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PresentationConnection::OnMessage() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PresentationConnection_OnMessage_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      bool in_success);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PresentationConnection_OnMessage_ProxyToResponder);
};

bool PresentationConnection_OnMessage_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PresentationConnection_OnMessage_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PresentationConnection_OnMessage_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_success{};
  PresentationConnection_OnMessage_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_success = input_data_view.success();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PresentationConnection::OnMessage response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_success));
  return true;
}

void PresentationConnection_OnMessage_ProxyToResponder::Run(
    bool in_success) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationConnection_OnMessage_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kPresentationConnection_OnMessage_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationConnection_OnMessage_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->success = in_success;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool PresentationConnectionStubDispatch::Accept(
    PresentationConnection* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPresentationConnection_OnMessage_Name: {
      break;
    }
    case internal::kPresentationConnection_DidChangeState_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationConnection_DidChangeState_Params_Data* params =
          reinterpret_cast<internal::PresentationConnection_DidChangeState_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      content::PresentationConnectionState p_state{};
      PresentationConnection_DidChangeState_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadState(&p_state))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationConnection::DidChangeState deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DidChangeState(
std::move(p_state));
      return true;
    }
    case internal::kPresentationConnection_OnClose_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationConnection_OnClose_Params_Data* params =
          reinterpret_cast<internal::PresentationConnection_OnClose_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PresentationConnection_OnClose_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationConnection::OnClose deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnClose();
      return true;
    }
  }
  return false;
}

// static
bool PresentationConnectionStubDispatch::AcceptWithResponder(
    PresentationConnection* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPresentationConnection_OnMessage_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PresentationConnection_OnMessage_Params_Data* params =
          reinterpret_cast<
              internal::PresentationConnection_OnMessage_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      content::PresentationConnectionMessage p_message{};
      PresentationConnection_OnMessage_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadMessage(&p_message))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationConnection::OnMessage deserializer");
        return false;
      }
      PresentationConnection::OnMessageCallback callback =
          PresentationConnection_OnMessage_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnMessage(
std::move(p_message), std::move(callback));
      return true;
    }
    case internal::kPresentationConnection_DidChangeState_Name: {
      break;
    }
    case internal::kPresentationConnection_OnClose_Name: {
      break;
    }
  }
  return false;
}

bool PresentationConnectionRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PresentationConnection RequestValidator");

  switch (message->header()->name) {
    case internal::kPresentationConnection_OnMessage_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationConnection_OnMessage_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationConnection_DidChangeState_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationConnection_DidChangeState_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationConnection_OnClose_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationConnection_OnClose_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool PresentationConnectionResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PresentationConnection ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kPresentationConnection_OnMessage_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationConnection_OnMessage_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char PresentationService::Name_[] = "blink::mojom::PresentationService";

class PresentationService_StartPresentation_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PresentationService_StartPresentation_ForwardToCallback(
      PresentationService::StartPresentationCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PresentationService::StartPresentationCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PresentationService_StartPresentation_ForwardToCallback);
};

class PresentationService_ReconnectPresentation_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  PresentationService_ReconnectPresentation_ForwardToCallback(
      PresentationService::ReconnectPresentationCallback callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  PresentationService::ReconnectPresentationCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(PresentationService_ReconnectPresentation_ForwardToCallback);
};

PresentationServiceProxy::PresentationServiceProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PresentationServiceProxy::SetClient(
    PresentationServiceClientPtr in_client) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_SetClient_Params_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationServiceClientPtrDataView>(
      in_client, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_SetClient_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_SetClient_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::blink::mojom::PresentationServiceClientPtrDataView>(
      in_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in PresentationService.SetClient request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceProxy::SetDefaultPresentationUrls(
    const std::vector<GURL>& in_presentation_urls) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_SetDefaultPresentationUrls_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
      in_presentation_urls, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_SetDefaultPresentationUrls_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_SetDefaultPresentationUrls_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_urls)::BaseType* presentation_urls_ptr;
  const mojo::internal::ContainerValidateParams presentation_urls_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
      in_presentation_urls, buffer, &presentation_urls_ptr, &presentation_urls_validate_params,
      &serialization_context);
  params->presentation_urls.Set(presentation_urls_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_urls.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_urls in PresentationService.SetDefaultPresentationUrls request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceProxy::ListenForScreenAvailability(
    const GURL& in_availability_url) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_ListenForScreenAvailability_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_availability_url, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_ListenForScreenAvailability_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_ListenForScreenAvailability_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->availability_url)::BaseType* availability_url_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_availability_url, buffer, &availability_url_ptr, &serialization_context);
  params->availability_url.Set(availability_url_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->availability_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null availability_url in PresentationService.ListenForScreenAvailability request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceProxy::StopListeningForScreenAvailability(
    const GURL& in_availability_url) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_StopListeningForScreenAvailability_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_availability_url, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_StopListeningForScreenAvailability_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_StopListeningForScreenAvailability_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->availability_url)::BaseType* availability_url_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_availability_url, buffer, &availability_url_ptr, &serialization_context);
  params->availability_url.Set(availability_url_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->availability_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null availability_url in PresentationService.StopListeningForScreenAvailability request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceProxy::StartPresentation(
    const std::vector<GURL>& in_presentation_urls, StartPresentationCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_StartPresentation_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
      in_presentation_urls, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_StartPresentation_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_StartPresentation_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_urls)::BaseType* presentation_urls_ptr;
  const mojo::internal::ContainerValidateParams presentation_urls_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
      in_presentation_urls, buffer, &presentation_urls_ptr, &presentation_urls_validate_params,
      &serialization_context);
  params->presentation_urls.Set(presentation_urls_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_urls.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_urls in PresentationService.StartPresentation request");
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PresentationService_StartPresentation_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PresentationServiceProxy::ReconnectPresentation(
    const std::vector<GURL>& in_presentation_urls, const base::Optional<std::string>& in_presentation_id, ReconnectPresentationCallback callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_ReconnectPresentation_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
      in_presentation_urls, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_presentation_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_ReconnectPresentation_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_ReconnectPresentation_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_urls)::BaseType* presentation_urls_ptr;
  const mojo::internal::ContainerValidateParams presentation_urls_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::url::mojom::UrlDataView>>(
      in_presentation_urls, buffer, &presentation_urls_ptr, &presentation_urls_validate_params,
      &serialization_context);
  params->presentation_urls.Set(presentation_urls_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_urls.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_urls in PresentationService.ReconnectPresentation request");
  typename decltype(params->presentation_id)::BaseType* presentation_id_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_presentation_id, buffer, &presentation_id_ptr, &serialization_context);
  params->presentation_id.Set(presentation_id_ptr);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new PresentationService_ReconnectPresentation_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void PresentationServiceProxy::SetPresentationConnection(
    const content::PresentationInfo& in_presentation_info, PresentationConnectionPtr in_controller_connection_ptr, PresentationConnectionRequest in_receiver_connection_request) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_SetPresentationConnection_Params_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationConnectionPtrDataView>(
      in_controller_connection_ptr, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationConnectionRequestDataView>(
      in_receiver_connection_request, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_SetPresentationConnection_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_SetPresentationConnection_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_info)::BaseType* presentation_info_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, buffer, &presentation_info_ptr, &serialization_context);
  params->presentation_info.Set(presentation_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_info in PresentationService.SetPresentationConnection request");
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionPtrDataView>(
      in_controller_connection_ptr, &params->controller_connection_ptr, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->controller_connection_ptr),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid controller_connection_ptr in PresentationService.SetPresentationConnection request");
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionRequestDataView>(
      in_receiver_connection_request, &params->receiver_connection_request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->receiver_connection_request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid receiver_connection_request in PresentationService.SetPresentationConnection request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceProxy::CloseConnection(
    const GURL& in_presentation_url, const std::string& in_presentation_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_CloseConnection_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_presentation_url, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_presentation_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_CloseConnection_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_CloseConnection_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_url)::BaseType* presentation_url_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_presentation_url, buffer, &presentation_url_ptr, &serialization_context);
  params->presentation_url.Set(presentation_url_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_url in PresentationService.CloseConnection request");
  typename decltype(params->presentation_id)::BaseType* presentation_id_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_presentation_id, buffer, &presentation_id_ptr, &serialization_context);
  params->presentation_id.Set(presentation_id_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_id in PresentationService.CloseConnection request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceProxy::Terminate(
    const GURL& in_presentation_url, const std::string& in_presentation_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_Terminate_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_presentation_url, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_presentation_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_Terminate_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_Terminate_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_url)::BaseType* presentation_url_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_presentation_url, buffer, &presentation_url_ptr, &serialization_context);
  params->presentation_url.Set(presentation_url_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_url in PresentationService.Terminate request");
  typename decltype(params->presentation_id)::BaseType* presentation_id_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_presentation_id, buffer, &presentation_id_ptr, &serialization_context);
  params->presentation_id.Set(presentation_id_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_id in PresentationService.Terminate request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class PresentationService_StartPresentation_ProxyToResponder {
 public:
  static PresentationService::StartPresentationCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PresentationService_StartPresentation_ProxyToResponder> proxy(
        new PresentationService_StartPresentation_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PresentationService_StartPresentation_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PresentationService_StartPresentation_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PresentationService::StartPresentation() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PresentationService_StartPresentation_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const base::Optional<content::PresentationInfo>& in_presentation_info, const base::Optional<content::PresentationError>& in_error);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PresentationService_StartPresentation_ProxyToResponder);
};

bool PresentationService_StartPresentation_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PresentationService_StartPresentation_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PresentationService_StartPresentation_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Optional<content::PresentationInfo> p_presentation_info{};
  base::Optional<content::PresentationError> p_error{};
  PresentationService_StartPresentation_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadPresentationInfo(&p_presentation_info))
    success = false;
  if (!input_data_view.ReadError(&p_error))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PresentationService::StartPresentation response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_presentation_info), 
std::move(p_error));
  return true;
}

void PresentationService_StartPresentation_ProxyToResponder::Run(
    const base::Optional<content::PresentationInfo>& in_presentation_info, const base::Optional<content::PresentationError>& in_error) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_StartPresentation_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationErrorDataView>(
      in_error, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_StartPresentation_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_StartPresentation_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_info)::BaseType* presentation_info_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, buffer, &presentation_info_ptr, &serialization_context);
  params->presentation_info.Set(presentation_info_ptr);
  typename decltype(params->error)::BaseType* error_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationErrorDataView>(
      in_error, buffer, &error_ptr, &serialization_context);
  params->error.Set(error_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class PresentationService_ReconnectPresentation_ProxyToResponder {
 public:
  static PresentationService::ReconnectPresentationCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<PresentationService_ReconnectPresentation_ProxyToResponder> proxy(
        new PresentationService_ReconnectPresentation_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&PresentationService_ReconnectPresentation_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~PresentationService_ReconnectPresentation_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "PresentationService::ReconnectPresentation() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  PresentationService_ReconnectPresentation_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      const base::Optional<content::PresentationInfo>& in_presentation_info, const base::Optional<content::PresentationError>& in_error);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(PresentationService_ReconnectPresentation_ProxyToResponder);
};

bool PresentationService_ReconnectPresentation_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::PresentationService_ReconnectPresentation_ResponseParams_Data* params =
      reinterpret_cast<
          internal::PresentationService_ReconnectPresentation_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  base::Optional<content::PresentationInfo> p_presentation_info{};
  base::Optional<content::PresentationError> p_error{};
  PresentationService_ReconnectPresentation_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadPresentationInfo(&p_presentation_info))
    success = false;
  if (!input_data_view.ReadError(&p_error))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "PresentationService::ReconnectPresentation response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_presentation_info), 
std::move(p_error));
  return true;
}

void PresentationService_ReconnectPresentation_ProxyToResponder::Run(
    const base::Optional<content::PresentationInfo>& in_presentation_info, const base::Optional<content::PresentationError>& in_error) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationService_ReconnectPresentation_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationErrorDataView>(
      in_error, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationService_ReconnectPresentation_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationService_ReconnectPresentation_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_info)::BaseType* presentation_info_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, buffer, &presentation_info_ptr, &serialization_context);
  params->presentation_info.Set(presentation_info_ptr);
  typename decltype(params->error)::BaseType* error_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationErrorDataView>(
      in_error, buffer, &error_ptr, &serialization_context);
  params->error.Set(error_ptr);

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool PresentationServiceStubDispatch::Accept(
    PresentationService* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPresentationService_SetClient_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationService_SetClient_Params_Data* params =
          reinterpret_cast<internal::PresentationService_SetClient_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      PresentationServiceClientPtr p_client{};
      PresentationService_SetClient_ParamsDataView input_data_view(params, &serialization_context);
      
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::SetClient deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetClient(
std::move(p_client));
      return true;
    }
    case internal::kPresentationService_SetDefaultPresentationUrls_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationService_SetDefaultPresentationUrls_Params_Data* params =
          reinterpret_cast<internal::PresentationService_SetDefaultPresentationUrls_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::vector<GURL> p_presentation_urls{};
      PresentationService_SetDefaultPresentationUrls_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationUrls(&p_presentation_urls))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::SetDefaultPresentationUrls deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetDefaultPresentationUrls(
std::move(p_presentation_urls));
      return true;
    }
    case internal::kPresentationService_ListenForScreenAvailability_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationService_ListenForScreenAvailability_Params_Data* params =
          reinterpret_cast<internal::PresentationService_ListenForScreenAvailability_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_availability_url{};
      PresentationService_ListenForScreenAvailability_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAvailabilityUrl(&p_availability_url))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::ListenForScreenAvailability deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ListenForScreenAvailability(
std::move(p_availability_url));
      return true;
    }
    case internal::kPresentationService_StopListeningForScreenAvailability_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationService_StopListeningForScreenAvailability_Params_Data* params =
          reinterpret_cast<internal::PresentationService_StopListeningForScreenAvailability_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_availability_url{};
      PresentationService_StopListeningForScreenAvailability_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadAvailabilityUrl(&p_availability_url))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::StopListeningForScreenAvailability deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->StopListeningForScreenAvailability(
std::move(p_availability_url));
      return true;
    }
    case internal::kPresentationService_StartPresentation_Name: {
      break;
    }
    case internal::kPresentationService_ReconnectPresentation_Name: {
      break;
    }
    case internal::kPresentationService_SetPresentationConnection_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationService_SetPresentationConnection_Params_Data* params =
          reinterpret_cast<internal::PresentationService_SetPresentationConnection_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      content::PresentationInfo p_presentation_info{};
      PresentationConnectionPtr p_controller_connection_ptr{};
      PresentationConnectionRequest p_receiver_connection_request{};
      PresentationService_SetPresentationConnection_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationInfo(&p_presentation_info))
        success = false;
      p_controller_connection_ptr =
          input_data_view.TakeControllerConnectionPtr<decltype(p_controller_connection_ptr)>();
      p_receiver_connection_request =
          input_data_view.TakeReceiverConnectionRequest<decltype(p_receiver_connection_request)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::SetPresentationConnection deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetPresentationConnection(
std::move(p_presentation_info), 
std::move(p_controller_connection_ptr), 
std::move(p_receiver_connection_request));
      return true;
    }
    case internal::kPresentationService_CloseConnection_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationService_CloseConnection_Params_Data* params =
          reinterpret_cast<internal::PresentationService_CloseConnection_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_presentation_url{};
      std::string p_presentation_id{};
      PresentationService_CloseConnection_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationUrl(&p_presentation_url))
        success = false;
      if (!input_data_view.ReadPresentationId(&p_presentation_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::CloseConnection deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CloseConnection(
std::move(p_presentation_url), 
std::move(p_presentation_id));
      return true;
    }
    case internal::kPresentationService_Terminate_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationService_Terminate_Params_Data* params =
          reinterpret_cast<internal::PresentationService_Terminate_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_presentation_url{};
      std::string p_presentation_id{};
      PresentationService_Terminate_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationUrl(&p_presentation_url))
        success = false;
      if (!input_data_view.ReadPresentationId(&p_presentation_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::Terminate deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Terminate(
std::move(p_presentation_url), 
std::move(p_presentation_id));
      return true;
    }
  }
  return false;
}

// static
bool PresentationServiceStubDispatch::AcceptWithResponder(
    PresentationService* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPresentationService_SetClient_Name: {
      break;
    }
    case internal::kPresentationService_SetDefaultPresentationUrls_Name: {
      break;
    }
    case internal::kPresentationService_ListenForScreenAvailability_Name: {
      break;
    }
    case internal::kPresentationService_StopListeningForScreenAvailability_Name: {
      break;
    }
    case internal::kPresentationService_StartPresentation_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PresentationService_StartPresentation_Params_Data* params =
          reinterpret_cast<
              internal::PresentationService_StartPresentation_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::vector<GURL> p_presentation_urls{};
      PresentationService_StartPresentation_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationUrls(&p_presentation_urls))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::StartPresentation deserializer");
        return false;
      }
      PresentationService::StartPresentationCallback callback =
          PresentationService_StartPresentation_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->StartPresentation(
std::move(p_presentation_urls), std::move(callback));
      return true;
    }
    case internal::kPresentationService_ReconnectPresentation_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::PresentationService_ReconnectPresentation_Params_Data* params =
          reinterpret_cast<
              internal::PresentationService_ReconnectPresentation_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::vector<GURL> p_presentation_urls{};
      base::Optional<std::string> p_presentation_id{};
      PresentationService_ReconnectPresentation_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationUrls(&p_presentation_urls))
        success = false;
      if (!input_data_view.ReadPresentationId(&p_presentation_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationService::ReconnectPresentation deserializer");
        return false;
      }
      PresentationService::ReconnectPresentationCallback callback =
          PresentationService_ReconnectPresentation_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ReconnectPresentation(
std::move(p_presentation_urls), 
std::move(p_presentation_id), std::move(callback));
      return true;
    }
    case internal::kPresentationService_SetPresentationConnection_Name: {
      break;
    }
    case internal::kPresentationService_CloseConnection_Name: {
      break;
    }
    case internal::kPresentationService_Terminate_Name: {
      break;
    }
  }
  return false;
}

bool PresentationServiceRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PresentationService RequestValidator");

  switch (message->header()->name) {
    case internal::kPresentationService_SetClient_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_SetClient_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_SetDefaultPresentationUrls_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_SetDefaultPresentationUrls_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_ListenForScreenAvailability_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_ListenForScreenAvailability_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_StopListeningForScreenAvailability_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_StopListeningForScreenAvailability_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_StartPresentation_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_StartPresentation_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_ReconnectPresentation_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_ReconnectPresentation_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_SetPresentationConnection_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_SetPresentationConnection_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_CloseConnection_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_CloseConnection_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_Terminate_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_Terminate_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool PresentationServiceResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PresentationService ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kPresentationService_StartPresentation_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_StartPresentation_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationService_ReconnectPresentation_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationService_ReconnectPresentation_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char PresentationServiceClient::Name_[] = "blink::mojom::PresentationServiceClient";

PresentationServiceClientProxy::PresentationServiceClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void PresentationServiceClientProxy::OnScreenAvailabilityUpdated(
    const GURL& in_url, ScreenAvailability in_availability) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data);
  size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
      in_url, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationServiceClient_OnScreenAvailabilityUpdated_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->url)::BaseType* url_ptr;
  mojo::internal::Serialize<::url::mojom::UrlDataView>(
      in_url, buffer, &url_ptr, &serialization_context);
  params->url.Set(url_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->url.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null url in PresentationServiceClient.OnScreenAvailabilityUpdated request");
  mojo::internal::Serialize<::blink::mojom::ScreenAvailability>(
      in_availability, &params->availability);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceClientProxy::OnDefaultPresentationStarted(
    const content::PresentationInfo& in_presentation_info) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationServiceClient_OnDefaultPresentationStarted_Params_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationServiceClient_OnDefaultPresentationStarted_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationServiceClient_OnDefaultPresentationStarted_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_info)::BaseType* presentation_info_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, buffer, &presentation_info_ptr, &serialization_context);
  params->presentation_info.Set(presentation_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_info in PresentationServiceClient.OnDefaultPresentationStarted request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceClientProxy::OnReceiverConnectionAvailable(
    const content::PresentationInfo& in_presentation_info, PresentationConnectionPtr in_controller_connection_ptr, PresentationConnectionRequest in_receiver_connection_request) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationConnectionPtrDataView>(
      in_controller_connection_ptr, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationConnectionRequestDataView>(
      in_receiver_connection_request, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationServiceClient_OnReceiverConnectionAvailable_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_info)::BaseType* presentation_info_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, buffer, &presentation_info_ptr, &serialization_context);
  params->presentation_info.Set(presentation_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_info in PresentationServiceClient.OnReceiverConnectionAvailable request");
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionPtrDataView>(
      in_controller_connection_ptr, &params->controller_connection_ptr, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->controller_connection_ptr),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid controller_connection_ptr in PresentationServiceClient.OnReceiverConnectionAvailable request");
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionRequestDataView>(
      in_receiver_connection_request, &params->receiver_connection_request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->receiver_connection_request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid receiver_connection_request in PresentationServiceClient.OnReceiverConnectionAvailable request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceClientProxy::OnConnectionStateChanged(
    const content::PresentationInfo& in_presentation_info, content::PresentationConnectionState in_newState) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationServiceClient_OnConnectionStateChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationServiceClient_OnConnectionStateChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationServiceClient_OnConnectionStateChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_info)::BaseType* presentation_info_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, buffer, &presentation_info_ptr, &serialization_context);
  params->presentation_info.Set(presentation_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_info in PresentationServiceClient.OnConnectionStateChanged request");
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionState>(
      in_newState, &params->newState);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void PresentationServiceClientProxy::OnConnectionClosed(
    const content::PresentationInfo& in_presentation_info, content::PresentationConnectionCloseReason in_reason, const std::string& in_message) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::blink::mojom::internal::PresentationServiceClient_OnConnectionClosed_Params_Data);
  size += mojo::internal::PrepareToSerialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_message, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kPresentationServiceClient_OnConnectionClosed_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::blink::mojom::internal::PresentationServiceClient_OnConnectionClosed_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->presentation_info)::BaseType* presentation_info_ptr;
  mojo::internal::Serialize<::blink::mojom::PresentationInfoDataView>(
      in_presentation_info, buffer, &presentation_info_ptr, &serialization_context);
  params->presentation_info.Set(presentation_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->presentation_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null presentation_info in PresentationServiceClient.OnConnectionClosed request");
  mojo::internal::Serialize<::blink::mojom::PresentationConnectionCloseReason>(
      in_reason, &params->reason);
  typename decltype(params->message)::BaseType* message_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_message, buffer, &message_ptr, &serialization_context);
  params->message.Set(message_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->message.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null message in PresentationServiceClient.OnConnectionClosed request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool PresentationServiceClientStubDispatch::Accept(
    PresentationServiceClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kPresentationServiceClient_OnScreenAvailabilityUpdated_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data* params =
          reinterpret_cast<internal::PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      GURL p_url{};
      ScreenAvailability p_availability{};
      PresentationServiceClient_OnScreenAvailabilityUpdated_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadUrl(&p_url))
        success = false;
      if (!input_data_view.ReadAvailability(&p_availability))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationServiceClient::OnScreenAvailabilityUpdated deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnScreenAvailabilityUpdated(
std::move(p_url), 
std::move(p_availability));
      return true;
    }
    case internal::kPresentationServiceClient_OnDefaultPresentationStarted_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationServiceClient_OnDefaultPresentationStarted_Params_Data* params =
          reinterpret_cast<internal::PresentationServiceClient_OnDefaultPresentationStarted_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      content::PresentationInfo p_presentation_info{};
      PresentationServiceClient_OnDefaultPresentationStarted_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationInfo(&p_presentation_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationServiceClient::OnDefaultPresentationStarted deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDefaultPresentationStarted(
std::move(p_presentation_info));
      return true;
    }
    case internal::kPresentationServiceClient_OnReceiverConnectionAvailable_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data* params =
          reinterpret_cast<internal::PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      content::PresentationInfo p_presentation_info{};
      PresentationConnectionPtr p_controller_connection_ptr{};
      PresentationConnectionRequest p_receiver_connection_request{};
      PresentationServiceClient_OnReceiverConnectionAvailable_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationInfo(&p_presentation_info))
        success = false;
      p_controller_connection_ptr =
          input_data_view.TakeControllerConnectionPtr<decltype(p_controller_connection_ptr)>();
      p_receiver_connection_request =
          input_data_view.TakeReceiverConnectionRequest<decltype(p_receiver_connection_request)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationServiceClient::OnReceiverConnectionAvailable deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnReceiverConnectionAvailable(
std::move(p_presentation_info), 
std::move(p_controller_connection_ptr), 
std::move(p_receiver_connection_request));
      return true;
    }
    case internal::kPresentationServiceClient_OnConnectionStateChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationServiceClient_OnConnectionStateChanged_Params_Data* params =
          reinterpret_cast<internal::PresentationServiceClient_OnConnectionStateChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      content::PresentationInfo p_presentation_info{};
      content::PresentationConnectionState p_newState{};
      PresentationServiceClient_OnConnectionStateChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationInfo(&p_presentation_info))
        success = false;
      if (!input_data_view.ReadNewState(&p_newState))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationServiceClient::OnConnectionStateChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnConnectionStateChanged(
std::move(p_presentation_info), 
std::move(p_newState));
      return true;
    }
    case internal::kPresentationServiceClient_OnConnectionClosed_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::PresentationServiceClient_OnConnectionClosed_Params_Data* params =
          reinterpret_cast<internal::PresentationServiceClient_OnConnectionClosed_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      content::PresentationInfo p_presentation_info{};
      content::PresentationConnectionCloseReason p_reason{};
      std::string p_message{};
      PresentationServiceClient_OnConnectionClosed_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadPresentationInfo(&p_presentation_info))
        success = false;
      if (!input_data_view.ReadReason(&p_reason))
        success = false;
      if (!input_data_view.ReadMessage(&p_message))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "PresentationServiceClient::OnConnectionClosed deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnConnectionClosed(
std::move(p_presentation_info), 
std::move(p_reason), 
std::move(p_message));
      return true;
    }
  }
  return false;
}

// static
bool PresentationServiceClientStubDispatch::AcceptWithResponder(
    PresentationServiceClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kPresentationServiceClient_OnScreenAvailabilityUpdated_Name: {
      break;
    }
    case internal::kPresentationServiceClient_OnDefaultPresentationStarted_Name: {
      break;
    }
    case internal::kPresentationServiceClient_OnReceiverConnectionAvailable_Name: {
      break;
    }
    case internal::kPresentationServiceClient_OnConnectionStateChanged_Name: {
      break;
    }
    case internal::kPresentationServiceClient_OnConnectionClosed_Name: {
      break;
    }
  }
  return false;
}

bool PresentationServiceClientRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "PresentationServiceClient RequestValidator");

  switch (message->header()->name) {
    case internal::kPresentationServiceClient_OnScreenAvailabilityUpdated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationServiceClient_OnScreenAvailabilityUpdated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationServiceClient_OnDefaultPresentationStarted_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationServiceClient_OnDefaultPresentationStarted_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationServiceClient_OnReceiverConnectionAvailable_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationServiceClient_OnReceiverConnectionAvailable_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationServiceClient_OnConnectionStateChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationServiceClient_OnConnectionStateChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kPresentationServiceClient_OnConnectionClosed_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::PresentationServiceClient_OnConnectionClosed_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace blink

namespace mojo {


// static
bool StructTraits<::blink::mojom::PresentationInfo::DataView, ::blink::mojom::PresentationInfoPtr>::Read(
    ::blink::mojom::PresentationInfo::DataView input,
    ::blink::mojom::PresentationInfoPtr* output) {
  bool success = true;
  ::blink::mojom::PresentationInfoPtr result(::blink::mojom::PresentationInfo::New());
  
      if (!input.ReadUrl(&result->url))
        success = false;
      if (!input.ReadId(&result->id))
        success = false;
  *output = std::move(result);
  return success;
}


// static
bool StructTraits<::blink::mojom::PresentationError::DataView, ::blink::mojom::PresentationErrorPtr>::Read(
    ::blink::mojom::PresentationError::DataView input,
    ::blink::mojom::PresentationErrorPtr* output) {
  bool success = true;
  ::blink::mojom::PresentationErrorPtr result(::blink::mojom::PresentationError::New());
  
      if (!input.ReadErrorType(&result->error_type))
        success = false;
      if (!input.ReadMessage(&result->message))
        success = false;
  *output = std::move(result);
  return success;
}

// static
bool UnionTraits<::blink::mojom::PresentationConnectionMessage::DataView, ::blink::mojom::PresentationConnectionMessagePtr>::Read(
    ::blink::mojom::PresentationConnectionMessage::DataView input,
    ::blink::mojom::PresentationConnectionMessagePtr* output) {
  using UnionType = ::blink::mojom::PresentationConnectionMessage;
  using Tag = UnionType::Tag;

  switch (input.tag()) {
    case Tag::MESSAGE: {
      std::string result_message;
      if (!input.ReadMessage(&result_message))
        return false;

      *output = UnionType::NewMessage(
          std::move(result_message));
      break;
    }
    case Tag::DATA: {
      std::vector<uint8_t> result_data;
      if (!input.ReadData(&result_data))
        return false;

      *output = UnionType::NewData(
          std::move(result_data));
      break;
    }
    default:
      return false;
  }
  return true;
}

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif