// Copyright 2016 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBAUTH_AUTHENTICATOR_MOJOM_SHARED_H_
#define THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBAUTH_AUTHENTICATOR_MOJOM_SHARED_H_

#include <stdint.h>

#include <functional>
#include <ostream>
#include <type_traits>
#include <utility>

#include "base/compiler_specific.h"
#include "mojo/public/cpp/bindings/array_data_view.h"
#include "mojo/public/cpp/bindings/enum_traits.h"
#include "mojo/public/cpp/bindings/interface_data_view.h"
#include "mojo/public/cpp/bindings/lib/bindings_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/map_data_view.h"
#include "mojo/public/cpp/bindings/native_enum.h"
#include "mojo/public/cpp/bindings/native_struct_data_view.h"
#include "mojo/public/cpp/bindings/string_data_view.h"
#include "third_party/WebKit/public/platform/modules/webauth/authenticator.mojom-shared-internal.h"
#include "mojo/common/time.mojom-shared.h"
#include "url/mojo/url.mojom-shared.h"
#include "third_party/WebKit/public/mojo_bindings_shared_export.h"


namespace webauth {
namespace mojom {
class PublicKeyCredentialInfoDataView;

class AuthenticatorResponseDataView;

class PublicKeyCredentialEntityDataView;

class PublicKeyCredentialParametersDataView;

class MakeCredentialOptionsDataView;

class PublicKeyCredentialDescriptorDataView;



}  // namespace mojom
}  // namespace webauth

namespace mojo {
namespace internal {

template <>
struct MojomTypeTraits<::webauth::mojom::PublicKeyCredentialInfoDataView> {
  using Data = ::webauth::mojom::internal::PublicKeyCredentialInfo_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::webauth::mojom::AuthenticatorResponseDataView> {
  using Data = ::webauth::mojom::internal::AuthenticatorResponse_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::webauth::mojom::PublicKeyCredentialEntityDataView> {
  using Data = ::webauth::mojom::internal::PublicKeyCredentialEntity_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::webauth::mojom::PublicKeyCredentialParametersDataView> {
  using Data = ::webauth::mojom::internal::PublicKeyCredentialParameters_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::webauth::mojom::MakeCredentialOptionsDataView> {
  using Data = ::webauth::mojom::internal::MakeCredentialOptions_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

template <>
struct MojomTypeTraits<::webauth::mojom::PublicKeyCredentialDescriptorDataView> {
  using Data = ::webauth::mojom::internal::PublicKeyCredentialDescriptor_Data;
  using DataAsArrayElement = Pointer<Data>;
  static constexpr MojomTypeCategory category = MojomTypeCategory::STRUCT;
};

}  // namespace internal
}  // namespace mojo


namespace webauth {
namespace mojom {

enum class AuthenticatorStatus : int32_t {
  SUCCESS,
  CANCELLED,
  UNKNOWN_ERROR,
  NOT_ALLOWED_ERROR,
  NOT_SUPPORTED_ERROR,
  SECURITY_ERROR,
  NOT_IMPLEMENTED,
};

inline std::ostream& operator<<(std::ostream& os, AuthenticatorStatus value) {
  switch(value) {
    case AuthenticatorStatus::SUCCESS:
      return os << "AuthenticatorStatus::SUCCESS";
    case AuthenticatorStatus::CANCELLED:
      return os << "AuthenticatorStatus::CANCELLED";
    case AuthenticatorStatus::UNKNOWN_ERROR:
      return os << "AuthenticatorStatus::UNKNOWN_ERROR";
    case AuthenticatorStatus::NOT_ALLOWED_ERROR:
      return os << "AuthenticatorStatus::NOT_ALLOWED_ERROR";
    case AuthenticatorStatus::NOT_SUPPORTED_ERROR:
      return os << "AuthenticatorStatus::NOT_SUPPORTED_ERROR";
    case AuthenticatorStatus::SECURITY_ERROR:
      return os << "AuthenticatorStatus::SECURITY_ERROR";
    case AuthenticatorStatus::NOT_IMPLEMENTED:
      return os << "AuthenticatorStatus::NOT_IMPLEMENTED";
    default:
      return os << "Unknown AuthenticatorStatus value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AuthenticatorStatus value) {
  return internal::AuthenticatorStatus_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class PublicKeyCredentialType : int32_t {
  PUBLIC_KEY,
};

inline std::ostream& operator<<(std::ostream& os, PublicKeyCredentialType value) {
  switch(value) {
    case PublicKeyCredentialType::PUBLIC_KEY:
      return os << "PublicKeyCredentialType::PUBLIC_KEY";
    default:
      return os << "Unknown PublicKeyCredentialType value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(PublicKeyCredentialType value) {
  return internal::PublicKeyCredentialType_Data::IsKnownValue(
      static_cast<int32_t>(value));
}

enum class AuthenticatorTransport : int32_t {
  USB,
  NFC,
  BLE,
};

inline std::ostream& operator<<(std::ostream& os, AuthenticatorTransport value) {
  switch(value) {
    case AuthenticatorTransport::USB:
      return os << "AuthenticatorTransport::USB";
    case AuthenticatorTransport::NFC:
      return os << "AuthenticatorTransport::NFC";
    case AuthenticatorTransport::BLE:
      return os << "AuthenticatorTransport::BLE";
    default:
      return os << "Unknown AuthenticatorTransport value: " << static_cast<int32_t>(value);
  }
}
inline bool IsKnownEnumValue(AuthenticatorTransport value) {
  return internal::AuthenticatorTransport_Data::IsKnownValue(
      static_cast<int32_t>(value));
}
// Interface base classes. They are used for type safety check.
class AuthenticatorInterfaceBase {};

using AuthenticatorPtrDataView =
    mojo::InterfacePtrDataView<AuthenticatorInterfaceBase>;
using AuthenticatorRequestDataView =
    mojo::InterfaceRequestDataView<AuthenticatorInterfaceBase>;
using AuthenticatorAssociatedPtrInfoDataView =
    mojo::AssociatedInterfacePtrInfoDataView<AuthenticatorInterfaceBase>;
using AuthenticatorAssociatedRequestDataView =
    mojo::AssociatedInterfaceRequestDataView<AuthenticatorInterfaceBase>;
class PublicKeyCredentialInfoDataView {
 public:
  PublicKeyCredentialInfoDataView() {}

  PublicKeyCredentialInfoDataView(
      internal::PublicKeyCredentialInfo_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetRawIdDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRawId(UserType* output) {
    auto* pointer = data_->raw_id.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  inline void GetClientDataJsonDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadClientDataJson(UserType* output) {
    auto* pointer = data_->client_data_json.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  inline void GetResponseDataView(
      AuthenticatorResponseDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadResponse(UserType* output) {
    auto* pointer = data_->response.Get();
    return mojo::internal::Deserialize<::webauth::mojom::AuthenticatorResponseDataView>(
        pointer, output, context_);
  }
 private:
  internal::PublicKeyCredentialInfo_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class AuthenticatorResponseDataView {
 public:
  AuthenticatorResponseDataView() {}

  AuthenticatorResponseDataView(
      internal::AuthenticatorResponse_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetAttestationObjectDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAttestationObject(UserType* output) {
    auto* pointer = data_->attestation_object.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  inline void GetAuthenticatorDataDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAuthenticatorData(UserType* output) {
    auto* pointer = data_->authenticator_data.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  inline void GetSignatureDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadSignature(UserType* output) {
    auto* pointer = data_->signature.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
 private:
  internal::AuthenticatorResponse_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PublicKeyCredentialEntityDataView {
 public:
  PublicKeyCredentialEntityDataView() {}

  PublicKeyCredentialEntityDataView(
      internal::PublicKeyCredentialEntity_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetIdDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadName(UserType* output) {
    auto* pointer = data_->name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
  inline void GetIconDataView(
      ::url::mojom::UrlDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadIcon(UserType* output) {
    auto* pointer = data_->icon.Get();
    return mojo::internal::Deserialize<::url::mojom::UrlDataView>(
        pointer, output, context_);
  }
  inline void GetDisplayNameDataView(
      mojo::StringDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadDisplayName(UserType* output) {
    auto* pointer = data_->display_name.Get();
    return mojo::internal::Deserialize<mojo::StringDataView>(
        pointer, output, context_);
  }
 private:
  internal::PublicKeyCredentialEntity_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PublicKeyCredentialParametersDataView {
 public:
  PublicKeyCredentialParametersDataView() {}

  PublicKeyCredentialParametersDataView(
      internal::PublicKeyCredentialParameters_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::webauth::mojom::PublicKeyCredentialType>(
        data_value, output);
  }

  PublicKeyCredentialType type() const {
    return static_cast<PublicKeyCredentialType>(data_->type);
  }
 private:
  internal::PublicKeyCredentialParameters_Data* data_ = nullptr;
};

class MakeCredentialOptionsDataView {
 public:
  MakeCredentialOptionsDataView() {}

  MakeCredentialOptionsDataView(
      internal::MakeCredentialOptions_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetRelyingPartyDataView(
      PublicKeyCredentialEntityDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadRelyingParty(UserType* output) {
    auto* pointer = data_->relying_party.Get();
    return mojo::internal::Deserialize<::webauth::mojom::PublicKeyCredentialEntityDataView>(
        pointer, output, context_);
  }
  inline void GetUserDataView(
      PublicKeyCredentialEntityDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadUser(UserType* output) {
    auto* pointer = data_->user.Get();
    return mojo::internal::Deserialize<::webauth::mojom::PublicKeyCredentialEntityDataView>(
        pointer, output, context_);
  }
  inline void GetChallengeDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadChallenge(UserType* output) {
    auto* pointer = data_->challenge.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  inline void GetCryptoParametersDataView(
      mojo::ArrayDataView<PublicKeyCredentialParametersDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCryptoParameters(UserType* output) {
    auto* pointer = data_->crypto_parameters.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::webauth::mojom::PublicKeyCredentialParametersDataView>>(
        pointer, output, context_);
  }
  inline void GetAdjustedTimeoutDataView(
      ::mojo::common::mojom::TimeDeltaDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadAdjustedTimeout(UserType* output) {
    auto* pointer = data_->adjusted_timeout.Get();
    return mojo::internal::Deserialize<::mojo::common::mojom::TimeDeltaDataView>(
        pointer, output, context_);
  }
  inline void GetExcludeCredentialsDataView(
      mojo::ArrayDataView<PublicKeyCredentialDescriptorDataView>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadExcludeCredentials(UserType* output) {
    auto* pointer = data_->exclude_credentials.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::webauth::mojom::PublicKeyCredentialDescriptorDataView>>(
        pointer, output, context_);
  }
 private:
  internal::MakeCredentialOptions_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class PublicKeyCredentialDescriptorDataView {
 public:
  PublicKeyCredentialDescriptorDataView() {}

  PublicKeyCredentialDescriptorDataView(
      internal::PublicKeyCredentialDescriptor_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadType(UserType* output) const {
    auto data_value = data_->type;
    return mojo::internal::Deserialize<::webauth::mojom::PublicKeyCredentialType>(
        data_value, output);
  }

  PublicKeyCredentialType type() const {
    return static_cast<PublicKeyCredentialType>(data_->type);
  }
  inline void GetIdDataView(
      mojo::ArrayDataView<uint8_t>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadId(UserType* output) {
    auto* pointer = data_->id.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<uint8_t>>(
        pointer, output, context_);
  }
  inline void GetTransportsDataView(
      mojo::ArrayDataView<AuthenticatorTransport>* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadTransports(UserType* output) {
    auto* pointer = data_->transports.Get();
    return mojo::internal::Deserialize<mojo::ArrayDataView<::webauth::mojom::AuthenticatorTransport>>(
        pointer, output, context_);
  }
 private:
  internal::PublicKeyCredentialDescriptor_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Authenticator_MakeCredential_ParamsDataView {
 public:
  Authenticator_MakeCredential_ParamsDataView() {}

  Authenticator_MakeCredential_ParamsDataView(
      internal::Authenticator_MakeCredential_Params_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  inline void GetOptionsDataView(
      MakeCredentialOptionsDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadOptions(UserType* output) {
    auto* pointer = data_->options.Get();
    return mojo::internal::Deserialize<::webauth::mojom::MakeCredentialOptionsDataView>(
        pointer, output, context_);
  }
 private:
  internal::Authenticator_MakeCredential_Params_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};

class Authenticator_MakeCredential_ResponseParamsDataView {
 public:
  Authenticator_MakeCredential_ResponseParamsDataView() {}

  Authenticator_MakeCredential_ResponseParamsDataView(
      internal::Authenticator_MakeCredential_ResponseParams_Data* data,
      mojo::internal::SerializationContext* context)
      : data_(data), context_(context) {}

  bool is_null() const { return !data_; }
  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadStatus(UserType* output) const {
    auto data_value = data_->status;
    return mojo::internal::Deserialize<::webauth::mojom::AuthenticatorStatus>(
        data_value, output);
  }

  AuthenticatorStatus status() const {
    return static_cast<AuthenticatorStatus>(data_->status);
  }
  inline void GetCredentialDataView(
      PublicKeyCredentialInfoDataView* output);

  template <typename UserType>
  WARN_UNUSED_RESULT bool ReadCredential(UserType* output) {
    auto* pointer = data_->credential.Get();
    return mojo::internal::Deserialize<::webauth::mojom::PublicKeyCredentialInfoDataView>(
        pointer, output, context_);
  }
 private:
  internal::Authenticator_MakeCredential_ResponseParams_Data* data_ = nullptr;
  mojo::internal::SerializationContext* context_ = nullptr;
};



}  // namespace mojom
}  // namespace webauth

namespace std {

template <>
struct hash<::webauth::mojom::AuthenticatorStatus>
    : public mojo::internal::EnumHashImpl<::webauth::mojom::AuthenticatorStatus> {};

template <>
struct hash<::webauth::mojom::PublicKeyCredentialType>
    : public mojo::internal::EnumHashImpl<::webauth::mojom::PublicKeyCredentialType> {};

template <>
struct hash<::webauth::mojom::AuthenticatorTransport>
    : public mojo::internal::EnumHashImpl<::webauth::mojom::AuthenticatorTransport> {};

}  // namespace std

namespace mojo {


template <>
struct EnumTraits<::webauth::mojom::AuthenticatorStatus, ::webauth::mojom::AuthenticatorStatus> {
  static ::webauth::mojom::AuthenticatorStatus ToMojom(::webauth::mojom::AuthenticatorStatus input) { return input; }
  static bool FromMojom(::webauth::mojom::AuthenticatorStatus input, ::webauth::mojom::AuthenticatorStatus* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::AuthenticatorStatus, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::webauth::mojom::AuthenticatorStatus, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::webauth::mojom::AuthenticatorStatus>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::webauth::mojom::PublicKeyCredentialType, ::webauth::mojom::PublicKeyCredentialType> {
  static ::webauth::mojom::PublicKeyCredentialType ToMojom(::webauth::mojom::PublicKeyCredentialType input) { return input; }
  static bool FromMojom(::webauth::mojom::PublicKeyCredentialType input, ::webauth::mojom::PublicKeyCredentialType* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::PublicKeyCredentialType, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::webauth::mojom::PublicKeyCredentialType, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::webauth::mojom::PublicKeyCredentialType>(input), output);
  }
};

}  // namespace internal


template <>
struct EnumTraits<::webauth::mojom::AuthenticatorTransport, ::webauth::mojom::AuthenticatorTransport> {
  static ::webauth::mojom::AuthenticatorTransport ToMojom(::webauth::mojom::AuthenticatorTransport input) { return input; }
  static bool FromMojom(::webauth::mojom::AuthenticatorTransport input, ::webauth::mojom::AuthenticatorTransport* output) {
    *output = input;
    return true;
  }
};

namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::AuthenticatorTransport, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = EnumTraits<::webauth::mojom::AuthenticatorTransport, UserType>;

  static void Serialize(UserType input, int32_t* output) {
    *output = static_cast<int32_t>(Traits::ToMojom(input));
  }

  static bool Deserialize(int32_t input, UserType* output) {
    return Traits::FromMojom(static_cast<::webauth::mojom::AuthenticatorTransport>(input), output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::PublicKeyCredentialInfoDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::webauth::mojom::PublicKeyCredentialInfoDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::webauth::mojom::internal::PublicKeyCredentialInfo_Data);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_id, context);
    decltype(CallWithContext(Traits::raw_id, input, custom_context)) in_raw_id = CallWithContext(Traits::raw_id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_raw_id, context);
    decltype(CallWithContext(Traits::client_data_json, input, custom_context)) in_client_data_json = CallWithContext(Traits::client_data_json, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_client_data_json, context);
    decltype(CallWithContext(Traits::response, input, custom_context)) in_response = CallWithContext(Traits::response, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::webauth::mojom::AuthenticatorResponseDataView>(
        in_response, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::webauth::mojom::internal::PublicKeyCredentialInfo_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::webauth::mojom::internal::PublicKeyCredentialInfo_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    typename decltype(result->id)::BaseType* id_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_id, buffer, &id_ptr, context);
    result->id.Set(id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null id in PublicKeyCredentialInfo struct");
    decltype(CallWithContext(Traits::raw_id, input, custom_context)) in_raw_id = CallWithContext(Traits::raw_id, input, custom_context);
    typename decltype(result->raw_id)::BaseType* raw_id_ptr;
    const mojo::internal::ContainerValidateParams raw_id_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_raw_id, buffer, &raw_id_ptr, &raw_id_validate_params,
        context);
    result->raw_id.Set(raw_id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->raw_id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null raw_id in PublicKeyCredentialInfo struct");
    decltype(CallWithContext(Traits::client_data_json, input, custom_context)) in_client_data_json = CallWithContext(Traits::client_data_json, input, custom_context);
    typename decltype(result->client_data_json)::BaseType* client_data_json_ptr;
    const mojo::internal::ContainerValidateParams client_data_json_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_client_data_json, buffer, &client_data_json_ptr, &client_data_json_validate_params,
        context);
    result->client_data_json.Set(client_data_json_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->client_data_json.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null client_data_json in PublicKeyCredentialInfo struct");
    decltype(CallWithContext(Traits::response, input, custom_context)) in_response = CallWithContext(Traits::response, input, custom_context);
    typename decltype(result->response)::BaseType* response_ptr;
    mojo::internal::Serialize<::webauth::mojom::AuthenticatorResponseDataView>(
        in_response, buffer, &response_ptr, context);
    result->response.Set(response_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->response.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null response in PublicKeyCredentialInfo struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::webauth::mojom::internal::PublicKeyCredentialInfo_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::webauth::mojom::PublicKeyCredentialInfoDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::AuthenticatorResponseDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::webauth::mojom::AuthenticatorResponseDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::webauth::mojom::internal::AuthenticatorResponse_Data);
    decltype(CallWithContext(Traits::attestation_object, input, custom_context)) in_attestation_object = CallWithContext(Traits::attestation_object, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_attestation_object, context);
    decltype(CallWithContext(Traits::authenticator_data, input, custom_context)) in_authenticator_data = CallWithContext(Traits::authenticator_data, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_authenticator_data, context);
    decltype(CallWithContext(Traits::signature, input, custom_context)) in_signature = CallWithContext(Traits::signature, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_signature, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::webauth::mojom::internal::AuthenticatorResponse_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::webauth::mojom::internal::AuthenticatorResponse_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::attestation_object, input, custom_context)) in_attestation_object = CallWithContext(Traits::attestation_object, input, custom_context);
    typename decltype(result->attestation_object)::BaseType* attestation_object_ptr;
    const mojo::internal::ContainerValidateParams attestation_object_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_attestation_object, buffer, &attestation_object_ptr, &attestation_object_validate_params,
        context);
    result->attestation_object.Set(attestation_object_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->attestation_object.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null attestation_object in AuthenticatorResponse struct");
    decltype(CallWithContext(Traits::authenticator_data, input, custom_context)) in_authenticator_data = CallWithContext(Traits::authenticator_data, input, custom_context);
    typename decltype(result->authenticator_data)::BaseType* authenticator_data_ptr;
    const mojo::internal::ContainerValidateParams authenticator_data_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_authenticator_data, buffer, &authenticator_data_ptr, &authenticator_data_validate_params,
        context);
    result->authenticator_data.Set(authenticator_data_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->authenticator_data.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null authenticator_data in AuthenticatorResponse struct");
    decltype(CallWithContext(Traits::signature, input, custom_context)) in_signature = CallWithContext(Traits::signature, input, custom_context);
    typename decltype(result->signature)::BaseType* signature_ptr;
    const mojo::internal::ContainerValidateParams signature_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_signature, buffer, &signature_ptr, &signature_validate_params,
        context);
    result->signature.Set(signature_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->signature.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null signature in AuthenticatorResponse struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::webauth::mojom::internal::AuthenticatorResponse_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::webauth::mojom::AuthenticatorResponseDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::PublicKeyCredentialEntityDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::webauth::mojom::PublicKeyCredentialEntityDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::webauth::mojom::internal::PublicKeyCredentialEntity_Data);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_id, context);
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_name, context);
    decltype(CallWithContext(Traits::icon, input, custom_context)) in_icon = CallWithContext(Traits::icon, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::url::mojom::UrlDataView>(
        in_icon, context);
    decltype(CallWithContext(Traits::display_name, input, custom_context)) in_display_name = CallWithContext(Traits::display_name, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
        in_display_name, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::webauth::mojom::internal::PublicKeyCredentialEntity_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::webauth::mojom::internal::PublicKeyCredentialEntity_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    typename decltype(result->id)::BaseType* id_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_id, buffer, &id_ptr, context);
    result->id.Set(id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null id in PublicKeyCredentialEntity struct");
    decltype(CallWithContext(Traits::name, input, custom_context)) in_name = CallWithContext(Traits::name, input, custom_context);
    typename decltype(result->name)::BaseType* name_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_name, buffer, &name_ptr, context);
    result->name.Set(name_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->name.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null name in PublicKeyCredentialEntity struct");
    decltype(CallWithContext(Traits::icon, input, custom_context)) in_icon = CallWithContext(Traits::icon, input, custom_context);
    typename decltype(result->icon)::BaseType* icon_ptr;
    mojo::internal::Serialize<::url::mojom::UrlDataView>(
        in_icon, buffer, &icon_ptr, context);
    result->icon.Set(icon_ptr);
    decltype(CallWithContext(Traits::display_name, input, custom_context)) in_display_name = CallWithContext(Traits::display_name, input, custom_context);
    typename decltype(result->display_name)::BaseType* display_name_ptr;
    mojo::internal::Serialize<mojo::StringDataView>(
        in_display_name, buffer, &display_name_ptr, context);
    result->display_name.Set(display_name_ptr);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::webauth::mojom::internal::PublicKeyCredentialEntity_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::webauth::mojom::PublicKeyCredentialEntityDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::PublicKeyCredentialParametersDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::webauth::mojom::PublicKeyCredentialParametersDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::webauth::mojom::internal::PublicKeyCredentialParameters_Data);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::webauth::mojom::internal::PublicKeyCredentialParameters_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::webauth::mojom::internal::PublicKeyCredentialParameters_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    mojo::internal::Serialize<::webauth::mojom::PublicKeyCredentialType>(
        CallWithContext(Traits::type, input, custom_context), &result->type);
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::webauth::mojom::internal::PublicKeyCredentialParameters_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::webauth::mojom::PublicKeyCredentialParametersDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::MakeCredentialOptionsDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::webauth::mojom::MakeCredentialOptionsDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::webauth::mojom::internal::MakeCredentialOptions_Data);
    decltype(CallWithContext(Traits::relying_party, input, custom_context)) in_relying_party = CallWithContext(Traits::relying_party, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::webauth::mojom::PublicKeyCredentialEntityDataView>(
        in_relying_party, context);
    decltype(CallWithContext(Traits::user, input, custom_context)) in_user = CallWithContext(Traits::user, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::webauth::mojom::PublicKeyCredentialEntityDataView>(
        in_user, context);
    decltype(CallWithContext(Traits::challenge, input, custom_context)) in_challenge = CallWithContext(Traits::challenge, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_challenge, context);
    decltype(CallWithContext(Traits::crypto_parameters, input, custom_context)) in_crypto_parameters = CallWithContext(Traits::crypto_parameters, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::webauth::mojom::PublicKeyCredentialParametersDataView>>(
        in_crypto_parameters, context);
    decltype(CallWithContext(Traits::adjusted_timeout, input, custom_context)) in_adjusted_timeout = CallWithContext(Traits::adjusted_timeout, input, custom_context);
    size += mojo::internal::PrepareToSerialize<::mojo::common::mojom::TimeDeltaDataView>(
        in_adjusted_timeout, context);
    decltype(CallWithContext(Traits::exclude_credentials, input, custom_context)) in_exclude_credentials = CallWithContext(Traits::exclude_credentials, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::webauth::mojom::PublicKeyCredentialDescriptorDataView>>(
        in_exclude_credentials, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::webauth::mojom::internal::MakeCredentialOptions_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::webauth::mojom::internal::MakeCredentialOptions_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    decltype(CallWithContext(Traits::relying_party, input, custom_context)) in_relying_party = CallWithContext(Traits::relying_party, input, custom_context);
    typename decltype(result->relying_party)::BaseType* relying_party_ptr;
    mojo::internal::Serialize<::webauth::mojom::PublicKeyCredentialEntityDataView>(
        in_relying_party, buffer, &relying_party_ptr, context);
    result->relying_party.Set(relying_party_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->relying_party.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null relying_party in MakeCredentialOptions struct");
    decltype(CallWithContext(Traits::user, input, custom_context)) in_user = CallWithContext(Traits::user, input, custom_context);
    typename decltype(result->user)::BaseType* user_ptr;
    mojo::internal::Serialize<::webauth::mojom::PublicKeyCredentialEntityDataView>(
        in_user, buffer, &user_ptr, context);
    result->user.Set(user_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->user.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null user in MakeCredentialOptions struct");
    decltype(CallWithContext(Traits::challenge, input, custom_context)) in_challenge = CallWithContext(Traits::challenge, input, custom_context);
    typename decltype(result->challenge)::BaseType* challenge_ptr;
    const mojo::internal::ContainerValidateParams challenge_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_challenge, buffer, &challenge_ptr, &challenge_validate_params,
        context);
    result->challenge.Set(challenge_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->challenge.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null challenge in MakeCredentialOptions struct");
    decltype(CallWithContext(Traits::crypto_parameters, input, custom_context)) in_crypto_parameters = CallWithContext(Traits::crypto_parameters, input, custom_context);
    typename decltype(result->crypto_parameters)::BaseType* crypto_parameters_ptr;
    const mojo::internal::ContainerValidateParams crypto_parameters_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::webauth::mojom::PublicKeyCredentialParametersDataView>>(
        in_crypto_parameters, buffer, &crypto_parameters_ptr, &crypto_parameters_validate_params,
        context);
    result->crypto_parameters.Set(crypto_parameters_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->crypto_parameters.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null crypto_parameters in MakeCredentialOptions struct");
    decltype(CallWithContext(Traits::adjusted_timeout, input, custom_context)) in_adjusted_timeout = CallWithContext(Traits::adjusted_timeout, input, custom_context);
    typename decltype(result->adjusted_timeout)::BaseType* adjusted_timeout_ptr;
    mojo::internal::Serialize<::mojo::common::mojom::TimeDeltaDataView>(
        in_adjusted_timeout, buffer, &adjusted_timeout_ptr, context);
    result->adjusted_timeout.Set(adjusted_timeout_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->adjusted_timeout.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null adjusted_timeout in MakeCredentialOptions struct");
    decltype(CallWithContext(Traits::exclude_credentials, input, custom_context)) in_exclude_credentials = CallWithContext(Traits::exclude_credentials, input, custom_context);
    typename decltype(result->exclude_credentials)::BaseType* exclude_credentials_ptr;
    const mojo::internal::ContainerValidateParams exclude_credentials_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<::webauth::mojom::PublicKeyCredentialDescriptorDataView>>(
        in_exclude_credentials, buffer, &exclude_credentials_ptr, &exclude_credentials_validate_params,
        context);
    result->exclude_credentials.Set(exclude_credentials_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->exclude_credentials.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null exclude_credentials in MakeCredentialOptions struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::webauth::mojom::internal::MakeCredentialOptions_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::webauth::mojom::MakeCredentialOptionsDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal


namespace internal {

template <typename MaybeConstUserType>
struct Serializer<::webauth::mojom::PublicKeyCredentialDescriptorDataView, MaybeConstUserType> {
  using UserType = typename std::remove_const<MaybeConstUserType>::type;
  using Traits = StructTraits<::webauth::mojom::PublicKeyCredentialDescriptorDataView, UserType>;

  static size_t PrepareToSerialize(MaybeConstUserType& input,
                                   SerializationContext* context) {
    const bool is_null = CallIsNullIfExists<Traits>(input);
    context->PushNextNullState(is_null);
    if (is_null)
      return 0;

    void* custom_context = CustomContextHelper<Traits>::SetUp(input, context);
    ALLOW_UNUSED_LOCAL(custom_context);

    size_t size = sizeof(::webauth::mojom::internal::PublicKeyCredentialDescriptor_Data);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
        in_id, context);
    decltype(CallWithContext(Traits::transports, input, custom_context)) in_transports = CallWithContext(Traits::transports, input, custom_context);
    size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::webauth::mojom::AuthenticatorTransport>>(
        in_transports, context);
    return size;
  }

  static void Serialize(MaybeConstUserType& input,
                        Buffer* buffer,
                        ::webauth::mojom::internal::PublicKeyCredentialDescriptor_Data** output,
                        SerializationContext* context) {
    if (context->IsNextFieldNull()) {
      *output = nullptr;
      return;
    }

    void* custom_context = CustomContextHelper<Traits>::GetNext(context);

    auto result =
        ::webauth::mojom::internal::PublicKeyCredentialDescriptor_Data::New(buffer);
    ALLOW_UNUSED_LOCAL(result);
    mojo::internal::Serialize<::webauth::mojom::PublicKeyCredentialType>(
        CallWithContext(Traits::type, input, custom_context), &result->type);
    decltype(CallWithContext(Traits::id, input, custom_context)) in_id = CallWithContext(Traits::id, input, custom_context);
    typename decltype(result->id)::BaseType* id_ptr;
    const mojo::internal::ContainerValidateParams id_validate_params(
        0, false, nullptr);
    mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
        in_id, buffer, &id_ptr, &id_validate_params,
        context);
    result->id.Set(id_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->id.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null id in PublicKeyCredentialDescriptor struct");
    decltype(CallWithContext(Traits::transports, input, custom_context)) in_transports = CallWithContext(Traits::transports, input, custom_context);
    typename decltype(result->transports)::BaseType* transports_ptr;
    const mojo::internal::ContainerValidateParams transports_validate_params(
        0, ::webauth::mojom::internal::AuthenticatorTransport_Data::Validate);
    mojo::internal::Serialize<mojo::ArrayDataView<::webauth::mojom::AuthenticatorTransport>>(
        in_transports, buffer, &transports_ptr, &transports_validate_params,
        context);
    result->transports.Set(transports_ptr);
    MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
        result->transports.is_null(),
        mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
        "null transports in PublicKeyCredentialDescriptor struct");
    *output = result;

    CustomContextHelper<Traits>::TearDown(input, custom_context);
  }

  static bool Deserialize(::webauth::mojom::internal::PublicKeyCredentialDescriptor_Data* input,
                          UserType* output,
                          SerializationContext* context) {
    if (!input)
      return CallSetToNullIfExists<Traits>(output);

    ::webauth::mojom::PublicKeyCredentialDescriptorDataView data_view(input, context);
    return Traits::Read(data_view, output);
  }
};

}  // namespace internal

}  // namespace mojo


namespace webauth {
namespace mojom {

inline void PublicKeyCredentialInfoDataView::GetIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PublicKeyCredentialInfoDataView::GetRawIdDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->raw_id.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}
inline void PublicKeyCredentialInfoDataView::GetClientDataJsonDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->client_data_json.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}
inline void PublicKeyCredentialInfoDataView::GetResponseDataView(
    AuthenticatorResponseDataView* output) {
  auto pointer = data_->response.Get();
  *output = AuthenticatorResponseDataView(pointer, context_);
}


inline void AuthenticatorResponseDataView::GetAttestationObjectDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->attestation_object.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}
inline void AuthenticatorResponseDataView::GetAuthenticatorDataDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->authenticator_data.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}
inline void AuthenticatorResponseDataView::GetSignatureDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->signature.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}


inline void PublicKeyCredentialEntityDataView::GetIdDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->id.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PublicKeyCredentialEntityDataView::GetNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->name.Get();
  *output = mojo::StringDataView(pointer, context_);
}
inline void PublicKeyCredentialEntityDataView::GetIconDataView(
    ::url::mojom::UrlDataView* output) {
  auto pointer = data_->icon.Get();
  *output = ::url::mojom::UrlDataView(pointer, context_);
}
inline void PublicKeyCredentialEntityDataView::GetDisplayNameDataView(
    mojo::StringDataView* output) {
  auto pointer = data_->display_name.Get();
  *output = mojo::StringDataView(pointer, context_);
}




inline void MakeCredentialOptionsDataView::GetRelyingPartyDataView(
    PublicKeyCredentialEntityDataView* output) {
  auto pointer = data_->relying_party.Get();
  *output = PublicKeyCredentialEntityDataView(pointer, context_);
}
inline void MakeCredentialOptionsDataView::GetUserDataView(
    PublicKeyCredentialEntityDataView* output) {
  auto pointer = data_->user.Get();
  *output = PublicKeyCredentialEntityDataView(pointer, context_);
}
inline void MakeCredentialOptionsDataView::GetChallengeDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->challenge.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}
inline void MakeCredentialOptionsDataView::GetCryptoParametersDataView(
    mojo::ArrayDataView<PublicKeyCredentialParametersDataView>* output) {
  auto pointer = data_->crypto_parameters.Get();
  *output = mojo::ArrayDataView<PublicKeyCredentialParametersDataView>(pointer, context_);
}
inline void MakeCredentialOptionsDataView::GetAdjustedTimeoutDataView(
    ::mojo::common::mojom::TimeDeltaDataView* output) {
  auto pointer = data_->adjusted_timeout.Get();
  *output = ::mojo::common::mojom::TimeDeltaDataView(pointer, context_);
}
inline void MakeCredentialOptionsDataView::GetExcludeCredentialsDataView(
    mojo::ArrayDataView<PublicKeyCredentialDescriptorDataView>* output) {
  auto pointer = data_->exclude_credentials.Get();
  *output = mojo::ArrayDataView<PublicKeyCredentialDescriptorDataView>(pointer, context_);
}


inline void PublicKeyCredentialDescriptorDataView::GetIdDataView(
    mojo::ArrayDataView<uint8_t>* output) {
  auto pointer = data_->id.Get();
  *output = mojo::ArrayDataView<uint8_t>(pointer, context_);
}
inline void PublicKeyCredentialDescriptorDataView::GetTransportsDataView(
    mojo::ArrayDataView<AuthenticatorTransport>* output) {
  auto pointer = data_->transports.Get();
  *output = mojo::ArrayDataView<AuthenticatorTransport>(pointer, context_);
}


inline void Authenticator_MakeCredential_ParamsDataView::GetOptionsDataView(
    MakeCredentialOptionsDataView* output) {
  auto pointer = data_->options.Get();
  *output = MakeCredentialOptionsDataView(pointer, context_);
}


inline void Authenticator_MakeCredential_ResponseParamsDataView::GetCredentialDataView(
    PublicKeyCredentialInfoDataView* output) {
  auto pointer = data_->credential.Get();
  *output = PublicKeyCredentialInfoDataView(pointer, context_);
}



}  // namespace mojom
}  // namespace webauth

#endif  // THIRD_PARTY_WEBKIT_PUBLIC_PLATFORM_MODULES_WEBAUTH_AUTHENTICATOR_MOJOM_SHARED_H_
