// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#ifndef UI_DISPLAY_MOJO_NATIVE_DISPLAY_DELEGATE_MOJOM_H_
#define UI_DISPLAY_MOJO_NATIVE_DISPLAY_DELEGATE_MOJOM_H_

#include <stdint.h>

#include <limits>
#include <type_traits>
#include <utility>

#include "base/callback.h"
#include "base/macros.h"
#include "base/optional.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr.h"
#include "mojo/public/cpp/bindings/associated_interface_ptr_info.h"
#include "mojo/public/cpp/bindings/associated_interface_request.h"
#include "mojo/public/cpp/bindings/clone_traits.h"
#include "mojo/public/cpp/bindings/equals_traits.h"
#include "mojo/public/cpp/bindings/interface_ptr.h"
#include "mojo/public/cpp/bindings/interface_request.h"
#include "mojo/public/cpp/bindings/lib/control_message_handler.h"
#include "mojo/public/cpp/bindings/lib/control_message_proxy.h"
#include "mojo/public/cpp/bindings/lib/serialization.h"
#include "mojo/public/cpp/bindings/native_struct.h"
#include "mojo/public/cpp/bindings/raw_ptr_impl_ref_traits.h"
#include "mojo/public/cpp/bindings/struct_ptr.h"
#include "mojo/public/cpp/bindings/struct_traits.h"
#include "mojo/public/cpp/bindings/thread_safe_interface_ptr.h"
#include "mojo/public/cpp/bindings/union_traits.h"
#include "ui/display/mojo/native_display_delegate.mojom-shared.h"
#include "ui/display/mojo/display_constants.mojom.h"
#include "ui/display/mojo/display_mode.mojom.h"
#include "ui/display/mojo/display_snapshot_mojo.mojom.h"
#include "ui/display/mojo/gamma_ramp_rgb_entry.mojom.h"
#include "ui/gfx/geometry/mojo/geometry.mojom.h"
#include <string>
#include <vector>


namespace display {
namespace mojom {
class NativeDisplayObserver;
using NativeDisplayObserverPtr = mojo::InterfacePtr<NativeDisplayObserver>;
using NativeDisplayObserverPtrInfo = mojo::InterfacePtrInfo<NativeDisplayObserver>;
using ThreadSafeNativeDisplayObserverPtr =
    mojo::ThreadSafeInterfacePtr<NativeDisplayObserver>;
using NativeDisplayObserverRequest = mojo::InterfaceRequest<NativeDisplayObserver>;
using NativeDisplayObserverAssociatedPtr =
    mojo::AssociatedInterfacePtr<NativeDisplayObserver>;
using ThreadSafeNativeDisplayObserverAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NativeDisplayObserver>;
using NativeDisplayObserverAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NativeDisplayObserver>;
using NativeDisplayObserverAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NativeDisplayObserver>;

class NativeDisplayDelegate;
using NativeDisplayDelegatePtr = mojo::InterfacePtr<NativeDisplayDelegate>;
using NativeDisplayDelegatePtrInfo = mojo::InterfacePtrInfo<NativeDisplayDelegate>;
using ThreadSafeNativeDisplayDelegatePtr =
    mojo::ThreadSafeInterfacePtr<NativeDisplayDelegate>;
using NativeDisplayDelegateRequest = mojo::InterfaceRequest<NativeDisplayDelegate>;
using NativeDisplayDelegateAssociatedPtr =
    mojo::AssociatedInterfacePtr<NativeDisplayDelegate>;
using ThreadSafeNativeDisplayDelegateAssociatedPtr =
    mojo::ThreadSafeAssociatedInterfacePtr<NativeDisplayDelegate>;
using NativeDisplayDelegateAssociatedPtrInfo =
    mojo::AssociatedInterfacePtrInfo<NativeDisplayDelegate>;
using NativeDisplayDelegateAssociatedRequest =
    mojo::AssociatedInterfaceRequest<NativeDisplayDelegate>;


class NativeDisplayObserverProxy;

template <typename ImplRefTraits>
class NativeDisplayObserverStub;

class NativeDisplayObserverRequestValidator;

class  NativeDisplayObserver
    : public NativeDisplayObserverInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = false;

  using Proxy_ = NativeDisplayObserverProxy;

  template <typename ImplRefTraits>
  using Stub_ = NativeDisplayObserverStub<ImplRefTraits>;

  using RequestValidator_ = NativeDisplayObserverRequestValidator;
  using ResponseValidator_ = mojo::PassThroughFilter;
  enum MethodMinVersions : uint32_t {
    kOnConfigurationChangedMinVersion = 0,
  };
  virtual ~NativeDisplayObserver() {}

  virtual void OnConfigurationChanged() = 0;
};

class NativeDisplayDelegateProxy;

template <typename ImplRefTraits>
class NativeDisplayDelegateStub;

class NativeDisplayDelegateRequestValidator;
class NativeDisplayDelegateResponseValidator;

class  NativeDisplayDelegate
    : public NativeDisplayDelegateInterfaceBase {
 public:
  static const char Name_[];
  static constexpr uint32_t Version_ = 0;
  static constexpr bool PassesAssociatedKinds_ = false;
  static constexpr bool HasSyncMethods_ = true;

  using Proxy_ = NativeDisplayDelegateProxy;

  template <typename ImplRefTraits>
  using Stub_ = NativeDisplayDelegateStub<ImplRefTraits>;

  using RequestValidator_ = NativeDisplayDelegateRequestValidator;
  using ResponseValidator_ = NativeDisplayDelegateResponseValidator;
  enum MethodMinVersions : uint32_t {
    kInitializeMinVersion = 0,
    kTakeDisplayControlMinVersion = 0,
    kRelinquishDisplayControlMinVersion = 0,
    kGetDisplaysMinVersion = 0,
    kConfigureMinVersion = 0,
    kGetHDCPStateMinVersion = 0,
    kSetHDCPStateMinVersion = 0,
    kSetColorCorrectionMinVersion = 0,
  };
  virtual ~NativeDisplayDelegate() {}

  // Sync method. This signature is used by the client side; the service side
  // should implement the signature with callback below.
  virtual bool Initialize(NativeDisplayObserverPtr observer, std::vector<std::unique_ptr<display::DisplaySnapshotMojo>>* snapshots);

  using InitializeCallback = base::Callback<void(std::vector<std::unique_ptr<display::DisplaySnapshotMojo>>)>;
  virtual void Initialize(NativeDisplayObserverPtr observer, const InitializeCallback& callback) = 0;


  using TakeDisplayControlCallback = base::Callback<void(bool)>;
  virtual void TakeDisplayControl(const TakeDisplayControlCallback& callback) = 0;


  using RelinquishDisplayControlCallback = base::Callback<void(bool)>;
  virtual void RelinquishDisplayControl(const RelinquishDisplayControlCallback& callback) = 0;


  using GetDisplaysCallback = base::Callback<void(std::vector<std::unique_ptr<display::DisplaySnapshotMojo>>)>;
  virtual void GetDisplays(const GetDisplaysCallback& callback) = 0;


  using ConfigureCallback = base::Callback<void(bool)>;
  virtual void Configure(int64_t display_id, base::Optional<std::unique_ptr<display::DisplayMode>> mode, const gfx::Point& origin, const ConfigureCallback& callback) = 0;


  using GetHDCPStateCallback = base::Callback<void(bool, display::HDCPState)>;
  virtual void GetHDCPState(int64_t display_id, const GetHDCPStateCallback& callback) = 0;


  using SetHDCPStateCallback = base::Callback<void(bool)>;
  virtual void SetHDCPState(int64_t display_id, display::HDCPState state, const SetHDCPStateCallback& callback) = 0;

  virtual void SetColorCorrection(int64_t display_id, const std::vector<display::GammaRampRGBEntry>& degamma_lut, const std::vector<display::GammaRampRGBEntry>& gamma_lut, const std::vector<float>& correction_matrix) = 0;
};

class  NativeDisplayObserverProxy
    : public NativeDisplayObserver {
 public:
  explicit NativeDisplayObserverProxy(mojo::MessageReceiverWithResponder* receiver);
  void OnConfigurationChanged() override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};

class  NativeDisplayDelegateProxy
    : public NativeDisplayDelegate {
 public:
  explicit NativeDisplayDelegateProxy(mojo::MessageReceiverWithResponder* receiver);
  bool Initialize(NativeDisplayObserverPtr observer, std::vector<std::unique_ptr<display::DisplaySnapshotMojo>>* snapshots) override;
  void Initialize(NativeDisplayObserverPtr observer, const InitializeCallback& callback) override;
  void TakeDisplayControl(const TakeDisplayControlCallback& callback) override;
  void RelinquishDisplayControl(const RelinquishDisplayControlCallback& callback) override;
  void GetDisplays(const GetDisplaysCallback& callback) override;
  void Configure(int64_t display_id, base::Optional<std::unique_ptr<display::DisplayMode>> mode, const gfx::Point& origin, const ConfigureCallback& callback) override;
  void GetHDCPState(int64_t display_id, const GetHDCPStateCallback& callback) override;
  void SetHDCPState(int64_t display_id, display::HDCPState state, const SetHDCPStateCallback& callback) override;
  void SetColorCorrection(int64_t display_id, const std::vector<display::GammaRampRGBEntry>& degamma_lut, const std::vector<display::GammaRampRGBEntry>& gamma_lut, const std::vector<float>& correction_matrix) override;

 private:
  mojo::MessageReceiverWithResponder* receiver_;
};
class  NativeDisplayObserverStubDispatch {
 public:
  static bool Accept(NativeDisplayObserver* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NativeDisplayObserver* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NativeDisplayObserver>>
class NativeDisplayObserverStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NativeDisplayObserverStub() {}
  ~NativeDisplayObserverStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NativeDisplayObserverStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NativeDisplayObserverStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  NativeDisplayDelegateStubDispatch {
 public:
  static bool Accept(NativeDisplayDelegate* impl, mojo::Message* message);
  static bool AcceptWithResponder(
      NativeDisplayDelegate* impl,
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder);
};

template <typename ImplRefTraits =
              mojo::RawPtrImplRefTraits<NativeDisplayDelegate>>
class NativeDisplayDelegateStub
    : public NON_EXPORTED_BASE(mojo::MessageReceiverWithResponderStatus) {
 public:
  using ImplPointerType = typename ImplRefTraits::PointerType;

  NativeDisplayDelegateStub() {}
  ~NativeDisplayDelegateStub() override {}

  void set_sink(ImplPointerType sink) { sink_ = std::move(sink); }
  ImplPointerType& sink() { return sink_; }

  bool Accept(mojo::Message* message) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NativeDisplayDelegateStubDispatch::Accept(
        ImplRefTraits::GetRawPointer(&sink_), message);
  }

  bool AcceptWithResponder(
      mojo::Message* message,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) override {
    if (ImplRefTraits::IsNull(sink_))
      return false;
    return NativeDisplayDelegateStubDispatch::AcceptWithResponder(
        ImplRefTraits::GetRawPointer(&sink_), message, std::move(responder));
  }

 private:
  ImplPointerType sink_;
};
class  NativeDisplayObserverRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  NativeDisplayDelegateRequestValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};
class  NativeDisplayDelegateResponseValidator : public NON_EXPORTED_BASE(mojo::MessageReceiver) {
 public:
  bool Accept(mojo::Message* message) override;
};





}  // namespace mojom
}  // namespace display

namespace mojo {

}  // namespace mojo

#endif  // UI_DISPLAY_MOJO_NATIVE_DISPLAY_DELEGATE_MOJOM_H_