// Copyright 2013 The Chromium Authors. All rights reserved.
// Use of this source code is governed by a BSD-style license that can be
// found in the LICENSE file.

#if defined(__clang__)
#pragma clang diagnostic push
#pragma clang diagnostic ignored "-Wunused-private-field"
#elif defined(_MSC_VER)
#pragma warning(push)
#pragma warning(disable:4056)
#pragma warning(disable:4065)
#pragma warning(disable:4756)
#endif

#include "services/ui/public/interfaces/window_tree.mojom.h"

#include <math.h>
#include <stdint.h>
#include <utility>

#include "base/logging.h"
#include "mojo/public/cpp/bindings/lib/message_internal.h"
#include "mojo/public/cpp/bindings/lib/serialization_util.h"
#include "mojo/public/cpp/bindings/lib/unserialized_message_context.h"
#include "mojo/public/cpp/bindings/lib/validate_params.h"
#include "mojo/public/cpp/bindings/lib/validation_context.h"
#include "mojo/public/cpp/bindings/lib/validation_errors.h"
#include "mojo/public/interfaces/bindings/interface_control_messages.mojom.h"
#include "cc/ipc/frame_sink_id_struct_traits.h"
#include "cc/ipc/local_surface_id_struct_traits.h"
#include "cc/ipc/surface_id_struct_traits.h"
#include "cc/ipc/surface_info_struct_traits.h"
#include "ipc/ipc_message_utils.h"
#include "mojo/common/common_custom_types_struct_traits.h"
#include "services/ui/public/interfaces/cursor/cursor_struct_traits.h"
#include "skia/public/interfaces/bitmap_skbitmap_struct_traits.h"
#include "ui/base/mojo/ui_base_types_struct_traits.h"
#include "ui/events/mojo/event_struct_traits.h"
#include "ui/gfx/geometry/mojo/geometry_struct_traits.h"
#include "ui/gfx/mojo/transform_struct_traits.h"
#include "ui/latency/mojo/latency_info_struct_traits.h"
namespace ui {
namespace mojom {
const char WindowTree::Name_[] = "ui::mojom::WindowTree";

class WindowTree_GetWindowTree_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  WindowTree_GetWindowTree_ForwardToCallback(
      const WindowTree::GetWindowTreeCallback& callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  WindowTree::GetWindowTreeCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(WindowTree_GetWindowTree_ForwardToCallback);
};

class WindowTree_Embed_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  WindowTree_Embed_ForwardToCallback(
      const WindowTree::EmbedCallback& callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  WindowTree::EmbedCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(WindowTree_Embed_ForwardToCallback);
};

class WindowTree_GetCursorLocationMemory_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  WindowTree_GetCursorLocationMemory_ForwardToCallback(
      const WindowTree::GetCursorLocationMemoryCallback& callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  WindowTree::GetCursorLocationMemoryCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(WindowTree_GetCursorLocationMemory_ForwardToCallback);
};

WindowTreeProxy::WindowTreeProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WindowTreeProxy::NewWindow(
    uint32_t in_change_id, uint32_t in_window_id, const base::Optional<std::unordered_map<std::string, std::vector<uint8_t>>>& in_properties) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_NewWindow_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_properties, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_NewWindow_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_NewWindow_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  typename decltype(params->properties)::BaseType* properties_ptr;
  const mojo::internal::ContainerValidateParams properties_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
  mojo::internal::Serialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_properties, buffer, &properties_ptr, &properties_validate_params,
      &serialization_context);
  params->properties.Set(properties_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::NewTopLevelWindow(
    uint32_t in_change_id, uint32_t in_window_id, const std::unordered_map<std::string, std::vector<uint8_t>>& in_properties) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_NewTopLevelWindow_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_properties, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_NewTopLevelWindow_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_NewTopLevelWindow_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  typename decltype(params->properties)::BaseType* properties_ptr;
  const mojo::internal::ContainerValidateParams properties_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
  mojo::internal::Serialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_properties, buffer, &properties_ptr, &properties_validate_params,
      &serialization_context);
  params->properties.Set(properties_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->properties.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null properties in WindowTree.NewTopLevelWindow request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::DeleteWindow(
    uint32_t in_change_id, uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_DeleteWindow_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_DeleteWindow_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_DeleteWindow_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetCapture(
    uint32_t in_change_id, uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetCapture_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetCapture_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetCapture_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::ReleaseCapture(
    uint32_t in_change_id, uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_ReleaseCapture_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_ReleaseCapture_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_ReleaseCapture_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::StartPointerWatcher(
    bool in_want_moves) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_StartPointerWatcher_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_StartPointerWatcher_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_StartPointerWatcher_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->want_moves = in_want_moves;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::StopPointerWatcher(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_StopPointerWatcher_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_StopPointerWatcher_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_StopPointerWatcher_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetWindowBounds(
    uint32_t in_change_id, uint32_t in_window_id, const gfx::Rect& in_bounds, const base::Optional<viz::LocalSurfaceId>& in_local_surface_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetWindowBounds_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
      in_bounds, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetWindowBounds_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetWindowBounds_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  typename decltype(params->bounds)::BaseType* bounds_ptr;
  mojo::internal::Serialize<::gfx::mojom::RectDataView>(
      in_bounds, buffer, &bounds_ptr, &serialization_context);
  params->bounds.Set(bounds_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->bounds.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null bounds in WindowTree.SetWindowBounds request");
  typename decltype(params->local_surface_id)::BaseType* local_surface_id_ptr;
  mojo::internal::Serialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, buffer, &local_surface_id_ptr, &serialization_context);
  params->local_surface_id.Set(local_surface_id_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetWindowTransform(
    uint32_t in_change_id, uint32_t in_window_id, const gfx::Transform& in_transform) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetWindowTransform_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::TransformDataView>(
      in_transform, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetWindowTransform_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetWindowTransform_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  typename decltype(params->transform)::BaseType* transform_ptr;
  mojo::internal::Serialize<::gfx::mojom::TransformDataView>(
      in_transform, buffer, &transform_ptr, &serialization_context);
  params->transform.Set(transform_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->transform.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null transform in WindowTree.SetWindowTransform request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetClientArea(
    uint32_t in_window_id, const gfx::Insets& in_insets, const base::Optional<std::vector<gfx::Rect>>& in_additional_client_areas) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetClientArea_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::InsetsDataView>(
      in_insets, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::gfx::mojom::RectDataView>>(
      in_additional_client_areas, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetClientArea_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetClientArea_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  typename decltype(params->insets)::BaseType* insets_ptr;
  mojo::internal::Serialize<::gfx::mojom::InsetsDataView>(
      in_insets, buffer, &insets_ptr, &serialization_context);
  params->insets.Set(insets_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->insets.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null insets in WindowTree.SetClientArea request");
  typename decltype(params->additional_client_areas)::BaseType* additional_client_areas_ptr;
  const mojo::internal::ContainerValidateParams additional_client_areas_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::gfx::mojom::RectDataView>>(
      in_additional_client_areas, buffer, &additional_client_areas_ptr, &additional_client_areas_validate_params,
      &serialization_context);
  params->additional_client_areas.Set(additional_client_areas_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetHitTestMask(
    uint32_t in_window_id, const base::Optional<gfx::Rect>& in_mask) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetHitTestMask_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
      in_mask, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetHitTestMask_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetHitTestMask_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  typename decltype(params->mask)::BaseType* mask_ptr;
  mojo::internal::Serialize<::gfx::mojom::RectDataView>(
      in_mask, buffer, &mask_ptr, &serialization_context);
  params->mask.Set(mask_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetCanAcceptDrops(
    uint32_t in_window_id, bool in_accepts_drops) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetCanAcceptDrops_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetCanAcceptDrops_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetCanAcceptDrops_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  params->accepts_drops = in_accepts_drops;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetWindowVisibility(
    uint32_t in_change_id, uint32_t in_window_id, bool in_visible) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetWindowVisibility_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetWindowVisibility_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetWindowVisibility_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  params->visible = in_visible;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetWindowProperty(
    uint32_t in_change_id, uint32_t in_window_id, const std::string& in_name, const base::Optional<std::vector<uint8_t>>& in_value) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetWindowProperty_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_name, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
      in_value, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetWindowProperty_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetWindowProperty_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  typename decltype(params->name)::BaseType* name_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_name, buffer, &name_ptr, &serialization_context);
  params->name.Set(name_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->name.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null name in WindowTree.SetWindowProperty request");
  typename decltype(params->value)::BaseType* value_ptr;
  const mojo::internal::ContainerValidateParams value_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      in_value, buffer, &value_ptr, &value_validate_params,
      &serialization_context);
  params->value.Set(value_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetWindowOpacity(
    uint32_t in_change_id, uint32_t in_window_id, float in_opacity) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetWindowOpacity_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetWindowOpacity_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetWindowOpacity_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  params->opacity = in_opacity;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::AttachCompositorFrameSink(
    uint32_t in_window_id, ::cc::mojom::CompositorFrameSinkRequest in_compositor_frame_sink, ::cc::mojom::CompositorFrameSinkClientPtr in_client) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_AttachCompositorFrameSink_Params_Data);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::CompositorFrameSinkRequestDataView>(
      in_compositor_frame_sink, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::CompositorFrameSinkClientPtrDataView>(
      in_client, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_AttachCompositorFrameSink_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_AttachCompositorFrameSink_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  mojo::internal::Serialize<::cc::mojom::CompositorFrameSinkRequestDataView>(
      in_compositor_frame_sink, &params->compositor_frame_sink, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->compositor_frame_sink),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid compositor_frame_sink in WindowTree.AttachCompositorFrameSink request");
  mojo::internal::Serialize<::cc::mojom::CompositorFrameSinkClientPtrDataView>(
      in_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in WindowTree.AttachCompositorFrameSink request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::AddWindow(
    uint32_t in_change_id, uint32_t in_parent, uint32_t in_child) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_AddWindow_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_AddWindow_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_AddWindow_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->parent = in_parent;
  params->child = in_child;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::RemoveWindowFromParent(
    uint32_t in_change_id, uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_RemoveWindowFromParent_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_RemoveWindowFromParent_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_RemoveWindowFromParent_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::AddTransientWindow(
    uint32_t in_change_id, uint32_t in_window_id, uint32_t in_transient_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_AddTransientWindow_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_AddTransientWindow_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_AddTransientWindow_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  params->transient_window_id = in_transient_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::RemoveTransientWindowFromParent(
    uint32_t in_change_id, uint32_t in_transient_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_RemoveTransientWindowFromParent_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_RemoveTransientWindowFromParent_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_RemoveTransientWindowFromParent_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->transient_window_id = in_transient_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetModalType(
    uint32_t in_change_id, uint32_t in_window_id, ui::ModalType in_type) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetModalType_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetModalType_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetModalType_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  mojo::internal::Serialize<::ui::mojom::ModalType>(
      in_type, &params->type);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::ReorderWindow(
    uint32_t in_change_id, uint32_t in_window_id, uint32_t in_relative_window_id, ::ui::mojom::OrderDirection in_direction) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_ReorderWindow_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_ReorderWindow_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_ReorderWindow_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  params->relative_window_id = in_relative_window_id;
  mojo::internal::Serialize<::ui::mojom::OrderDirection>(
      in_direction, &params->direction);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::GetWindowTree(
    uint32_t in_window_id, const GetWindowTreeCallback& callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_GetWindowTree_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_GetWindowTree_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_GetWindowTree_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new WindowTree_GetWindowTree_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void WindowTreeProxy::Embed(
    uint32_t in_window_id, WindowTreeClientPtr in_client, uint32_t in_embed_flags, const EmbedCallback& callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_Embed_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowTreeClientPtrDataView>(
      in_client, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_Embed_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_Embed_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  mojo::internal::Serialize<::ui::mojom::WindowTreeClientPtrDataView>(
      in_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in WindowTree.Embed request");
  params->embed_flags = in_embed_flags;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new WindowTree_Embed_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void WindowTreeProxy::SetFocus(
    uint32_t in_change_id, uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetFocus_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetFocus_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetFocus_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetCanFocus(
    uint32_t in_window_id, bool in_can_focus) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetCanFocus_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetCanFocus_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetCanFocus_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  params->can_focus = in_can_focus;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetCursor(
    uint32_t in_change_id, uint32_t in_window_id, ui::CursorData in_cursor) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetCursor_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::CursorDataDataView>(
      in_cursor, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetCursor_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetCursor_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  typename decltype(params->cursor)::BaseType* cursor_ptr;
  mojo::internal::Serialize<::ui::mojom::CursorDataDataView>(
      in_cursor, buffer, &cursor_ptr, &serialization_context);
  params->cursor.Set(cursor_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cursor.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cursor in WindowTree.SetCursor request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetWindowTextInputState(
    uint32_t in_window_id, ::mojo::TextInputStatePtr in_state) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetWindowTextInputState_Params_Data);
  size += mojo::internal::PrepareToSerialize<::mojo::TextInputStateDataView>(
      in_state, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetWindowTextInputState_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetWindowTextInputState_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  typename decltype(params->state)::BaseType* state_ptr;
  mojo::internal::Serialize<::mojo::TextInputStateDataView>(
      in_state, buffer, &state_ptr, &serialization_context);
  params->state.Set(state_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->state.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null state in WindowTree.SetWindowTextInputState request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetImeVisibility(
    uint32_t in_window_id, bool in_visible, ::mojo::TextInputStatePtr in_state) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetImeVisibility_Params_Data);
  size += mojo::internal::PrepareToSerialize<::mojo::TextInputStateDataView>(
      in_state, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetImeVisibility_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetImeVisibility_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  params->visible = in_visible;
  typename decltype(params->state)::BaseType* state_ptr;
  mojo::internal::Serialize<::mojo::TextInputStateDataView>(
      in_state, buffer, &state_ptr, &serialization_context);
  params->state.Set(state_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::SetEventTargetingPolicy(
    uint32_t in_window_id, ::ui::mojom::EventTargetingPolicy in_policy) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_SetEventTargetingPolicy_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_SetEventTargetingPolicy_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_SetEventTargetingPolicy_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  mojo::internal::Serialize<::ui::mojom::EventTargetingPolicy>(
      in_policy, &params->policy);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::OnWindowInputEventAck(
    uint32_t in_event_id, ::ui::mojom::EventResult in_result) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_OnWindowInputEventAck_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_OnWindowInputEventAck_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_OnWindowInputEventAck_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->event_id = in_event_id;
  mojo::internal::Serialize<::ui::mojom::EventResult>(
      in_result, &params->result);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::DeactivateWindow(
    uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_DeactivateWindow_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_DeactivateWindow_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_DeactivateWindow_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::StackAbove(
    uint32_t in_change_id, uint32_t in_above_id, uint32_t in_below_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_StackAbove_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_StackAbove_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_StackAbove_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->above_id = in_above_id;
  params->below_id = in_below_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::StackAtTop(
    uint32_t in_change_id, uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_StackAtTop_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_StackAtTop_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_StackAtTop_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::GetWindowManagerClient(
    ::ui::mojom::WindowManagerClientAssociatedRequest in_internal) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_GetWindowManagerClient_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowManagerClientAssociatedRequestDataView>(
      in_internal, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_GetWindowManagerClient_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_GetWindowManagerClient_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::ui::mojom::WindowManagerClientAssociatedRequestDataView>(
      in_internal, &params->internal, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->internal),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_INTERFACE_ID,
      "invalid internal in WindowTree.GetWindowManagerClient request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::GetCursorLocationMemory(
    const GetCursorLocationMemoryCallback& callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_GetCursorLocationMemory_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_GetCursorLocationMemory_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_GetCursorLocationMemory_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  std::unique_ptr<mojo::MessageReceiver> responder(
      new WindowTree_GetCursorLocationMemory_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void WindowTreeProxy::PerformWindowMove(
    uint32_t in_change_id, uint32_t in_window_id, ::ui::mojom::MoveLoopSource in_source, const gfx::Point& in_cursor) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_PerformWindowMove_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::PointDataView>(
      in_cursor, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_PerformWindowMove_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_PerformWindowMove_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->window_id = in_window_id;
  mojo::internal::Serialize<::ui::mojom::MoveLoopSource>(
      in_source, &params->source);
  typename decltype(params->cursor)::BaseType* cursor_ptr;
  mojo::internal::Serialize<::gfx::mojom::PointDataView>(
      in_cursor, buffer, &cursor_ptr, &serialization_context);
  params->cursor.Set(cursor_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cursor.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cursor in WindowTree.PerformWindowMove request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::CancelWindowMove(
    uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_CancelWindowMove_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_CancelWindowMove_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_CancelWindowMove_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::PerformDragDrop(
    uint32_t in_change_id, uint32_t in_source_window_id, const gfx::Point& in_screen_location, const std::unordered_map<std::string, std::vector<uint8_t>>& in_drag_data, const SkBitmap& in_drag_image, const gfx::Vector2d& in_drag_image_offset, uint32_t in_drag_operation, ::ui::mojom::PointerKind in_source) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_PerformDragDrop_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::PointDataView>(
      in_screen_location, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_drag_data, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::skia::mojom::BitmapDataView>(
      in_drag_image, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::Vector2dDataView>(
      in_drag_image_offset, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_PerformDragDrop_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_PerformDragDrop_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->source_window_id = in_source_window_id;
  typename decltype(params->screen_location)::BaseType* screen_location_ptr;
  mojo::internal::Serialize<::gfx::mojom::PointDataView>(
      in_screen_location, buffer, &screen_location_ptr, &serialization_context);
  params->screen_location.Set(screen_location_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->screen_location.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null screen_location in WindowTree.PerformDragDrop request");
  typename decltype(params->drag_data)::BaseType* drag_data_ptr;
  const mojo::internal::ContainerValidateParams drag_data_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
  mojo::internal::Serialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_drag_data, buffer, &drag_data_ptr, &drag_data_validate_params,
      &serialization_context);
  params->drag_data.Set(drag_data_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->drag_data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null drag_data in WindowTree.PerformDragDrop request");
  typename decltype(params->drag_image)::BaseType* drag_image_ptr;
  mojo::internal::Serialize<::skia::mojom::BitmapDataView>(
      in_drag_image, buffer, &drag_image_ptr, &serialization_context);
  params->drag_image.Set(drag_image_ptr);
  typename decltype(params->drag_image_offset)::BaseType* drag_image_offset_ptr;
  mojo::internal::Serialize<::gfx::mojom::Vector2dDataView>(
      in_drag_image_offset, buffer, &drag_image_offset_ptr, &serialization_context);
  params->drag_image_offset.Set(drag_image_offset_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->drag_image_offset.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null drag_image_offset in WindowTree.PerformDragDrop request");
  params->drag_operation = in_drag_operation;
  mojo::internal::Serialize<::ui::mojom::PointerKind>(
      in_source, &params->source);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeProxy::CancelDragDrop(
    uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_CancelDragDrop_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_CancelDragDrop_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_CancelDragDrop_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class WindowTree_GetWindowTree_ProxyToResponder {
 public:
  static WindowTree::GetWindowTreeCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<WindowTree_GetWindowTree_ProxyToResponder> proxy(
        new WindowTree_GetWindowTree_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&WindowTree_GetWindowTree_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~WindowTree_GetWindowTree_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "WindowTree::GetWindowTree() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  WindowTree_GetWindowTree_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      std::vector<::ui::mojom::WindowDataPtr> in_windows);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(WindowTree_GetWindowTree_ProxyToResponder);
};

bool WindowTree_GetWindowTree_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::WindowTree_GetWindowTree_ResponseParams_Data* params =
      reinterpret_cast<
          internal::WindowTree_GetWindowTree_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  std::vector<::ui::mojom::WindowDataPtr> p_windows{};
  WindowTree_GetWindowTree_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  if (!input_data_view.ReadWindows(&p_windows))
    success = false;
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "WindowTree::GetWindowTree response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_windows));
  return true;
}

void WindowTree_GetWindowTree_ProxyToResponder::Run(
    std::vector<::ui::mojom::WindowDataPtr> in_windows) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_GetWindowTree_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::ui::mojom::WindowDataDataView>>(
      in_windows, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_GetWindowTree_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_GetWindowTree_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->windows)::BaseType* windows_ptr;
  const mojo::internal::ContainerValidateParams windows_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::ui::mojom::WindowDataDataView>>(
      in_windows, buffer, &windows_ptr, &windows_validate_params,
      &serialization_context);
  params->windows.Set(windows_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->windows.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null windows in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class WindowTree_Embed_ProxyToResponder {
 public:
  static WindowTree::EmbedCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<WindowTree_Embed_ProxyToResponder> proxy(
        new WindowTree_Embed_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&WindowTree_Embed_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~WindowTree_Embed_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "WindowTree::Embed() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  WindowTree_Embed_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      bool in_success);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(WindowTree_Embed_ProxyToResponder);
};

bool WindowTree_Embed_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::WindowTree_Embed_ResponseParams_Data* params =
      reinterpret_cast<
          internal::WindowTree_Embed_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  bool p_success{};
  WindowTree_Embed_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_success = input_data_view.success();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "WindowTree::Embed response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_success));
  return true;
}

void WindowTree_Embed_ProxyToResponder::Run(
    bool in_success) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_Embed_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTree_Embed_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_Embed_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->success = in_success;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class WindowTree_GetCursorLocationMemory_ProxyToResponder {
 public:
  static WindowTree::GetCursorLocationMemoryCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<WindowTree_GetCursorLocationMemory_ProxyToResponder> proxy(
        new WindowTree_GetCursorLocationMemory_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&WindowTree_GetCursorLocationMemory_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~WindowTree_GetCursorLocationMemory_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "WindowTree::GetCursorLocationMemory() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  WindowTree_GetCursorLocationMemory_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      mojo::ScopedSharedBufferHandle in_cursor_buffer);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(WindowTree_GetCursorLocationMemory_ProxyToResponder);
};

bool WindowTree_GetCursorLocationMemory_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::WindowTree_GetCursorLocationMemory_ResponseParams_Data* params =
      reinterpret_cast<
          internal::WindowTree_GetCursorLocationMemory_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  mojo::ScopedSharedBufferHandle p_cursor_buffer{};
  WindowTree_GetCursorLocationMemory_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_cursor_buffer = input_data_view.TakeCursorBuffer();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "WindowTree::GetCursorLocationMemory response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_cursor_buffer));
  return true;
}

void WindowTree_GetCursorLocationMemory_ProxyToResponder::Run(
    mojo::ScopedSharedBufferHandle in_cursor_buffer) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTree_GetCursorLocationMemory_ResponseParams_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ScopedSharedBufferHandle>(
      in_cursor_buffer, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTree_GetCursorLocationMemory_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTree_GetCursorLocationMemory_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<mojo::ScopedSharedBufferHandle>(
      in_cursor_buffer, &params->cursor_buffer, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->cursor_buffer),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid cursor_buffer in ");

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool WindowTreeStubDispatch::Accept(
    WindowTree* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWindowTree_NewWindow_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_NewWindow_Params_Data* params =
          reinterpret_cast<internal::WindowTree_NewWindow_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      base::Optional<std::unordered_map<std::string, std::vector<uint8_t>>> p_properties{};
      WindowTree_NewWindow_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadProperties(&p_properties))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::NewWindow deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->NewWindow(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_properties));
      return true;
    }
    case internal::kWindowTree_NewTopLevelWindow_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_NewTopLevelWindow_Params_Data* params =
          reinterpret_cast<internal::WindowTree_NewTopLevelWindow_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      std::unordered_map<std::string, std::vector<uint8_t>> p_properties{};
      WindowTree_NewTopLevelWindow_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadProperties(&p_properties))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::NewTopLevelWindow deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->NewTopLevelWindow(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_properties));
      return true;
    }
    case internal::kWindowTree_DeleteWindow_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_DeleteWindow_Params_Data* params =
          reinterpret_cast<internal::WindowTree_DeleteWindow_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      WindowTree_DeleteWindow_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::DeleteWindow deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DeleteWindow(
std::move(p_change_id), 
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_SetCapture_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetCapture_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetCapture_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      WindowTree_SetCapture_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetCapture deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCapture(
std::move(p_change_id), 
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_ReleaseCapture_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_ReleaseCapture_Params_Data* params =
          reinterpret_cast<internal::WindowTree_ReleaseCapture_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      WindowTree_ReleaseCapture_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::ReleaseCapture deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ReleaseCapture(
std::move(p_change_id), 
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_StartPointerWatcher_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_StartPointerWatcher_Params_Data* params =
          reinterpret_cast<internal::WindowTree_StartPointerWatcher_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      bool p_want_moves{};
      WindowTree_StartPointerWatcher_ParamsDataView input_data_view(params, &serialization_context);
      
      p_want_moves = input_data_view.want_moves();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::StartPointerWatcher deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->StartPointerWatcher(
std::move(p_want_moves));
      return true;
    }
    case internal::kWindowTree_StopPointerWatcher_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_StopPointerWatcher_Params_Data* params =
          reinterpret_cast<internal::WindowTree_StopPointerWatcher_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WindowTree_StopPointerWatcher_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::StopPointerWatcher deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->StopPointerWatcher();
      return true;
    }
    case internal::kWindowTree_SetWindowBounds_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetWindowBounds_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetWindowBounds_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      gfx::Rect p_bounds{};
      base::Optional<viz::LocalSurfaceId> p_local_surface_id{};
      WindowTree_SetWindowBounds_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadBounds(&p_bounds))
        success = false;
      if (!input_data_view.ReadLocalSurfaceId(&p_local_surface_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetWindowBounds deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetWindowBounds(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_bounds), 
std::move(p_local_surface_id));
      return true;
    }
    case internal::kWindowTree_SetWindowTransform_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetWindowTransform_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetWindowTransform_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      gfx::Transform p_transform{};
      WindowTree_SetWindowTransform_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadTransform(&p_transform))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetWindowTransform deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetWindowTransform(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_transform));
      return true;
    }
    case internal::kWindowTree_SetClientArea_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetClientArea_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetClientArea_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      gfx::Insets p_insets{};
      base::Optional<std::vector<gfx::Rect>> p_additional_client_areas{};
      WindowTree_SetClientArea_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadInsets(&p_insets))
        success = false;
      if (!input_data_view.ReadAdditionalClientAreas(&p_additional_client_areas))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetClientArea deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetClientArea(
std::move(p_window_id), 
std::move(p_insets), 
std::move(p_additional_client_areas));
      return true;
    }
    case internal::kWindowTree_SetHitTestMask_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetHitTestMask_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetHitTestMask_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      base::Optional<gfx::Rect> p_mask{};
      WindowTree_SetHitTestMask_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadMask(&p_mask))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetHitTestMask deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetHitTestMask(
std::move(p_window_id), 
std::move(p_mask));
      return true;
    }
    case internal::kWindowTree_SetCanAcceptDrops_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetCanAcceptDrops_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetCanAcceptDrops_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      bool p_accepts_drops{};
      WindowTree_SetCanAcceptDrops_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_accepts_drops = input_data_view.accepts_drops();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetCanAcceptDrops deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCanAcceptDrops(
std::move(p_window_id), 
std::move(p_accepts_drops));
      return true;
    }
    case internal::kWindowTree_SetWindowVisibility_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetWindowVisibility_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetWindowVisibility_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      bool p_visible{};
      WindowTree_SetWindowVisibility_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      p_visible = input_data_view.visible();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetWindowVisibility deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetWindowVisibility(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_visible));
      return true;
    }
    case internal::kWindowTree_SetWindowProperty_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetWindowProperty_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetWindowProperty_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      std::string p_name{};
      base::Optional<std::vector<uint8_t>> p_value{};
      WindowTree_SetWindowProperty_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadName(&p_name))
        success = false;
      if (!input_data_view.ReadValue(&p_value))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetWindowProperty deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetWindowProperty(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_name), 
std::move(p_value));
      return true;
    }
    case internal::kWindowTree_SetWindowOpacity_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetWindowOpacity_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetWindowOpacity_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      float p_opacity{};
      WindowTree_SetWindowOpacity_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      p_opacity = input_data_view.opacity();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetWindowOpacity deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetWindowOpacity(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_opacity));
      return true;
    }
    case internal::kWindowTree_AttachCompositorFrameSink_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_AttachCompositorFrameSink_Params_Data* params =
          reinterpret_cast<internal::WindowTree_AttachCompositorFrameSink_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      ::cc::mojom::CompositorFrameSinkRequest p_compositor_frame_sink{};
      ::cc::mojom::CompositorFrameSinkClientPtr p_client{};
      WindowTree_AttachCompositorFrameSink_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_compositor_frame_sink =
          input_data_view.TakeCompositorFrameSink<decltype(p_compositor_frame_sink)>();
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::AttachCompositorFrameSink deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AttachCompositorFrameSink(
std::move(p_window_id), 
std::move(p_compositor_frame_sink), 
std::move(p_client));
      return true;
    }
    case internal::kWindowTree_AddWindow_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_AddWindow_Params_Data* params =
          reinterpret_cast<internal::WindowTree_AddWindow_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_parent{};
      uint32_t p_child{};
      WindowTree_AddWindow_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_parent = input_data_view.parent();
      p_child = input_data_view.child();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::AddWindow deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AddWindow(
std::move(p_change_id), 
std::move(p_parent), 
std::move(p_child));
      return true;
    }
    case internal::kWindowTree_RemoveWindowFromParent_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_RemoveWindowFromParent_Params_Data* params =
          reinterpret_cast<internal::WindowTree_RemoveWindowFromParent_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      WindowTree_RemoveWindowFromParent_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::RemoveWindowFromParent deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RemoveWindowFromParent(
std::move(p_change_id), 
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_AddTransientWindow_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_AddTransientWindow_Params_Data* params =
          reinterpret_cast<internal::WindowTree_AddTransientWindow_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      uint32_t p_transient_window_id{};
      WindowTree_AddTransientWindow_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      p_transient_window_id = input_data_view.transient_window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::AddTransientWindow deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->AddTransientWindow(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_transient_window_id));
      return true;
    }
    case internal::kWindowTree_RemoveTransientWindowFromParent_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_RemoveTransientWindowFromParent_Params_Data* params =
          reinterpret_cast<internal::WindowTree_RemoveTransientWindowFromParent_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_transient_window_id{};
      WindowTree_RemoveTransientWindowFromParent_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_transient_window_id = input_data_view.transient_window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::RemoveTransientWindowFromParent deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RemoveTransientWindowFromParent(
std::move(p_change_id), 
std::move(p_transient_window_id));
      return true;
    }
    case internal::kWindowTree_SetModalType_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetModalType_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetModalType_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      ui::ModalType p_type{};
      WindowTree_SetModalType_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadType(&p_type))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetModalType deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetModalType(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_type));
      return true;
    }
    case internal::kWindowTree_ReorderWindow_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_ReorderWindow_Params_Data* params =
          reinterpret_cast<internal::WindowTree_ReorderWindow_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      uint32_t p_relative_window_id{};
      ::ui::mojom::OrderDirection p_direction{};
      WindowTree_ReorderWindow_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      p_relative_window_id = input_data_view.relative_window_id();
      if (!input_data_view.ReadDirection(&p_direction))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::ReorderWindow deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->ReorderWindow(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_relative_window_id), 
std::move(p_direction));
      return true;
    }
    case internal::kWindowTree_GetWindowTree_Name: {
      break;
    }
    case internal::kWindowTree_Embed_Name: {
      break;
    }
    case internal::kWindowTree_SetFocus_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetFocus_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetFocus_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      WindowTree_SetFocus_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetFocus deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetFocus(
std::move(p_change_id), 
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_SetCanFocus_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetCanFocus_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetCanFocus_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      bool p_can_focus{};
      WindowTree_SetCanFocus_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_can_focus = input_data_view.can_focus();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetCanFocus deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCanFocus(
std::move(p_window_id), 
std::move(p_can_focus));
      return true;
    }
    case internal::kWindowTree_SetCursor_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetCursor_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetCursor_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      ui::CursorData p_cursor{};
      WindowTree_SetCursor_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadCursor(&p_cursor))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetCursor deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetCursor(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_cursor));
      return true;
    }
    case internal::kWindowTree_SetWindowTextInputState_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetWindowTextInputState_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetWindowTextInputState_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      ::mojo::TextInputStatePtr p_state{};
      WindowTree_SetWindowTextInputState_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadState(&p_state))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetWindowTextInputState deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetWindowTextInputState(
std::move(p_window_id), 
std::move(p_state));
      return true;
    }
    case internal::kWindowTree_SetImeVisibility_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetImeVisibility_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetImeVisibility_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      bool p_visible{};
      ::mojo::TextInputStatePtr p_state{};
      WindowTree_SetImeVisibility_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_visible = input_data_view.visible();
      if (!input_data_view.ReadState(&p_state))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetImeVisibility deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetImeVisibility(
std::move(p_window_id), 
std::move(p_visible), 
std::move(p_state));
      return true;
    }
    case internal::kWindowTree_SetEventTargetingPolicy_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_SetEventTargetingPolicy_Params_Data* params =
          reinterpret_cast<internal::WindowTree_SetEventTargetingPolicy_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      ::ui::mojom::EventTargetingPolicy p_policy{};
      WindowTree_SetEventTargetingPolicy_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadPolicy(&p_policy))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::SetEventTargetingPolicy deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->SetEventTargetingPolicy(
std::move(p_window_id), 
std::move(p_policy));
      return true;
    }
    case internal::kWindowTree_OnWindowInputEventAck_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_OnWindowInputEventAck_Params_Data* params =
          reinterpret_cast<internal::WindowTree_OnWindowInputEventAck_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_event_id{};
      ::ui::mojom::EventResult p_result{};
      WindowTree_OnWindowInputEventAck_ParamsDataView input_data_view(params, &serialization_context);
      
      p_event_id = input_data_view.event_id();
      if (!input_data_view.ReadResult(&p_result))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::OnWindowInputEventAck deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowInputEventAck(
std::move(p_event_id), 
std::move(p_result));
      return true;
    }
    case internal::kWindowTree_DeactivateWindow_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_DeactivateWindow_Params_Data* params =
          reinterpret_cast<internal::WindowTree_DeactivateWindow_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      WindowTree_DeactivateWindow_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::DeactivateWindow deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->DeactivateWindow(
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_StackAbove_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_StackAbove_Params_Data* params =
          reinterpret_cast<internal::WindowTree_StackAbove_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_above_id{};
      uint32_t p_below_id{};
      WindowTree_StackAbove_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_above_id = input_data_view.above_id();
      p_below_id = input_data_view.below_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::StackAbove deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->StackAbove(
std::move(p_change_id), 
std::move(p_above_id), 
std::move(p_below_id));
      return true;
    }
    case internal::kWindowTree_StackAtTop_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_StackAtTop_Params_Data* params =
          reinterpret_cast<internal::WindowTree_StackAtTop_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      WindowTree_StackAtTop_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::StackAtTop deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->StackAtTop(
std::move(p_change_id), 
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_GetWindowManagerClient_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_GetWindowManagerClient_Params_Data* params =
          reinterpret_cast<internal::WindowTree_GetWindowManagerClient_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::ui::mojom::WindowManagerClientAssociatedRequest p_internal{};
      WindowTree_GetWindowManagerClient_ParamsDataView input_data_view(params, &serialization_context);
      
      p_internal =
          input_data_view.TakeInternal<decltype(p_internal)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::GetWindowManagerClient deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetWindowManagerClient(
std::move(p_internal));
      return true;
    }
    case internal::kWindowTree_GetCursorLocationMemory_Name: {
      break;
    }
    case internal::kWindowTree_PerformWindowMove_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_PerformWindowMove_Params_Data* params =
          reinterpret_cast<internal::WindowTree_PerformWindowMove_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_window_id{};
      ::ui::mojom::MoveLoopSource p_source{};
      gfx::Point p_cursor{};
      WindowTree_PerformWindowMove_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadSource(&p_source))
        success = false;
      if (!input_data_view.ReadCursor(&p_cursor))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::PerformWindowMove deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->PerformWindowMove(
std::move(p_change_id), 
std::move(p_window_id), 
std::move(p_source), 
std::move(p_cursor));
      return true;
    }
    case internal::kWindowTree_CancelWindowMove_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_CancelWindowMove_Params_Data* params =
          reinterpret_cast<internal::WindowTree_CancelWindowMove_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      WindowTree_CancelWindowMove_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::CancelWindowMove deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CancelWindowMove(
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTree_PerformDragDrop_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_PerformDragDrop_Params_Data* params =
          reinterpret_cast<internal::WindowTree_PerformDragDrop_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      uint32_t p_source_window_id{};
      gfx::Point p_screen_location{};
      std::unordered_map<std::string, std::vector<uint8_t>> p_drag_data{};
      SkBitmap p_drag_image{};
      gfx::Vector2d p_drag_image_offset{};
      uint32_t p_drag_operation{};
      ::ui::mojom::PointerKind p_source{};
      WindowTree_PerformDragDrop_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_source_window_id = input_data_view.source_window_id();
      if (!input_data_view.ReadScreenLocation(&p_screen_location))
        success = false;
      if (!input_data_view.ReadDragData(&p_drag_data))
        success = false;
      if (!input_data_view.ReadDragImage(&p_drag_image))
        success = false;
      if (!input_data_view.ReadDragImageOffset(&p_drag_image_offset))
        success = false;
      p_drag_operation = input_data_view.drag_operation();
      if (!input_data_view.ReadSource(&p_source))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::PerformDragDrop deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->PerformDragDrop(
std::move(p_change_id), 
std::move(p_source_window_id), 
std::move(p_screen_location), 
std::move(p_drag_data), 
std::move(p_drag_image), 
std::move(p_drag_image_offset), 
std::move(p_drag_operation), 
std::move(p_source));
      return true;
    }
    case internal::kWindowTree_CancelDragDrop_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTree_CancelDragDrop_Params_Data* params =
          reinterpret_cast<internal::WindowTree_CancelDragDrop_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      WindowTree_CancelDragDrop_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::CancelDragDrop deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CancelDragDrop(
std::move(p_window_id));
      return true;
    }
  }
  return false;
}

// static
bool WindowTreeStubDispatch::AcceptWithResponder(
    WindowTree* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWindowTree_NewWindow_Name: {
      break;
    }
    case internal::kWindowTree_NewTopLevelWindow_Name: {
      break;
    }
    case internal::kWindowTree_DeleteWindow_Name: {
      break;
    }
    case internal::kWindowTree_SetCapture_Name: {
      break;
    }
    case internal::kWindowTree_ReleaseCapture_Name: {
      break;
    }
    case internal::kWindowTree_StartPointerWatcher_Name: {
      break;
    }
    case internal::kWindowTree_StopPointerWatcher_Name: {
      break;
    }
    case internal::kWindowTree_SetWindowBounds_Name: {
      break;
    }
    case internal::kWindowTree_SetWindowTransform_Name: {
      break;
    }
    case internal::kWindowTree_SetClientArea_Name: {
      break;
    }
    case internal::kWindowTree_SetHitTestMask_Name: {
      break;
    }
    case internal::kWindowTree_SetCanAcceptDrops_Name: {
      break;
    }
    case internal::kWindowTree_SetWindowVisibility_Name: {
      break;
    }
    case internal::kWindowTree_SetWindowProperty_Name: {
      break;
    }
    case internal::kWindowTree_SetWindowOpacity_Name: {
      break;
    }
    case internal::kWindowTree_AttachCompositorFrameSink_Name: {
      break;
    }
    case internal::kWindowTree_AddWindow_Name: {
      break;
    }
    case internal::kWindowTree_RemoveWindowFromParent_Name: {
      break;
    }
    case internal::kWindowTree_AddTransientWindow_Name: {
      break;
    }
    case internal::kWindowTree_RemoveTransientWindowFromParent_Name: {
      break;
    }
    case internal::kWindowTree_SetModalType_Name: {
      break;
    }
    case internal::kWindowTree_ReorderWindow_Name: {
      break;
    }
    case internal::kWindowTree_GetWindowTree_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::WindowTree_GetWindowTree_Params_Data* params =
          reinterpret_cast<
              internal::WindowTree_GetWindowTree_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      WindowTree_GetWindowTree_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::GetWindowTree deserializer");
        return false;
      }
      WindowTree::GetWindowTreeCallback callback =
          WindowTree_GetWindowTree_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetWindowTree(
std::move(p_window_id), std::move(callback));
      return true;
    }
    case internal::kWindowTree_Embed_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::WindowTree_Embed_Params_Data* params =
          reinterpret_cast<
              internal::WindowTree_Embed_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      WindowTreeClientPtr p_client{};
      uint32_t p_embed_flags{};
      WindowTree_Embed_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      p_embed_flags = input_data_view.embed_flags();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::Embed deserializer");
        return false;
      }
      WindowTree::EmbedCallback callback =
          WindowTree_Embed_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->Embed(
std::move(p_window_id), 
std::move(p_client), 
std::move(p_embed_flags), std::move(callback));
      return true;
    }
    case internal::kWindowTree_SetFocus_Name: {
      break;
    }
    case internal::kWindowTree_SetCanFocus_Name: {
      break;
    }
    case internal::kWindowTree_SetCursor_Name: {
      break;
    }
    case internal::kWindowTree_SetWindowTextInputState_Name: {
      break;
    }
    case internal::kWindowTree_SetImeVisibility_Name: {
      break;
    }
    case internal::kWindowTree_SetEventTargetingPolicy_Name: {
      break;
    }
    case internal::kWindowTree_OnWindowInputEventAck_Name: {
      break;
    }
    case internal::kWindowTree_DeactivateWindow_Name: {
      break;
    }
    case internal::kWindowTree_StackAbove_Name: {
      break;
    }
    case internal::kWindowTree_StackAtTop_Name: {
      break;
    }
    case internal::kWindowTree_GetWindowManagerClient_Name: {
      break;
    }
    case internal::kWindowTree_GetCursorLocationMemory_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::WindowTree_GetCursorLocationMemory_Params_Data* params =
          reinterpret_cast<
              internal::WindowTree_GetCursorLocationMemory_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WindowTree_GetCursorLocationMemory_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTree::GetCursorLocationMemory deserializer");
        return false;
      }
      WindowTree::GetCursorLocationMemoryCallback callback =
          WindowTree_GetCursorLocationMemory_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetCursorLocationMemory(std::move(callback));
      return true;
    }
    case internal::kWindowTree_PerformWindowMove_Name: {
      break;
    }
    case internal::kWindowTree_CancelWindowMove_Name: {
      break;
    }
    case internal::kWindowTree_PerformDragDrop_Name: {
      break;
    }
    case internal::kWindowTree_CancelDragDrop_Name: {
      break;
    }
  }
  return false;
}

bool WindowTreeRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WindowTree RequestValidator");

  switch (message->header()->name) {
    case internal::kWindowTree_NewWindow_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_NewWindow_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_NewTopLevelWindow_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_NewTopLevelWindow_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_DeleteWindow_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_DeleteWindow_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetCapture_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetCapture_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_ReleaseCapture_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_ReleaseCapture_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_StartPointerWatcher_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_StartPointerWatcher_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_StopPointerWatcher_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_StopPointerWatcher_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetWindowBounds_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetWindowBounds_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetWindowTransform_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetWindowTransform_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetClientArea_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetClientArea_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetHitTestMask_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetHitTestMask_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetCanAcceptDrops_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetCanAcceptDrops_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetWindowVisibility_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetWindowVisibility_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetWindowProperty_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetWindowProperty_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetWindowOpacity_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetWindowOpacity_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_AttachCompositorFrameSink_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_AttachCompositorFrameSink_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_AddWindow_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_AddWindow_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_RemoveWindowFromParent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_RemoveWindowFromParent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_AddTransientWindow_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_AddTransientWindow_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_RemoveTransientWindowFromParent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_RemoveTransientWindowFromParent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetModalType_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetModalType_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_ReorderWindow_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_ReorderWindow_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_GetWindowTree_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_GetWindowTree_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_Embed_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_Embed_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetFocus_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetFocus_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetCanFocus_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetCanFocus_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetCursor_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetCursor_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetWindowTextInputState_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetWindowTextInputState_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetImeVisibility_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetImeVisibility_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_SetEventTargetingPolicy_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_SetEventTargetingPolicy_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_OnWindowInputEventAck_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_OnWindowInputEventAck_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_DeactivateWindow_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_DeactivateWindow_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_StackAbove_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_StackAbove_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_StackAtTop_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_StackAtTop_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_GetWindowManagerClient_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_GetWindowManagerClient_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_GetCursorLocationMemory_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_GetCursorLocationMemory_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_PerformWindowMove_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_PerformWindowMove_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_CancelWindowMove_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_CancelWindowMove_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_PerformDragDrop_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_PerformDragDrop_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_CancelDragDrop_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_CancelDragDrop_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool WindowTreeResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WindowTree ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kWindowTree_GetWindowTree_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_GetWindowTree_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_Embed_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_Embed_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTree_GetCursorLocationMemory_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTree_GetCursorLocationMemory_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char WindowTreeClient::Name_[] = "ui::mojom::WindowTreeClient";

class WindowTreeClient_OnDragEnter_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  WindowTreeClient_OnDragEnter_ForwardToCallback(
      const WindowTreeClient::OnDragEnterCallback& callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  WindowTreeClient::OnDragEnterCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(WindowTreeClient_OnDragEnter_ForwardToCallback);
};

class WindowTreeClient_OnDragOver_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  WindowTreeClient_OnDragOver_ForwardToCallback(
      const WindowTreeClient::OnDragOverCallback& callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  WindowTreeClient::OnDragOverCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(WindowTreeClient_OnDragOver_ForwardToCallback);
};

class WindowTreeClient_OnCompleteDrop_ForwardToCallback
    : public mojo::MessageReceiver {
 public:
  WindowTreeClient_OnCompleteDrop_ForwardToCallback(
      const WindowTreeClient::OnCompleteDropCallback& callback
      ) : callback_(std::move(callback)) {
  }
  bool Accept(mojo::Message* message) override;
 private:
  WindowTreeClient::OnCompleteDropCallback callback_;
  DISALLOW_COPY_AND_ASSIGN(WindowTreeClient_OnCompleteDrop_ForwardToCallback);
};

WindowTreeClientProxy::WindowTreeClientProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WindowTreeClientProxy::OnEmbed(
    uint16_t in_connection_id, ::ui::mojom::WindowDataPtr in_root, WindowTreePtr in_tree, int64_t in_display_id, uint32_t in_focused_window, bool in_parent_drawn, const base::Optional<viz::LocalSurfaceId>& in_local_surface_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnEmbed_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowDataDataView>(
      in_root, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowTreePtrDataView>(
      in_tree, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnEmbed_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnEmbed_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->connection_id = in_connection_id;
  typename decltype(params->root)::BaseType* root_ptr;
  mojo::internal::Serialize<::ui::mojom::WindowDataDataView>(
      in_root, buffer, &root_ptr, &serialization_context);
  params->root.Set(root_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->root.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null root in WindowTreeClient.OnEmbed request");
  mojo::internal::Serialize<::ui::mojom::WindowTreePtrDataView>(
      in_tree, &params->tree, &serialization_context);
  params->display_id = in_display_id;
  params->focused_window = in_focused_window;
  params->parent_drawn = in_parent_drawn;
  typename decltype(params->local_surface_id)::BaseType* local_surface_id_ptr;
  mojo::internal::Serialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, buffer, &local_surface_id_ptr, &serialization_context);
  params->local_surface_id.Set(local_surface_id_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnEmbeddedAppDisconnected(
    uint32_t in_window) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnEmbeddedAppDisconnected_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnEmbeddedAppDisconnected_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnEmbeddedAppDisconnected_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnUnembed(
    uint32_t in_window) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnUnembed_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnUnembed_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnUnembed_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnCaptureChanged(
    uint32_t in_new_capture, uint32_t in_old_capture) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnCaptureChanged_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnCaptureChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnCaptureChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->new_capture = in_new_capture;
  params->old_capture = in_old_capture;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnFrameSinkIdAllocated(
    uint32_t in_window, const viz::FrameSinkId& in_frame_sink_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnFrameSinkIdAllocated_Params_Data);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::FrameSinkIdDataView>(
      in_frame_sink_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnFrameSinkIdAllocated_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnFrameSinkIdAllocated_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  typename decltype(params->frame_sink_id)::BaseType* frame_sink_id_ptr;
  mojo::internal::Serialize<::cc::mojom::FrameSinkIdDataView>(
      in_frame_sink_id, buffer, &frame_sink_id_ptr, &serialization_context);
  params->frame_sink_id.Set(frame_sink_id_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->frame_sink_id.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null frame_sink_id in WindowTreeClient.OnFrameSinkIdAllocated request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnTopLevelCreated(
    uint32_t in_change_id, ::ui::mojom::WindowDataPtr in_data, int64_t in_display_id, bool in_parent_drawn, const base::Optional<viz::LocalSurfaceId>& in_local_surface_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnTopLevelCreated_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowDataDataView>(
      in_data, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnTopLevelCreated_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnTopLevelCreated_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  typename decltype(params->data)::BaseType* data_ptr;
  mojo::internal::Serialize<::ui::mojom::WindowDataDataView>(
      in_data, buffer, &data_ptr, &serialization_context);
  params->data.Set(data_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null data in WindowTreeClient.OnTopLevelCreated request");
  params->display_id = in_display_id;
  params->parent_drawn = in_parent_drawn;
  typename decltype(params->local_surface_id)::BaseType* local_surface_id_ptr;
  mojo::internal::Serialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, buffer, &local_surface_id_ptr, &serialization_context);
  params->local_surface_id.Set(local_surface_id_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowBoundsChanged(
    uint32_t in_window, const gfx::Rect& in_old_bounds, const gfx::Rect& in_new_bounds, const base::Optional<viz::LocalSurfaceId>& in_local_surface_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowBoundsChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
      in_old_bounds, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::RectDataView>(
      in_new_bounds, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowBoundsChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowBoundsChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  typename decltype(params->old_bounds)::BaseType* old_bounds_ptr;
  mojo::internal::Serialize<::gfx::mojom::RectDataView>(
      in_old_bounds, buffer, &old_bounds_ptr, &serialization_context);
  params->old_bounds.Set(old_bounds_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->old_bounds.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null old_bounds in WindowTreeClient.OnWindowBoundsChanged request");
  typename decltype(params->new_bounds)::BaseType* new_bounds_ptr;
  mojo::internal::Serialize<::gfx::mojom::RectDataView>(
      in_new_bounds, buffer, &new_bounds_ptr, &serialization_context);
  params->new_bounds.Set(new_bounds_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_bounds.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_bounds in WindowTreeClient.OnWindowBoundsChanged request");
  typename decltype(params->local_surface_id)::BaseType* local_surface_id_ptr;
  mojo::internal::Serialize<::cc::mojom::LocalSurfaceIdDataView>(
      in_local_surface_id, buffer, &local_surface_id_ptr, &serialization_context);
  params->local_surface_id.Set(local_surface_id_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowTransformChanged(
    uint32_t in_window, const gfx::Transform& in_old_transform, const gfx::Transform& in_new_transform) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowTransformChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::TransformDataView>(
      in_old_transform, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::TransformDataView>(
      in_new_transform, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowTransformChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowTransformChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  typename decltype(params->old_transform)::BaseType* old_transform_ptr;
  mojo::internal::Serialize<::gfx::mojom::TransformDataView>(
      in_old_transform, buffer, &old_transform_ptr, &serialization_context);
  params->old_transform.Set(old_transform_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->old_transform.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null old_transform in WindowTreeClient.OnWindowTransformChanged request");
  typename decltype(params->new_transform)::BaseType* new_transform_ptr;
  mojo::internal::Serialize<::gfx::mojom::TransformDataView>(
      in_new_transform, buffer, &new_transform_ptr, &serialization_context);
  params->new_transform.Set(new_transform_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_transform.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_transform in WindowTreeClient.OnWindowTransformChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnClientAreaChanged(
    uint32_t in_window_id, const gfx::Insets& in_new_client_area, const std::vector<gfx::Rect>& in_new_additional_client_areas) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnClientAreaChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::InsetsDataView>(
      in_new_client_area, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::gfx::mojom::RectDataView>>(
      in_new_additional_client_areas, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnClientAreaChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnClientAreaChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  typename decltype(params->new_client_area)::BaseType* new_client_area_ptr;
  mojo::internal::Serialize<::gfx::mojom::InsetsDataView>(
      in_new_client_area, buffer, &new_client_area_ptr, &serialization_context);
  params->new_client_area.Set(new_client_area_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_client_area.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_client_area in WindowTreeClient.OnClientAreaChanged request");
  typename decltype(params->new_additional_client_areas)::BaseType* new_additional_client_areas_ptr;
  const mojo::internal::ContainerValidateParams new_additional_client_areas_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::gfx::mojom::RectDataView>>(
      in_new_additional_client_areas, buffer, &new_additional_client_areas_ptr, &new_additional_client_areas_validate_params,
      &serialization_context);
  params->new_additional_client_areas.Set(new_additional_client_areas_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->new_additional_client_areas.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null new_additional_client_areas in WindowTreeClient.OnClientAreaChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnTransientWindowAdded(
    uint32_t in_window_id, uint32_t in_transient_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnTransientWindowAdded_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnTransientWindowAdded_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnTransientWindowAdded_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  params->transient_window_id = in_transient_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnTransientWindowRemoved(
    uint32_t in_window_id, uint32_t in_transient_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnTransientWindowRemoved_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnTransientWindowRemoved_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnTransientWindowRemoved_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  params->transient_window_id = in_transient_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowHierarchyChanged(
    uint32_t in_window, uint32_t in_old_parent, uint32_t in_new_parent, std::vector<::ui::mojom::WindowDataPtr> in_windows) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowHierarchyChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<::ui::mojom::WindowDataDataView>>(
      in_windows, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowHierarchyChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowHierarchyChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->old_parent = in_old_parent;
  params->new_parent = in_new_parent;
  typename decltype(params->windows)::BaseType* windows_ptr;
  const mojo::internal::ContainerValidateParams windows_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<::ui::mojom::WindowDataDataView>>(
      in_windows, buffer, &windows_ptr, &windows_validate_params,
      &serialization_context);
  params->windows.Set(windows_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->windows.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null windows in WindowTreeClient.OnWindowHierarchyChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowReordered(
    uint32_t in_window_id, uint32_t in_relative_window_id, ::ui::mojom::OrderDirection in_direction) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowReordered_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowReordered_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowReordered_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  params->relative_window_id = in_relative_window_id;
  mojo::internal::Serialize<::ui::mojom::OrderDirection>(
      in_direction, &params->direction);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowDeleted(
    uint32_t in_window) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowDeleted_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowDeleted_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowDeleted_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowVisibilityChanged(
    uint32_t in_window, bool in_visible) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowVisibilityChanged_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowVisibilityChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowVisibilityChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->visible = in_visible;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowOpacityChanged(
    uint32_t in_window, float in_old_opacity, float in_new_opacity) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowOpacityChanged_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowOpacityChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowOpacityChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->old_opacity = in_old_opacity;
  params->new_opacity = in_new_opacity;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowParentDrawnStateChanged(
    uint32_t in_window, bool in_drawn) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowParentDrawnStateChanged_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowParentDrawnStateChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowParentDrawnStateChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->drawn = in_drawn;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowSharedPropertyChanged(
    uint32_t in_window, const std::string& in_name, const base::Optional<std::vector<uint8_t>>& in_new_data) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowSharedPropertyChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::StringDataView>(
      in_name, &serialization_context);
  size += mojo::internal::PrepareToSerialize<mojo::ArrayDataView<uint8_t>>(
      in_new_data, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowSharedPropertyChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowSharedPropertyChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  typename decltype(params->name)::BaseType* name_ptr;
  mojo::internal::Serialize<mojo::StringDataView>(
      in_name, buffer, &name_ptr, &serialization_context);
  params->name.Set(name_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->name.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null name in WindowTreeClient.OnWindowSharedPropertyChanged request");
  typename decltype(params->new_data)::BaseType* new_data_ptr;
  const mojo::internal::ContainerValidateParams new_data_validate_params(
      0, false, nullptr);
  mojo::internal::Serialize<mojo::ArrayDataView<uint8_t>>(
      in_new_data, buffer, &new_data_ptr, &new_data_validate_params,
      &serialization_context);
  params->new_data.Set(new_data_ptr);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowInputEvent(
    uint32_t in_event_id, uint32_t in_window, int64_t in_display_id, std::unique_ptr<ui::Event> in_event, bool in_matches_pointer_watcher) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowInputEvent_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::EventDataView>(
      in_event, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowInputEvent_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowInputEvent_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->event_id = in_event_id;
  params->window = in_window;
  params->display_id = in_display_id;
  typename decltype(params->event)::BaseType* event_ptr;
  mojo::internal::Serialize<::ui::mojom::EventDataView>(
      in_event, buffer, &event_ptr, &serialization_context);
  params->event.Set(event_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->event.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null event in WindowTreeClient.OnWindowInputEvent request");
  params->matches_pointer_watcher = in_matches_pointer_watcher;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnPointerEventObserved(
    std::unique_ptr<ui::Event> in_event, uint32_t in_window_id, int64_t in_display_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnPointerEventObserved_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::EventDataView>(
      in_event, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnPointerEventObserved_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnPointerEventObserved_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->event)::BaseType* event_ptr;
  mojo::internal::Serialize<::ui::mojom::EventDataView>(
      in_event, buffer, &event_ptr, &serialization_context);
  params->event.Set(event_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->event.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null event in WindowTreeClient.OnPointerEventObserved request");
  params->window_id = in_window_id;
  params->display_id = in_display_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowFocused(
    uint32_t in_focused_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowFocused_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowFocused_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowFocused_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->focused_window_id = in_focused_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowCursorChanged(
    uint32_t in_window_id, ui::CursorData in_cursor) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowCursorChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::CursorDataDataView>(
      in_cursor, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowCursorChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowCursorChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  typename decltype(params->cursor)::BaseType* cursor_ptr;
  mojo::internal::Serialize<::ui::mojom::CursorDataDataView>(
      in_cursor, buffer, &cursor_ptr, &serialization_context);
  params->cursor.Set(cursor_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->cursor.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null cursor in WindowTreeClient.OnWindowCursorChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnWindowSurfaceChanged(
    uint32_t in_window_id, const viz::SurfaceInfo& in_surface_info) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnWindowSurfaceChanged_Params_Data);
  size += mojo::internal::PrepareToSerialize<::cc::mojom::SurfaceInfoDataView>(
      in_surface_info, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnWindowSurfaceChanged_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnWindowSurfaceChanged_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  typename decltype(params->surface_info)::BaseType* surface_info_ptr;
  mojo::internal::Serialize<::cc::mojom::SurfaceInfoDataView>(
      in_surface_info, buffer, &surface_info_ptr, &serialization_context);
  params->surface_info.Set(surface_info_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->surface_info.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null surface_info in WindowTreeClient.OnWindowSurfaceChanged request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnDragDropStart(
    const std::unordered_map<std::string, std::vector<uint8_t>>& in_drag_data) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnDragDropStart_Params_Data);
  size += mojo::internal::PrepareToSerialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_drag_data, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnDragDropStart_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnDragDropStart_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  typename decltype(params->drag_data)::BaseType* drag_data_ptr;
  const mojo::internal::ContainerValidateParams drag_data_validate_params(
      new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)), new mojo::internal::ContainerValidateParams(0, false, new mojo::internal::ContainerValidateParams(0, false, nullptr)));
  mojo::internal::Serialize<mojo::MapDataView<mojo::StringDataView, mojo::ArrayDataView<uint8_t>>>(
      in_drag_data, buffer, &drag_data_ptr, &drag_data_validate_params,
      &serialization_context);
  params->drag_data.Set(drag_data_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->drag_data.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null drag_data in WindowTreeClient.OnDragDropStart request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnDragEnter(
    uint32_t in_window, uint32_t in_key_state, const gfx::Point& in_screen_position, uint32_t in_effect_bitmask, const OnDragEnterCallback& callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnDragEnter_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::PointDataView>(
      in_screen_position, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnDragEnter_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnDragEnter_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->key_state = in_key_state;
  typename decltype(params->screen_position)::BaseType* screen_position_ptr;
  mojo::internal::Serialize<::gfx::mojom::PointDataView>(
      in_screen_position, buffer, &screen_position_ptr, &serialization_context);
  params->screen_position.Set(screen_position_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->screen_position.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null screen_position in WindowTreeClient.OnDragEnter request");
  params->effect_bitmask = in_effect_bitmask;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new WindowTreeClient_OnDragEnter_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void WindowTreeClientProxy::OnDragOver(
    uint32_t in_window, uint32_t in_key_state, const gfx::Point& in_screen_position, uint32_t in_effect_bitmask, const OnDragOverCallback& callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnDragOver_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::PointDataView>(
      in_screen_position, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnDragOver_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnDragOver_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->key_state = in_key_state;
  typename decltype(params->screen_position)::BaseType* screen_position_ptr;
  mojo::internal::Serialize<::gfx::mojom::PointDataView>(
      in_screen_position, buffer, &screen_position_ptr, &serialization_context);
  params->screen_position.Set(screen_position_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->screen_position.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null screen_position in WindowTreeClient.OnDragOver request");
  params->effect_bitmask = in_effect_bitmask;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new WindowTreeClient_OnDragOver_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void WindowTreeClientProxy::OnDragLeave(
    uint32_t in_window) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnDragLeave_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnDragLeave_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnDragLeave_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnCompleteDrop(
    uint32_t in_window, uint32_t in_key_state, const gfx::Point& in_screen_position, uint32_t in_effect_bitmask, const OnCompleteDropCallback& callback) {
  const bool kExpectsResponse = true;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnCompleteDrop_Params_Data);
  size += mojo::internal::PrepareToSerialize<::gfx::mojom::PointDataView>(
      in_screen_position, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnCompleteDrop_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnCompleteDrop_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->key_state = in_key_state;
  typename decltype(params->screen_position)::BaseType* screen_position_ptr;
  mojo::internal::Serialize<::gfx::mojom::PointDataView>(
      in_screen_position, buffer, &screen_position_ptr, &serialization_context);
  params->screen_position.Set(screen_position_ptr);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      params->screen_position.is_null(),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_NULL_POINTER,
      "null screen_position in WindowTreeClient.OnCompleteDrop request");
  params->effect_bitmask = in_effect_bitmask;
  std::unique_ptr<mojo::MessageReceiver> responder(
      new WindowTreeClient_OnCompleteDrop_ForwardToCallback(
          std::move(callback)));
  ignore_result(receiver_->AcceptWithResponder(&message, std::move(responder)));
}

void WindowTreeClientProxy::OnPerformDragDropCompleted(
    uint32_t in_window, bool in_success, uint32_t in_action_taken) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnPerformDragDropCompleted_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnPerformDragDropCompleted_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnPerformDragDropCompleted_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window = in_window;
  params->success = in_success;
  params->action_taken = in_action_taken;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnDragDropDone(
    ) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnDragDropDone_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnDragDropDone_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnDragDropDone_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::OnChangeCompleted(
    uint32_t in_change_id, bool in_success) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnChangeCompleted_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnChangeCompleted_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnChangeCompleted_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->change_id = in_change_id;
  params->success = in_success;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::RequestClose(
    uint32_t in_window_id) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_RequestClose_Params_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_RequestClose_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_RequestClose_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->window_id = in_window_id;
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

void WindowTreeClientProxy::GetWindowManager(
    ::ui::mojom::WindowManagerAssociatedRequest in_internal) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_GetWindowManager_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowManagerAssociatedRequestDataView>(
      in_internal, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_GetWindowManager_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_GetWindowManager_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::ui::mojom::WindowManagerAssociatedRequestDataView>(
      in_internal, &params->internal, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->internal),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_INTERFACE_ID,
      "invalid internal in WindowTreeClient.GetWindowManager request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}
class WindowTreeClient_OnDragEnter_ProxyToResponder {
 public:
  static WindowTreeClient::OnDragEnterCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<WindowTreeClient_OnDragEnter_ProxyToResponder> proxy(
        new WindowTreeClient_OnDragEnter_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&WindowTreeClient_OnDragEnter_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~WindowTreeClient_OnDragEnter_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "WindowTreeClient::OnDragEnter() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  WindowTreeClient_OnDragEnter_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      uint32_t in_supported_op_bitmask);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(WindowTreeClient_OnDragEnter_ProxyToResponder);
};

bool WindowTreeClient_OnDragEnter_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::WindowTreeClient_OnDragEnter_ResponseParams_Data* params =
      reinterpret_cast<
          internal::WindowTreeClient_OnDragEnter_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  uint32_t p_supported_op_bitmask{};
  WindowTreeClient_OnDragEnter_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_supported_op_bitmask = input_data_view.supported_op_bitmask();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "WindowTreeClient::OnDragEnter response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_supported_op_bitmask));
  return true;
}

void WindowTreeClient_OnDragEnter_ProxyToResponder::Run(
    uint32_t in_supported_op_bitmask) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnDragEnter_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnDragEnter_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnDragEnter_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->supported_op_bitmask = in_supported_op_bitmask;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class WindowTreeClient_OnDragOver_ProxyToResponder {
 public:
  static WindowTreeClient::OnDragOverCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<WindowTreeClient_OnDragOver_ProxyToResponder> proxy(
        new WindowTreeClient_OnDragOver_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&WindowTreeClient_OnDragOver_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~WindowTreeClient_OnDragOver_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "WindowTreeClient::OnDragOver() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  WindowTreeClient_OnDragOver_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      uint32_t in_supported_op_bitmask);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(WindowTreeClient_OnDragOver_ProxyToResponder);
};

bool WindowTreeClient_OnDragOver_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::WindowTreeClient_OnDragOver_ResponseParams_Data* params =
      reinterpret_cast<
          internal::WindowTreeClient_OnDragOver_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  uint32_t p_supported_op_bitmask{};
  WindowTreeClient_OnDragOver_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_supported_op_bitmask = input_data_view.supported_op_bitmask();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "WindowTreeClient::OnDragOver response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_supported_op_bitmask));
  return true;
}

void WindowTreeClient_OnDragOver_ProxyToResponder::Run(
    uint32_t in_supported_op_bitmask) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnDragOver_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnDragOver_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnDragOver_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->supported_op_bitmask = in_supported_op_bitmask;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}
class WindowTreeClient_OnCompleteDrop_ProxyToResponder {
 public:
  static WindowTreeClient::OnCompleteDropCallback CreateCallback(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
    std::unique_ptr<WindowTreeClient_OnCompleteDrop_ProxyToResponder> proxy(
        new WindowTreeClient_OnCompleteDrop_ProxyToResponder(
            request_id, is_sync, std::move(responder)));
    return base::Bind(&WindowTreeClient_OnCompleteDrop_ProxyToResponder::Run,
                      base::Passed(&proxy));
  }

  ~WindowTreeClient_OnCompleteDrop_ProxyToResponder() {
#if DCHECK_IS_ON()
    if (responder_) {
      // Is the Service destroying the callback without running it
      // and without first closing the pipe?
      responder_->DCheckInvalid("The callback passed to "
          "WindowTreeClient::OnCompleteDrop() was never run.");
    }
#endif
    // If the Callback was dropped then deleting the responder will close
    // the pipe so the calling application knows to stop waiting for a reply.
    responder_ = nullptr;
  }

 private:
  WindowTreeClient_OnCompleteDrop_ProxyToResponder(
      uint64_t request_id,
      bool is_sync,
      std::unique_ptr<mojo::MessageReceiverWithStatus> responder)
      : request_id_(request_id),
        is_sync_(is_sync),
        responder_(std::move(responder)) {
  }

  void Run(
      uint32_t in_action_taken);

  uint64_t request_id_;
  bool is_sync_;
  std::unique_ptr<mojo::MessageReceiverWithStatus> responder_;

  DISALLOW_COPY_AND_ASSIGN(WindowTreeClient_OnCompleteDrop_ProxyToResponder);
};

bool WindowTreeClient_OnCompleteDrop_ForwardToCallback::Accept(
    mojo::Message* message) {
  mojo::internal::MessageDispatchContext dispatch_context(message);

  DCHECK(message->is_serialized());
  internal::WindowTreeClient_OnCompleteDrop_ResponseParams_Data* params =
      reinterpret_cast<
          internal::WindowTreeClient_OnCompleteDrop_ResponseParams_Data*>(
              message->mutable_payload());
  
  mojo::internal::SerializationContext serialization_context;
  serialization_context.TakeHandlesFromMessage(message);
  bool success = true;
  uint32_t p_action_taken{};
  WindowTreeClient_OnCompleteDrop_ResponseParamsDataView input_data_view(params, &serialization_context);
  
  p_action_taken = input_data_view.action_taken();
  if (!success) {
    ReportValidationErrorForMessage(
        message,
        mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
        "WindowTreeClient::OnCompleteDrop response deserializer");
    return false;
  }
  if (!callback_.is_null())
    std::move(callback_).Run(
std::move(p_action_taken));
  return true;
}

void WindowTreeClient_OnCompleteDrop_ProxyToResponder::Run(
    uint32_t in_action_taken) {
  mojo::Message message;
  
  const uint32_t kFlags = mojo::Message::kFlagIsResponse |
      ((is_sync_) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeClient_OnCompleteDrop_ResponseParams_Data);
  serialization_context.PrepareMessage(
      internal::kWindowTreeClient_OnCompleteDrop_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeClient_OnCompleteDrop_ResponseParams_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  params->action_taken = in_action_taken;

  message.set_request_id(request_id_);
  ignore_result(responder_->Accept(&message));
  // TODO(darin): Accept() returning false indicates a malformed message, and
  // that may be good reason to close the connection. However, we don't have a
  // way to do that from here. We should add a way.
  responder_ = nullptr;
}

// static
bool WindowTreeClientStubDispatch::Accept(
    WindowTreeClient* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWindowTreeClient_OnEmbed_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnEmbed_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnEmbed_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint16_t p_connection_id{};
      ::ui::mojom::WindowDataPtr p_root{};
      WindowTreePtr p_tree{};
      int64_t p_display_id{};
      uint32_t p_focused_window{};
      bool p_parent_drawn{};
      base::Optional<viz::LocalSurfaceId> p_local_surface_id{};
      WindowTreeClient_OnEmbed_ParamsDataView input_data_view(params, &serialization_context);
      
      p_connection_id = input_data_view.connection_id();
      if (!input_data_view.ReadRoot(&p_root))
        success = false;
      p_tree =
          input_data_view.TakeTree<decltype(p_tree)>();
      p_display_id = input_data_view.display_id();
      p_focused_window = input_data_view.focused_window();
      p_parent_drawn = input_data_view.parent_drawn();
      if (!input_data_view.ReadLocalSurfaceId(&p_local_surface_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnEmbed deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnEmbed(
std::move(p_connection_id), 
std::move(p_root), 
std::move(p_tree), 
std::move(p_display_id), 
std::move(p_focused_window), 
std::move(p_parent_drawn), 
std::move(p_local_surface_id));
      return true;
    }
    case internal::kWindowTreeClient_OnEmbeddedAppDisconnected_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnEmbeddedAppDisconnected_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnEmbeddedAppDisconnected_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      WindowTreeClient_OnEmbeddedAppDisconnected_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnEmbeddedAppDisconnected deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnEmbeddedAppDisconnected(
std::move(p_window));
      return true;
    }
    case internal::kWindowTreeClient_OnUnembed_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnUnembed_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnUnembed_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      WindowTreeClient_OnUnembed_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnUnembed deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnUnembed(
std::move(p_window));
      return true;
    }
    case internal::kWindowTreeClient_OnCaptureChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnCaptureChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnCaptureChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_new_capture{};
      uint32_t p_old_capture{};
      WindowTreeClient_OnCaptureChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_new_capture = input_data_view.new_capture();
      p_old_capture = input_data_view.old_capture();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnCaptureChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCaptureChanged(
std::move(p_new_capture), 
std::move(p_old_capture));
      return true;
    }
    case internal::kWindowTreeClient_OnFrameSinkIdAllocated_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnFrameSinkIdAllocated_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnFrameSinkIdAllocated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      viz::FrameSinkId p_frame_sink_id{};
      WindowTreeClient_OnFrameSinkIdAllocated_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!input_data_view.ReadFrameSinkId(&p_frame_sink_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnFrameSinkIdAllocated deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnFrameSinkIdAllocated(
std::move(p_window), 
std::move(p_frame_sink_id));
      return true;
    }
    case internal::kWindowTreeClient_OnTopLevelCreated_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnTopLevelCreated_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnTopLevelCreated_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      ::ui::mojom::WindowDataPtr p_data{};
      int64_t p_display_id{};
      bool p_parent_drawn{};
      base::Optional<viz::LocalSurfaceId> p_local_surface_id{};
      WindowTreeClient_OnTopLevelCreated_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      if (!input_data_view.ReadData(&p_data))
        success = false;
      p_display_id = input_data_view.display_id();
      p_parent_drawn = input_data_view.parent_drawn();
      if (!input_data_view.ReadLocalSurfaceId(&p_local_surface_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnTopLevelCreated deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnTopLevelCreated(
std::move(p_change_id), 
std::move(p_data), 
std::move(p_display_id), 
std::move(p_parent_drawn), 
std::move(p_local_surface_id));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowBoundsChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowBoundsChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowBoundsChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      gfx::Rect p_old_bounds{};
      gfx::Rect p_new_bounds{};
      base::Optional<viz::LocalSurfaceId> p_local_surface_id{};
      WindowTreeClient_OnWindowBoundsChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!input_data_view.ReadOldBounds(&p_old_bounds))
        success = false;
      if (!input_data_view.ReadNewBounds(&p_new_bounds))
        success = false;
      if (!input_data_view.ReadLocalSurfaceId(&p_local_surface_id))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowBoundsChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowBoundsChanged(
std::move(p_window), 
std::move(p_old_bounds), 
std::move(p_new_bounds), 
std::move(p_local_surface_id));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowTransformChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowTransformChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowTransformChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      gfx::Transform p_old_transform{};
      gfx::Transform p_new_transform{};
      WindowTreeClient_OnWindowTransformChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!input_data_view.ReadOldTransform(&p_old_transform))
        success = false;
      if (!input_data_view.ReadNewTransform(&p_new_transform))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowTransformChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowTransformChanged(
std::move(p_window), 
std::move(p_old_transform), 
std::move(p_new_transform));
      return true;
    }
    case internal::kWindowTreeClient_OnClientAreaChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnClientAreaChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnClientAreaChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      gfx::Insets p_new_client_area{};
      std::vector<gfx::Rect> p_new_additional_client_areas{};
      WindowTreeClient_OnClientAreaChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadNewClientArea(&p_new_client_area))
        success = false;
      if (!input_data_view.ReadNewAdditionalClientAreas(&p_new_additional_client_areas))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnClientAreaChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnClientAreaChanged(
std::move(p_window_id), 
std::move(p_new_client_area), 
std::move(p_new_additional_client_areas));
      return true;
    }
    case internal::kWindowTreeClient_OnTransientWindowAdded_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnTransientWindowAdded_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnTransientWindowAdded_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      uint32_t p_transient_window_id{};
      WindowTreeClient_OnTransientWindowAdded_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_transient_window_id = input_data_view.transient_window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnTransientWindowAdded deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnTransientWindowAdded(
std::move(p_window_id), 
std::move(p_transient_window_id));
      return true;
    }
    case internal::kWindowTreeClient_OnTransientWindowRemoved_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnTransientWindowRemoved_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnTransientWindowRemoved_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      uint32_t p_transient_window_id{};
      WindowTreeClient_OnTransientWindowRemoved_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_transient_window_id = input_data_view.transient_window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnTransientWindowRemoved deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnTransientWindowRemoved(
std::move(p_window_id), 
std::move(p_transient_window_id));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowHierarchyChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowHierarchyChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowHierarchyChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      uint32_t p_old_parent{};
      uint32_t p_new_parent{};
      std::vector<::ui::mojom::WindowDataPtr> p_windows{};
      WindowTreeClient_OnWindowHierarchyChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_old_parent = input_data_view.old_parent();
      p_new_parent = input_data_view.new_parent();
      if (!input_data_view.ReadWindows(&p_windows))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowHierarchyChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowHierarchyChanged(
std::move(p_window), 
std::move(p_old_parent), 
std::move(p_new_parent), 
std::move(p_windows));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowReordered_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowReordered_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowReordered_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      uint32_t p_relative_window_id{};
      ::ui::mojom::OrderDirection p_direction{};
      WindowTreeClient_OnWindowReordered_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      p_relative_window_id = input_data_view.relative_window_id();
      if (!input_data_view.ReadDirection(&p_direction))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowReordered deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowReordered(
std::move(p_window_id), 
std::move(p_relative_window_id), 
std::move(p_direction));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowDeleted_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowDeleted_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowDeleted_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      WindowTreeClient_OnWindowDeleted_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowDeleted deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowDeleted(
std::move(p_window));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowVisibilityChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowVisibilityChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowVisibilityChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      bool p_visible{};
      WindowTreeClient_OnWindowVisibilityChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_visible = input_data_view.visible();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowVisibilityChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowVisibilityChanged(
std::move(p_window), 
std::move(p_visible));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowOpacityChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowOpacityChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowOpacityChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      float p_old_opacity{};
      float p_new_opacity{};
      WindowTreeClient_OnWindowOpacityChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_old_opacity = input_data_view.old_opacity();
      p_new_opacity = input_data_view.new_opacity();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowOpacityChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowOpacityChanged(
std::move(p_window), 
std::move(p_old_opacity), 
std::move(p_new_opacity));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowParentDrawnStateChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowParentDrawnStateChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowParentDrawnStateChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      bool p_drawn{};
      WindowTreeClient_OnWindowParentDrawnStateChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_drawn = input_data_view.drawn();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowParentDrawnStateChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowParentDrawnStateChanged(
std::move(p_window), 
std::move(p_drawn));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowSharedPropertyChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowSharedPropertyChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowSharedPropertyChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      std::string p_name{};
      base::Optional<std::vector<uint8_t>> p_new_data{};
      WindowTreeClient_OnWindowSharedPropertyChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!input_data_view.ReadName(&p_name))
        success = false;
      if (!input_data_view.ReadNewData(&p_new_data))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowSharedPropertyChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowSharedPropertyChanged(
std::move(p_window), 
std::move(p_name), 
std::move(p_new_data));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowInputEvent_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowInputEvent_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowInputEvent_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_event_id{};
      uint32_t p_window{};
      int64_t p_display_id{};
      std::unique_ptr<ui::Event> p_event{};
      bool p_matches_pointer_watcher{};
      WindowTreeClient_OnWindowInputEvent_ParamsDataView input_data_view(params, &serialization_context);
      
      p_event_id = input_data_view.event_id();
      p_window = input_data_view.window();
      p_display_id = input_data_view.display_id();
      if (!input_data_view.ReadEvent(&p_event))
        success = false;
      p_matches_pointer_watcher = input_data_view.matches_pointer_watcher();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowInputEvent deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowInputEvent(
std::move(p_event_id), 
std::move(p_window), 
std::move(p_display_id), 
std::move(p_event), 
std::move(p_matches_pointer_watcher));
      return true;
    }
    case internal::kWindowTreeClient_OnPointerEventObserved_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnPointerEventObserved_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnPointerEventObserved_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::unique_ptr<ui::Event> p_event{};
      uint32_t p_window_id{};
      int64_t p_display_id{};
      WindowTreeClient_OnPointerEventObserved_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadEvent(&p_event))
        success = false;
      p_window_id = input_data_view.window_id();
      p_display_id = input_data_view.display_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnPointerEventObserved deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnPointerEventObserved(
std::move(p_event), 
std::move(p_window_id), 
std::move(p_display_id));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowFocused_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowFocused_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowFocused_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_focused_window_id{};
      WindowTreeClient_OnWindowFocused_ParamsDataView input_data_view(params, &serialization_context);
      
      p_focused_window_id = input_data_view.focused_window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowFocused deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowFocused(
std::move(p_focused_window_id));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowCursorChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowCursorChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowCursorChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      ui::CursorData p_cursor{};
      WindowTreeClient_OnWindowCursorChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadCursor(&p_cursor))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowCursorChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowCursorChanged(
std::move(p_window_id), 
std::move(p_cursor));
      return true;
    }
    case internal::kWindowTreeClient_OnWindowSurfaceChanged_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnWindowSurfaceChanged_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnWindowSurfaceChanged_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      viz::SurfaceInfo p_surface_info{};
      WindowTreeClient_OnWindowSurfaceChanged_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!input_data_view.ReadSurfaceInfo(&p_surface_info))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnWindowSurfaceChanged deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnWindowSurfaceChanged(
std::move(p_window_id), 
std::move(p_surface_info));
      return true;
    }
    case internal::kWindowTreeClient_OnDragDropStart_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnDragDropStart_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnDragDropStart_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      std::unordered_map<std::string, std::vector<uint8_t>> p_drag_data{};
      WindowTreeClient_OnDragDropStart_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!input_data_view.ReadDragData(&p_drag_data))
        success = false;
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnDragDropStart deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDragDropStart(
std::move(p_drag_data));
      return true;
    }
    case internal::kWindowTreeClient_OnDragEnter_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnDragOver_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnDragLeave_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnDragLeave_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnDragLeave_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      WindowTreeClient_OnDragLeave_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnDragLeave deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDragLeave(
std::move(p_window));
      return true;
    }
    case internal::kWindowTreeClient_OnCompleteDrop_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnPerformDragDropCompleted_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnPerformDragDropCompleted_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnPerformDragDropCompleted_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      bool p_success{};
      uint32_t p_action_taken{};
      WindowTreeClient_OnPerformDragDropCompleted_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_success = input_data_view.success();
      p_action_taken = input_data_view.action_taken();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnPerformDragDropCompleted deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnPerformDragDropCompleted(
std::move(p_window), 
std::move(p_success), 
std::move(p_action_taken));
      return true;
    }
    case internal::kWindowTreeClient_OnDragDropDone_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnDragDropDone_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnDragDropDone_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WindowTreeClient_OnDragDropDone_ParamsDataView input_data_view(params, &serialization_context);
      
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnDragDropDone deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDragDropDone();
      return true;
    }
    case internal::kWindowTreeClient_OnChangeCompleted_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_OnChangeCompleted_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_OnChangeCompleted_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_change_id{};
      bool p_success{};
      WindowTreeClient_OnChangeCompleted_ParamsDataView input_data_view(params, &serialization_context);
      
      p_change_id = input_data_view.change_id();
      p_success = input_data_view.success();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnChangeCompleted deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnChangeCompleted(
std::move(p_change_id), 
std::move(p_success));
      return true;
    }
    case internal::kWindowTreeClient_RequestClose_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_RequestClose_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_RequestClose_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window_id{};
      WindowTreeClient_RequestClose_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window_id = input_data_view.window_id();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::RequestClose deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->RequestClose(
std::move(p_window_id));
      return true;
    }
    case internal::kWindowTreeClient_GetWindowManager_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeClient_GetWindowManager_Params_Data* params =
          reinterpret_cast<internal::WindowTreeClient_GetWindowManager_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      ::ui::mojom::WindowManagerAssociatedRequest p_internal{};
      WindowTreeClient_GetWindowManager_ParamsDataView input_data_view(params, &serialization_context);
      
      p_internal =
          input_data_view.TakeInternal<decltype(p_internal)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::GetWindowManager deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->GetWindowManager(
std::move(p_internal));
      return true;
    }
  }
  return false;
}

// static
bool WindowTreeClientStubDispatch::AcceptWithResponder(
    WindowTreeClient* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWindowTreeClient_OnEmbed_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnEmbeddedAppDisconnected_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnUnembed_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnCaptureChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnFrameSinkIdAllocated_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnTopLevelCreated_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowBoundsChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowTransformChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnClientAreaChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnTransientWindowAdded_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnTransientWindowRemoved_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowHierarchyChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowReordered_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowDeleted_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowVisibilityChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowOpacityChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowParentDrawnStateChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowSharedPropertyChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowInputEvent_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnPointerEventObserved_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowFocused_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowCursorChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnWindowSurfaceChanged_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnDragDropStart_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnDragEnter_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::WindowTreeClient_OnDragEnter_Params_Data* params =
          reinterpret_cast<
              internal::WindowTreeClient_OnDragEnter_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      uint32_t p_key_state{};
      gfx::Point p_screen_position{};
      uint32_t p_effect_bitmask{};
      WindowTreeClient_OnDragEnter_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_key_state = input_data_view.key_state();
      if (!input_data_view.ReadScreenPosition(&p_screen_position))
        success = false;
      p_effect_bitmask = input_data_view.effect_bitmask();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnDragEnter deserializer");
        return false;
      }
      WindowTreeClient::OnDragEnterCallback callback =
          WindowTreeClient_OnDragEnter_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDragEnter(
std::move(p_window), 
std::move(p_key_state), 
std::move(p_screen_position), 
std::move(p_effect_bitmask), std::move(callback));
      return true;
    }
    case internal::kWindowTreeClient_OnDragOver_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::WindowTreeClient_OnDragOver_Params_Data* params =
          reinterpret_cast<
              internal::WindowTreeClient_OnDragOver_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      uint32_t p_key_state{};
      gfx::Point p_screen_position{};
      uint32_t p_effect_bitmask{};
      WindowTreeClient_OnDragOver_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_key_state = input_data_view.key_state();
      if (!input_data_view.ReadScreenPosition(&p_screen_position))
        success = false;
      p_effect_bitmask = input_data_view.effect_bitmask();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnDragOver deserializer");
        return false;
      }
      WindowTreeClient::OnDragOverCallback callback =
          WindowTreeClient_OnDragOver_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnDragOver(
std::move(p_window), 
std::move(p_key_state), 
std::move(p_screen_position), 
std::move(p_effect_bitmask), std::move(callback));
      return true;
    }
    case internal::kWindowTreeClient_OnDragLeave_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnCompleteDrop_Name: {
      mojo::internal::MessageDispatchContext context(message);

      internal::WindowTreeClient_OnCompleteDrop_Params_Data* params =
          reinterpret_cast<
              internal::WindowTreeClient_OnCompleteDrop_Params_Data*>(
                  message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      uint32_t p_window{};
      uint32_t p_key_state{};
      gfx::Point p_screen_position{};
      uint32_t p_effect_bitmask{};
      WindowTreeClient_OnCompleteDrop_ParamsDataView input_data_view(params, &serialization_context);
      
      p_window = input_data_view.window();
      p_key_state = input_data_view.key_state();
      if (!input_data_view.ReadScreenPosition(&p_screen_position))
        success = false;
      p_effect_bitmask = input_data_view.effect_bitmask();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeClient::OnCompleteDrop deserializer");
        return false;
      }
      WindowTreeClient::OnCompleteDropCallback callback =
          WindowTreeClient_OnCompleteDrop_ProxyToResponder::CreateCallback(
              message->request_id(),
              message->has_flag(mojo::Message::kFlagIsSync),
              std::move(responder));
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->OnCompleteDrop(
std::move(p_window), 
std::move(p_key_state), 
std::move(p_screen_position), 
std::move(p_effect_bitmask), std::move(callback));
      return true;
    }
    case internal::kWindowTreeClient_OnPerformDragDropCompleted_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnDragDropDone_Name: {
      break;
    }
    case internal::kWindowTreeClient_OnChangeCompleted_Name: {
      break;
    }
    case internal::kWindowTreeClient_RequestClose_Name: {
      break;
    }
    case internal::kWindowTreeClient_GetWindowManager_Name: {
      break;
    }
  }
  return false;
}

bool WindowTreeClientRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WindowTreeClient RequestValidator");

  switch (message->header()->name) {
    case internal::kWindowTreeClient_OnEmbed_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnEmbed_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnEmbeddedAppDisconnected_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnEmbeddedAppDisconnected_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnUnembed_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnUnembed_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnCaptureChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnCaptureChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnFrameSinkIdAllocated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnFrameSinkIdAllocated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnTopLevelCreated_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnTopLevelCreated_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowBoundsChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowBoundsChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowTransformChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowTransformChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnClientAreaChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnClientAreaChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnTransientWindowAdded_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnTransientWindowAdded_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnTransientWindowRemoved_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnTransientWindowRemoved_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowHierarchyChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowHierarchyChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowReordered_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowReordered_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowDeleted_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowDeleted_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowVisibilityChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowVisibilityChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowOpacityChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowOpacityChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowParentDrawnStateChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowParentDrawnStateChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowSharedPropertyChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowSharedPropertyChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowInputEvent_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowInputEvent_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnPointerEventObserved_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnPointerEventObserved_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowFocused_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowFocused_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowCursorChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowCursorChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnWindowSurfaceChanged_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnWindowSurfaceChanged_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnDragDropStart_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnDragDropStart_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnDragEnter_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnDragEnter_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnDragOver_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnDragOver_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnDragLeave_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnDragLeave_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnCompleteDrop_Name: {
      if (!mojo::internal::ValidateMessageIsRequestExpectingResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnCompleteDrop_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnPerformDragDropCompleted_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnPerformDragDropCompleted_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnDragDropDone_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnDragDropDone_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnChangeCompleted_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnChangeCompleted_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_RequestClose_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_RequestClose_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_GetWindowManager_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_GetWindowManager_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

bool WindowTreeClientResponseValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WindowTreeClient ResponseValidator");

  if (!mojo::internal::ValidateMessageIsResponse(message, &validation_context))
    return false;
  switch (message->header()->name) {
    case internal::kWindowTreeClient_OnDragEnter_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnDragEnter_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnDragOver_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnDragOver_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    case internal::kWindowTreeClient_OnCompleteDrop_Name: {
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeClient_OnCompleteDrop_ResponseParams_Data>(
                    message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}
const char WindowTreeFactory::Name_[] = "ui::mojom::WindowTreeFactory";

WindowTreeFactoryProxy::WindowTreeFactoryProxy(mojo::MessageReceiverWithResponder* receiver)
    : receiver_(receiver) {
}

void WindowTreeFactoryProxy::CreateWindowTree(
    WindowTreeRequest in_tree_request, WindowTreeClientPtr in_client) {
  const bool kExpectsResponse = false;
  const bool kIsSync = false;
  mojo::Message message;
  
  const uint32_t kFlags =
      ((kExpectsResponse) ? mojo::Message::kFlagExpectsResponse : 0) |
      ((kIsSync) ? mojo::Message::kFlagIsSync : 0);
  
  mojo::internal::SerializationContext serialization_context;
  size_t size = sizeof(::ui::mojom::internal::WindowTreeFactory_CreateWindowTree_Params_Data);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowTreeRequestDataView>(
      in_tree_request, &serialization_context);
  size += mojo::internal::PrepareToSerialize<::ui::mojom::WindowTreeClientPtrDataView>(
      in_client, &serialization_context);
  serialization_context.PrepareMessage(
      internal::kWindowTreeFactory_CreateWindowTree_Name,
      kFlags, size, &message);
  auto* buffer = (&message)->payload_buffer();
  auto params =
      ::ui::mojom::internal::WindowTreeFactory_CreateWindowTree_Params_Data::New(buffer);
  ALLOW_UNUSED_LOCAL(params);
  mojo::internal::Serialize<::ui::mojom::WindowTreeRequestDataView>(
      in_tree_request, &params->tree_request, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->tree_request),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid tree_request in WindowTreeFactory.CreateWindowTree request");
  mojo::internal::Serialize<::ui::mojom::WindowTreeClientPtrDataView>(
      in_client, &params->client, &serialization_context);
  MOJO_INTERNAL_DLOG_SERIALIZATION_WARNING(
      !mojo::internal::IsHandleOrInterfaceValid(params->client),
      mojo::internal::VALIDATION_ERROR_UNEXPECTED_INVALID_HANDLE,
      "invalid client in WindowTreeFactory.CreateWindowTree request");
  // This return value may be ignored as false implies the Connector has
  // encountered an error, which will be visible through other means.
  ignore_result(receiver_->Accept(&message));
}

// static
bool WindowTreeFactoryStubDispatch::Accept(
    WindowTreeFactory* impl,
    mojo::Message* message) {
  switch (message->header()->name) {
    case internal::kWindowTreeFactory_CreateWindowTree_Name: {
      mojo::internal::MessageDispatchContext context(message);

      DCHECK(message->is_serialized());
      internal::WindowTreeFactory_CreateWindowTree_Params_Data* params =
          reinterpret_cast<internal::WindowTreeFactory_CreateWindowTree_Params_Data*>(
              message->mutable_payload());
      
      mojo::internal::SerializationContext serialization_context;
      serialization_context.TakeHandlesFromMessage(message);
      bool success = true;
      WindowTreeRequest p_tree_request{};
      WindowTreeClientPtr p_client{};
      WindowTreeFactory_CreateWindowTree_ParamsDataView input_data_view(params, &serialization_context);
      
      p_tree_request =
          input_data_view.TakeTreeRequest<decltype(p_tree_request)>();
      p_client =
          input_data_view.TakeClient<decltype(p_client)>();
      if (!success) {
        ReportValidationErrorForMessage(
            message,
            mojo::internal::VALIDATION_ERROR_DESERIALIZATION_FAILED,
            "WindowTreeFactory::CreateWindowTree deserializer");
        return false;
      }
      // A null |impl| means no implementation was bound.
      assert(impl);
      impl->CreateWindowTree(
std::move(p_tree_request), 
std::move(p_client));
      return true;
    }
  }
  return false;
}

// static
bool WindowTreeFactoryStubDispatch::AcceptWithResponder(
    WindowTreeFactory* impl,
    mojo::Message* message,
    std::unique_ptr<mojo::MessageReceiverWithStatus> responder) {
  switch (message->header()->name) {
    case internal::kWindowTreeFactory_CreateWindowTree_Name: {
      break;
    }
  }
  return false;
}

bool WindowTreeFactoryRequestValidator::Accept(mojo::Message* message) {
  if (mojo::internal::IsUnserializedOrControlMessage(message))
    return true;

  mojo::internal::ValidationContext validation_context(
    message->payload(), message->payload_num_bytes(),
    message->handles()->size(), message->payload_num_interface_ids(), message,
    "WindowTreeFactory RequestValidator");

  switch (message->header()->name) {
    case internal::kWindowTreeFactory_CreateWindowTree_Name: {
      if (!mojo::internal::ValidateMessageIsRequestWithoutResponse(
              message, &validation_context)) {
        return false;
      }
      if (!mojo::internal::ValidateMessagePayload<
               internal::WindowTreeFactory_CreateWindowTree_Params_Data>(
                  message, &validation_context)) {
        return false;
      }
      return true;
    }
    default:
      break;
  }

  // Unrecognized message.
  ReportValidationError(
      &validation_context,
      mojo::internal::VALIDATION_ERROR_MESSAGE_HEADER_UNKNOWN_METHOD);
  return false;
}

}  // namespace mojom
}  // namespace ui

namespace mojo {

}  // namespace mojo

#if defined(__clang__)
#pragma clang diagnostic pop
#elif defined(_MSC_VER)
#pragma warning(pop)
#endif